unit peach;
{
 DESCRIPTION     :  Application providing GUI for calling PEA and third parts
                    open source executables, giving graphic tools for composing
                    appropriate command lines, such as file and folder selection,
                    import/export lists, choose parameters from graphic menus etc.
                    Archive theming, cross-platform desktop and script integration
                    features.
 REQUIREMENTS    :  FPC, Lazarus
                    Compiled pea and pealauncher in ./res/ path
                    third parts executables in ./res/executablename/ path

 EXTERNAL DATA   :  ---

 MEMORY USAGE    :  ---

 DISPLAY MODE    :  ---

 REFERENCES      :  ---

 REMARK          :  ---

 Version  Date      Author      Modification
 -------  --------  -------     ------------------------------------------
 PEA-PEACH
 0.10     20060908  G.Tani      Initial version named Pea-Peach 0.10
 0.11     20060920  G.Tani
 0.12     20060927  G.Tani
 0.13     20061103  G.Tani      Changed name of the release to PeaZip 1.0
 0.14     20061201  G.Tani      1.1
 0.15     20070118  G.Tani      1.2
 0.15b    20070122  G.Tani      1.2b
 0.16     20070201  G.Tani      1.3
 0.17     20070305  G.Tani      1.4
 0.18     20070319  G.Tani      1.5
 0.19     20070407  G.Tani      1.6
 0.20     20070512  G.Tani      1.7
 0.21     20070605  G.Tani      1.8
 0.22     20070708  G.Tani      1.8.1
 0.23     20070717  G.Tani      1.8.2
 0.24     20070807  G.Tani      1.9
 0.25     20070904  G.Tani      1.9.1
 0.26     20071003  G.Tani      1.9.2
 0.27     20071101  G.Tani      1.9.3
 0.28     20071206  G.Tani      1.10
 0.29     20080125  G.Tani      1.11
 0.30     20080319  G.Tani      2.0
 0.31     20080517  G.Tani      2.1
 0.32     20080807  G.Tani      2.2
 0.33     20080922  G.Tani      2.3a
 0.34     20081103  G.Tani      2.4
 0.35     20081124  G.Tani      2.4.1
 0.36     20090127  G.Tani      2.5
 0.37     20090224  G.Tani      2.5.1
 0.38     20090403  G.Tani      2.6.beta
 0.39     20090430  G.Tani      2.6
 0.40     20090604  G.Tani      2.6.1
 0.41     20090611  G.Tani      2.6.2
 0.42     20090713  G.Tani      2.6.3
 0.43     20090826  G.Tani      2.7.beta
 0.44     20090914  G.Tani      2.7
 0.45     20091017  G.Tani      2.7.1

Recompiled with Lazarus 0.9.28 / FPC 2.2.4
Updated Pea to 0.28

FILE MANAGER
Fixed: it is now possible to use keyboard shortcut for go back and go up when browsing any archive type
Fixed: it is now possible to open with custom applications items in subfolders of archives
Fixed: it is now possible to scan selected items with AVG (inside archives or in the filesystem) - the old non working command can be fixed using reset in Options > Settings > Open with... tab
Full path column now is shown only in flat view, to reduce the amount of displayed information in browsing mode
Various minor fixes and improvements (better sorting, faster translation, simpler message on archive browsing error)

EXTRACTION
New keyboard shortcuts for extraction:
 Ctrl+Alt+Enter start archive(s) extraction, as Ok button
 Shift+Backspace cancel archive(s) extraction, as Cancel button
 Shift+N toggle extract in new folder on/off

ARCHIVING
Fixed: "Add to separate archives" automatically unchecked when using add to archive mode
Fixed: clicking on file name dialog now always starts the dialog in the currently selected directory
New keyboard shortcuts for archiving:
 Ctrl+Alt+Enter start archive(s) creation, as Ok button
 Shift+Backspace  archive(s) creation, as Cancel button
 Shift+7 change archive type to 7z, +B to bz2, +G to gz, +S to 7z-sfx, + T to tar, +Z to zip

95 file extensions supported

All translations updated

(C) Copyright 2006 Giorgio Tani giorgiotani@interfree.it
official PeaZip site http://sourceforge.net/projects/PeaZip/
The program is released under GNU LGPL http://www.gnu.org/licenses/lgpl.txt

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
}

{$mode objfpc}{$H+}

interface

uses
{$IFDEF MSWINDOWS}
Windows, activex,
{$ENDIF}
LResources, Forms, Controls, Graphics, Dialogs, EditBtn,
StdCtrls, Buttons, objects, ExtCtrls, ComCtrls, ExtDlgs, Grids, Menus, Spin,
hash, sha512, whirl512, sha1, sha256, mem_util,
list_utils,pea_utils,ansiutf8_utils,unit1,unit2,unit3,unit4,unit5,unit6,unit7,
Classes, SysUtils, Process;

type

   TClipContent = array of array [0..3] of utf8string;

   { TForm_peach }

   TForm_peach = class(TForm)
     Bevel1: TBevel;
     Bevel2: TBevel;
     Bevel3: TBevel;
     Bevel8: TBevel;
     Button1: TBitBtn;
     ButtonArchive: TSpeedButton;
     ButtonArchive1: TSpeedButton;
     ButtonEditName12: TSpeedButton;
     ButtonEditName13: TSpeedButton;
     ButtonEditName3: TButton;
     ButtonEditName4: TSpeedButton;
     ButtonEditName5: TButton;
     ButtonEditName8: TSpeedButton;
     ButtonEditNameCustom: TButton;
     ButtonDesk: TButton;
     ButtonEditNameCustom1: TButton;
     ButtonExtCancel: TSpeedButton;
     ButtonExtOk: TSpeedButton;
     ButtonUn7za1: TSpeedButton;
     ButtonUn7za10: TSpeedButton;
     ButtonUn7za4: TSpeedButton;
     ButtonUn7za6: TSpeedButton;
     ButtonUn7zaTo: TSpeedButton;
     cbfSfx1: TCheckBox;
     cbType: TComboBox;
     cbUn7zaRecursive: TCheckBox;
     cbUn7zaRecursiveExclude: TCheckBox;
     CheckBox1: TCheckBox;
     CheckBox2: TCheckBox;
     CheckBox4: TCheckBox;
     CheckBoxARC1: TCheckBox;
     CheckBoxARC2: TCheckBox;
     CheckBoxArchive4: TCheckBox;
     CheckBoxArchive6: TCheckBox;
     CheckBoxArchiveHistory: TCheckBox;
     CheckBoxArchiveOF: TCheckBox;
     CheckBoxDate: TCheckBox;
     CheckBoxEncoding: TCheckBox;
     CheckBoxFolder: TCheckBox;
     CheckBoxIgnoreDisp: TCheckBox;
     CheckBoxIgnoreSel: TCheckBox;
     CheckBoxJobEncoding: TCheckBox;
     CheckBoxAutoOpen: TCheckBox;
     CheckBoxPEAEncryption: TCheckBox;
     CheckBoxQuadMax: TCheckBox;
     CheckBoxSeparate: TCheckBox;
     CheckBoxShowHints: TCheckBox;
     CheckBoxTarBefore: TCheckBox;
     CheckBoxTBW: TCheckBox;
     CheckBoxIH: TCheckBox;
     CheckBoxUnPeaAttributes: TCheckBox;
     CheckBoxUPX1: TCheckBox;
     CheckBoxUPX2: TCheckBox;
     CheckBoxWindowStatus: TCheckBox;
     ComboBox7zalgo: TComboBox;
     ComboBoxARC: TComboBox;
     ComboBoxARC2: TComboBox;
     ComboBoxARCalgo: TComboBox;
     ComboBoxArchive1: TComboBox;
     ComboBoxArchive2: TComboBox;
     ComboBoxArchive3: TComboBox;
     ComboBoxArchive4: TComboBox;
     ComboBoxArchive5: TComboBox;
     ComboBoxArchive6: TComboBox;
     ComboBoxArchive7: TComboBox;
     ComboBoxArchive8: TComboBox;
     ComboBoxArchiveCustom: TComboBox;
     ComboBoxArchiveCustom1: TComboBox;
     ComboBoxArchivePasses: TComboBox;
     ComboBoxArchiveSolid: TComboBox;
     ComboBoxArcSfx: TComboBox;
     ComboBoxArcSolid: TComboBox;
     ComboBoxOverwrite: TComboBox;
     ComboBoxOverwriteARC: TComboBox;
     ComboBoxPAQ1: TComboBox;
     ComboBoxPEA2: TComboBox;
     ComboBoxPEA3: TComboBox;
     ComboBoxPEA4: TComboBox;
     ComboBoxPEACompression1: TComboBox;
     ComboBoxSplit3: TComboBox;
     ComboBoxUPX: TComboBox;
     CompressionARC: TGroupBox;
     CompressionCustom: TGroupBox;
     CompressionCustom1: TGroupBox;
     CompressionPaq: TGroupBox;
     CompressionPea: TGroupBox;
     CompressionQUAD: TGroupBox;
     CompressionUPX: TGroupBox;
     ConsoleCreate: TPage;
     ConsoleOpen: TPage;
     ctrlrefresh: TSpeedButton;
     Edit4: TEdit;
     Edit5: TEdit;
     EditDesk: TEdit;
     EditExtCustom: TEdit;
     EditName3: TEdit;
     EditNameCustom: TEdit;
     EditNameCustom1: TEdit;
     EditOP: TEdit;
     EditOParc: TEdit;
     EditOPcustom: TEdit;
     EditOPcustom1: TEdit;
     EditOpenOut: TEdit;
     EncryptionPAQ: TGroupBox;
     EncryptionPEA: TGroupBox;
     EncryptionQUAD: TGroupBox;
     EncryptionUPX: TGroupBox;
     gbConsoleCreate: TGroupBox;
     gbConsoleCreate1: TGroupBox;
     Group7za1: TGroupBox;
     Group7za2: TGroupBox;
     GroupBox1: TGroupBox;
     GroupBox10: TGroupBox;
     GroupBox11: TGroupBox;
     GroupBox12: TGroupBox;
     GroupBox13: TGroupBox;
     GroupBox4: TGroupBox;
     GroupBox5: TGroupBox;
     GroupBoxCreateOut: TGroupBox;
     GroupBox7za3: TGroupBox;
     GroupBox7za4: TGroupBox;
     GroupBox9: TGroupBox;
     GroupBoxCreateOut1: TGroupBox;
     GroupBoxPEAAdvanced1: TGroupBox;
     iConsoleCreateImport: TImage;
     iConsoleCreateImport1: TSpeedButton;
     Image1: TImage;
     Image2: TImage;
     Image5: TImage;
     Image6: TImage;
     Image7: TImage;
     Image7zPj: TBitBtn;
     Image7zPj1: TBitBtn;
     Image7zPj3: TBitBtn;
     Image8: TImage;
     ImageBookmarks: TSpeedButton;
     ImageClip: TSpeedButton;
     ImageInfoArchive2: TImage;
     ImageInfoArchive4: TImage;
     ImageIO: TSpeedButton;
     ImageInfoArchive3: TImage;
     ImageInfoDefault2: TImage;
     ImageInfoDefault3: TImage;
     ImageListHistory: TSpeedButton;
     ImageOpenBack2: TImage;
     ImagePassword1: TSpeedButton;
     ImagePassword2: TSpeedButton;
     ImageSplit: TImage;
     ImageUn7zPj: TBitBtn;
     Label2: TLabel;
     Label3: TLabel;
     Label4: TLabel;
     Label7za1: TLabel;
     Label7za10: TLabel;
     Label7za11: TLabel;
     Label7za12: TLabel;
     Label7za14: TLabel;
     Label7za18: TLabel;
     Label7za2: TLabel;
     Label7za3: TLabel;
     Label7za4: TLabel;
     Label7za5: TLabel;
     Label7za6: TLabel;
     Label7za7: TLabel;
     Label7za9: TLabel;
     Label7zaPasses: TLabel;
     Label7zsolidblock: TLabel;
     LabelArchiveOther: TLabel;
     LabelArchiveOther1: TLabel;
     LabelArchiveOther2: TLabel;
     LabelArchiveOther3: TLabel;
     LabelArchiveOther4: TLabel;
     LabelArchiveOther5: TLabel;
     LabelArchiveOther6: TLabel;
     LabelArchiveOther7: TLabel;
     LabelArchiveOther8: TLabel;
     LabelARCsolidblock: TLabel;
     Labelcb10: TLabel;
     Labelcb13: TLabel;
     Labelcb16: TLabel;
     Labelcb17: TLabel;
     LabelCustom: TLabel;
     LabelCustom1: TLabel;
     LabelOpenBack2: TLabel;
     LabelPEA6: TLabel;
     LabelPEA7: TLabel;
     LabelPEA8: TLabel;
     LabelSplit: TLabel;
     LabelStatus: TLabel;
     LabelStatus1: TLabel;
     LabelStatus2: TLabel;
     LabelStatus3: TLabel;
     LabelStatus4: TLabel;
     lConsoleCreate1: TLabel;
     lConsoleCreate2: TLabel;
     lConsoleCreate3: TLabel;
     lConsoleCreate4: TLabel;
     lConsoleCreate_jobcode: TLabel;
     lConsoleCreate_jobcode1: TLabel;
     lConsoleCreate_outname: TLabel;
     lConsoleCreate_outname1: TLabel;
     mConsoleCreate: TMemo;
     mConsoleCreate1: TMemo;
     mctrlf3: TMenuItem;
     mctrlshiftf2: TMenuItem;
     mcancel: TMenuItem;
     MenuItem13: TMenuItem;
     mgotypegz: TMenuItem;
     mgotype7zsfx: TMenuItem;
     mgotypetar: TMenuItem;
     mgotypezip: TMenuItem;
     mgotypebz2: TMenuItem;
     mgotype7z: TMenuItem;
     mgotype: TMenuItem;
     mgobrowser: TMenuItem;
     mextractarchive: TMenuItem;
     mtogglenew: TMenuItem;
     mLayout: TMenuItem;
     pmsecuredelete: TMenuItem;
     pmquickdelete: TMenuItem;
     pmcpyto: TMenuItem;
     pmmvto: TMenuItem;
     pmexplore: TMenuItem;
     pmcommand: TMenuItem;
     MenuItem37: TMenuItem;
     pmh2: TMenuItem;
     pmh3: TMenuItem;
     pmh4: TMenuItem;
     pmh5: TMenuItem;
     pmh7: TMenuItem;
     pmh6: TMenuItem;
     pmh8: TMenuItem;
     pmh1: TMenuItem;
     mLang: TMenuItem;
     MenuItemOpen_Last: TMenuItem;
     MenuItem17: TMenuItem;
     MenuItem2: TMenuItem;
     madvfilters: TMenuItem;
     MenuItem33: TMenuItem;
     MenuItem34: TMenuItem;
     mkeyfile: TMenuItem;
     msetpw: TMenuItem;
     mOptions: TMenuItem;
     mgotoextlayout: TMenuItem;
     MenuItem28: TMenuItem;
     MenuItem31: TMenuItem;
     MenuItem32: TMenuItem;
     MenuItemextexplore: TMenuItem;
     MenuItemArchive_documents1: TMenuItem;
     mType7z: TMenuItem;
     mTypeARC: TMenuItem;
     mTypebzip2: TMenuItem;
     mTypeCustom: TMenuItem;
     mTypegz: TMenuItem;
     mTypePAQ: TMenuItem;
     mTypePEA: TMenuItem;
     mTypeQUAD: TMenuItem;
     mTypeSfx: TMenuItem;
     mTypeSfxArc: TMenuItem;
     mTypeSplit: TMenuItem;
     mTypeTAR: TMenuItem;
     mTypeUPX: TMenuItem;
     mTypeZIP: TMenuItem;
     NotebookOpen: TNotebook;
     OptionsOpen: TPage;
     PageExtAdv: TPage;
     Panel5: TPanel;
     Panel6: TPanel;
     PanelExtract: TPanel;
     PanelFileBrowserOptions: TPanel;
     PEA: TGroupBox;
     pmhistory: TPopupMenu;
     PopupMenuType: TPopupMenu;
     pmBrowser: TMenuItem;
     mpathself1: TMenuItem;
     MenuItemArchive_home1: TMenuItem;
     MenuItemArchive_documents: TMenuItem;
     MenuItemA30: TMenuItem;
     mpathdesk1: TMenuItem;
     mpathexplore1: TMenuItem;
     mpathreset1: TMenuItem;
     mf8: TMenuItem;
     mctrlf8: TMenuItem;
     mshiftf8: TMenuItem;
     mf7: TMenuItem;
     mctrlf7: TMenuItem;
     mshiftf7: TMenuItem;
     mshiftf2: TMenuItem;
     mf2: TMenuItem;
     mctrf2: TMenuItem;
     mf4: TMenuItem;
     mfun: TMenuItem;
     OptionsAdv: TPage;
     OptionsARC: TGroupBox;
     OptionsPAQ: TGroupBox;
     OptionsQUAD: TGroupBox;
     OptionsUPX: TGroupBox;
     Panel7za: TPanel;
     PanelARC: TPanel;
     PanelArchive2: TPanel;
     PanelCustom: TPanel;
     PanelPAQ: TPanel;
     PanelPEA: TPanel;
     PanelQuad: TPanel;
     PanelSplitFile: TPanel;
     PanelSplitFile1: TPanel;
     PanelSplitFile2: TPanel;
     PanelUPX: TPanel;
     pmjbadd: TMenuItem;
     MenuItem25: TMenuItem;
     PanelBack1: TPanel;
     PopupMenupath1: TPopupMenu;
     RadioGroupAction: TComboBox;
     RadioGroupAction1: TComboBox;
     RadioGroupActionArc: TComboBox;
     RadioGroupArchive: TRadioGroup;
     RadioGroupBrowser: TComboBox;
     RadioGroupClip: TRadioGroup;
     RadioGroupPaq: TRadioGroup;
     RadioGroupQuad: TRadioGroup;
     Shape6: TShape;
     smBookmarksa1: TMenuItem;
     smfs1: TMenuItem;
     pmjumpfs: TMenuItem;
     mf3: TMenuItem;
     MenuItemHistoryBook: TMenuItem;
     PopupMenuHistory: TPopupMenu;
     RadioGroup6: TComboBox;
     RadioGrouppwrequest: TComboBox;
     smfsa1: TMenuItem;
     smRecent1: TMenuItem;
     pnr1: TMenuItem;
     pnr2: TMenuItem;
     pnr3: TMenuItem;
     pnr4: TMenuItem;
     pnr5: TMenuItem;
     pnr6: TMenuItem;
     pnr7: TMenuItem;
     pnr8: TMenuItem;
     pnra1: TMenuItem;
     pnra2: TMenuItem;
     pnra3: TMenuItem;
     pnra4: TMenuItem;
     pnra5: TMenuItem;
     pnra6: TMenuItem;
     pnra7: TMenuItem;
     pnra8: TMenuItem;
     MenuItemOpen_here: TMenuItem;
     porecent9: TMenuItem;
     mRecent9: TMenuItem;
     pmr9: TMenuItem;
     pmr1: TMenuItem;
     pmr2: TMenuItem;
     pmr3: TMenuItem;
     pmr4: TMenuItem;
     pmr5: TMenuItem;
     pmr6: TMenuItem;
     pmr7: TMenuItem;
     pmr8: TMenuItem;
     pmRecent: TMenuItem;
     pmjb2: TMenuItem;
     pmjb3: TMenuItem;
     pmjb4: TMenuItem;
     pmjb5: TMenuItem;
     pmjb6: TMenuItem;
     pmjb7: TMenuItem;
     pmjb8: TMenuItem;
     pmjb9: TMenuItem;
     pmjb10: TMenuItem;
     pmjb11: TMenuItem;
     pmjb12: TMenuItem;
     pmjb13: TMenuItem;
     pmjb14: TMenuItem;
     pmjb15: TMenuItem;
     pmjb16: TMenuItem;
     pmjb17: TMenuItem;
     pmjb18: TMenuItem;
     pmjb19: TMenuItem;
     pmjb20: TMenuItem;
     pmjb21: TMenuItem;
     pmjb22: TMenuItem;
     pmjb23: TMenuItem;
     pmjb24: TMenuItem;
     pmjumpsbook: TMenuItem;
     MenuItem29: TMenuItem;
     MenuItem30: TMenuItem;
     pmjb1: TMenuItem;
     MenuItemOpen_desktop1: TMenuItem;
     MenuItemOpen_documents1: TMenuItem;
     MenuItemOpen_home1: TMenuItem;
     Notebook2: TNotebook;
     Options: TPage;
     pcustompathexplore: TMenuItem;
     pcustompathreset: TMenuItem;
     mpathexplore: TMenuItem;
     mpathreset: TMenuItem;
     pojdn1: TMenuItem;
     pojdn2: TMenuItem;
     pojdn3: TMenuItem;
     pojdn4: TMenuItem;
     pojdn5: TMenuItem;
     pojdn6: TMenuItem;
     pojdn7: TMenuItem;
     pojdn8: TMenuItem;
     pojdn9: TMenuItem;
     pojdn10: TMenuItem;
     pojdn11: TMenuItem;
     pojdn12: TMenuItem;
     pojdn13: TMenuItem;
     pojdn14: TMenuItem;
     pojdn15: TMenuItem;
     pojdn16: TMenuItem;
     pojdn17: TMenuItem;
     pojdn18: TMenuItem;
     pojdn19: TMenuItem;
     pojdn20: TMenuItem;
     pojdn21: TMenuItem;
     pojdn22: TMenuItem;
     pojdn23: TMenuItem;
     pojdn24: TMenuItem;
     pojdn25: TMenuItem;
     pojdn26: TMenuItem;
     pajdn1: TMenuItem;
     pajdn2: TMenuItem;
     pajdn3: TMenuItem;
     pajdn4: TMenuItem;
     pajdn5: TMenuItem;
     pajdn6: TMenuItem;
     pajdn7: TMenuItem;
     pajdn8: TMenuItem;
     pajdn9: TMenuItem;
     pajdn10: TMenuItem;
     pajdn11: TMenuItem;
     pajdn12: TMenuItem;
     pajdn13: TMenuItem;
     pajdn14: TMenuItem;
     pajdn15: TMenuItem;
     pajdn16: TMenuItem;
     pajdn17: TMenuItem;
     pajdn18: TMenuItem;
     pajdn19: TMenuItem;
     pajdn20: TMenuItem;
     pajdn21: TMenuItem;
     pajdn22: TMenuItem;
     pajdn23: TMenuItem;
     pajdn24: TMenuItem;
     pajdn25: TMenuItem;
     pajdn26: TMenuItem;
     pnb1: TMenuItem;
     pnb2: TMenuItem;
     pnb3: TMenuItem;
     pnb4: TMenuItem;
     pnb5: TMenuItem;
     pnb6: TMenuItem;
     pnb7: TMenuItem;
     pnb8: TMenuItem;
     pnb9: TMenuItem;
     pnb10: TMenuItem;
     pnb11: TMenuItem;
     pnb12: TMenuItem;
     pnb13: TMenuItem;
     pnb14: TMenuItem;
     pnb15: TMenuItem;
     pnb16: TMenuItem;
     pnb17: TMenuItem;
     pnb18: TMenuItem;
     pnb19: TMenuItem;
     pnb20: TMenuItem;
     pnb21: TMenuItem;
     pnb22: TMenuItem;
     pnb23: TMenuItem;
     pnba1: TMenuItem;
     pnba2: TMenuItem;
     pnba3: TMenuItem;
     pnba4: TMenuItem;
     pnba5: TMenuItem;
     pnba6: TMenuItem;
     pnba7: TMenuItem;
     pnba8: TMenuItem;
     pnba9: TMenuItem;
     pnba10: TMenuItem;
     pnba11: TMenuItem;
     pnba12: TMenuItem;
     pnba13: TMenuItem;
     pnba14: TMenuItem;
     pnba15: TMenuItem;
     pnba16: TMenuItem;
     pnba17: TMenuItem;
     pnba18: TMenuItem;
     pnba19: TMenuItem;
     pnba20: TMenuItem;
     pnba21: TMenuItem;
     pnba22: TMenuItem;
     pnba23: TMenuItem;
     PopupMenupath: TPopupMenu;
     PopupOpen1: TPopupMenu;
     EditOpenIn1: TEdit;
     ctrlback: TSpeedButton;
     ctrlfwd: TSpeedButton;
     ctrlup: TSpeedButton;
     EditOpenIn: TEdit;
     EditUn7zaFilter: TEdit;
     EditUn7zaFilter1: TEdit;
     EditUn7zaFilterExclude: TEdit;
     pmmorefun_info: TMenuItem;
     pmmorefun_list: TMenuItem;
     Imagece2: TImage;
     ImageDefaults3: TBitBtn;
     Imagece1: TImage;
     ImageListRoot: TSpeedButton;
     ImageListSearch: TSpeedButton;
     ImagePassword: TSpeedButton;
     jddesktop: TMenuItem;
     jdhome: TMenuItem;
     jdroot: TMenuItem;
     Labelcb14: TLabel;
     Labelcb15: TLabel;
     Labelcb4: TLabel;
     Labelcb6: TLabel;
     Labelcb7: TLabel;
     Labelcb8: TLabel;
     Labelcb9: TLabel;
     LabelDefaultColor7: TLabel;
     jddocuments: TMenuItem;
     aowseparator2: TMenuItem;
     eowseparator2: TMenuItem;
     MemoList: TMemo;
     MenuItem10: TMenuItem;
     MenuItem23: TMenuItem;
     OpenDialogKF: TOpenDialog;
     PanelFilters: TPanel;
     PanelListBar: TPanel;
     PanelSide: TPanel;
     pmjumpspacer2: TMenuItem;
     MenuItemPW: TMenuItem;
     MenuItem27: TMenuItem;
     MenuItemNavigate: TMenuItem;
     OpenDialog4: TOpenDialog;
     pojd1: TMenuItem;
     pojd2: TMenuItem;
     pojd3: TMenuItem;
     pojd4: TMenuItem;
     pojd5: TMenuItem;
     pojd6: TMenuItem;
     pojd7: TMenuItem;
     pojd8: TMenuItem;
     pojd9: TMenuItem;
     pojd10: TMenuItem;
     pojd11: TMenuItem;
     pojd12: TMenuItem;
     pojd13: TMenuItem;
     pojd14: TMenuItem;
     pojd15: TMenuItem;
     pojd16: TMenuItem;
     pojd17: TMenuItem;
     pojd18: TMenuItem;
     pojd19: TMenuItem;
     pojd20: TMenuItem;
     pojd21: TMenuItem;
     pojd22: TMenuItem;
     pojd23: TMenuItem;
     pojd24: TMenuItem;
     pojd25: TMenuItem;
     pojd26: TMenuItem;
     pmjd1: TMenuItem;
     pmjd2: TMenuItem;
     pmjd3: TMenuItem;
     pmjd4: TMenuItem;
     pmjd5: TMenuItem;
     pmjd6: TMenuItem;
     pmjd7: TMenuItem;
     pmjd8: TMenuItem;
     pmjd9: TMenuItem;
     pmjd10: TMenuItem;
     pmjd11: TMenuItem;
     pmjd12: TMenuItem;
     pmjd13: TMenuItem;
     pmjd14: TMenuItem;
     pmjd15: TMenuItem;
     pmjd16: TMenuItem;
     pmjd17: TMenuItem;
     pmjd18: TMenuItem;
     pmjd19: TMenuItem;
     pmjd20: TMenuItem;
     pmjd21: TMenuItem;
     pmjd22: TMenuItem;
     pmjd23: TMenuItem;
     pmjd24: TMenuItem;
     pmjd25: TMenuItem;
     pmjd26: TMenuItem;
     pmmorefun: TPopupMenu;
     porecent2: TMenuItem;
     porecent3: TMenuItem;
     porecent4: TMenuItem;
     porecent5: TMenuItem;
     porecent6: TMenuItem;
     porecent7: TMenuItem;
     porecent8: TMenuItem;
     munpeabrowse: TMenuItem;
     munpea: TMenuItem;
     jdspacer: TMenuItem;
     po_searchhere: TMenuItem;
     po_searchrec: TMenuItem;
     Shape4: TShape;
     smBookmarks1: TMenuItem;
     smOpen1: TMenuItem;
     smRecenta1: TMenuItem;
     smSearch: TMenuItem;
     MenuItemOpen_addtobookmarks: TMenuItem;
     porecent1: TMenuItem;
     MenuItem26: TMenuItem;
     po_delete: TMenuItem;
     po_securedelete: TMenuItem;
     sm_delete: TMenuItem;
     smRecent: TMenuItem;
     smBookmarks: TMenuItem;
     sbBrowse: TMenuItem;
     MenuItemOpen_Up: TMenuItem;
     MenuItemOpen_back: TMenuItem;
     MenuItemOpen_fwd: TMenuItem;
     MenuItemOpen_root: TMenuItem;
     MenuItemOpen_jump: TMenuItem;
     MenuItemOpen_Flat: TMenuItem;
     MenuItemOpen_refresh: TMenuItem;
     po_spacer: TMenuItem;
     po_setpw: TMenuItem;
     mpeahelpoffline: TMenuItem;
     mTutorial: TMenuItem;
     powseparator2: TMenuItem;
     owseparator2: TMenuItem;
     owseparator1: TMenuItem;
     eowseparator1: TMenuItem;
     eowcustom1: TMenuItem;
     eowcustom2: TMenuItem;
     eowcustom3: TMenuItem;
     eowcustom4: TMenuItem;
     eowcustom5: TMenuItem;
     eowcustom6: TMenuItem;
     eowcustom7: TMenuItem;
     eowcustom8: TMenuItem;
     powseparator1: TMenuItem;
     aowseparator1: TMenuItem;
     aowcustom1: TMenuItem;
     aowcustom2: TMenuItem;
     aowcustom3: TMenuItem;
     aowcustom4: TMenuItem;
     aowcustom6: TMenuItem;
     aowcustom5: TMenuItem;
     aowcustom7: TMenuItem;
     aowcustom8: TMenuItem;
     eowcustom9: TMenuItem;
     eowcustom10: TMenuItem;
     eowcustom11: TMenuItem;
     eowcustom12: TMenuItem;
     eowcustom13: TMenuItem;
     eowcustom14: TMenuItem;
     eowcustom15: TMenuItem;
     eowcustom16: TMenuItem;
     aowcustom9: TMenuItem;
     aowcustom10: TMenuItem;
     aowcustom11: TMenuItem;
     aowcustom12: TMenuItem;
     aowcustom13: TMenuItem;
     aowcustom14: TMenuItem;
     aowcustom15: TMenuItem;
     aowcustom16: TMenuItem;
     powcustom9: TMenuItem;
     powcustom10: TMenuItem;
     powcustom11: TMenuItem;
     powcustom12: TMenuItem;
     powcustom13: TMenuItem;
     powcustom14: TMenuItem;
     powcustom15: TMenuItem;
     powcustom16: TMenuItem;
     owcustom10: TMenuItem;
     owcustom11: TMenuItem;
     owcustom12: TMenuItem;
     owcustom13: TMenuItem;
     owcustom14: TMenuItem;
     owcustom15: TMenuItem;
     owcustom16: TMenuItem;
     owcustom9: TMenuItem;
     pmCustEditDesc: TMenuItem;
     pmCustEditDesc1: TMenuItem;
     pmCustEditEdit1after: TMenuItem;
     Page6: TPage;
     pmCustEdit1: TPopupMenu;
     pmCustEditAdd1: TMenuItem;
     pmCustEditEdit1: TMenuItem;
     pmCustEditRemove1: TMenuItem;
     powcustom1: TMenuItem;
     powcustom2: TMenuItem;
     powcustom3: TMenuItem;
     powcustom4: TMenuItem;
     powcustom5: TMenuItem;
     powcustom6: TMenuItem;
     powcustom7: TMenuItem;
     powcustom8: TMenuItem;
     owcustom1: TMenuItem;
     owcustom2: TMenuItem;
     owcustom3: TMenuItem;
     owcustom4: TMenuItem;
     owcustom5: TMenuItem;
     owcustom6: TMenuItem;
     owcustom7: TMenuItem;
     owcustom8: TMenuItem;
     pmCustEditEdit: TMenuItem;
     pmCustEditAdd: TMenuItem;
     pmCustEditRemove: TMenuItem;
     pmCustEdit: TPopupMenu;
     RadioGroup0: TComboBox;
     RadioGroup4: TComboBox;
     SpeedButton4: TSpeedButton;
     SpinEditArchive1: TSpinEdit;
     Splitter1: TSplitter;
     StringGrid1: TStringGrid;
     StringGrid2: TStringGrid;
     StringGridBookmarks: TStringGrid;
     StringGridClipboard: TStringGrid;
     StringGridCustEdit: TStringGrid;
     StringGridCustEdit1: TStringGrid;
     StringGridHistory: TStringGrid;
     StringGridList: TStringGrid;
     Timer2: TTimer;
     tmpeazip: TMenuItem;
     MenuItem22: TMenuItem;
     tmcreate: TMenuItem;
     MenuItem21: TMenuItem;
     tmopenpath: TMenuItem;
     tmopenarchive: TMenuItem;
     MenuItem24: TMenuItem;
     tmquit: TMenuItem;
     mTray: TMenuItem;
     MenuItemVideo: TMenuItem;
     MenuItemCommons: TMenuItem;
     MenuItemBooks: TMenuItem;
     MenuItemSrc: TMenuItem;
     MenuItemwnews: TMenuItem;
     MenuItemDictionary: TMenuItem;
     MenuItemyweb: TMenuItem;
     MenuItemyanswers: TMenuItem;
     PopupMenuTray: TPopupMenu;
     TrayIcon1: TTrayIcon;
     webYahoo: TMenuItem;
     MenuItemBlog: TMenuItem;
     MenuItemImg: TMenuItem;
     MenuItemNews: TMenuItem;
     MenuItemUsenet: TMenuItem;
     webGoogle: TMenuItem;
     webWiki: TMenuItem;
     MenuItemWeb: TMenuItem;
     MenuItemWikipedia: TMenuItem;
     po_hexp: TMenuItem;
     MenuItemToolhexp: TMenuItem;
     mhexp: TMenuItem;
     MenuItemOpen_documents: TMenuItem;
     pmJumpDocuments: TMenuItem;
     pclip_removeitem: TMenuItem;
     pclip_clearclip: TMenuItem;
     MenuItemOpen_home: TMenuItem;
     Panel3: TPanel;
     pmJump3: TMenuItem;
     PopupMenuClipboard: TPopupMenu;
     ButtonLang: TButton;
     cbcrc16: TCheckBox;
     cbcheckall: TCheckBox;
     cbfSfx: TCheckBox;
     cbfCustom: TCheckBox;
     cbfARC: TCheckBox;
     cbRememberIOin1: TCheckBox;
     cbsha224: TCheckBox;
     cbsha256: TCheckBox;
     cbsha384: TCheckBox;
     cbsha512: TCheckBox;
     cbwhirlpool: TCheckBox;
     cbmd4: TCheckBox;
     cbmd5: TCheckBox;
     cbrmd: TCheckBox;
     cbsha1: TCheckBox;
     cbed2k: TCheckBox;
     cbcrc64: TCheckBox;
     cbcrc32: TCheckBox;
     cbcrc24: TCheckBox;
     cbf11: TCheckBox;
     cbf2: TCheckBox;
     cbf3: TCheckBox;
     cbf5: TCheckBox;
     cbf6: TCheckBox;
     cbf7: TCheckBox;
     cbf8: TCheckBox;
     cbf9: TCheckBox;
     cbf10: TCheckBox;
     cbadler: TCheckBox;
     CheckBoxARCpw: TCheckBox;
     CheckBoxARCpw2: TCheckBox;
     CheckBoxArchiveAES1: TCheckBox;
      cbf1: TCheckBox;
      CheckBoxPrebrowse: TCheckBox;
      ColorButton3: TColorButton;
      ColorButton4: TColorButton;
      ColorButton5: TColorButton;
      ColorButton2: TColorButton;
      ColorButton1: TColorButton;
      ComboBoxPasses: TComboBox;
      ComboBoxARC1: TComboBox;
      ComboBoxARC3: TComboBox;
      ComboBoxTheme: TComboBox;
      ctrlRefresh1: TImage;
      DirectoryEdit2: TDirectoryEdit;
      DirectoryEditthemegraphics: TDirectoryEdit;
      Edit1: TEdit;
      Edit2: TEdit;
      Edit3: TEdit;
      EditARCpw3: TEdit;
      EditARCpw4: TEdit;
      GroupBox2: TGroupBox;
      GroupBox6: TGroupBox;
      GroupBox7: TGroupBox;
      GroupBox7za5: TGroupBox;
      GroupBox8: TGroupBox;
      GroupBoxCheck: TGroupBox;
      Image7zPj2: TBitBtn;
      ImageDefaults2: TBitBtn;
      ImageExploreIn1: TImage;
      ImageInfoDefault1: TImage;
      Label1: TLabel;
      LabelConf: TLabel;
      LabelARCsolidblock1: TLabel;
      LabelDefaults6: TLabel;
      LabelLang2: TLabel;
      LabelPAQ2: TLabel;
      LabelTheme: TLabel;
      mAttributes: TMenuItem;
      mDate: TMenuItem;
      mDateRecent: TMenuItem;
      mDateEqual: TMenuItem;
      mDateOlder: TMenuItem;
      mDateERecent: TMenuItem;
      mDateEOlder: TMenuItem;
      mCompare: TMenuItem;
      mCheck: TMenuItem;
      jd22: TMenuItem;
      jd21: TMenuItem;
      jd20: TMenuItem;
      jd19: TMenuItem;
      jd18: TMenuItem;
      jd17: TMenuItem;
      jd16: TMenuItem;
      jd15: TMenuItem;
      jd14: TMenuItem;
      jd13: TMenuItem;
      mDefrag: TMenuItem;
      mDiskmanagement: TMenuItem;
      mComp: TMenuItem;
      MenuItem11: TMenuItem;
      MenuItem12: TMenuItem;
      mbAddArchive: TMenuItem;
      mbAddPath: TMenuItem;
      mbDelete: TMenuItem;
      mBookmarks: TMenuItem;
      mb12: TMenuItem;
      mb1: TMenuItem;
      mb11: TMenuItem;
      mb10: TMenuItem;
      mb9: TMenuItem;
      mb8: TMenuItem;
      mb7: TMenuItem;
      mb6: TMenuItem;
      mb5: TMenuItem;
      mb4: TMenuItem;
      mb3: TMenuItem;
      mb2: TMenuItem;
      mb13: TMenuItem;
      mb14: TMenuItem;
      mb15: TMenuItem;
      mb24: TMenuItem;
      mb16: TMenuItem;
      mb17: TMenuItem;
      mb18: TMenuItem;
      mb19: TMenuItem;
      mb20: TMenuItem;
      mb21: TMenuItem;
      mb22: TMenuItem;
      mb23: TMenuItem;
      pomb12: TMenuItem;
      pomb1: TMenuItem;
      pomb11: TMenuItem;
      pomb10: TMenuItem;
      pomb9: TMenuItem;
      pomb8: TMenuItem;
      pomb7: TMenuItem;
      pomb6: TMenuItem;
      pomb5: TMenuItem;
      pomb4: TMenuItem;
      pomb3: TMenuItem;
      pomb2: TMenuItem;
      pomb13: TMenuItem;
      pomb14: TMenuItem;
      pomb15: TMenuItem;
      pomb24: TMenuItem;
      pomb16: TMenuItem;
      pomb17: TMenuItem;
      pomb18: TMenuItem;
      pomb19: TMenuItem;
      pomb20: TMenuItem;
      pomb21: TMenuItem;
      pomb22: TMenuItem;
      pomb23: TMenuItem;
      MenuItem15: TMenuItem;
      mbNotes: TMenuItem;
      MenuItem16: TMenuItem;
      mAbout: TMenuItem;
      mDiskConvert: TMenuItem;
      MenuItem18: TMenuItem;
      MenuItem19: TMenuItem;
      MenuItem20: TMenuItem;
      mEnvstr: TMenuItem;
      po_envstr: TMenuItem;
      po_convert: TMenuItem;
      MenuItemBookmarks: TMenuItem;
      MenuItem6: TMenuItem;
      mcp: TMenuItem;
      mMisc: TMenuItem;
      po_copyto: TMenuItem;
      po_moveto: TMenuItem;
      MenuItemOpen_explore: TMenuItem;
      MenuItemOpen_commandprompt: TMenuItem;
      PopupMenuBookmarks: TPopupMenu;
      po_misc: TMenuItem;
      MenuItemRefresh: TMenuItem;
      mUser: TMenuItem;
      po_computer: TMenuItem;
      po_manage: TMenuItem;
      po_removeunits: TMenuItem;
      mRemoveUnits: TMenuItem;
      po_cpl: TMenuItem;
      po_taskman: TMenuItem;
      MenuItem8: TMenuItem;
      po_diskclean: TMenuItem;
      po_defrag: TMenuItem;
      MenuItem4: TMenuItem;
      po_systemtools: TMenuItem;
      mTaskmanager: TMenuItem;
      mDiskClean: TMenuItem;
      mCPL: TMenuItem;
      mSystem: TMenuItem;
      mgotolayout: TMenuItem;
      MenuItemOpen_my: TMenuItem;
      MenuItemOpen_desktop: TMenuItem;
      MenuItem5: TMenuItem;
      MenuItemOpen_path: TMenuItem;
      MenuItemOpen_archive: TMenuItem;
      smOpen: TMenuItem;
      po_join: TMenuItem;
      po_compare: TMenuItem;
      po_check: TMenuItem;
      po_paste: TMenuItem;
      po_split: TMenuItem;
      po_newfolder: TMenuItem;
      po_rename: TMenuItem;
      MenuItem7: TMenuItem;
      po_copy: TMenuItem;
      po_cut: TMenuItem;
      po_modify: TMenuItem;
      po_filetools: TMenuItem;
      po_open_peazip: TMenuItem;
      po_open_associated: TMenuItem;
      po_open_custom: TMenuItem;
      po_open: TMenuItem;
      MenuItemOpen_list_all: TMenuItem;
      MenuItemOpen_list_selected: TMenuItem;
      MenuItemOpen_list_displayed: TMenuItem;
      MenuItemOpen_list: TMenuItem;
      po_list: TMenuItem;
      po_test: TMenuItem;
      po_info: TMenuItem;
      po_extto: TMenuItem;
      po_exthere: TMenuItem;
      po_addtoarchive: TMenuItem;
      po_archive_spacer: TMenuItem;
      mRefresh: TMenuItem;
      pmJumpSpacer1: TMenuItem;
      jd12: TMenuItem;
      jd11: TMenuItem;
      jd10: TMenuItem;
      jd9: TMenuItem;
      jd8: TMenuItem;
      jd7: TMenuItem;
      jd6: TMenuItem;
      jd5: TMenuItem;
      jd4: TMenuItem;
      jd3: TMenuItem;
      jd2: TMenuItem;
      jd1: TMenuItem;
      jd26: TMenuItem;
      jd25: TMenuItem;
      jd24: TMenuItem;
      jd23: TMenuItem;
      mPaths: TMenuItem;
      pmJumpArchive: TMenuItem;
      pmJumpPath: TMenuItem;
      pmJump2: TMenuItem;
      pmJump1: TMenuItem;
      pcustom3: TMenuItem;
      pcustom2: TMenuItem;
      pcustom1: TMenuItem;
      pcustom8: TMenuItem;
      pcustom7: TMenuItem;
      pcustom6: TMenuItem;
      pcustom5: TMenuItem;
      pcustom4: TMenuItem;
      mSpacer2_1: TMenuItem;
      mSortBySelection: TMenuItem;
      OptionsARC1: TGroupBox;
      Page5: TPage;
      pmSearchAdd: TMenuItem;
      mSpacer0: TMenuItem;
      MenuItemToolCheck: TMenuItem;
      MenuItemToolCompare: TMenuItem;
      MenuItemOpen_ext_all: TMenuItem;
      MenuItemOpen_ext_selected: TMenuItem;
      MenuItemOpen_ext_displayed: TMenuItem;
      MenuItemOpen_ExtractHere: TMenuItem;
      mSplitFile: TMenuItem;
      mJoinFiles: TMenuItem;
      mTools: TMenuItem;
      mQuit: TMenuItem;
      mSpacer0_1: TMenuItem;
      mSizeESmaller: TMenuItem;
      mSizeELarger: TMenuItem;
      mPSLarger: TMenuItem;
      mPSELarger: TMenuItem;
      mPSEqual: TMenuItem;
      mPSESmaller: TMenuItem;
      mPackedSize: TMenuItem;
      mPSSmaller: TMenuItem;
      mSizeEqual: TMenuItem;
      mSizeLarger: TMenuItem;
      mSizeSmaller: TMenuItem;
      mSize: TMenuItem;
      mSpacer2_0: TMenuItem;
      mType: TMenuItem;
      Page2: TPage;
      pCustom: TPopupMenu;
      pmJump: TPopupMenu;
      RadioGroup1: TRadioGroup;
      RadioGroupPaq1: TRadioGroup;
      SpinEdit3: TSpinEdit;
      theme_label: TLabel;
      LabelColor2: TLabel;
      LabelColor1: TLabel;
      LabelColor6: TLabel;
      LabelDefaultColor2: TLabel;
      LabelDefaultColor1: TLabel;
      LabelDefaultColor6: TLabel;
      LabelSplit1: TLabel;
      mSpacer6: TMenuItem;
      ImageDefaults1: TImage;
      ImagePEA6: TImage;
      ImagePEA7: TImage;
      ImageListExpandAll1: TImage;
      LabelThemeLoaded: TLabel;
      LabelDefaultGraphicsFolder: TLabel;
      Labelthemegraphic: TLabel;
      LabelPEAVolumes: TLabel;
      LabelColor3: TLabel;
      LabelColor4: TLabel;
      LabelColor5: TLabel;
      LabelDefaultColor3: TLabel;
      LabelDefaultColor4: TLabel;
      LabelDefaultColor5: TLabel;
      LabelDefaultOpacity: TLabel;
      LabelDefaults1: TLabel;
      LabelDefaults2: TLabel;
      LabelMain3: TLabel;
      LabelMain4: TLabel;
      LabelMainWrite1: TLabel;
      LabelOpacity: TLabel;
      LabelTheme1: TLabel;
      LabelTheme2: TLabel;
      LabelTheme3: TLabel;
      LabelTheme4: TLabel;
      LabelUnPeaExplore2: TLabel;
      MainMenu1: TMainMenu;
      mDefaults: TMenuItem;
      MenuItem1: TMenuItem;
      mBench: TMenuItem;
      mEdit: TMenuItem;
      mArchive: TMenuItem;
      MenuItemToolJoin: TMenuItem;
      MenuItemToolSplit: TMenuItem;
      MenuItemTools: TMenuItem;
      MenuItemOpen_info_selected: TMenuItem;
      MenuItemOpen_info_all: TMenuItem;
      MenuItemOpen_info_displayed: TMenuItem;
      mFlat: TMenuItem;
      mSpacer3: TMenuItem;
      openwithpeazip: TMenuItem;
      openwithassociated: TMenuItem;
      openwithcustom: TMenuItem;
      MenuItemERCustom: TMenuItem;
      MenuItemPrePeaZip: TMenuItem;
      MenuItemPreAssociated: TMenuItem;
      MenuItemPreCustom: TMenuItem;
      MenuItemERPeaZip: TMenuItem;
      MenuItemERAssociated: TMenuItem;
      mRecent4: TMenuItem;
      mRecent3: TMenuItem;
      mRecent2: TMenuItem;
      mRecent1: TMenuItem;
      mRecent8: TMenuItem;
      mRecent7: TMenuItem;
      mRecent6: TMenuItem;
      mRecent5: TMenuItem;
      mRecent: TMenuItem;
      mInvertSelection: TMenuItem;
      mSelectAll: TMenuItem;
      MenuItemOpen_preview: TMenuItem;
      MenuItemOpenWeb: TMenuItem;
      MenuItemOpen_test_all: TMenuItem;
      MenuItemOpen_test_selected: TMenuItem;
      MenuItemOpen_test_displayed: TMenuItem;
      MenuItemOpen_test: TMenuItem;
      MenuItemOpen_info: TMenuItem;
      MenuItemOpen_spacer5: TMenuItem;
      Page4: TPage;
      pmOpenPJall1: TMenuItem;
      pmOpenPJdisplayed: TMenuItem;
      pmOpenPJselected: TMenuItem;
      pmOpenPJall: TMenuItem;
      pmOpenPJ: TPopupMenu;
      MenuItemOpen_ExtractAllTo: TMenuItem;
      MenuItemOpen_ExtractDisplayedTo: TMenuItem;
      MenuItemOpen_ExtractSelectedTo: TMenuItem;
      MenuItemOpen_ExtractTo: TMenuItem;
      MenuItem5b: TMenuItem;
      MenuItemOpen_extandopenwith: TMenuItem;
      mPeaSite: TMenuItem;
      Notebook1: TNotebook;
      Page1: TPage;
      Page3: TPage;
      Panel1: TPanel;
      Panel2: TPanel;
      PanelArchiveMain: TPanel;
      PanelBarOpen: TPanel;
      PanelDefaults: TPanel;
      PanelMainTop1: TPanel;
      PanelBack: TPanel;
      PanelOpen: TPanel;
      PanelOptions: TPanel;
      SpinEdit1: TSpinEdit;
      SpinEdit2: TSpinEdit;
      SpinEditPrebrowse: TSpinEdit;
      pmSaveList: TMenuItem;
      pmLoadList: TMenuItem;
      pmAddFolder: TMenuItem;
      pmAddFile: TMenuItem;
      MenuItem3: TMenuItem;
      pmSpacer4: TMenuItem;
      mPEA: TMenuItem;
      mFeatures: TMenuItem;
      mPEAhelp: TMenuItem;
      mHelp: TMenuItem;
      mOptionsMenu: TMenuItem;
      OpenDialog1: TOpenDialog;
      OpenDialog2: TOpenDialog;
      OpenDialog3: TOpenDialog;
      PopupMenu1: TPopupMenu;
      PopupOpen: TPopupMenu;
      SaveDialog2: TSaveDialog;
      SaveDialogPJ: TSaveDialog;
      SelectDirectoryDialog1: TSelectDirectoryDialog;
      StringGridTmp: TStringGrid;
      themesave_label: TLabel;
      themesave_label_info: TLabel;
      theme_label1: TImage;
      Theming: TPage;
      Timer1: TTimer;
      procedure aowcustom10Click(Sender: TObject);
      procedure aowcustom11Click(Sender: TObject);
      procedure aowcustom12Click(Sender: TObject);
      procedure aowcustom13Click(Sender: TObject);
      procedure aowcustom14Click(Sender: TObject);
      procedure aowcustom15Click(Sender: TObject);
      procedure aowcustom16Click(Sender: TObject);
      procedure aowcustom1Click(Sender: TObject);
      procedure aowcustom2Click(Sender: TObject);
      procedure aowcustom3Click(Sender: TObject);
      procedure aowcustom4Click(Sender: TObject);
      procedure aowcustom5Click(Sender: TObject);
      procedure aowcustom6Click(Sender: TObject);
      procedure aowcustom7Click(Sender: TObject);
      procedure aowcustom8Click(Sender: TObject);
      procedure aowcustom9Click(Sender: TObject);
      procedure ButtonArchive1Click(Sender: TObject);
      procedure ButtonDeskClick(Sender: TObject);
      procedure ButtonEditName12Click(Sender: TObject);
      procedure ButtonEditName13Click(Sender: TObject);
      procedure ButtonEditName3Click(Sender: TObject);
      procedure ButtonEditName4Click(Sender: TObject);
      procedure ButtonEditName5Click(Sender: TObject);
      procedure ButtonEditName8Click(Sender: TObject);
      procedure ButtonEditNameCustom1Click(Sender: TObject);
      procedure ButtonExtCancelClick(Sender: TObject);
      procedure ButtonExtOkClick(Sender: TObject);
      procedure ButtonUn7za10Click(Sender: TObject);
      procedure ButtonUn7za6Click(Sender: TObject);
      procedure cbfSfx1Click(Sender: TObject);
      procedure CheckBox1Click(Sender: TObject);
      procedure CheckBox2Click(Sender: TObject);
      procedure CheckBoxARC2Click(Sender: TObject);
      procedure CheckBoxAutoOpenClick(Sender: TObject);
      procedure CheckBoxEncodingClick(Sender: TObject);
      procedure CheckBoxIHClick(Sender: TObject);
      procedure CheckBoxJobEncodingClick(Sender: TObject);
      procedure CheckBoxSeparateClick(Sender: TObject);
      procedure CheckBoxTBWClick(Sender: TObject);
      procedure ComboBoxARC2Change(Sender: TObject);
      procedure ComboBoxArchive7Change(Sender: TObject);
      procedure ctrlrefreshClick(Sender: TObject);
      procedure EditOpenIn1KeyPress(Sender: TObject; var Key: char);
      procedure EditOpenInChange(Sender: TObject);
      procedure EditUn7zaFilter1KeyPress(Sender: TObject; var Key: char);
      procedure EditUn7zaFilterChange(Sender: TObject);
      procedure eowcustom10Click(Sender: TObject);
      procedure eowcustom11Click(Sender: TObject);
      procedure eowcustom12Click(Sender: TObject);
      procedure eowcustom13Click(Sender: TObject);
      procedure eowcustom14Click(Sender: TObject);
      procedure eowcustom15Click(Sender: TObject);
      procedure eowcustom16Click(Sender: TObject);
      procedure eowcustom1Click(Sender: TObject);
      procedure eowcustom2Click(Sender: TObject);
      procedure eowcustom3Click(Sender: TObject);
      procedure eowcustom4Click(Sender: TObject);
      procedure eowcustom5Click(Sender: TObject);
      procedure eowcustom6Click(Sender: TObject);
      procedure eowcustom7Click(Sender: TObject);
      procedure eowcustom8Click(Sender: TObject);
      procedure eowcustom9Click(Sender: TObject);
      procedure ImageBookmarksClick(Sender: TObject);
      procedure ImageClipClick(Sender: TObject);
      procedure ImageInfoArchive2Click(Sender: TObject);
      procedure ImageInfoDefault1Click(Sender: TObject);
      procedure ImageInfoDefault2Click(Sender: TObject);
      procedure ImageInfoDefault3Click(Sender: TObject);
      procedure ImageIOClick(Sender: TObject);
      procedure ImageIOMouseEnter(Sender: TObject);
      procedure ImageListHistoryClick(Sender: TObject);
      procedure ImagePassword1Click(Sender: TObject);
      procedure Button1Click(Sender: TObject);
      procedure ButtonArchiveClick(Sender: TObject);
      procedure ButtonEditNameCustomClick(Sender: TObject);
      procedure ButtonInputArchiveClick(Sender: TObject);
      procedure ButtonLangClick(Sender: TObject);
      procedure ButtonPWOpenClick(Sender: TObject);
      procedure ButtonUn7za1Click(Sender: TObject);
      procedure ButtonUn7za4Click(Sender: TObject);
      procedure ButtonUn7zaToClick(Sender: TObject);
      procedure cbadlerClick(Sender: TObject);
      procedure cbcheckallClick(Sender: TObject);
      procedure cbcrc16Click(Sender: TObject);
      procedure cbcrc24Click(Sender: TObject);
      procedure cbcrc32Click(Sender: TObject);
      procedure cbcrc64Click(Sender: TObject);
      procedure cbed2kClick(Sender: TObject);
      procedure cbf11Click(Sender: TObject);
      procedure cbf1Click(Sender: TObject);
      procedure cbf2Click(Sender: TObject);
      procedure cbf3Click(Sender: TObject);
      procedure cbf5Click(Sender: TObject);
      procedure cbf6Click(Sender: TObject);
      procedure cbf7Click(Sender: TObject);
      procedure cbf8Click(Sender: TObject);
      procedure cbf9Click(Sender: TObject);
      procedure cbf10Click(Sender: TObject);
      procedure cbfARCClick(Sender: TObject);
      procedure cbfCustomClick(Sender: TObject);
      procedure cbfSfxClick(Sender: TObject);
      procedure cbmd4Click(Sender: TObject);
      procedure cbmd5Click(Sender: TObject);
      procedure cbrmdClick(Sender: TObject);
      procedure cbsha1Click(Sender: TObject);
      procedure cbsha224Click(Sender: TObject);
      procedure cbsha256Click(Sender: TObject);
      procedure cbsha384Click(Sender: TObject);
      procedure cbsha512Click(Sender: TObject);
      procedure cbTypeChange(Sender: TObject);
      procedure ComboBox7zalgoChange(Sender: TObject);
      procedure ComboBoxArchiveCustom1Change(Sender: TObject);
      procedure ComboBoxArchiveCustomChange(Sender: TObject);
      procedure ComboBoxOverwriteARCChange(Sender: TObject);
      procedure ComboBoxOverwriteChange(Sender: TObject);
      procedure ComboBoxPassesChange(Sender: TObject);
      procedure ImageDefaults3Click(Sender: TObject);
      procedure ImagePassword2Click(Sender: TObject);
      procedure jd10Click(Sender: TObject);
      procedure jd11Click(Sender: TObject);
      procedure jd12Click(Sender: TObject);
      procedure jd13Click(Sender: TObject);
      procedure jd14Click(Sender: TObject);
      procedure jd15Click(Sender: TObject);
      procedure jd16Click(Sender: TObject);
      procedure jd17Click(Sender: TObject);
      procedure jd18Click(Sender: TObject);
      procedure jd19Click(Sender: TObject);
      procedure jd1Click(Sender: TObject);
      procedure jd20Click(Sender: TObject);
      procedure jd21Click(Sender: TObject);
      procedure jd22Click(Sender: TObject);
      procedure jd23Click(Sender: TObject);
      procedure jd24Click(Sender: TObject);
      procedure jd25Click(Sender: TObject);
      procedure jd26Click(Sender: TObject);
      procedure jd2Click(Sender: TObject);
      procedure jd3Click(Sender: TObject);
      procedure jd4Click(Sender: TObject);
      procedure jd5Click(Sender: TObject);
      procedure jd6Click(Sender: TObject);
      procedure jd7Click(Sender: TObject);
      procedure jd8Click(Sender: TObject);
      procedure jd9Click(Sender: TObject);
      procedure jddesktopClick(Sender: TObject);
      procedure jddocumentsClick(Sender: TObject);
      procedure jdhomeClick(Sender: TObject);
      procedure jdrootClick(Sender: TObject);
      procedure Label4Click(Sender: TObject);
      procedure Labelcb17Click(Sender: TObject);
      procedure Labelcb7Click(Sender: TObject);
      procedure LabelConfClick(Sender: TObject);
      procedure LabelDefaultColor7Click(Sender: TObject);
      procedure LabelLang2Click(Sender: TObject);
      procedure LabelStatusClick(Sender: TObject);
      procedure mAboutClick(Sender: TObject);
      procedure mAddClick(Sender: TObject);
      procedure madvfiltersClick(Sender: TObject);
      procedure mb10Click(Sender: TObject);
      procedure mb11Click(Sender: TObject);
      procedure mb12Click(Sender: TObject);
      procedure mb13Click(Sender: TObject);
      procedure mb14Click(Sender: TObject);
      procedure mb15Click(Sender: TObject);
      procedure mb16Click(Sender: TObject);
      procedure mb17Click(Sender: TObject);
      procedure mb18Click(Sender: TObject);
      procedure mb19Click(Sender: TObject);
      procedure mb1Click(Sender: TObject);
      procedure mb20Click(Sender: TObject);
      procedure mb21Click(Sender: TObject);
      procedure mb22Click(Sender: TObject);
      procedure mb23Click(Sender: TObject);
      procedure mb24Click(Sender: TObject);
      procedure mb2Click(Sender: TObject);
      procedure mb3Click(Sender: TObject);
      procedure mb4Click(Sender: TObject);
      procedure mb5Click(Sender: TObject);
      procedure mb6Click(Sender: TObject);
      procedure mb7Click(Sender: TObject);
      procedure mb8Click(Sender: TObject);
      procedure mb9Click(Sender: TObject);
      procedure mbAddArchiveClick(Sender: TObject);
      procedure mbAddPathClick(Sender: TObject);
      procedure mbDeleteClick(Sender: TObject);
      procedure mbNotesClick(Sender: TObject);
      procedure mbOpenClick(Sender: TObject);
      procedure mcancelClick(Sender: TObject);
      procedure mCompClick(Sender: TObject);
      procedure mcpClick(Sender: TObject);
      procedure mctrf2Click(Sender: TObject);
      procedure mctrlf3Click(Sender: TObject);
      procedure mctrlf7Click(Sender: TObject);
      procedure mctrlf8Click(Sender: TObject);
      procedure mctrlshiftf2Click(Sender: TObject);
      procedure mgobrowserClick(Sender: TObject);
      procedure mextractarchiveClick(Sender: TObject);
      procedure MenuItemArchive_documents1Click(Sender: TObject);
      procedure MenuItemArchive_home1Click(Sender: TObject);
      procedure MenuItemextexploreClick(Sender: TObject);
      procedure MenuItemOpen_LastClick(Sender: TObject);
      procedure mf8Click(Sender: TObject);
      procedure mgotoextlayoutClick(Sender: TObject);
      procedure mgotype7zClick(Sender: TObject);
      procedure mgotype7zsfxClick(Sender: TObject);
      procedure mgotypebz2Click(Sender: TObject);
      procedure mgotypegzClick(Sender: TObject);
      procedure mgotypetarClick(Sender: TObject);
      procedure mgotypezipClick(Sender: TObject);
      procedure mLangClick(Sender: TObject);
      procedure mpathdesk1Click(Sender: TObject);
      procedure mpathexplore1Click(Sender: TObject);
      procedure mpathreset1Click(Sender: TObject);
      procedure mpathself1Click(Sender: TObject);
      procedure msetpwClick(Sender: TObject);
      procedure mshiftf7Click(Sender: TObject);
      procedure mDiskCleanClick(Sender: TObject);
      procedure mCPLClick(Sender: TObject);
      procedure mDefragClick(Sender: TObject);
      procedure mDiskmanagementClick(Sender: TObject);
      procedure mDiskConvertClick(Sender: TObject);
      procedure MenuItemBooksClick(Sender: TObject);
      procedure MenuItemCommonsClick(Sender: TObject);
      procedure MenuItemDictionaryClick(Sender: TObject);
      procedure MenuItemHistoryBookClick(Sender: TObject);
      procedure MenuItemNavigateClick(Sender: TObject);
      procedure MenuItemOpen_desktop1Click(Sender: TObject);
      procedure MenuItemOpen_documents1Click(Sender: TObject);
      procedure MenuItemOpen_hereClick(Sender: TObject);
      procedure MenuItemOpen_home1Click(Sender: TObject);
      procedure MenuItemPWClick(Sender: TObject);
      procedure MenuItemSrcClick(Sender: TObject);
      procedure MenuItemVideoClick(Sender: TObject);
      procedure MenuItemOpen_documentsClick(Sender: TObject);
      procedure MenuItemOpen_homeClick(Sender: TObject);
      procedure MenuItemToolhexpClick(Sender: TObject);
      procedure MenuItemwnewsClick(Sender: TObject);
      procedure MenuItemyanswersClick(Sender: TObject);
      procedure MenuItemywebClick(Sender: TObject);
      procedure mEnvstrClick(Sender: TObject);
      procedure MenuItemBookmarksClick(Sender: TObject);
      procedure MenuItemOpen_addtobookmarksClick(Sender: TObject);
      procedure MenuItemOpen_commandpromptClick(Sender: TObject);
      procedure MenuItemRefreshClick(Sender: TObject);
      procedure mf2Click(Sender: TObject);
      procedure mf3Click(Sender: TObject);
      procedure mf4Click(Sender: TObject);
      procedure mf7Click(Sender: TObject);
      procedure mhexpClick(Sender: TObject);
      procedure mpathexploreClick(Sender: TObject);
      procedure mpeahelpofflineClick(Sender: TObject);
      procedure mRecent9Click(Sender: TObject);
      procedure mRemoveUnitsClick(Sender: TObject);
      procedure mshiftf2Click(Sender: TObject);
      procedure mshiftf8Click(Sender: TObject);
      procedure mtogglenewClick(Sender: TObject);
      procedure mTrayClick(Sender: TObject);
      procedure mTutorialClick(Sender: TObject);
      procedure mTypeSfxArcClick(Sender: TObject);
      procedure mUserClick(Sender: TObject);
      procedure owcustom10Click(Sender: TObject);
      procedure owcustom11Click(Sender: TObject);
      procedure owcustom12Click(Sender: TObject);
      procedure owcustom13Click(Sender: TObject);
      procedure owcustom14Click(Sender: TObject);
      procedure owcustom15Click(Sender: TObject);
      procedure owcustom16Click(Sender: TObject);
      procedure owcustom1Click(Sender: TObject);
      procedure owcustom2Click(Sender: TObject);
      procedure owcustom3Click(Sender: TObject);
      procedure owcustom4Click(Sender: TObject);
      procedure owcustom5Click(Sender: TObject);
      procedure owcustom6Click(Sender: TObject);
      procedure owcustom7Click(Sender: TObject);
      procedure owcustom8Click(Sender: TObject);
      procedure owcustom9Click(Sender: TObject);
      procedure pajdn10Click(Sender: TObject);
      procedure pajdn11Click(Sender: TObject);
      procedure pajdn12Click(Sender: TObject);
      procedure pajdn13Click(Sender: TObject);
      procedure pajdn14Click(Sender: TObject);
      procedure pajdn15Click(Sender: TObject);
      procedure pajdn16Click(Sender: TObject);
      procedure pajdn17Click(Sender: TObject);
      procedure pajdn18Click(Sender: TObject);
      procedure pajdn19Click(Sender: TObject);
      procedure pajdn1Click(Sender: TObject);
      procedure pajdn20Click(Sender: TObject);
      procedure pajdn21Click(Sender: TObject);
      procedure pajdn22Click(Sender: TObject);
      procedure pajdn23Click(Sender: TObject);
      procedure pajdn24Click(Sender: TObject);
      procedure pajdn25Click(Sender: TObject);
      procedure pajdn26Click(Sender: TObject);
      procedure pajdn2Click(Sender: TObject);
      procedure pajdn3Click(Sender: TObject);
      procedure pajdn4Click(Sender: TObject);
      procedure pajdn5Click(Sender: TObject);
      procedure pajdn6Click(Sender: TObject);
      procedure pajdn7Click(Sender: TObject);
      procedure pajdn8Click(Sender: TObject);
      procedure pajdn9Click(Sender: TObject);
      procedure PanelSideClick(Sender: TObject);
      procedure pclip_clearclipClick(Sender: TObject);
      procedure pclip_removeitemClick(Sender: TObject);
      procedure pcustompathexploreClick(Sender: TObject);
      procedure pcustompathresetClick(Sender: TObject);
      procedure pmaddresstobookmarksClick(Sender: TObject);
      procedure pmArchiveNewsClick(Sender: TObject);
      procedure pmBrowserClick(Sender: TObject);
      procedure pmcommandClick(Sender: TObject);
      procedure pmcpytoClick(Sender: TObject);
      procedure pmCustEditAdd1Click(Sender: TObject);
      procedure pmCustEditAddClick(Sender: TObject);
      procedure pmCustEditDesc1Click(Sender: TObject);
      procedure pmCustEditDescClick(Sender: TObject);
      procedure pmCustEditEdit1afterClick(Sender: TObject);
      procedure pmCustEditEdit1Click(Sender: TObject);
      procedure pmCustEditEditClick(Sender: TObject);
      procedure pmCustEditRemove1Click(Sender: TObject);
      procedure pmCustEditRemoveClick(Sender: TObject);
      procedure pmexploreClick(Sender: TObject);
      procedure pmGolayoutClick(Sender: TObject);
      procedure pmh1Click(Sender: TObject);
      procedure pmh2Click(Sender: TObject);
      procedure pmh3Click(Sender: TObject);
      procedure pmh4Click(Sender: TObject);
      procedure pmh5Click(Sender: TObject);
      procedure pmh6Click(Sender: TObject);
      procedure pmh7Click(Sender: TObject);
      procedure pmh8Click(Sender: TObject);
      procedure pmjb10Click(Sender: TObject);
      procedure pmjb11Click(Sender: TObject);
      procedure pmjb12Click(Sender: TObject);
      procedure pmjb13Click(Sender: TObject);
      procedure pmjb14Click(Sender: TObject);
      procedure pmjb15Click(Sender: TObject);
      procedure pmjb16Click(Sender: TObject);
      procedure pmjb17Click(Sender: TObject);
      procedure pmjb18Click(Sender: TObject);
      procedure pmjb19Click(Sender: TObject);
      procedure pmjb1Click(Sender: TObject);
      procedure pmjb20Click(Sender: TObject);
      procedure pmjb21Click(Sender: TObject);
      procedure pmjb22Click(Sender: TObject);
      procedure pmjb23Click(Sender: TObject);
      procedure pmjb24Click(Sender: TObject);
      procedure pmjb2Click(Sender: TObject);
      procedure pmjb3Click(Sender: TObject);
      procedure pmjb4Click(Sender: TObject);
      procedure pmjb5Click(Sender: TObject);
      procedure pmjb6Click(Sender: TObject);
      procedure pmjb7Click(Sender: TObject);
      procedure pmjb8Click(Sender: TObject);
      procedure pmjb9Click(Sender: TObject);
      procedure pmjbaddClick(Sender: TObject);
      procedure pmJump3Click(Sender: TObject);
      procedure pmJumpDocumentsClick(Sender: TObject);
      procedure pmmorefun_infoClick(Sender: TObject);
      procedure pmmorefun_listClick(Sender: TObject);
      procedure pmmorefun_testClick(Sender: TObject);
      procedure pmmvtoClick(Sender: TObject);
      procedure pmquickdeleteClick(Sender: TObject);
      procedure pmr1Click(Sender: TObject);
      procedure pmr2Click(Sender: TObject);
      procedure pmr3Click(Sender: TObject);
      procedure pmr4Click(Sender: TObject);
      procedure pmr5Click(Sender: TObject);
      procedure pmr6Click(Sender: TObject);
      procedure pmr7Click(Sender: TObject);
      procedure pmr8Click(Sender: TObject);
      procedure pmr9Click(Sender: TObject);
      procedure pmsecuredeleteClick(Sender: TObject);
      procedure pnb10Click(Sender: TObject);
      procedure pnb11Click(Sender: TObject);
      procedure pnb12Click(Sender: TObject);
      procedure pnb13Click(Sender: TObject);
      procedure pnb14Click(Sender: TObject);
      procedure pnb15Click(Sender: TObject);
      procedure pnb16Click(Sender: TObject);
      procedure pnb17Click(Sender: TObject);
      procedure pnb18Click(Sender: TObject);
      procedure pnb19Click(Sender: TObject);
      procedure pnb1Click(Sender: TObject);
      procedure pnb20Click(Sender: TObject);
      procedure pnb21Click(Sender: TObject);
      procedure pnb22Click(Sender: TObject);
      procedure pnb23Click(Sender: TObject);
      procedure pnb2Click(Sender: TObject);
      procedure pnb3Click(Sender: TObject);
      procedure pnb4Click(Sender: TObject);
      procedure pnb5Click(Sender: TObject);
      procedure pnb6Click(Sender: TObject);
      procedure pnb7Click(Sender: TObject);
      procedure pnb8Click(Sender: TObject);
      procedure pnb9Click(Sender: TObject);
      procedure pnba10Click(Sender: TObject);
      procedure pnba11Click(Sender: TObject);
      procedure pnba12Click(Sender: TObject);
      procedure pnba13Click(Sender: TObject);
      procedure pnba14Click(Sender: TObject);
      procedure pnba15Click(Sender: TObject);
      procedure pnba16Click(Sender: TObject);
      procedure pnba17Click(Sender: TObject);
      procedure pnba18Click(Sender: TObject);
      procedure pnba19Click(Sender: TObject);
      procedure pnba1Click(Sender: TObject);
      procedure pnba20Click(Sender: TObject);
      procedure pnba21Click(Sender: TObject);
      procedure pnba22Click(Sender: TObject);
      procedure pnba23Click(Sender: TObject);
      procedure pnba2Click(Sender: TObject);
      procedure pnba3Click(Sender: TObject);
      procedure pnba4Click(Sender: TObject);
      procedure pnba5Click(Sender: TObject);
      procedure pnba6Click(Sender: TObject);
      procedure pnba7Click(Sender: TObject);
      procedure pnba8Click(Sender: TObject);
      procedure pnba9Click(Sender: TObject);
      procedure pnr1Click(Sender: TObject);
      procedure pnr2Click(Sender: TObject);
      procedure pnr3Click(Sender: TObject);
      procedure pnr4Click(Sender: TObject);
      procedure pnr5Click(Sender: TObject);
      procedure pnr6Click(Sender: TObject);
      procedure pnr7Click(Sender: TObject);
      procedure pnr8Click(Sender: TObject);
      procedure pnra1Click(Sender: TObject);
      procedure pnra2Click(Sender: TObject);
      procedure pnra3Click(Sender: TObject);
      procedure pnra4Click(Sender: TObject);
      procedure pnra5Click(Sender: TObject);
      procedure pnra6Click(Sender: TObject);
      procedure pnra7Click(Sender: TObject);
      procedure pnra8Click(Sender: TObject);
      procedure podevicesClick(Sender: TObject);
      procedure pojd10Click(Sender: TObject);
      procedure pojd11Click(Sender: TObject);
      procedure pojd12Click(Sender: TObject);
      procedure pojd13Click(Sender: TObject);
      procedure pojd14Click(Sender: TObject);
      procedure pojd15Click(Sender: TObject);
      procedure pojd16Click(Sender: TObject);
      procedure pojd17Click(Sender: TObject);
      procedure pojd18Click(Sender: TObject);
      procedure pojd19Click(Sender: TObject);
      procedure pojd1Click(Sender: TObject);
      procedure pojd20Click(Sender: TObject);
      procedure pojd21Click(Sender: TObject);
      procedure pojd22Click(Sender: TObject);
      procedure pojd23Click(Sender: TObject);
      procedure pojd24Click(Sender: TObject);
      procedure pojd25Click(Sender: TObject);
      procedure pojd26Click(Sender: TObject);
      procedure pojd2Click(Sender: TObject);
      procedure pojd3Click(Sender: TObject);
      procedure pojd4Click(Sender: TObject);
      procedure pojd5Click(Sender: TObject);
      procedure pojd6Click(Sender: TObject);
      procedure pojd7Click(Sender: TObject);
      procedure pojd8Click(Sender: TObject);
      procedure pojd9Click(Sender: TObject);
      procedure pojdn10Click(Sender: TObject);
      procedure pojdn11Click(Sender: TObject);
      procedure pojdn12Click(Sender: TObject);
      procedure pojdn13Click(Sender: TObject);
      procedure pojdn14Click(Sender: TObject);
      procedure pojdn15Click(Sender: TObject);
      procedure pojdn16Click(Sender: TObject);
      procedure pojdn17Click(Sender: TObject);
      procedure pojdn18Click(Sender: TObject);
      procedure pojdn19Click(Sender: TObject);
      procedure pojdn1Click(Sender: TObject);
      procedure pojdn20Click(Sender: TObject);
      procedure pojdn21Click(Sender: TObject);
      procedure pojdn22Click(Sender: TObject);
      procedure pojdn23Click(Sender: TObject);
      procedure pojdn24Click(Sender: TObject);
      procedure pojdn25Click(Sender: TObject);
      procedure pojdn26Click(Sender: TObject);
      procedure pojdn2Click(Sender: TObject);
      procedure pojdn3Click(Sender: TObject);
      procedure pojdn4Click(Sender: TObject);
      procedure pojdn5Click(Sender: TObject);
      procedure pojdn6Click(Sender: TObject);
      procedure pojdn7Click(Sender: TObject);
      procedure pojdn8Click(Sender: TObject);
      procedure pojdn9Click(Sender: TObject);
      procedure pomb10Click(Sender: TObject);
      procedure pomb11Click(Sender: TObject);
      procedure pomb12Click(Sender: TObject);
      procedure pomb13Click(Sender: TObject);
      procedure pomb14Click(Sender: TObject);
      procedure pomb15Click(Sender: TObject);
      procedure pomb16Click(Sender: TObject);
      procedure pomb17Click(Sender: TObject);
      procedure pomb18Click(Sender: TObject);
      procedure pomb19Click(Sender: TObject);
      procedure pomb1Click(Sender: TObject);
      procedure pomb20Click(Sender: TObject);
      procedure pomb21Click(Sender: TObject);
      procedure pomb22Click(Sender: TObject);
      procedure pomb23Click(Sender: TObject);
      procedure pomb24Click(Sender: TObject);
      procedure pomb2Click(Sender: TObject);
      procedure pomb3Click(Sender: TObject);
      procedure pomb4Click(Sender: TObject);
      procedure pomb5Click(Sender: TObject);
      procedure pomb6Click(Sender: TObject);
      procedure pomb7Click(Sender: TObject);
      procedure pomb8Click(Sender: TObject);
      procedure pomb9Click(Sender: TObject);
      procedure porecent1Click(Sender: TObject);
      procedure porecent2Click(Sender: TObject);
      procedure porecent3Click(Sender: TObject);
      procedure porecent4Click(Sender: TObject);
      procedure porecent5Click(Sender: TObject);
      procedure porecent6Click(Sender: TObject);
      procedure porecent7Click(Sender: TObject);
      procedure porecent8Click(Sender: TObject);
      procedure porecent9Click(Sender: TObject);
      procedure powcustom10Click(Sender: TObject);
      procedure powcustom11Click(Sender: TObject);
      procedure powcustom12Click(Sender: TObject);
      procedure powcustom13Click(Sender: TObject);
      procedure powcustom14Click(Sender: TObject);
      procedure powcustom15Click(Sender: TObject);
      procedure powcustom16Click(Sender: TObject);
      procedure powcustom1Click(Sender: TObject);
      procedure powcustom2Click(Sender: TObject);
      procedure powcustom3Click(Sender: TObject);
      procedure powcustom4Click(Sender: TObject);
      procedure powcustom5Click(Sender: TObject);
      procedure powcustom6Click(Sender: TObject);
      procedure powcustom7Click(Sender: TObject);
      procedure powcustom8Click(Sender: TObject);
      procedure powcustom9Click(Sender: TObject);
      procedure po_computerClick(Sender: TObject);
      procedure po_convertClick(Sender: TObject);
      procedure po_copytoClick(Sender: TObject);
      procedure po_cplClick(Sender: TObject);
      procedure po_envstrClick(Sender: TObject);
      procedure po_hexpClick(Sender: TObject);
      procedure po_layoutClick(Sender: TObject);
      procedure po_manageClick(Sender: TObject);
      procedure po_movetoClick(Sender: TObject);
      procedure po_removeunitsClick(Sender: TObject);
      procedure po_searchhereClick(Sender: TObject);
      procedure po_searchrecClick(Sender: TObject);
      procedure po_setpwClick(Sender: TObject);
      procedure po_taskmanClick(Sender: TObject);
      procedure MenuItemNewsClick(Sender: TObject);
      procedure MenuItemOpen_archiveClick(Sender: TObject);
      procedure MenuItemOpen_desktopClick(Sender: TObject);
      procedure MenuItemOpen_exploreClick(Sender: TObject);
      procedure MenuItemOpen_ExtractHereClick(Sender: TObject);
      procedure MenuItemOpen_myClick(Sender: TObject);
      procedure MenuItemOpen_pathClick(Sender: TObject);
      procedure MenuItemOpen_refreshClick(Sender: TObject);
      procedure mgotolayoutClick(Sender: TObject);
      procedure mPathsClick(Sender: TObject);
      procedure mRefreshClick(Sender: TObject);
      procedure mTaskmanagerClick(Sender: TObject);
      procedure mTypeSfxClick(Sender: TObject);
      procedure cbwhirlpoolClick(Sender: TObject);
      procedure CheckBox4Click(Sender: TObject);
      procedure CheckBoxARC1Click(Sender: TObject);
      procedure CheckBoxArchive6Click(Sender: TObject);
      procedure CheckBoxArchiveHistoryClick(Sender: TObject);
      procedure CheckBoxArchiveOFClick(Sender: TObject);
      procedure CheckBoxIgnoreDispClick(Sender: TObject);
      procedure CheckBoxIgnoreSelClick(Sender: TObject);
      procedure CheckBoxPrebrowseClick(Sender: TObject);
      procedure CheckBoxQuadMaxClick(Sender: TObject);
      procedure CheckBoxShowHintsClick(Sender: TObject);
      procedure CheckBoxTarBeforeClick(Sender: TObject);
      procedure CheckBoxWindowStatusClick(Sender: TObject);
      procedure ColorButton1ColorChanged(Sender: TObject);
      procedure ColorButton2ColorChanged(Sender: TObject);
      procedure ColorButton3ColorChanged(Sender: TObject);
      procedure ColorButton4ColorChanged(Sender: TObject);
      procedure ColorButton5ColorChanged(Sender: TObject);
      procedure ComboBoxARCalgoChange(Sender: TObject);
      procedure ComboBoxARCChange(Sender: TObject);
      procedure ComboBoxArchive1Change(Sender: TObject);
      procedure ComboBoxArchive3Change(Sender: TObject);
      procedure ComboBoxArchive4Change(Sender: TObject);
      procedure ComboBoxArcSolidChange(Sender: TObject);
      procedure ComboBoxPAQ1Change(Sender: TObject);
      procedure ComboBoxPEA2Change(Sender: TObject);
      procedure ComboBoxPEA3Change(Sender: TObject);
      procedure ComboBoxPEA4Change(Sender: TObject);
      procedure ComboBoxPEACompression1Change(Sender: TObject);
      procedure ComboBoxSplit3Change(Sender: TObject);
      procedure ComboBoxThemeChange(Sender: TObject);
      procedure ComboBoxUPXChange(Sender: TObject);
      procedure ctrlbackClick(Sender: TObject);
      procedure ctrlfwdClick(Sender: TObject);
      procedure ctrlupClick(Sender: TObject);
      procedure DirectoryEditthemegraphicsChange(Sender: TObject);
      procedure EditUn7zaFilterKeyPress(Sender: TObject; var Key: char);
      procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
      procedure FormCreate(Sender: TObject);
      procedure FormDropFiles(Sender: TObject; const FileNames: array of String
        );
      procedure FormResize(Sender: TObject);
      procedure FormWindowStateChange(Sender: TObject);
      procedure HowToClick(Sender: TObject);
      procedure iConsoleCreateImport1Click(Sender: TObject);
      procedure iConsoleCreateImportClick(Sender: TObject);
      procedure Image7zPj1Click(Sender: TObject);
      procedure Image7zPj3Click(Sender: TObject);
      procedure Image7zPjClick(Sender: TObject);
      procedure ImageDefaults2Click(Sender: TObject);
      procedure ImageExploreOutClick(Sender: TObject);
      procedure ImageListRootClick(Sender: TObject);
      procedure ImageListSearchClick(Sender: TObject);
      procedure ImageNav1Click(Sender: TObject);
      procedure ImagePasswordClick(Sender: TObject);
      procedure ImageUn7zPjClick(Sender: TObject);
      procedure LabelDefaultColor1Click(Sender: TObject);
      procedure LabelDefaultColor2Click(Sender: TObject);
      procedure LabelDefaultColor3Click(Sender: TObject);
      procedure LabelDefaultColor4Click(Sender: TObject);
      procedure LabelDefaultColor5Click(Sender: TObject);
      procedure LabelDefaultColor6Click(Sender: TObject);
      procedure LabelDefaultGraphicsFolderClick(Sender: TObject);
      procedure LabelDefaultOpacityClick(Sender: TObject);
      procedure LabelUn7zaFilterClick(Sender: TObject);
      procedure LableListBackClick(Sender: TObject);
      procedure lConsoleCreate2Click(Sender: TObject);
      procedure lConsoleCreate4Click(Sender: TObject);
      procedure mAddDirClick(Sender: TObject);
      procedure mAddFileClick(Sender: TObject);
      procedure mAddfilesClick(Sender: TObject);
      procedure mArchiveSearchAddClick(Sender: TObject);
      procedure mAttributesClick(Sender: TObject);
      procedure mBenchClick(Sender: TObject);
      procedure mCheckClick(Sender: TObject);
      procedure mClearListClick(Sender: TObject);
      procedure mCompareClick(Sender: TObject);
      procedure mDateEOlderClick(Sender: TObject);
      procedure mDateEqualClick(Sender: TObject);
      procedure mDateERecentClick(Sender: TObject);
      procedure mDateOlderClick(Sender: TObject);
      procedure mDateRecentClick(Sender: TObject);
      procedure mDefaultsClick(Sender: TObject);
      procedure MenuItem1Click(Sender: TObject);
      procedure MenuItem3Click(Sender: TObject);
      procedure MenuItem6Click(Sender: TObject);
      procedure MenuItemBlogClick(Sender: TObject);
      procedure MenuItemERAssociatedClick(Sender: TObject);
      procedure MenuItemERCustomClick(Sender: TObject);
      procedure MenuItemERPeaZipClick(Sender: TObject);
      procedure MenuItemImgClick(Sender: TObject);
      procedure MenuItemOpen_backClick(Sender: TObject);
      procedure MenuItemOpen_detete_selectedClick(Sender: TObject);
      procedure MenuItemOpen_ext_allClick(Sender: TObject);
      procedure MenuItemOpen_ext_displayedClick(Sender: TObject);
      procedure MenuItemOpen_ext_selectedClick(Sender: TObject);
      procedure MenuItemOpen_ExtractAllToClick(Sender: TObject);
      procedure MenuItemOpen_ExtractDisplayedToClick(Sender: TObject);
      procedure MenuItemOpen_ExtractSelectedToClick(Sender: TObject);
      procedure MenuItemOpen_FlatClick(Sender: TObject);
      procedure MenuItemOpen_fwdClick(Sender: TObject);
      procedure MenuItemOpen_info_allClick(Sender: TObject);
      procedure MenuItemOpen_info_displayedClick(Sender: TObject);
      procedure MenuItemOpen_info_selectedClick(Sender: TObject);
      procedure MenuItemOpen_jumpClick(Sender: TObject);
      procedure MenuItemOpen_list_allClick(Sender: TObject);
      procedure MenuItemOpen_list_displayedClick(Sender: TObject);
      procedure MenuItemOpen_list_selectedClick(Sender: TObject);
      procedure MenuItemOpen_rootClick(Sender: TObject);
      procedure MenuItemOpen_SearchAddClick(Sender: TObject);
      procedure MenuItemOpen_test_allClick(Sender: TObject);
      procedure MenuItemOpen_test_displayedClick(Sender: TObject);
      procedure MenuItemOpen_test_selectedClick(Sender: TObject);
      procedure MenuItemOpen_upClick(Sender: TObject);
      procedure MenuItemPreAssociatedClick(Sender: TObject);
      procedure MenuItemPreCustomClick(Sender: TObject);
      procedure MenuItemPrePeaZipClick(Sender: TObject);
      procedure MenuItemToolCheckClick(Sender: TObject);
      procedure MenuItemToolCompareClick(Sender: TObject);
      procedure MenuItemToolJoinClick(Sender: TObject);
      procedure MenuItemToolSplitClick(Sender: TObject);
      procedure MenuItemUsenetClick(Sender: TObject);
      procedure MenuItemWebClick(Sender: TObject);
      procedure MenuItemWikipediaClick(Sender: TObject);
      procedure mFlatClick(Sender: TObject);
      procedure mInvertSelectionClick(Sender: TObject);
      procedure mJoinFilesClick(Sender: TObject);
      procedure mKeyFileClick(Sender: TObject);
      procedure mOptionsClick(Sender: TObject);
      procedure mPEAClick(Sender: TObject);
      procedure mPEAhelpClick(Sender: TObject);
      procedure mPeaSiteClick(Sender: TObject);
      procedure mPSELargerClick(Sender: TObject);
      procedure mPSEqualClick(Sender: TObject);
      procedure mPSESmallerClick(Sender: TObject);
      procedure mPSLargerClick(Sender: TObject);
      procedure mPSSmallerClick(Sender: TObject);
      procedure mQuitClick(Sender: TObject);
      procedure mRecent1Click(Sender: TObject);
      procedure mRecent2Click(Sender: TObject);
      procedure mRecent3Click(Sender: TObject);
      procedure mRecent4Click(Sender: TObject);
      procedure mRecent5Click(Sender: TObject);
      procedure mRecent6Click(Sender: TObject);
      procedure mRecent7Click(Sender: TObject);
      procedure mRecent8Click(Sender: TObject);
      procedure mRFJClick(Sender: TObject);
      procedure mRFSClick(Sender: TObject);
      procedure mSelectAllClick(Sender: TObject);
      procedure mSizeELargerClick(Sender: TObject);
      procedure mSizeEqualClick(Sender: TObject);
      procedure mSizeESmallerClick(Sender: TObject);
      procedure mSizeLargerClick(Sender: TObject);
      procedure mSizeSmallerClick(Sender: TObject);
      procedure mSortBySelectionClick(Sender: TObject);
      procedure mSplitFileClick(Sender: TObject);
      procedure mType7zClick(Sender: TObject);
      procedure mTypeARCClick(Sender: TObject);
      procedure mTypebzip2Click(Sender: TObject);
      procedure mTypeClick(Sender: TObject);
      procedure mTypeCustomClick(Sender: TObject);
      procedure mTypegzClick(Sender: TObject);
      procedure mTypePAQClick(Sender: TObject);
      procedure mTypePEAClick(Sender: TObject);
      procedure mTypeQUADClick(Sender: TObject);
      procedure mTypeSplitClick(Sender: TObject);
      procedure mTypeTARClick(Sender: TObject);
      procedure mTypeUPXClick(Sender: TObject);
      procedure mTypeZIPClick(Sender: TObject);
      procedure mUnPEAbrowseClick(Sender: TObject);
      procedure mUnPEAClick(Sender: TObject);
      procedure openwithassociatedClick(Sender: TObject);
      procedure openwithcustomClick(Sender: TObject);
      procedure openwithpeazipClick(Sender: TObject);
      procedure pcustom1Click(Sender: TObject);
      procedure pcustom2Click(Sender: TObject);
      procedure pcustom3Click(Sender: TObject);
      procedure pcustom4Click(Sender: TObject);
      procedure pcustom5Click(Sender: TObject);
      procedure pcustom6Click(Sender: TObject);
      procedure pcustom7Click(Sender: TObject);
      procedure pcustom8Click(Sender: TObject);
      procedure pmAddFileClick(Sender: TObject);
      procedure pmAddFolderClick(Sender: TObject);
      procedure pmArchiveBlogClick(Sender: TObject);
      procedure pmArchiveImageClick(Sender: TObject);
      procedure pmArchiveUsenetClick(Sender: TObject);
      procedure pmArchiveWikipediaClick(Sender: TObject);
      procedure pmJump1Click(Sender: TObject);
      procedure pmJump2Click(Sender: TObject);
      procedure pmJumpArchiveClick(Sender: TObject);
      procedure pmJumpPathClick(Sender: TObject);
      procedure pmLoadListClick(Sender: TObject);
      procedure pmOpenPJallClick(Sender: TObject);
      procedure pmOpenPJdisplayedClick(Sender: TObject);
      procedure pmOpenPJselectedClick(Sender: TObject);
      procedure pmSaveListClick(Sender: TObject);
      procedure pmSearchAddClick(Sender: TObject);
      procedure po_addtoarchiveClick(Sender: TObject);
      procedure po_checkClick(Sender: TObject);
      procedure po_compareClick(Sender: TObject);
      procedure po_copyClick(Sender: TObject);
      procedure po_cutClick(Sender: TObject);
      procedure po_defragClick(Sender: TObject);
      procedure po_deleteClick(Sender: TObject);
      procedure po_diskcleanClick(Sender: TObject);
      procedure po_exthereClick(Sender: TObject);
      procedure po_exttoClick(Sender: TObject);
      procedure po_infoClick(Sender: TObject);
      procedure po_joinClick(Sender: TObject);
      procedure po_listClick(Sender: TObject);
      procedure po_newfolderClick(Sender: TObject);
      procedure po_open_associatedClick(Sender: TObject);
      procedure po_open_customClick(Sender: TObject);
      procedure po_open_peazipClick(Sender: TObject);
      procedure po_pasteClick(Sender: TObject);
      procedure po_renameClick(Sender: TObject);
      procedure po_securedeleteClick(Sender: TObject);
      procedure po_splitClick(Sender: TObject);
      procedure po_testClick(Sender: TObject);
      procedure pwArchiveWebsitesClick(Sender: TObject);
      procedure RadioGroup0Change(Sender: TObject);
      procedure RadioGroup1Click(Sender: TObject);
      procedure RadioGroup4Change(Sender: TObject);
      procedure RadioGroup6Change(Sender: TObject);
      procedure RadioGroupBrowserChange(Sender: TObject);
      procedure RadioGroupClipClick(Sender: TObject);
      procedure RadioGroupLPAQClick(Sender: TObject);
      procedure RadioGroupPaqClick(Sender: TObject);
      procedure RadioGrouppwrequestChange(Sender: TObject);
      procedure RadioGroupQuadClick(Sender: TObject);
      procedure SpeedButton4Click(Sender: TObject);
      procedure SpinEdit1Change(Sender: TObject);
      procedure SpinEdit2Change(Sender: TObject);
      procedure SpinEdit3Change(Sender: TObject);
      procedure SpinEditPrebrowseChange(Sender: TObject);
      procedure Splitter1Moved(Sender: TObject);
      procedure StringGrid1DblClick(Sender: TObject);
      procedure StringGrid1DrawCell(Sender: TObject; aCol, aRow: Integer;
        aRect: TRect; aState: TGridDrawState);
      procedure StringGrid1HeaderClick(Sender: TObject; IsColumn: Boolean;
        Index: Integer);
      procedure StringGrid1HeaderSized(Sender: TObject; IsColumn: Boolean;
        Index: Integer);
      procedure StringGrid1KeyDown(Sender: TObject; var Key: Word;
        Shift: TShiftState);
      procedure StringGrid1KeyPress(Sender: TObject; var Key: char);
      procedure StringGrid1KeyUp(Sender: TObject; var Key: Word;
        Shift: TShiftState);
      procedure StringGrid1MouseDown(Sender: TOBject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
      procedure StringGrid1MouseUp(Sender: TOBject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
      procedure StringGrid2DblClick(Sender: TObject);
      procedure StringGrid2DrawCell(Sender: TObject; aCol, aRow: Integer;
        aRect: TRect; aState: TGridDrawState);
      procedure StringGrid2HeaderClick(Sender: TObject; IsColumn: Boolean;
        Index: Integer);
      procedure StringGrid2HeaderSized(Sender: TObject; IsColumn: Boolean;
        Index: Integer);
      procedure StringGrid2KeyDown(Sender: TObject; var Key: Word;
        Shift: TShiftState);
      procedure StringGrid2KeyPress(Sender: TObject; var Key: char);
      procedure StringGrid2KeyUp(Sender: TObject; var Key: Word;
        Shift: TShiftState);
      procedure StringGrid2MouseDown(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
      procedure StringGrid2MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
      procedure StringGridBookmarksColRowMoved(Sender: TObject;
        IsColumn: Boolean; sIndex, tIndex: Integer);
      procedure StringGridBookmarksDblClick(Sender: TObject);
      procedure StringGridBookmarksDrawCell(Sender: TObject; aCol,
        aRow: Integer; aRect: TRect; aState: TGridDrawState);
      procedure StringGridBookmarksHeaderClick(Sender: TObject;
        IsColumn: Boolean; Index: Integer);
      procedure StringGridBookmarksHeaderSized(Sender: TObject;
        IsColumn: Boolean; Index: Integer);
      procedure StringGridBookmarksKeyPress(Sender: TObject; var Key: char);
      procedure StringGridBookmarksMouseDown(Sender: TObject;
        Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      procedure StringGridClipboardDblClick(Sender: TObject);
      procedure StringGridClipboardKeyPress(Sender: TObject; var Key: char);
      procedure StringGridCustEdit1ColRowMoved(Sender: TObject;
        IsColumn: Boolean; sIndex, tIndex: Integer);
      procedure StringGridCustEdit1DblClick(Sender: TObject);
      procedure StringGridCustEdit1MouseDown(Sender: TObject;
        Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      procedure StringGridCustEdit1SelectCell(Sender: TObject; aCol,
        aRow: Integer; var CanSelect: Boolean);
      procedure StringGridCustEditColRowMoved(Sender: TObject;
        IsColumn: Boolean; sIndex, tIndex: Integer);
      procedure StringGridCustEditDblClick(Sender: TObject);
      procedure StringGridCustEditMouseDown(Sender: TObject;
        Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      procedure StringGridHistoryDblClick(Sender: TObject);
      procedure StringGridHistoryDrawCell(Sender: TObject; aCol, aRow: Integer;
        aRect: TRect; aState: TGridDrawState);
      procedure StringGridHistoryHeaderSized(Sender: TObject;
        IsColumn: Boolean; Index: Integer);
      procedure StringGridListDblClick(Sender: TObject);
      procedure StringGridListDrawCell(Sender: TObject; aCol, aRow: Integer;
        aRect: TRect; aState: TGridDrawState);
      procedure StringGridListHeaderClick(Sender: TObject; IsColumn: Boolean;
        Index: Integer);
      procedure StringGridListHeaderSized(Sender: TObject; IsColumn: Boolean;
        Index: Integer);
      procedure StringGridListKeyDown(Sender: TObject; var Key: Word;
        Shift: TShiftState);
      procedure StringGridListKeyPress(Sender: TObject; var Key: char);
      procedure StringGridListKeyUp(Sender: TObject; var Key: Word;
        Shift: TShiftState);
      procedure StringGridListMouseDown(Sender: TOBject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
      procedure StringGridListMouseMove(Sender: TObject; Shift: TShiftState; X,
        Y: Integer);
      procedure StringGridListMouseUp(Sender: TOBject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
      procedure theme_label1Click(Sender: TObject);
      procedure themesave_label_infoClick(Sender: TObject);
      procedure themesave_labelClick(Sender: TObject);
      procedure theme_labelClick(Sender: TObject);
      procedure Timer1Timer(Sender: TObject);
      procedure Timer2Timer(Sender: TObject);
      procedure tmcreateClick(Sender: TObject);
      procedure tmopenarchiveClick(Sender: TObject);
      procedure tmopenpathClick(Sender: TObject);
      procedure tmpeazipClick(Sender: TObject);
      procedure tmquitClick(Sender: TObject);
      procedure TrayIcon1DblClick(Sender: TObject);
   private
      { private declarations }
   public
      { public declarations }
   end;

{$IFDEF MSWINDOWS}//used for transparence
type
   TSetLayeredWindowAttributes = function(hwnd: HWND; crKey: COLORREF; bAlpha: Byte; dwFlags: Longint): Longint;
   stdcall;
{$ENDIF}

function compose_unace_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
function compose_unarc_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
function compose_unzpaq_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
function setoutname(var s,archive_type:utf8string; i:integer):integer;
function control_outpath(var out_param:utf8string):integer;
function testencrypted:integer;
function testencrypted_open:integer;
procedure listdir(s:utf8string; recmode:boolean);
procedure navrefresh;
procedure navgoup;
procedure rsearch;

const
  MAX_CL        = 31744; //most of modern OSes handle about up to 32KB command line, checked conservatively at 31KB
  WS_EX_LAYERED = $80000;
  LWA_ALPHA     = $2;
  INTVERSION    = '0.45';
  PEAZIPVERSION = '2.7';
  PEAZIPREVISION= '.1';
  DEFAULT_THEME = 'firecrystal';
  SIDESIZE      = 0;
  ROWSIZE       = 20;
  DROWHEIGHT     = 18;
  ZCHAR         = '+';
  ZSTRING       = '----------';
  ZSTRINGL      = '------------------- ----- ------------ ------------  ------------------------';
  READE_LIST    = '7Z, ACE, ARC/WRC, ARJ, BZ/TBZ, CAB, CHM/CHW/HXS, COMPOUND (MSI, DOC, XLS, PPT), CPIO, GZ/TGZ, ISO, Java (JAR, EAR, WAR), LZH/LHA, Linux (DEB, PET/PUP, RPM, SLP), NSIS, OOo, PAK/PK3/PK4, PAQ/LPAQ/ZPAQ, PEA, QUAD/BALZ, RAR, TAR, WIM/SWM, XPI, Z/TZ, ZIP...';
  WRITEE_LIST   = '7Z, 7Z-sfx, ARC, BZ2, GZ, PAQ/LPAQ/ZPAQ, PEA, QUAD/BALZ, split, TAR, UPX, ZIP';
  APPMAIN       = 'PeaZip';
  APPLICATION1  = 'Pea 0.28 (LGPL, Giorgio Tani);';
  STR_7Z        = '7Z';
  STR_ARC       = 'ARC';
  STR_BZIP2     = 'BZip2';
  STR_GZIP      = 'GZip';
  STR_LPAQ      = 'LPAQ';
  STR_PAQ       = 'PAQ';
  STR_PEA       = 'PEA';
  STR_QUAD      = 'QUAD/BALZ';
  STR_TAR       = 'TAR';
  STR_UPX       = 'UPX';
  STR_ZIP       = 'ZIP';
  STR_ZPAQ      = '*PAQ';
  STR_CRC       = 'CRC';
  STR_TMP       = '.peazip_tmp';
  COL1          = 'clWindow';
  COL2          = 'clBtnFace';
  COL3          = 'clBtnFace';
  COL4          = '$00669999';
  COL5          = 'clWindowText';
  COLBOX        = '$00BBFFFF';
  COLGROUP1     = '$00C0F8F8';
  COLGROUP2     = '$00C0F8E8';
  COLFOLDERROW  = '$00DDFFFF';
  COLARCHIVEROW = '$00DDFFF8';
  {$IFDEF MSWINDOWS}
  EXEEXT        = '.exe';
  APPLICATION2  = '7z 9.07 (LGPL, Igor Pavlov);';
  APPLICATION3  = 'PAQ8F/JD/L/O, LPAQ1/5/8, ZPAQ 1.02 [Matt Mahoney et al. (GPL)];';
  APPLICATION4  = 'Strip (GPL, GNU binutils), UPX 3.03 (GPL, Markus F.X.J. Oberhumer, Laszlo Molnar and John F. Reiser);';
  APPLICATION5  = 'QUAD 1.12 (LGPL) / BALZ 1.15 (Public Domain, Ilia Muraviev);';
  APPLICATION6  = 'UNACEV2.DLL 2.6.0.0 (royalty-free UNACEV2.DLL license, ACE Compression Software);';
  APPLICATION7  = 'FreeArc 0.51 (GPL, Bulat Ziganshin).';
  {$ENDIF}
  {$IFDEF LINUX}
  EXEEXT        = '';
  APPLICATION2  = 'POSIX 7z 9.04 (LGPL, Myspace, based on Igor Pavlov''s 7z);';
  APPLICATION3  = 'PAQ8F/JD/L/O, LPAQ1/5/8 [Matt Mahoney et al. (GPL)];';
  APPLICATION4  = 'Strip (GPL, GNU binutils), UPX 3.03 (GPL, Markus F.X.J. Oberhumer, Laszlo Molnar and John F. Reiser);';
  APPLICATION5  = 'QUAD 1.12 (LGPL) / BALZ 1.15 (Public Domain, Ilia Muraviev);';
  APPLICATION6  = 'UNACE (royalty-free UNACE for Linux license, Marcel Lemke, ACE Compression Software);';
  APPLICATION7  = 'FreeArc 0.51 (GPL, Bulat Ziganshin).';
  {$ENDIF}
  {$IFDEF FREEBSD}
  EXEEXT        = '';
  APPLICATION2  = 'POSIX 7z 9.04 (LGPL, Myspace, based on Igor Pavlov''s 7z);';
  APPLICATION3  = '';
  APPLICATION4  = 'Strip (GPL, GNU binutils), UPX 3.03 (GPL, Markus F.X.J. Oberhumer, Laszlo Molnar and John F. Reiser);';
  APPLICATION5  = '';
  APPLICATION6  = '';
  APPLICATION7  = '';
  {$ENDIF}
  {$IFDEF NETBSD}
  EXEEXT        = '';
  APPLICATION2  = 'POSIX 7z 9.04 (LGPL, Myspace, based on Igor Pavlov''s 7z);';
  APPLICATION3  = '';
  APPLICATION4  = 'Strip (GPL, GNU binutils), UPX 3.03 (GPL, Markus F.X.J. Oberhumer, Laszlo Molnar and John F. Reiser);';
  APPLICATION5  = '';
  APPLICATION6  = '';
  APPLICATION7  = '';
  {$ENDIF}
  {$IFDEF DARWIN}
  EXEEXT        = '';
  APPLICATION2  = 'POSIX 7z 9.04 (LGPL, Myspace, based on Igor Pavlov''s 7z);';
  APPLICATION3  = '';
  APPLICATION4  = 'Strip (GPL, GNU binutils), UPX 3.03 (GPL, Markus F.X.J. Oberhumer, Laszlo Molnar and John F. Reiser);';
  APPLICATION5  = '';
  APPLICATION6  = '';
  APPLICATION7  = '';
  {$ENDIF}

var
   Form_peach: TForm_peach;
   patheditor:TEdit;
   Baddfile,Baddfolder,Baddsearch,Barchive_bigger,Barchive_bigger2,
   Barchive_big,Barchive,BArchiveSupported,BFileSupported,BExesupported,
   Bunsupported,Bback,
   Bbookmark,Bbookmarkadd,Bclearlayout,Bclearlayout_big,
   Bdefaults,Bdelete,Bdvd_supported,Bexpand,
   Bextractall,Bextractall_big,Bextractallto,Bextractallto_big,Bfd_supported,
   BFolder,Bft,Bgo,Bgopath,Bgoup,Bhd_supported,
   Bhistory,Binfo,Bkeyfile,Blisted,Blistedto,Bloadlayout,
   Blocker,Blocker2,Bmail,Bnav,Bp0,Bp1,Bp2,Bp3,Bp4,Bp5,Bp6,Bp7,Bp8,
   Bpea,Bpreview,Bram_supported,Brefresh_up,Brefresh,Bremote_supported,
   Bremovable_supported,Bresetpath,Broot,Brun,Bsavelayout,
   Bsearch,Bselected,Bselectedto,Bsystemtools,Btestall,Btestall_big,Bstop,
   Bweb,Bwipe,BCutarchive,BCutfile,BCutexe,BCutfolder,BCutunsupported,Bfolder2,
   Blink,Bcutlink: TBitmap;
   devicon:array[1..26]of TBitmap;
   TS:TTimeStamp;
   conf,book,cedit,t: text;
   browse_option,caption_release,caption_build,tempstring,archive_type,archive_t,
   compression_method,compression_level,parameters,kdir,in_name,in_file,in_folder,
   output_name,out_qualified_name,size_u,cks,data_compression,save_cks,bcomp,pw,
   keyfname,themedir,theme_name,theme_author,theme_license,theme_path,wrk_path,executable_path,
   persistent_source,fun,subfun,tools_path,color1,color2,color3,color4,color5,color1d,
   color2d,color3d,color4d,color5d,prev_type,
   prev_method,prev_level,method_7z,method_zip,ptmpcode,ptmpdir,graphicsfolder,graphicsfolderd,
   archive_content,wincomspec,delimiter,desktop_path,prevpanel,winver,indir,destdir,
   prev_destdir,local_desktop,home_path,confpath,currentcomp,currentfs,usr_documents,
   usr_name,lastobj,lastobjarch,custedit1,custedit2,custedit3,custedit4,custedit5,
   custedit6,custedit7,custedit8,prevcaption,advedit1before,advedit2before,advedit3before,
   advedit4before,advedit5before,advedit6before,advedit7before,advedit8before,advedit1after,
   advedit2after,advedit3after,advedit4after,advedit5after,advedit6after,advedit7after,
   advedit8after,winpfolder,winappdatafolder,fun_status,lastoutpath,status0,status1,
   status2,checkver,extsel:utf8string;
   patchclip:array of array [0..13] of utf8string;
   clipcontent,clipcontent1:TClipcontent;
   listrow,listsortcol,listsortcol1,listsortcol2,listsortbook,title_lines_7z,rowheight,rowheightd,
   itemheight,itemheightd,currow1,browsingmode7z,splittersize,lastbar,
   cbitemheight,editheight,barheight,toolbtnheight,stdbtnheight,refgroupheight,
   secgroupheight,listcol,ares,esna,euns:integer;
   ent_buffer: array [0..63] of byte;
   tvolumes,tdirs,tfiles,tsize:qword;
   opacity,opacityd,df,pf,pobj,pcompr,pvol,pstream,salgo,zaout,ws,ws_status,
   ignorepathextand,level_7z,
   level_bzip2,level_arc,solid_arc,rr_arc,algo_arc,level_gz,level_paq,
   level_quad,level_upx,level_zip,openw_all7z,showpwfield,autoclosegwrap,autoopen,
   browserstyle,desk_env,prebrowse,ignorepathdisp,
   ignorepathsel,savehistory,fav7z,favARC,favbz,favcustom,favgz,favpea,favquad,favsfx,
   favsfxarc,favsplit,favtar,favupx,favzip,favzpaq,
   extpw,algodisp,algoall,algoadler,algocrc16,
   algocrc24,algocrc32,algocrc64,algoed2k,algomd4,algomd5,algoripemd160,algosha1,
   algosha224,algosha256,algosha384,algosha512,algowhirl,showguihints,customsyntax,
   customsyntax1,erasepasses,extopt7z,extoptarc,autosizeitemheight,
   autosizetoolbarbutton,autosizeitemheightd,autosizetoolbarbuttond,clipmode,jobdefenc,
   archivenameenc:byte;
   ws_top,ws_left,ws_height,ws_width,archive_type_selected,prebrowsesize:integer;
   call_validated,keyf_opened,no_more_files,save_theme_clicked,
   using_tarbefore,seemencrypted,seemnamesencrypted,az,az1,az2,azbook,imgloaded,imgloading,
   settingvalues,archiveopened,browsinghistory,filecopying,openstarted,dragcancelled,
   listingdir,control_listingdir,waitdrawok,setlastbar,updatingarchive,stayopen,
   pcmenupopulated:boolean;
   {$IFDEF MSWINDOWS}//used for transparence
   hUser32: HMODULE;
   SetLayeredWindowAttributes: TSetLayeredWindowAttributes;
   osVerInfo: TOSVersionInfo;
   {$ENDIF}
   lang_file:utf8string;
   //text strings
   txt_2_7_encfn,txt_2_7_validatecl,txt_2_7_validatefn,txt_2_7_cancel,txt_2_7_ok,
   txt_2_7_separate,txt_2_7_output,txt_2_7_archivehint,txt_2_7_optional,txt_2_7_pwnotset,
   txt_2_7_pwarcset,txt_2_7_pwextset,txt_2_7_setpw,txt_2_7_pwsupported,txt_2_7_spc,
   txt_2_7_extfrom,txt_2_7_ext,txt_2_7_updating,txt_2_7_goarclayout,txt_2_7_goextlayout,
   txt_2_7_setadvf,txt_2_7_advdesc,txt_2_7_selpath,txt_2_7_dirsize,txt_2_7_exthint,
   txt_2_7_clipboard,txt_2_7_noinput,txt_2_7_es,txt_2_7_eu,txt_2_7_separateerror,
   txt_2_7_drag_archive,txt_2_7_list_tryflatorpw,txt_2_7_un7z_browse_flat,txt_2_7_oop,
   txt_2_7_experimental,
   txt_2_6_special,txt_2_6_folders,txt_2_6_rec,txt_2_6_plalways,
   txt_2_6_plsmart,txt_2_6_advanced,
   txt_2_5_sessionio,txt_2_5_advanced,txt_2_5_basic,txt_2_5_cannotrun,txt_2_5_custeditors,
   txt_2_5_delete,txt_2_5_delete_fromarchive,txt_2_5_langflag,txt_2_5_encpj,
   txt_2_5_execommand,txt_2_5_help,txt_2_5_langhint,
   txt_2_5_mini_help,txt_2_5_offline_help,txt_2_5_tray,txt_2_5_remove,txt_2_5_hintpaths,
   txt_2_5_selectapp,txt_2_5_strafter,txt_2_5_strbefore,txt_2_5_encoding,
   txt_2_5_nopw,
   txt_2_4_itemsheight,txt_2_4_tb,txt_2_4_deletebookmarks,txt_2_4_totalmem,txt_2_4_operation,
   txt_2_4_advclip,txt_2_4_stdclip,txt_2_4_copyfrom,txt_2_4_extractfrom,
   txt_2_4_draghint,txt_2_4_clearclipboard,txt_2_4_removefromclipboard,txt_2_4_path,
   txt_2_4_adding,txt_2_4_documents,txt_2_4_hexp,txt_2_4_gvideo,txt_2_4_wenc,txt_2_4_yanswers,
   txt_2_4_wdict,txt_2_4_wbook,txt_2_4_wsrc,txt_2_4_wcommons,txt_2_4_wnews,
   txt_2_3_pw_errorchar,txt_2_3_envstr,txt_2_3_never_pw,txt_2_3_home,txt_2_3_on_pw,txt_2_3_test_pw100,txt_2_3_test_pw,
   txt_list_isfolder,txt_action_extopen,txt_error_passwordnotmatch,txt_action_preview,
   txt_preview_hint,txt_removeselected,txt_savelayout,txt_better,txt_default2,txt_faster,
   txt_fastermem,txt_tempdir,txt_stream,txt_slowermem,txt_store,txt_newfolder,
   txt_7z_exitcodeunknown,
   txt_none,txt_fd,txt_7z_exitcode1,txt_attach10,txt_7z_exitcode2,
   txt_7z_exitcode255,txt_fat32,txt_dvd,txt_attach5,txt_cd650,txt_7z_exitcode7,txt_cd700,
   txt_type_description_7z,txt_dvddl,txt_7z_exitcode8,txt_abort,txt_about,txt_action,
   txt_action_hint,txt_add,txt_add_existing_archive,txt_add_archive,
   txt_add_files,txt_add_folder,txt_add_path,txt_add_tolayout,txt_add_toarchive,txt_add_tobookmarks,
   txt_address_hint,txt_adv_filters,txt_algo,txt_all,txt_all_default,txt_all_type,
   txt_all_date,txt_all_psize,txt_all_attributes,txt_all_size,txt_error_input_upx,
   txt_always_pw,txt_ignore_ext,txt_ignore_disp,txt_ignore_sel,txt_key_hint,
   txt_timestamp,txt_appoptions,txt_type_description_arc,txt_archive,txt_un7z_browse_ok,
   txt_interface,txt_archivecreation,txt_tarbefore_hint,
   txt_archive_hint,txt_compressionratio_hint,txt_archiving,txt_cl_long,txt_overwrite_askbefore,
   txt_associated,txt_attributes,txt_author,txt_ren_existing,txt_ren_extracted,txt_autofolder,
   txt_back,txt_backend,txt_backupexe,txt_bettercompression,txt_blogs,txt_blowfish,
   txt_bookmarks,txt_browse,txt_browser,txt_aborted_error,txt_list_browsing,
   txt_archive_root,txt_type_description_bzip2,txt_pw_empty,txt_add_error,txt_un7z_browse_failure,
   txt_list_error,txt_conf_cannotsave,txt_check_hint,txt_check,txt_check_select,txt_clear,
   txt_clearlayout,txt_pj_hint,txt_autoclose,txt_cl,txt_compare,txt_compress,
   txt_compress_executable,txt_compress_openforwriting,txt_compression,txt_compmanagement,
   txt_pw_confirm,txt_console,txt_console_interface,txt_content,
   txt_controlpanel,txt_copy,txt_copyto,txt_create,txt_create_archive,txt_title_create,
   txt_create_keyfile,txt_create_folder,txt_create_theme,txt_rr,txt_create_sfx,txt_cr_current,
   txt_custom,txt_type_description_custom,txt_customapp,txt_custom_parameters,txt_customsize,
   txt_cut,txt_datetime,txt_default,txt_default_compression,
   txt_default_format,txt_theme_default,
   txt_hard_reset_hint,txt_desktop,txt_dictionary,txt_dirs,txt_dis,txt_disk_cleanup,
   txt_disk_defrag,txt_disk_management,txt_dispaly,txt_displayedmnu_obj,txt_displayedobjects,
   txt_nocompress,txt_delete,txt_wipe,txt_done,txt_edit,txt_elapsed,
   txt_error_emptycl,txt_encrypt,txt_encrypted,txt_encryption,txt_note,
   txt_random_keys,txt_random_keys_hint,txt_ent,txt_ent_tools,txt_eqorlarger,
   txt_eqorrecent,txt_eqorolder,txt_eqorsmaller,txt_equal,txt_erase_hint,txt_extraction_error,
   txt_exclude_hint,txt_exclude_recourse,txt_exclusion_recourse,txt_exclusion,txt_exe,
   txt_overwrite_qry,txt_confirm_overwrite,txt_explore_outpath,txt_explore_path,txt_ext,
   txt_caption_extract,txt_ext_nopath,txt_ext_all,txt_ext_allhere,txt_ext_allto,txt_extopen_custom,
   txt_extopen_with,txt_ext_disp_here,txt_ext_disp,txt_ext_disp_to,txt_ext_here,txt_ext_sel_here,
   txt_ext_sel,txt_ext_sel_to,txt_extobj,txt_newfoldermenu,txt_extto,txt_level_fast,
   txt_fastcompr,txt_fastopen,txt_level_fastest,txt_favformats,txt_file,txt_filebrowser,
   txt_filelist_savedas,txt_filetools,txt_files,txt_nfiles,txt_fs,txt_type_custom_selected,
   txt_filters_recourse,txt_filters,txt_flat,txt_list_flat,txt_unit_floppy,txt_foldername,
   txt_nfolders,txt_error_input_multi,txt_format_options,txt_fwd,txt_list_found,txt_free,
   txt_free2,txt_name_full,txt_function,txt_general,txt_multithreading,
   txt_go_browser,txt_go_path,txt_guicl,txt_guipealauncher,txt_graphic,
   txt_gridaltcolor,txt_gridrowheight,txt_gui,txt_type_description_gzip,txt_hardreset,
   txt_here,txt_list_history,txt_homeroot,txt_quickbrowse_hint,
   txt_backupexe_hint,txt_attach,txt_images,txt_include_hint,txt_filters_hint,txt_inclusion_recourse,
   txt_inclusion,txt_error_function,txt_info,txt_infoall,txt_infodisp,txt_infosel,
   txt_inputinfo,txt_input_list,txt_iop,txt_ipo,txt_input,txt_integrity,txt_chunk_size,
   txt_invertsel,txt_type_exe,txt_return_to_archive,txt_not_accessible,txt_type_unsupported,
   txt_checkname_failed,txt_not_accessible_list,txt_theme_create_error,txt_theme_exists,
   txt_job_code,txt_job_definition_saved,txt_job_success,txt_join,txt_joinfiles,
   txt_keyfile,txt_keyfile_not_found,txt_keyfile_notcreated,txt_larger,
   txt_lastused,txt_launch,txt_layout,txt_level,txt_license,txt_caption_list,txt_list_details,
   txt_list_all,txt_list_disp,txt_list_sel,txt_toggle_warning,txt_loadfile,txt_loadlayout,
   txt_unit_hd,txt_localization,txt_lpaqver,txt_type_description_lpaq,txt_maininterface,
   txt_maxcomp,txt_level_maximum,txt_restartrequired,txt_required_memory,
   txt_method,txt_misc,txt_modify,txt_morecontrols,txt_morerecent,
   txt_ent_hint,txt_moveto,txt_mypc,txt_list_na,txt_name,txt_naming,txt_unit_remote,
   txt_newarchive,txt_news,txt_no,txt_noinput,txt_nocompress_hint,
   txt_split_noinput,txt_open_noinput,txt_list_nomatch,txt_singlethread,txt_none2,
   txt_nonsolid,txt_level_normal,txt_copy_error,txt_description,txt_compare_second,
   txt_peaobj,txt_displayed_obj,txt_olderthan,txt_ondblclick,
   txt_opacity,txt_open,txt_openarchive,txt_title_open,txt_open_bookmark,txt_cphere,
   txt_open_file,txt_open_files,txt_open_path,txt_opensource,txt_openwith,
   txt_aborted,txt_unit_cd,txt_options,txt_other,txt_otherparams,
   txt_oip,txt_opi,txt_output,txt_overwrite,txt_compressed_size,txt_paqver,
   txt_type_description_paq,txt_pio,txt_poi,txt_parameters,txt_error_partial,
   txt_passes,txt_pw,txt_pwlength,txt_un7z_browse_pw,txt_un7z_browse_pw_other,txt_paste,
   txt_path,txt_pea_appcolor,txt_pea_textcolor,txt_type_description_pea,txt_peazip_new,
   txt_peazip_help,txt_peazip_web,txt_performall,txt_upxorstrip,
   txt_not_removable_file,txt_not_removable,txt_custom_executable_missing,
   txt_type_unsupported_select,txt_no_theme_name,txt_please_wait,txt_copy_wait,txt_previewwith,
   txt_projectadmin,txt_type_description_quad,txt_quickdelete,txt_quit,txt_unit_ram,
   txt_read,txt_recentarchives,txt_rr_hint,txt_search_refine,txt_fefreshf5,
   txt_release,txt_unit_removable,txt_remove_bookmark,txt_remove_external_unit,
   txt_rename,txt_caption_repair,txt_restartrequired2,txt_reset,txt_reset_archivename,
   txt_reset_theme,
   txt_soft_reset,txt_restore_att,txt_run_as,txt_run_as2,txt_sample,
   txt_saveas,txt_savehistory,txt_save_infolder,txt_savejob,txt_savejobdefinition,
   txt_savejobdefinition_hint,txt_save_winstate,
   txt_search,txt_searchanddrag,txt_nrsearch,txt_search_hint,txt_search_web,txt_list_searching,
   txt_securedelete,txt_default_description,txt_selectall,txt_selectdir,
   txt_selected_obj,txt_selected_objects,txt_sfx,txt_sendbymail,txt_set_defaults,
   txt_settings,txt_sfx_interface,txt_showhints,
   txt_show_messages,txt_showpw,txt_singlevol,txt_size,txt_sizeb,txt_skip_existing,
   txt_slowercomp,txt_smaller,txt_solid,txt_solid_block,txt_solid_auto,txt_solid_extension,
   txt_tarbeforepj,txt_listtest,txt_sortbysel,txt_list_sorting,
   txt_speed,txt_split,txt_type_description_split,txt_split_file,txt_list_nostats,
   txt_status,txt_level_store,txt_stream_control,txt_strip,txt_keyfile_created,txt_suggestpw,
   txt_noupx,txt_syntax,txt_sysbenchmark,txt_benchmark,txt_systools,txt_tarbefore,
   txt_type_description_tar,txt_taskman,txt_caption_test,txt_testall,txt_testdisp,
   txt_testpw,txt_testsel,txt_col_hint,txt_bookmarks_hint,txt_archive_noinput_tolist,
   txt_theme,txt_icons_found,txt_themename,txt_icons_not_found,txt_theme_create_success,
   txt_theme_not_loadable,txt_theming,txt_extand_error,txt_threads,txt_titlescolor,
   txt_to,txt_toggle_browseflat,txt_toggle_historybookmarks,txt_toolbarscolor,txt_tools,
   txt_best,txt_type,txt_level_ultra,txt_error_openfile,txt_cl_hint,
   txt_ace_missing,txt_units,txt_unit_unknown,txt_un7z_pw_untested,txt_up,txt_update,
   txt_type_description_upx,txt_advfilters,txt_openfiles_hint,txt_usenet,txt_user_name,
   txt_using,txt_volumepea,txt_volume_size,txt_type_ext_uns,txt_websites,
   txt_word,txt_write,txt_ramdompw_hint,txt_exe_hint,
   txt_pj_hint2,txt_type_description_zip,txt_zipcrypto_hint,txt_name_provide,txt_cnewfolder,
   txt_searchfor,txt_websearch,txt_on,txt_convert:utf8string;

implementation

{ TForm_peach }

procedure conf_critical_error_msg; //hardcoded
begin
MessageDlg('PeaZip cannot find or parse critical configuration files (probably because deleted, moved or corrupted) and should be reinstalled', mtError, [mbOK], 0);
end;

procedure lang_critical_error_msg; //hardcoded
begin
MessageDlg('PeaZip cannot parse language file '+lang_file+' and will now try to fall back to default language file en.txt', mtError, [mbOK], 0)
end;

{///////////////////////////////////////////////////////////////////////////////

text and translation

///////////////////////////////////////////////////////////////////////////////}

function valorize_text:integer;
var
   s:utf8string;
begin
valorize_text:=-1;
try
readln(t,s);
readln(t,s); txt_2_7_experimental:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_optional:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_list_tryflatorpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_separate:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_advdesc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_pwsupported:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_cancel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_encfn:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_setpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_ext:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_extfrom:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_es:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_eu:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_clipboard:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_goarclayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_goextlayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_ok:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_drag_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_oop:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_validatefn:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_validatecl:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_output:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_pwnotset:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_pwarcset:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_pwextset:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_archivehint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_exthint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_spc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_setadvf:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_selpath:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_separateerror:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_dirsize:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_un7z_browse_flat:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_7_updating:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_6_folders:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_6_advanced:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_6_plalways:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_6_plsmart:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_6_rec:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_6_special:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_sessionio:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_advanced:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_basic:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_cannotrun:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_custeditors:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_delete:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_delete_fromarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_langflag:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_encpj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_execommand:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_langhint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_mini_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_offline_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_tray:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_remove:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_hintpaths:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_selectapp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_strafter:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_strbefore:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_encoding:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_5_nopw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_draghint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_tb:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_adding:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_advclip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_yanswers:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_itemsheight:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_clearclipboard:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_wcommons:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_copyfrom:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_deletebookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_documents:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_wenc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_extractfrom:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_hexp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_operation:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_removefromclipboard:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_stdclip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_totalmem:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_gvideo:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_wbook:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_wnews:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_wsrc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_4_wdict:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_pw_errorchar:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_envstr:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_never_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_home:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_on_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_test_pw100:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_2_3_test_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_exclude_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_action_extopen:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_passwordnotmatch:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_action_preview:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_preview_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_better:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_default2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_faster:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fastermem:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_tempdir:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_stream:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_slowermem:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_store:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_newfolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_7z_exitcodeunknown:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_isfolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_none:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fd:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_7z_exitcode1:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_attach10:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_7z_exitcode2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_7z_exitcode255:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fat32:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_dvd:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_attach5:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cd650:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_7z_exitcode7:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cd700:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_7z:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_dvddl:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_7z_exitcode8:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_abort:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_about:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_action:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_action_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_existing_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_files:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_folder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_tolayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_toarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_tobookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_address_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_adv_filters:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_algo:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all_default:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all_type:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all_date:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all_psize:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all_attributes:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_all_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_input_upx:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_always_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ignore_ext:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ignore_disp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ignore_sel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_key_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_timestamp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_appoptions:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_arc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_un7z_browse_ok:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_interface:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_archivecreation:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_tarbefore_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_archive_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compressionratio_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_archiving:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cl_long:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_overwrite_askbefore:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_associated:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_attributes:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_author:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ren_existing:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ren_extracted:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_autofolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_back:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_backend:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_backupexe:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_bettercompression:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_blogs:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_blowfish:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_bookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_browse:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_browser:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_aborted_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_browsing:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_archive_root:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_bzip2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pw_empty:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_add_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_un7z_browse_failure:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_conf_cannotsave:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_check_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_check:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_check_select:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_clear:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_clearlayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pj_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_autoclose:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cl:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compare:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compress:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compress_executable:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compress_openforwriting:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compression:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compmanagement:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pw_confirm:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_console:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_console_interface:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_content:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_controlpanel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_convert:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_copy:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_copyto:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_create:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_create_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_title_create:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_create_keyfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_create_folder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_create_theme:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_rr:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_create_sfx:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cr_current:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_custom:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_custom:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_customapp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_custom_parameters:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_customsize:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cut:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_datetime:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_default:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_default_compression:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_default_format:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theme_default:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_hard_reset_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_desktop:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_dictionary:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_dirs:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_dis:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_disk_cleanup:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_disk_defrag:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_disk_management:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_dispaly:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_displayedmnu_obj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_displayedobjects:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_nocompress:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_delete:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_wipe:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_done:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_edit:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_elapsed:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_emptycl:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_encrypt:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_encrypted:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_encryption:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_note:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_random_keys:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_random_keys_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ent:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ent_tools:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_eqorlarger:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_eqorrecent:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_eqorolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_eqorsmaller:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_equal:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_erase_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_extraction_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_exclude_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_exclusion_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_exclusion:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_exe:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_overwrite_qry:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_confirm_overwrite:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_explore_outpath:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_explore_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_caption_extract:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_nopath:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_all:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_allhere:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_allto:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_extopen_custom:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_extopen_with:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_disp_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_disp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_disp_to:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_sel_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_sel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ext_sel_to:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_extobj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_newfoldermenu:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_extto:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level_fast:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fastcompr:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fastopen:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level_fastest:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_favformats:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_filebrowser:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_filetools:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_files:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_nfiles:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fs:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_custom_selected:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_filters_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_filters:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_flat:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_flat:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_floppy:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_foldername:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_nfolders:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_input_multi:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_format_options:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fwd:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_free:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_free2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_name_full:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_function:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_general:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_multithreading:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_go_browser:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_go_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_guicl:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_guipealauncher:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_graphic:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_gridaltcolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_gridrowheight:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_gui:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_gzip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_history:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_homeroot:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_quickbrowse_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_backupexe_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_attach:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_images:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_include_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_filters_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_inclusion_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_inclusion:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_function:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_info:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_infoall:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_infodisp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_infosel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_inputinfo:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_input_list:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_iop:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ipo:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_input:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_integrity:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_chunk_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_invertsel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_exe:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_return_to_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_not_accessible:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_unsupported:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_checkname_failed:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_not_accessible_list:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theme_create_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theme_exists:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_job_code:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_job_definition_saved:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_job_success:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_join:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_joinfiles:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_keyfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_keyfile_not_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_keyfile_notcreated:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_larger:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_lastused:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_launch:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_layout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_filelist_savedas:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_license:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_caption_list:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_details:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_all:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_disp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_sel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_toggle_warning:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_loadfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_loadlayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_hd:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_localization:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_lpaqver:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_lpaq:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_maininterface:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_maxcomp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level_maximum:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_restartrequired:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_required_memory:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_method:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_misc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_modify:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_morecontrols:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_morerecent:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ent_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_moveto:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_mypc:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_na:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_name:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_naming:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_remote:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_newarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cnewfolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_news:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_no:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_nocompress_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_split_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_open_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_nomatch:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_singlethread:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_none2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_nonsolid:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level_normal:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_copy_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_description:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compare_second:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_peaobj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_displayed_obj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_olderthan:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_on:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ondblclick:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_opacity:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_open:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_openarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_title_open:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_open_bookmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cphere:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_open_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_open_files:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_open_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_opensource:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_openwith:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_aborted:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_cd:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_options:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_other:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_otherparams:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_oip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_opi:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_output:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_overwrite:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_compressed_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_paqver:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_paq:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pio:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_poi:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_parameters:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_partial:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_passes:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pwlength:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_un7z_browse_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_un7z_browse_pw_other:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_paste:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pea_appcolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pea_textcolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_pea:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_peazip_new:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_peazip_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_peazip_web:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_performall:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_name_provide:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_upxorstrip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_not_removable_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_not_removable:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_custom_executable_missing:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_unsupported_select:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_no_theme_name:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_please_wait:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_copy_wait:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_previewwith:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_projectadmin:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_quad:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_quickdelete:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_quit:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_ram:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_read:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_recentarchives:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_rr_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_search_refine:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_fefreshf5:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_release:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_removable:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_remove_bookmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_remove_external_unit:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_removeselected:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_rename:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_caption_repair:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_restartrequired2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_reset:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_reset_archivename:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_hardreset:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_reset_theme:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_soft_reset:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_restore_att:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_run_as:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_run_as2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sample:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_saveas:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_savehistory:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_save_infolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_savejob:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_savejobdefinition:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_savejobdefinition_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_savelayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_save_winstate:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_search:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_searchanddrag:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_searchfor:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_nrsearch:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_search_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_search_web:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_searching:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_securedelete:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_default_description:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_selectall:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_selectdir:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_selected_obj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_selected_objects:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sfx:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sendbymail:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_set_defaults:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_settings:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sfx_interface:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_showhints:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_show_messages:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_showpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_singlevol:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sizeb:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_skip_existing:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_slowercomp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_smaller:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_solid:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_solid_block:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_solid_auto:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_solid_extension:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_tarbeforepj:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_listtest:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sortbysel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_sorting:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_speed:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_split:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_split:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_split_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_list_nostats:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_status:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level_store:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_stream_control:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_strip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_keyfile_created:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_suggestpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_noupx:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_syntax:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_sysbenchmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_benchmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_systools:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_tarbefore:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_tar:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_taskman:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_caption_test:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_testall:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_testdisp:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_testpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_testsel:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_col_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_bookmarks_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_archive_noinput_tolist:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theme:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_icons_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_themename:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_icons_not_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theme_create_success:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theme_not_loadable:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_theming:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_extand_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_threads:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_titlescolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_to:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_toggle_browseflat:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_toggle_historybookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_toolbarscolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_tools:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_best:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_level_ultra:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_error_openfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_cl_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ace_missing:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_units:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_unit_unknown:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_un7z_pw_untested:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_up:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_update:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_upx:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_advfilters:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_openfiles_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_usenet:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_user_name:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_using:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_volumepea:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_volume_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_ext_uns:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_websearch:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_websites:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_word:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_write:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_ramdompw_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_exe_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_pj_hint2:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_type_description_zip:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s); txt_zipcrypto_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s));
readln(t,s);
readln(t,s); if s<>'=== end PeaZip text group ===' then exit;
valorize_text:=0;
except
valorize_text:=-1;
end;
end;

procedure get_fallback_text(s:utf8string);
begin
if copy(s,0,pos(':',s)-1)='txt_2_7_experimental' then begin txt_2_7_experimental:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_optional' then begin txt_2_7_optional:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_list_tryflatorpw' then begin txt_2_7_list_tryflatorpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_separate' then begin txt_2_7_separate:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_advdesc' then begin txt_2_7_advdesc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_pwsupported' then begin txt_2_7_pwsupported:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_cancel' then begin txt_2_7_cancel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_encfn' then begin txt_2_7_encfn:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_setpw' then begin txt_2_7_setpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_ext' then begin txt_2_7_ext:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_extfrom' then begin txt_2_7_extfrom:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_es' then begin txt_2_7_es:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_eu' then begin txt_2_7_eu:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_clipboard' then begin txt_2_7_clipboard:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_goarclayout' then begin txt_2_7_goarclayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_goextlayout' then begin txt_2_7_goextlayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_ok' then begin txt_2_7_ok:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_drag_archive' then begin txt_2_7_drag_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_oop' then begin txt_2_7_oop:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_validatefn' then begin txt_2_7_validatefn:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_validatecl' then begin txt_2_7_validatecl:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_output' then begin txt_2_7_output:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_pwnotset' then begin txt_2_7_pwnotset:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_pwarcset' then begin txt_2_7_pwarcset:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_pwextset' then begin txt_2_7_pwextset:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_archivehint' then begin txt_2_7_archivehint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_exthint' then begin txt_2_7_exthint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_spc' then begin txt_2_7_spc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_setadvf' then begin txt_2_7_setadvf:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_selpath' then begin txt_2_7_selpath:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_separateerror' then begin txt_2_7_separateerror:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_noinput' then begin txt_2_7_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_dirsize' then begin txt_2_7_dirsize:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_un7z_browse_flat' then begin txt_2_7_un7z_browse_flat:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_7_updating' then begin txt_2_7_updating:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_6_folders' then begin txt_2_6_folders:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_6_advanced' then begin txt_2_6_advanced:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_6_plalways' then begin txt_2_6_plalways:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_6_plsmart' then begin txt_2_6_plsmart:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_6_rec' then begin txt_2_6_rec:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_6_special' then begin txt_2_6_special:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_sessionio' then begin txt_2_5_sessionio:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_advanced' then begin txt_2_5_advanced:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_basic' then begin txt_2_5_basic:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_cannotrun' then begin txt_2_5_cannotrun:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_custeditors' then begin txt_2_5_custeditors:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_delete' then begin txt_2_5_delete:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_delete_fromarchive' then begin txt_2_5_delete_fromarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_langflag' then begin txt_2_5_langflag:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_encpj' then begin txt_2_5_encpj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_execommand' then begin txt_2_5_execommand:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_help' then begin txt_2_5_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_langhint' then begin txt_2_5_langhint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_mini_help' then begin txt_2_5_mini_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_offline_help' then begin txt_2_5_offline_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_tray' then begin txt_2_5_tray:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_remove' then begin txt_2_5_remove:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_hintpaths' then begin txt_2_5_hintpaths:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_selectapp' then begin txt_2_5_selectapp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_strafter' then begin txt_2_5_strafter:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_strbefore' then begin txt_2_5_strbefore:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_encoding' then begin txt_2_5_encoding:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_5_nopw' then begin txt_2_5_nopw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_draghint' then begin txt_2_4_draghint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_tb' then begin txt_2_4_tb:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_adding' then begin txt_2_4_adding:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_advclip' then begin txt_2_4_advclip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_yanswers' then begin txt_2_4_yanswers:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_itemsheight' then begin txt_2_4_itemsheight:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_clearclipboard' then begin txt_2_4_clearclipboard:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_wcommons' then begin txt_2_4_wcommons:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_copyfrom' then begin txt_2_4_copyfrom:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_deletebookmarks' then begin txt_2_4_deletebookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_documents' then begin txt_2_4_documents:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_wenc' then begin txt_2_4_wenc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_extractfrom' then begin txt_2_4_extractfrom:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_hexp' then begin txt_2_4_hexp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_operation' then begin txt_2_4_operation:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_path' then begin txt_2_4_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_removefromclipboard' then begin txt_2_4_removefromclipboard:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_stdclip' then begin txt_2_4_stdclip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_totalmem' then begin txt_2_4_totalmem:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_gvideo' then begin txt_2_4_gvideo:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_wbook' then begin txt_2_4_wbook:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_wnews' then begin txt_2_4_wnews:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_wsrc' then begin txt_2_4_wsrc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_4_wdict' then begin txt_2_4_wdict:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_pw_errorchar' then begin txt_2_3_pw_errorchar:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_envstr' then begin txt_2_3_envstr:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_never_pw' then begin txt_2_3_never_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_home' then begin txt_2_3_home:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_on_pw' then begin txt_2_3_on_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_test_pw100' then begin txt_2_3_test_pw100:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_2_3_test_pw' then begin txt_2_3_test_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_exclude_recourse' then begin txt_exclude_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_action_extopen' then begin txt_action_extopen:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_passwordnotmatch' then begin txt_error_passwordnotmatch:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_action_preview' then begin txt_action_preview:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_preview_hint' then begin txt_preview_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_better' then begin txt_better:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_default2' then begin txt_default2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_faster' then begin txt_faster:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fastermem' then begin txt_fastermem:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_tempdir' then begin txt_tempdir:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_stream' then begin txt_stream:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_slowermem' then begin txt_slowermem:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_store' then begin txt_store:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_newfolder' then begin txt_newfolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_7z_exitcodeunknown' then begin txt_7z_exitcodeunknown:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_isfolder' then begin txt_list_isfolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_none' then begin txt_none:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fd' then begin txt_fd:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_7z_exitcode1' then begin txt_7z_exitcode1:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_attach10' then begin txt_attach10:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_7z_exitcode2' then begin txt_7z_exitcode2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_7z_exitcode255' then begin txt_7z_exitcode255:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fat32' then begin txt_fat32:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_dvd' then begin txt_dvd:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_attach5' then begin txt_attach5:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cd650' then begin txt_cd650:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_7z_exitcode7' then begin txt_7z_exitcode7:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cd700' then begin txt_cd700:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_7z' then begin txt_type_description_7z:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_dvddl' then begin txt_dvddl:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_7z_exitcode8' then begin txt_7z_exitcode8:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_abort' then begin txt_abort:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_about' then begin txt_about:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_action' then begin txt_action:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_action_hint' then begin txt_action_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add' then begin txt_add:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_existing_archive' then begin txt_add_existing_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_archive' then begin txt_add_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_files' then begin txt_add_files:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_folder' then begin txt_add_folder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_path' then begin txt_add_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_tolayout' then begin txt_add_tolayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_toarchive' then begin txt_add_toarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_tobookmarks' then begin txt_add_tobookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_address_hint' then begin txt_address_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_adv_filters' then begin txt_adv_filters:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_algo' then begin txt_algo:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all' then begin txt_all:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all_default' then begin txt_all_default:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all_type' then begin txt_all_type:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all_date' then begin txt_all_date:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all_psize' then begin txt_all_psize:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all_attributes' then begin txt_all_attributes:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_all_size' then begin txt_all_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_input_upx' then begin txt_error_input_upx:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_always_pw' then begin txt_always_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ignore_ext' then begin txt_ignore_ext:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ignore_disp' then begin txt_ignore_disp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ignore_sel' then begin txt_ignore_sel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_key_hint' then begin txt_key_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_timestamp' then begin txt_timestamp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_appoptions' then begin txt_appoptions:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_arc' then begin txt_type_description_arc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_archive' then begin txt_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_un7z_browse_ok' then begin txt_un7z_browse_ok:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_interface' then begin txt_interface:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_archivecreation' then begin txt_archivecreation:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_tarbefore_hint' then begin txt_tarbefore_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_archive_hint' then begin txt_archive_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compressionratio_hint' then begin txt_compressionratio_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_archiving' then begin txt_archiving:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cl_long' then begin txt_cl_long:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_overwrite_askbefore' then begin txt_overwrite_askbefore:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_associated' then begin txt_associated:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_attributes' then begin txt_attributes:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_author' then begin txt_author:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ren_existing' then begin txt_ren_existing:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ren_extracted' then begin txt_ren_extracted:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_autofolder' then begin txt_autofolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_back' then begin txt_back:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_backend' then begin txt_backend:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_backupexe' then begin txt_backupexe:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_bettercompression' then begin txt_bettercompression:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_blogs' then begin txt_blogs:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_blowfish' then begin txt_blowfish:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_bookmarks' then begin txt_bookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_browse' then begin txt_browse:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_browser' then begin txt_browser:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_aborted_error' then begin txt_aborted_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_browsing' then begin txt_list_browsing:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_archive_root' then begin txt_archive_root:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_bzip2' then begin txt_type_description_bzip2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pw_empty' then begin txt_pw_empty:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_add_error' then begin txt_add_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_un7z_browse_failure' then begin txt_un7z_browse_failure:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_error' then begin txt_list_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_conf_cannotsave' then begin txt_conf_cannotsave:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_check_hint' then begin txt_check_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_check' then begin txt_check:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_check_select' then begin txt_check_select:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_clear' then begin txt_clear:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_clearlayout' then begin txt_clearlayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pj_hint' then begin txt_pj_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_autoclose' then begin txt_autoclose:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cl' then begin txt_cl:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compare' then begin txt_compare:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compress' then begin txt_compress:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compress_executable' then begin txt_compress_executable:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compress_openforwriting' then begin txt_compress_openforwriting:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compression' then begin txt_compression:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compmanagement' then begin txt_compmanagement:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pw_confirm' then begin txt_pw_confirm:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_console' then begin txt_console:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_console_interface' then begin txt_console_interface:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_content' then begin txt_content:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_controlpanel' then begin txt_controlpanel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_convert' then begin txt_convert:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_copy' then begin txt_copy:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_copyto' then begin txt_copyto:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_create' then begin txt_create:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_create_archive' then begin txt_create_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_title_create' then begin txt_title_create:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_create_keyfile' then begin txt_create_keyfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_create_folder' then begin txt_create_folder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_create_theme' then begin txt_create_theme:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_rr' then begin txt_rr:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_create_sfx' then begin txt_create_sfx:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cr_current' then begin txt_cr_current:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_custom' then begin txt_custom:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_custom' then begin txt_type_description_custom:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_customapp' then begin txt_customapp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_custom_parameters' then begin txt_custom_parameters:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_customsize' then begin txt_customsize:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cut' then begin txt_cut:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_datetime' then begin txt_datetime:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_default' then begin txt_default:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_default_compression' then begin txt_default_compression:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_default_format' then begin txt_default_format:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theme_default' then begin txt_theme_default:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_hard_reset_hint' then begin txt_hard_reset_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_desktop' then begin txt_desktop:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_dictionary' then begin txt_dictionary:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_dirs' then begin txt_dirs:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_dis' then begin txt_dis:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_disk_cleanup' then begin txt_disk_cleanup:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_disk_defrag' then begin txt_disk_defrag:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_disk_management' then begin txt_disk_management:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_dispaly' then begin txt_dispaly:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_displayedmnu_obj' then begin txt_displayedmnu_obj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_displayedobjects' then begin txt_displayedobjects:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_nocompress' then begin txt_nocompress:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_delete' then begin txt_delete:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_wipe' then begin txt_wipe:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_done' then begin txt_done:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_edit' then begin txt_edit:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_elapsed' then begin txt_elapsed:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_emptycl' then begin txt_error_emptycl:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_encrypt' then begin txt_encrypt:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_encrypted' then begin txt_encrypted:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_encryption' then begin txt_encryption:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_note' then begin txt_note:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_random_keys' then begin txt_random_keys:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_random_keys_hint' then begin txt_random_keys_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ent' then begin txt_ent:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ent_tools' then begin txt_ent_tools:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_eqorlarger' then begin txt_eqorlarger:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_eqorrecent' then begin txt_eqorrecent:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_eqorolder' then begin txt_eqorolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_eqorsmaller' then begin txt_eqorsmaller:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_equal' then begin txt_equal:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_erase_hint' then begin txt_erase_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_extraction_error' then begin txt_extraction_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_exclude_hint' then begin txt_exclude_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_exclusion_recourse' then begin txt_exclusion_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_exclusion' then begin txt_exclusion:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_exe' then begin txt_exe:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_overwrite_qry' then begin txt_overwrite_qry:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_confirm_overwrite' then begin txt_confirm_overwrite:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_explore_outpath' then begin txt_explore_outpath:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_explore_path' then begin txt_explore_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext' then begin txt_ext:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_caption_extract' then begin txt_caption_extract:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_nopath' then begin txt_ext_nopath:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_all' then begin txt_ext_all:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_allhere' then begin txt_ext_allhere:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_allto' then begin txt_ext_allto:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_extopen_custom' then begin txt_extopen_custom:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_extopen_with' then begin txt_extopen_with:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_disp_here' then begin txt_ext_disp_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_disp' then begin txt_ext_disp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_disp_to' then begin txt_ext_disp_to:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_here' then begin txt_ext_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_sel_here' then begin txt_ext_sel_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_sel' then begin txt_ext_sel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ext_sel_to' then begin txt_ext_sel_to:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_extobj' then begin txt_extobj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_newfoldermenu' then begin txt_newfoldermenu:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_extto' then begin txt_extto:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level_fast' then begin txt_level_fast:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fastcompr' then begin txt_fastcompr:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fastopen' then begin txt_fastopen:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level_fastest' then begin txt_level_fastest:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_favformats' then begin txt_favformats:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_file' then begin txt_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_filebrowser' then begin txt_filebrowser:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_filetools' then begin txt_filetools:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_files' then begin txt_files:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_nfiles' then begin txt_nfiles:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fs' then begin txt_fs:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_custom_selected' then begin txt_type_custom_selected:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_filters_recourse' then begin txt_filters_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_filters' then begin txt_filters:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_flat' then begin txt_flat:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_flat' then begin txt_list_flat:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_floppy' then begin txt_unit_floppy:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_foldername' then begin txt_foldername:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_nfolders' then begin txt_nfolders:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_input_multi' then begin txt_error_input_multi:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_format_options' then begin txt_format_options:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fwd' then begin txt_fwd:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_found' then begin txt_list_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_free' then begin txt_free:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_free2' then begin txt_free2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_name_full' then begin txt_name_full:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_function' then begin txt_function:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_general' then begin txt_general:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_multithreading' then begin txt_multithreading:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_go_browser' then begin txt_go_browser:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_go_path' then begin txt_go_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_guicl' then begin txt_guicl:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_guipealauncher' then begin txt_guipealauncher:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_graphic' then begin txt_graphic:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_gridaltcolor' then begin txt_gridaltcolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_gridrowheight' then begin txt_gridrowheight:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_gui' then begin txt_gui:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_gzip' then begin txt_type_description_gzip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_here' then begin txt_here:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_history' then begin txt_list_history:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_homeroot' then begin txt_homeroot:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_quickbrowse_hint' then begin txt_quickbrowse_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_backupexe_hint' then begin txt_backupexe_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_attach' then begin txt_attach:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_images' then begin txt_images:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_include_hint' then begin txt_include_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_filters_hint' then begin txt_filters_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_inclusion_recourse' then begin txt_inclusion_recourse:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_inclusion' then begin txt_inclusion:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_function' then begin txt_error_function:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_info' then begin txt_info:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_infoall' then begin txt_infoall:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_infodisp' then begin txt_infodisp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_infosel' then begin txt_infosel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_inputinfo' then begin txt_inputinfo:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_input_list' then begin txt_input_list:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_iop' then begin txt_iop:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ipo' then begin txt_ipo:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_input' then begin txt_input:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_integrity' then begin txt_integrity:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_chunk_size' then begin txt_chunk_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_invertsel' then begin txt_invertsel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_exe' then begin txt_type_exe:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_return_to_archive' then begin txt_return_to_archive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_not_accessible' then begin txt_not_accessible:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_unsupported' then begin txt_type_unsupported:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_checkname_failed' then begin txt_checkname_failed:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_not_accessible_list' then begin txt_not_accessible_list:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theme_create_error' then begin txt_theme_create_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theme_exists' then begin txt_theme_exists:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_job_code' then begin txt_job_code:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_job_definition_saved' then begin txt_job_definition_saved:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_job_success' then begin txt_job_success:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_join' then begin txt_join:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_joinfiles' then begin txt_joinfiles:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_keyfile' then begin txt_keyfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_keyfile_not_found' then begin txt_keyfile_not_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_keyfile_notcreated' then begin txt_keyfile_notcreated:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_larger' then begin txt_larger:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_lastused' then begin txt_lastused:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_launch' then begin txt_launch:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_layout' then begin txt_layout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_filelist_savedas' then begin txt_filelist_savedas:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level' then begin txt_level:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_license' then begin txt_license:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_caption_list' then begin txt_caption_list:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_details' then begin txt_list_details:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_all' then begin txt_list_all:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_disp' then begin txt_list_disp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_sel' then begin txt_list_sel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_toggle_warning' then begin txt_toggle_warning:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_loadfile' then begin txt_loadfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_loadlayout' then begin txt_loadlayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_hd' then begin txt_unit_hd:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_localization' then begin txt_localization:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_lpaqver' then begin txt_lpaqver:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_lpaq' then begin txt_type_description_lpaq:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_maininterface' then begin txt_maininterface:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_maxcomp' then begin txt_maxcomp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level_maximum' then begin txt_level_maximum:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_restartrequired' then begin txt_restartrequired:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_required_memory' then begin txt_required_memory:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_method' then begin txt_method:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_misc' then begin txt_misc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_modify' then begin txt_modify:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_morecontrols' then begin txt_morecontrols:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_morerecent' then begin txt_morerecent:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ent_hint' then begin txt_ent_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_moveto' then begin txt_moveto:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_mypc' then begin txt_mypc:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_na' then begin txt_list_na:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_name' then begin txt_name:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_naming' then begin txt_naming:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_remote' then begin txt_unit_remote:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_newarchive' then begin txt_newarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cnewfolder' then begin txt_cnewfolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_news' then begin txt_news:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_no' then begin txt_no:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_noinput' then begin txt_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_nocompress_hint' then begin txt_nocompress_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_split_noinput' then begin txt_split_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_open_noinput' then begin txt_open_noinput:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_nomatch' then begin txt_list_nomatch:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_singlethread' then begin txt_singlethread:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_none2' then begin txt_none2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_nonsolid' then begin txt_nonsolid:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level_normal' then begin txt_level_normal:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_copy_error' then begin txt_copy_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_description' then begin txt_description:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compare_second' then begin txt_compare_second:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_peaobj' then begin txt_peaobj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_displayed_obj' then begin txt_displayed_obj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_olderthan' then begin txt_olderthan:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_on' then begin txt_on:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ondblclick' then begin txt_ondblclick:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_opacity' then begin txt_opacity:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_open' then begin txt_open:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_openarchive' then begin txt_openarchive:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_title_open' then begin txt_title_open:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_open_bookmark' then begin txt_open_bookmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cphere' then begin txt_cphere:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_open_file' then begin txt_open_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_open_files' then begin txt_open_files:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_open_path' then begin txt_open_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_opensource' then begin txt_opensource:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_openwith' then begin txt_openwith:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_aborted' then begin txt_aborted:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_cd' then begin txt_unit_cd:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_options' then begin txt_options:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_other' then begin txt_other:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_otherparams' then begin txt_otherparams:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_oip' then begin txt_oip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_opi' then begin txt_opi:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_output' then begin txt_output:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_overwrite' then begin txt_overwrite:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_compressed_size' then begin txt_compressed_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_paqver' then begin txt_paqver:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_paq' then begin txt_type_description_paq:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pio' then begin txt_pio:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_poi' then begin txt_poi:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_parameters' then begin txt_parameters:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_partial' then begin txt_error_partial:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_passes' then begin txt_passes:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pw' then begin txt_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pwlength' then begin txt_pwlength:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_un7z_browse_pw' then begin txt_un7z_browse_pw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_un7z_browse_pw_other' then begin txt_un7z_browse_pw_other:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_paste' then begin txt_paste:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_path' then begin txt_path:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pea_appcolor' then begin txt_pea_appcolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pea_textcolor' then begin txt_pea_textcolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_pea' then begin txt_type_description_pea:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_peazip_new' then begin txt_peazip_new:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_peazip_help' then begin txt_peazip_help:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_peazip_web' then begin txt_peazip_web:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_performall' then begin txt_performall:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_name_provide' then begin txt_name_provide:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_upxorstrip' then begin txt_upxorstrip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_not_removable_file' then begin txt_not_removable_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_not_removable' then begin txt_not_removable:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_custom_executable_missing' then begin txt_custom_executable_missing:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_unsupported_select' then begin txt_type_unsupported_select:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_no_theme_name' then begin txt_no_theme_name:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_please_wait' then begin txt_please_wait:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_copy_wait' then begin txt_copy_wait:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_previewwith' then begin txt_previewwith:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_projectadmin' then begin txt_projectadmin:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_quad' then begin txt_type_description_quad:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_quickdelete' then begin txt_quickdelete:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_quit' then begin txt_quit:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_ram' then begin txt_unit_ram:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_read' then begin txt_read:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_recentarchives' then begin txt_recentarchives:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_rr_hint' then begin txt_rr_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_search_refine' then begin txt_search_refine:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_fefreshf5' then begin txt_fefreshf5:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_release' then begin txt_release:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_removable' then begin txt_unit_removable:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_remove_bookmark' then begin txt_remove_bookmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_remove_external_unit' then begin txt_remove_external_unit:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_removeselected' then begin txt_removeselected:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_rename' then begin txt_rename:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_caption_repair' then begin txt_caption_repair:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_restartrequired2' then begin txt_restartrequired2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_reset' then begin txt_reset:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_reset_archivename' then begin txt_reset_archivename:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_hardreset' then begin txt_hardreset:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_reset_theme' then begin txt_reset_theme:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_soft_reset' then begin txt_soft_reset:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_restore_att' then begin txt_restore_att:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_run_as' then begin txt_run_as:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_run_as2' then begin txt_run_as2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sample' then begin txt_sample:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_saveas' then begin txt_saveas:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_savehistory' then begin txt_savehistory:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_save_infolder' then begin txt_save_infolder:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_savejob' then begin txt_savejob:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_savejobdefinition' then begin txt_savejobdefinition:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_savejobdefinition_hint' then begin txt_savejobdefinition_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_savelayout' then begin txt_savelayout:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_save_winstate' then begin txt_save_winstate:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_search' then begin txt_search:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_searchanddrag' then begin txt_searchanddrag:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_searchfor' then begin txt_searchfor:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_nrsearch' then begin txt_nrsearch:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_search_hint' then begin txt_search_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_search_web' then begin txt_search_web:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_searching' then begin txt_list_searching:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_securedelete' then begin txt_securedelete:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_default_description' then begin txt_default_description:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_selectall' then begin txt_selectall:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_selectdir' then begin txt_selectdir:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_selected_obj' then begin txt_selected_obj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_selected_objects' then begin txt_selected_objects:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sfx' then begin txt_sfx:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sendbymail' then begin txt_sendbymail:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_set_defaults' then begin txt_set_defaults:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_settings' then begin txt_settings:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sfx_interface' then begin txt_sfx_interface:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_showhints' then begin txt_showhints:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_show_messages' then begin txt_show_messages:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_showpw' then begin txt_showpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_singlevol' then begin txt_singlevol:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_size' then begin txt_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sizeb' then begin txt_sizeb:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_skip_existing' then begin txt_skip_existing:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_slowercomp' then begin txt_slowercomp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_smaller' then begin txt_smaller:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_solid' then begin txt_solid:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_solid_block' then begin txt_solid_block:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_solid_auto' then begin txt_solid_auto:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_solid_extension' then begin txt_solid_extension:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_tarbeforepj' then begin txt_tarbeforepj:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_listtest' then begin txt_listtest:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sortbysel' then begin txt_sortbysel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_sorting' then begin txt_list_sorting:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_speed' then begin txt_speed:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_split' then begin txt_split:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_split' then begin txt_type_description_split:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_split_file' then begin txt_split_file:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_list_nostats' then begin txt_list_nostats:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_status' then begin txt_status:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level_store' then begin txt_level_store:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_stream_control' then begin txt_stream_control:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_strip' then begin txt_strip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_keyfile_created' then begin txt_keyfile_created:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_suggestpw' then begin txt_suggestpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_noupx' then begin txt_noupx:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_syntax' then begin txt_syntax:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_sysbenchmark' then begin txt_sysbenchmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_benchmark' then begin txt_benchmark:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_systools' then begin txt_systools:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_tarbefore' then begin txt_tarbefore:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_tar' then begin txt_type_description_tar:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_taskman' then begin txt_taskman:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_caption_test' then begin txt_caption_test:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_testall' then begin txt_testall:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_testdisp' then begin txt_testdisp:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_testpw' then begin txt_testpw:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_testsel' then begin txt_testsel:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_col_hint' then begin txt_col_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_bookmarks_hint' then begin txt_bookmarks_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_archive_noinput_tolist' then begin txt_archive_noinput_tolist:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theme' then begin txt_theme:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_icons_found' then begin txt_icons_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_themename' then begin txt_themename:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_icons_not_found' then begin txt_icons_not_found:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theme_create_success' then begin txt_theme_create_success:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theme_not_loadable' then begin txt_theme_not_loadable:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_theming' then begin txt_theming:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_extand_error' then begin txt_extand_error:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_threads' then begin txt_threads:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_titlescolor' then begin txt_titlescolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_to' then begin txt_to:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_toggle_browseflat' then begin txt_toggle_browseflat:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_toggle_historybookmarks' then begin txt_toggle_historybookmarks:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_toolbarscolor' then begin txt_toolbarscolor:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_tools' then begin txt_tools:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_best' then begin txt_best:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type' then begin txt_type:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_level_ultra' then begin txt_level_ultra:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_error_openfile' then begin txt_error_openfile:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_cl_hint' then begin txt_cl_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ace_missing' then begin txt_ace_missing:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_units' then begin txt_units:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_unit_unknown' then begin txt_unit_unknown:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_un7z_pw_untested' then begin txt_un7z_pw_untested:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_up' then begin txt_up:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_update' then begin txt_update:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_upx' then begin txt_type_description_upx:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_advfilters' then begin txt_advfilters:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_openfiles_hint' then begin txt_openfiles_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_usenet' then begin txt_usenet:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_user_name' then begin txt_user_name:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_using' then begin txt_using:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_volumepea' then begin txt_volumepea:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_volume_size' then begin txt_volume_size:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_ext_uns' then begin txt_type_ext_uns:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_websearch' then begin txt_websearch:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_websites' then begin txt_websites:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_word' then begin txt_word:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_write' then begin txt_write:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_ramdompw_hint' then begin txt_ramdompw_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_exe_hint' then begin txt_exe_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_pj_hint2' then begin txt_pj_hint2:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_type_description_zip' then begin txt_type_description_zip:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
if copy(s,0,pos(':',s)-1)='txt_zipcrypto_hint' then begin txt_zipcrypto_hint:=copy(s,pos(':',s)+2,length(s)-pos(':',s)); exit; end;
end;

function fallback_valorize_text:integer;
var
   s:utf8string;
begin
fallback_valorize_text:=-1;
repeat
readln(t,s);
get_fallback_text(s);
until eof(t) or (s='=== end PeaZip text group ===');
if eof(t)=false then fallback_valorize_text:=0;
end;

procedure assign_guitext;
begin
Form_peach.Caption:=APPMAIN;
with Form_peach do
begin
try
MenuItemPW.Caption:=txt_2_7_setpw;
CheckBox1.Caption:=txt_2_7_es;
CheckBox2.Caption:=txt_2_7_eu;
ImageListHistory.Hint:=txt_list_history;
ImageBookmarks.Hint:=txt_bookmarks;
ImageClip.Hint:=txt_2_7_clipboard;
mb24.Caption:=txt_bookmarks+'...';
pomb24.Caption:=txt_bookmarks+'...';
pmjb24.Caption:=txt_bookmarks+'...';
smbookmarks1.Caption:=txt_bookmarks+' '+txt_2_6_folders;
smbookmarksa1.Caption:=txt_bookmarks+' '+txt_2_6_folders;
smfs1.Caption:=txt_fs;
smfsa1.Caption:=txt_fs;
MenuItemOpen_Last.Caption:=txt_lastused;
mpathreset1.Caption:=txt_lastused;
MenuItemArchive_home1.Caption:=txt_2_3_home;
mpathdesk1.Caption:=txt_desktop;
MenuItemArchive_documents1.Caption:=txt_2_4_documents;
smrecent1.Caption:=txt_recentarchives+' '+txt_2_6_folders;
smrecenta1.Caption:=txt_recentarchives+' '+txt_2_6_folders;
mpathexplore1.Caption:=txt_explore_path;
menuitemextexplore.Caption:=txt_explore_path;
MenuItemOpen_here.Caption:=txt_2_7_selpath;
mpathself1.Caption:=txt_2_7_selpath;
pmjumpsbook.Caption:=txt_bookmarks;
pmjumpfs.Caption:=txt_fs;
pmrecent.Caption:=txt_recentarchives;
mrecent9.Caption:=txt_list_history+'...';
porecent9.Caption:=txt_list_history+'...';
pmr9.Caption:=txt_list_history+'...';
mtray.Caption:=txt_2_5_tray;
tmquit.Caption:=txt_quit;
tmcreate.Caption:=txt_create_archive;
tmopenpath.Caption:=txt_open_path;
tmopenarchive.Caption:=txt_openarchive;
MenuItemVideo.Caption:=txt_2_4_gvideo;
MenuItemWikipedia.Caption:=txt_2_4_wenc;
MenuItemCommons.Caption:=txt_2_4_wcommons;
MenuItemBooks.Caption:=txt_2_4_wbook;
MenuItemSrc.Caption:=txt_2_4_wsrc;
MenuItemwnews.Caption:=txt_2_4_wnews;
MenuItemdictionary.Caption:=txt_2_4_wdict;
MenuItemyweb.Caption:=txt_websites;
MenuItemyanswers.Caption:=txt_2_4_yanswers;
mhexp.Caption:=txt_2_4_hexp;
po_hexp.Caption:=txt_2_4_hexp;
MenuItemToolhexp.Caption:=txt_2_4_hexp;
MenuItemOpen_documents.Caption:=txt_2_4_documents;
MenuItemOpen_documents1.Caption:=txt_2_4_documents;
pmJumpDocuments.Caption:=txt_2_4_documents;
jddocuments.Caption:=txt_2_4_documents;
pclip_removeitem.Caption:=txt_2_4_removefromclipboard;
pclip_clearclip.Caption:=txt_2_4_clearclipboard;
CheckBoxTBW.Caption:=txt_2_4_tb;
CheckBoxIH.Caption:=txt_2_4_itemsheight;
LabelDefaultColor7.Caption:=txt_reset;
mEnvstr.Caption:=txt_2_3_envstr;
jdroot.Caption:=txt_mypc;
jdhome.Caption:=txt_2_3_home;
jddesktop.Caption:=txt_desktop;
po_Envstr.Caption:=txt_2_3_envstr;
cbType.Items[0]:=STR_7Z;
cbType.Items[1]:=STR_ARC;
cbType.Items[2]:=STR_BZIP2;
cbType.Items[3]:=txt_custom;
cbType.Items[4]:=STR_GZIP;
cbType.Items[5]:=STR_PEA;
cbType.Items[6]:=STR_QUAD;
cbType.Items[7]:=txt_sfx+' '+STR_7Z;
cbType.Items[8]:=txt_sfx+' '+STR_ARC;
cbType.Items[9]:=txt_split;
cbType.Items[10]:=STR_TAR;
cbType.Items[11]:=STR_UPX;
cbType.Items[12]:=STR_ZIP;
cbType.Items[13]:=STR_ZPAQ;
RadioGroup0.Items[0]:=STR_7Z;
RadioGroup0.Items[1]:=STR_ARC;
RadioGroup0.Items[2]:=STR_BZIP2;
RadioGroup0.Items[3]:=txt_custom;
RadioGroup0.Items[4]:=STR_GZIP;
RadioGroup0.Items[5]:=STR_PEA;
RadioGroup0.Items[6]:=STR_QUAD;
RadioGroup0.Items[7]:=txt_sfx+' '+STR_7Z;
RadioGroup0.Items[8]:=txt_sfx+' '+STR_ARC;
RadioGroup0.Items[9]:=txt_split;
RadioGroup0.Items[10]:=STR_TAR;
RadioGroup0.Items[11]:=STR_UPX;
RadioGroup0.Items[12]:=STR_ZIP;
RadioGroup0.Items[13]:=STR_ZPAQ;
RadioGroup0.Items[14]:=txt_lastused;
status0:=APPMAIN+' '+PEAZIPVERSION+PEAZIPREVISION;
Notebook2.PageIndex:=1; OptionsAdv.Caption:=txt_2_6_advanced+'   ';
ComboBoxArchive1.Items.Strings[0]:=txt_singlevol;
ComboBoxArchive1.Items.Strings[1]:=txt_customsize;
ComboBoxArchive1.Items.Strings[2]:=txt_fd;
ComboBoxArchive1.Items.Strings[3]:=txt_attach5;
ComboBoxArchive1.Items.Strings[4]:=txt_attach10;
ComboBoxArchive1.Items.Strings[5]:=txt_cd650;
ComboBoxArchive1.Items.Strings[6]:=txt_cd700;
ComboBoxArchive1.Items.Strings[7]:=txt_fat32;
ComboBoxArchive1.Items.Strings[8]:=txt_dvd;
ComboBoxArchive1.Items.Strings[9]:=txt_dvddl;
CheckBoxTarBefore.Caption:=txt_tarbefore;
CheckBoxTarBefore.Hint:=txt_tarbefore_hint;
GroupBoxCreateOut.Caption:=txt_format_options;
GroupBoxCreateOut1.Caption:=txt_2_7_output;
StringGrid1.Hint:=txt_2_7_archivehint;
StringGrid2.Hint:=txt_2_7_exthint;
CheckBoxSeparate.Caption:=txt_2_7_separate;
EditOParc.Hint:=txt_otherparams;
EditOP.Hint:=txt_otherparams;
EditOPcustom.Hint:=txt_otherparams;
ImageInfoArchive2.Hint:=txt_archive_hint;
CompressionPaq.Caption:=txt_compression;
Label7za9.Caption:=txt_level;
OptionsPAQ.Caption:=txt_options;
RadioGroupPaq.Caption:=txt_paqver;
EncryptionPAQ.Caption:=txt_encryption;
CompressionUPX.Caption:=txt_compression;
Label7za7.Caption:=txt_level;
OptionsUPX.Caption:=txt_options;
CheckBoxUPX1.Caption:=txt_strip;
CheckBoxUPX2.Caption:=txt_backupexe;
CheckBoxUPX2.Hint:=txt_backupexe_hint;
EncryptionUPX.Caption:=txt_encryption;
CompressionQUAD.Caption:=txt_compression;
RadioGroupQuad.Caption:=txt_exe;
CheckBoxQuadMax.Caption:=txt_maxcomp;
CheckBoxQuadMax.Hint:=txt_slowercomp;
OptionsQUAD.Caption:=txt_options;
EncryptionQUAD.Caption:=txt_encryption;
CompressionPea.Caption:=txt_compression;
Label7za10.Caption:=txt_level;
GroupBoxPEAAdvanced1.Caption:=txt_options;
LabelPEA6.Caption:=txt_stream_control;
LabelPEA7.Caption:=txt_peaobj;
LabelPEA8.Caption:=txt_volumepea;
LabelSplit.Caption:=txt_integrity;
LabelSplit.Hint:=txt_check_hint;
EncryptionPEA.Caption:=txt_encryption;
CheckBoxPEAEncryption.Caption:=txt_stream;
Group7za1.Caption:=txt_options;
Label7za1.Caption:=txt_function;
Label7za2.Caption:=txt_threads;
LabelArchiveOther.Caption:=txt_other+':';
CheckBoxArchive4.Caption:=txt_sendbymail;
CheckBoxArchive4.Hint:=txt_attach;
GroupBox7za3.Caption:=txt_encryption;
Label7za18.Caption:=txt_algo;
Group7za2.Caption:=txt_compression;
Label7za4.Caption:=txt_method;
Label7za3.Caption:=txt_level;
Label7za5.Caption:=txt_dictionary;
Label7za6.Caption:=txt_word;
Label7zaPasses.Caption:=txt_passes;
Label7zsolidblock.Caption:=txt_solid_block;
CheckBoxArchive6.Caption:=txt_create_sfx;
RadioGroupArchive.Caption:=txt_sfx_interface;
CheckBoxArchiveOF.Caption:=txt_compress_openforwriting;
CheckBoxArchiveOF.Hint:=txt_openfiles_hint;
CompressionCustom.Caption:=txt_custom_parameters;
LabelArchiveOther2.Caption:=txt_parameters;
LabelCustom.Caption:=txt_exe_hint;
LabelArchiveOther3.Caption:=txt_syntax;
LabelArchiveOther8.Caption:=txt_exe;
LabelArchiveOther4.Caption:=txt_ext;
CompressionARC.Caption:=txt_compression;
Label7za11.Caption:=txt_level;
LabelARCsolidblock.Caption:=txt_solid;
CheckBoxARC1.Caption:=txt_rr;
CheckBoxARC1.Hint:=txt_rr_hint;
CheckBoxARC2.Caption:=txt_create_sfx;
OptionsARC.Caption:=txt_options;
Label7za12.Caption:=txt_function;
LabelArchiveOther1.Caption:=txt_other;
GroupBox7za4.Caption:=txt_encryption;
Label7za14.Caption:=txt_algo;
CheckBoxDate.Caption:=txt_timestamp;
Notebook2.PageIndex:=2; ConsoleCreate.Caption:=txt_console+'   ';
iConsoleCreateImport.Hint:=txt_clear;
Notebook2.PageIndex:=0; Options.Caption:=txt_archive+'   ';
gbConsoleCreate.Caption:=txt_console;
lConsoleCreate1.Caption:=txt_pj_hint2;
lConsoleCreate2.Caption:=txt_pj_hint;
lConsoleCreate_jobcode.Caption:=txt_job_code;
lConsoleCreate_outname.Caption:='outname'; //overwritten, don't need to be translated
Image7zPj.Caption:=txt_savejob;
Image7zPj.Hint:=txt_savejobdefinition_hint;
Image7zPj1.Caption:=txt_launch;
ButtonArchive.Caption:=txt_2_7_ok;
ButtonExtOk.Caption:=txt_2_7_ok;
ButtonArchive1.Caption:=txt_2_7_cancel;
ButtonExtCancel.Caption:=txt_2_7_cancel;
mkeyfile.Caption:=txt_create_keyfile;
ButtonUn7zaTo.Caption:=txt_caption_extract;
ButtonUn7za1.Caption:=txt_caption_test;
ButtonUn7za4.Caption:=txt_add;
cbUn7zaRecursive.Caption:=txt_filters_recourse;
cbUn7zaRecursiveExclude.Caption:=txt_exclude_recourse;
MenuItemHistoryBook.Caption:=txt_add_tobookmarks;
ButtonUn7za4.Hint:=txt_add_tolayout;
EditUn7zaFilter1.Hint:=txt_address_hint;
Imagelistsearch.Hint:=txt_search;
NotebookOpen.PageIndex:=0; OptionsOpen.Caption:=txt_caption_extract+'   ';
RadioGroupAction.Caption:=txt_action;
GroupBox12.Caption:=txt_2_6_special;
CheckBoxUnPeaAttributes.Caption:=txt_restore_att;
CompressionCustom1.Caption:=txt_custom_parameters;
LabelArchiveOther5.Caption:=txt_parameters;
LabelCustom1.Caption:=txt_exe_hint;
LabelArchiveOther6.Caption:=txt_syntax;
LabelArchiveOther7.Caption:=txt_exe;
pcustompathreset.Caption:=txt_reset;
pcustompathexplore.Caption:=txt_explore_path;
mpathreset.Caption:=txt_reset;
groupbox4.Caption:=txt_2_7_output;
groupbox5.Caption:=txt_format_options;
label4.Caption:=txt_2_7_spc;
mpathexplore.Caption:=txt_explore_path;
CheckBoxFolder.Caption:=txt_newfoldermenu;
NotebookOpen.PageIndex:=1; PageExtAdv.Caption:=txt_2_6_advanced+'   ';
RadioGroupClip.Caption:=txt_2_7_clipboard;
RadioGroupClip.Items[0]:=txt_2_4_stdclip;
RadioGroupClip.Items[1]:=txt_2_4_advclip;
NotebookOpen.PageIndex:=2; ConsoleOpen.Caption:=txt_console+'   ';
ImageUn7zPj.Caption:=txt_savejob;
ImageUn7zPj.Hint:=txt_savejobdefinition_hint;
iConsoleCreateImport1.Hint:=txt_clear;
NotebookOpen.PageIndex:=0;
gbConsoleCreate1.Caption:=txt_console;
lConsoleCreate3.Caption:=txt_pj_hint2;
lConsoleCreate4.Caption:=txt_pj_hint;
lConsoleCreate_jobcode1.Caption:=txt_job_code;
lConsoleCreate_outname1.Caption:='outname'; //overwritten, don't need translation
Image7zPj3.Caption:=txt_launch;
LabelDefaults1.Caption:=txt_settings;
LabelDefaults2.Caption:=txt_set_defaults;
ImageInfoDefault1.Hint:=txt_cl_hint;
ImageInfoDefault2.Hint:=txt_2_5_langhint;
GroupBox2.Caption:=txt_localization;
Labelcb4.Caption:=txt_backend;
Labelcb17.Caption:=txt_desktop;
CheckBoxAutoOpen.Caption:=txt_2_7_oop;
GroupBox1.Caption:=txt_other;
Notebook1.PageIndex:=4; Page1.Caption:=txt_openwith+'   ';
Labelcb6.Caption:=txt_2_5_custeditors;
Labelcb7.Caption:=txt_reset;
Labelcb8.Caption:=txt_2_5_advanced;
Labelcb9.Caption:=txt_2_5_basic;
pmCusteditDesc.Caption:=txt_note;
pmCusteditAdd.Caption:=txt_2_5_selectapp;
pmCusteditEdit.Caption:=txt_edit;
pmCusteditRemove.Caption:=txt_2_5_remove;
pmCusteditDesc1.Caption:=txt_note;
pmCusteditAdd1.Caption:=txt_2_5_selectapp;
pmCusteditEdit1.Caption:=txt_2_5_strbefore;
pmCusteditEdit1after.Caption:=txt_2_5_strafter;
pmCusteditRemove1.Caption:=txt_2_5_remove;
Notebook1.PageIndex:=1; Page6.Caption:=txt_general+' (2)   ';
GroupBox9.Caption:=txt_2_5_encoding;
CheckBoxJobEncoding.Caption:=txt_2_5_encpj;
CheckBoxEncoding.Caption:=txt_2_5_langflag;
CheckBoxWindowStatus.Caption:=txt_save_winstate;
CheckBoxArchiveHistory.Caption:=txt_savehistory;
CheckBoxShowHints.Caption:=txt_showhints;
Notebook1.PageIndex:=2; Page3.Caption:=txt_create_archive+'   ';
Labelcb14.Caption:=txt_default_format;
GroupBox6.Caption:=txt_favformats;
cbf8.Caption:=txt_split;
cbfCustom.Caption:=txt_custom;
cbfSfx.Caption:=txt_sfx+' '+STR_7Z;
cbfSfx1.Caption:=txt_sfx+' '+STR_ARC;
Notebook1.PageIndex:=3; Page4.Caption:=txt_openarchive+'   ';
Labelcb10.Caption:=txt_interface;
RadioGroupBrowser.Items.Strings[0]:=txt_browser;
RadioGroupBrowser.Items.Strings[1]:=txt_flat;
RadioGroupBrowser.Items.Strings[2]:=txt_lastused;
CheckBox4.Caption:=txt_ignore_ext;
CheckBoxIgnoreSel.Caption:=txt_ignore_sel;
CheckBoxIgnoreDisp.Caption:=txt_ignore_disp;
CheckBoxPrebrowse.Caption:=txt_fastopen;
ImageInfoDefault3.Hint:=txt_quickbrowse_hint;
GroupBox13.Caption:=txt_2_6_advanced;
labelcb13.Caption:=txt_2_3_on_pw;
RadioGrouppwrequest.Items[0]:=txt_2_3_never_pw;
RadioGrouppwrequest.Items[1]:=txt_2_3_test_pw100;
RadioGrouppwrequest.Items[2]:=txt_2_3_test_pw;
RadioGrouppwrequest.Items[3]:=txt_always_pw;
Notebook1.PageIndex:=5;  Page2.Caption:=txt_filetools+'   ';
GroupBox7.Caption:=txt_check;
RadioGroup1.Caption:=txt_dispaly;
GroupBoxCheck.Caption:=txt_check_select;
cbcheckall.Caption:=txt_performall;
GroupBox8.Caption:=txt_erase_hint;
Label1.Caption:=txt_passes;
Notebook1.PageIndex:=6; Theming.Caption:=txt_theming+'   ';
theme_label1.Hint:=txt_reset_theme;
LabelOpacity.Hint:=txt_restartrequired;
LabelDefaultOpacity.Hint:=txt_restartrequired2;
Notebook1.PageIndex:=0; Page5.Caption:=txt_general+' (1)   ';
RadioGroup4.Items.Strings[0]:=txt_console_interface;
RadioGroup4.Items.Strings[1]:=txt_guicl;
RadioGroup4.Items.Strings[2]:=txt_guipealauncher;
RadioGroup6.Items.Strings[0]:=txt_2_6_plalways;
RadioGroup6.Items.Strings[1]:=txt_2_6_plsmart;
RadioGroup6.Items.Strings[2]:=txt_autoclose;
ImageDefaults2.Hint:=txt_soft_reset;
ImageDefaults3.Hint:=txt_hard_reset_hint;
LabelColor3.Caption:=txt_pea_appcolor;
LabelColor4.Caption:=txt_titlescolor;
LabelDefaultColor3.Caption:=txt_reset;
LabelDefaultColor4.Caption:=txt_reset;
themesave_label.Caption:=txt_create+'!';
LabelTheme1.Caption:=txt_themename;
LabelTheme2.Caption:=txt_author;
LabelTheme3.Caption:=txt_license;
LabelTheme4.Caption:=txt_save_infolder;
themesave_label_info.Caption:=txt_create_theme;
LabelDefaultColor5.Caption:=txt_reset;
LabelColor5.Caption:=txt_pea_textcolor;
Labelthemegraphic.Caption:=txt_graphic;
LabelDefaultGraphicsFolder.Caption:=txt_reset;
LabelColor2.Caption:=txt_toolbarscolor;
LabelColor1.Caption:=txt_gridaltcolor;
LabelDefaultColor2.Caption:=txt_reset;
LabelDefaultColor1.Caption:=txt_reset;
LabelColor6.Caption:=txt_gridrowheight;
LabelDefaultColor6.Caption:=txt_reset;
theme_label.Caption:=txt_default;
LabelTheme.Caption:=txt_theme;
LabelOpacity.Caption:=txt_opacity;
LabelDefaultOpacity.Caption:=txt_reset;
Button1.Caption:=txt_done;
ImageDefaults2.Caption:=txt_reset;
ImageDefaults3.Caption:=txt_hardreset;
pmBrowser.Caption:=txt_go_browser;
mgobrowser.Caption:=txt_go_browser;
pmAddFile.Caption:=txt_add_files;
pmAddFolder.Caption:=txt_add_folder;
pmSearchAdd.Caption:=txt_searchanddrag;
pmLoadList.Caption:=txt_loadlayout;
pmSaveList.Caption:=txt_savelayout;
MenuItem3.Caption:=txt_clearlayout;
MenuItem1.Caption:=txt_removeselected;
sm_delete.Caption:=txt_2_5_delete;
MenuItemNavigate.Caption:=txt_go_path;
MenuItem5b.Caption:=txt_openwith;
openwithpeazip.Caption:=txt_peazip_new;
openwithassociated.Caption:=txt_associated;
openwithcustom.Caption:=txt_customapp;
mMisc.Caption:=txt_misc;
MenuItemBookmarks.Caption:=txt_add_tobookmarks;
MenuItem6.Caption:=txt_explore_path;
mcp.Caption:=txt_cphere;
MenuItemTools.Caption:=txt_filetools;
MenuItemToolSplit.Caption:=txt_split_file;
MenuItemToolJoin.Caption:=txt_joinfiles;
MenuItemToolCompare.Caption:=txt_compare;
MenuItemToolCheck.Caption:=txt_check;
MenuItemRefresh.Caption:=txt_fefreshf5;
mFeatures.Caption:=txt_file;
mPEA.Caption:=txt_create_archive;
mUnPEAbrowse.Caption:=txt_open_path;
mUnPEA.Caption:=txt_openarchive;
mBookmarks.Caption:=txt_bookmarks;
mPaths.Caption:=txt_fs;
mRecent.Caption:=txt_recentarchives+'   ';
mTools.Caption:=txt_filetools;
mSplitFile.Caption:=txt_split_file;
mJoinFiles.Caption:=txt_joinfiles;
mCompare.Caption:=txt_compare;
mCheck.Caption:=txt_check;
mQuit.Caption:=txt_quit;
mEdit.Caption:=txt_edit;
mSelectAll.Caption:=txt_selectall;
mInvertSelection.Caption:=txt_invertsel;
mType.Caption:=txt_all_type;
mAttributes.Caption:=txt_all_attributes;
mDate.Caption:=txt_all_date;
mDateRecent.Caption:=txt_morerecent;
mDateERecent.Caption:=txt_eqorrecent;
mDateEqual.Caption:=txt_equal;
mDateEOlder.Caption:=txt_eqorolder;
mDateOlder.Caption:=txt_olderthan;
mPackedSize.Caption:=txt_all_psize;
mPSSmaller.Caption:=txt_smaller;
mPSESmaller.Caption:=txt_eqorsmaller;
mPSEqual.Caption:=txt_equal;
mPSELarger.Caption:=txt_eqorlarger;
mPSLarger.Caption:=txt_larger;
mSize.Caption:=txt_all_size;
mSizeSmaller.Caption:=txt_smaller;
mSizeESmaller.Caption:=txt_eqorsmaller;
mSizeEqual.Caption:=txt_equal;
mSizeELarger.Caption:=txt_eqorlarger;
mSizeLarger.Caption:=txt_larger;
mSortBySelection.Caption:=txt_sortbysel;
mArchive.Caption:=txt_browser;
mgotolayout.Caption:=txt_2_7_goarclayout;
mgotoextlayout.Caption:=txt_2_7_goextlayout;
mFlat.Caption:=txt_toggle_browseflat;
mRefresh.Caption:=txt_fefreshf5;
mOptionsMenu.Caption:=txt_tools;
mUser.Caption:=txt_run_as2;
mSystem.Caption:=txt_systools;
mDiskClean.Caption:=txt_disk_cleanup;
mDefrag.Caption:=txt_disk_defrag;
mDiskmanagement.Caption:=txt_disk_management;
mRemoveUnits.Caption:=txt_remove_external_unit;
mCPL.Caption:=txt_controlpanel;
mComp.Caption:=txt_compmanagement;
mTaskmanager.Caption:=txt_taskman;
mBench.Caption:=txt_sysbenchmark;
mLang.Caption:=txt_localization;
mDefaults.Caption:=txt_settings;
mHelp.Caption:=txt_2_5_help;
mAbout.Caption:=txt_about+'...';
mPeaSite.Caption:=txt_peazip_web;
mPEAhelp.Caption:=txt_peazip_help;
mpeahelpoffline.Caption:=txt_2_5_offline_help;
mtutorial.Caption:=txt_2_5_mini_help;
po_addtoarchive.Caption:=txt_add;
MenuItemOpen_ExtractHere.Caption:=txt_caption_extract;
MenuItemOpen_ext_all.Caption:=txt_ext_all;
MenuItemOpen_ext_displayed.Caption:=txt_ext_disp;
MenuItemOpen_ext_selected.Caption:=txt_ext_sel;
po_exthere.Caption:=txt_caption_extract;
MenuItemOpen_ExtractTo.Caption:=txt_caption_extract+' '+txt_newfolder;
MenuItemOpen_ExtractAllTo.Caption:=txt_ext_all+' '+txt_newfolder;
MenuItemOpen_ExtractDisplayedTo.Caption:=txt_ext_disp+' '+txt_newfolder;
MenuItemOpen_ExtractSelectedTo.Caption:=txt_ext_sel+' '+txt_newfolder;
po_extto.Caption:=txt_caption_extract+' '+txt_newfolder;
MenuItemOpen_test.Caption:=txt_caption_test;
MenuItemOpen_test_all.Caption:=txt_testall;
MenuItemOpen_test_displayed.Caption:=txt_testdisp;
MenuItemOpen_test_selected.Caption:=txt_testsel;
po_test.Caption:=txt_caption_test;
MenuItemOpen_info.Caption:=txt_info;
MenuItemOpen_info_all.Caption:=txt_infoall;
MenuItemOpen_info_displayed.Caption:=txt_infodisp;
MenuItemOpen_info_selected.Caption:=txt_infosel;
po_info.Caption:=txt_info;
pmmorefun_info.Caption:=txt_info;
pmcpyto.Caption:=txt_copyto;
pmmvto.Caption:=txt_moveto;
pmexplore.Caption:=txt_explore_path;
pmcommand.Caption:=txt_cphere;
pmquickdelete.Caption:=txt_quickdelete;
pmsecuredelete.Caption:=txt_securedelete;
MenuItemOpen_list.Caption:=txt_caption_list;
MenuItemOpen_list_all.Caption:=txt_list_all;
MenuItemOpen_list_displayed.Caption:=txt_list_disp;
MenuItemOpen_list_selected.Caption:=txt_list_sel;
po_list.Caption:=txt_caption_list;
pmmorefun_list.Caption:=txt_caption_list;
MenuItemOpen_preview.Caption:=txt_previewwith;
MenuItemPrePeaZip.Caption:=txt_peazip_new;
MenuItemPreAssociated.Caption:=txt_associated;
MenuItemPreCustom.Caption:=txt_customapp;
MenuItemOpen_extandopenwith.Caption:=txt_extopen_with;
MenuItemERPeaZip.Caption:=txt_peazip_new;
MenuItemERAssociated.Caption:=txt_associated;
MenuItemERCustom.Caption:=txt_customapp;
po_open.Caption:=txt_openwith;
po_open_peazip.Caption:=txt_peazip_new;
po_open_associated.Caption:=txt_associated;
po_open_custom.Caption:=txt_customapp;
po_misc.Caption:=txt_misc;
MenuItemOpen_addtobookmarks.Caption:=txt_add_tobookmarks;
pmjbadd.Caption:=txt_add_tobookmarks;
MenuItemOpen_explore.Caption:=txt_explore_path;
MenuItemOpen_commandprompt.Caption:=txt_cphere;
po_filetools.Caption:=txt_filetools;
po_split.Caption:=txt_split_file;
po_join.Caption:=txt_joinfiles;
po_compare.Caption:=txt_compare;
po_check.Caption:=txt_check;
po_systemtools.Caption:=txt_systools;
po_diskclean.Caption:=txt_disk_cleanup;
po_defrag.Caption:=txt_disk_defrag;
po_manage.Caption:=txt_disk_management;
po_removeunits.Caption:=txt_remove_external_unit;
po_cpl.Caption:=txt_controlpanel;
po_computer.Caption:=txt_compmanagement;
po_taskman.Caption:=txt_taskman;
po_delete.Caption:=txt_quickdelete;
po_securedelete.Caption:=txt_securedelete;
po_modify.Caption:=txt_modify;
po_newfolder.Caption:=txt_create_folder;
po_rename.Caption:=txt_rename;
po_copyto.Caption:=txt_copyto;
po_moveto.Caption:=txt_moveto;
po_copy.Caption:=txt_copy;
po_cut.Caption:=txt_cut;
po_paste.Caption:=txt_paste;
smBookmarks.Caption:=txt_bookmarks;
smRecent.Caption:=txt_recentarchives;
smSearch.Caption:=txt_searchfor;
po_setpw.Caption:=txt_2_7_setpw;
msetpw.Caption:=txt_2_7_setpw;
madvfilters.caption:=txt_2_7_setadvf;
mOptions.Caption:=txt_options;
po_searchhere.Caption:=txt_nrsearch;
po_searchrec.Caption:=txt_search;
sbBrowse.Caption:=txt_browse;
MenuItemOpen_up.Caption:=txt_up;
MenuItemOpen_back.Caption:=txt_back;
MenuItemOpen_fwd.Caption:=txt_fwd;
MenuItemOpen_root.Caption:=txt_homeroot;
MenuItemOpen_jump.Caption:=txt_go_path;
MenuItemOpen_Flat.Caption:=txt_toggle_browseflat;
MenuItemOpen_refresh.Caption:=txt_fefreshf5;
smOpen.Caption:=txt_fs;
MenuItemOpen_my.Caption:=txt_mypc;
MenuItemOpen_home.Caption:=txt_2_3_home;
MenuItemOpen_home1.Caption:=txt_2_3_home;
MenuItemOpen_desktop.Caption:=txt_desktop;
MenuItemOpen_desktop1.Caption:=txt_desktop;
MenuItemOpen_path.Caption:=txt_open_path;
MenuItemOpen_archive.Caption:=txt_openarchive;
MenuItemOpenWeb.Caption:=txt_search_web;
MenuItemWeb.Caption:=txt_websites;
MenuItemBlog.Caption:=txt_blogs;
MenuItemImg.Caption:=txt_images;
MenuItemNews.Caption:=txt_news;
MenuItemUsenet.Caption:=txt_usenet;
pmOpenPJall.Caption:=txt_all_default;
pmOpenPJdisplayed.Caption:=txt_displayedobjects;
pmOpenPJselected.Caption:=txt_selected_objects;
mTypeCustom.Caption:=txt_custom;
mTypeSfx.Caption:=txt_sfx+' '+STR_7Z;
mTypeSfxArc.Caption:=txt_sfx+' '+STR_ARC;
mTypeSplit.Caption:=txt_split;
pmJump1.Caption:=txt_mypc;
pmJump3.Caption:=txt_2_3_home;
pmJump2.Caption:=txt_desktop;
pmJumpPath.Caption:=txt_open_path;
pmJumpArchive.Caption:=txt_openarchive;
mbNotes.Caption:=txt_note;
mbDelete.Caption:=txt_remove_bookmark;
mbAddPath.Caption:=txt_add_path;
mbAddArchive.Caption:=txt_add_archive;
ComboBoxPAQ1.Items.Strings[0]:=txt_level_fast;
ComboBoxPAQ1.Items.Strings[1]:=txt_default;
ComboBoxPAQ1.Items.Strings[2]:=txt_level_maximum;
RadioGroupPaq.Items.Strings[2]:='ZPAQ '+txt_2_7_experimental;
ComboBoxUPX.Items.Strings[0]:=txt_nocompress;
ComboBoxUPX.Items.Strings[1]:='1 '+txt_faster;
ComboBoxUPX.Items.Strings[9]:='9 '+txt_better;
ComboBoxUPX.Items.Strings[10]:=txt_best;
ComboBoxPEACompression1.Items.Strings[0]:=txt_bettercompression;
ComboBoxPEACompression1.Items.Strings[1]:=txt_default_compression;
ComboBoxPEACompression1.Items.Strings[2]:=txt_fastcompr;
ComboBoxPEACompression1.Items.Strings[3]:=txt_nocompress_hint;
ComboBoxPEA2.Items.Strings[12]:=txt_none2;
ComboBoxPEA3.Items.Strings[9]:=txt_none2;
ComboBoxPEA4.Items.Strings[9]:=txt_none2;
ComboBoxSplit3.Items.Strings[9]:=txt_none2;
ComboBoxArchive7.Items.Strings[0]:=txt_newarchive;
ComboBoxArchive7.Items.Strings[1]:=txt_add_existing_archive;
ComboBoxArchive7.Items.Strings[2]:=txt_update;
ComboBoxArchive8.Items.Strings[0]:=txt_singlethread;
ComboBoxArchive8.Items.Strings[1]:=txt_multithreading;
ComboBox7zalgo.Items.Strings[1]:=txt_zipcrypto_hint;
ComboBoxArchive4.Items.Strings[0]:=txt_level_store;
ComboBoxArchive4.Items.Strings[1]:=txt_level_fastest;
ComboBoxArchive4.Items.Strings[2]:=txt_level_fast;
ComboBoxArchive4.Items.Strings[3]:=txt_level_normal;
ComboBoxArchive4.Items.Strings[4]:=txt_level_maximum;
ComboBoxArchive4.Items.Strings[5]:=txt_level_ultra;
ComboBoxArchiveSolid.Items.Strings[0]:=txt_nonsolid;
ComboBoxArchiveSolid.Items.Strings[18]:=txt_solid;
ComboBoxArchiveSolid.Items.Strings[19]:=txt_solid_extension;
RadioGroupArchive.Items.Strings[0]:=txt_console;
RadioGroupArchive.Items.Strings[1]:=txt_gui;
ComboBoxArchiveCustom.Items.Strings[0]:=txt_pio;
ComboBoxArchiveCustom.Items.Strings[1]:=txt_poi;
ComboBoxArchiveCustom.Items.Strings[2]:=txt_ipo;
ComboBoxArchiveCustom.Items.Strings[3]:=txt_iop;
ComboBoxArchiveCustom.Items.Strings[4]:=txt_opi;
ComboBoxArchiveCustom.Items.Strings[5]:=txt_oip;
ComboBoxARC.Items.Strings[0]:='0 '+txt_store;
ComboBoxARC.Items.Strings[1]:='1 '+txt_fastermem;
ComboBoxARC.Items.Strings[4]:='4 '+txt_default2;
ComboBoxARC.Items.Strings[9]:='9 '+txt_slowermem;
ComboBoxArcSolid.Items.Strings[0]:=txt_nonsolid;
ComboBoxArcSolid.Items.Strings[1]:=txt_solid_auto;
ComboBoxArcSolid.Items.Strings[2]:=txt_solid_extension;
ComboBoxARC2.Items.Strings[0]:=txt_newarchive;
ComboBoxARC2.Items.Strings[1]:=txt_add_existing_archive;
ComboBoxARC2.Items.Strings[2]:=txt_update;
ComboBoxARCalgo.Items.Strings[3]:=txt_blowfish;
cbType.Items.Strings[3]:=txt_custom;
cbType.Items.Strings[7]:=txt_sfx+' '+STR_7Z;
cbType.Items.Strings[8]:=txt_sfx+' '+STR_ARC;
cbType.Items.Strings[9]:=txt_split;
ComboBoxArchive1.Items.Strings[0]:=txt_singlevol;
ComboBoxArchive1.Items.Strings[1]:=txt_customsize;
ComboBoxArchive1.Items.Strings[2]:=txt_fd;
ComboBoxArchive1.Items.Strings[3]:=txt_attach5;
ComboBoxArchive1.Items.Strings[4]:=txt_attach10;
ComboBoxArchive1.Items.Strings[5]:=txt_cd650;
ComboBoxArchive1.Items.Strings[6]:=txt_cd700;
ComboBoxArchive1.Items.Strings[7]:=txt_fat32;
ComboBoxArchive1.Items.Strings[8]:=txt_dvd;
ComboBoxArchive1.Items.Strings[9]:=txt_dvddl;
ComboBoxOverwrite.Items.Strings[0]:=txt_ren_extracted;
ComboBoxOverwrite.Items.Strings[1]:=txt_ren_existing;
ComboBoxOverwrite.Items.Strings[2]:=txt_overwrite;
ComboBoxOverwrite.Items.Strings[3]:=txt_skip_existing;
RadioGroupAction.Items.Strings[0]:=txt_caption_extract;
RadioGroupAction.Items.Strings[1]:=txt_ext_nopath;
RadioGroupAction.Items.Strings[2]:=txt_caption_list;
RadioGroupAction.Items.Strings[3]:=txt_list_details;
RadioGroupAction.Items.Strings[4]:=txt_caption_test;
RadioGroupAction.ItemIndex:=0;
RadioGroupAction1.Items.Strings[0]:=txt_caption_extract;
RadioGroupAction1.Items.Strings[1]:=txt_ext_nopath;
RadioGroupAction1.Items.Strings[2]:=txt_caption_list;
RadioGroupAction1.Items.Strings[3]:=txt_caption_test;
RadioGroupAction1.ItemIndex:=0;
ComboBoxOverwriteARC.Items.Strings[0]:=txt_overwrite_askbefore;
ComboBoxOverwriteARC.Items.Strings[1]:=txt_overwrite;
ComboBoxOverwriteARC.Items.Strings[2]:=txt_skip_existing;
RadioGroupActionARC.Items.Strings[0]:=txt_caption_extract;
RadioGroupActionARC.Items.Strings[1]:=txt_ext_nopath;
RadioGroupActionARC.Items.Strings[2]:=txt_caption_list;
RadioGroupActionARC.Items.Strings[3]:=txt_list_details;
RadioGroupActionARC.Items.Strings[4]:=txt_caption_test;
RadioGroupActionARC.Items.Strings[5]:=txt_caption_repair;
RadioGroupActionARC.ItemIndex:=0;
ComboBoxArchiveCustom1.Items.Strings[0]:=txt_pio;
ComboBoxArchiveCustom1.Items.Strings[1]:=txt_poi;
ComboBoxArchiveCustom1.Items.Strings[2]:=txt_ipo;
ComboBoxArchiveCustom1.Items.Strings[3]:=txt_iop;
ComboBoxArchiveCustom1.Items.Strings[4]:=txt_opi;
ComboBoxArchiveCustom1.Items.Strings[5]:=txt_oip;
RadioGroup0.Items.Strings[3]:=txt_custom;
RadioGroup0.Items[7]:=txt_sfx+' '+STR_7Z;
RadioGroup0.Items[8]:=txt_sfx+' '+STR_ARC;
RadioGroup0.Items.Strings[9]:=txt_split;
RadioGroup0.Items.Strings[14]:=txt_lastused;
cbUn7zaRecursive.Hint:=txt_search_hint;
cbUn7zaRecursiveExclude.Hint:=txt_search_hint;
ComboBoxOverwrite.Hint:=txt_naming;
ComboBoxOverwriteARC.Hint:=txt_naming;
EditOPcustom1.Hint:=txt_otherparams;
LabelColor2.Hint:=txt_col_hint;
ComboBoxUPX.Text:=txt_best;
ComboBoxPEACompression1.Text:=txt_default_compression;
ComboBoxArchive7.Text:=txt_newarchive;
ComboBoxArchive8.Text:=txt_singlethread;
ComboBoxArchiveCustom.Text:=txt_pio;
ComboBoxArcSolid.Text:=txt_solid_auto;
ComboBoxARC2.Text:=txt_newarchive;
ComboBoxArchive1.Text:=txt_singlevol;
EditOpenIn.Text:=txt_none;
ComboBoxOverwrite.Text:=txt_ren_extracted;
ComboBoxArchiveCustom1.Text:=txt_pio;
DirectoryEdit2.DialogTitle:=txt_selectdir;
DirectoryEditthemegraphics.DialogTitle:=txt_selectdir;
OpenDialog1.Title:=txt_open_file;
OpenDialog2.Title:=txt_open_files;
OpenDialog4.Title:=txt_open_file;
OpenDialogKF.Title:=txt_keyfile;
SelectDirectoryDialog1.Title:=txt_selectdir;
OpenDialog3.Title:=txt_open_file;
SaveDialogPJ.Title:=txt_savejobdefinition;
SaveDialog2.Title:=txt_saveas;
mDiskConvert.Caption:=txt_convert;
po_convert.Caption:=txt_convert;
mextractarchive.caption:=txt_archive;
mtogglenew.caption:=txt_newfoldermenu+' ON/OFF';
mcancel.caption:=txt_2_7_cancel;
mgotype.caption:=txt_type;
except end; //try..except was added since some of those instruction goes in error for unknown reason under some circumstances
end;
end;

function valorize_headers:integer;
var
   s:utf8string;
begin
valorize_headers:=-1;
readln(t,s);//translator(s)
Form_peach.LabelLang2.Caption:=Form_peach.LabelLang2.Caption+' - '+s;
readln(t,s);//last revision's translator(s)
Form_peach.LabelLang2.Caption:=Form_peach.LabelLang2.Caption+' - '+s;
readln(t,s);//last revision date
Form_peach.LabelLang2.Caption:=Form_peach.LabelLang2.Caption+' - '+s;
readln(t,s);
readln(t,s);//peazip text group
if s<>'=== PeaZip text group ===' then exit
else valorize_headers:=0;
end;

function load_texts(lang:utf8string):integer; //valorize localized text strings
var
   s:utf8string;
   i:integer;
begin
load_texts:=-1;
try
   uassigntext(t,(executable_path+'res'+directoryseparator+'lang'+directoryseparator+lang));
   filemode:=0;
   reset(t);
   read_header(t);
   readln(t,s); //declaration
   if s<>'=== PeaZip language file ===' then
      begin
      closefile(t);
      exit;
      end;
   readln(t,s);//language
   Form_peach.LabelLang2.Caption:=s;
   readln(t,s); //version
   Form_peach.LabelLang2.Caption:=Form_peach.LabelLang2.Caption+' - '+s;
   i:=-1;
   if s<>PEAZIPVERSION then //PEAZIPREVISION is ignored since revisions are meant to share the same text and doesn't need to load localizations out of order as a different release will do
      if valorize_headers=0 then i:=fallback_valorize_text
      else
   else
      if valorize_headers=0 then i:=valorize_text;
   if i=0 then assign_guitext
   else
      begin
      closefile(t);
      exit;
      end;
   repeat //skip until about text group
      readln(t,s);
   until (eof(t)) or (s='=== about text group ===');
   if not eof(t) then
      begin
      Application.CreateForm(TForm3, Form3);
      readln(t,s);
      Form3.Caption:=txt_2_5_mini_help;
      Form3.MemoAbout.Lines.Clear;
      Form3.MemoAbout.Lines.Append(APPMAIN+' '+PEAZIPVERSION+PEAZIPREVISION+' - '+caption_build);
      Form3.MemoAbout.Lines.Append('');
      Form3.MemoAbout.Lines.Append(txt_read+' '+READE_LIST);
      Form3.MemoAbout.Lines.Append('');
      Form3.MemoAbout.Lines.Append(txt_write+' '+WRITEE_LIST);
      Form3.MemoAbout.Lines.Append('');
      repeat
         readln(t,s);
         Form3.MemoAbout.Lines.Append(s);
      until eof(t);
      end;
   closefile(t);
   load_texts:=0;
except
   try
      closefile(t);
   except
   end;
   load_texts:=-1;
end;
end;

function texts(lang:utf8string):integer;
begin
texts:=load_texts('en.txt'); //preload english language to valorize possibly untranslated strings (i.e. older translations)
if lang<>'en.txt' then
   if load_texts(lang)<>0 then //try to load language file sequentially; fallback to a slower recoursive procedure to valorize each known variable from lang file strings if the language file version doesn't match with PeaZip's version
      begin
      lang_critical_error_msg;
      lang_file:='en.txt'; //load english language on failure
      texts:=load_texts(lang_file);
      end
   else texts:=0;
if texts<>0 then
   begin
   conf_critical_error_msg;
   Application.Terminate;
   end;
end;

procedure msg_not_accessible(i:integer);
begin
MessageDlg(envtoutf8(paramstr(i))+' '+txt_not_accessible, mtWarning, [mbOK], 0);
end;

procedure msg_not_accessible_s(s:utf8string);
begin
MessageDlg(s+' '+txt_not_accessible, mtWarning, [mbOK], 0);
end;

procedure msg_not_accessible_list(s:utf8string);
begin
MessageDlg(s+' '+txt_not_accessible_list, mtWarning, [mbOK], 0);
end;

procedure msg_no_input;
begin
MessageDlg(txt_noinput, mtError, [mbOK], 0);
end;

procedure msg_pw_errorchar;
var
   errchar:utf8string;
begin
{$IFDEF MSWINDOWS}
errchar:='"';
{$ELSE}
errchar:='''';
{$ENDIF}
MessageDlg(errchar+' '+txt_2_3_pw_errorchar, mtWarning, [mbOK], 0);
end;

{///////////////////////////////////////////////////////////////////////////////

configuration and theming

///////////////////////////////////////////////////////////////////////////////}

procedure load_icons; //load icons from bitmaps
var
   i:integer;
begin
if imgloaded=true then exit;
if imgloading=true then exit;
imgloading:=true;
with Form_peach do
   begin
   try
   Barchive_bigger2:=TBitmap.Create;
   Baddfile:=TBitmap.Create;
   Baddfolder:=TBitmap.Create;
   Baddsearch:=TBitmap.Create;
   Barchive_bigger:=TBitmap.Create;
   BArchive_big:=TBitmap.Create;
   BCutarchive:=TBitmap.Create;
   BCutfile:=TBitmap.Create;
   BCutexe:=TBitmap.Create;
   BCutfolder:=TBitmap.Create;
   BCutunsupported:=TBitmap.Create;
   BArchive:=TBitmap.Create;
   BArchiveSupported:=TBitmap.Create;
   BFileSupported:=TBitmap.Create;
   BExesupported:=TBitmap.Create;
   BUnsupported:=TBitmap.Create;
   Bback:=TBitmap.Create;
   Bbookmark:=TBitmap.Create;
   Bbookmarkadd:=TBitmap.Create;
   Bclearlayout:=TBitmap.Create;
   Bclearlayout_big:=TBitmap.Create;
   BDefaults:=TBitmap.Create;
   Bdelete:=TBitmap.Create;
   Bdvd_supported:=TBitmap.Create;
   Bexpand:=TBitmap.Create;
   Bextractall:=TBitmap.Create;
   Bextractall_big:=TBitmap.Create;
   Bextractallto:=TBitmap.Create;
   Bextractallto_big:=TBitmap.Create;
   Bfd_supported:=TBitmap.Create;
   BFolder:=TBitmap.Create;
   BFolder2:=TBitmap.Create;
   Bft:=TBitmap.Create;
   Bgo:=TBitmap.Create;
   Bgopath:=TBitmap.Create;
   Bgoup:=TBitmap.Create;
   Bhd_supported:=TBitmap.Create;
   Bhistory:=TBitmap.Create;
   Binfo:=TBitmap.Create;
   Bkeyfile:=TBitmap.Create;
   Blisted:=TBitmap.Create;
   Blistedto:=TBitmap.Create;
   Bloadlayout:=TBitmap.Create;
   Blocker:=TBitmap.Create;
   Blocker2:=TBitmap.Create;
   Bmail:=TBitmap.Create;
   Bnav:=TBitmap.Create;
   Bp0:=TBitmap.Create;
   Bp1:=TBitmap.Create;
   Bp2:=TBitmap.Create;
   Bp3:=TBitmap.Create;
   Bp4:=TBitmap.Create;
   Bp5:=TBitmap.Create;
   Bp6:=TBitmap.Create;
   Bp7:=TBitmap.Create;
   Bp8:=TBitmap.Create;
   Bpea:=TBitmap.Create;
   Bpreview:=TBitmap.Create;
   Bram_supported:=TBitmap.Create;
   Brefresh_up:=TBitmap.Create;
   Brefresh:=TBitmap.Create;
   Bremote_supported:=TBitmap.Create;
   Bremovable_supported:=TBitmap.Create;
   Bresetpath:=TBitmap.Create;
   Broot:=TBitmap.Create;
   Brun:=TBitmap.Create;
   Bsavelayout:=TBitmap.Create;
   Bsearch:=TBitmap.Create;
   Bselected:=TBitmap.Create;
   Bselectedto:=TBitmap.Create;
   Bsystemtools:=TBitmap.Create;
   Btestall:=TBitmap.Create;
   Bstop:=TBitmap.Create;
   Bweb:=TBitmap.Create;
   Bwipe:=TBitmap.Create;
   Btestall_big:=TBitmap.Create;
   Blink:=TBitmap.Create;
   Bcutlink:=TBitmap.Create;
   Barchive_bigger2.LoadFromFile(executable_path+graphicsfolder+'archive_bigger2.bmp');
   Baddfile.LoadFromFile(executable_path+graphicsfolder+'addfile.bmp');
   Baddfolder.LoadFromFile(executable_path+graphicsfolder+'addfolder.bmp');
   Baddsearch.LoadFromFile(executable_path+graphicsfolder+'addsearch.bmp');
   Barchive_bigger.LoadFromFile(executable_path+graphicsfolder+'archive_bigger.bmp');
   BArchive_big.LoadFromFile(executable_path+graphicsfolder+'archive_big.bmp');
   BCutarchive.LoadFromFile(executable_path+graphicsfolder+'cut_archive.bmp');
   BCutfile.LoadFromFile(executable_path+graphicsfolder+'cut_file.bmp');
   BCutexe.LoadFromFile(executable_path+graphicsfolder+'cut_exe.bmp');
   BCutfolder.LoadFromFile(executable_path+graphicsfolder+'cut_folder.bmp');
   BCutunsupported.LoadFromFile(executable_path+graphicsfolder+'cut_unsupported.bmp');
   BArchive.LoadFromFile(executable_path+graphicsfolder+'archive.bmp');
   BArchiveSupported.LoadFromFile(executable_path+graphicsfolder+'archive_supported.bmp');
   BFileSupported.LoadFromFile(executable_path+graphicsfolder+'file_supported.bmp');
   BExeSupported.LoadFromFile(executable_path+graphicsfolder+'exe_supported.bmp');
   BUnsupported.LoadFromFile(executable_path+graphicsfolder+'unsupported.bmp');
   Bback.LoadFromFile(executable_path+graphicsfolder+'back.bmp');
   Bbookmark.LoadFromFile(executable_path+graphicsfolder+'bookmark.bmp');
   Bbookmarkadd.LoadFromFile(executable_path+graphicsfolder+'bookmarkadd.bmp');
   Bclearlayout.LoadFromFile(executable_path+graphicsfolder+'clearlayout.bmp');
   Bclearlayout_big.LoadFromFile(executable_path+graphicsfolder+'clearlayout_big.bmp');
   BDefaults.LoadFromFile(executable_path+graphicsfolder+'defaults.bmp');
   Bdelete.LoadFromFile(executable_path+graphicsfolder+'delete.bmp');
   Bdvd_supported.LoadFromFile(executable_path+graphicsfolder+'dvd_supported.bmp');
   Bexpand.LoadFromFile(executable_path+graphicsfolder+'expand.bmp');
   Bextractall.LoadFromFile(executable_path+graphicsfolder+'extractall.bmp');
   Bextractall_big.LoadFromFile(executable_path+graphicsfolder+'extractall_big.bmp');
   Bextractallto.LoadFromFile(executable_path+graphicsfolder+'extractallto.bmp');
   Bextractallto_big.LoadFromFile(executable_path+graphicsfolder+'extractallto_big.bmp');
   Bfd_supported.LoadFromFile(executable_path+graphicsfolder+'fd_supported.bmp');
   BFolder.LoadFromFile(executable_path+graphicsfolder+'folder.bmp');
   BFolder2.LoadFromFile(executable_path+graphicsfolder+'folder2.bmp');
   Bft.LoadFromFile(executable_path+graphicsfolder+'ft.bmp');
   Bgo.LoadFromFile(executable_path+graphicsfolder+'go.bmp');
   Bgopath.LoadFromFile(executable_path+graphicsfolder+'gopath.bmp');
   Bgoup.LoadFromFile(executable_path+graphicsfolder+'goup.bmp');
   Bhd_supported.LoadFromFile(executable_path+graphicsfolder+'hd_supported.bmp');
   Bhistory.LoadFromFile(executable_path+graphicsfolder+'history.bmp');
   Binfo.LoadFromFile(executable_path+graphicsfolder+'info.bmp');
   Bkeyfile.LoadFromFile(executable_path+graphicsfolder+'keyfile.bmp');
   Blisted.LoadFromFile(executable_path+graphicsfolder+'listed.bmp');
   Blistedto.LoadFromFile(executable_path+graphicsfolder+'listedto.bmp');
   Bloadlayout.LoadFromFile(executable_path+graphicsfolder+'loadlayout.bmp');
   Blocker.LoadFromFile(executable_path+graphicsfolder+'locker.bmp');
   Blocker2.LoadFromFile(executable_path+graphicsfolder+'locker2.bmp');
   Bmail.LoadFromFile(executable_path+graphicsfolder+'mail.bmp');
   Bnav.LoadFromFile(executable_path+graphicsfolder+'nav.bmp');
   Bp0.LoadFromFile(executable_path+graphicsfolder+'p0.bmp');
   Bp1.LoadFromFile(executable_path+graphicsfolder+'p1.bmp');
   Bp2.LoadFromFile(executable_path+graphicsfolder+'p2.bmp');
   Bp3.LoadFromFile(executable_path+graphicsfolder+'p3.bmp');
   Bp4.LoadFromFile(executable_path+graphicsfolder+'p4.bmp');
   Bp5.LoadFromFile(executable_path+graphicsfolder+'p5.bmp');
   Bp6.LoadFromFile(executable_path+graphicsfolder+'p6.bmp');
   Bp7.LoadFromFile(executable_path+graphicsfolder+'p7.bmp');
   Bp8.LoadFromFile(executable_path+graphicsfolder+'p8.bmp');
   Bpea.LoadFromFile(executable_path+graphicsfolder+'pea.bmp');
   Bpreview.LoadFromFile(executable_path+graphicsfolder+'preview.bmp');
   Bram_supported.LoadFromFile(executable_path+graphicsfolder+'ram_supported.bmp');
   Brefresh_up.LoadFromFile(executable_path+graphicsfolder+'refresh_up.bmp');
   Brefresh.LoadFromFile(executable_path+graphicsfolder+'refresh.bmp');
   Bremote_supported.LoadFromFile(executable_path+graphicsfolder+'remote_supported.bmp');
   Bremovable_supported.LoadFromFile(executable_path+graphicsfolder+'removable_supported.bmp');
   Bresetpath.LoadFromFile(executable_path+graphicsfolder+'resetpath.bmp');
   Broot.LoadFromFile(executable_path+graphicsfolder+'root.bmp');
   Brun.LoadFromFile(executable_path+graphicsfolder+'run.bmp');
   Bsavelayout.LoadFromFile(executable_path+graphicsfolder+'savelayout.bmp');
   Bsearch.LoadFromFile(executable_path+graphicsfolder+'search.bmp');
   Bselected.LoadFromFile(executable_path+graphicsfolder+'selected.bmp');
   Bselectedto.LoadFromFile(executable_path+graphicsfolder+'selectedto.bmp');
   Bsystemtools.LoadFromFile(executable_path+graphicsfolder+'systemtools.bmp');
   Bstop.LoadFromFile(executable_path+graphicsfolder+'stop.bmp');
   Btestall.LoadFromFile(executable_path+graphicsfolder+'testall.bmp');
   Bweb.LoadFromFile(executable_path+graphicsfolder+'web.bmp');
   Bwipe.LoadFromFile(executable_path+graphicsfolder+'wipe.bmp');
   Btestall_big.LoadFromFile(executable_path+graphicsfolder+'testall_big.bmp');
   Blink.LoadFromFile(executable_path+graphicsfolder+'link.bmp');
   Bcutlink.LoadFromFile(executable_path+graphicsfolder+'cut_link.bmp');
   Barchive_bigger2.Transparent:=True;
   Baddfile.Transparent:=True;
   Baddfolder.Transparent:=True;
   Baddsearch.Transparent:=True;
   Barchive_bigger.Transparent:=True;
   Barchive_big.Transparent:=True;
   BCutarchive.Transparent:=True;
   BCutfile.Transparent:=True;
   BCutexe.Transparent:=True;
   BCutfolder.Transparent:=True;
   BCutunsupported.Transparent:=True;
   BArchive.Transparent:=True;
   BArchiveSupported.Transparent:=True;
   BFileSupported.Transparent:=True;
   Bunsupported.Transparent:=True;
   for i:=1 to 26 do devicon[i]:=Bunsupported;//show "unsupported" icon if the bookmarked unit is not mounted
   BExeSupported.Transparent:=True;
   Bback.Transparent:=True;
   Bbookmark.Transparent:=True;
   Bbookmarkadd.Transparent:=True;
   Bclearlayout.Transparent:=True;
   Bclearlayout_big.Transparent:=True;
   Bdefaults.Transparent:=True;
   Bdelete.Transparent:=True;
   Bdvd_supported.Transparent:=True;
   Bexpand.Transparent:=True;
   Bextractall.Transparent:=True;
   Bextractall_big.Transparent:=True;
   Bextractallto.Transparent:=True;
   Bextractallto_big.Transparent:=True;
   Bfd_supported.Transparent:=True;
   BFolder.Transparent:=True;
   BFolder2.Transparent:=True;
   Bft.Transparent:=True;
   Bgo.Transparent:=True;
   Bgopath.Transparent:=True;
   Bgoup.Transparent:=True;
   Bhd_supported.Transparent:=True;
   Bhistory.Transparent:=True;
   Binfo.Transparent:=True;
   Bkeyfile.Transparent:=True;
   Blisted.Transparent:=True;
   Blistedto.Transparent:=True;
   Bloadlayout.Transparent:=True;
   Blocker.Transparent:=True;
   Blocker2.Transparent:=True;
   Bmail.Transparent:=True;
   Bnav.Transparent:=True;
   Bp0.Transparent:=True;
   Bp1.Transparent:=True;
   Bp2.Transparent:=True;
   Bp3.Transparent:=True;
   Bp4.Transparent:=True;
   Bp5.Transparent:=True;
   Bp6.Transparent:=True;
   Bp7.Transparent:=True;
   Bp8.Transparent:=True;
   Bpea.Transparent:=True;
   Bpreview.Transparent:=True;
   Bram_supported.Transparent:=True;
   Brefresh_up.Transparent:=True;
   Brefresh.Transparent:=True;
   Bremote_supported.Transparent:=True;
   Bremovable_supported.Transparent:=True;
   Bresetpath.Transparent:=True;
   Broot.Transparent:=True;
   Brun.Transparent:=True;
   Bsavelayout.Transparent:=True;
   Bsearch.Transparent:=True;
   Bselected.Transparent:=True;
   Bselectedto.Transparent:=True;
   Bsystemtools.Transparent:=True;
   Bstop.Transparent:=True;
   Btestall.Transparent:=True;
   Bweb.Transparent:=True;
   Bwipe.Transparent:=True;
   Blink.Transparent:=True;
   Bcutlink.Transparent:=True;
   ButtonArchive.Glyph:=Btestall;
   ButtonExtOk.Glyph:=Btestall;
   ButtonArchive1.Glyph:=Bstop;
   ButtonExtCancel.Glyph:=Bstop;
   ImageBookmarks.Glyph:=Bbookmark;
   ImageClip.Glyph:=Bloadlayout;
   ImageListHistory.Glyph:=Bhistory;
   ButtonUn7za1.Glyph:=Btestall_big;
   ButtonUn7za4.Glyph:=Barchive_big;
   ButtonUn7zaTo.Glyph:=Bextractall_big;
   ctrlback.Glyph:=Bback;
   ctrlfwd.Glyph:=Bgo;
   ctrlup.Glyph:=Bgoup;
   ctrlrefresh.Glyph:=Brefresh;
   ImageListSearch.Glyph:=Bsearch;
   msetpw.Bitmap:=Blocker2;
   ImagePassword.Glyph:=Blocker2;
   ImagePassword1.Glyph:=Blocker2;
   ImagePassword2.Glyph:=Blocker2;
   mkeyfile.Bitmap:=Bkeyfile;
   Imagece1.Picture.Bitmap:=Brun;
   Imagece1.Transparent:=true;
   Imagece2.Picture.Bitmap:=Brun;
   Imagece2.Transparent:=true;
   Imageopenback2.Picture.Bitmap:=Barchive_bigger;
   Imageopenback2.Transparent:=true;
   Image1.Picture.Bitmap:=Barchive_bigger2;
   Image1.Transparent:=true;
   Image2.Picture.Bitmap:=Barchive_bigger2;
   Image2.Transparent:=true;
   Image5.Picture.Bitmap:=Bft;
   Image5.Transparent:=true;
   Image6.Picture.Bitmap:=Bft;
   Image6.Transparent:=true;
   Image7.Picture.Bitmap:=Bft;
   Image7.Transparent:=true;
   Image8.Picture.Bitmap:=Barchive_bigger2;
   Image8.Transparent:=true;
   iConsoleCreateImport.Picture.Bitmap:=Bresetpath;
   iConsoleCreateImport.Transparent:=true;
   iConsoleCreateImport1.Glyph:=Bresetpath;
   ImageDefaults1.Picture.Bitmap:=Bpea;
   ImageDefaults1.Transparent:=true;
   ImageDefaults2.Glyph:=Bdefaults;
   ImageInfoArchive2.Picture.Bitmap:=Binfo;
   ImageInfoArchive2.Transparent:=true;
   ImageInfoArchive3.Picture.Bitmap:=Binfo;
   ImageInfoArchive3.Transparent:=true;
   ImageInfoArchive4.Picture.Bitmap:=Binfo;
   ImageInfoArchive4.Transparent:=true;
   ImageInfoDefault1.Picture.Bitmap:=Binfo;
   ImageInfoDefault1.Transparent:=true;
   ImageInfoDefault2.Picture.Bitmap:=Binfo;
   ImageInfoDefault2.Transparent:=true;
   ImageInfoDefault3.Picture.Bitmap:=Binfo;
   ImageInfoDefault3.Transparent:=true;
   ImageSplit.Transparent:=true;
   mBookmarks.Bitmap:=BBookmark;
   mRecent.Bitmap:=BHistory;
   smBookmarks1.Bitmap:=BBookmark;
   smBookmarksa1.Bitmap:=BBookmark;
   smRecent.Bitmap:=BHistory;
   smRecenta1.Bitmap:=BHistory;
   smfs1.Bitmap:=BRoot;
   smfsa1.Bitmap:=BRoot;
   pmjumpsbook.Bitmap:=BBookmark;
   pmjumpfs.Bitmap:=BRoot;
   mDefaults.Bitmap:=BDefaults;
   MenuItem3.Bitmap:=Bclearlayout;
   MenuItem5b.Bitmap:=Brun;
   pmjbadd.Bitmap:=BBookmarkadd;
   MenuItemBookmarks.Bitmap:=BBookmarkadd;
   MenuItemOpen_addtobookmarks.Bitmap:=BBookmarkadd;
   MenuItemHistoryBook.Bitmap:=Bbookmarkadd;
   MenuItemOpen_back.Bitmap:=Bback;
   MenuItemOpen_ext_all.Bitmap:=Bextractall;
   MenuItemOpen_ext_displayed.Bitmap:=Blisted;
   MenuItemOpen_ext_selected.Bitmap:=Bselected;
   MenuItemOpen_extandopenwith.Bitmap:=Brun;
   MenuItemOpen_ExtractAllTo.Bitmap:=Bextractallto;
   MenuItemOpen_ExtractDisplayedTo.Bitmap:=Blistedto;
   MenuItemOpen_ExtractHere.Bitmap:=Bextractall;
   MenuItemOpen_ExtractSelectedTo.Bitmap:=Bselectedto;
   MenuItemOpen_ExtractTo.Bitmap:=Bextractallto;
   MenuItemPW.Bitmap:=Blocker2;
   po_setpw.Bitmap:=Blocker2;
   po_searchhere.Bitmap:=Bgopath;
   po_searchrec.Bitmap:=Bsearch;
   smOpen.Bitmap:=Broot;
   smBookmarks.Bitmap:=BBookmark;
   smRecent1.Bitmap:=BHistory;
   pmRecent.Bitmap:=BHistory;
   MenuItemOpen_Flat.Bitmap:=Bexpand;
   MenuItemOpen_fwd.Bitmap:=Bgo;
   MenuItemOpen_jump.Bitmap:=Bgopath;
   MenuItemOpen_Preview.Bitmap:=Bpreview;
   MenuItemOpen_refresh.Bitmap:=Brefresh;
   MenuItemOpen_root.Bitmap:=Broot;
   MenuItemOpen_test.Bitmap:=Btestall;
   MenuItemOpen_up.Bitmap:=Bgoup;
   MenuItemOpenWeb.Bitmap:=Bweb;
   MenuItemRefresh.Bitmap:=Brefresh;
   mFlat.Bitmap:=Bexpand;
   mGotolayout.Bitmap:=Bnav;
   mPaths.Bitmap:=Broot;
   mPea.Bitmap:=BArchive;
   mPeaSite.Bitmap:=BWeb;
   mRefresh.Bitmap:=Brefresh;
   mSystem.Bitmap:=Bsystemtools;
   pmBrowser.Bitmap:=Bnav;
   mgoBrowser.Bitmap:=Bnav;
   pmAddFile.Bitmap:=Baddfile;
   pmAddFolder.Bitmap:=Baddfolder;
   pmLoadList.Bitmap:=Bloadlayout;
   pmSaveList.Bitmap:=Bsavelayout;
   pmSearchAdd.Bitmap:=Baddsearch;
   po_addtoarchive.Bitmap:=BArchive;
   mgotoextlayout.Bitmap:=Bnav;
   po_delete.Bitmap:=Bdelete;
   pmquickdelete.Bitmap:=Bdelete;
   po_exthere.Bitmap:=Bextractall;
   po_extto.Bitmap:=Bextractallto;
   po_open.Bitmap:=Brun;
   po_securedelete.Bitmap:=Bwipe;
   pmsecuredelete.Bitmap:=Bwipe;
   po_systemtools.Bitmap:=Bsystemtools;
   jdroot.Bitmap:=Bsystemtools;
   pmjump1.Bitmap:=Bsystemtools;
   MenuItemOpen_my.Bitmap:=Bsystemtools;
   po_test.Bitmap:=Btestall;
   theme_label1.Picture.Bitmap:=Bresetpath;
   theme_label1.Transparent:=true;
   LabelThemeLoaded.Caption:=txt_icons_found;
   imgloaded:=true;
   imgloading:=false;
   except
   LabelThemeLoaded.Caption:=txt_icons_not_found;
   end;
   end;
end;

procedure default_defaults;
begin
lang_file:='en.txt';//lang file
autoclosegwrap:=1; //close pealauncher when job completes if no error happens
autoopen:=0; //don't open output dir by default when job completes
df:=14; //default format: last used
pf:=12; //previous format, by default zip
pcompr:=1; //pea default Compression: average
pobj:=6; //pea default object control algorithm CRC64
pvol:=2; //pea default volume control algorithm SHA256
pstream:=0; //pea default stream control algorithm EAX AES 256
salgo:=6; //file split default integrity check algorithm: CRC64
zaout:=2; //ConsoleCreate binaries interface option: GUI, no pipes (except for list/test/benchmark)
clipmode:=0; //clipboard mode for cut/copy/paste: 0 standard mode, 1 advanced mode allowing multiple mixed cut and copy selection from various sources
{$IFDEF MSWINDOWS}
indir:=txt_mypc;
showguihints:=1; //peazip shows hint popups: 1 yes (default) 0 no
{$ELSE}
indir:=home_path;//later replaced using local_desktop
if indir<>'' then if indir[length(indir)]<>directoryseparator then indir:=indir+directoryseparator;
showguihints:=0; //show hint popups no by default on Linux, may be slow on some configurations
{$ENDIF} //initial directory (file/archive browser interface) (in case of conf failure at startup txt_my pc is empty, but this condition is checked in check_defaults_failure)
showpwfield:=0; //shows password fields: off (requires pw confirmation)
extpw:=1; //ask for password on Extract/list/test functions from system's meus entries: 0, no (default) 1,2 test (slower; 1 limits test to <100MB archives, otherwise skip), 3 always
browserstyle:=0; //0:browser 1:flat 2*:last used (*=0 browser, *=1 flat);
extopt7z:=0; //overwrite policy for extraction with 7z (rename files being extracted)
extoptarc:=2; //overwrite policy for extraction with arc (skip existing files)
ignorepathextand:=1; //extract and... extracts to current path ignoring internal archive's path: on
ignorepathdisp:=0; //ignore path extracting displayed objects: off
ignorepathsel:=0; //ignore path extracting selected objects: off
prebrowse:=1; //stop browsing if archive is listed in more than prebrowse*1M blocks of memory (dramatically faster for very much populated archives): on by default
prebrowsesize:=4; //max M blocks of memory allowed to list archive content in the browser
savehistory:=1; //save 8 last accessed archives
customsyntax:=0; //syntax type for custom RadioGroupQuad, creation
customsyntax1:=0; //syntax type for custom RadioGroupQuad, extraction
ws:=1; //save window status: on
ws_status:=0; //ws maximized: off
ws_left:=32;
ws_top:=16;
ws_width:=800;
ws_height:=512;
fav7z:=1;//archive types to be added to favourite popup
favARC:=0;
favbz:=1;
favcustom:=0;
favgz:=1;
favpea:=0;
favquad:=0;
favsfx:=1;
favsfxarc:=0;
favsplit:=0;
favtar:=1;
favupx:=0;
favzip:=1;
favzpaq:=0;
level_7z:=3;  //default Compression level for .7z: average
method_7z:='LZMA'; //default method for .7z: LZMA
level_arc:=4; //default Compression level for ARC
solid_arc:=1; //default solid mode for ARC 0 non-solid 1 solid, auto-adjust
rr_arc:=1; //default recovery records for ARC: ON
algo_arc:=0; //default encryption algorithm for ARC: 0 AES 1 Serpent 2 Twofish 3 Blowfish (always max keysize)
level_bzip2:=2; //default Compression level for bzip2: average
level_gz:=2; //default Compression level for gzip: average
level_paq:=1; //midi Compression level for zpaq
level_quad:=0; //default Compression level for quad: normal (high off)
level_upx:=10; //default Compression level for upx: best
level_zip:=3; //default Compression level for zip: average
method_zip:='Deflate'; //default method for .zip: deflate
openw_all7z:=0; //default archive files open for writing for 7z binary: off
algodisp:=0; //default way to display checksum/hash
algoall:=0; //default don't use all algorithms for file check; below are specified algorithms to be used
algoadler:=1;
algocrc16:=1;
algocrc24:=1;
algocrc32:=1;
algocrc64:=1;
algoed2k:=1;
algomd4:=0;
algomd5:=1;
algoripemd160:=1;
algosha1:=1;
algosha224:=0;
algosha256:=1;
algosha384:=0;
algosha512:=0;
algowhirl:=0;
erasepasses:=1;
jobdefenc:=1; //encoding for job definition 0 replace extended chars with ?; 1 utf8
archivenameenc:=1; //encoding for filenames in archives 0 replace extended chars with ?; 1 utf8
lastoutpath:=''; //last output path, saved only if save history feature is on (default)
splittersize:=192;
lastbar:=0;
esna:=0;
euns:=0;
checkver:='unchecked';//it make configuration fall back to defaults if conf is not read correctly, i.e. because of I/O errors or because it is found incompatible version's configuration (different number of lines)
end;

procedure check_theme_failure;
begin
if graphicsfolderd<>'' then if graphicsfolderd[length(graphicsfolderd)]<>directoryseparator then graphicsfolderd:=graphicsfolderd+directoryseparator;
//if opacityd<0 then opacityd:=0;
if opacityd>100 then opacityd:=100;
if opacityd=0 then opacityd:=100;
if color1d='' then color1d:=COL1;
if color2d='' then color2d:=COL2;
if color3d='' then color3d:=COL3;
if color4d='' then color4d:=COL4;
if color5d='' then color5d:=COL5;
if (rowheightd<12) or (rowheightd>32) then rowheightd:=DROWHEIGHT;
if (itemheightd<12) or (itemheightd>32) then itemheightd:=DROWHEIGHT+3;
if (autosizeitemheightd<0) or (autosizeitemheightd>1) then autosizeitemheightd:=1;
if (autosizetoolbarbuttond<0) or (autosizetoolbarbuttond>1) then autosizetoolbarbuttond:=0;
end;

function wingetprogramsfolder:integer;
{$IFDEF MSWINDOWS}
var
  pidl: PItemIDList;
  Buf: array [0..MAX_PATH] of Char;
{$ENDIF}
begin
wingetprogramsfolder:=-1;
{$IFDEF MSWINDOWS}
try
   if Succeeded(ShGetSpecialFolderLocation(Form_peach.Handle,$0026,pidl)) then //$0026 is CSIDL_PROGRAM_FILES numerical hex value
      if ShGetPathfromIDList(pidl, Buf ) then
         begin
         winpfolder:=envtoutf8(Buf);
         if winpfolder<>'' then if winpfolder[length(winpfolder)]<>directoryseparator then winpfolder:=winpfolder+directoryseparator;
         CoTaskMemFree(pidl);
         wingetprogramsfolder:=0;
         end
      else CoTaskMemFree(pidl);
except
end;
{$ENDIF}
end;

function wingetdesk:integer;
{$IFDEF MSWINDOWS}
var
  pidl: PItemIDList;
  Buf: array [0..MAX_PATH] of Char;
{$ENDIF}
begin
wingetdesk:=-1;
{$IFDEF MSWINDOWS}
try
   if Succeeded(ShGetSpecialFolderLocation(Form_peach.Handle,0,pidl)) then //0 is CSIDL_DESKTOP numerical value
      if ShGetPathfromIDList(pidl, Buf ) then
         begin
         desktop_path:=envtoutf8(Buf);
         CoTaskMemFree(pidl);
         if desktop_path<>'' then if desktop_path[length(desktop_path)]<>directoryseparator then desktop_path:=desktop_path+directoryseparator;
         wingetdesk:=0;
         end
      else CoTaskMemFree(pidl);
except
end;
{$ENDIF}
end;

function wingetuserprofile:integer;
{$IFDEF MSWINDOWS}
var
  pidl: PItemIDList;
  Buf: array [0..MAX_PATH] of Char;
{$ENDIF}
begin
wingetuserprofile:=-1;
{$IFDEF MSWINDOWS}
try
   if Succeeded(ShGetSpecialFolderLocation(Form_peach.Handle,0,pidl)) then //0 is CSIDL_DESKTOP numerical value
      if ShGetPathfromIDList(pidl, Buf ) then
         begin
         home_path:=envtoutf8(Buf);
         if home_path<>'' then if home_path[length(home_path)]=directoryseparator then setlength(home_path,length(home_path)-1);
         home_path:=uextractfilepath(home_path);
         if home_path<>'' then if home_path[length(home_path)]=directoryseparator then setlength(home_path,length(home_path)-1);
         usr_name:=uextractfilename(home_path);
         home_path:=home_path+directoryseparator;
         CoTaskMemFree(pidl);
         wingetuserprofile:=0;
         end
      else CoTaskMemFree(pidl);
except
end;
{$ENDIF}
end;

function wingetappdatafolder:integer;
{$IFDEF MSWINDOWS}
var
  pidl: PItemIDList;
  Buf: array [0..MAX_PATH] of Char;
{$ENDIF}
begin
wingetappdatafolder:=-1;
{$IFDEF MSWINDOWS}
try
   if Succeeded(ShGetSpecialFolderLocation(Form_peach.Handle,26,pidl)) then //26 is CSIDL_APPDATA numerical value
      if ShGetPathfromIDList(pidl, Buf ) then
         begin
         winappdatafolder:=envtoutf8(Buf);
         if winappdatafolder<>'' then if winappdatafolder[length(winappdatafolder)]<>directoryseparator then winappdatafolder:=winappdatafolder+directoryseparator;
         CoTaskMemFree(pidl);
         wingetappdatafolder:=0;
         end
      else CoTaskMemFree(pidl);
except
end;
{$ENDIF}
end;

function wingetappdata(var s:utf8string):integer;
{$IFDEF MSWINDOWS}
var
  pidl: PItemIDList;
  Buf: array [0..MAX_PATH] of Char;
{$ENDIF}
begin
wingetappdata:=-1;
{$IFDEF MSWINDOWS}
try
   if Succeeded(ShGetSpecialFolderLocation(Form_peach.Handle,26,pidl)) then //26 is CSIDL_APPDATA numerical value
      if ShGetPathfromIDList(pidl, Buf ) then
         begin
         s:=envtoutf8(Buf)+'\PeaZip\';
         CoTaskMemFree(pidl);
         wingetappdata:=0;
         end
      else CoTaskMemFree(pidl);
except
end;
{$ENDIF}
end;

function wingetdocuments:integer;
{$IFDEF MSWINDOWS}
var
  pidl: PItemIDList;
  Buf: array [0..MAX_PATH] of Char;
{$ENDIF}
begin
wingetdocuments:=-1;
{$IFDEF MSWINDOWS}
try
   if Succeeded(ShGetSpecialFolderLocation(Form_peach.Handle,5,pidl)) then //5 is CSIDL_PERSONAL (user's documents) numerical value
      if ShGetPathfromIDList(pidl, Buf ) then
         begin
         usr_documents:=envtoutf8(Buf);
         CoTaskMemFree(pidl);
         if usr_documents<>'' then if usr_documents[length(usr_documents)]<>directoryseparator then usr_documents:=usr_documents+directoryseparator;
         wingetdocuments:=0;
         end
      else CoTaskMemFree(pidl);
except
end;
{$ENDIF}
end;

procedure check_defaults_failure;
begin
//no control on lang_file name
if (autoclosegwrap<0) or (autoclosegwrap>2) then autoclosegwrap:=1;
if (autoopen<0) or (autoopen>1) then autoopen:=01;
if (df<0) or (df>14) then df:=14;
if (pf<0) or (pf>13) then pf:=12;
if (pcompr<0) or (pcompr>3) then pcompr:=1;
if (pobj<0) or (pobj>9) then pobj:=6;
if (pvol<0) or (pvol>9) then pvol:=2;
if (pstream<0) or (pstream>12) then pstream:=0;
if (salgo<0) or (salgo>9) then salgo:=6;
if (zaout<0) or (zaout>2) then zaout:=2;
if (clipmode<0) or (clipmode>1) then clipmode:=0;
if indir='' then
   begin
   {$IFDEF MSWINDOWS}
   indir:=txt_mypc;
   {$ELSE}
   indir:=home_path;
   if indir<>'' then if indir[length(indir)]<>directoryseparator then indir:=indir+directoryseparator;
   {$ENDIF}
   end;
if indir<>txt_mypc then
   if not(udirectoryexists((indir))) then
      begin
      {$IFDEF MSWINDOWS}
      indir:=txt_mypc;
      {$ELSE}
      indir:=home_path;
      if indir<>'' then if indir[length(indir)]<>directoryseparator then indir:=indir+directoryseparator;
      {$ENDIF}
      end;
if (showguihints<0) or (showguihints>1) then showguihints:=1;
if (showpwfield<0) or (showpwfield>1) then showpwfield:=0;
if (extpw<0) or (extpw>3) then extpw:=1;
if (extopt7z<0) or (extopt7z>3) then extopt7z:=0;
if (extoptarc<0) or (extoptarc>2) then extoptarc:=0;
if (ignorepathextand<0) or (ignorepathextand>1) then ignorepathextand:=1;
if (ignorepathdisp<0) or (ignorepathdisp>1) then ignorepathdisp:=0;
if (ignorepathsel<0) or (ignorepathsel>1) then ignorepathsel:=0;
if (prebrowse<0) or (prebrowse>1) then prebrowse:=1;
if (prebrowsesize<1) or (prebrowsesize>100) then prebrowsesize:=4;
if (savehistory<0) or (savehistory>1) then savehistory:=1;
if (customsyntax<0) or (customsyntax>5) then customsyntax:=0;
if (customsyntax1<0) or (customsyntax1>5) then customsyntax:=0;
if (ws<0) or (ws>1) then ws:=1;
if (ws_status<0) or (ws_status>1) then ws_status:=0; //default wsNormal
if (ws_left<0) or (ws_left>1024) then ws_left:=32; //don't allow to save the window position more than 1024*768 from lef-top corner of the screen
if (ws_top<0) or (ws_top>768) then ws_top:=32; //allow to use any window size, so don't check ws size
if (fav7z<0) or (fav7z>1) then fav7z:=1;
if (favARC<0) or (favARC>1) then favARC:=0;
if (favbz<0) or (favbz>1) then favbz:=1;
if (favcustom<0) or (favcustom>1) then favcustom:=0;
if (favgz<0) or (favgz>1) then favgz:=1;
if (favpea<0) or (favpea>1) then favpea:=0;
if (favquad<0) or (favquad>1) then favquad:=0;
if (favsplit<0) or (favsplit>1) then favsplit:=0;
if (favtar<0) or (favtar>1) then favtar:=1;
if (favsfx<0) or (favsfx>1) then favsfx:=1;
if (favsfxarc<0) or (favsfxarc>1) then favsfxarc:=0;
if (favupx<0) or (favupx>1) then favupx:=0;
if (favzip<0) or (favzip>1) then favzip:=1;
if (favzpaq<0) or (favzpaq>1) then favzpaq:=0;
if (level_7z<0) or (level_7z>5) then level_7z:=3;
if (method_7z<>'LZMA') and (method_7z<>'LZMA2') and(method_7z<>'PPMd') and (method_7z<>'BZip2') then method_7z:='LZMA';
if (level_arc<0) or (level_arc>9) then level_arc:=4;
if (solid_arc<0) or (solid_arc>2) then solid_arc:=1;
if (rr_arc<0) or (rr_arc>1) then rr_arc:=1;
if (algo_arc<0) or (algo_arc>3) then algo_arc:=0;
if (level_bzip2<0) or (level_bzip2>4) then level_bzip2:=2;
if (level_gz<0) or (level_gz>4) then level_gz:=2;
if (level_paq<0) or (level_paq>2) then level_paq:=1;
if (level_quad<0) or (level_quad>1) then level_quad:=0;
if (level_upx<0) or (level_upx>10) then level_upx:=10;
if (level_zip<0) or (level_zip>5) then level_zip:=3;
if (method_zip<>'Deflate') and (method_zip<>'Deflate64') and (method_zip<>'BZip2') and (method_zip<>'LZMA') then method_zip:='Deflate';
if (openw_all7z<0) or (openw_all7z>1) then openw_all7z:=0;
if (algodisp<0) or (algodisp>2) then algodisp:=0;
if (algoall<0) or (algoall>1) then algoall:=0;
if (algoadler<0) or (algoadler>1) then algoadler:=1;
if (algocrc16<0) or (algocrc16>1) then algocrc16:=0;
if (algocrc24<0) or (algocrc24>1) then algocrc24:=0;
if (algocrc32<0) or (algocrc32>1) then algocrc32:=1;
if (algocrc64<0) or (algocrc64>1) then algocrc64:=0;
if (algoed2k<0) or (algoed2k>1) then algoed2k:=0;
if (algomd4<0) or (algomd4>1) then algomd4:=0;
if (algomd5<0) or (algomd5>1) then algomd5:=1;
if (algoripemd160<0) or (algoripemd160>1) then algoripemd160:=1;
if (algosha1<0) or (algosha1>1) then algosha1:=1;
if (algosha224<0) or (algosha224>1) then algosha224:=0;
if (algosha256<0) or (algosha256>1) then algosha256:=1;
if (algosha384<0) or (algosha384>1) then algosha384:=0;
if (algosha512<0) or (algosha512>1) then algosha512:=0;
if (algowhirl<0) or (algowhirl>1) then algowhirl:=0;
if (erasepasses<0) or (erasepasses>3) then erasepasses:=1;
if (jobdefenc<0) or (jobdefenc>1) then jobdefenc:=1;
if (archivenameenc<0) or (archivenameenc>1) then archivenameenc:=1;
if (local_desktop='') or (not(udirectoryexists(local_desktop))) then local_desktop:=desktop_path;
if local_desktop<>'' then if local_desktop[length(local_desktop)]<>directoryseparator then local_desktop:=local_desktop+directoryseparator;
if (splittersize<16) then splittersize:=128;
if (lastbar<0) or (lastbar>3) then lastbar:=0;
if (esna<0) or (esna>1) then esna:=0;
if (euns<0) or (euns>1) then euns:=0;
end;

procedure set_items_height;
begin
with Form_peach do
begin

if autosizeitemheight=0 then editheight:=itemheight
else
   {$IFDEF MSWINDOWS}
   if PixelsPerInch<100 then editheight:=21 //standard height on Windows 96 DPI
   else
      if PixelsPerInch<125 then editheight:=24 //covers Windows 120 DPI
      else editheight:=26;
   {$ELSE}
   editheight:=26;//being unknown font and decorations, use a big size
   {$ENDIF}

cbitemheight:=editheight-6;
barheight:=56+editheight-21;//superseeded if autosizetoolbarbutton=1
refgroupheight:=286+(editheight-21)*6;
secgroupheight:=140+(editheight-21)*3;
toolbtnheight:=barheight-7;
if editheight>26 then stdbtnheight:=editheight else stdbtnheight:=26;
StringGrid1.DefaultRowHeight:=rowheight;
StringGridList.DefaultRowHeight:=rowheight;
StringGridHistory.DefaultRowHeight:=rowheight;
StringGridBookmarks.DefaultRowHeight:=rowheight;
StringGridClipboard.DefaultRowHeight:=rowheight;
StringGridCustEdit.DefaultRowHeight:=rowheight;
Image7zPj.Height:=stdbtnheight;
Image7zPj1.Height:=stdbtnheight;
EditUn7zafilter.Height:=editheight;
EditUn7zafilter1.Height:=editheight;
EditOpenIn.Height:=editheight;
EditOpenIn1.Height:=editheight;
imagebookmarks.height:=editheight+3;
imagelisthistory.height:=editheight+3;
ctrlback.Height:=editheight+3;
ctrlfwd.Height:=editheight+3;
ctrlup.Height:=editheight+3;
ctrlrefresh.Height:=editheight+3;
imagelistroot.Height:=editheight+3;
imagelistsearch.Height:=editheight+3;
if editheight>21 then PanelListBar.Height:=editheight+8 else PanelListBar.Height:=29;
Panelside.height:=PanelListBar.Height;
imagepassword.height:=editheight+3;
imagepassword1.height:=editheight+3;
imagepassword2.height:=editheight+3;
ImageUn7zPj.Height:=stdbtnheight;
Image7zPj3.Height:=stdbtnheight;
EditOpenOut.Height:=editheight;
ButtonEditName5.Height:=editheight;
ButtonUn7za10.Height:=editheight;
EditNameCustom1.Height:=editheight;
ButtonEditName4.Height:=editheight;
ButtonEditName8.Height:=editheight;
EditOPCustom1.Height:=editheight;
RadioGroup4.ItemHeight:=cbitemheight;
RadioGroup6.ItemHeight:=cbitemheight;
ButtonDesk.Height:=editheight;
EditDesk.Height:=editheight;
RadioGroup0.ItemHeight:=cbitemheight;
RadioGroupBrowser.ItemHeight:=cbitemheight;
RadioGrouppwrequest.ItemHeight:=cbitemheight;
ComboBoxOverwrite.ItemHeight:=cbitemheight;
RadioGroupAction.ItemHeight:=cbitemheight;
ButtonLang.Height:=editheight;
ComboBoxPasses.ItemHeight:=cbitemheight;
ComboBoxTheme.ItemHeight:=cbitemheight;
DirectoryEditThemeGraphics.Height:=editheight;
Edit1.Height:=editheight;
Edit2.Height:=editheight;
Edit3.Height:=editheight;
DirectoryEdit2.Height:=editheight;
Button1.Height:=stdbtnheight;
ImageDefaults2.Height:=stdbtnheight;
ImageDefaults3.Height:=stdbtnheight;
PanelArchive2.Height:=refgroupheight+2;
Panel7za.Height:=refgroupheight+2;
Group7za2.Height:=refgroupheight;
Group7za1.Height:=secgroupheight;
GroupBox7za3.Height:=refgroupheight-secgroupheight-6;
PanelARC.Height:=refgroupheight+2;
CompressionARC.Height:=refgroupheight;
OptionsARC.Height:=secgroupheight;
GroupBox7za4.Height:=refgroupheight-secgroupheight-6;
PanelPea.Height:=refgroupheight+2;
CompressionPea.Height:=refgroupheight;
GroupBoxPeaAdvanced1.Height:=secgroupheight;
EncryptionPea.Height:=refgroupheight-secgroupheight-6;
PanelPAQ.Height:=refgroupheight+2;
CompressionPAQ.Height:=refgroupheight;
OptionsPAQ.Height:=secgroupheight;
EncryptionPAQ.Height:=refgroupheight-secgroupheight-6;
PanelQUAD.Height:=refgroupheight+2;
CompressionQUAD.Height:=refgroupheight;
OptionsQUAD.Height:=secgroupheight;
EncryptionQUAD.Height:=refgroupheight-secgroupheight-6;
PanelUPX.Height:=refgroupheight+2;
CompressionUPX.Height:=refgroupheight;
OptionsUPX.Height:=secgroupheight;
EncryptionUPX.Height:=refgroupheight-secgroupheight-6;
PanelCustom.Height:=refgroupheight+2;
CompressionCustom.Height:=refgroupheight;
cbType.ItemHeight:=cbitemheight;
ComboBoxARC.ItemHeight:=cbitemheight;
ComboBoxARC2.ItemHeight:=cbitemheight;
ComboBoxARCsolid.ItemHeight:=cbitemheight;
ComboBoxARCalgo.ItemHeight:=cbitemheight;
EditName3.Height:=editheight;
Edit4.Height:=editheight;
ButtonEditName3.Height:=editheight;
ButtonEditName12.Height:=editheight;
ComboBoxArchive1.ItemHeight:=cbitemheight;
ComboBoxArchive2.ItemHeight:=cbitemheight;
ComboBoxArchive3.ItemHeight:=cbitemheight;
ComboBoxArchive4.ItemHeight:=cbitemheight;
ComboBoxArchive5.ItemHeight:=cbitemheight;
ComboBoxArchive6.ItemHeight:=cbitemheight;
ComboBoxArchive7.ItemHeight:=cbitemheight;
ComboBoxArchive8.ItemHeight:=cbitemheight;
ComboBoxArchiveSolid.ItemHeight:=cbitemheight;
ComboBoxArchivePasses.ItemHeight:=cbitemheight;
ComboBoxArchiveCustom.ItemHeight:=cbitemheight;
ComboBox7zalgo.ItemHeight:=cbitemheight;
ComboBoxOverwrite.ItemHeight:=cbitemheight;
ComboBoxPasses.ItemHeight:=cbitemheight;
ComboBoxPAQ1.ItemHeight:=cbitemheight;
ComboBoxPEA2.ItemHeight:=cbitemheight;
ComboBoxPEA3.ItemHeight:=cbitemheight;
ComboBoxPEA4.ItemHeight:=cbitemheight;
ComboBoxPEACompression1.ItemHeight:=cbitemheight;
ComboBoxSplit3.ItemHeight:=cbitemheight;
ComboBoxTheme.ItemHeight:=cbitemheight;
ComboBoxUPX.ItemHeight:=cbitemheight;
EditOP.Height:=editheight;
EditOParc.Height:=editheight;
EditOPcustom.Height:=editheight;
Editnamecustom.Height:=editheight;
Editextcustom.Height:=editheight;
SpinEdit1.Height:=editheight;
SpinEdit2.Height:=editheight;
SpinEdit3.Height:=editheight;
SpinEditArchive1.Height:=editheight;
SpinEditPrebrowse.Height:=editheight;
ButtonEditNameCustom.Height:=editheight;
ButtonEditNameCustom1.Height:=editheight;
Edit5.Height:=editheight;
{ComboBoxArchiveCustom1.ItemHeight:=cbitemheight;
RadioGroupAction1.ItemHeight:=cbitemheight;
RadioGroupActionARC.ItemHeight:=cbitemheight;
ComboBoxOverwriteARC.ItemHeight:=cbitemheight;}
end;
end;

procedure apply_theme; //apply colors but not opacity
begin
//grids
Form_peach.StringGrid1.AlternateColor:=stringtocolor(color1);
Form_peach.StringGridList.AlternateColor:=stringtocolor(color1);
Form_peach.StringGridHistory.AlternateColor:=stringtocolor(color1);
Form_peach.StringGridBookmarks.AlternateColor:=stringtocolor(color1);
Form_peach.StringGridClipboard.AlternateColor:=stringtocolor(color1);
Form_peach.StringGridCustEdit.AlternateColor:=stringtocolor(color1);
Form_peach.StringGridCustEdit1.AlternateColor:=stringtocolor(color1);
set_items_height;//set custom height to other GUI items which may be to small for some font sizes, accordingly to grid height
//toolbars
Form_peach.PanelBarOpen.Color:=stringtocolor(color2);
//PanelDefaults
Form_peach.ColorButton1.ButtonColor:=stringtocolor(color1);
Form_peach.ColorButton2.ButtonColor:=stringtocolor(color2);
Form_peach.ColorButton3.ButtonColor:=stringtocolor(color3);
Form_peach.ColorButton4.ButtonColor:=stringtocolor(color4);
Form_peach.ColorButton5.ButtonColor:=stringtocolor(color5);
Form_peach.SpinEdit2.Value:=rowheight;
Form_peach.SpinEdit3.Value:=itemheight;
if autosizeitemheight=1 then
   begin
   Form_peach.CheckBoxIH.State:=cbChecked;
   Form_peach.Panel3.visible:=true;
   end
else
   begin
   Form_peach.CheckBoxIH.State:=cbUnchecked;
   Form_peach.Panel3.visible:=false;
   end;
if autosizetoolbarbutton=1 then
   begin
   Form_peach.CheckBoxTBW.State:=cbChecked;
   end
else
   begin
   Form_peach.CheckBoxTBW.State:=cbUnchecked;
   end;
Form_peach.LabelDefaults1.Font.Color:=stringtocolor(color4);
//PanelOpen
   //PanelFilters
   Form_peach.LabelOpenBack2.Font.Color:=stringtocolor(color4);
//PanelArchiveMain
//PanelArchive
//PanelArchive2
//PanelKeyFile
//load icons
load_icons;
end;

procedure readconf_colors;
var
   s:utf8string;
begin
readln(conf,graphicsfolder);
readln(conf,opacity);
readln(conf,color1);
readln(conf,color2);
readln(conf,color3);
readln(conf,color4);
readln(conf,color5);
readln(conf,s);
rowheight:=strtoint(s);
readln(conf,s);
itemheight:=strtoint(s);
readln(conf,s);
autosizeitemheight:=strtoint(s);
readln(conf,s);
autosizetoolbarbutton:=strtoint(s);
end;

procedure readconf_default_colors;
var
   s:utf8string;
begin
readln(conf,graphicsfolderd);
udodirseparators(graphicsfolderd);
readln(conf,opacityd);
readln(conf,color1d);
readln(conf,color2d);
readln(conf,color3d);
readln(conf,color4d);
readln(conf,color5d);
readln(conf,s);
rowheightd:=strtoint(s);
readln(conf,s);
itemheightd:=strtoint(s);
readln(conf,s);
autosizeitemheightd:=strtoint(s);
readln(conf,s);
autosizetoolbarbuttond:=strtoint(s);
end;

procedure set_ow_custedit_menu;
begin
with Form_peach do
begin
if StringGridCustEdit.Cells[1,1]<>'' then owcustom1.Caption:=StringGridCustEdit.Cells[1,1]
else
   if uextractfilename(custedit1)<>'' then owcustom1.Caption:=uextractfilename(custedit1)
   else owcustom1.Caption:=custedit1;
if StringGridCustEdit.Cells[1,2]<>'' then owcustom2.Caption:=StringGridCustEdit.Cells[1,2]
else
   if uextractfilename(custedit2)<>'' then owcustom2.Caption:=uextractfilename(custedit2)
   else owcustom2.Caption:=custedit2;
if StringGridCustEdit.Cells[1,3]<>'' then owcustom3.Caption:=StringGridCustEdit.Cells[1,3]
else
   if uextractfilename(custedit3)<>'' then owcustom3.Caption:=uextractfilename(custedit3)
   else owcustom3.Caption:=custedit3;
if StringGridCustEdit.Cells[1,4]<>'' then owcustom4.Caption:=StringGridCustEdit.Cells[1,4]
else
   if uextractfilename(custedit4)<>'' then owcustom4.Caption:=uextractfilename(custedit4)
   else owcustom4.Caption:=custedit4;
if StringGridCustEdit.Cells[1,5]<>'' then owcustom5.Caption:=StringGridCustEdit.Cells[1,5]
else
   if uextractfilename(custedit5)<>'' then owcustom5.Caption:=uextractfilename(custedit5)
   else owcustom5.Caption:=custedit5;
if StringGridCustEdit.Cells[1,6]<>'' then owcustom6.Caption:=StringGridCustEdit.Cells[1,6]
else
   if uextractfilename(custedit6)<>'' then owcustom6.Caption:=uextractfilename(custedit6)
   else owcustom6.Caption:=custedit6;
if StringGridCustEdit.Cells[1,7]<>'' then owcustom7.Caption:=StringGridCustEdit.Cells[1,7]
else
   if uextractfilename(custedit7)<>'' then owcustom7.Caption:=uextractfilename(custedit7)
   else owcustom7.Caption:=custedit7;
if StringGridCustEdit.Cells[1,8]<>'' then owcustom8.Caption:=StringGridCustEdit.Cells[1,8]
else
   if uextractfilename(custedit8)<>'' then owcustom8.Caption:=uextractfilename(custedit8)
   else owcustom8.Caption:=custedit8;
if StringGridCustEdit1.Cells[1,1]<>'' then owcustom9.Caption:=StringGridCustEdit1.Cells[1,1]
else owcustom9.Caption:=advedit1before+delimiter+'%f'+delimiter+advedit1after;
if StringGridCustEdit1.Cells[1,2]<>'' then owcustom10.Caption:=StringGridCustEdit1.Cells[1,2]
else owcustom10.Caption:=advedit2before+delimiter+'%f'+delimiter+advedit2after;
if StringGridCustEdit1.Cells[1,3]<>'' then owcustom11.Caption:=StringGridCustEdit1.Cells[1,3]
else owcustom11.Caption:=advedit3before+delimiter+'%f'+delimiter+advedit3after;
if StringGridCustEdit1.Cells[1,4]<>'' then owcustom12.Caption:=StringGridCustEdit1.Cells[1,4]
else owcustom12.Caption:=advedit4before+delimiter+'%f'+delimiter+advedit4after;
if StringGridCustEdit1.Cells[1,5]<>'' then owcustom13.Caption:=StringGridCustEdit1.Cells[1,5]
else owcustom13.Caption:=advedit5before+delimiter+'%f'+delimiter+advedit5after;
if StringGridCustEdit1.Cells[1,6]<>'' then owcustom14.Caption:=StringGridCustEdit1.Cells[1,6]
else owcustom14.Caption:=advedit6before+delimiter+'%f'+delimiter+advedit6after;
if StringGridCustEdit1.Cells[1,7]<>'' then owcustom15.Caption:=StringGridCustEdit1.Cells[1,7]
else owcustom15.Caption:=advedit7before+delimiter+'%f'+delimiter+advedit7after;
if StringGridCustEdit1.Cells[1,8]<>'' then owcustom16.Caption:=StringGridCustEdit1.Cells[1,8]
else owcustom16.Caption:=advedit8before+delimiter+'%f'+delimiter+advedit8after;
if custedit1<>'' then owcustom1.visible:=true else owcustom1.visible:=false;
if custedit2<>'' then owcustom2.visible:=true else owcustom2.visible:=false;
if custedit3<>'' then owcustom3.visible:=true else owcustom3.visible:=false;
if custedit4<>'' then owcustom4.visible:=true else owcustom4.visible:=false;
if custedit5<>'' then owcustom5.visible:=true else owcustom5.visible:=false;
if custedit6<>'' then owcustom6.visible:=true else owcustom6.visible:=false;
if custedit7<>'' then owcustom7.visible:=true else owcustom7.visible:=false;
if custedit8<>'' then owcustom8.visible:=true else owcustom8.visible:=false;
if advedit1before+advedit1after<>'' then owcustom9.visible:=true else owcustom9.visible:=false;
if advedit2before+advedit2after<>'' then owcustom10.visible:=true else owcustom10.visible:=false;
if advedit3before+advedit3after<>'' then owcustom11.visible:=true else owcustom11.visible:=false;
if advedit4before+advedit4after<>'' then owcustom12.visible:=true else owcustom12.visible:=false;
if advedit5before+advedit5after<>'' then owcustom13.visible:=true else owcustom13.visible:=false;
if advedit6before+advedit6after<>'' then owcustom14.visible:=true else owcustom14.visible:=false;
if advedit7before+advedit7after<>'' then owcustom15.visible:=true else owcustom15.visible:=false;
if advedit8before+advedit8after<>'' then owcustom16.visible:=true else owcustom16.visible:=false;
if custedit1+custedit2+custedit3+custedit4+custedit5+custedit6+custedit7+custedit8<>'' then owseparator1.visible:=true else owseparator1.visible:=false;
if advedit1before+advedit2before+advedit3before+advedit4before+advedit5before+advedit6before+advedit7before+advedit8before+
   advedit1after+advedit2after+advedit3after+advedit4after+advedit5after+advedit6after+advedit7after+advedit8after<>'' then owseparator2.visible:=true else owseparator2.visible:=false;
end;
end;

procedure set_eow_custedit_menu;
begin
with Form_peach do
begin
if StringGridCustEdit.Cells[1,1]<>'' then eowcustom1.Caption:=StringGridCustEdit.Cells[1,1]
else
   if uextractfilename(custedit1)<>'' then eowcustom1.Caption:=uextractfilename(custedit1)
   else eowcustom1.Caption:=custedit1;
if StringGridCustEdit.Cells[1,2]<>'' then eowcustom2.Caption:=StringGridCustEdit.Cells[1,2]
else
   if uextractfilename(custedit2)<>'' then eowcustom2.Caption:=uextractfilename(custedit2)
   else eowcustom2.Caption:=custedit2;
if StringGridCustEdit.Cells[1,3]<>'' then eowcustom3.Caption:=StringGridCustEdit.Cells[1,3]
else
   if uextractfilename(custedit3)<>'' then eowcustom3.Caption:=uextractfilename(custedit3)
   else eowcustom3.Caption:=custedit3;
if StringGridCustEdit.Cells[1,4]<>'' then eowcustom4.Caption:=StringGridCustEdit.Cells[1,4]
else
   if uextractfilename(custedit4)<>'' then eowcustom4.Caption:=uextractfilename(custedit4)
   else eowcustom4.Caption:=custedit4;
if StringGridCustEdit.Cells[1,5]<>'' then eowcustom5.Caption:=StringGridCustEdit.Cells[1,5]
else
   if uextractfilename(custedit5)<>'' then eowcustom5.Caption:=uextractfilename(custedit5)
   else eowcustom5.Caption:=custedit5;
if StringGridCustEdit.Cells[1,6]<>'' then eowcustom6.Caption:=StringGridCustEdit.Cells[1,6]
else
   if uextractfilename(custedit6)<>'' then eowcustom6.Caption:=uextractfilename(custedit6)
   else eowcustom6.Caption:=custedit6;
if StringGridCustEdit.Cells[1,7]<>'' then eowcustom7.Caption:=StringGridCustEdit.Cells[1,7]
else
   if uextractfilename(custedit7)<>'' then eowcustom7.Caption:=uextractfilename(custedit7)
   else eowcustom7.Caption:=custedit7;
if StringGridCustEdit.Cells[1,8]<>'' then eowcustom8.Caption:=StringGridCustEdit.Cells[1,8]
else
   if uextractfilename(custedit8)<>'' then eowcustom8.Caption:=uextractfilename(custedit8)
   else eowcustom8.Caption:=custedit8;
if StringGridCustEdit1.Cells[1,1]<>'' then eowcustom9.Caption:=StringGridCustEdit1.Cells[1,1]
else eowcustom9.Caption:=advedit1before+delimiter+'%f'+delimiter+advedit1after;
if StringGridCustEdit1.Cells[1,2]<>'' then eowcustom10.Caption:=StringGridCustEdit1.Cells[1,2]
else eowcustom10.Caption:=advedit2before+delimiter+'%f'+delimiter+advedit2after;
if StringGridCustEdit1.Cells[1,3]<>'' then eowcustom11.Caption:=StringGridCustEdit1.Cells[1,3]
else eowcustom11.Caption:=advedit3before+delimiter+'%f'+delimiter+advedit3after;
if StringGridCustEdit1.Cells[1,4]<>'' then eowcustom12.Caption:=StringGridCustEdit1.Cells[1,4]
else eowcustom12.Caption:=advedit4before+delimiter+'%f'+delimiter+advedit4after;
if StringGridCustEdit1.Cells[1,5]<>'' then eowcustom13.Caption:=StringGridCustEdit1.Cells[1,5]
else eowcustom13.Caption:=advedit5before+delimiter+'%f'+delimiter+advedit5after;
if StringGridCustEdit1.Cells[1,6]<>'' then eowcustom14.Caption:=StringGridCustEdit1.Cells[1,6]
else eowcustom14.Caption:=advedit6before+delimiter+'%f'+delimiter+advedit6after;
if StringGridCustEdit1.Cells[1,7]<>'' then eowcustom15.Caption:=StringGridCustEdit1.Cells[1,7]
else eowcustom15.Caption:=advedit7before+delimiter+'%f'+delimiter+advedit7after;
if StringGridCustEdit1.Cells[1,8]<>'' then eowcustom16.Caption:=StringGridCustEdit1.Cells[1,8]
else eowcustom16.Caption:=advedit8before+delimiter+'%f'+delimiter+advedit8after;
if custedit1<>'' then eowcustom1.visible:=true else eowcustom1.visible:=false;
if custedit2<>'' then eowcustom2.visible:=true else eowcustom2.visible:=false;
if custedit3<>'' then eowcustom3.visible:=true else eowcustom3.visible:=false;
if custedit4<>'' then eowcustom4.visible:=true else eowcustom4.visible:=false;
if custedit5<>'' then eowcustom5.visible:=true else eowcustom5.visible:=false;
if custedit6<>'' then eowcustom6.visible:=true else eowcustom6.visible:=false;
if custedit7<>'' then eowcustom7.visible:=true else eowcustom7.visible:=false;
if custedit8<>'' then eowcustom8.visible:=true else eowcustom8.visible:=false;
if advedit1before+advedit1after<>'' then eowcustom9.visible:=true else eowcustom9.visible:=false;
if advedit2before+advedit2after<>'' then eowcustom10.visible:=true else eowcustom10.visible:=false;
if advedit3before+advedit3after<>'' then eowcustom11.visible:=true else eowcustom11.visible:=false;
if advedit4before+advedit4after<>'' then eowcustom12.visible:=true else eowcustom12.visible:=false;
if advedit5before+advedit5after<>'' then eowcustom13.visible:=true else eowcustom13.visible:=false;
if advedit6before+advedit6after<>'' then eowcustom14.visible:=true else eowcustom14.visible:=false;
if advedit7before+advedit7after<>'' then eowcustom15.visible:=true else eowcustom15.visible:=false;
if advedit8before+advedit8after<>'' then eowcustom16.visible:=true else eowcustom16.visible:=false;
if custedit1+custedit2+custedit3+custedit4+custedit5+custedit6+custedit7+custedit8<>'' then eowseparator1.visible:=true else eowseparator1.visible:=false;
if advedit1before+advedit2before+advedit3before+advedit4before+advedit5before+advedit6before+advedit7before+advedit8before+
   advedit1after+advedit2after+advedit3after+advedit4after+advedit5after+advedit6after+advedit7after+advedit8after<>'' then eowseparator2.visible:=true else eowseparator2.visible:=false;
end;
end;

procedure set_pow_custedit_menu;
begin
with Form_peach do
begin
if StringGridCustEdit.Cells[1,1]<>'' then powcustom1.Caption:=StringGridCustEdit.Cells[1,1]
else
   if uextractfilename(custedit1)<>'' then powcustom1.Caption:=uextractfilename(custedit1)
   else powcustom1.Caption:=custedit1;
if StringGridCustEdit.Cells[1,2]<>'' then powcustom2.Caption:=StringGridCustEdit.Cells[1,2]
else
   if uextractfilename(custedit2)<>'' then powcustom2.Caption:=uextractfilename(custedit2)
   else powcustom2.Caption:=custedit2;
if StringGridCustEdit.Cells[1,3]<>'' then powcustom3.Caption:=StringGridCustEdit.Cells[1,3]
else
   if uextractfilename(custedit3)<>'' then powcustom3.Caption:=uextractfilename(custedit3)
   else powcustom3.Caption:=custedit3;
if StringGridCustEdit.Cells[1,4]<>'' then powcustom4.Caption:=StringGridCustEdit.Cells[1,4]
else
   if uextractfilename(custedit4)<>'' then powcustom4.Caption:=uextractfilename(custedit4)
   else powcustom4.Caption:=custedit4;
if StringGridCustEdit.Cells[1,5]<>'' then powcustom5.Caption:=StringGridCustEdit.Cells[1,5]
else
   if uextractfilename(custedit5)<>'' then powcustom5.Caption:=uextractfilename(custedit5)
   else powcustom5.Caption:=custedit5;
if StringGridCustEdit.Cells[1,6]<>'' then powcustom6.Caption:=StringGridCustEdit.Cells[1,6]
else
   if uextractfilename(custedit6)<>'' then powcustom6.Caption:=uextractfilename(custedit6)
   else powcustom6.Caption:=custedit6;
if StringGridCustEdit.Cells[1,7]<>'' then powcustom7.Caption:=StringGridCustEdit.Cells[1,7]
else
   if uextractfilename(custedit7)<>'' then powcustom7.Caption:=uextractfilename(custedit7)
   else powcustom7.Caption:=custedit7;
if StringGridCustEdit.Cells[1,8]<>'' then powcustom8.Caption:=StringGridCustEdit.Cells[1,8]
else
   if uextractfilename(custedit8)<>'' then powcustom8.Caption:=uextractfilename(custedit8)
   else powcustom8.Caption:=custedit8;
if StringGridCustEdit1.Cells[1,1]<>'' then powcustom9.Caption:=StringGridCustEdit1.Cells[1,1]
else powcustom9.Caption:=advedit1before+delimiter+'%f'+delimiter+advedit1after;
if StringGridCustEdit1.Cells[1,2]<>'' then powcustom10.Caption:=StringGridCustEdit1.Cells[1,2]
else powcustom10.Caption:=advedit2before+delimiter+'%f'+delimiter+advedit2after;
if StringGridCustEdit1.Cells[1,3]<>'' then powcustom11.Caption:=StringGridCustEdit1.Cells[1,3]
else powcustom11.Caption:=advedit3before+delimiter+'%f'+delimiter+advedit3after;
if StringGridCustEdit1.Cells[1,4]<>'' then powcustom12.Caption:=StringGridCustEdit1.Cells[1,4]
else powcustom12.Caption:=advedit4before+delimiter+'%f'+delimiter+advedit4after;
if StringGridCustEdit1.Cells[1,5]<>'' then powcustom13.Caption:=StringGridCustEdit1.Cells[1,5]
else powcustom13.Caption:=advedit5before+delimiter+'%f'+delimiter+advedit5after;
if StringGridCustEdit1.Cells[1,6]<>'' then powcustom14.Caption:=StringGridCustEdit1.Cells[1,6]
else powcustom14.Caption:=advedit6before+delimiter+'%f'+delimiter+advedit6after;
if StringGridCustEdit1.Cells[1,7]<>'' then powcustom15.Caption:=StringGridCustEdit1.Cells[1,7]
else powcustom15.Caption:=advedit7before+delimiter+'%f'+delimiter+advedit7after;
if StringGridCustEdit1.Cells[1,8]<>'' then powcustom16.Caption:=StringGridCustEdit1.Cells[1,8]
else powcustom16.Caption:=advedit8before+delimiter+'%f'+delimiter+advedit8after;
if custedit1<>'' then powcustom1.visible:=true else powcustom1.visible:=false;
if custedit2<>'' then powcustom2.visible:=true else powcustom2.visible:=false;
if custedit3<>'' then powcustom3.visible:=true else powcustom3.visible:=false;
if custedit4<>'' then powcustom4.visible:=true else powcustom4.visible:=false;
if custedit5<>'' then powcustom5.visible:=true else powcustom5.visible:=false;
if custedit6<>'' then powcustom6.visible:=true else powcustom6.visible:=false;
if custedit7<>'' then powcustom7.visible:=true else powcustom7.visible:=false;
if custedit8<>'' then powcustom8.visible:=true else powcustom8.visible:=false;
if advedit1before+advedit1after<>'' then powcustom9.visible:=true else powcustom9.visible:=false;
if advedit2before+advedit2after<>'' then powcustom10.visible:=true else powcustom10.visible:=false;
if advedit3before+advedit3after<>'' then powcustom11.visible:=true else powcustom11.visible:=false;
if advedit4before+advedit4after<>'' then powcustom12.visible:=true else powcustom12.visible:=false;
if advedit5before+advedit5after<>'' then powcustom13.visible:=true else powcustom13.visible:=false;
if advedit6before+advedit6after<>'' then powcustom14.visible:=true else powcustom14.visible:=false;
if advedit7before+advedit7after<>'' then powcustom15.visible:=true else powcustom15.visible:=false;
if advedit8before+advedit8after<>'' then powcustom16.visible:=true else powcustom16.visible:=false;
if custedit1+custedit2+custedit3+custedit4+custedit5+custedit6+custedit7+custedit8<>'' then powseparator1.visible:=true else powseparator1.visible:=false;
if advedit1before+advedit2before+advedit3before+advedit4before+advedit5before+advedit6before+advedit7before+advedit8before+
   advedit1after+advedit2after+advedit3after+advedit4after+advedit5after+advedit6after+advedit7after+advedit8after<>'' then powseparator2.visible:=true else powseparator2.visible:=false;
end;
end;

procedure set_aow_custedit_menu;
begin
with Form_peach do
begin
if StringGridCustEdit.Cells[1,1]<>'' then aowcustom1.Caption:=StringGridCustEdit.Cells[1,1]
else
   if uextractfilename(custedit1)<>'' then aowcustom1.Caption:=uextractfilename(custedit1)
   else aowcustom1.Caption:=custedit1;
if StringGridCustEdit.Cells[1,2]<>'' then aowcustom2.Caption:=StringGridCustEdit.Cells[1,2]
else
   if uextractfilename(custedit2)<>'' then aowcustom2.Caption:=uextractfilename(custedit2)
   else aowcustom2.Caption:=custedit2;
if StringGridCustEdit.Cells[1,3]<>'' then aowcustom3.Caption:=StringGridCustEdit.Cells[1,3]
else
   if uextractfilename(custedit3)<>'' then aowcustom3.Caption:=uextractfilename(custedit3)
   else aowcustom3.Caption:=custedit3;
if StringGridCustEdit.Cells[1,4]<>'' then aowcustom4.Caption:=StringGridCustEdit.Cells[1,4]
else
   if uextractfilename(custedit4)<>'' then aowcustom4.Caption:=uextractfilename(custedit4)
   else aowcustom4.Caption:=custedit4;
if StringGridCustEdit.Cells[1,5]<>'' then aowcustom5.Caption:=StringGridCustEdit.Cells[1,5]
else
   if uextractfilename(custedit5)<>'' then aowcustom5.Caption:=uextractfilename(custedit5)
   else aowcustom5.Caption:=custedit5;
if StringGridCustEdit.Cells[1,6]<>'' then aowcustom6.Caption:=StringGridCustEdit.Cells[1,6]
else
   if uextractfilename(custedit6)<>'' then aowcustom6.Caption:=uextractfilename(custedit6)
   else aowcustom6.Caption:=custedit6;
if StringGridCustEdit.Cells[1,7]<>'' then aowcustom7.Caption:=StringGridCustEdit.Cells[1,7]
else
   if uextractfilename(custedit7)<>'' then aowcustom7.Caption:=uextractfilename(custedit7)
   else aowcustom7.Caption:=custedit7;
if StringGridCustEdit.Cells[1,8]<>'' then aowcustom8.Caption:=StringGridCustEdit.Cells[1,8]
else
   if uextractfilename(custedit8)<>'' then aowcustom8.Caption:=uextractfilename(custedit8)
   else aowcustom8.Caption:=custedit8;
if StringGridCustEdit1.Cells[1,1]<>'' then aowcustom9.Caption:=StringGridCustEdit1.Cells[1,1]
else aowcustom9.Caption:=advedit1before+delimiter+'%f'+delimiter+advedit1after;
if StringGridCustEdit1.Cells[1,2]<>'' then aowcustom10.Caption:=StringGridCustEdit1.Cells[1,2]
else aowcustom10.Caption:=advedit2before+delimiter+'%f'+delimiter+advedit2after;
if StringGridCustEdit1.Cells[1,3]<>'' then aowcustom11.Caption:=StringGridCustEdit1.Cells[1,3]
else aowcustom11.Caption:=advedit3before+delimiter+'%f'+delimiter+advedit3after;
if StringGridCustEdit1.Cells[1,4]<>'' then aowcustom12.Caption:=StringGridCustEdit1.Cells[1,4]
else aowcustom12.Caption:=advedit4before+delimiter+'%f'+delimiter+advedit4after;
if StringGridCustEdit1.Cells[1,5]<>'' then aowcustom13.Caption:=StringGridCustEdit1.Cells[1,5]
else aowcustom13.Caption:=advedit5before+delimiter+'%f'+delimiter+advedit5after;
if StringGridCustEdit1.Cells[1,6]<>'' then aowcustom14.Caption:=StringGridCustEdit1.Cells[1,6]
else aowcustom14.Caption:=advedit6before+delimiter+'%f'+delimiter+advedit6after;
if StringGridCustEdit1.Cells[1,7]<>'' then aowcustom15.Caption:=StringGridCustEdit1.Cells[1,7]
else aowcustom15.Caption:=advedit7before+delimiter+'%f'+delimiter+advedit7after;
if StringGridCustEdit1.Cells[1,8]<>'' then aowcustom16.Caption:=StringGridCustEdit1.Cells[1,8]
else aowcustom16.Caption:=advedit8before+delimiter+'%f'+delimiter+advedit8after;
if custedit1<>'' then aowcustom1.visible:=true else aowcustom1.visible:=false;
if custedit2<>'' then aowcustom2.visible:=true else aowcustom2.visible:=false;
if custedit3<>'' then aowcustom3.visible:=true else aowcustom3.visible:=false;
if custedit4<>'' then aowcustom4.visible:=true else aowcustom4.visible:=false;
if custedit5<>'' then aowcustom5.visible:=true else aowcustom5.visible:=false;
if custedit6<>'' then aowcustom6.visible:=true else aowcustom6.visible:=false;
if custedit7<>'' then aowcustom7.visible:=true else aowcustom7.visible:=false;
if custedit8<>'' then aowcustom8.visible:=true else aowcustom8.visible:=false;
if advedit1before+advedit1after<>'' then aowcustom9.visible:=true else aowcustom9.visible:=false;
if advedit2before+advedit2after<>'' then aowcustom10.visible:=true else aowcustom10.visible:=false;
if advedit3before+advedit3after<>'' then aowcustom11.visible:=true else aowcustom11.visible:=false;
if advedit4before+advedit4after<>'' then aowcustom12.visible:=true else aowcustom12.visible:=false;
if advedit5before+advedit5after<>'' then aowcustom13.visible:=true else aowcustom13.visible:=false;
if advedit6before+advedit6after<>'' then aowcustom14.visible:=true else aowcustom14.visible:=false;
if advedit7before+advedit7after<>'' then aowcustom15.visible:=true else aowcustom15.visible:=false;
if advedit8before+advedit8after<>'' then aowcustom16.visible:=true else aowcustom16.visible:=false;
if custedit1+custedit2+custedit3+custedit4+custedit5+custedit6+custedit7+custedit8<>'' then aowseparator1.visible:=true else aowseparator1.visible:=false;
if advedit1before+advedit2before+advedit3before+advedit4before+advedit5before+advedit6before+advedit7before+advedit8before+
   advedit1after+advedit2after+advedit3after+advedit4after+advedit5after+advedit6after+advedit7after+advedit8after<>'' then aowseparator2.visible:=true else aowseparator2.visible:=false;
end;
end;

procedure set_custedit_menus;
begin
set_ow_custedit_menu;
set_eow_custedit_menu;
set_pow_custedit_menu;
set_aow_custedit_menu;
end;

procedure set_custedit_fromtable;
begin
custedit1:=Form_peach.StringGridCustEdit.Cells[2,1];
custedit2:=Form_peach.StringGridCustEdit.Cells[2,2];
custedit3:=Form_peach.StringGridCustEdit.Cells[2,3];
custedit4:=Form_peach.StringGridCustEdit.Cells[2,4];
custedit5:=Form_peach.StringGridCustEdit.Cells[2,5];
custedit6:=Form_peach.StringGridCustEdit.Cells[2,6];
custedit7:=Form_peach.StringGridCustEdit.Cells[2,7];
custedit8:=Form_peach.StringGridCustEdit.Cells[2,8];
advedit1before:=Form_peach.StringGridCustEdit1.Cells[2,1];
advedit2before:=Form_peach.StringGridCustEdit1.Cells[2,2];
advedit3before:=Form_peach.StringGridCustEdit1.Cells[2,3];
advedit4before:=Form_peach.StringGridCustEdit1.Cells[2,4];
advedit5before:=Form_peach.StringGridCustEdit1.Cells[2,5];
advedit6before:=Form_peach.StringGridCustEdit1.Cells[2,6];
advedit7before:=Form_peach.StringGridCustEdit1.Cells[2,7];
advedit8before:=Form_peach.StringGridCustEdit1.Cells[2,8];
advedit1after:=Form_peach.StringGridCustEdit1.Cells[3,1];
advedit2after:=Form_peach.StringGridCustEdit1.Cells[3,2];
advedit3after:=Form_peach.StringGridCustEdit1.Cells[3,3];
advedit4after:=Form_peach.StringGridCustEdit1.Cells[3,4];
advedit5after:=Form_peach.StringGridCustEdit1.Cells[3,5];
advedit6after:=Form_peach.StringGridCustEdit1.Cells[3,6];
advedit7after:=Form_peach.StringGridCustEdit1.Cells[3,7];
advedit8after:=Form_peach.StringGridCustEdit1.Cells[3,8];
end;

procedure set_custedit_totable;
begin
Form_peach.StringGridCustEdit.Cells[2,1]:=custedit1;
Form_peach.StringGridCustEdit.Cells[2,2]:=custedit2;
Form_peach.StringGridCustEdit.Cells[2,3]:=custedit3;
Form_peach.StringGridCustEdit.Cells[2,4]:=custedit4;
Form_peach.StringGridCustEdit.Cells[2,5]:=custedit5;
Form_peach.StringGridCustEdit.Cells[2,6]:=custedit6;
Form_peach.StringGridCustEdit.Cells[2,7]:=custedit7;
Form_peach.StringGridCustEdit.Cells[2,8]:=custedit8;
Form_peach.StringGridCustEdit.AutoSizeColumns;
Form_peach.StringGridCustEdit1.Cells[2,1]:=advedit1before;
Form_peach.StringGridCustEdit1.Cells[2,2]:=advedit2before;
Form_peach.StringGridCustEdit1.Cells[2,3]:=advedit3before;
Form_peach.StringGridCustEdit1.Cells[2,4]:=advedit4before;
Form_peach.StringGridCustEdit1.Cells[2,5]:=advedit5before;
Form_peach.StringGridCustEdit1.Cells[2,6]:=advedit6before;
Form_peach.StringGridCustEdit1.Cells[2,7]:=advedit7before;
Form_peach.StringGridCustEdit1.Cells[2,8]:=advedit8before;
Form_peach.StringGridCustEdit1.Cells[3,1]:=advedit1after;
Form_peach.StringGridCustEdit1.Cells[3,2]:=advedit2after;
Form_peach.StringGridCustEdit1.Cells[3,3]:=advedit3after;
Form_peach.StringGridCustEdit1.Cells[3,4]:=advedit4after;
Form_peach.StringGridCustEdit1.Cells[3,5]:=advedit5after;
Form_peach.StringGridCustEdit1.Cells[3,6]:=advedit6after;
Form_peach.StringGridCustEdit1.Cells[3,7]:=advedit7after;
Form_peach.StringGridCustEdit1.Cells[3,8]:=advedit8after;
Form_peach.StringGridCustEdit1.AutoSizeColumns;
end;

procedure readconf_defaults;
var
   s:utf8string;
begin
readln(conf,s);
readln(conf,s);
readln(conf,s); autoclosegwrap:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); autoopen:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); zaout:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); clipmode:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); indir:=s;
readln(conf,s);
readln(conf,s);
readln(conf,s); df:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); pf:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); pcompr:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); pobj:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); pvol:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); pstream:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); salgo:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); showguihints:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); showpwfield:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); extpw:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); browserstyle:=strtoint(s);
if (browserstyle=0) or (browserstyle=20) then browsingmode7z:=0 else browsingmode7z:=1;
readln(conf,s);
readln(conf,s);
readln(conf,s); ignorepathextand:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ignorepathdisp:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ignorepathsel:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); prebrowse:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); prebrowsesize:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); extopt7z:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); extoptarc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); savehistory:=strtoint(s);
readln(conf,s); Form_peach.mRecent1.Caption:=s;
readln(conf,s); Form_peach.mRecent2.Caption:=s;
readln(conf,s); Form_peach.mRecent3.Caption:=s;
readln(conf,s); Form_peach.mRecent4.Caption:=s;
readln(conf,s); Form_peach.mRecent5.Caption:=s;
readln(conf,s); Form_peach.mRecent6.Caption:=s;
readln(conf,s); Form_peach.mRecent7.Caption:=s;
readln(conf,s); Form_peach.mRecent8.Caption:=s;
Form_peach.poRecent1.Caption:=Form_peach.mRecent1.Caption;
Form_peach.poRecent2.Caption:=Form_peach.mRecent2.Caption;
Form_peach.poRecent3.Caption:=Form_peach.mRecent3.Caption;
Form_peach.poRecent4.Caption:=Form_peach.mRecent4.Caption;
Form_peach.poRecent5.Caption:=Form_peach.mRecent5.Caption;
Form_peach.poRecent6.Caption:=Form_peach.mRecent6.Caption;
Form_peach.poRecent7.Caption:=Form_peach.mRecent7.Caption;
Form_peach.poRecent8.Caption:=Form_peach.mRecent8.Caption;
Form_peach.pmr1.Caption:=Form_peach.mRecent1.Caption;
Form_peach.pmr2.Caption:=Form_peach.mRecent2.Caption;
Form_peach.pmr3.Caption:=Form_peach.mRecent3.Caption;
Form_peach.pmr4.Caption:=Form_peach.mRecent4.Caption;
Form_peach.pmr5.Caption:=Form_peach.mRecent5.Caption;
Form_peach.pmr6.Caption:=Form_peach.mRecent6.Caption;
Form_peach.pmr7.Caption:=Form_peach.mRecent7.Caption;
Form_peach.pmr8.Caption:=Form_peach.mRecent8.Caption;
Form_peach.pnr1.Caption:=uextractfilepath(Form_peach.mRecent1.Caption);
Form_peach.pnr2.Caption:=uextractfilepath(Form_peach.mRecent2.Caption);
Form_peach.pnr3.Caption:=uextractfilepath(Form_peach.mRecent3.Caption);
Form_peach.pnr4.Caption:=uextractfilepath(Form_peach.mRecent4.Caption);
Form_peach.pnr5.Caption:=uextractfilepath(Form_peach.mRecent5.Caption);
Form_peach.pnr6.Caption:=uextractfilepath(Form_peach.mRecent6.Caption);
Form_peach.pnr7.Caption:=uextractfilepath(Form_peach.mRecent7.Caption);
Form_peach.pnr8.Caption:=uextractfilepath(Form_peach.mRecent8.Caption);
Form_peach.pnra1.Caption:=uextractfilepath(Form_peach.mRecent1.Caption);
Form_peach.pnra2.Caption:=uextractfilepath(Form_peach.mRecent2.Caption);
Form_peach.pnra3.Caption:=uextractfilepath(Form_peach.mRecent3.Caption);
Form_peach.pnra4.Caption:=uextractfilepath(Form_peach.mRecent4.Caption);
Form_peach.pnra5.Caption:=uextractfilepath(Form_peach.mRecent5.Caption);
Form_peach.pnra6.Caption:=uextractfilepath(Form_peach.mRecent6.Caption);
Form_peach.pnra7.Caption:=uextractfilepath(Form_peach.mRecent7.Caption);
Form_peach.pnra8.Caption:=uextractfilepath(Form_peach.mRecent8.Caption);
readln(conf,s);
readln(conf,s);
readln(conf,s); Form_peach.pcustom1.Caption:=s;
readln(conf,s); Form_peach.pcustom2.Caption:=s;
readln(conf,s); Form_peach.pcustom3.Caption:=s;
readln(conf,s); Form_peach.pcustom4.Caption:=s;
readln(conf,s); Form_peach.pcustom5.Caption:=s;
readln(conf,s); Form_peach.pcustom6.Caption:=s;
readln(conf,s); Form_peach.pcustom7.Caption:=s;
readln(conf,s); Form_peach.pcustom8.Caption:=s;
readln(conf,s); Form_peach.EditOPCustom.Caption:=s;
readln(conf,s); customsyntax:=strtoint(s);
readln(conf,s); Form_peach.EditOPCustom1.Caption:=s;
readln(conf,s); customsyntax1:=strtoint(s);
//load last window's status, if ws=0 those values are ignored
readln(conf,s);
readln(conf,s);
readln(conf,s); ws:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ws_status:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ws_left:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ws_top:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ws_width:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); ws_height:=strtoint(s);
//load favourite archive types
readln(conf,s);
readln(conf,s);
readln(conf,s); fav7z:=strtoint(s);
readln(conf,s); favARC:=strtoint(s);
readln(conf,s); favbz:=strtoint(s);
readln(conf,s); favcustom:=strtoint(s);
readln(conf,s); favgz:=strtoint(s);
readln(conf,s); favpea:=strtoint(s);
readln(conf,s); favquad:=strtoint(s);
readln(conf,s); favsfx:=strtoint(s);
readln(conf,s); favsfxarc:=strtoint(s);
readln(conf,s); favsplit:=strtoint(s);
readln(conf,s); favtar:=strtoint(s);
readln(conf,s); favupx:=strtoint(s);
readln(conf,s); favzip:=strtoint(s);
readln(conf,s); favzpaq:=strtoint(s);
//load default Compression levels
readln(conf,s);
readln(conf,s);
readln(conf,s); level_7z:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); method_7z:=s;
readln(conf,s);
readln(conf,s);
readln(conf,s); level_arc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); solid_arc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); rr_arc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); algo_arc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); level_bzip2:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); level_gz:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); level_paq:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); level_quad:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); level_upx:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); level_zip:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); method_zip:=s;
readln(conf,s);
readln(conf,s);
readln(conf,s); openw_all7z:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); algodisp:=strtoint(s);
readln(conf,s); algoall:=strtoint(s);
readln(conf,s); algoadler:=strtoint(s);
readln(conf,s); algocrc16:=strtoint(s);
readln(conf,s); algocrc24:=strtoint(s);
readln(conf,s); algocrc32:=strtoint(s);
readln(conf,s); algocrc64:=strtoint(s);
readln(conf,s); algoed2k:=strtoint(s);
readln(conf,s); algomd4:=strtoint(s);
readln(conf,s); algomd5:=strtoint(s);
readln(conf,s); algoripemd160:=strtoint(s);
readln(conf,s); algosha1:=strtoint(s);
readln(conf,s); algosha224:=strtoint(s);
readln(conf,s); algosha256:=strtoint(s);
readln(conf,s); algosha384:=strtoint(s);
readln(conf,s); algosha512:=strtoint(s);
readln(conf,s); algowhirl:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); erasepasses:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); local_desktop:=s;
readln(conf,s);
readln(conf,s);
readln(conf,s); jobdefenc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); archivenameenc:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); lastoutpath:=s;
if lastoutpath='' then lastoutpath:=local_desktop;
readln(conf,s);
readln(conf,s);
readln(conf,s); splittersize:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); lastbar:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); esna:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); euns:=strtoint(s);
readln(conf,s);
readln(conf,s);
readln(conf,s); checkver:=s;
end;

procedure writeconf_colors;
begin
writeln(conf,graphicsfolder);
writeln(conf,opacity);
writeln(conf,color1);
writeln(conf,color2);
writeln(conf,color3);
writeln(conf,color4);
writeln(conf,color5);
writeln(conf,inttostr(rowheight));
writeln(conf,inttostr(itemheight));
writeln(conf,inttostr(autosizeitemheight));
writeln(conf,inttostr(autosizetoolbarbutton));
end;

procedure writeconf_default_colors;
begin
writeln(conf,graphicsfolderd);
writeln(conf,opacityd);
writeln(conf,color1d);
writeln(conf,color2d);
writeln(conf,color3d);
writeln(conf,color4d);
writeln(conf,color5d);
writeln(conf,inttostr(rowheightd));
writeln(conf,inttostr(itemheightd));
writeln(conf,inttostr(autosizeitemheightd));
writeln(conf,inttostr(autosizetoolbarbuttond));
end;

procedure writeconf_defaults;
begin
writeln(conf,'[language]');
writeln(conf,lang_file);
writeln(conf,'');
writeln(conf,'[pealauncher: close when job ends]');
writeln(conf,inttostr(autoclosegwrap));
writeln(conf,'');
writeln(conf,'[pealauncher: open output directory when job ends]');
writeln(conf,inttostr(autoopen));
writeln(conf,'');
writeln(conf,'[console binaries interface]');
writeln(conf,inttostr(zaout));
writeln(conf,'');
writeln(conf,'[Clipboard mode]');
writeln(conf,inttostr(clipmode));
writeln(conf,'');
writeln(conf,'[Initial dir for file/archive browser]');
writeln(conf,indir);
writeln(conf,'');
writeln(conf,'[default archiving format]');
writeln(conf,inttostr(df));
writeln(conf,'');
writeln(conf,'[last used archiving format]');
writeln(conf,inttostr(pf));
writeln(conf,'');
writeln(conf,'[default PEA compression level]');
writeln(conf,inttostr(pcompr));
writeln(conf,'');
writeln(conf,'[default PEA object level control algorithm]');
writeln(conf,inttostr(pobj));
writeln(conf,'');
writeln(conf,'[default PEA volume level control algorithm]');
writeln(conf,inttostr(pvol));
writeln(conf,'');
writeln(conf,'[default PEA stream level control algorithm]');
writeln(conf,inttostr(pstream));
writeln(conf,'');
writeln(conf,'[default integrity check algorithm for file split]');
writeln(conf,inttostr(salgo));
writeln(conf,'');
writeln(conf,'[show hints popup]');
writeln(conf,inttostr(showguihints));
writeln(conf,'');
writeln(conf,'[show password field in clear]');
writeln(conf,inttostr(showpwfield));
writeln(conf,'');
writeln(conf,'[ask for password on extract/list/test operations from system''s menus]');
writeln(conf,inttostr(extpw));
writeln(conf,'');
writeln(conf,'[archive browser style]');
writeln(conf,inttostr(browserstyle));
writeln(conf,'');
writeln(conf,'[ignore archived path structure when performing extract and run or preview jobs]');
writeln(conf,inttostr(ignorepathextand));
writeln(conf,'');
writeln(conf,'[ignore archived path structure when performing extraction of displayed objects]');
writeln(conf,inttostr(ignorepathdisp));
writeln(conf,'');
writeln(conf,'[ignore archived path structure when performing extraction of selected objects]');
writeln(conf,inttostr(ignorepathsel));
writeln(conf,'');
writeln(conf,'[pre-browse archive to give statistical information about all content]');
writeln(conf,inttostr(prebrowse));
writeln(conf,'');
writeln(conf,'[don''t pre-browse archives over this size]');
writeln(conf,inttostr(prebrowsesize));
writeln(conf,'');
writeln(conf,'[overwrite / skip policy using 7z binary]');
writeln(conf,inttostr(extopt7z));
writeln(conf,'');
writeln(conf,'[overwrite / skip policy using arc binary]');
writeln(conf,inttostr(extoptarc));
writeln(conf,'');
writeln(conf,'[save history of recent archives]');
writeln(conf,inttostr(savehistory));
writeln(conf,Form_peach.mRecent1.Caption);
writeln(conf,Form_peach.mRecent2.Caption);
writeln(conf,Form_peach.mRecent3.Caption);
writeln(conf,Form_peach.mRecent4.Caption);
writeln(conf,Form_peach.mRecent5.Caption);
writeln(conf,Form_peach.mRecent6.Caption);
writeln(conf,Form_peach.mRecent7.Caption);
writeln(conf,Form_peach.mRecent8.Caption);
writeln(conf,'');
writeln(conf,'[recently used custom executables]');
writeln(conf,Form_peach.pcustom1.Caption);
writeln(conf,Form_peach.pcustom2.Caption);
writeln(conf,Form_peach.pcustom3.Caption);
writeln(conf,Form_peach.pcustom4.Caption);
writeln(conf,Form_peach.pcustom5.Caption);
writeln(conf,Form_peach.pcustom6.Caption);
writeln(conf,Form_peach.pcustom7.Caption);
writeln(conf,Form_peach.pcustom8.Caption);
writeln(conf,Form_peach.EditOPcustom.Caption);
writeln(conf,inttostr(customsyntax));
writeln(conf,Form_peach.EditOPcustom1.Caption);
writeln(conf,inttostr(customsyntax1));
writeln(conf,'');
writeln(conf,'[save program''s windows status]');
writeln(conf,inttostr(ws));
if Form_peach.Visible=true then
   if Form_peach.WindowState=wsNormal then
      begin
      ws_status:=0;
      ws_left:=Form_peach.Left;
      ws_top:=Form_peach.Top;
      ws_width:=Form_peach.Width;
      ws_height:=Form_peach.Height;
      end
   else ws_status:=1;
writeln(conf,'');
writeln(conf,'[maximized status]');
writeln(conf,inttostr(ws_status));
writeln(conf,'');
writeln(conf,'[left]');
writeln(conf,inttostr(ws_left));
writeln(conf,'');
writeln(conf,'[top]');
writeln(conf,inttostr(ws_top));
writeln(conf,'');
writeln(conf,'[width]');
writeln(conf,inttostr(ws_width));
writeln(conf,'');
writeln(conf,'[height]');
writeln(conf,inttostr(ws_height));
//favourite archive types
writeln(conf,'');
writeln(conf,'[archive formats to show in favourite popup menu]');
writeln(conf,inttostr(fav7z));
writeln(conf,inttostr(favARC));
writeln(conf,inttostr(favbz));
writeln(conf,inttostr(favcustom));
writeln(conf,inttostr(favgz));
writeln(conf,inttostr(favpea));
writeln(conf,inttostr(favquad));
writeln(conf,inttostr(favsfx));
writeln(conf,inttostr(favsfxarc));
writeln(conf,inttostr(favsplit));
writeln(conf,inttostr(favtar));
writeln(conf,inttostr(favupx));
writeln(conf,inttostr(favzip));
writeln(conf,inttostr(favzpaq));
//Compression default levels
writeln(conf,'');
writeln(conf,'[default 7z compression level]');
writeln(conf,inttostr(level_7z));
writeln(conf,'');
writeln(conf,'[default 7z compression method]');
writeln(conf,method_7z);
writeln(conf,'');
writeln(conf,'[default ARC compression level]');
writeln(conf,inttostr(level_arc));
writeln(conf,'');
writeln(conf,'[default ARC solid mode]');
writeln(conf,inttostr(solid_arc));
writeln(conf,'');
writeln(conf,'[use recovery records in ARC]');
writeln(conf,inttostr(rr_arc));
writeln(conf,'');
writeln(conf,'[encryption algorithm for ARC 0 AES 1 Serpent 2 Twofish 3 Blowfish]');
writeln(conf,inttostr(algo_arc));
writeln(conf,'');
writeln(conf,'[default bzip2 compression level]');
writeln(conf,inttostr(level_bzip2));
writeln(conf,'');
writeln(conf,'[default gz compression level]');
writeln(conf,inttostr(level_gz));
writeln(conf,'');
writeln(conf,'[default zpaq compression level]');
writeln(conf,inttostr(level_paq));
writeln(conf,'');
writeln(conf,'[default quad/balz compression level]');
writeln(conf,inttostr(level_quad));
writeln(conf,'');
writeln(conf,'[default upx compression level]');
writeln(conf,inttostr(level_upx));
writeln(conf,'');
writeln(conf,'[default zip compression level]');
writeln(conf,inttostr(level_zip));
writeln(conf,'');
writeln(conf,'[default zip compression method]');
writeln(conf,method_zip);
writeln(conf,'');
writeln(conf,'[archive files open for writing (from 7z binary)]');
writeln(conf,inttostr(openw_all7z));
writeln(conf,'');
writeln(conf,'[checksum/hash file options: display format, use all algorithms, single algorithms to be used]');
writeln(conf,inttostr(algodisp));
writeln(conf,inttostr(algoall));
writeln(conf,inttostr(algoadler));
writeln(conf,inttostr(algocrc16));
writeln(conf,inttostr(algocrc24));
writeln(conf,inttostr(algocrc32));
writeln(conf,inttostr(algocrc64));
writeln(conf,inttostr(algoed2k));
writeln(conf,inttostr(algomd4));
writeln(conf,inttostr(algomd5));
writeln(conf,inttostr(algoripemd160));
writeln(conf,inttostr(algosha1));
writeln(conf,inttostr(algosha224));
writeln(conf,inttostr(algosha256));
writeln(conf,inttostr(algosha384));
writeln(conf,inttostr(algosha512));
writeln(conf,inttostr(algowhirl));
writeln(conf,'');
writeln(conf,'[erase file options: security level]');
writeln(conf,inttostr(erasepasses));
writeln(conf,'');
writeln(conf,'[desktop path]');
writeln(conf,local_desktop);
writeln(conf,'');
writeln(conf,'[Job definition encoding]');
writeln(conf,inttostr(jobdefenc));
writeln(conf,'');
writeln(conf,'[Archived object''s name encoding]');
writeln(conf,inttostr(archivenameenc));
writeln(conf,'');
writeln(conf,'[Last output path]');
if savehistory=0 then lastoutpath:='';
writeln(conf,lastoutpath);
writeln(conf,'');
writeln(conf,'[Height of the navigation''s splitter]');
writeln(conf,inttostr(splittersize));
writeln(conf,'');
writeln(conf,'[Last used navigation''s bar]');
writeln(conf,inttostr(lastbar));
writeln(conf,'');
writeln(conf,'[Extract supported non-archive types]');
writeln(conf,inttostr(esna));
writeln(conf,'');
writeln(conf,'[Extract unsupported archive types, specifying custom executable]');
writeln(conf,inttostr(euns));
writeln(conf,'');
writeln(conf,'[Configuration check]');
writeln(conf,'PeaZip');
end;

function drawrecenticon(s:utf8string):TBitmap; //some occourrences cannot be in "recent", however it's possible to associate them with an icon
var
   i:integer;
begin
with Form_peach do
begin
if s='' then
   begin
   result:=nil;
   exit;
   end;
{$IFDEF MSWINDOWS}
if s = txt_mypc then
   begin
   result:=Bsystemtools;
   exit;
   end;
if length(s) = 3 then
   begin
   i:=ord(s[1])-64;
   result:=devicon[i];
   exit;
   end;
{$ENDIF}
if s<>'' then if s[length(s)] = directoryseparator then
   begin
   result:=BFolder;
   exit;
   end;
if uextractfileext(s) = '.lnk' then
   begin
   result:=BLink;
   exit;
   end;
if (testinput(s,false)=0) then result:=BUnsupported //unsupported file type
else
   begin
      case testext(s) of
         503: result:=BFileSupported;
         502: result:=BFileSupported;
         501: result:=BFileSupported; //non-archives
         500: result:=BExeSupported; //Windows executables, installers
         else
            result:=BArchiveSupported;
         end;
   end;
end;
end;

procedure populaterecenticons;
begin
with Form_peach do
begin
if mRecent1.visible=true then mRecent1.Bitmap:=drawrecenticon(mRecent1.Caption);
if mRecent2.visible=true then mRecent2.Bitmap:=drawrecenticon(mRecent2.Caption);
if mRecent3.visible=true then mRecent3.Bitmap:=drawrecenticon(mRecent3.Caption);
if mRecent4.visible=true then mRecent4.Bitmap:=drawrecenticon(mRecent4.Caption);
if mRecent5.visible=true then mRecent5.Bitmap:=drawrecenticon(mRecent5.Caption);
if mRecent6.visible=true then mRecent6.Bitmap:=drawrecenticon(mRecent6.Caption);
if mRecent7.visible=true then mRecent7.Bitmap:=drawrecenticon(mRecent7.Caption);
if mRecent8.visible=true then mRecent8.Bitmap:=drawrecenticon(mRecent8.Caption);
if poRecent1.visible=true then poRecent1.Bitmap:=drawrecenticon(poRecent1.Caption);
if poRecent2.visible=true then poRecent2.Bitmap:=drawrecenticon(poRecent2.Caption);
if poRecent3.visible=true then poRecent3.Bitmap:=drawrecenticon(poRecent3.Caption);
if poRecent4.visible=true then poRecent4.Bitmap:=drawrecenticon(poRecent4.Caption);
if poRecent5.visible=true then poRecent5.Bitmap:=drawrecenticon(poRecent5.Caption);
if poRecent6.visible=true then poRecent6.Bitmap:=drawrecenticon(poRecent6.Caption);
if poRecent7.visible=true then poRecent7.Bitmap:=drawrecenticon(poRecent7.Caption);
if poRecent8.visible=true then poRecent8.Bitmap:=drawrecenticon(poRecent8.Caption);
if pmr1.visible=true then pmr1.Bitmap:=drawrecenticon(pmr1.Caption);
if pmr2.visible=true then pmr2.Bitmap:=drawrecenticon(pmr2.Caption);
if pmr3.visible=true then pmr3.Bitmap:=drawrecenticon(pmr3.Caption);
if pmr4.visible=true then pmr4.Bitmap:=drawrecenticon(pmr4.Caption);
if pmr5.visible=true then pmr5.Bitmap:=drawrecenticon(pmr5.Caption);
if pmr6.visible=true then pmr6.Bitmap:=drawrecenticon(pmr6.Caption);
if pmr7.visible=true then pmr7.Bitmap:=drawrecenticon(pmr7.Caption);
if pmr8.visible=true then pmr8.Bitmap:=drawrecenticon(pmr8.Caption);
if pnr1.visible=true then pnr1.Bitmap:=drawrecenticon(pnr1.Caption);
if pnr2.visible=true then pnr2.Bitmap:=drawrecenticon(pnr2.Caption);
if pnr3.visible=true then pnr3.Bitmap:=drawrecenticon(pnr3.Caption);
if pnr4.visible=true then pnr4.Bitmap:=drawrecenticon(pnr4.Caption);
if pnr5.visible=true then pnr5.Bitmap:=drawrecenticon(pnr5.Caption);
if pnr6.visible=true then pnr6.Bitmap:=drawrecenticon(pnr6.Caption);
if pnr7.visible=true then pnr7.Bitmap:=drawrecenticon(pnr7.Caption);
if pnr8.visible=true then pnr8.Bitmap:=drawrecenticon(pnr8.Caption);
if pnra1.visible=true then pnra1.Bitmap:=drawrecenticon(pnra1.Caption);
if pnra2.visible=true then pnra2.Bitmap:=drawrecenticon(pnra2.Caption);
if pnra3.visible=true then pnra3.Bitmap:=drawrecenticon(pnra3.Caption);
if pnra4.visible=true then pnra4.Bitmap:=drawrecenticon(pnra4.Caption);
if pnra5.visible=true then pnra5.Bitmap:=drawrecenticon(pnra5.Caption);
if pnra6.visible=true then pnra6.Bitmap:=drawrecenticon(pnra6.Caption);
if pnra7.visible=true then pnra7.Bitmap:=drawrecenticon(pnra7.Caption);
if pnra8.visible=true then pnra8.Bitmap:=drawrecenticon(pnra8.Caption);
end;
end;

procedure addtoarchivehistory(s:utf8string);
begin
if savehistory=0 then exit;
if s<>Form_peach.mRecent1.Caption then
   if s<>Form_peach.mRecent2.Caption then
      if s<>Form_peach.mRecent3.Caption then
         if s<>Form_peach.mRecent4.Caption then
            if s<>Form_peach.mRecent5.Caption then
               if s<>Form_peach.mRecent6.Caption then
                  if s<>Form_peach.mRecent7.Caption then
                     begin
                     Form_peach.mRecent8.Caption:=Form_peach.mRecent7.Caption;
                     Form_peach.mRecent7.Caption:=Form_peach.mRecent6.Caption;
                     Form_peach.mRecent6.Caption:=Form_peach.mRecent5.Caption;
                     Form_peach.mRecent5.Caption:=Form_peach.mRecent4.Caption;
                     Form_peach.mRecent4.Caption:=Form_peach.mRecent3.Caption;
                     Form_peach.mRecent3.Caption:=Form_peach.mRecent2.Caption;
                     Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
                     Form_peach.mRecent1.Caption:=s;
                     end
                  else
                     begin
                     Form_peach.mRecent7.Caption:=Form_peach.mRecent6.Caption;
                     Form_peach.mRecent6.Caption:=Form_peach.mRecent5.Caption;
                     Form_peach.mRecent5.Caption:=Form_peach.mRecent4.Caption;
                     Form_peach.mRecent4.Caption:=Form_peach.mRecent3.Caption;
                     Form_peach.mRecent3.Caption:=Form_peach.mRecent2.Caption;
                     Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
                     Form_peach.mRecent1.Caption:=s;
                     end
               else
                  begin
                  Form_peach.mRecent6.Caption:=Form_peach.mRecent5.Caption;
                  Form_peach.mRecent5.Caption:=Form_peach.mRecent4.Caption;
                  Form_peach.mRecent4.Caption:=Form_peach.mRecent3.Caption;
                  Form_peach.mRecent3.Caption:=Form_peach.mRecent2.Caption;
                  Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
                  Form_peach.mRecent1.Caption:=s;
                  end
            else
               begin
               Form_peach.mRecent5.Caption:=Form_peach.mRecent4.Caption;
               Form_peach.mRecent4.Caption:=Form_peach.mRecent3.Caption;
               Form_peach.mRecent3.Caption:=Form_peach.mRecent2.Caption;
               Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
               Form_peach.mRecent1.Caption:=s;
               end
         else
            begin
            Form_peach.mRecent4.Caption:=Form_peach.mRecent3.Caption;
            Form_peach.mRecent3.Caption:=Form_peach.mRecent2.Caption;
            Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
            Form_peach.mRecent1.Caption:=s;
            end
      else
         begin
         Form_peach.mRecent3.Caption:=Form_peach.mRecent2.Caption;
         Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
         Form_peach.mRecent1.Caption:=s;
         end
   else
      begin
      Form_peach.mRecent2.Caption:=Form_peach.mRecent1.Caption;
      Form_peach.mRecent1.Caption:=s;
      end;
if Form_peach.mRecent1.Caption<>'' then Form_peach.mRecent1.Visible:=true else Form_peach.mRecent1.Visible:=false;
if Form_peach.mRecent2.Caption<>'' then Form_peach.mRecent2.Visible:=true else Form_peach.mRecent2.Visible:=false;
if Form_peach.mRecent3.Caption<>'' then Form_peach.mRecent3.Visible:=true else Form_peach.mRecent3.Visible:=false;
if Form_peach.mRecent4.Caption<>'' then Form_peach.mRecent4.Visible:=true else Form_peach.mRecent4.Visible:=false;
if Form_peach.mRecent5.Caption<>'' then Form_peach.mRecent5.Visible:=true else Form_peach.mRecent5.Visible:=false;
if Form_peach.mRecent6.Caption<>'' then Form_peach.mRecent6.Visible:=true else Form_peach.mRecent6.Visible:=false;
if Form_peach.mRecent7.Caption<>'' then Form_peach.mRecent7.Visible:=true else Form_peach.mRecent7.Visible:=false;
if Form_peach.mRecent8.Caption<>'' then Form_peach.mRecent8.Visible:=true else Form_peach.mRecent8.Visible:=false;
Form_peach.poRecent1.Caption:=Form_peach.mRecent1.Caption;
Form_peach.poRecent2.Caption:=Form_peach.mRecent2.Caption;
Form_peach.poRecent3.Caption:=Form_peach.mRecent3.Caption;
Form_peach.poRecent4.Caption:=Form_peach.mRecent4.Caption;
Form_peach.poRecent5.Caption:=Form_peach.mRecent5.Caption;
Form_peach.poRecent6.Caption:=Form_peach.mRecent6.Caption;
Form_peach.poRecent7.Caption:=Form_peach.mRecent7.Caption;
Form_peach.poRecent8.Caption:=Form_peach.mRecent8.Caption;
Form_peach.poRecent1.Visible:=Form_peach.mRecent1.Visible;
Form_peach.poRecent2.Visible:=Form_peach.mRecent2.Visible;
Form_peach.poRecent3.Visible:=Form_peach.mRecent3.Visible;
Form_peach.poRecent4.Visible:=Form_peach.mRecent4.Visible;
Form_peach.poRecent5.Visible:=Form_peach.mRecent5.Visible;
Form_peach.poRecent6.Visible:=Form_peach.mRecent6.Visible;
Form_peach.poRecent7.Visible:=Form_peach.mRecent7.Visible;
Form_peach.poRecent8.Visible:=Form_peach.mRecent8.Visible;
Form_peach.pmr1.Caption:=Form_peach.mRecent1.Caption;
Form_peach.pmr2.Caption:=Form_peach.mRecent2.Caption;
Form_peach.pmr3.Caption:=Form_peach.mRecent3.Caption;
Form_peach.pmr4.Caption:=Form_peach.mRecent4.Caption;
Form_peach.pmr5.Caption:=Form_peach.mRecent5.Caption;
Form_peach.pmr6.Caption:=Form_peach.mRecent6.Caption;
Form_peach.pmr7.Caption:=Form_peach.mRecent7.Caption;
Form_peach.pmr8.Caption:=Form_peach.mRecent8.Caption;
Form_peach.pmr1.Visible:=Form_peach.mRecent1.Visible;
Form_peach.pmr2.Visible:=Form_peach.mRecent2.Visible;
Form_peach.pmr3.Visible:=Form_peach.mRecent3.Visible;
Form_peach.pmr4.Visible:=Form_peach.mRecent4.Visible;
Form_peach.pmr5.Visible:=Form_peach.mRecent5.Visible;
Form_peach.pmr6.Visible:=Form_peach.mRecent6.Visible;
Form_peach.pmr7.Visible:=Form_peach.mRecent7.Visible;
Form_peach.pmr8.Visible:=Form_peach.mRecent8.Visible;
Form_peach.pnr1.Caption:=uextractfilepath(Form_peach.mRecent1.Caption);
Form_peach.pnr2.Caption:=uextractfilepath(Form_peach.mRecent2.Caption);
Form_peach.pnr3.Caption:=uextractfilepath(Form_peach.mRecent3.Caption);
Form_peach.pnr4.Caption:=uextractfilepath(Form_peach.mRecent4.Caption);
Form_peach.pnr5.Caption:=uextractfilepath(Form_peach.mRecent5.Caption);
Form_peach.pnr6.Caption:=uextractfilepath(Form_peach.mRecent6.Caption);
Form_peach.pnr7.Caption:=uextractfilepath(Form_peach.mRecent7.Caption);
Form_peach.pnr8.Caption:=uextractfilepath(Form_peach.mRecent8.Caption);
Form_peach.pnra1.Caption:=uextractfilepath(Form_peach.mRecent1.Caption);
Form_peach.pnra2.Caption:=uextractfilepath(Form_peach.mRecent2.Caption);
Form_peach.pnra3.Caption:=uextractfilepath(Form_peach.mRecent3.Caption);
Form_peach.pnra4.Caption:=uextractfilepath(Form_peach.mRecent4.Caption);
Form_peach.pnra5.Caption:=uextractfilepath(Form_peach.mRecent5.Caption);
Form_peach.pnra6.Caption:=uextractfilepath(Form_peach.mRecent6.Caption);
Form_peach.pnra7.Caption:=uextractfilepath(Form_peach.mRecent7.Caption);
Form_peach.pnra8.Caption:=uextractfilepath(Form_peach.mRecent8.Caption);
Form_peach.pnr1.Visible:=Form_peach.mRecent1.Visible;
Form_peach.pnr2.Visible:=Form_peach.mRecent2.Visible;
Form_peach.pnr3.Visible:=Form_peach.mRecent3.Visible;
Form_peach.pnr4.Visible:=Form_peach.mRecent4.Visible;
Form_peach.pnr5.Visible:=Form_peach.mRecent5.Visible;
Form_peach.pnr6.Visible:=Form_peach.mRecent6.Visible;
Form_peach.pnr7.Visible:=Form_peach.mRecent7.Visible;
Form_peach.pnr8.Visible:=Form_peach.mRecent8.Visible;
Form_peach.pnra1.Visible:=Form_peach.mRecent1.Visible;
Form_peach.pnra2.Visible:=Form_peach.mRecent2.Visible;
Form_peach.pnra3.Visible:=Form_peach.mRecent3.Visible;
Form_peach.pnra4.Visible:=Form_peach.mRecent4.Visible;
Form_peach.pnra5.Visible:=Form_peach.mRecent5.Visible;
Form_peach.pnra6.Visible:=Form_peach.mRecent6.Visible;
Form_peach.pnra7.Visible:=Form_peach.mRecent7.Visible;
Form_peach.pnra8.Visible:=Form_peach.mRecent8.Visible;
populaterecenticons;
end;

procedure addtocustomhistory(s:utf8string);
begin
if s<>Form_peach.pcustom1.Caption then
   if s<>Form_peach.pcustom2.Caption then
      if s<>Form_peach.pcustom3.Caption then
         if s<>Form_peach.pcustom4.Caption then
            if s<>Form_peach.pcustom5.Caption then
               if s<>Form_peach.pcustom6.Caption then
                  if s<>Form_peach.pcustom7.Caption then
                     begin
                     Form_peach.pcustom8.Caption:=Form_peach.pcustom7.Caption;
                     Form_peach.pcustom7.Caption:=Form_peach.pcustom6.Caption;
                     Form_peach.pcustom6.Caption:=Form_peach.pcustom5.Caption;
                     Form_peach.pcustom5.Caption:=Form_peach.pcustom4.Caption;
                     Form_peach.pcustom4.Caption:=Form_peach.pcustom3.Caption;
                     Form_peach.pcustom3.Caption:=Form_peach.pcustom2.Caption;
                     Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
                     Form_peach.pcustom1.Caption:=s;
                     end
                  else
                     begin
                     Form_peach.pcustom7.Caption:=Form_peach.pcustom6.Caption;
                     Form_peach.pcustom6.Caption:=Form_peach.pcustom5.Caption;
                     Form_peach.pcustom5.Caption:=Form_peach.pcustom4.Caption;
                     Form_peach.pcustom4.Caption:=Form_peach.pcustom3.Caption;
                     Form_peach.pcustom3.Caption:=Form_peach.pcustom2.Caption;
                     Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
                     Form_peach.pcustom1.Caption:=s;
                     end
               else
                  begin
                  Form_peach.pcustom6.Caption:=Form_peach.pcustom5.Caption;
                  Form_peach.pcustom5.Caption:=Form_peach.pcustom4.Caption;
                  Form_peach.pcustom4.Caption:=Form_peach.pcustom3.Caption;
                  Form_peach.pcustom3.Caption:=Form_peach.pcustom2.Caption;
                  Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
                  Form_peach.pcustom1.Caption:=s;
                  end
            else
               begin
               Form_peach.pcustom5.Caption:=Form_peach.pcustom4.Caption;
               Form_peach.pcustom4.Caption:=Form_peach.pcustom3.Caption;
               Form_peach.pcustom3.Caption:=Form_peach.pcustom2.Caption;
               Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
               Form_peach.pcustom1.Caption:=s;
               end
         else
            begin
            Form_peach.pcustom4.Caption:=Form_peach.pcustom3.Caption;
            Form_peach.pcustom3.Caption:=Form_peach.pcustom2.Caption;
            Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
            Form_peach.pcustom1.Caption:=s;
            end
      else
         begin
         Form_peach.pcustom3.Caption:=Form_peach.pcustom2.Caption;
         Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
         Form_peach.pcustom1.Caption:=s;
         end
   else
      begin
      Form_peach.pcustom2.Caption:=Form_peach.pcustom1.Caption;
      Form_peach.pcustom1.Caption:=s;
      end;
if Form_peach.pcustom1.Caption<>'' then Form_peach.pcustom1.Visible:=true else Form_peach.pcustom1.Visible:=false;
if Form_peach.pcustom2.Caption<>'' then Form_peach.pcustom2.Visible:=true else Form_peach.pcustom2.Visible:=false;
if Form_peach.pcustom3.Caption<>'' then Form_peach.pcustom3.Visible:=true else Form_peach.pcustom3.Visible:=false;
if Form_peach.pcustom4.Caption<>'' then Form_peach.pcustom4.Visible:=true else Form_peach.pcustom4.Visible:=false;
if Form_peach.pcustom5.Caption<>'' then Form_peach.pcustom5.Visible:=true else Form_peach.pcustom5.Visible:=false;
if Form_peach.pcustom6.Caption<>'' then Form_peach.pcustom6.Visible:=true else Form_peach.pcustom6.Visible:=false;
if Form_peach.pcustom7.Caption<>'' then Form_peach.pcustom7.Visible:=true else Form_peach.pcustom7.Visible:=false;
if Form_peach.pcustom8.Caption<>'' then Form_peach.pcustom8.Visible:=true else Form_peach.pcustom8.Visible:=false;
end;

procedure updateconf;
begin
Form_peach.RadioGroup0.ItemIndex:=df;
Form_peach.RadioGroup4.ItemIndex:=zaout;
Form_peach.EditDesk.Text:=local_desktop;
Form_peach.RadioGroup6.ItemIndex:=autoclosegwrap;
Form_peach.RadioGroupClip.ItemIndex:=clipmode;
case browserstyle of
   0: Form_peach.RadioGroupBrowser.ItemIndex:=0;
   1: Form_peach.RadioGroupBrowser.ItemIndex:=1;
   else Form_peach.RadioGroupBrowser.ItemIndex:=2;
   end;
if autoopen=1 then Form_peach.CheckBoxAutoOpen.State:=cbChecked else Form_peach.CheckBoxAutoOpen.State:=cbUnChecked;
if showguihints=1 then Form_peach.CheckBoxShowHints.State:=cbChecked else Form_peach.CheckBoxShowHints.State:=cbUnChecked;
if showguihints=1 then Form_peach.ShowHint:=true else Form_peach.ShowHint:=false;
Form_peach.RadioGrouppwrequest.ItemIndex:=extpw;
Form_peach.ComboBoxOverwrite.Itemindex:=extopt7z;
Form_peach.ComboBoxOverwriteARC.Itemindex:=extoptarc;
if ignorepathextand=1 then Form_peach.CheckBox4.State:=cbChecked else Form_peach.CheckBox4.State:=cbUnChecked;
if ignorepathdisp=1 then Form_peach.CheckBoxIgnoreDisp.State:=cbChecked else Form_peach.CheckBoxIgnoreDisp.State:=cbUnChecked;
if ignorepathsel=1 then Form_peach.CheckBoxIgnoreSel.State:=cbChecked else Form_peach.CheckBoxIgnoreSel.State:=cbUnChecked;
if prebrowse=1 then Form_peach.CheckBoxPrebrowse.State:=cbChecked else Form_peach.CheckBoxPrebrowse.State:=cbUnChecked;
Form_peach.SpinEditPrebrowse.Value:=prebrowsesize;
if savehistory=1 then Form_peach.CheckBoxArchiveHistory.State:=cbChecked else Form_peach.CheckBoxArchiveHistory.State:=cbUnChecked;
if jobdefenc=1 then Form_peach.CheckBoxJobEncoding.State:=cbChecked else Form_peach.CheckBoxJobEncoding.State:=cbUnChecked;
if archivenameenc=1 then Form_peach.CheckBoxEncoding.State:=cbChecked else Form_peach.CheckBoxEncoding.State:=cbUnChecked;
if ws=1 then Form_peach.CheckBoxWindowStatus.State:=cbChecked else Form_peach.CheckBoxWindowStatus.State:=cbUnChecked;
if fav7z=1 then
   begin
   Form_peach.cbf1.State:=cbChecked;
   Form_peach.mType7z.Visible:=true;
   end
else
   begin
   Form_peach.cbf1.State:=cbUnChecked;
   Form_peach.mType7z.Visible:=false;
   end;
if favARC=1 then
   begin
   Form_peach.cbfARC.State:=cbChecked;
   Form_peach.mTypeARC.Visible:=true;
   end
else
   begin
   Form_peach.cbfARC.State:=cbUnChecked;
   Form_peach.mTypeARC.Visible:=false;
   end;
if favcustom=1 then
   begin
   Form_peach.cbfCustom.State:=cbChecked;
   Form_peach.mTypeCustom.Visible:=true;
   end
else
   begin
   Form_peach.cbfCustom.State:=cbUnChecked;
   Form_peach.mTypeCustom.Visible:=false;
   end;
if favsfx=1 then
   begin
   Form_peach.cbfsfx.State:=cbChecked;
   Form_peach.mTypesfx.Visible:=true;
   end
else
   begin
   Form_peach.cbfsfx.State:=cbUnChecked;
   Form_peach.mTypesfx.Visible:=false;
   end;
if favsfxarc=1 then
   begin
   Form_peach.cbfsfx1.State:=cbChecked;
   Form_peach.mTypesfxarc.Visible:=true;
   end
else
   begin
   Form_peach.cbfsfx1.State:=cbUnChecked;
   Form_peach.mTypesfxarc.Visible:=false;
   end;
if favbz=1 then
   begin
   Form_peach.cbf2.State:=cbChecked;
   Form_peach.mTypeBZip2.Visible:=true;
   end
else
   begin
   Form_peach.cbf2.State:=cbUnChecked;
   Form_peach.mTypeBZip2.Visible:=false;
   end;
if favgz=1 then
   begin
   Form_peach.cbf3.State:=cbChecked;
   Form_peach.mTypeGZ.Visible:=true;
   end
else
   begin
   Form_peach.cbf3.State:=cbUnChecked;
   Form_peach.mTypeGZ.Visible:=false;
   end;
if favzpaq=1 then
   begin
   Form_peach.cbf5.State:=cbChecked;
   Form_peach.mTypePAQ.Visible:=true;
   end
else
   begin
   Form_peach.cbf5.State:=cbUnChecked;
   Form_peach.mTypePAQ.Visible:=false;
   end;
if favpea=1 then
   begin
   Form_peach.cbf6.State:=cbChecked;
   Form_peach.mTypePEA.Visible:=true;
   end
else
   begin
   Form_peach.cbf6.State:=cbUnChecked;
   Form_peach.mTypePEA.Visible:=false;
   end;
if favquad=1 then
   begin
   Form_peach.cbf7.State:=cbChecked;
   Form_peach.mTypeQUAD.Visible:=true;
   end
else
   begin
   Form_peach.cbf7.State:=cbUnChecked;
   Form_peach.mTypeQUAD.Visible:=false;
   end;
if favsplit=1 then
   begin
   Form_peach.cbf8.State:=cbChecked;
   Form_peach.mTypeSplit.Visible:=true;
   end
else
   begin
   Form_peach.cbf8.State:=cbUnChecked;
   Form_peach.mTypeSplit.Visible:=false;
   end;
if favtar=1 then
   begin
   Form_peach.cbf9.State:=cbChecked;
   Form_peach.mTypeTAR.Visible:=true;
   end
else
   begin
   Form_peach.cbf9.State:=cbUnChecked;
   Form_peach.mTypeTAR.Visible:=false;
   end;
if favupx=1 then
   begin
   Form_peach.cbf10.State:=cbChecked;
   Form_peach.mTypeUPX.Visible:=true;
   end
else
   begin
   Form_peach.cbf10.State:=cbUnChecked;
   Form_peach.mTypeUPX.Visible:=false;
   end;
if favzip=1 then
   begin
   Form_peach.cbf11.State:=cbChecked;
   Form_peach.mTypeZIP.Visible:=true;
   end
else
   begin
   Form_peach.cbf11.State:=cbUnChecked;
   Form_peach.mTypeZIP.Visible:=false;
   end;
if savehistory=1 then
   begin
   Form_peach.mRecent.Enabled:=true;
   Form_peach.smRecent.Enabled:=true;
   if Form_peach.mRecent1.Caption<>'' then Form_peach.mRecent1.Visible:=true else Form_peach.mRecent1.Visible:=false;
   if Form_peach.mRecent2.Caption<>'' then Form_peach.mRecent2.Visible:=true else Form_peach.mRecent2.Visible:=false;
   if Form_peach.mRecent3.Caption<>'' then Form_peach.mRecent3.Visible:=true else Form_peach.mRecent3.Visible:=false;
   if Form_peach.mRecent4.Caption<>'' then Form_peach.mRecent4.Visible:=true else Form_peach.mRecent4.Visible:=false;
   if Form_peach.mRecent5.Caption<>'' then Form_peach.mRecent5.Visible:=true else Form_peach.mRecent5.Visible:=false;
   if Form_peach.mRecent6.Caption<>'' then Form_peach.mRecent6.Visible:=true else Form_peach.mRecent6.Visible:=false;
   if Form_peach.mRecent7.Caption<>'' then Form_peach.mRecent7.Visible:=true else Form_peach.mRecent7.Visible:=false;
   if Form_peach.mRecent8.Caption<>'' then Form_peach.mRecent8.Visible:=true else Form_peach.mRecent8.Visible:=false;
   Form_peach.poRecent1.Visible:=Form_peach.mRecent1.Visible;
   Form_peach.poRecent2.Visible:=Form_peach.mRecent2.Visible;
   Form_peach.poRecent3.Visible:=Form_peach.mRecent3.Visible;
   Form_peach.poRecent4.Visible:=Form_peach.mRecent4.Visible;
   Form_peach.poRecent5.Visible:=Form_peach.mRecent5.Visible;
   Form_peach.poRecent6.Visible:=Form_peach.mRecent6.Visible;
   Form_peach.poRecent7.Visible:=Form_peach.mRecent7.Visible;
   Form_peach.poRecent8.Visible:=Form_peach.mRecent8.Visible;
   Form_peach.pmr1.Visible:=Form_peach.mRecent1.Visible;
   Form_peach.pmr2.Visible:=Form_peach.mRecent2.Visible;
   Form_peach.pmr3.Visible:=Form_peach.mRecent3.Visible;
   Form_peach.pmr4.Visible:=Form_peach.mRecent4.Visible;
   Form_peach.pmr5.Visible:=Form_peach.mRecent5.Visible;
   Form_peach.pmr6.Visible:=Form_peach.mRecent6.Visible;
   Form_peach.pmr7.Visible:=Form_peach.mRecent7.Visible;
   Form_peach.pmr8.Visible:=Form_peach.mRecent8.Visible;
   Form_peach.pnr1.Visible:=Form_peach.mRecent1.Visible;
   Form_peach.pnr2.Visible:=Form_peach.mRecent2.Visible;
   Form_peach.pnr3.Visible:=Form_peach.mRecent3.Visible;
   Form_peach.pnr4.Visible:=Form_peach.mRecent4.Visible;
   Form_peach.pnr5.Visible:=Form_peach.mRecent5.Visible;
   Form_peach.pnr6.Visible:=Form_peach.mRecent6.Visible;
   Form_peach.pnr7.Visible:=Form_peach.mRecent7.Visible;
   Form_peach.pnr8.Visible:=Form_peach.mRecent8.Visible;
   Form_peach.pnra1.Visible:=Form_peach.mRecent1.Visible;
   Form_peach.pnra2.Visible:=Form_peach.mRecent2.Visible;
   Form_peach.pnra3.Visible:=Form_peach.mRecent3.Visible;
   Form_peach.pnra4.Visible:=Form_peach.mRecent4.Visible;
   Form_peach.pnra5.Visible:=Form_peach.mRecent5.Visible;
   Form_peach.pnra6.Visible:=Form_peach.mRecent6.Visible;
   Form_peach.pnra7.Visible:=Form_peach.mRecent7.Visible;
   Form_peach.pnra8.Visible:=Form_peach.mRecent8.Visible;
   end
else
   begin
   Form_peach.mRecent.Enabled:=false;
   Form_peach.smRecent.Enabled:=false;
   Form_peach.mRecent1.Caption:='';
   Form_peach.mRecent2.Caption:='';
   Form_peach.mRecent3.Caption:='';
   Form_peach.mRecent4.Caption:='';
   Form_peach.mRecent5.Caption:='';
   Form_peach.mRecent6.Caption:='';
   Form_peach.mRecent7.Caption:='';
   Form_peach.mRecent8.Caption:='';
   end;
if Form_peach.pcustom1.Caption<>'' then Form_peach.pcustom1.Visible:=true else Form_peach.pcustom1.Visible:=false;
if Form_peach.pcustom2.Caption<>'' then Form_peach.pcustom2.Visible:=true else Form_peach.pcustom2.Visible:=false;
if Form_peach.pcustom3.Caption<>'' then Form_peach.pcustom3.Visible:=true else Form_peach.pcustom3.Visible:=false;
if Form_peach.pcustom4.Caption<>'' then Form_peach.pcustom4.Visible:=true else Form_peach.pcustom4.Visible:=false;
if Form_peach.pcustom5.Caption<>'' then Form_peach.pcustom5.Visible:=true else Form_peach.pcustom5.Visible:=false;
if Form_peach.pcustom6.Caption<>'' then Form_peach.pcustom6.Visible:=true else Form_peach.pcustom6.Visible:=false;
if Form_peach.pcustom7.Caption<>'' then Form_peach.pcustom7.Visible:=true else Form_peach.pcustom7.Visible:=false;
if Form_peach.pcustom8.Caption<>'' then Form_peach.pcustom8.Visible:=true else Form_peach.pcustom8.Visible:=false;
Form_peach.ComboBoxArchiveCustom.Itemindex:=customsyntax;
Form_peach.ComboBoxArchiveCustom1.Itemindex:=customsyntax1;
Form_peach.RadioGroup1.ItemIndex:=algodisp;
if algoall=1 then
   begin
   Form_peach.cbcheckall.State:=cbChecked;
   Form_peach.groupboxcheck.visible:=false;
   end
else
   begin
   Form_peach.cbcheckall.State:=cbUnChecked;
   Form_peach.groupboxcheck.visible:=true;
   end;
if algoadler=1 then Form_peach.cbadler.State:=cbChecked else Form_peach.cbadler.State:=cbUnChecked;
if algocrc16=1 then Form_peach.cbcrc16.State:=cbChecked else Form_peach.cbcrc16.State:=cbUnChecked;
if algocrc24=1 then Form_peach.cbcrc24.State:=cbChecked else Form_peach.cbcrc24.State:=cbUnChecked;
if algocrc32=1 then Form_peach.cbcrc32.State:=cbChecked else Form_peach.cbcrc32.State:=cbUnChecked;
if algocrc64=1 then Form_peach.cbcrc64.State:=cbChecked else Form_peach.cbcrc64.State:=cbUnChecked;
if algoed2k=1 then Form_peach.cbed2k.State:=cbChecked else Form_peach.cbed2k.State:=cbUnChecked;
if algomd4=1 then Form_peach.cbmd4.State:=cbChecked else Form_peach.cbmd4.State:=cbUnChecked;
if algomd5=1 then Form_peach.cbmd5.State:=cbChecked else Form_peach.cbmd5.State:=cbUnChecked;
if algoripemd160=1 then Form_peach.cbrmd.State:=cbChecked else Form_peach.cbrmd.State:=cbUnChecked;
if algosha1=1 then Form_peach.cbsha1.State:=cbChecked else Form_peach.cbsha1.State:=cbUnChecked;
if algosha224=1 then Form_peach.cbsha224.State:=cbChecked else Form_peach.cbsha224.State:=cbUnChecked;
if algosha256=1 then Form_peach.cbsha256.State:=cbChecked else Form_peach.cbsha256.State:=cbUnChecked;
if algosha384=1 then Form_peach.cbsha384.State:=cbChecked else Form_peach.cbsha384.State:=cbUnChecked;
if algosha512=1 then Form_peach.cbsha512.State:=cbChecked else Form_peach.cbsha512.State:=cbUnChecked;
if algowhirl=1 then Form_peach.cbwhirlpool.State:=cbChecked else Form_peach.cbwhirlpool.State:=cbUnChecked;
if esna=1 then Form_peach.checkbox1.State:=cbChecked else Form_peach.checkbox1.State:=cbUnChecked;
if euns=1 then
   begin
   Form_peach.checkbox2.State:=cbChecked;
   Form_peach.CompressionCustom1.Enabled:=true;
   end
else
   begin
   Form_peach.checkbox2.State:=cbUnChecked;
   Form_peach.CompressionCustom1.Enabled:=false;
   end;
Form_peach.ComboBoxPasses.Itemindex:=erasepasses;
end;

procedure updateconf_modified;
begin
updateconf;
end;

procedure load_theme;
begin
if theme_path='' then exit;
uassigntext(conf,(executable_path+theme_path));
filemode:=0;
reset(conf);
read_header(conf);
readln(conf,theme_name);
readln(conf,theme_author);
readln(conf,theme_license);
readconf_default_colors;
CloseFile(conf);
Form_peach.theme_label.Caption:='['+theme_name+']';
if theme_name='Tango' then Form_peach.ComboBoxTheme.ItemIndex:=3
else
   if theme_name='NuoveXT' then Form_peach.ComboBoxTheme.ItemIndex:=2
   else
      if theme_name='FireCrystal' then Form_peach.ComboBoxTheme.ItemIndex:=1
      else
         Form_peach.ComboBoxTheme.ItemIndex:=0;
end;

procedure load_default_theme;
begin
theme_path:='res'+directoryseparator+'themes'+directoryseparator+DEFAULT_THEME+directoryseparator+'theme.txt';
load_theme;
end;

procedure save_bookmarks;
var
   i:integer;
begin
try
uassigntext(book,(confpath+'bookmarks.txt'));
rewrite(book);
write_header(book);
if Form_peach.StringGridBookmarks.RowCount>1 then
   for i:=1 to Form_peach.StringGridBookmarks.RowCount-1 do
      begin
      writeln(book,Form_peach.StringGridBookmarks.Cells[1,i]);
      writeln(book,Form_peach.StringGridBookmarks.Cells[2,i]);
      writeln(book,Form_peach.StringGridBookmarks.Cells[3,i]);
      writeln(book,Form_peach.StringGridBookmarks.Cells[4,i]);
      writeln(book,Form_peach.StringGridBookmarks.Cells[5,i]);
      writeln(book,'');
      writeln(book,'');
      writeln(book,'');//reserved for future expansion
      end;
CloseFile(book);
except
   try
   CloseFile(book);
   except
   end;
end;
end;

procedure save_custedit;
begin
try
uassigntext(cedit,(confpath+'custedit.txt'));
rewrite(cedit);
write_header(cedit);
writeln(cedit,'[custom editors/players, ovveride system''s file associations]');
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,1]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,2]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,3]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,4]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,5]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,6]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,7]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[1,8]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,1]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,2]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,3]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,4]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,5]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,6]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,7]);
writeln(cedit,Form_peach.StringGridCustEdit.Cells[2,8]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,1]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,2]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,3]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,4]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,5]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,6]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,7]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[1,8]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,1]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,2]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,3]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,4]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,5]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,6]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,7]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[2,8]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,1]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,2]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,3]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,4]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,5]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,6]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,7]);
writeln(cedit,Form_peach.StringGridCustEdit1.Cells[3,8]);
CloseFile(cedit);
except
   try
   CloseFile(cedit);
   except
   end;
end;
end;

procedure set_win_antivirus;
begin
{$IFDEF MSWINDOWS}
if ufileexists(winpfolder+'AVG\AVG8\avgscanx.exe') then
   begin
   Form_peach.StringGridCustEdit1.Cells[1,1]:='AVG Anti-Virus Free';
   advedit1before:=wincomspec+' /k ""'+winpfolder+'AVG\AVG8\avgscanx.exe" /SCAN=';
   advedit1after:='"';
   end;
if ufileexists(winpfolder+'Spybot - Search & Destroy\SDFiles.exe') then
   begin
   Form_peach.StringGridCustEdit1.Cells[1,2]:='Spybot - Search & Destroy';
   advedit2before:='"'+winpfolder+'Spybot - Search & Destroy\SDFiles.exe" ';
   advedit2after:='';
   end;
if ufileexists(winpfolder+'Avira\AntiVir PersonalEdition Classic\avscan.exe') then
   begin
   Form_peach.StringGridCustEdit1.Cells[1,3]:='Avira AntiVir Personal';
   advedit3before:='"'+winpfolder+'Avira\AntiVir PersonalEdition Classic\avscan.exe" /GUIMODE=1 /PATH=';
   advedit3after:='';
   end;
if ufileexists(winpfolder+'Malwarebytes'' Anti-Malware\mbam.exe') then
   begin
   Form_peach.StringGridCustEdit1.Cells[1,4]:='Malwarebyte''s Anti-Malware';
   advedit4before:='"'+winpfolder+'Malwarebytes'' Anti-Malware\mbam.exe" ';
   advedit4after:='';
   end;
if ufileexists(winpfolder+'VirusTotalUploader\VirusTotalUpload.exe') then
   begin
   Form_peach.StringGridCustEdit1.Cells[1,5]:='VirusTotal';
   advedit5before:='"'+winpfolder+'VirusTotalUploader\VirusTotalUpload.exe" ';
   advedit5after:='';
   end;
{$ENDIF}
end;

procedure reset_custedit;
var
   i:integer;
begin
for i:=1 to 8 do
   begin
   Form_peach.StringGridCustEdit.Cells[1,i]:='';
   Form_peach.StringGridCustEdit.Cells[2,i]:='';
   Form_peach.StringGridCustEdit1.Cells[1,i]:='';
   Form_peach.StringGridCustEdit1.Cells[2,i]:='';
   Form_peach.StringGridCustEdit1.Cells[3,i]:='';
   end;
custedit1:='';
advedit1before:='';
advedit1after:='';
custedit2:='';
advedit2before:='';
advedit2after:='';
custedit3:='';
advedit3before:='';
advedit3after:='';
custedit4:='';
advedit4before:='';
advedit4after:='';
custedit5:='';
advedit5before:='';
advedit5after:='';
custedit6:='';
advedit6before:='';
advedit6after:='';
custedit7:='';
advedit7before:='';
advedit7after:='';
custedit8:='';
advedit8before:='';
advedit8after:='';
{$IFDEF MSWINDOWS}
wingetprogramsfolder;
wingetappdatafolder;
Form_peach.StringGridCustEdit.Cells[1,1]:='Notepad';
custedit1:='notepad.exe';
Form_peach.StringGridCustEdit.Cells[1,2]:='Wordpad';
custedit2:='wordpad.exe';
Form_peach.StringGridCustEdit.Cells[1,3]:='Windows Media Player';
custedit3:='wmplayer.exe';
if ufileexists(winpfolder+'VideoLAN\VLC\vlc.exe') then
   begin
   Form_peach.StringGridCustEdit.Cells[1,4]:='VLC Media Player';
   custedit4:=winpfolder+'VideoLAN\VLC\vlc.exe';
   end
else
   if ufileexists(winpfolder+'Winamp\winamp.exe') then
      begin
      Form_peach.StringGridCustEdit.Cells[1,4]:='Winamp';
      custedit6:=winpfolder+'Winamp\winamp.exe';
      end;
Form_peach.StringGridCustEdit.Cells[1,5]:='Internet Explorer';
custedit5:='iexplore.exe';
if ufileexists(winpfolder+'Mozilla Firefox\firefox.exe') then
   begin
   Form_peach.StringGridCustEdit.Cells[1,6]:='Mozilla Firefox';
   custedit6:=winpfolder+'Mozilla Firefox\firefox.exe';
   end
else
   if ufileexists(winpfolder+'Opera\opera.exe') then
      begin
      Form_peach.StringGridCustEdit.Cells[1,6]:='Opera';
      custedit6:=winpfolder+'Opera\opera.exe';
      end;
if ufileexists(winpfolder+'FastStone Image Viewer\FSViewer.exe') then
   begin
   Form_peach.StringGridCustEdit.Cells[1,7]:='FastStone Image Viewer';
   custedit7:=winpfolder+'FastStone Image Viewer\FSViewer.exe';
   end
else
   if ufileexists(winpfolder+'Google\Picasa3\Picasa3.exe') then
      begin
      Form_peach.StringGridCustEdit.Cells[1,7]:='Picasa';
      custedit7:=winpfolder+'Google\Picasa3\Picasa3.exe';
      end
   else
      if ufileexists(winpfolder+'IrfanView\i_view32.exe') then
         begin
         Form_peach.StringGridCustEdit.Cells[1,7]:='IrfanView';
         custedit7:=winpfolder+'IrfanView\i_view32.exe';
         end;
if ufileexists(winpfolder+'Paint.NET\PaintDotNet.exe') then
   begin
   Form_peach.StringGridCustEdit.Cells[1,8]:='Paint.NET';
   custedit8:=winpfolder+'Paint.NET\PaintDotNet.exe';
   end
else
   begin
   Form_peach.StringGridCustEdit.Cells[1,8]:='MS Paint';
   custedit8:='mspaint.exe';
   end;
set_win_antivirus;
{$ELSE}
if desk_env=1 then custedit1:='gedit'
else
   if desk_env=2 then custedit1:='kedit';
Form_peach.StringGridCustEdit.Cells[1,2]:='OpenOffice Writer';
custedit2:='oowriter';
Form_peach.StringGridCustEdit.Cells[1,3]:='VLC Media Player';
custedit3:='vlc';
Form_peach.StringGridCustEdit.Cells[1,4]:='Totem Movie Player';
custedit4:='totem';
Form_peach.StringGridCustEdit.Cells[1,5]:='Amarok Audio Player';
custedit5:='amarock';
Form_peach.StringGridCustEdit.Cells[1,6]:='Mozilla Firefox';
custedit6:='firefox';
Form_peach.StringGridCustEdit.Cells[1,7]:='Gimp';
custedit7:='gimp';
Form_peach.StringGridCustEdit1.Cells[1,1]:='F-Spot Photo Browser';
advedit1before:='f-spot -v ';
advedit1after:='';
{$ENDIF}
set_custedit_totable;
set_custedit_menus;
end;

procedure load_bookmarks;
var
   s:utf8string;
   rc:integer;
begin
try
uassigntext(book,(confpath+'bookmarks.txt'));
filemode:=0;
reset(book);
read_header(book);
if eof(book)=false then
   repeat
   rc:=Form_peach.StringGridBookmarks.Rowcount+1;
   Form_peach.StringGridBookmarks.RowCount:=rc;
   //Form_peach.StringGridBookmarks.Cells[0,rc-1]:=inttostr(rc-1);
   readln(book,s);
   Form_peach.StringGridBookmarks.Cells[1,rc-1]:=s;
   readln(book,s);
   Form_peach.StringGridBookmarks.Cells[2,rc-1]:=s;
   readln(book,s);
   Form_peach.StringGridBookmarks.Cells[3,rc-1]:=s;
   readln(book,s);
   Form_peach.StringGridBookmarks.Cells[4,rc-1]:=s;
   readln(book,s);
   Form_peach.StringGridBookmarks.Cells[5,rc-1]:=s;
   readln(book,s);
   readln(book,s);
   readln(book,s); //reserved for future use
   Form_peach.StringGridBookmarks.Cells[6,rc-1]:='0';
   until eof(book);
Form_peach.StringGridBookmarks.AutosizeColumns;
Form_peach.StringGridBookmarks.ColWidths[6]:=0;
Form_peach.StringGridBookmarks.ColWidths[0]:=ROWSIZE;
Form_peach.StringGridBookmarks.Cells[6,1]:='1';
CloseFile(book);
except
   try
   CloseFile(book);
   except
   end;
end;
end;

procedure load_custedit;
var
   s:utf8string;
begin
try
uassigntext(cedit,(confpath+'custedit.txt'));
filemode:=0;
reset(cedit);
read_header(cedit);
readln(cedit,s);
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,1]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,2]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,3]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,4]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,5]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,6]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,7]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit.Cells[1,8]:=s;
readln(cedit,s); custedit1:=s;
readln(cedit,s); custedit2:=s;
readln(cedit,s); custedit3:=s;
readln(cedit,s); custedit4:=s;
readln(cedit,s); custedit5:=s;
readln(cedit,s); custedit6:=s;
readln(cedit,s); custedit7:=s;
readln(cedit,s); custedit8:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,1]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,2]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,3]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,4]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,5]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,6]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,7]:=s;
readln(cedit,s); Form_peach.StringGridCustEdit1.Cells[1,8]:=s;
readln(cedit,s); advedit1before:=s;
readln(cedit,s); advedit2before:=s;
readln(cedit,s); advedit3before:=s;
readln(cedit,s); advedit4before:=s;
readln(cedit,s); advedit5before:=s;
readln(cedit,s); advedit6before:=s;
readln(cedit,s); advedit7before:=s;
readln(cedit,s); advedit8before:=s;
readln(cedit,s); advedit1after:=s;
readln(cedit,s); advedit2after:=s;
readln(cedit,s); advedit3after:=s;
readln(cedit,s); advedit4after:=s;
readln(cedit,s); advedit5after:=s;
readln(cedit,s); advedit6after:=s;
readln(cedit,s); advedit7after:=s;
readln(cedit,s); advedit8after:=s;
set_custedit_totable;
set_custedit_menus;
CloseFile(cedit);
except
   reset_custedit; //reset to defaults if custedit is not found (don't reset if it is empty)
   try
   CloseFile(cedit);
   except
   end;
end;
end;

procedure save_conf;
{
if configuration is modified (theming or defaults) the conf file is saved
Peach checks it at FormClose or when launching Pea (that will read the conf file)
}
begin
   try
      uassigntext(conf,confpath+'conf.txt');
      rewrite(conf);
      write_header(conf);
      if theme_path<>'' then writeln(conf,theme_path)
      else writeln(conf,'');
      writeconf_colors;
      writeconf_defaults;
      CloseFile(conf);
   except
      try
         CloseFile(conf);
      except
      end;
   end;
end;

procedure theme_failure;
begin
   load_default_theme;
   graphicsfolder:=graphicsfolderd;
   Form_peach.DirectoryEditthemegraphics.Text:=executable_path+graphicsfolderd;
   opacity:=opacityd;
   color1:=color1d;
   color2:=color2d;
   color3:=color3d;
   color4:=color4d;
   color5:=color5d;
   rowheight:=rowheightd;
   itemheight:=itemheightd;
   autosizeitemheight:=autosizeitemheightd;
   autosizetoolbarbutton:=autosizetoolbarbuttond;
   imgloaded:=false;
   apply_theme;
   try
      CloseFile(conf);
   except
   end;
end;

procedure load_conf;
{
1) Load configuration values, on failure write a new conf with coded values.
2) If a theme is used in configuration, load theme's theming values as defaults
(conf theming values overrides them and are actually displayed); on failure
loading declared theme ask if quit using the theme;
3) Check if configuration values found are correct, then apply the resulting
theming and configuration values
}

procedure conf_failure;
var
   s:utf8string;
begin
try
      try
      CloseFile(conf);
      except
      end;
      default_defaults;
      load_default_theme;
      graphicsfolder:=graphicsfolderd;
      opacity:=opacityd;
      color1:=color1d;
      color2:=color2d;
      color3:=color3d;
      color4:=color4d;
      color5:=color5d;
      rowheight:=rowheightd;
      itemheight:=itemheightd;
      autosizeitemheight:=autosizeitemheightd;
      autosizetoolbarbutton:=autosizetoolbarbuttond;
      imgloaded:=false;
      apply_theme;
      uassigntext(conf,confpath+'conf.txt');
      rewrite(conf);
      write_header(conf);
      writeln(conf,theme_path);
      writeconf_default_colors;
      lang_file:='en.txt'; //load english language
      texts(lang_file);
      writeconf_defaults;
      CloseFile(conf);
      uassigntext(conf,confpath+'conf.txt'); //try again to read the conf file
      filemode:=0;
      reset(conf);
      read_header(conf);
      readln(conf,theme_path);
      readconf_colors;
      readln(conf,s);
      readln(conf,lang_file);
      texts(lang_file);
      readconf_defaults;
      CloseFile(conf);
   except
      conf_critical_error_msg;
      Application.Terminate;
   end;
end;

var
   s:utf8string;
begin
//1) load conf
default_defaults;
try
   uassigntext(conf,executable_path+'res'+directoryseparator+'altconf.txt'); //load alternative configuration path (for conf and bookmarks)
   filemode:=0;
   reset(conf);
   read_header(conf);
   readln(conf,s);
   readln(conf,confpath);
   CloseFile(conf);
except
   try
      CloseFile(conf);
   except
   end;
   uassigntext(conf,executable_path+'res'+directoryseparator+'altconf.txt');
   rewrite(conf);
   write_header(conf);
   writeln(conf,'[alternative configuration''s path: specify a location for bookmarks.txt, conf.txt and rnd (either absolute or relative path, use " and '' if needed); write "same" to use this path, "appdata" to use user''s application data or home path]');
   writeln(conf,'same');
   CloseFile(conf);
end;
if (confpath='same') or (confpath='"same"') or (confpath='''same''') or (confpath=' ') or (confpath='') then confpath:=executable_path+'res'+directoryseparator; //if confpath parameter is set to 'same' or empty use classic conf location (in res folder)
{$IFDEF MSWINDOWS}
if (confpath='appdata') or (confpath='"appdata"') or (confpath='''appdata''') or (confpath='%appdata%') then
   if wingetappdata(confpath)<>0 then confpath:=envtoutf8(GetEnvironmentVariable('APPDATA'))+'\PeaZip\'; //if wingetappdata fails use env variables
{$ENDIF}
{$IFDEF LINUX}
if (confpath='appdata') or (confpath='"appdata"') or (confpath='''appdata''') or (confpath='%appdata%') then confpath:=GetEnvironmentVariable('HOME')+'/.PeaZip/';
{$ENDIF}
{$IFDEF FREEBSD}
if (confpath='appdata') or (confpath='"appdata"') or (confpath='''appdata''') or (confpath='%appdata%') then confpath:=GetEnvironmentVariable('HOME')+'/.PeaZip/';
{$ENDIF}
{$IFDEF NETBSD}
if (confpath='appdata') or (confpath='"appdata"') or (confpath='''appdata''') or (confpath='%appdata%') then confpath:=GetEnvironmentVariable('HOME')+'/.PeaZip/';
{$ENDIF}
if not(udirectoryexists(confpath)) then umkdir(confpath);
confpath:=uexpandfilename(confpath);
if confpath<>'' then if confpath[length(confpath)]<>directoryseparator then confpath:=confpath+directoryseparator;
if not(udirectoryexists(confpath)) then confpath:=executable_path+'res'+directoryseparator; //if alternative configuration directory does not exist or is not accessible, use res path
persistent_source:=confpath+'rnd';
try
   uassigntext(conf,confpath+'conf.txt'); //try to read the conf file in the specified location, on failure try to create a new one where specified by altconf
   filemode:=0;
   reset(conf);
   read_header(conf);
   readln(conf,theme_path);
   readconf_colors;
   readln(conf,s);
   readln(conf,lang_file);
   texts(lang_file);
   readconf_defaults;
   CloseFile(conf);
   if checkver<>'PeaZip' then conf_failure;
except
   conf_failure;
end;
//2) load theme (for default theming values)
try
   load_theme;
except
   theme_failure;
end;
//3) check theming and configuration values, apply if correct
check_theme_failure;
apply_theme;
check_defaults_failure;
updateconf;
Form_peach.LabelConf.Caption:=confpath;
end;

procedure save_new_theme;
begin
try
try
   umkdir(executable_path+themedir);
except
   MessageDlg(themedir+' '+txt_theme_exists, mtError, [mbOK], 0);
   exit;
end;
uassigntext(conf,(executable_path+themedir+'theme.txt'));
rewrite(conf);
write_header(conf);
writeln(conf,Form_peach.Edit1.text);
writeln(conf,Form_peach.Edit2.text);
writeln(conf,Form_peach.Edit3.text);
writeconf_colors;
closefile(conf);
except
   MessageDlg(txt_theme_create_error, mtError, [mbOK], 0);
   exit;
end;
end;

procedure do_resize_trick; //(seem no longer needed, deactivated in 2.5.1) this workaround resolved some issues in resizing form's content on creation and changing windowstate;
begin
{Form_peach.changebounds(ws_left,ws_top,form_peach.width+1,form_peach.height+1);
Form_peach.changebounds(ws_left,ws_top,form_peach.width-1,form_peach.height-1);}
end;

procedure set_multi(s:utf8string); //sets multithreading option for 7za
begin
if s='on' then //allow multithread selection for Compression (LZMA/2 and BZip2), may be replaced by code more oriented toward processors number
   begin
   Form_peach.ComboBoxArchive8.Enabled:=true;
   {$IFDEF MSWINDOWS}
   if GetEnvironmentVariable('NUMBER_OF_PROCESSORS')='1' then Form_peach.ComboBoxArchive8.ItemIndex:=0 //set multithread off, faster on single cpu machines
   else Form_peach.ComboBoxArchive8.ItemIndex:=1;
   {$ELSE}
   Form_peach.ComboBoxArchive8.ItemIndex:=1;
   {$ENDIF}
   end;
if s='off' then //disallow multithread selection since Compression algorithms used are not supported
   begin
   Form_peach.ComboBoxArchive8.Enabled:=false;
   Form_peach.ComboBoxArchive8.ItemIndex:=0;
   end;
end;

procedure set_values(archive_type,compression_method,compression_level:utf8string);
begin
if archive_type=prev_type then
   if compression_method=prev_method then
     if compression_level=prev_level then exit;
settingvalues:=true;
prev_type:=archive_type;
prev_method:=compression_method;
prev_level:=compression_level;
//this code for updating ComboBoxes content and ItemIndex (for 7za) at runtime works properly on Windows, on Linux the code seem not working properly, maybe basically because different graphical libraries are used by the very same code on the two systems so different cares should be taken in account.
Form_peach.ComboBoxArchiveSolid.Enabled:=false;
Form_peach.ComboBoxArchiveSolid.ItemIndex:=0;
if archive_type='7z' then
   begin
   Form_peach.ComboBoxArchiveSolid.Enabled:=true;
   Form_peach.ComboBoxArchive3.Clear;
   Form_peach.ComboBoxArchive3.DropDownCount:=4;
   Form_peach.ComboBoxArchive3.Items.Append('LZMA');
   Form_peach.ComboBoxArchive3.Items.Append('LZMA2');
   Form_peach.ComboBoxArchive3.Items.Append('PPMd');
   Form_peach.ComboBoxArchive3.Items.Append('BZip2');
   Form_peach.ComboBoxArchive4.Clear;
   Form_peach.ComboBoxArchive4.DropDownCount:=6;
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_store);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fastest);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fast);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_normal);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_maximum);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_ultra);
   Form_peach.Label7za18.Visible:=true;
   Form_peach.Combobox7zalgo.Visible:=true;
   Form_peach.Combobox7zalgo.Clear;
   Form_peach.Combobox7zalgo.DropDownCount:=1;
   Form_peach.Combobox7zalgo.Items.Append('AES256');
   Form_peach.Combobox7zalgo.ItemIndex:=0;
   if compression_level=txt_level_store then
      begin
      set_multi('off');
      Form_peach.ComboBoxArchive4.ItemIndex:=0;
      compression_method:='';
      Form_peach.ComboBoxArchive3.Clear;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchive3.Enabled:=false;
      Form_peach.ComboBoxArchive5.Enabled:=false;
      Form_peach.ComboBoxArchive6.Enabled:=false;
      Form_peach.ComboBoxArchivePasses.Enabled:=false;
      Form_peach.ComboBoxArchiveSolid.Enabled:=false;
      end;
   if compression_method='LZMA' then
      begin
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=0;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=15;
      Form_peach.ComboBoxArchive5.Items.Append('64 KB');
      Form_peach.ComboBoxArchive5.Items.Append('1 MB');
      Form_peach.ComboBoxArchive5.Items.Append('2 MB');
      Form_peach.ComboBoxArchive5.Items.Append('3 MB');
      Form_peach.ComboBoxArchive5.Items.Append('4 MB');
      Form_peach.ComboBoxArchive5.Items.Append('6 MB');
      Form_peach.ComboBoxArchive5.Items.Append('8 MB');
      Form_peach.ComboBoxArchive5.Items.Append('12 MB');
      Form_peach.ComboBoxArchive5.Items.Append('16 MB');
      Form_peach.ComboBoxArchive5.Items.Append('24 MB');
      Form_peach.ComboBoxArchive5.Items.Append('32 MB');
      Form_peach.ComboBoxArchive5.Items.Append('48 MB');
      Form_peach.ComboBoxArchive5.Items.Append('64 MB');
      Form_peach.ComboBoxArchive5.Items.Append('96 MB');
      Form_peach.ComboBoxArchive5.Items.Append('128 MB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchive6.DropDownCount:=12;
      Form_peach.ComboBoxArchive6.Items.Append('8');
      Form_peach.ComboBoxArchive6.Items.Append('12');
      Form_peach.ComboBoxArchive6.Items.Append('16');
      Form_peach.ComboBoxArchive6.Items.Append('24');
      Form_peach.ComboBoxArchive6.Items.Append('32');
      Form_peach.ComboBoxArchive6.Items.Append('48');
      Form_peach.ComboBoxArchive6.Items.Append('64');
      Form_peach.ComboBoxArchive6.Items.Append('96');
      Form_peach.ComboBoxArchive6.Items.Append('128');
      Form_peach.ComboBoxArchive6.Items.Append('192');
      Form_peach.ComboBoxArchive6.Items.Append('256');
      Form_peach.ComboBoxArchive6.Items.Append('273');
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=true;
      Form_peach.ComboBoxArchivePasses.Enabled:=false;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=4;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=1;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=8;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=12;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=10;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=13;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=12;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=18;//in 7-Zip 4.49 it's set to 4GB, in PeaZip is set to 'solid' for all methods in ultra mode
         end;
      end;
   if compression_method='LZMA2' then
      begin
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=1;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=15;
      Form_peach.ComboBoxArchive5.Items.Append('64 KB');
      Form_peach.ComboBoxArchive5.Items.Append('1 MB');
      Form_peach.ComboBoxArchive5.Items.Append('2 MB');
      Form_peach.ComboBoxArchive5.Items.Append('3 MB');
      Form_peach.ComboBoxArchive5.Items.Append('4 MB');
      Form_peach.ComboBoxArchive5.Items.Append('6 MB');
      Form_peach.ComboBoxArchive5.Items.Append('8 MB');
      Form_peach.ComboBoxArchive5.Items.Append('12 MB');
      Form_peach.ComboBoxArchive5.Items.Append('16 MB');
      Form_peach.ComboBoxArchive5.Items.Append('24 MB');
      Form_peach.ComboBoxArchive5.Items.Append('32 MB');
      Form_peach.ComboBoxArchive5.Items.Append('48 MB');
      Form_peach.ComboBoxArchive5.Items.Append('64 MB');
      Form_peach.ComboBoxArchive5.Items.Append('96 MB');
      Form_peach.ComboBoxArchive5.Items.Append('128 MB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchive6.DropDownCount:=12;
      Form_peach.ComboBoxArchive6.Items.Append('8');
      Form_peach.ComboBoxArchive6.Items.Append('12');
      Form_peach.ComboBoxArchive6.Items.Append('16');
      Form_peach.ComboBoxArchive6.Items.Append('24');
      Form_peach.ComboBoxArchive6.Items.Append('32');
      Form_peach.ComboBoxArchive6.Items.Append('48');
      Form_peach.ComboBoxArchive6.Items.Append('64');
      Form_peach.ComboBoxArchive6.Items.Append('96');
      Form_peach.ComboBoxArchive6.Items.Append('128');
      Form_peach.ComboBoxArchive6.Items.Append('192');
      Form_peach.ComboBoxArchive6.Items.Append('256');
      Form_peach.ComboBoxArchive6.Items.Append('273');
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=true;
      Form_peach.ComboBoxArchivePasses.Enabled:=false;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=4;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=1;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=8;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=12;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=10;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=13;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=12;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=18;
         end;
      end;
   if compression_method='PPMd' then
      begin
      set_multi('off');
      Form_peach.ComboBoxArchive3.ItemIndex:=2;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=21;
      Form_peach.ComboBoxArchive5.Items.Append('1 KB');
      Form_peach.ComboBoxArchive5.Items.Append('2 MB');
      Form_peach.ComboBoxArchive5.Items.Append('3 MB');
      Form_peach.ComboBoxArchive5.Items.Append('4 MB');
      Form_peach.ComboBoxArchive5.Items.Append('6 MB');
      Form_peach.ComboBoxArchive5.Items.Append('8 MB');
      Form_peach.ComboBoxArchive5.Items.Append('12 MB');
      Form_peach.ComboBoxArchive5.Items.Append('16 MB');
      Form_peach.ComboBoxArchive5.Items.Append('24 MB');
      Form_peach.ComboBoxArchive5.Items.Append('32 MB');
      Form_peach.ComboBoxArchive5.Items.Append('48 MB');
      Form_peach.ComboBoxArchive5.Items.Append('64 MB');
      Form_peach.ComboBoxArchive5.Items.Append('96 MB');
      Form_peach.ComboBoxArchive5.Items.Append('128 MB');
      Form_peach.ComboBoxArchive5.Items.Append('192 MB');
      Form_peach.ComboBoxArchive5.Items.Append('256 MB');
      Form_peach.ComboBoxArchive5.Items.Append('384 MB');
      Form_peach.ComboBoxArchive5.Items.Append('512 MB');
      Form_peach.ComboBoxArchive5.Items.Append('768 MB');
      Form_peach.ComboBoxArchive5.Items.Append('1024 MB');
      Form_peach.ComboBoxArchive5.Items.Append('1536 MB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchive6.DropDownCount:=15;
      Form_peach.ComboBoxArchive6.Items.Append('2');
      Form_peach.ComboBoxArchive6.Items.Append('3');
      Form_peach.ComboBoxArchive6.Items.Append('4');
      Form_peach.ComboBoxArchive6.Items.Append('5');
      Form_peach.ComboBoxArchive6.Items.Append('6');
      Form_peach.ComboBoxArchive6.Items.Append('7');
      Form_peach.ComboBoxArchive6.Items.Append('8');
      Form_peach.ComboBoxArchive6.Items.Append('10');
      Form_peach.ComboBoxArchive6.Items.Append('12');
      Form_peach.ComboBoxArchive6.Items.Append('14');
      Form_peach.ComboBoxArchive6.Items.Append('16');
      Form_peach.ComboBoxArchive6.Items.Append('20');
      Form_peach.ComboBoxArchive6.Items.Append('24');
      Form_peach.ComboBoxArchive6.Items.Append('28');
      Form_peach.ComboBoxArchive6.Items.Append('32');
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=true;
      Form_peach.ComboBoxArchivePasses.Enabled:=false;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=3;
         Form_peach.ComboBoxArchive6.ItemIndex:=2;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=4;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=3;
         Form_peach.ComboBoxArchive6.ItemIndex:=2;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=8;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=7;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=12;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=11;
         Form_peach.ComboBoxArchive6.ItemIndex:=10;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=13;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=14;
         Form_peach.ComboBoxArchive6.ItemIndex:=14;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=18;
         end;
      end;
   if compression_method='BZip2' then
      begin
      Form_peach.ComboBoxArchive8.Enabled:=true;
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=3;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=9;
      Form_peach.ComboBoxArchive5.Items.Append('100 KB');
      Form_peach.ComboBoxArchive5.Items.Append('200 KB');
      Form_peach.ComboBoxArchive5.Items.Append('300 KB');
      Form_peach.ComboBoxArchive5.Items.Append('400 KB');
      Form_peach.ComboBoxArchive5.Items.Append('500 KB');
      Form_peach.ComboBoxArchive5.Items.Append('600 KB');
      Form_peach.ComboBoxArchive5.Items.Append('700 KB');
      Form_peach.ComboBoxArchive5.Items.Append('800 KB');
      Form_peach.ComboBoxArchive5.Items.Append('900 KB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchivePasses.DropDownCount:=10;
      Form_peach.ComboBoxArchivePasses.Items.Append('1');
      Form_peach.ComboBoxArchivePasses.Items.Append('2');
      Form_peach.ComboBoxArchivePasses.Items.Append('3');
      Form_peach.ComboBoxArchivePasses.Items.Append('4');
      Form_peach.ComboBoxArchivePasses.Items.Append('5');
      Form_peach.ComboBoxArchivePasses.Items.Append('6');
      Form_peach.ComboBoxArchivePasses.Items.Append('7');
      Form_peach.ComboBoxArchivePasses.Items.Append('8');
      Form_peach.ComboBoxArchivePasses.Items.Append('9');
      Form_peach.ComboBoxArchivePasses.Items.Append('10');
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=false;
      Form_peach.ComboBoxArchivePasses.Enabled:=true;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=4;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=4;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=6;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=7;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=1;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=7;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=6;
         Form_peach.ComboBoxArchiveSolid.ItemIndex:=18;
         end;
      end;
   end;
if archive_type='tar' then
   begin
   set_multi('off');
   Form_peach.ComboBoxArchive3.Clear;
   Form_peach.ComboBoxArchive3.DropDownCount:=1;
   Form_peach.ComboBoxArchive3.Items.Append('');
   Form_peach.ComboBoxArchive4.Clear;
   Form_peach.ComboBoxArchive4.DropDownCount:=1;
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_store);
   Form_peach.ComboBoxArchive5.Clear;
   Form_peach.ComboBoxArchive5.DropDownCount:=1;
   Form_peach.ComboBoxArchive5.Items.Append('');
   Form_peach.ComboBoxArchive6.Clear;
   Form_peach.ComboBoxArchive6.DropDownCount:=1;
   Form_peach.ComboBoxArchive6.Items.Append('');
   Form_peach.ComboBoxArchivePasses.Clear;
   Form_peach.ComboBoxArchive3.ItemIndex:=0;
   Form_peach.ComboBoxArchive4.ItemIndex:=0;
   Form_peach.ComboBoxArchive5.ItemIndex:=0;
   Form_peach.ComboBoxArchive6.ItemIndex:=0;
   Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
   Form_peach.ComboBoxArchive3.Enabled:=false;
   Form_peach.ComboBoxArchive5.Enabled:=false;
   Form_peach.ComboBoxArchive6.Enabled:=false;
   Form_peach.Label7za18.Visible:=false;
   Form_peach.Combobox7zalgo.Visible:=false;
   Form_peach.ComboBoxArchivePasses.Enabled:=false;
   end;
if archive_type='zip' then
   begin
   Form_peach.ComboBoxArchive3.Clear;
   Form_peach.ComboBoxArchive3.DropDownCount:=4;
   Form_peach.ComboBoxArchive3.Items.Append('Deflate');
   Form_peach.ComboBoxArchive3.Items.Append('Deflate64');
   Form_peach.ComboBoxArchive3.Items.Append('BZip2');
   Form_peach.ComboBoxArchive3.Items.Append('LZMA');
   Form_peach.ComboBoxArchive4.Clear;
   Form_peach.ComboBoxArchive4.DropDownCount:=6;
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_store);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fastest);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fast);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_normal);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_maximum);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_ultra);
   Form_peach.Label7za18.Visible:=true;
   Form_peach.Combobox7zalgo.Visible:=true;
   Form_peach.Combobox7zalgo.Clear;
   Form_peach.Combobox7zalgo.DropDownCount:=2;
   Form_peach.Combobox7zalgo.Items.Append('AES256');
   Form_peach.Combobox7zalgo.Items.Append(txt_zipcrypto_hint);
   Form_peach.Combobox7zalgo.ItemIndex:=0;
   if compression_level=txt_level_store then
      begin
      set_multi('off');
      Form_peach.ComboBoxArchive4.ItemIndex:=0;
      compression_method:='';
      Form_peach.ComboBoxArchive3.Clear;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchive3.Enabled:=false;
      Form_peach.ComboBoxArchive5.Enabled:=false;
      Form_peach.ComboBoxArchive6.Enabled:=false;
      Form_peach.ComboBoxArchivePasses.Enabled:=false;
      end;
   if compression_method='Deflate' then
      begin
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=0;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=1;
      Form_peach.ComboBoxArchive5.Items.Append('32 KB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchive6.DropDownCount:=12;
      Form_peach.ComboBoxArchive6.Items.Append('8');
      Form_peach.ComboBoxArchive6.Items.Append('12');
      Form_peach.ComboBoxArchive6.Items.Append('16');
      Form_peach.ComboBoxArchive6.Items.Append('24');
      Form_peach.ComboBoxArchive6.Items.Append('32');
      Form_peach.ComboBoxArchive6.Items.Append('48');
      Form_peach.ComboBoxArchive6.Items.Append('64');
      Form_peach.ComboBoxArchive6.Items.Append('96');
      Form_peach.ComboBoxArchive6.Items.Append('128');
      Form_peach.ComboBoxArchive6.Items.Append('192');
      Form_peach.ComboBoxArchive6.Items.Append('256');
      Form_peach.ComboBoxArchive6.Items.Append('258');
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchivePasses.DropDownCount:=15;
      Form_peach.ComboBoxArchivePasses.Items.Append('1');
      Form_peach.ComboBoxArchivePasses.Items.Append('2');
      Form_peach.ComboBoxArchivePasses.Items.Append('3');
      Form_peach.ComboBoxArchivePasses.Items.Append('4');
      Form_peach.ComboBoxArchivePasses.Items.Append('5');
      Form_peach.ComboBoxArchivePasses.Items.Append('6');
      Form_peach.ComboBoxArchivePasses.Items.Append('7');
      Form_peach.ComboBoxArchivePasses.Items.Append('8');
      Form_peach.ComboBoxArchivePasses.Items.Append('9');
      Form_peach.ComboBoxArchivePasses.Items.Append('10');
      Form_peach.ComboBoxArchivePasses.Items.Append('11');
      Form_peach.ComboBoxArchivePasses.Items.Append('12');
      Form_peach.ComboBoxArchivePasses.Items.Append('13');
      Form_peach.ComboBoxArchivePasses.Items.Append('14');
      Form_peach.ComboBoxArchivePasses.Items.Append('15');
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=true;
      Form_peach.ComboBoxArchivePasses.Enabled:=true;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;//4;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=2;//4;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=2;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=8;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=9;
         end;
      end;
   if compression_method='Deflate64' then
      begin
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=1;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=1;
      Form_peach.ComboBoxArchive5.Items.Append('64 KB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchive6.DropDownCount:=12;
      Form_peach.ComboBoxArchive6.Items.Append('8');
      Form_peach.ComboBoxArchive6.Items.Append('12');
      Form_peach.ComboBoxArchive6.Items.Append('16');
      Form_peach.ComboBoxArchive6.Items.Append('24');
      Form_peach.ComboBoxArchive6.Items.Append('32');
      Form_peach.ComboBoxArchive6.Items.Append('48');
      Form_peach.ComboBoxArchive6.Items.Append('64');
      Form_peach.ComboBoxArchive6.Items.Append('96');
      Form_peach.ComboBoxArchive6.Items.Append('128');
      Form_peach.ComboBoxArchive6.Items.Append('192');
      Form_peach.ComboBoxArchive6.Items.Append('256');
      Form_peach.ComboBoxArchive6.Items.Append('257');
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchivePasses.DropDownCount:=15;
      Form_peach.ComboBoxArchivePasses.Items.Append('1');
      Form_peach.ComboBoxArchivePasses.Items.Append('2');
      Form_peach.ComboBoxArchivePasses.Items.Append('3');
      Form_peach.ComboBoxArchivePasses.Items.Append('4');
      Form_peach.ComboBoxArchivePasses.Items.Append('5');
      Form_peach.ComboBoxArchivePasses.Items.Append('6');
      Form_peach.ComboBoxArchivePasses.Items.Append('7');
      Form_peach.ComboBoxArchivePasses.Items.Append('8');
      Form_peach.ComboBoxArchivePasses.Items.Append('9');
      Form_peach.ComboBoxArchivePasses.Items.Append('10');
      Form_peach.ComboBoxArchivePasses.Items.Append('11');
      Form_peach.ComboBoxArchivePasses.Items.Append('12');
      Form_peach.ComboBoxArchivePasses.Items.Append('13');
      Form_peach.ComboBoxArchivePasses.Items.Append('14');
      Form_peach.ComboBoxArchivePasses.Items.Append('15');
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=true;
      Form_peach.ComboBoxArchivePasses.Enabled:=true;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;//4;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=2;//4;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=2;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=8;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=9;
         end;
      end;
   if compression_method='BZip2' then
      begin
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=2;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=9;
      Form_peach.ComboBoxArchive5.Items.Append('100 KB');
      Form_peach.ComboBoxArchive5.Items.Append('200 KB');
      Form_peach.ComboBoxArchive5.Items.Append('300 KB');
      Form_peach.ComboBoxArchive5.Items.Append('400 KB');
      Form_peach.ComboBoxArchive5.Items.Append('500 KB');
      Form_peach.ComboBoxArchive5.Items.Append('600 KB');
      Form_peach.ComboBoxArchive5.Items.Append('700 KB');
      Form_peach.ComboBoxArchive5.Items.Append('800 KB');
      Form_peach.ComboBoxArchive5.Items.Append('900 KB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchivePasses.DropDownCount:=10;
      Form_peach.ComboBoxArchivePasses.Items.Append('1');
      Form_peach.ComboBoxArchivePasses.Items.Append('2');
      Form_peach.ComboBoxArchivePasses.Items.Append('3');
      Form_peach.ComboBoxArchivePasses.Items.Append('4');
      Form_peach.ComboBoxArchivePasses.Items.Append('5');
      Form_peach.ComboBoxArchivePasses.Items.Append('6');
      Form_peach.ComboBoxArchivePasses.Items.Append('7');
      Form_peach.ComboBoxArchivePasses.Items.Append('8');
      Form_peach.ComboBoxArchivePasses.Items.Append('9');
      Form_peach.ComboBoxArchivePasses.Items.Append('10');
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=false;
      Form_peach.ComboBoxArchivePasses.Enabled:=true;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=4;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=1;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=0;
         Form_peach.ComboBoxArchivePasses.ItemIndex:=6;
         end;
      end;
   if compression_method='LZMA' then
      begin
      set_multi('on');
      Form_peach.ComboBoxArchive3.ItemIndex:=3;
      Form_peach.ComboBoxArchive5.Clear;
      Form_peach.ComboBoxArchive5.DropDownCount:=15;
      Form_peach.ComboBoxArchive5.Items.Append('64 KB');
      Form_peach.ComboBoxArchive5.Items.Append('1 MB');
      Form_peach.ComboBoxArchive5.Items.Append('2 MB');
      Form_peach.ComboBoxArchive5.Items.Append('3 MB');
      Form_peach.ComboBoxArchive5.Items.Append('4 MB');
      Form_peach.ComboBoxArchive5.Items.Append('6 MB');
      Form_peach.ComboBoxArchive5.Items.Append('8 MB');
      Form_peach.ComboBoxArchive5.Items.Append('12 MB');
      Form_peach.ComboBoxArchive5.Items.Append('16 MB');
      Form_peach.ComboBoxArchive5.Items.Append('24 MB');
      Form_peach.ComboBoxArchive5.Items.Append('32 MB');
      Form_peach.ComboBoxArchive5.Items.Append('48 MB');
      Form_peach.ComboBoxArchive5.Items.Append('64 MB');
      Form_peach.ComboBoxArchive5.Items.Append('96 MB');
      Form_peach.ComboBoxArchive5.Items.Append('128 MB');
      Form_peach.ComboBoxArchive6.Clear;
      Form_peach.ComboBoxArchive6.DropDownCount:=12;
      Form_peach.ComboBoxArchive6.Items.Append('8');
      Form_peach.ComboBoxArchive6.Items.Append('12');
      Form_peach.ComboBoxArchive6.Items.Append('16');
      Form_peach.ComboBoxArchive6.Items.Append('24');
      Form_peach.ComboBoxArchive6.Items.Append('32');
      Form_peach.ComboBoxArchive6.Items.Append('48');
      Form_peach.ComboBoxArchive6.Items.Append('64');
      Form_peach.ComboBoxArchive6.Items.Append('96');
      Form_peach.ComboBoxArchive6.Items.Append('128');
      Form_peach.ComboBoxArchive6.Items.Append('192');
      Form_peach.ComboBoxArchive6.Items.Append('256');
      Form_peach.ComboBoxArchive6.Items.Append('273');
      Form_peach.ComboBoxArchivePasses.Clear;
      Form_peach.ComboBoxArchive3.Enabled:=true;
      Form_peach.ComboBoxArchive5.Enabled:=true;
      Form_peach.ComboBoxArchive6.Enabled:=true;
      Form_peach.ComboBoxArchivePasses.Enabled:=false;
      if compression_level=txt_level_fastest then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=1;
         Form_peach.ComboBoxArchive5.ItemIndex:=0;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         end;
      if compression_level=txt_level_fast then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=2;
         Form_peach.ComboBoxArchive5.ItemIndex:=1;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         end;
      if compression_level=txt_level_normal then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=3;
         Form_peach.ComboBoxArchive5.ItemIndex:=4;
         Form_peach.ComboBoxArchive6.ItemIndex:=4;
         end;
      if compression_level=txt_level_maximum then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=4;
         Form_peach.ComboBoxArchive5.ItemIndex:=8;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         end;
      if compression_level=txt_level_ultra then
         begin
         Form_peach.ComboBoxArchive4.ItemIndex:=5;
         Form_peach.ComboBoxArchive5.ItemIndex:=10;
         Form_peach.ComboBoxArchive6.ItemIndex:=6;
         end;
      end;
   end;
if archive_type='bz2' then
   begin
   set_multi('on');
   Form_peach.ComboBoxArchive3.Clear;
   Form_peach.ComboBoxArchive3.DropDownCount:=1;
   Form_peach.ComboBoxArchive3.Items.Append('BZip2');
   Form_peach.ComboBoxArchive3.ItemIndex:=0;
   Form_peach.ComboBoxArchive4.Clear;
   Form_peach.ComboBoxArchive4.DropDownCount:=5;
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fastest);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fast);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_normal);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_maximum);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_ultra);
   Form_peach.ComboBoxArchive5.Clear;
   Form_peach.ComboBoxArchive5.DropDownCount:=9;
   Form_peach.ComboBoxArchive5.Items.Append('100 KB');
   Form_peach.ComboBoxArchive5.Items.Append('200 KB');
   Form_peach.ComboBoxArchive5.Items.Append('300 KB');
   Form_peach.ComboBoxArchive5.Items.Append('400 KB');
   Form_peach.ComboBoxArchive5.Items.Append('500 KB');
   Form_peach.ComboBoxArchive5.Items.Append('600 KB');
   Form_peach.ComboBoxArchive5.Items.Append('700 KB');
   Form_peach.ComboBoxArchive5.Items.Append('800 KB');
   Form_peach.ComboBoxArchive5.Items.Append('900 KB');
   Form_peach.ComboBoxArchive6.Clear;
   Form_peach.ComboBoxArchivePasses.Clear;
   Form_peach.ComboBoxArchivePasses.DropDownCount:=10;
   Form_peach.ComboBoxArchivePasses.Items.Append('1');
   Form_peach.ComboBoxArchivePasses.Items.Append('2');
   Form_peach.ComboBoxArchivePasses.Items.Append('3');
   Form_peach.ComboBoxArchivePasses.Items.Append('4');
   Form_peach.ComboBoxArchivePasses.Items.Append('5');
   Form_peach.ComboBoxArchivePasses.Items.Append('6');
   Form_peach.ComboBoxArchivePasses.Items.Append('7');
   Form_peach.ComboBoxArchivePasses.Items.Append('8');
   Form_peach.ComboBoxArchivePasses.Items.Append('9');
   Form_peach.ComboBoxArchivePasses.Items.Append('10');
   Form_peach.ComboBoxArchive3.Enabled:=true;
   Form_peach.ComboBoxArchive5.Enabled:=true;
   Form_peach.ComboBoxArchive6.Enabled:=false;
   Form_peach.Label7za18.Visible:=false;
   Form_peach.Combobox7zalgo.Visible:=false;
   Form_peach.ComboBoxArchivePasses.Enabled:=true;
   if compression_level=txt_level_fastest then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=0;
      Form_peach.ComboBoxArchive5.ItemIndex:=0;
      Form_peach.ComboBoxArchive6.ItemIndex:=0;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
      end;
   if compression_level=txt_level_fast then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=1;
      Form_peach.ComboBoxArchive5.ItemIndex:=4;
      Form_peach.ComboBoxArchive6.ItemIndex:=0;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
      end;
   if compression_level=txt_level_normal then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=2;
      Form_peach.ComboBoxArchive5.ItemIndex:=8;
      Form_peach.ComboBoxArchive6.ItemIndex:=0;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
      end;
   if compression_level=txt_level_maximum then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=3;
      Form_peach.ComboBoxArchive5.ItemIndex:=8;
      Form_peach.ComboBoxArchive6.ItemIndex:=0;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=1;
      end;
   if compression_level=txt_level_ultra then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=4;
      Form_peach.ComboBoxArchive5.ItemIndex:=8;
      Form_peach.ComboBoxArchive6.ItemIndex:=0;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=6;
      end;
   end;
if archive_type='gz' then
   begin
   set_multi('on');
   Form_peach.ComboBoxArchive3.Clear;
   Form_peach.ComboBoxArchive3.DropDownCount:=1;
   Form_peach.ComboBoxArchive3.Items.Append('Deflate');
   Form_peach.ComboBoxArchive3.ItemIndex:=0;
   Form_peach.ComboBoxArchive4.Clear;
   Form_peach.ComboBoxArchive4.DropDownCount:=5;
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fastest);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_fast);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_normal);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_maximum);
   Form_peach.ComboBoxArchive4.Items.Append(txt_level_ultra);
   Form_peach.ComboBoxArchive5.Clear;
   Form_peach.ComboBoxArchive5.DropDownCount:=1;
   Form_peach.ComboBoxArchive5.Items.Append('32 KB');
   Form_peach.ComboBoxArchive6.Clear;
   Form_peach.ComboBoxArchive6.DropDownCount:=12;
   Form_peach.ComboBoxArchive6.Items.Append('8');
   Form_peach.ComboBoxArchive6.Items.Append('12');
   Form_peach.ComboBoxArchive6.Items.Append('16');
   Form_peach.ComboBoxArchive6.Items.Append('24');
   Form_peach.ComboBoxArchive6.Items.Append('32');
   Form_peach.ComboBoxArchive6.Items.Append('48');
   Form_peach.ComboBoxArchive6.Items.Append('64');
   Form_peach.ComboBoxArchive6.Items.Append('96');
   Form_peach.ComboBoxArchive6.Items.Append('128');
   Form_peach.ComboBoxArchive6.Items.Append('192');
   Form_peach.ComboBoxArchive6.Items.Append('256');
   Form_peach.ComboBoxArchive6.Items.Append('258');
   Form_peach.ComboBoxArchivePasses.Clear;
   Form_peach.ComboBoxArchivePasses.DropDownCount:=15;
   Form_peach.ComboBoxArchivePasses.Items.Append('1');
   Form_peach.ComboBoxArchivePasses.Items.Append('2');
   Form_peach.ComboBoxArchivePasses.Items.Append('3');
   Form_peach.ComboBoxArchivePasses.Items.Append('4');
   Form_peach.ComboBoxArchivePasses.Items.Append('5');
   Form_peach.ComboBoxArchivePasses.Items.Append('6');
   Form_peach.ComboBoxArchivePasses.Items.Append('7');
   Form_peach.ComboBoxArchivePasses.Items.Append('8');
   Form_peach.ComboBoxArchivePasses.Items.Append('9');
   Form_peach.ComboBoxArchivePasses.Items.Append('10');
   Form_peach.ComboBoxArchivePasses.Items.Append('11');
   Form_peach.ComboBoxArchivePasses.Items.Append('12');
   Form_peach.ComboBoxArchivePasses.Items.Append('13');
   Form_peach.ComboBoxArchivePasses.Items.Append('14');
   Form_peach.ComboBoxArchivePasses.Items.Append('15');
   Form_peach.ComboBoxArchive3.Enabled:=true;
   Form_peach.ComboBoxArchive5.Enabled:=true;
   Form_peach.ComboBoxArchive6.Enabled:=true;
   Form_peach.Label7za18.Visible:=false;
   Form_peach.Combobox7zalgo.Visible:=false;
   Form_peach.ComboBoxArchivePasses.Enabled:=true;
   if compression_level=txt_level_fastest then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=0;
      Form_peach.ComboBoxArchive5.ItemIndex:=0;
      Form_peach.ComboBoxArchive6.ItemIndex:=0;//4;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
      end;
   if compression_level=txt_level_fast then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=1;
      Form_peach.ComboBoxArchive5.ItemIndex:=0;
      Form_peach.ComboBoxArchive6.ItemIndex:=2;//4;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
      end;
   if compression_level=txt_level_normal then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=2;
      Form_peach.ComboBoxArchive5.ItemIndex:=0;
      Form_peach.ComboBoxArchive6.ItemIndex:=4;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=0;
      end;
   if compression_level=txt_level_maximum then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=3;
      Form_peach.ComboBoxArchive5.ItemIndex:=0;
      Form_peach.ComboBoxArchive6.ItemIndex:=6;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=2;
      end;
   if compression_level=txt_level_ultra then
      begin
      Form_peach.ComboBoxArchive4.ItemIndex:=4;
      Form_peach.ComboBoxArchive5.ItemIndex:=0;
      Form_peach.ComboBoxArchive6.ItemIndex:=8;
      Form_peach.ComboBoxArchivePasses.ItemIndex:=9;
      end;
   end;
if openw_all7z=1 then Form_peach.CheckBoxArchiveOF.State:=cbChecked
else Form_peach.CheckBoxArchiveOF.State:=cbUnChecked;
settingvalues:=false;
end;

procedure change_imagesplit;
begin
   case Form_peach.ComboBoxArchive1.ItemIndex of
      0: Form_peach.ImageSplit.Picture.Bitmap:=nil;
      1: Form_peach.ImageSplit.Picture.Bitmap:=nil;
      2: Form_peach.ImageSplit.Picture.Bitmap:=Bfd_supported;
      3: Form_peach.ImageSplit.Picture.Bitmap:=Bmail;
      4: Form_peach.ImageSplit.Picture.Bitmap:=Bmail;
      5: Form_peach.ImageSplit.Picture.Bitmap:=Bdvd_supported;
      6: Form_peach.ImageSplit.Picture.Bitmap:=Bdvd_supported;
      7: Form_peach.ImageSplit.Picture.Bitmap:=Bhd_supported;
      8: Form_peach.ImageSplit.Picture.Bitmap:=Bdvd_supported;
      9: Form_peach.ImageSplit.Picture.Bitmap:=Bdvd_supported;
      end;
end;

procedure suggest_outname(var s:utf8string; t:utf8string; i:integer); //suggest output name, without extension (if applicable)
var
   str,aname,bname,f_ext:utf8string;
begin
if (i=0) or (i>Form_peach.StringGrid1.RowCount-1) then i:=Form_peach.StringGrid1.Row;
if Form_peach.StringGrid1.RowCount>1 then
   begin
   //use specified name (if entered) or compose name from selected object's name
   str:=Form_peach.StringGrid1.Cells[8,i];
   aname:=str;
   f_ext:=uextractfileext(aname);
   cutextension(aname); //aname has the extension cut
   bname:=aname+f_ext; //bname has extension
   if udirectoryexists(str) then s:=bname //never cut after last dot for directories
   else s:=aname; //else cut extension by default
   //exceptions:
   if t=txt_split then s:=bname; //full name if using split, to have filename.ext.001
   if t=STR_BZIP2 then s:=bname; //full name if using Bzip2, since otherwhise bz2 format will not store original filename
   if t=STR_QUAD then s:=bname; //full name if using QUAD/BALZ, to keep original file extension
   if (t=STR_ZPAQ) and (Form_peach.RadioGroupPaq.ItemIndex=0) then s:=bname; //full name if using LPAQ, to keep original file extension
   if (Form_peach.StringGrid1.Rowcount=2) and (Form_peach.StringGrid1.Cells[2,1]=txt_list_isfolder) then s:=bname; //don't strip "extension" if compressing a single dir (it is indeed part of the name)
   if (Form_peach.StringGrid1.Rowcount=2) and (upcase(copy(bname,length(bname)-3,4))='.TAR') then s:=bname; //don't strip .tar extension if compressing a single tar, to have filename.tar.compression
   end;
end;

function testname(name:utf8string; sg:TStringGrid):integer; //test if an object is jet listed in the stringgrid
var
   i:integer;
begin
testname:=1;
if checkfiledirname(name)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+name, mtWarning, [mbOK], 0); exit; end;
for i:=1 to sg.RowCount-1 do
   if sg.Cells[8,i]=name then
      begin
      testname:=1;
      exit;
      end;
testname:=0;
end;

procedure deselectingrid(sg:tstringgrid);
var
   i:integer;
begin
for i:=1 to sg.Rowcount-1 do
   sg.Cells[11,i]:='0';
end;

procedure addfilestr(sg:TStringGrid; filename:utf8string);
var
   f:file of byte;
   fsize:qword;
   s:utf8string;
   rc:integer;
begin
if sg=Form_Peach.Stringgrid1 then
   begin
   try
   srcfilesize(filename,fsize);
   rc:=sg.Rowcount+1;
   sg.RowCount:=rc;
   sg.Cells[0,rc-1]:='';
   sg.Cells[1,rc-1]:=uextractfilename(filename);
   sg.Cells[2,rc-1]:=uextractfileext(filename);
   sg.Cells[3,rc-1]:=inttostr(fsize);
   sg.Cells[4,rc-1]:='0';
   sg.Cells[5,rc-1]:='1';
   sg.Cells[6,rc-1]:=datetimetostr(filedatetodatetime(ufileage(filename)));
   dword2decodedFileAttributes(ufilegetattr(filename),s);
   sg.Cells[7,rc-1]:=s;
   sg.Cells[8,rc-1]:=filename;
   sg.Cells[9,rc-1]:='2'+filename;
   sg.Cells[10,rc-1]:=inttostr(length(sg.Cells[3,rc-1]))+sg.Cells[3,rc-1];
   sg.Cells[11,rc-1]:='0';
   except
   exit;
   end;
   end
else
   begin
   try
   srcfilesize(filename,fsize);
   rc:=sg.Rowcount+1;
   sg.RowCount:=rc;
   sg.Cells[0,rc-1]:='';
   sg.Cells[1,rc-1]:=uextractfilename(filename);
   sg.Cells[2,rc-1]:=uextractfileext(filename);
   sg.Cells[3,rc-1]:=inttostr(fsize);
   sg.Cells[4,rc-1]:='';
   sg.Cells[5,rc-1]:=datetimetostr(filedatetodatetime(ufileage(filename)));
   dword2decodedFileAttributes(ufilegetattr(filename),s);
   sg.Cells[6,rc-1]:=s;
   sg.Cells[7,rc-1]:='';
   sg.Cells[8,rc-1]:=filename;
   sg.Cells[9,rc-1]:='2'+filename;
   sg.Cells[10,rc-1]:=inttostr(length(sg.Cells[3,rc-1]))+sg.Cells[3,rc-1];
   sg.Cells[11,rc-1]:='0';
   except
   exit;
   end;
   end
end;

procedure addfolderstr(sg:TStringGrid; foldername:utf8string);
var
   r:TSearchRec;
   s:utf8string;
   nfiles,ndirs,size:qword;
   rc:integer;
begin
DirSize(foldername,'*',FaAnyFile,true,true,nfiles,ndirs,size);
rc:=sg.Rowcount+1;
sg.RowCount:=rc;
sg.Cells[0,rc-1]:='';
sg.Cells[1,rc-1]:=uextractfilename(foldername);
sg.Cells[2,rc-1]:=txt_list_isfolder;
sg.Cells[3,rc-1]:=inttostr(size);
sg.Cells[4,rc-1]:=inttostr(ndirs);
sg.Cells[5,rc-1]:=inttostr(nfiles-ndirs);
if uFindFirst(foldername+'.',faDirectory,r) = 0 then
   begin
   sg.Cells[6,rc-1]:=datetimetostr(filedatetodatetime(r.Time));
   dword2decodedFileAttributes(r.Attr,s);
   sg.Cells[7,rc-1]:=s;
   end;
sg.Cells[8,rc-1]:=foldername;
sg.Cells[9,rc-1]:='1'+foldername;
sg.Cells[10,rc-1]:=inttostr(length(sg.Cells[3,rc-1]))+sg.Cells[3,rc-1];
sg.Cells[11,rc-1]:='0';
FindClose(r);
end;

procedure check_tarbefore(tdirs:integer);
begin
Form_peach.CheckBoxSeparate.State:=cbUnChecked;
Form_peach.CheckBoxSeparate.Enabled:=False;
if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) then
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked
else
   Form_peach.CheckBoxTarBefore.State:=cbChecked;
end;

function addext(s:utf8string):utf8string;
begin
if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=s+'.tar';
if Form_peach.cbType.Text=txt_custom then begin result:=s; exit; end;
if Form_peach.cbType.Text=STR_PEA then begin result:=s+'.pea'; exit; end;
if Form_peach.cbType.Text=txt_split then begin result:=s+'.001'; exit; end;
if Form_peach.cbType.Text=STR_UPX then begin result:=s+'.exe'; exit; end;
if ((archive_type='7z') and (Form_peach.CheckBoxArchive6.Visible=true) and (Form_peach.CheckBoxArchive6.State=cbChecked)) or
   ((archive_type='arc') and (Form_peach.CheckBoxArc2.State=cbChecked))
   then result:=s+'.exe'
   else result:=s+'.'+archive_type;
end;

procedure updatecontent(sg:TStringGrid; var tvolumes,tdirs,tfiles,tsize:qword; upsort:boolean); //update label showing bytes/files/dirs listed in the stringgrid1; update list of archive types
var
   i:integer;
   s:utf8string;
begin
tfiles:=0;
tdirs:=0;
tvolumes:=0;
tsize:=0;
for i:=1 to sg.RowCount-1 do
   begin
   tdirs:=tdirs+strtoint(sg.Cells[4,i]);
   tfiles:=tfiles+strtoint(sg.Cells[5,i]);
   try
      tsize:=tsize+strtoqword(sg.Cells[3,i]);
   except
   end;
   if upsort=true then sg.Cells[11,i]:='0';
   end;
Form_peach.StringGrid1.AutoSizeColumns;
Form_peach.StringGrid1.ColWidths[0]:=ROWSIZE;
Form_peach.StringGrid1.ColWidths[1]:=Form_peach.StringGrid1.ColWidths[1]+48;
Form_peach.StringGrid1.ColWidths[9]:=0;
Form_peach.StringGrid1.ColWidths[10]:=0;
Form_peach.StringGrid1.ColWidths[11]:=0;
Form_peach.StringGrid1.ColWidths[12]:=0;
Form_peach.StringGrid1.colwidths[13]:=8;
if using_tarbefore=false then
   begin
   if (Form_peach.cbType.Text=STR_GZIP) or
      (Form_peach.cbType.Text=STR_BZIP2) or
      (Form_peach.cbType.Text=STR_QUAD) or
      (Form_peach.cbType.Text=txt_split) or
      ((Form_peach.cbType.Text=STR_ZPAQ) and (Form_peach.RadioGroupPaq.ItemIndex=0)) then
      check_tarbefore(tdirs);
   end;
if Form_peach.StringGrid1.RowCount=1 then
   begin
   exit;
   end;
if upsort=true then
   begin
   Form_peach.StringGrid1.SortColRow(true,9);
   Form_peach.StringGrid1.Cells[11,1]:='1';
   Form_peach.StringGrid1.Row:=1;
   end;
status1:=inttostr(tdirs)+' '+txt_dirs+' '+inttostr(tfiles)+' '+txt_files+' '+inttostr(tsize)+' B';
if updatingarchive=false then
   begin
   Form_peach.LabelStatus1.Caption:=status1;
   Form_peach.LabelStatus4.Caption:=txt_archiving;
   end
else
   begin
   Form_peach.LabelStatus1.Caption:=status1;
   Form_peach.LabelStatus4.Caption:=txt_2_7_updating;
   end;
if (Form_peach.EditName3.Text='') or (Form_peach.Edit4.Text='') then setoutname(s,archive_type,0);
end;

procedure autosize_stringgrid2;
begin
with Form_peach do
begin
StringGrid2.AutoSizeColumns;
StringGrid2.ColWidths[0]:=ROWSIZE;
StringGrid2.ColWidths[1]:=StringGrid2.ColWidths[1]+48;
StringGrid2.ColWidths[9]:=0;
StringGrid2.ColWidths[10]:=0;
StringGrid2.ColWidths[11]:=0;
StringGrid2.ColWidths[12]:=0;
StringGrid2.colwidths[13]:=8;
end;
end;

procedure updatecontent_ext;
var
   i,rc:integer;
   s:utf8string;
begin
tfiles:=0;
tdirs:=0;
tsize:=0;
if Form_peach.StringGrid2.RowCount=1 then
else
   begin
   rc:=Form_peach.StringGrid2.RowCount-1;
   tfiles:=0;
   tdirs:=0;
   tsize:=0;
   for i:=1 to rc do
      begin
      if Form_peach.StringGrid2.Cells[2,i] <> txt_list_isfolder then tfiles:=tfiles+1
      else tdirs:=tdirs+1;
      try
         tsize:=tsize+strtoqword(Form_peach.StringGrid2.Cells[3,i]);
      except
      end;
      Form_peach.StringGrid2.Cells[11,i]:='0';
      end;
   end;
Form_peach.labelstatus3.Caption:=inttostr(tdirs)+' '+txt_dirs+' '+inttostr(tfiles)+' '+txt_files+' '+inttostr(tsize)+' B';
autosize_stringgrid2;
if Form_peach.StringGrid2.RowCount>1 then Form_peach.StringGrid2.Cells[11,1]:='1';
end;

procedure OpenKeyFile;
begin
if Form_peach.OpenDialog2.Execute then
   if Form_peach.OpenDialog2.FileName<>'' then keyf_opened:=true
   else keyf_opened:=false;
end;

procedure addfile;
var
   i:integer;
begin
if Form_peach.OpenDialog2.Execute then
   if Form_peach.OpenDialog2.FileName<>'' then
      begin
      Form_peach.StringGrid1.Cursor:=crHourGlass;
      deselectingrid(Form_peach.StringGrid1);
      Form_peach.OpenDialog2.Files.Count;
      for i:=0 to Form_peach.OpenDialog2.Files.Count-1 do
         if testname(Form_peach.OpenDialog2.Files[i], Form_peach.StringGrid1)=0 then
            begin
            //show names of items being added is not implemented here since adding files is very fast
            addfilestr(Form_peach.StringGrid1,Form_peach.OpenDialog2.Files[i]);
            Form_peach.StringGrid1.AutoSizeColumns;
            updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
            end;
      Form_peach.StringGrid1.Cursor:=crDefault;
      end;
end;

procedure test_extfile(s:utf8string; var okfile:boolean);
begin
okfile:=false;
if testname(s, Form_peach.StringGrid2)=0 then //no duplicate entries
   if (testinput(s,false)=0) then //unsupported
      if Form_peach.Checkbox2.State=cbChecked then okfile:=true
      else
   else
      case testext(s) of
         503: if Form_peach.Checkbox1.State=cbChecked then okfile:=true;
         502: if Form_peach.Checkbox1.State=cbChecked then okfile:=true;
         501: if Form_peach.Checkbox1.State=cbChecked then okfile:=true; //non-archives
         500: if Form_peach.Checkbox1.State=cbChecked then okfile:=true; //Windows executables, installers
         else okfile:=true;
         end;
end;

procedure addfile_ext;
var
   i:integer;
   okfile:boolean;
begin
if Form_peach.OpenDialog2.Execute then
   if Form_peach.OpenDialog2.FileName<>'' then
      begin
      Form_peach.StringGrid2.Cursor:=crHourGlass;
      deselectingrid(Form_peach.StringGrid2);
      Form_peach.OpenDialog2.Files.Count;
      for i:=0 to Form_peach.OpenDialog2.Files.Count-1 do
         begin
         test_extfile(Form_peach.OpenDialog2.Files[i],okfile);
         if okfile=true then
            begin
            if checkfiledirname(Form_peach.OpenDialog2.Files[i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.OpenDialog2.Files[i], mtWarning, [mbOK], 0); exit; end;
            addfilestr(Form_peach.StringGrid2,Form_peach.OpenDialog2.Files[i]);
            end;
         end;
      updatecontent_ext;
      Form_peach.StringGrid2.Cursor:=crDefault;
      end;
end;

procedure addfolder;
var
   s:utf8string;
begin
s:=Form_peach.Caption;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      if testname(Form_peach.SelectDirectoryDialog1.FileName, Form_peach.StringGrid1)=0 then
         begin
         Form_peach.StringGrid1.Cursor:=crHourGlass;
         Form_peach.Caption:=txt_2_4_adding+' '+Form_peach.SelectDirectoryDialog1.FileName;
         Application.ProcessMessages;
         deselectingrid(Form_peach.StringGrid1);
         addfolderstr(Form_peach.StringGrid1,Form_peach.SelectDirectoryDialog1.Filename);
         Form_peach.StringGrid1.AutoSizeColumns;
         updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
         Form_peach.StringGrid1.Cursor:=crDefault;
         end;
Form_peach.Caption:=s;
end;

procedure setsearch_disabled;
begin
Form_peach.EditUn7zaFilter.Text:='*';
Form_peach.EditUn7zaFilter.Enabled:=false;
Form_peach.EditUn7zaFilter.Color:=clBtnFace;
Form_peach.EditUn7zaFilter1.Enabled:=Form_peach.EditUn7zaFilter.Enabled;
Form_peach.EditUn7zaFilter1.Color:=Form_peach.EditUn7zaFilter.Color;
Form_peach.ImageListSearch.Enabled:=false;
end;

procedure setsearch_enabled;
begin
Form_peach.EditUn7zaFilter.Enabled:=true;
Form_peach.EditUn7zaFilter.Color:=clWindow;
Form_peach.EditUn7zaFilter1.Enabled:=Form_peach.EditUn7zaFilter.Enabled;
Form_peach.EditUn7zaFilter1.Color:=Form_peach.EditUn7zaFilter.Color;
Form_peach.ImageListSearch.Enabled:=true;
end;

procedure advfiltersupdate;
begin
with Form_peach do
begin
if Form7.CheckBoxAdvFilters.State=cbChecked then
   begin
   if fun='UN7Z' then setsearch_disabled;
   status0:=txt_list_found;
   madvfilters.caption:=txt_2_7_setadvf+' *';
   end
else
   begin
   setsearch_enabled;
   status0:=txt_list_browsing;
   madvfilters.caption:=txt_2_7_setadvf;
   browsingmode7z:=0;
   end;
end;
end;

procedure loadlist(sg:TStringGrid);
var
   in_param:TFoundlist;
   listfile:utf8string;
   i:integer;
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.FileName<>'' then
      begin
      listfile:=Form_peach.OpenDialog3.FileName;
      read_filelist(listfile,in_param);
      for i:=0 to length(in_param)-1 do
         begin
         if pos('*filter* ',in_param[i])=1 then
            begin
            Form7.MemoAdvExclude.Append(copy(in_param[i],10,length(in_param[i])-9));
            if Form7.CheckBoxAdvFilters.State=cbUnchecked then
               begin
               Form7.CheckBoxAdvFilters.State:=cbChecked;
               Form7.GroupBoxOpenFilters.Enabled:=True;
               advfiltersupdate;
               end;
            end
         else
            if pos('*filter_include* ',in_param[i])=1 then
               begin
               Form7.MemoAdvInclude.Append(copy(in_param[i],18,length(in_param[i])-17));
               if Form7.CheckBoxAdvFilters.State=cbUnchecked then
                  begin
                  Form7.CheckBoxAdvFilters.State:=cbChecked;
                  Form7.GroupBoxOpenFilters.Enabled:=True;
                  advfiltersupdate;
                  end;
               end
            else
               begin
               if sg=Form_peach.StringGrid1 then
                  if testname(in_param[i], sg)=0 then
                     if ufilegetattr(in_param[i]) > 0 then
                        if ufilegetattr(in_param[i]) and faDirectory =0 then addfilestr(sg,in_param[i])
                        else addfolderstr(sg,in_param[i])
                     else msg_not_accessible_list(in_param[i])
                  else
               else
                  if testname(in_param[i], sg)=0 then
                     if ufilegetattr(in_param[i]) > 0 then
                        if ufilegetattr(in_param[i]) and faDirectory =0 then addfilestr(sg,in_param[i])
                        else
                     else msg_not_accessible_list(in_param[i]);
               end;
         end;
      if sg=Form_peach.StringGrid1 then updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true)
      else updatecontent_ext;
      end;
end;

procedure savelist(sg:TStringGrid);
var
   x:dword;
   tname:utf8string;
begin
if sg.RowCount=1 then
   begin
   MessageDlg(txt_2_7_noinput, mtWarning, [mbOK], 0);
   exit;
   end;
if Form_Peach.SaveDialogPJ.Execute then
   if Form_Peach.SaveDialogPJ.FileName<>'' then tname:=Form_Peach.SaveDialogPJ.FileName;
if tname='' then exit;
uassigntext(t,tname);
rewrite(t);
write_header(t);
for x:=1 to sg.RowCount-1 do writeln(t,sg.Cells[8,x]);
if Form7.MemoAdvExclude.Lines.Count>0 then
   for x:=0 to Form7.MemoAdvExclude.Lines.Count-1 do writeln(t,'*filter* '+Form7.MemoAdvExclude.Lines[x]);
if Form7.MemoAdvInclude.Lines.Count>0 then
   for x:=0 to Form7.MemoAdvInclude.Lines.Count-1 do writeln(t,'*filter_include* '+Form7.MemoAdvInclude.Lines[x]);
closefile(t);
end;

procedure clearlist;
begin
Form_peach.StringGrid1.Row:=0;
Form_peach.StringGrid1.RowCount:=1;
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
end;

procedure clearlist_ext;
begin
Form_peach.StringGrid2.Row:=0;
Form_peach.StringGrid2.RowCount:=1;
Form_peach.labelstatus2.Caption:='';
end;

{procedure display_lpaq_memory;
var
   i:integer;
begin
case Form_peach.ComboBoxLpaqComp.ItemIndex of
   0: i:=6;
   1: i:=9;
   2: i:=15;
   3: i:=27;
   4: i:=51;
   5: i:=99;
   6: i:=195;
   7: i:=387;
   8: i:=771;
   9: i:=1539;
   end;
Form_peach.LabelLpaqMem.Caption:=inttostr(i)+' '+txt_required_memory;
end;}

procedure setcompressionlevel7z;
begin
if (archive_type='7z') then
   begin
   case level_7z of
      0: compression_level:=txt_level_store;
      1: compression_level:=txt_level_fastest;
      2: compression_level:=txt_level_fast;
      3: compression_level:=txt_level_normal;
      4: compression_level:=txt_level_maximum;
      5: compression_level:=txt_level_ultra;
      end;
   end;
if (archive_type='zip') then
   begin
   case level_zip of
      0: compression_level:=txt_level_store;
      1: compression_level:=txt_level_fastest;
      2: compression_level:=txt_level_fast;
      3: compression_level:=txt_level_normal;
      4: compression_level:=txt_level_maximum;
      5: compression_level:=txt_level_ultra;
      end;
   end;
if (archive_type='bz2') then
   begin
   case level_bzip2 of
      0: compression_level:=txt_level_fastest;
      1: compression_level:=txt_level_fast;
      2: compression_level:=txt_level_normal;
      3: compression_level:=txt_level_maximum;
      4: compression_level:=txt_level_ultra;
      end;
   end;
if (archive_type='gz') then
   begin
   case level_gz of
      0: compression_level:=txt_level_fastest;
      1: compression_level:=txt_level_fast;
      2: compression_level:=txt_level_normal;
      3: compression_level:=txt_level_maximum;
      4: compression_level:=txt_level_ultra;
      end;
   end;
end;

procedure setcompressionlevel7zindex;
begin
if (archive_type='7z') then Form_peach.ComboBoxArchive4.ItemIndex:=level_7z;
if (archive_type='zip') then Form_peach.ComboBoxArchive4.ItemIndex:=level_zip;
if (archive_type='bz2') then Form_peach.ComboBoxArchive4.ItemIndex:=level_bzip2;
if (archive_type='gz') then Form_peach.ComboBoxArchive4.ItemIndex:=level_gz;
end;

procedure on_CheckBoxArchive6_click;
begin
with Form_peach do
begin
if CheckBoxArchive6.State=cbChecked then
   begin
   ComboBoxArchive1.ItemIndex:=0;
   ComboBoxArchive1.Enabled:=false;
   SpinEditArchive1.Visible:=false;
   ComboBoxArchive2.Visible:=false;
   RadioGroupArchive.Visible:=true;
   CheckBoxArchive4.Visible:=true;
   end
else
   begin
   ComboBoxArchive1.Enabled:=true;
   RadioGroupArchive.Visible:=false;
   end;
change_imagesplit;
end;
end;

procedure setpwicons;
var
   s:utf8string;
begin
try
if Form2.EditUn7zaPW.Text='' then
   begin
   Form_peach.msetpw.Bitmap:=Blocker2;
   Form_peach.msetpw.Caption:=txt_2_7_setpw;
   Form_peach.ImagePassword2.Glyph:=blocker2;
   Form_peach.ImagePassword1.Glyph:=blocker2;
   Form_peach.ImagePassword.Glyph:=blocker2;
   Form_peach.ImagePassword.Hint:=txt_2_7_pwnotset;
   Form_peach.po_setpw.Bitmap:=Blocker2;
   Form_peach.po_setpw.Caption:=txt_2_7_setpw;
   Form_peach.menuitempw.Bitmap:=Blocker2;
   Form_peach.menuitempw.Caption:=txt_2_7_setpw;
   if (archive_type<>'7z') and (archive_type<>'zip') and (archive_type<>'arc') and (fun<>'PEA') then
      Form_peach.Label2.Caption:=txt_2_7_pwnotset+', '+txt_2_5_nopw
   else
      Form_peach.Label2.Caption:=txt_2_7_pwnotset;
   Form_peach.Label3.Caption:=txt_2_7_pwnotset;
   end
else
   begin
   Form_peach.msetpw.Bitmap:=Blocker;
   Form_peach.msetpw.Caption:=txt_2_7_setpw+' *';
   Form_peach.ImagePassword2.Glyph:=blocker;
   Form_peach.ImagePassword1.Glyph:=blocker;
   Form_peach.ImagePassword.Glyph:=blocker;
   Form_peach.ImagePassword.Hint:=txt_2_7_pwextset;
   Form_peach.po_setpw.Bitmap:=Blocker;
   Form_peach.po_setpw.Caption:=txt_2_7_setpw+' *';
   Form_peach.menuitempw.Bitmap:=Blocker;
   Form_peach.menuitempw.Caption:=txt_2_7_setpw+' *';
   if (archive_type<>'7z') and (archive_type<>'zip') and (archive_type<>'arc') and (fun<>'PEA') then s:=txt_2_5_nopw
   else s:=txt_2_7_pwsupported;
   Form_peach.Label2.Caption:=txt_2_7_pwarcset+', '+s;
   Form_peach.Label3.Caption:=txt_2_7_pwextset;
   end;
except
end;
end;

procedure archive_type_select(s:utf8string);
var
   i:integer;
   s1:utf8string;
begin
Form_peach.ComboBoxArchive1.Enabled:=true;
Form_peach.Panel7za.Visible:=false;
Form_peach.PanelARC.Visible:=false;
Form_peach.PanelCustom.Visible:=false;
Form_peach.PanelPaq.Visible:=false;
Form_peach.PanelPea.Visible:=false;
Form_peach.PanelQuad.Visible:=false;
Form_peach.PanelUPX.Visible:=false;
subfun:='';
if s=STR_7Z then
   begin
   Form_peach.Panel7za.Visible:=true;
   archive_type:='7z';
   fun:='7Z';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_7z;
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxArchive7.Text;
   if (compression_method='') or (prev_type<>'7z') then compression_method:=method_7z;//'LZMA';
   setcompressionlevel7z;//set compression_level
   set_values(archive_type,compression_method,compression_level);
   Form_peach.CheckBoxArchive6.Visible:=true;
   Form_peach.CheckBoxArchive6.State:=cbUnchecked;
   Form_peach.ComboBoxArchive4.Text:=txt_level_normal;
   Form_peach.ComboBoxArchive4.Enabled:=true;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   if Form_peach.ComboBoxArchive1.ItemIndex > 0 then Form_peach.CheckBoxArchive4.Visible:=false
   else Form_peach.CheckBoxArchive4.Visible:=true;
   end;
if s=STR_ARC then
   begin
   Form_peach.PanelARC.Visible:=true;
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   archive_type:='arc';
   fun:='ARC';
   Form_peach.ComboBoxARC.ItemIndex:=level_arc;
   Form_peach.ComboBoxARCsolid.ItemIndex:=solid_arc;
   if rr_arc=1 then Form_peach.CheckBoxARC1.State:=cbChecked
   else Form_peach.CheckBoxARC1.State:=cbUnchecked;
   Form_peach.ComboBoxARCalgo.ItemIndex:=algo_arc;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_arc;
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxARC2.Text;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   Form_peach.CheckBoxARC2.State:=cbunChecked;
   Form_peach.ComboBoxARCsfx.visible:=false;
   end;
if s=STR_BZIP2 then
   begin
   Form_peach.Panel7za.Visible:=true;
   archive_type:='bz2';
   fun:='7Z';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_bzip2;
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxArchive7.Text;
   compression_method:='Bzip2';
   setcompressionlevel7z;//set compression_level
   set_values(archive_type,compression_method,compression_level);
   Form_peach.CheckBoxArchive6.Visible:=false;
   Form_peach.ComboBoxArchive4.Text:=txt_level_normal;
   Form_peach.ComboBoxArchive4.Enabled:=true;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   check_tarbefore(tdirs);
   if Form_peach.ComboBoxArchive1.ItemIndex > 0 then Form_peach.CheckBoxArchive4.Visible:=false
   else Form_peach.CheckBoxArchive4.Visible:=true;
   end;
if s=txt_custom then
   begin
   Form_peach.PanelCustom.Visible:=true;
   archive_type:=txt_custom;
   fun:='CUSTOM';
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_custom;
   Form_peach.Caption:=txt_custom;
   Form_peach.CheckBoxTarBefore.Enabled:=false;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=false;
   Form_peach.CheckBoxSeparate.State:=cbUnchecked;
   Form_peach.EditNameCustom.Caption:=Form_peach.pcustom1.caption;
   s1:=uextractfilename(Form_peach.pcustom1.caption);
   cutextension(s1);
   Form_peach.EditExtCustom.Caption:=s1;
   end;
if s=STR_GZIP then
   begin
   Form_peach.Panel7za.Visible:=true;
   archive_type:='gz';
   fun:='7Z';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_gzip;
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxArchive7.Text;
   compression_method:='Deflate';
   setcompressionlevel7z;//set compression_level
   set_values(archive_type,compression_method,compression_level);
   Form_peach.CheckBoxArchive6.Visible:=false;
   Form_peach.ComboBoxArchive4.Text:=txt_level_normal;
   Form_peach.ComboBoxArchive4.Enabled:=true;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   check_tarbefore(tdirs);
   if Form_peach.ComboBoxArchive1.ItemIndex > 0 then Form_peach.CheckBoxArchive4.Visible:=false
   else Form_peach.CheckBoxArchive4.Visible:=true;
   end;
{if s=STR_LPAQ then
   begin
   Form_peach.PanelLpaq.Visible:=true;
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   fun:='LPAQ';
   case Form_peach.RadioGroupLpaq.ItemIndex of
      0: archive_type:='lpaq1';
      1: archive_type:='lpaq5';
      2: archive_type:='lpaq8';
   end;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_lpaq;
   Form_peach.Caption:=txt_create+' .'+archive_type;
   Form_peach.ComboBoxLpaqComp.ItemIndex:=6;
   display_lpaq_memory;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   check_tarbefore(tdirs);
   end; }
if s=STR_ZPAQ then
   begin
   Form_peach.PanelPaq.Visible:=true;
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   fun:='PAQ';
   case Form_peach.RadioGroupPaq.ItemIndex of
      0: begin
         archive_type:='lpaq8';
         check_tarbefore(tdirs);
         if Form_peach.StringGrid1.Rowcount>1 then
            begin
            suggest_outname(s,Form_peach.cbType.Text,0);
            Form_peach.EditName3.Text:=uextractfilepath(s);
            Form_peach.Edit4.Text:=uextractfilename(s);
            Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
            end;
         end;
      1: archive_type:='paq8o';
      2: archive_type:='zpaq';
   end;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_paq;
   Form_peach.Caption:=txt_create+' .'+archive_type;
   Form_peach.ComboBoxPaq1.ItemIndex:=level_paq;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   end;
if s=STR_PEA then
   begin
   Form_peach.PanelPea.Visible:=true;
   Form_peach.ComboBoxPeaCompression1.ItemIndex:=pcompr;
   Form_peach.ComboBoxPea2.ItemIndex:=pstream;
   if pstream<3 then Form_peach.CheckBoxPeaEncryption.State:=cbChecked
   else Form_peach.CheckBoxPeaEncryption.State:=cbUnchecked;
   Form_peach.ComboBoxPea2.ItemIndex:=pstream;
   Form_peach.ComboBoxPea3.ItemIndex:=pobj;
   Form_peach.ComboBoxPea4.ItemIndex:=pvol;
   Form_peach.PanelSplitFile.Visible:=false;
   Form_peach.PanelSplitFile1.Visible:=false;
   Form_peach.PanelSplitFile2.Visible:=false;
   archive_type:='';
   fun:='PEA';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_pea;
   Form_peach.Caption:=txt_create+' .pea';
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   end;
if s=STR_QUAD then
   begin
   Form_peach.PanelQuad.Visible:=true;
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   fun:='QUAD';
   case Form_peach.RadioGroupQuad.ItemIndex of
      0: archive_type:='quad';
      1: archive_type:='balz';
   end;
   Form_peach.Caption:=txt_create+' .'+archive_type;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_quad;
   if level_quad=0 then Form_peach.CheckBoxQuadMax.State:=cbUnchecked else Form_peach.CheckBoxQuadMax.State:=cbChecked;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   check_tarbefore(tdirs);
   end;
if s=txt_sfx+' '+STR_7Z then
   begin
   Form_peach.Panel7za.Visible:=true;
   archive_type:='7z';
   fun:='7Z';
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_7z;
   Form_peach.Caption:=txt_create+' '+txt_sfx+' ('+STR_7Z+'), '+Form_peach.ComboboxArchive7.Text;
   if (compression_method='') or (prev_type<>'7z') then compression_method:=method_7z;//'LZMA';
   setcompressionlevel7z;//set compression_level
   set_values(archive_type,compression_method,compression_level);
   Form_peach.CheckBoxArchive6.Visible:=true;
   Form_peach.CheckBoxArchive6.State:=cbChecked;
   Form_peach.ComboBoxArchive4.Text:=txt_level_normal;
   Form_peach.ComboBoxArchive4.Enabled:=true;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   if Form_peach.ComboBoxArchive1.ItemIndex > 0 then Form_peach.CheckBoxArchive4.Visible:=false
   else Form_peach.CheckBoxArchive4.Visible:=true;
   on_CheckBoxArchive6_click;
   end;
if s=txt_sfx+' '+STR_ARC then
   begin
   Form_peach.PanelARC.Visible:=true;
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   archive_type:='arc';
   fun:='ARC';
   Form_peach.ComboBoxARC.ItemIndex:=level_arc;
   Form_peach.ComboBoxARCsolid.ItemIndex:=solid_arc;
   if rr_arc=1 then Form_peach.CheckBoxARC1.State:=cbChecked
   else Form_peach.CheckBoxARC1.State:=cbUnchecked;
   Form_peach.ComboBoxARCalgo.ItemIndex:=algo_arc;
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_arc;
   Form_peach.Caption:=txt_create+' '+txt_sfx+' ('+STR_ARC+'), '+Form_peach.ComboboxARC2.Text;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   Form_peach.CheckBoxARC2.State:=cbChecked;
   Form_peach.ComboBoxARCsfx.visible:=true;
   end;
if s=txt_split then
   begin
   Form_peach.PanelPea.Visible:=true;
   Form_peach.PanelSplitFile.Visible:=true;
   Form_peach.PanelSplitFile1.Visible:=true;
   Form_peach.PanelSplitFile2.Visible:=true;
   Form_peach.ComboBoxSplit3.ItemIndex:=salgo;
   archive_type:='';
   fun:='RFS';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_split;
   Form_peach.Caption:=txt_split;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   check_tarbefore(tdirs);
   end;
if s=STR_TAR then
   begin
   Form_peach.Panel7za.Visible:=true;
   archive_type:='tar';
   fun:='7Z';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_tar;
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxArchive7.Text;
   compression_method:='';
   compression_level:=txt_level_store;
   set_values(archive_type,compression_method,compression_level);
   Form_peach.CheckBoxArchive6.Visible:=false;
   Form_peach.ComboBoxArchive4.Text:=txt_level_store;
   Form_peach.ComboBoxArchive4.Enabled:=false;
   Form_peach.CheckBoxTarBefore.Enabled:=false;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   if Form_peach.ComboBoxArchive1.ItemIndex > 0 then Form_peach.CheckBoxArchive4.Visible:=false
   else Form_peach.CheckBoxArchive4.Visible:=true;
   end;
if s=STR_UPX then
   begin
   Form_peach.PanelUPX.Visible:=true;
   Form_peach.ComboBoxArchive1.ItemIndex:=0;
   change_imagesplit;
   Form_peach.ComboBoxArchive1.Enabled:=false;
   Form_peach.SpinEditArchive1.Visible:=false;
   Form_peach.ComboBoxArchive2.Visible:=false;
   archive_type:='upx';
   fun:='UPX';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_upx;
   Form_peach.Caption:=txt_compress_executable;
   Form_peach.ComboBoxUPX.ItemIndex:=level_upx;
   Form_peach.CheckBoxTarBefore.Enabled:=false;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=false;
   Form_peach.CheckBoxSeparate.State:=cbUnchecked;
   end;
if s=STR_ZIP then
   begin
   Form_peach.Panel7za.Visible:=true;
   archive_type:='zip';
   fun:='7Z';
   Form_peach.ImageInfoArchive2.Hint:=txt_type_description_zip;
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxArchive7.Text;
   if (compression_method='') or (prev_type<>'zip') then compression_method:=method_zip;//'Deflate';
   setcompressionlevel7z;//set compression_level
   set_values(archive_type,compression_method,compression_level);
   Form_peach.CheckBoxArchive6.Visible:=false;
   Form_peach.ComboBoxArchive4.Text:=txt_level_normal;
   Form_peach.ComboBoxArchive4.Enabled:=true;
   Form_peach.CheckBoxTarBefore.Enabled:=true;
   Form_peach.CheckBoxTarBefore.State:=cbUnchecked;
   Form_peach.CheckBoxSeparate.Enabled:=true;
   if Form_peach.ComboBoxArchive1.ItemIndex > 0 then Form_peach.CheckBoxArchive4.Visible:=false
   else Form_peach.CheckBoxArchive4.Visible:=true;
   end;
if (Form_peach.CheckBoxArchive6.Visible) and (Form_peach.CheckBoxArchive6.State=cbChecked) then Form_peach.RadioGroupArchive.Visible:=true
else Form_peach.RadioGroupArchive.Visible:=false;
fun_status:=fun;
//set combo box index
for i:=0 to Form_peach.cbType.Items.Count-1 do
   if Form_peach.cbType.Items[i]=s then Form_peach.cbType.ItemIndex:=i;
archive_type_selected:=Form_peach.cbType.ItemIndex;
setcompressionlevel7zindex; //otherwise combobox itemindex don't gets updated
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
setpwicons;
end;

procedure testarctype(s:utf8string; var i:integer; var fun:utf8string);
begin
{$IFDEF MSWINDOWS}
if s=txt_mypc then
   begin
   i:=1000;
   fun:='FILEBROWSER';
   fun_status:=fun;
   exit;
   end;
{$ENDIF}
i:=testinput(s,true);
case i of
   0: fun:='UNCUSTOM';
   1: fun:='UNPEA';
   2: fun:='RFJ';
   3: fun:='UN7Z';
   4: fun:='UN7Z';
   5: fun:='UNPAQ';
   6: fun:='UNQUAD';
   7: fun:='UNACE';
   8: fun:='UNLPAQ';
   9: fun:='UNARC';
   10: fun:='UNZPAQ';
   1000: fun:='FILEBROWSER';
   end;
fun_status:=fun;
end;

procedure set_lastobjarch;
var
   i:integer;
   lastobjfound:boolean;
begin
lastobjfound:=false;
if length(lastobjarch)>1 then
   if lastobjarch[length(lastobjarch)]=directoryseparator then setlength(lastobjarch,length(lastobjarch)-1);
if lastobjarch<>'' then
   for i:=1 to Form_peach.StringGridList.RowCount-1 do
      if Form_Peach.StringGridList.Cells[8,i]=lastobjarch then
         begin
         lastobjfound:=true;
         break;
         end;
if lastobjfound=false then i:=1;
Form_peach.StringGridList.Cells[12,i]:='1';
Form_peach.StringGridList.Row:=i;
end;

procedure listsizecol(mode:utf8string);
begin
   Form_peach.StringGridList.AutoSizeColumns;
   Form_peach.StringGridList.ColWidths[0]:=ROWSIZE;
   Form_peach.StringGridList.ColWidths[1]:=Form_peach.StringGridList.ColWidths[1]+48;
   if Form_peach.StringGridList.ColWidths[1]>350 then Form_peach.StringGridList.ColWidths[1]:=350;
   if mode='pc' then Form_peach.StringGridList.ColWidths[2]:=Form_peach.StringGridList.ColWidths[2]+48;
   if mode='browse' then Form_peach.StringGridList.ColWidths[8]:=0;
   if mode='pc' then Form_peach.StringGridList.ColWidths[8]:=0;
   if mode='pc' then Form_peach.StringGridList.ColWidths[6]:=0;
   if mode='pc' then Form_peach.StringGridList.ColWidths[7]:=0;
   Form_peach.StringGridList.ColWidths[9]:=0;
   Form_peach.StringGridList.ColWidths[10]:=0;
   Form_peach.StringGridList.ColWidths[11]:=0;
   Form_peach.StringGridList.ColWidths[12]:=0;
   Form_peach.StringGridList.colwidths[13]:=8;
end;

procedure list_l(mode:utf8string); //special formats
var
   i,j,k:integer;
   size,csize:qword;
begin
if (mode='browse') or (mode='flat') then //not used in silent mode
   begin
   Form_peach.StringGridList.RowCount:=Form_peach.MemoList.Lines.Count-title_lines_7z-1;
   for i:=title_lines_7z to Form_peach.MemoList.Lines.Count-3 do
      begin
      k:=i-title_lines_7z+1;
      Form_peach.StringGridList.Cells[1,k]:=uextractfilename(copy(Form_peach.MemoList.Lines[i],54,length(Form_peach.MemoList.Lines[i])-53));
      if copy(Form_peach.MemoList.Lines[i],21,1)='D' then
         begin
         Form_peach.StringGridList.Cells[2,k]:=txt_list_isfolder;
         Form_peach.StringGridList.Cells[8,k]:=copy(Form_peach.MemoList.Lines[i],54,length(Form_peach.MemoList.Lines[i])-53);
         Form_peach.StringGridList.Cells[9,k]:='1'+Form_peach.StringGridList.Cells[1,k];
         end
      else
         begin
         Form_peach.StringGridList.Cells[2,k]:=uextractfileext(Form_peach.StringGridList.Cells[1,k]);
         Form_peach.StringGridList.Cells[8,k]:=copy(Form_peach.MemoList.Lines[i],54,length(Form_peach.MemoList.Lines[i])-53);
         Form_peach.StringGridList.Cells[9,k]:='2'+Form_peach.StringGridList.Cells[1,k];
         end;
      Form_peach.StringGridList.Cells[3,k]:=trimleft(copy(Form_peach.MemoList.Lines[i],27,12));
      Form_peach.StringGridList.Cells[10,k]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,k]))))+inttostr(length(Form_peach.StringGridList.Cells[3,k]))+Form_peach.StringGridList.Cells[3,k];
      Form_peach.StringGridList.Cells[4,k]:=trimleft(copy(Form_peach.MemoList.Lines[i],40,12));
      Form_peach.StringGridList.Cells[11,k]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[4,k]))))+inttostr(length(Form_peach.StringGridList.Cells[4,k]))+Form_peach.StringGridList.Cells[4,k];
      Form_peach.StringGridList.Cells[5,k]:=copy(Form_peach.MemoList.Lines[i],1,19);
      Form_peach.StringGridList.Cells[6,k]:=copy(Form_peach.MemoList.Lines[i],21,5);
      Form_peach.StringGridList.Cells[12,k]:='0';
      end;
   Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
   Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
   listsizecol(mode);
   if Form_peach.StringGridList.RowCount>1 then
      begin
      Form_peach.StringGridList.SortColRow(true,listsortcol);
      //Form_peach.StringGridList.Cells[12,1]:='1';
      //Form_peach.StringGridList.Row:=1;
      end;
   end;
i:=Form_peach.MemoList.Lines.Count-3;
status1:=copy(Form_peach.MemoList.Lines[i+2],54,length(Form_peach.MemoList.Lines[i+2])-53)+'; '+trimleft(copy(Form_peach.MemoList.Lines[i+2],27,12))+' B > '+trimleft(copy(Form_peach.MemoList.Lines[i+2],40,12))+' B';
size:=0;
csize:=0;
j:=0;
try
   size:=strtoqword(trimleft(copy(Form_peach.MemoList.Lines[i+2],27,12)));
   csize:=strtoqword(trimleft(copy(Form_peach.MemoList.Lines[i+2],40,12)));
   j:=((csize div 1024) *100) div (size div 1024);
   if (j<101) and (j>1) then Form6.ShapeIO7.Width:=200 * j div 100
   else Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:=inttostr(j)+'%';
except
   Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:='??%';
end;
if (j>1) and (j<200) then status1:=status1+' ('+inttostr(j)+'%)';
if Form_peach.StringGridList.RowCount<2 then status1:=txt_list_nomatch+txt_2_7_list_tryflatorpw;
end;

function identify_field(s:utf8string):utf8string;
begin
if pos(' = ', s)<>0 then identify_field:=copy(s,1,pos(' = ', s)-1)
else identify_field:='new_record';
end;

procedure tempcharcodefix(var s:utf8string);
//optionally replace characters over ASCII 126 (~) with ? wildcard to avoid issues in translation of extended characters;
//otherwise, displays characters as utf-8; on Windows characters comes encoded in console's codepage (to memory stream reading backend stdout)
var
   i:integer;
begin
if archivenameenc=0 then
   for i:=1 to length(s) do
      begin
      if comparestr(s[i],'~')>0 then s[i]:='?';
      end
else
   begin
   {$IFDEF MSWINDOWS}
   convertwincp(s); //WideCharToMultiByte seem not working properly at least in case of conversion to UTF-8 on pre-Vista systems
   {$ENDIF}
   end;
end;

procedure setfocus_stringgridlist;
begin
if waitdrawok=true then
try
if Form_peach.StringGridList.Visible=true then Form_peach.StringGridList.Setfocus;
except
end;
end;

procedure setfocus_pw;
begin
if seemencrypted=true then
   begin
   status2:=currentcomp+', '+txt_encrypted;
   end;
end;

procedure find_7z_titles;
var
   i:integer;
   s1:utf8string;
begin
i:=0;
repeat
   s1:=Form_peach.MemoList.Lines[i];
   i:=i+1;
until s1=ZSTRING;
title_lines_7z:=i+1; //dinamically find title and comment lines
end;

procedure find_7zl_titles;
var
   i:integer;
   s1:utf8string;
begin
i:=0;
repeat
   s1:=Form_peach.MemoList.Lines[i];
   i:=i+1;
until s1=ZSTRINGL;
title_lines_7z:=i; //dinamically find title and comment lines
end;

procedure list_slt(mode:utf8string);
var
   i,j:integer;
   size,csize:qword;
   s1,s2,s3:utf8string;
   stopped_prebrowse:boolean;
begin
i:=title_lines_7z-2;
j:=1;
size:=0;
csize:=0;
stopped_prebrowse:=false;
Form_peach.StringGridList.RowCount:=1;
status0:=txt_list_searching;
status1:=txt_please_wait;
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.visible=true then Application.Processmessages;
if Form_peach.MemoList.Lines.Count>4096 then Form_peach.StringGridList.Visible:=false; //speed up the procedure cutting the redraw time if many lines has to be displayed
if (mode='browse') or (mode='flat') then
else Form_peach.StringGridList.Visible:=false;
Form_peach.StringGridList.RowCount:=1024;
repeat
   if j<1022 then
   else
      //if (j and 2)=0 then
      Form_peach.StringGridList.RowCount:=Form_peach.StringGridList.RowCount+1;
   repeat
      s1:=Form_peach.MemoList.Lines[i];
      s2:=identify_field(s1);
      if s2='Path' then
         begin
         s3:=copy(s1,8,length(s1)-7);
         tempcharcodefix(s3);
         Form_peach.StringGridList.Cells[1,j]:=uextractfilename(s3);
         Form_peach.StringGridList.Cells[8,j]:=s3;
         end;
      if s2='Folder' then
         if copy(s1,length(s1),1)=ZCHAR then
            begin
            Form_peach.StringGridList.Cells[2,j]:=txt_list_isfolder;
            Form_peach.StringGridList.Cells[9,j]:='1'+uextractfilename(s3);
            end
         else
            begin
            Form_peach.StringGridList.Cells[2,j]:=uextractfileext(s3);
            Form_peach.StringGridList.Cells[9,j]:='2'+uextractfilename(s3);
            end;
      if s2='Size' then
         begin
         Form_peach.StringGridList.Cells[3,j]:=copy(s1,8,length(s1)-7);
         Form_peach.StringGridList.Cells[10,j]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,j]))))+inttostr(length(Form_peach.StringGridList.Cells[3,j]))+Form_peach.StringGridList.Cells[3,j];
         try
            size:=size+strtoqword(Form_peach.StringGridList.Cells[3,j]);
         except
         end;
         end;
      if s2='Packed Size' then
         begin
         Form_peach.StringGridList.Cells[4,j]:=copy(s1,15,length(s1)-14);
         Form_peach.StringGridList.Cells[11,j]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,j]))))+inttostr(length(Form_peach.StringGridList.Cells[4,j]))+Form_peach.StringGridList.Cells[4,j];
         try
            csize:=csize+strtoqword(Form_peach.StringGridList.Cells[4,j]);
         except
         end;
         end;
      if s2='Modified' then Form_peach.StringGridList.Cells[5,j]:=copy(s1,12,length(s1)-11);
      if s2='CRC' then Form_peach.StringGridList.Cells[7,j]:=copy(s1,7,length(s1)-6);
      if s2='Attributes' then
         begin
         Form_peach.StringGridList.Cells[6,j]:=copy(s1,14,length(s1)-13);
         if copy(Form_peach.StringGridList.Cells[6,j],1,1)='D' then
            begin
            Form_peach.StringGridList.Cells[2,j]:=txt_list_isfolder;
            Form_peach.StringGridList.Cells[9,j]:='1'+uextractfilename(s3);
            end
         else
            begin
            Form_peach.StringGridList.Cells[2,j]:=uextractfileext(s3);
            Form_peach.StringGridList.Cells[9,j]:='2'+uextractfilename(s3);
            end;
         end;
      if s2='Encrypted' then
         if copy(s1,length(s1),1)=ZCHAR then
            begin
            Form_peach.StringGridList.Cells[1,j]:=uextractfilename(s3)+' *';
            seemencrypted:=true;
            setfocus_pw;
            end;
      if s2='Method' then
         if pos('AES',s1)<>0 then
            begin
            Form_peach.StringGridList.Cells[1,j]:=uextractfilename(s3)+' *';
            seemencrypted:=true;
            setfocus_pw;
            end;
      i:=i+1;
   until s2='new_record';
   if Form_peach.StringGridList.Cells[1,j]<>'' then
      begin
      Form_peach.StringGridList.Cells[12,j]:='0';
      j:=j+1;
      end
   else Form_peach.StringGridList.RowCount:=Form_peach.StringGridList.RowCount-1;
   if Form_peach.Visible=true then Application.ProcessMessages;
until i>=Form_peach.MemoList.Lines.Count-1;
Form_peach.StringGridList.RowCount:=j;
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
listsizecol(mode);
j:=j-1;
if stopped_prebrowse=true then
   begin
   j:=0;
   size:=0;
   csize:=0;
   Form_peach.StringGridList.RowCount:=Form_peach.StringGridList.RowCount-1;
   end
else status1:=inttostr(j)+' '+txt_displayed_obj+'; '+inttostr(size)+' B > '+inttostr(csize)+' B';
try
   j:=((csize div 1024) *100) div (size div 1024);
   if (j<101) and (j>1) then Form6.ShapeIO7.Width:=200 * j div 100
   else Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:=inttostr(j)+'%';
except
   Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:='??%';
end;
if (j>1) and (j<200) then status1:=status1+' ('+inttostr(j)+'%)';
status0:=txt_list_sorting;
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.visible=true then Application.Processmessages;
if Form_peach.StringGridList.RowCount>1 then
else
   begin
   //status1:=txt_list_nomatch+txt_2_7_list_tryflatorpw;
   //if mode='silent' then seemencrypted:=true;
   end;
if (mode='browse') or (mode='flat') then
   begin
   if Form_peach.StringGridList.RowCount>1 then Form_peach.StringGridList.SortColRow(true,listsortcol);
   Form_peach.StringGridList.Visible:=true;
   end
else Form_peach.StringGridList.Visible:=false;
end;

procedure teststatus(var i:integer; var s:utf8string);
begin
i:=0;
try
if Form_peach.StringGridList.Cells[1,1]='' then i:=1;
except
i:=1;
end;
if i=0 then s:=txt_un7z_browse_ok//ok
else //error or no match
   begin
   s:=txt_un7z_browse_failure;
   {if status0=txt_list_browsing then s:=s+', '+txt_2_7_un7z_browse_flat;
   if seemencrypted=true then
      begin
      if Form2.EditUn7zaPW.Text='' then s:=s+', '+txt_un7z_browse_pw
      else s:=s+', '+txt_un7z_browse_pw_other;
      seemnamesencrypted:=true;
      end;}
   end;
Form6.LabelUn7zInputStatus1.Caption:=status2+'; '+txt_status+': '+s;
Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus1.Caption;
end;

procedure animation_busy(var pcount:integer);
begin
case pcount of
   1: begin Form_peach.ctrlrefresh.Glyph:=Bp2; pcount:=2; end;
   2: begin Form_peach.ctrlrefresh.Glyph:=Bp3; pcount:=3; end;
   3: begin Form_peach.ctrlrefresh.Glyph:=Bp4; pcount:=4; end;
   4: begin Form_peach.ctrlrefresh.Glyph:=Bp5; pcount:=5; end;
   5: begin Form_peach.ctrlrefresh.Glyph:=Bp6; pcount:=6; end;
   6: begin Form_peach.ctrlrefresh.Glyph:=Bp7; pcount:=7; end;
   7: begin Form_peach.ctrlrefresh.Glyph:=Bp8; pcount:=8; end;
   8: begin Form_peach.ctrlrefresh.Glyph:=Bp1; pcount:=1; end;
   end;
end;

function execute_cl(cl,mode:utf8string):integer;
var
  P: TProcess;
  i,BytesRead,pcount:integer;
  skipped_prebrowse:boolean;
  M:TmemoryStream;
begin
execute_cl:=-1;
if cl='' then
   begin
   MessageDlg(txt_error_emptycl, mtError, [mbOK], 0);
   Application.Terminate;
   end;
Form_peach.StringGridList.Cursor:=crHourGlass;
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.Visible=true then Application.ProcessMessages;
skipped_prebrowse:=false;
pcount:=1;
i:=0;
P:=TProcess.Create(nil);
cl:=utf8toenv(cl); //on Windows it is passed to command line as ansi
P.CommandLine:=cl;
M := TMemoryStream.Create;
BytesRead := 0;
if (mode='extandrun') or (mode='delete') or (mode='add')  or (mode='preview') then P.Options := [poNoConsole]
else P.Options := [poUsePipes, poNoConsole];
if fun='UNARC' then P.Options := [poUsePipes, poNoConsole];
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
while P.Running do
   begin
   animation_busy(pcount);
   M.SetSize(BytesRead + MAX_CL);
   if prebrowse=1 then
      begin
      if M.Size>(prebrowsesize*1024*1024) then //limit memorystream size to 1M*prebrowsesize blocks
         begin
         if mode<>'silent' then MessageDlg(txt_aborted_error, mtError, [mbOK], 0);
         status0:=txt_aborted;
         skipped_prebrowse:=true;
         P.Terminate(255);
         break;
         end;
      end;
   if (mode<>'extandrun') and (mode<>'delete') and (mode<>'add')  and (mode<>'preview') then i := P.Output.Read((M.Memory + BytesRead)^, MAX_CL);
   if i > 0 then Inc(BytesRead, i)
   else Sleep(100);
   if Form_peach.Visible=true then Application.ProcessMessages;
   end;
if (mode<>'extandrun') and (mode<>'delete') and (mode<>'add')  and (mode<>'preview') then
   repeat
      M.SetSize(BytesRead + MAX_CL);
      i := P.Output.Read((M.Memory + BytesRead)^, MAX_CL);
      if i > 0 then Inc(BytesRead, i)
      else Sleep(100);
   until i <= 0;
M.SetSize(BytesRead);
Form_peach.StringGridList.Cursor:=crDefault;
execute_cl:=P.ExitStatus;
P.Free;
if ugetcurrentdir<>executable_path then usetcurrentdir((executable_path));
if mode='preview' then FileSetAttr(ptmpdir, faHidden);
if (mode='extandrun') or (mode='delete') or (mode='add')  or (mode='preview') then
   begin
   Form_peach.ctrlrefresh.Glyph:=Brefresh;
   exit;
   end;
Form_peach.StringGridList.RowCount:=1;
Form_peach.MemoList.Clear;
if execute_cl=0 then
   begin
   Form_peach.MemoList.Lines.LoadFromStream(M);
   M.Free;
   if browse_option='' then
      begin
      Form_peach.ctrlrefresh.Glyph:=Brefresh;
      exit; //if the program is browsing a format not supported through 7z, using its browse_$format method
      end;
   if browse_option='slt' then find_7z_titles
   else
      find_7zl_titles;
   i:=title_lines_7z+1;
   if Form_peach.MemoList.Lines.Count<i then //no matches
      begin
      status1:=txt_list_nomatch;
      if mode='silent' then
         begin
         //seemencrypted:=true;
         //setfocus_pw;
         status1:=txt_list_nomatch+txt_2_7_list_tryflatorpw;
         end
      else
         begin
         if Form_peach.EditUn7zaFilter.Text='*' then
            status1:=txt_list_nomatch+txt_2_7_list_tryflatorpw;
         if mode='browse' then lastobjarch:=uextractfilepath(Form_peach.EditUn7zaFilter.Text);
         end;
      Form_peach.StringGridList.RowCount:=2;
      Form_peach.StringGridList.Visible:=True;
      Form_peach.ctrlrefresh.Glyph:=Brefresh;
      exit;
      end
   else //matching objects found
      begin
      end;
   if (mode='silent') and (skipped_prebrowse=true) then exit;
   if browse_option='slt' then list_slt(mode)
   else list_l(mode);
   if mode='browse' then
      begin
      set_lastobjarch;
      lastobjarch:=uextractfilepath(Form_peach.EditUn7zaFilter.Text);
      end;
   if mode='flat' then
      begin
      Form_peach.StringGridList.Cells[12,1]:='1';
      Form_peach.StringGridList.Row:=1;
      end;
   end
else //error
   begin
   Form_peach.StringGridList.RowCount:=2;
   status1:=txt_list_nomatch+txt_2_7_list_tryflatorpw;
   if mode='silent' then
      begin
      status1:=txt_list_nomatch;
      if skipped_prebrowse=false then
         begin
         seemencrypted:=true;
         setfocus_pw;
         end;
      end;
   end;
Form_peach.ctrlrefresh.Glyph:=Brefresh;
end;

function openw_obj(outname:utf8string):integer;
var
   s:utf8string;
   woutname,wtempstring:widestring;
begin
openw_obj:=-1;
{$IFDEF MSWINDOWS}
s:=outname+tempstring;
if checkfiledirname(s)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+s, mtWarning, [mbOK], 0); exit; end;
woutname:=utf8decode(outname);
wtempstring:=utf8decode(tempstring);
if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); exit; end;
shellexecutew(Form_peach.handle, PWideChar('open'), PWideChar('RUNDLL32.EXE'), PWideChar('shell32.dll,OpenAs_RunDLL '+woutname+wtempstring), PWideChar (''), SW_SHOWNORMAL);
openw_obj:=0;
{$ENDIF}
end;

//open, cross platform, with sanitization of string passed to the function
function cp_open(s:utf8string; desk_env:byte):integer;
var
   w:widestring;
   //P:TProcess;
begin
cp_open:=-1;
if s='' then exit;
if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); exit; end;
{$IFDEF MSWINDOWS}
w:=utf8decode(s);
cp_open:=ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar(w), PWideChar (''), PWideChar (''), SW_SHOWNORMAL);
if cp_open<33 then
   cp_open:=shellexecuteW(Form_peach.handle,PWideChar('open'),PWideChar('RUNDLL32.EXE'),PWideChar('shell32.dll,OpenAs_RunDLL '+w),PWideChar (''), SW_SHOWNORMAL);
{P:=TProcess.Create(nil);
P.Options := [poNoConsole];
if w[1]=utf8decode(delimiter) then
else w:=utf8encode(delimiter)+w+utf8encode(delimiter);
if winver<>'9x' then P.CommandLine:=wincomspec+' /c start "" '+w
else P.CommandLine:=wincomspec+' /c start '+w;
if Form_peach.Visible=true then Application.ProcessMessages;
P.Execute;
P.Free;
cp_open:=0;}
{$ENDIF}
{$IFDEF LINUX}cp_open:=cp_open_linuxlike(s,desk_env);{$ENDIF}//try to open via Gnome or KDE
{$IFDEF FREEBSD}cp_open:=cp_open_linuxlike(s,desk_env);{$ENDIF}
{$IFDEF NETBSD}cp_open:=cp_open_linuxlike(s,desk_env);{$ENDIF}
end;

procedure cp_search(desk_env:byte);
begin
{$IFDEF MSWINDOWS}
cp_open(executable_path+'res'+directoryseparator+'empty.fnd',desk_env);
{$ENDIF}
{$IFDEF LINUX}cp_search_linuxlike(desk_env);{$ENDIF}//try to search via Gnome or KDE
{$IFDEF FREEBSD}cp_search_linuxlike(desk_env);{$ENDIF}
{$IFDEF NETBSD}cp_search_linuxlike(desk_env);{$ENDIF}
end;

function execute_obj(efun:byte; outname:utf8string):integer;
var
  i:integer;
  P: TProcess;
  cl,bin_name,s:utf8string;
  woutname,wtempstring:widestring;
begin
execute_obj:=-1;
if checkfiledirname(outname)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+outname, mtWarning, [mbOK], 0); exit; end;
if (fun='UNARC') or (fun='UN7Z') then
   if outname<>'' then if outname[length(outname)]<>directoryseparator then outname:=outname+directoryseparator;
case efun of
   {0: //(extract the object)
      begin
      execute_obj:=0;
      tempstring:='';
      end;
   1: //(extract the object and) try to open output path
      begin
      if cp_open(outname,desk_env)<>0 then exit;
      execute_obj:=0;
      tempstring:='';
      end;}
   2: //(extract the object and) open it with PeaZip
      begin
      udodirseparators(tempstring);
      i:=testinput(outname+tempstring,true);
      //if ((i>=0) and (i<>1000)) then
      if i>=0 then
         begin
         P:=TProcess.Create(nil);
         udodirseparators(tempstring);
         outname:=delimiter+escapefilename(outname+tempstring,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'peazip'+EXEEXT+delimiter+' -ext2open';
         {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
         cl:=bin_name+' '+outname;
         cl:=utf8toenv(cl);
         P.CommandLine:=cl;
         if Form_peach.Visible=true then Application.ProcessMessages;
         if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
         P.Execute;
         execute_obj:=P.ExitStatus;
         P.Free;
         end;
      execute_obj:=0;
      tempstring:='';
      end;
   3: //(extract the object and) open with PeaZip if it's a supported filetype (excluding custom formats, packed executables, MS Office and Open Office formats), otherwise open it with the associated application
      begin
      udodirseparators(tempstring);
      i:=testext(outname+tempstring);
      if (i<0) //not a PeaZip supported filetype
         or (i=503)
         or (i=502)
         or (i=501) //file types usually not handled as archives
      then
         begin
         udodirseparators(tempstring);
         i:=cp_open(outname+tempstring,desk_env);
         end
      else //open in PeaZip
         begin
         P:=TProcess.Create(nil);
         {$IFDEF MSWINDOWS}//if it's a Windows executable, offer the possibility to run it instead of open it in PeaZip (Windows only)
         if i=500 then
            begin
            udodirseparators(tempstring);
            if MessageDlg(txt_dis+' '+tempstring+' '+txt_type_exe, mtWarning,[mbYes, mbNo],0)=6 then
               begin
               P.Free;
               udodirseparators(tempstring);
               woutname:=utf8decode(outname);
               wtempstring:=utf8decode(tempstring);
               s:=outname+tempstring;
               if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); exit; end;
               ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar(woutname+wtempstring), PWideChar (''), PWideChar (''), SW_SHOWNORMAL);
               execute_obj:=0;
               tempstring:='';
               exit;
               end;
            end;
         {$ENDIF}
         udodirseparators(tempstring);
         outname:=delimiter+escapefilename(outname+tempstring,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'peazip'+EXEEXT+delimiter;
         {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
         cl:=bin_name+' -ext2open '+outname;
         cl:=utf8toenv(cl);
         P.CommandLine:=cl;
         if Form_peach.Visible=true then Application.ProcessMessages;
         if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
         P.Execute;
         execute_obj:=P.ExitStatus;
         P.Free;
         end;
      execute_obj:=0;
      tempstring:='';
      end;
   else //(extract the object and) open it with the associated application
      begin
      udodirseparators(tempstring);
      i:=cp_open(outname+tempstring,desk_env);
      execute_obj:=0;
      tempstring:='';
      end;
   end;
end;

function extract_excludes:utf8string;
var
   i:integer;
   s:utf8string;
begin
extract_excludes:='';
for i:=0 to Form7.MemoAdvExclude.Lines.Count-1 do
   if Form7.MemoAdvExclude.Lines[i]<>'' then
      begin
      s:=Form7.MemoAdvExclude.Lines[i];
      if copy(s,1,1)='.' then s:='*'+copy(s,2,length(s)-1);
      if Form7.CheckBoxAdvRecurse1.State=cbChecked then extract_excludes:=extract_excludes+' '+delimiter+'-xr!'+escapefilename(s,desk_env)+delimiter
      else extract_excludes:=extract_excludes+' '+delimiter+'-x!'+escapefilename(s,desk_env)+delimiter;
      end;
end;

function extract_includes:utf8string;
var
   i:integer;
   s:utf8string;
begin
extract_includes:='';
for i:=0 to Form7.MemoAdvInclude.Lines.Count-1 do
   if Form7.MemoAdvInclude.Lines[i]<>'' then
      begin
      s:=Form7.MemoAdvInclude.Lines[i];
      if copy(s,1,1)='.' then s:='*'+copy(s,2,length(s)-1);
      extract_includes:=extract_includes+' '+delimiter+escapefilename(s,desk_env)+delimiter;
      end;
end;

procedure adv_filters(var cl:utf8string);
begin
if Form7.CheckBoxAdvFilters.State=cbChecked then
   begin
   if extract_excludes<>'' then cl:=cl+extract_excludes;
   if Form7.CheckBoxAdvRecurse.State=cbChecked then cl:=cl+' -r'
   else cl:=cl+' -r-';
   if extract_includes<>'' then cl:=cl+' -- '+extract_includes
   else cl:=cl+' -- *';
   end;
end;

procedure basic_filters(var cl:utf8string);
var
   s1,s2:utf8string;
begin
s1:=Form_peach.EditUn7zaFilterExclude.Text;
if copy(s1,1,1)='.' then s1:='*'+copy(s1,2,length(s1)-1); //to allowing using paths beginning with "." that would otherwise result in 7z error "cannot use absolute path"
s2:=Form_peach.EditUn7zaFilter.Text;
if copy(s2,1,1)='.' then s2:='*'+copy(s2,2,length(s2)-1);
if Form_peach.EditUn7zaFilterExclude.Text<>'' then cl:=cl+' '+delimiter+'-x!'+escapefilename(s1,desk_env)+delimiter;
if Form_peach.EditUn7zaFilter.Text<>'' then cl:=cl+' -r -- '+delimiter+escapefilename(s2,desk_env)+delimiter;
end;

function list_un7z(mode:utf8string):integer;
var
   cl,in_param,bin_name,exe_name:utf8string;
begin
in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
if fun='UN7Z' then exe_name:='7z';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+exe_name+DirectorySeparator+exe_name+EXEEXT+delimiter;
pw:=Form2.EditUn7zaPW.Text;
if Form2.EditName3.Text<>'' then
   if prepend_keyfile(pw,Form2.EditName3.Text)<>0 then
      begin
      MessageDlg(Form2.EditName3.Text+' '+txt_keyfile_not_found, mtError,[mbOk],0);
      exit;
      end;
if pw<>'' then
   begin
   if pw4cl('-p',pw)<>0 then
      begin
      msg_pw_errorchar;
      exit;
      end;
   end
else pw:='-pdefault';
cl:=bin_name+' l';
if fun='UN7Z' then
   if browse_option='slt' then cl:=cl+' -slt';
cl:=cl+' '+pw+' '+in_param;
//filter(s)
if Form7.CheckBoxAdvFilters.State=cbChecked then //use advanced filters, ignore basic filters
   adv_filters(cl)
else //use basic filters (from browser)
   basic_filters(cl);
list_un7z:=execute_cl(cl,mode);
end;

procedure addtohistorypopup;
var
   rc:integer;
begin
with form_peach do
   begin
   rc:=StringGridHistory.RowCount;
   if rc>1 then pmh1.caption:=StringGridHistory.Cells[2,rc-1]+' '+StringGridHistory.Cells[3,rc-1] else pmh1.caption:='';
   if rc>2 then pmh2.caption:=StringGridHistory.Cells[2,rc-2]+' '+StringGridHistory.Cells[3,rc-2] else pmh2.caption:='';
   if rc>3 then pmh3.caption:=StringGridHistory.Cells[2,rc-3]+' '+StringGridHistory.Cells[3,rc-3] else pmh3.caption:='';
   if rc>4 then pmh4.caption:=StringGridHistory.Cells[2,rc-4]+' '+StringGridHistory.Cells[3,rc-4] else pmh4.caption:='';
   if rc>5 then pmh5.caption:=StringGridHistory.Cells[2,rc-5]+' '+StringGridHistory.Cells[3,rc-5] else pmh5.caption:='';
   if rc>6 then pmh6.caption:=StringGridHistory.Cells[2,rc-6]+' '+StringGridHistory.Cells[3,rc-6] else pmh6.caption:='';
   if rc>7 then pmh7.caption:=StringGridHistory.Cells[2,rc-7]+' '+StringGridHistory.Cells[3,rc-7] else pmh7.caption:='';
   if rc>8 then pmh8.caption:=StringGridHistory.Cells[2,rc-8]+' '+StringGridHistory.Cells[3,rc-8] else pmh8.caption:='';
   if pmh8.caption='' then pmh8.visible:=false else pmh8.visible:=true;
   if pmh7.caption='' then pmh7.visible:=false else pmh7.visible:=true;
   if pmh6.caption='' then pmh6.visible:=false else pmh6.visible:=true;
   if pmh5.caption='' then pmh5.visible:=false else pmh5.visible:=true;
   if pmh4.caption='' then pmh4.visible:=false else pmh4.visible:=true;
   if pmh3.caption='' then pmh3.visible:=false else pmh3.visible:=true;
   if pmh2.caption='' then pmh2.visible:=false else pmh2.visible:=true;
   end;
end;

procedure addtohistory;
var
   rc:integer;
begin
{$IFDEF MSWINDOWS}
{$ELSE} //In Lazarus 0.9.24 under GTK and GTK2 the panel need to be visible while working on its objects; before, the panel is resized in order to reduce "flashing" effect
{$ENDIF}
Form_peach.StringGridHistory.RowCount:=Form_peach.StringGridHistory.Row+1;
if fun='UN7Z' then
   begin
   if (Form_peach.EditUn7zaFilter.Text=Form_peach.StringGridHistory.Cells[1,Form_peach.StringGridHistory.RowCount-1]) and (Form_peach.EditUn7zaFilterExclude.Text=Form_peach.StringGridHistory.Cells[1,Form_peach.StringGridHistory.RowCount-1]) then exit;
   rc:=Form_peach.StringGridHistory.Rowcount+1;
   Form_peach.StringGridHistory.RowCount:=rc;
   Form_peach.StringGridHistory.Cells[1,rc-1]:=status0;
   Form_peach.StringGridHistory.Cells[2,rc-1]:=Form_peach.EditOpenIn.Text;
   Form_peach.StringGridHistory.Cells[3,rc-1]:=Form_peach.EditUn7zaFilter.Text;
   Form_peach.StringGridHistory.Cells[4,rc-1]:=Form_peach.EditUn7zaFilterExclude.Text;
   end
else
   begin
   rc:=Form_peach.StringGridHistory.Rowcount+1;
   Form_peach.StringGridHistory.RowCount:=rc;
   Form_peach.StringGridHistory.Cells[1,rc-1]:=status0;
   Form_peach.StringGridHistory.Cells[2,rc-1]:=Form_peach.EditOpenIn.Text;
   Form_peach.StringGridHistory.Cells[3,rc-1]:=Form_peach.EditUn7zaFilter.Text;
   Form_peach.StringGridHistory.Cells[4,rc-1]:='';
   end;
Form_peach.StringGridHistory.AutosizeColumns;
Form_peach.StringGridHistory.ColWidths[0]:=ROWSIZE;
if Form_peach.StringGridHistory.Width<250 then Form_peach.StringGridHistory.Width:=250;
Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-1;
addtohistorypopup;
{$IFDEF MSWINDOWS}
{$ELSE}
{$ENDIF}
end;

function drawbookmarkicon(arow:integer):TBitmap;
var
   s,s1:utf8string;
   i:integer;
begin
with Form_peach do
begin
if StringGridBookmarks.Cells[1,arow] = txt_list_found then
   begin
   result:=Bsearch;
   exit;
   end;
if StringGridBookmarks.Cells[1,arow] = txt_list_flat then
   begin
   result:=Bexpand;
   exit;
   end;
s:=StringGridBookmarks.Cells[2,arow];
s1:=StringGridBookmarks.Cells[3,arow];
if s1<>'' then s1:=s1[length(s1)];
{$IFDEF MSWINDOWS}
if s = txt_mypc then
   begin
   result:=Bsystemtools;
   exit;
   end;
if length(s) = 3 then
   begin
   i:=ord(s[1])-64;
   if s1='*' then result:=devicon[i]
   else result:=Bgopath;
   exit;
   end;
{$ENDIF}
if s<>'' then
   if s[length(s)] = directoryseparator then
      begin
      if s1='*' then result:=BFolder
      else result:=Bgopath;
      exit;
      end;
if uextractfileext(s) = '.lnk' then
   begin
   result:=BLink;
   exit;
   end;
if s1='*' then
   if (testinput(s,false)=0) then result:=BUnsupported //unsupported file type
   else
      begin
      case testext(s) of
         503: result:=BFileSupported;
         502: result:=BFileSupported;
         501: result:=BFileSupported; //non-archives
         500: result:=BExeSupported; //Windows executables, installers
         else
            result:=BArchiveSupported;
         end;
      end
else result:=Bgopath;
end;
end;

function drawbookmarkicon2(arow:integer):TBitmap;
var
   s:utf8string;
   i:integer;
begin
with Form_peach do
begin
result:=Bfolder;
s:=StringGridBookmarks.Cells[2,arow];
{$IFDEF MSWINDOWS}
if length(s) = 3 then
   begin
   i:=ord(s[1])-64;
   result:=devicon[i];
   exit;
   end;
{$ENDIF}
end;
end;

procedure populatebookmarks;
var
   i,j:integer;
   s,s1:utf8string;
begin
with Form_peach do
begin
mb1.visible:=false;
mb2.visible:=false;
mb3.visible:=false;
mb4.visible:=false;
mb5.visible:=false;
mb6.visible:=false;
mb7.visible:=false;
mb8.visible:=false;
mb9.visible:=false;
mb10.visible:=false;
mb11.visible:=false;
mb12.visible:=false;
mb13.visible:=false;
mb14.visible:=false;
mb15.visible:=false;
mb16.visible:=false;
mb17.visible:=false;
mb18.visible:=false;
mb19.visible:=false;
mb20.visible:=false;
mb21.visible:=false;
mb22.visible:=false;
mb23.visible:=false;
pomb1.visible:=false;
pomb2.visible:=false;
pomb3.visible:=false;
pomb4.visible:=false;
pomb5.visible:=false;
pomb6.visible:=false;
pomb7.visible:=false;
pomb8.visible:=false;
pomb9.visible:=false;
pomb10.visible:=false;
pomb11.visible:=false;
pomb12.visible:=false;
pomb13.visible:=false;
pomb14.visible:=false;
pomb15.visible:=false;
pomb16.visible:=false;
pomb17.visible:=false;
pomb18.visible:=false;
pomb19.visible:=false;
pomb20.visible:=false;
pomb21.visible:=false;
pomb22.visible:=false;
pomb23.visible:=false;
pnb1.visible:=false;
pnb2.visible:=false;
pnb3.visible:=false;
pnb4.visible:=false;
pnb5.visible:=false;
pnb6.visible:=false;
pnb7.visible:=false;
pnb8.visible:=false;
pnb9.visible:=false;
pnb10.visible:=false;
pnb11.visible:=false;
pnb12.visible:=false;
pnb13.visible:=false;
pnb14.visible:=false;
pnb15.visible:=false;
pnb16.visible:=false;
pnb17.visible:=false;
pnb18.visible:=false;
pnb19.visible:=false;
pnb20.visible:=false;
pnb21.visible:=false;
pnb22.visible:=false;
pnb23.visible:=false;
pnba1.visible:=false;
pnba2.visible:=false;
pnba3.visible:=false;
pnba4.visible:=false;
pnba5.visible:=false;
pnba6.visible:=false;
pnba7.visible:=false;
pnba8.visible:=false;
pnba9.visible:=false;
pnba10.visible:=false;
pnba11.visible:=false;
pnba12.visible:=false;
pnba13.visible:=false;
pnba14.visible:=false;
pnba15.visible:=false;
pnba16.visible:=false;
pnba17.visible:=false;
pnba18.visible:=false;
pnba19.visible:=false;
pnba20.visible:=false;
pnba21.visible:=false;
pnba22.visible:=false;
pnba23.visible:=false;
pmjb1.visible:=false;
pmjb2.visible:=false;
pmjb3.visible:=false;
pmjb4.visible:=false;
pmjb5.visible:=false;
pmjb6.visible:=false;
pmjb7.visible:=false;
pmjb8.visible:=false;
pmjb9.visible:=false;
pmjb10.visible:=false;
pmjb11.visible:=false;
pmjb12.visible:=false;
pmjb13.visible:=false;
pmjb14.visible:=false;
pmjb15.visible:=false;
pmjb16.visible:=false;
pmjb17.visible:=false;
pmjb18.visible:=false;
pmjb19.visible:=false;
pmjb20.visible:=false;
pmjb21.visible:=false;
pmjb22.visible:=false;
pmjb23.visible:=false;
j:=StringGridBookmarks.RowCount;
if j>24 then j:=24;
if StringGridBookmarks.RowCount>1 then
   for i:=1 to j-1 do
      begin
      s:=Form_peach.StringGridBookmarks.Cells[2,i];
      if length(s)>0 then
         if s[length(s)]=directoryseparator then s1:=s
            else s1:=uextractfilepath(s);
      case i of
         1: begin
            mb1.caption:=s;
            mb1.visible:=true;
            mb1.Bitmap:=drawbookmarkicon(1);
            pomb1.caption:=s;
            pomb1.visible:=true;
            pomb1.Bitmap:=drawbookmarkicon(1);
            pmjb1.caption:=s;
            pmjb1.visible:=true;
            pmjb1.Bitmap:=drawbookmarkicon(1);
            pnb1.caption:=s1;
            pnba1.caption:=s1;
            if s1<>'' then
               begin
               pnb1.visible:=true;
               pnb1.Bitmap:=drawbookmarkicon2(1);
               pnba1.visible:=true;
               pnba1.Bitmap:=drawbookmarkicon2(1);
               end;
            end;
         2: begin
            mb2.caption:=s;
            mb2.visible:=true;
            mb2.Bitmap:=drawbookmarkicon(2);
            pomb2.caption:=s;
            pomb2.visible:=true;
            pomb2.Bitmap:=drawbookmarkicon(2);
            pmjb2.caption:=s;
            pmjb2.visible:=true;
            pmjb2.Bitmap:=drawbookmarkicon(2);
            pnb2.caption:=s1;
            pnba2.caption:=s1;
            if s1<>'' then
               begin
               pnb2.visible:=true;
               pnb2.Bitmap:=drawbookmarkicon2(2);
               pnba2.visible:=true;
               pnba2.Bitmap:=drawbookmarkicon2(2);
               end;
            end;
         3: begin
            mb3.caption:=s;
            mb3.visible:=true;
            mb3.Bitmap:=drawbookmarkicon(3);
            pomb3.caption:=s;
            pomb3.visible:=true;
            pomb3.Bitmap:=drawbookmarkicon(3);
            pmjb3.caption:=s;
            pmjb3.visible:=true;
            pmjb3.Bitmap:=drawbookmarkicon(3);
            pnb3.caption:=s1;
            pnba3.caption:=s1;
            if s1<>'' then
               begin
               pnb3.visible:=true;
               pnb3.Bitmap:=drawbookmarkicon2(3);
               pnba3.visible:=true;
               pnba3.Bitmap:=drawbookmarkicon2(3);
               end;
            end;
         4: begin
            mb4.caption:=s;
            mb4.visible:=true;
            mb4.Bitmap:=drawbookmarkicon(4);
            pomb4.caption:=s;
            pomb4.visible:=true;
            pomb4.Bitmap:=drawbookmarkicon(4);
            pmjb4.caption:=s;
            pmjb4.visible:=true;
            pmjb4.Bitmap:=drawbookmarkicon(4);
            pnb4.caption:=s1;
            pnba4.caption:=s1;
            if s1<>'' then
               begin
               pnb4.visible:=true;
               pnb4.Bitmap:=drawbookmarkicon2(4);
               pnba4.visible:=true;
               pnba4.Bitmap:=drawbookmarkicon2(4);
               end;
            end;
         5: begin
            mb5.caption:=s;
            mb5.visible:=true;
            mb5.Bitmap:=drawbookmarkicon(5);
            pomb5.caption:=s;
            pomb5.visible:=true;
            pomb5.Bitmap:=drawbookmarkicon(5);
            pmjb5.caption:=s;
            pmjb5.visible:=true;
            pmjb5.Bitmap:=drawbookmarkicon(5);
            pnb5.caption:=s1;
            pnba5.caption:=s1;
            if s1<>'' then
               begin
               pnb5.visible:=true;
               pnb5.Bitmap:=drawbookmarkicon2(5);
               pnba5.visible:=true;
               pnba5.Bitmap:=drawbookmarkicon2(5);
               end;
            end;
         6: begin
            mb6.caption:=s;
            mb6.visible:=true;
            mb6.Bitmap:=drawbookmarkicon(6);
            pomb6.caption:=s;
            pomb6.visible:=true;
            pomb6.Bitmap:=drawbookmarkicon(6);
            pmjb6.caption:=s;
            pmjb6.visible:=true;
            pmjb6.Bitmap:=drawbookmarkicon(6);
            pnb6.caption:=s1;
            pnba6.caption:=s1;
            if s1<>'' then
               begin
               pnb6.visible:=true;
               pnb6.Bitmap:=drawbookmarkicon2(6);
               pnba6.visible:=true;
               pnba6.Bitmap:=drawbookmarkicon2(6);
               end;
            end;
         7: begin
            mb7.caption:=s;
            mb7.visible:=true;
            mb7.Bitmap:=drawbookmarkicon(7);
            pomb7.caption:=s;
            pomb7.visible:=true;
            pomb7.Bitmap:=drawbookmarkicon(7);
            pmjb7.caption:=s;
            pmjb7.visible:=true;
            pmjb7.Bitmap:=drawbookmarkicon(7);
            pnb7.caption:=s1;
            pnba7.caption:=s1;
            if s1<>'' then
               begin
               pnb7.visible:=true;
               pnb7.Bitmap:=drawbookmarkicon2(7);
               pnba7.visible:=true;
               pnba7.Bitmap:=drawbookmarkicon2(7);
               end;
            end;
         8: begin
            mb8.caption:=s;
            mb8.visible:=true;
            mb8.Bitmap:=drawbookmarkicon(8);
            pomb8.caption:=s;
            pomb8.visible:=true;
            pomb8.Bitmap:=drawbookmarkicon(8);
            pmjb8.caption:=s;
            pmjb8.visible:=true;
            pmjb8.Bitmap:=drawbookmarkicon(8);
            pnb8.caption:=s1;
            pnba8.caption:=s1;
            if s1<>'' then
               begin
               pnb8.visible:=true;
               pnb8.Bitmap:=drawbookmarkicon2(8);
               pnba8.visible:=true;
               pnba8.Bitmap:=drawbookmarkicon2(8);
               end;
            end;
         9: begin
            mb9.caption:=s;
            mb9.visible:=true;
            mb9.Bitmap:=drawbookmarkicon(9);
            pomb9.caption:=s;
            pomb9.visible:=true;
            pomb9.Bitmap:=drawbookmarkicon(9);
            pmjb9.caption:=s;
            pmjb9.visible:=true;
            pmjb9.Bitmap:=drawbookmarkicon(9);
            pnb9.caption:=s1;
            pnba9.caption:=s1;
            if s1<>'' then
               begin
               pnb9.visible:=true;
               pnb9.Bitmap:=drawbookmarkicon2(9);
               pnba9.visible:=true;
               pnba9.Bitmap:=drawbookmarkicon2(9);
               end;
            end;
         10: begin
            mb10.caption:=s;
            mb10.visible:=true;
            mb10.Bitmap:=drawbookmarkicon(10);
            pomb10.caption:=s;
            pomb10.visible:=true;
            pomb10.Bitmap:=drawbookmarkicon(10);
            pmjb10.caption:=s;
            pmjb10.visible:=true;
            pmjb10.Bitmap:=drawbookmarkicon(10);
            pnb10.caption:=s1;
            pnba10.caption:=s1;
            if s1<>'' then
               begin
               pnb10.visible:=true;
               pnb10.Bitmap:=drawbookmarkicon2(10);
               pnba10.visible:=true;
               pnba10.Bitmap:=drawbookmarkicon2(10);
               end;
            end;
         11: begin
            mb11.caption:=s;
            mb11.visible:=true;
            mb11.Bitmap:=drawbookmarkicon(11);
            pomb11.caption:=s;
            pomb11.visible:=true;
            pomb11.Bitmap:=drawbookmarkicon(11);
            pmjb11.caption:=s;
            pmjb11.visible:=true;
            pmjb11.Bitmap:=drawbookmarkicon(11);
            pnb11.caption:=s1;
            pnba11.caption:=s1;
            if s1<>'' then
               begin
               pnb11.visible:=true;
               pnb11.Bitmap:=drawbookmarkicon2(11);
               pnba11.visible:=true;
               pnba11.Bitmap:=drawbookmarkicon2(11);
               end;
            end;
         12: begin
            mb12.caption:=s;
            mb12.visible:=true;
            mb12.Bitmap:=drawbookmarkicon(12);
            pomb12.caption:=s;
            pomb12.visible:=true;
            pomb12.Bitmap:=drawbookmarkicon(12);
            pmjb12.caption:=s;
            pmjb12.visible:=true;
            pmjb12.Bitmap:=drawbookmarkicon(12);
            pnb12.caption:=s1;
            pnba12.caption:=s1;
            if s1<>'' then
               begin
               pnb12.visible:=true;
               pnb12.Bitmap:=drawbookmarkicon2(12);
               pnba12.visible:=true;
               pnba12.Bitmap:=drawbookmarkicon2(12);
               end;
            end;
         13: begin
            mb13.caption:=s;
            mb13.visible:=true;
            mb13.Bitmap:=drawbookmarkicon(13);
            pomb13.caption:=s;
            pomb13.visible:=true;
            pomb13.Bitmap:=drawbookmarkicon(13);
            pmjb13.caption:=s;
            pmjb13.visible:=true;
            pmjb13.Bitmap:=drawbookmarkicon(13);
            pnb13.caption:=s1;
            pnba13.caption:=s1;
            if s1<>'' then
               begin
               pnb13.visible:=true;
               pnb13.Bitmap:=drawbookmarkicon2(13);
               pnba13.visible:=true;
               pnba13.Bitmap:=drawbookmarkicon2(13);
               end;
            end;
         14: begin
            mb14.caption:=s;
            mb14.visible:=true;
            mb14.Bitmap:=drawbookmarkicon(14);
            pomb14.caption:=s;
            pomb14.visible:=true;
            pomb14.Bitmap:=drawbookmarkicon(14);
            pmjb14.caption:=s;
            pmjb14.visible:=true;
            pmjb14.Bitmap:=drawbookmarkicon(14);
            pnb14.caption:=s1;
            pnba14.caption:=s1;
            if s1<>'' then
               begin
               pnb14.visible:=true;
               pnb14.Bitmap:=drawbookmarkicon2(14);
               pnba14.visible:=true;
               pnba14.Bitmap:=drawbookmarkicon2(14);
               end;
            end;
         15: begin
            mb15.caption:=s;
            mb15.visible:=true;
            mb15.Bitmap:=drawbookmarkicon(15);
            pomb15.caption:=s;
            pomb15.visible:=true;
            pomb15.Bitmap:=drawbookmarkicon(15);
            pmjb15.caption:=s;
            pmjb15.visible:=true;
            pmjb15.Bitmap:=drawbookmarkicon(15);
            pnb15.caption:=s1;
            pnba15.caption:=s1;
            if s1<>'' then
               begin
               pnb15.visible:=true;
               pnb15.Bitmap:=drawbookmarkicon2(15);
               pnba15.visible:=true;
               pnba15.Bitmap:=drawbookmarkicon2(15);
               end;
            end;
         16: begin
            mb16.caption:=s;
            mb16.visible:=true;
            mb16.Bitmap:=drawbookmarkicon(16);
            pomb16.caption:=s;
            pomb16.visible:=true;
            pomb16.Bitmap:=drawbookmarkicon(16);
            pmjb16.caption:=s;
            pmjb16.visible:=true;
            pmjb16.Bitmap:=drawbookmarkicon(16);
            pnb16.caption:=s1;
            pnba16.caption:=s1;
            if s1<>'' then
               begin
               pnb16.visible:=true;
               pnb16.Bitmap:=drawbookmarkicon2(16);
               pnba16.visible:=true;
               pnba16.Bitmap:=drawbookmarkicon2(16);
               end;
            end;
         17: begin
            mb17.caption:=s;
            mb17.visible:=true;
            mb17.Bitmap:=drawbookmarkicon(17);
            pomb17.caption:=s;
            pomb17.visible:=true;
            pomb17.Bitmap:=drawbookmarkicon(17);
            pmjb17.caption:=s;
            pmjb17.visible:=true;
            pmjb17.Bitmap:=drawbookmarkicon(17);
            pnb17.caption:=s1;
            pnba17.caption:=s1;
            if s1<>'' then
               begin
               pnb17.visible:=true;
               pnb17.Bitmap:=drawbookmarkicon2(17);
               pnba17.visible:=true;
               pnba17.Bitmap:=drawbookmarkicon2(17);
               end;
            end;
         18: begin
            mb18.caption:=s;
            mb18.visible:=true;
            mb18.Bitmap:=drawbookmarkicon(18);
            pomb18.caption:=s;
            pomb18.visible:=true;
            pomb18.Bitmap:=drawbookmarkicon(18);
            pmjb18.caption:=s;
            pmjb18.visible:=true;
            pmjb18.Bitmap:=drawbookmarkicon(18);
            pnb18.caption:=s1;
            pnba18.caption:=s1;
            if s1<>'' then
               begin
               pnb18.visible:=true;
               pnb18.Bitmap:=drawbookmarkicon2(18);
               pnba18.visible:=true;
               pnba18.Bitmap:=drawbookmarkicon2(18);
               end;
            end;
         19: begin
            mb19.caption:=s;
            mb19.visible:=true;
            mb19.Bitmap:=drawbookmarkicon(19);
            pomb19.caption:=s;
            pomb19.visible:=true;
            pomb19.Bitmap:=drawbookmarkicon(19);
            pmjb19.caption:=s;
            pmjb19.visible:=true;
            pmjb19.Bitmap:=drawbookmarkicon(19);
            pnb19.caption:=s1;
            pnba19.caption:=s1;
            if s1<>'' then
               begin
               pnb19.visible:=true;
               pnb19.Bitmap:=drawbookmarkicon2(19);
               pnba19.visible:=true;
               pnba19.Bitmap:=drawbookmarkicon2(19);
               end;
            end;
         20: begin
            mb20.caption:=s;
            mb20.visible:=true;
            mb20.Bitmap:=drawbookmarkicon(20);
            pomb20.caption:=s;
            pomb20.visible:=true;
            pomb20.Bitmap:=drawbookmarkicon(20);
            pmjb20.caption:=s;
            pmjb20.visible:=true;
            pmjb20.Bitmap:=drawbookmarkicon(20);
            pnb20.caption:=s1;
            pnba20.caption:=s1;
            if s1<>'' then
               begin
               pnb20.visible:=true;
               pnb20.Bitmap:=drawbookmarkicon2(20);
               pnba20.visible:=true;
               pnba20.Bitmap:=drawbookmarkicon2(20);
               end;
            end;
         21: begin
            mb21.caption:=s;
            mb21.visible:=true;
            mb21.Bitmap:=drawbookmarkicon(21);
            pomb21.caption:=s;
            pomb21.visible:=true;
            pomb21.Bitmap:=drawbookmarkicon(21);
            pmjb21.caption:=s;
            pmjb21.visible:=true;
            pmjb21.Bitmap:=drawbookmarkicon(21);
            pnb21.caption:=s1;
            pnba21.caption:=s1;
            if s1<>'' then
               begin
               pnb21.visible:=true;
               pnb21.Bitmap:=drawbookmarkicon2(21);
               pnba21.visible:=true;
               pnba21.Bitmap:=drawbookmarkicon2(21);
               end;
            end;
         22: begin
            mb22.caption:=s;
            mb22.visible:=true;
            mb22.Bitmap:=drawbookmarkicon(22);
            pomb22.caption:=s;
            pomb22.visible:=true;
            pomb22.Bitmap:=drawbookmarkicon(22);
            pmjb22.caption:=s;
            pmjb22.visible:=true;
            pmjb22.Bitmap:=drawbookmarkicon(22);
            pnb22.caption:=s1;
            pnba22.caption:=s1;
            if s1<>'' then
               begin
               pnb22.visible:=true;
               pnb22.Bitmap:=drawbookmarkicon2(22);
               pnba22.visible:=true;
               pnba22.Bitmap:=drawbookmarkicon2(22);
               end;
            end;
         23: begin
            mb23.caption:=s;
            mb23.visible:=true;
            mb23.Bitmap:=drawbookmarkicon(23);
            pomb23.caption:=s;
            pomb23.visible:=true;
            pomb23.Bitmap:=drawbookmarkicon(23);
            pmjb23.caption:=s;
            pmjb23.visible:=true;
            pmjb23.Bitmap:=drawbookmarkicon(23);
            pnb23.caption:=s1;
            pnba23.caption:=s1;
            if s1<>'' then
               begin
               pnb23.visible:=true;
               pnb23.Bitmap:=drawbookmarkicon2(23);
               pnba23.visible:=true;
               pnba23.Bitmap:=drawbookmarkicon2(23);
               end;
            end;
         end;
      end;
end;
end;

function check_duplicated_bookmark(s,s1,s2,s3:utf8string):integer;
var
   i:integer;
begin
check_duplicated_bookmark:=-1;
for i:=1 to Form_peach.StringGridBookmarks.RowCount-1 do
   if (Form_peach.StringGridBookmarks.Cells[2,i]=s)
   and (Form_peach.StringGridBookmarks.Cells[1,i]=s1)
   and (Form_peach.StringGridBookmarks.Cells[3,i]=s2)
   and (Form_peach.StringGridBookmarks.Cells[4,i]=s3) then
      begin
      check_duplicated_bookmark:=-1;
      exit;
      end;
check_duplicated_bookmark:=0;
end;

procedure addtobookmarks(s:utf8string);
var
   i,rc:integer;
   s1,s2,s3:utf8string;
begin
s1:=txt_list_browsing;
s2:='*';
s3:='';
if s='*browser' then
   begin
   if fun='FILEBROWSER' then
      s:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]
   else
      s:=Form_peach.EditOpenIn.Text;
   end;
if s='*layout' then
   begin
   {$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.Top=0{$ENDIF} then s:=Form_peach.StringGrid1.Cells[8,Form_peach.StringGrid1.Row]
   else s:=Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row];
   end;
if s='*history' then
   begin
   s:=Form_peach.StringGridHistory.Cells[2,Form_peach.StringGridHistory.Row];
   s1:=Form_peach.StringGridHistory.Cells[1,Form_peach.StringGridHistory.Row];
   s2:=Form_peach.StringGridHistory.Cells[3,Form_peach.StringGridHistory.Row];
   s3:=Form_peach.StringGridHistory.Cells[4,Form_peach.StringGridHistory.Row];
   end;
if s='*current' then
   begin
   rc:=Form_peach.StringGridHistory.Rowcount-1;
   s:=Form_peach.StringGridHistory.Cells[2,rc];
   s1:=Form_peach.StringGridHistory.Cells[1,rc];
   s2:=Form_peach.StringGridHistory.Cells[3,rc];
   s3:=Form_peach.StringGridHistory.Cells[4,rc];
   end;
if s<>txt_mypc then
   if ufilegetattr(escapefilename(s,desk_env)) and faDirectory <>0 then
      if s<>'' then
         if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
if check_duplicated_bookmark(s,s1,s2,s3)=0 then
   begin
   rc:=Form_peach.StringGridBookmarks.Rowcount+1;
   Form_peach.StringGridBookmarks.RowCount:=rc;
   Form_peach.StringGridBookmarks.Cells[2,rc-1]:=s;
   //Form_peach.StringGridBookmarks.Cells[0,rc-1]:=inttostr(rc-1);
   Form_peach.StringGridBookmarks.Cells[1,rc-1]:=s1;
   Form_peach.StringGridBookmarks.Cells[3,rc-1]:=s2;
   Form_peach.StringGridBookmarks.Cells[4,rc-1]:=s3;
   Form_peach.StringGridBookmarks.AutosizeColumns;
   Form_peach.StringGridBookmarks.ColWidths[6]:=0;
   Form_peach.StringGridBookmarks.ColWidths[0]:=ROWSIZE;
   for i:=1 to Form_peach.StringGridBookmarks.RowCount-2 do Form_peach.StringGridBookmarks.Cells[6,i]:='0';
   Form_peach.StringGridBookmarks.Cells[6,Form_peach.StringGridBookmarks.RowCount-1]:='1';
   end;
{$IFDEF MSWINDOWS}
{$ELSE}
{$ENDIF}
populatebookmarks;
end;

function GetUNCName(const LocalPath: string): utf8string;
var
  BufferSize: DWord;
  DummyBuffer: Byte;
  Buffer: Pointer;
  Error: DWord;
begin
{$IFDEF MSWINDOWS}
BufferSize := 1;
WNetGetUniversalName(PChar(LocalPath), UNIVERSAL_NAME_INFO_LEVEL, @DummyBuffer, BufferSize);
Buffer := AllocMem(BufferSize);
try
   Error := WNetGetUniversalName(PChar(LocalPath), UNIVERSAL_NAME_INFO_LEVEL, Buffer, BufferSize);
   if Error <> NO_ERROR then
      begin
      Result:='';
      end;
   Result := PUniversalNameInfo(Buffer)^.lpUniversalName
finally
   FreeMem(Buffer);
end;
{$ENDIF}
end;

procedure filebrowsermenus;
begin
with Form_peach do
begin
//po_layout.visible:=true;
po_open.visible:=true;
if EditOpenIn.Text=txt_mypc then
   begin
   po_info.visible:=true;
   po_list.visible:=true;
   po_systemtools.visible:=true;
   sm_delete.enabled:=false;
   end
else
   begin
   po_exthere.visible:=true;
   po_extto.visible:=true;
   po_test.visible:=true;
   po_info.visible:=true;
   po_list.visible:=true;
   po_filetools.visible:=true;
   po_modify.visible:=true;
   sm_delete.enabled:=true;
   set_custedit_menus;
   end;
MenuItemOpen_ExtractHere.Visible:=false;
MenuItemOpen_ExtractTo.Visible:=false;
po_archive_spacer.visible:=true;
end;
end;

procedure advancedarchivebrowsermenus;
begin
with Form_peach do
begin
MenuItemOpen_ExtractHere.Visible:=true;
MenuItemOpen_ExtractTo.Visible:=true;
MenuItemOpen_test.Visible:=true;
MenuItemOpen_list.Visible:=true;
MenuItemOpen_preview.visible:=true;
MenuItemOpen_extandopenwith.visible:=true;
po_archive_spacer.visible:=true;
end;
end;

procedure setbuttonmenus;
begin
with Form_peach do
begin
ButtonUn7zaTo.Enabled:=true;
ButtonUn7zaTo.PopupMenu:=nil;
ButtonUn7za1.Enabled:=false;
ButtonUn7za6.Enabled:=false;
ButtonUn7za4.Enabled:=false;
ButtonUn7za4.Hint:='';
po_addtoarchive.Enabled:=false;
po_exthere.visible:=false;
po_extto.visible:=false;
po_test.visible:=false;
po_info.visible:=false;
po_list.visible:=false;
po_open.visible:=false;
sbBrowse.Visible:=false;
po_modify.visible:=false;
po_filetools.visible:=false;
po_systemtools.visible:=false;
MenuItemOpen_preview.visible:=false;
MenuItemOpen_test.Visible:=false;
MenuItemOpen_info.Visible:=false;
MenuItemOpen_list.Visible:=false;
po_archive_spacer.visible:=false;
MenuItemOpen_extandopenwith.visible:=false;
if fun='FILEBROWSER' then
   begin
   ButtonUn7za1.Enabled:=true;
   ButtonUn7za6.Enabled:=true;
   po_addtoarchive.Enabled:=true;
   ButtonUn7za4.Enabled:=true;
   ButtonUn7za4.Hint:=txt_add_tolayout;
   filebrowsermenus;
   sbBrowse.Visible:=true;
   end;
if fun='UN7Z' then
   begin
   ButtonUn7za1.Enabled:=true;
   ButtonUn7za6.Enabled:=true;
   po_addtoarchive.Enabled:=true;
   ButtonUn7za4.Enabled:=true;
   ButtonUn7za4.Hint:=txt_add_toarchive;
   advancedarchivebrowsermenus;
   sbBrowse.Visible:=true;
   MenuItemOpen_info.Visible:=true;
   end;
if fun='UNARC' then
   begin
   ButtonUn7za1.Enabled:=true;
   ButtonUn7za6.Enabled:=true;
   po_addtoarchive.Enabled:=true;
   ButtonUn7za4.Enabled:=true;
   ButtonUn7za4.Hint:=txt_add_toarchive;
   advancedarchivebrowsermenus;
   end;
end;
end;

procedure prepare_filebrowser;
begin
Form_peach.NoteBookOpen.Pageindex:=0;
Form_peach.gbConsoleCreate1.Enabled:=false;
Form_peach.iConsoleCreateImport1.Enabled:=false;
Form_peach.ButtonUn7zaTo.PopupMenu:=nil;
Form_peach.ButtonUn7za1.PopupMenu:=nil;
Form_peach.sm_delete.Enabled:=true;
Form_peach.po_securedelete.Visible:=true;
Form_peach.po_delete.Caption:=txt_quickdelete;
Form_peach.pmsecuredelete.Visible:=true;
Form_peach.pmcpyto.Visible:=true;
Form_peach.pmmvto.Visible:=true;
Form_peach.pmquickdelete.Caption:=txt_quickdelete;
end;

procedure getcurrentfs(var s,sfs:utf8string);
var
   sdrive:utf8string;
   bufVolumeName, bufFSName: array[0..255] of Char;
   sn,mc,flags:dword;
begin
{$IFDEF MSWINDOWS}
if (winver<>'nt6+') and (winver<>'nt5') then
   begin
   sfs:=txt_fs;
   exit;
   end;
sdrive:=extractfiledrive(s)+'\';
GetVolumeInformation(Pchar(sdrive),
@bufVolumeName, sizeof(bufVolumeName),
@sn, mc, flags,
@bufFSName, sizeof(bufFSName));
sfs:=bufFSName;
{$ELSE}
sfs:=txt_fs;
{$ENDIF}
end;

procedure listpc;
{$IFDEF MSWINDOWS}
var
   s,s1,dtype,dname,fsname,uncname:utf8string;
   c:char;
   i,a,b,j,k,t,sz,sizetotal,sizefree:qword;
   sn,mc,flags:dword;
   bufVolumeName, bufFSName: array[0..255] of Char;
   drivestr:array[1..255] of char;
   memory: TMemoryStatus; //note: supports up to 2 GB of RAM (4 if largememaware): GlobalMemoryStatus should be replaced with GlobalMemoryStatusEx call (currently unavailable)
   {$ENDIF}
begin
{$IFDEF MSWINDOWS}
if (winver<>'nt6+') and (winver<>'nt5') then
   begin
   listdir(local_desktop,false);
   exit;
   end;
if listingdir=true then exit;
listingdir:=true;
prepare_filebrowser;
status0:=txt_list_browsing;
currentcomp:='';
currentfs:='';
status1:=envtoutf8(GetEnvironmentVariable('COMPUTERNAME'));
status2:=usr_name;
fun:='FILEBROWSER';
fun_status:=fun;
Form_peach.EditOpenIn.Text:=txt_mypc;
setbuttonmenus;
GetLogicalDriveStrings(255,@drivestr);
i:=1;
j:=1;
sizetotal:=0;
sizefree:=0;
Form_peach.StringGridList.RowCount:=1;
Form_peach.StringGridList.Cells[4,0]:=txt_free;
Form_peach.StringGridList.Cells[5,0]:=txt_fs;
setsearch_disabled;
repeat
   s:='';
   dtype:='';
   dname:='';
   fsname:='';
   uncname:='';
   bufVolumeName:='';
   bufFSName:='';
   while (i<=255) and (drivestr[i]<>#00) do
      begin
      s:=s+char(drivestr[i]);
      inc(i);
      end;
   inc(i);
   if length(s)>0 then
      begin
      c:=upcase(s[1]);
      j:=ord(c)-64;
      k:=Form_peach.StringGridList.RowCount;
      if j>2 then
         begin
         //drive type
         t:=getdrivetype(Pchar(s));
         case t of
         2: dtype:=txt_unit_removable;
         3: dtype:=txt_unit_hd;
         4: dtype:=txt_unit_remote;
         5: dtype:=txt_unit_cd;
         6: dtype:=txt_unit_ram;
         else dtype:=txt_unit_unknown;
         end;
         //drive name
         GetVolumeInformation(Pchar(s),
         @bufVolumeName, sizeof(bufVolumeName),
         @sn, mc, flags,
         @bufFSName, sizeof(bufFSName));
         if bufVolumeName<>'' then dname:=bufVolumeName
         else dname:=dtype;
         fsname:=bufFSName;
         //unc name
         uncname:=getUNCname(Pchar(s));
         if uncname<>'' then
            if uncname[length(uncname)]=directoryseparator then setlength(uncname,length(uncname)-1);
         Form_peach.StringGridList.RowCount:=Form_peach.StringGridList.RowCount+1;
         s1:=s;
         setlength(s1,length(s1)-1);
         if uncname='' then Form_peach.StringGridList.Cells[1,k]:=dname+' ('+s1+')'
         else Form_peach.StringGridList.Cells[1,k]:=uextractfilename(uncname)+' '+txt_on+' '+uextractfilepath(uncname)+' '+dname+' ('+s1+')';
         Form_peach.StringGridList.Cells[2,k]:=dtype;
         if fsname<>'' then
            begin
            sz:=disksize(j);
            if sz=-1 then sz:=0;
            Form_peach.StringGridList.Cells[3,k]:=inttostr(sz);
            sizetotal:=sizetotal+sz;
            end;
         if fsname<>'' then
            begin
            sz:=diskfree(j);
            if sz=-1 then sz:=0;
            Form_peach.StringGridList.Cells[4,k]:=inttostr(sz);
            sizefree:=sizefree+sz;
            end;
         if Form_peach.StringGridList.Cells[3,k]='0' then Form_peach.StringGridList.Cells[3,k]:='';
         Form_Peach.StringGridList.Cells[10,k]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,k]))))+inttostr(length(Form_peach.StringGridList.Cells[3,k]))+Form_Peach.StringGridList.Cells[3,k];
         if Form_peach.StringGridList.Cells[4,k]='0' then Form_peach.StringGridList.Cells[4,k]:='';
         Form_peach.StringGridList.Cells[11,k]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[4,k]))))+inttostr(length(Form_peach.StringGridList.Cells[4,k]))+Form_peach.StringGridList.Cells[4,k];
         if Form_peach.StringGridList.Cells[3,k]<>'' then Form_peach.StringGridList.Cells[5,k]:=fsname;
         Form_peach.StringGridList.Cells[8,k]:=s;
         Form_peach.StringGridList.Cells[9,k]:=dtype+upcase(Form_peach.StringGridList.Cells[1,k]);
         Form_peach.StringGridList.Cells[12,k]:='0';
         end
      else
         begin
         s1:=s;
         setlength(s1,length(s1)-1);
         Form_peach.StringGridList.RowCount:=Form_peach.StringGridList.RowCount+1;
         Form_peach.StringGridList.Cells[1,k]:=txt_unit_floppy+' ('+s1+')';
         Form_peach.StringGridList.Cells[2,k]:=txt_unit_floppy;
         Form_peach.StringGridList.Cells[3,k]:='';
         Form_peach.StringGridList.Cells[4,k]:='';
         Form_Peach.StringGridList.Cells[10,k]:='';
         Form_peach.StringGridList.Cells[11,k]:='';
         Form_peach.StringGridList.Cells[5,k]:='';
         Form_peach.StringGridList.Cells[8,k]:=s;
         Form_peach.StringGridList.Cells[9,k]:=txt_unit_floppy+s;
         Form_peach.StringGridList.Cells[12,k]:='0';
         end;
      end;
until length(s)=0;
listsizecol('pc');
Form_peach.StringGridList.SortColRow(true,8);
Form_peach.StringGridList.Cells[12,1]:='1';
Form_peach.StringGridList.Row:=1;
Form_peach.Caption:=status1;
i:=(sizefree*100) div sizetotal;
memory.dwLength := SizeOf(memory);
GlobalMemoryStatus(memory); //note: currently declared as procedure, cannot be checked for overflow
a:=(memory.dwAvailPhys div (1024*1024)) * 100 div (memory.dwTotalPhys div (1024*1024));
b:=(memory.dwAvailPageFile div (1024*1024)) * 100 div (memory.dwTotalPagefile div (1024*1024));
Form6.LabelUn7zInputStatus2.Caption:=status1+'; '+GetEnvironmentVariable('NUMBER_OF_PROCESSORS')+' CPU';
Form6.LabelUn7zInputStatus3.Caption:=inttostr(memory.dwTotalPhys div(1024*1024))+' MB RAM, '+inttostr(memory.dwAvailPhys div(1024*1024))+' MB '+txt_free2+' ('+inttostr(a)+'%); '+inttostr(memory.dwTotalPageFile div(1024*1024))+' MB '+txt_2_4_totalmem+' '+inttostr(memory.dwAvailPageFile div(1024*1024))+' MB '+txt_free2+' ('+inttostr(b)+'%)';
Form6.LabelUn7zInputStatus1.Caption:=inttostr(Form_peach.StringGridList.Rowcount-1)+' '+txt_units+'; '+inttostr(sizetotal div(1024*1024))+' MB, '+inttostr(sizefree div(1024*1024))+' MB '+txt_free2+' ('+inttostr(i)+'%)';
Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus1.Caption;
Form6.ImageInfoOpen5.Visible:=true;
Form6.ImageInfoOpen6.Visible:=true;
Form6.ShapeIO6.Visible:=true;
Form6.ShapeIO7.Visible:=true;
Form6.ShapeIO8.Visible:=true;
Form6.ShapeIO9.Visible:=true;
Form6.LabelPercent.Visible:=true;
Form6.LabelPercent1.Visible:=true;
Form6.ShapeIO6.Width:=200 * i div 100;
Form6.LabelPercent.Caption:=inttostr(i)+'%';
Form6.ShapeIO7.Width:=200 * b div 100;
Form6.LabelPercent1.Caption:=inttostr(b)+'%';
Form6.ShapeIO6.Hint:='HD';
Form6.ShapeIO7.Hint:='RAM';
Form6.ImageInfoOpen5.Picture.Bitmap:=Bhd_supported;
Form6.ImageInfoOpen5.Transparent:=true;
Form_peach.ImageIO.Glyph:=Form6.ImageInfoOpen5.Picture.Bitmap;
Form6.ImageInfoOpen6.Picture.Bitmap:=Bram_supported;
Form6.ImageInfoOpen6.Transparent:=true;
indir:=txt_mypc;
listingdir:=false;
{$ENDIF}
end;

procedure list_root;
var
   i:integer;
   s:utf8string;
begin
if fun='UN7Z' then
   begin
   Form_peach.EditUn7zaFilter.Text:='*';
   Form_peach.EditUn7zaFilterExclude.Text:='*'+directoryseparator+'*';
   i:=list_un7z('browse');
   status0:=txt_list_browsing;
   browsingmode7z:=0;
   if (archive_content='') or
      (archive_content=txt_list_nomatch)
      then archive_content:=txt_list_nostats;
   teststatus(i,s);
   Form_peach.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename)+'; '+archive_content+'; '+s;
   addtohistory;
   end;
if fun='FILEBROWSER' then
   begin
   {$IFDEF MSWINDOWS}
   listpc;
   {$ELSE}
   Form_peach.EditUn7zaFilter.Text:='*';
   s:='/';
   Form_peach.EditOpenIn.Text:=s;
   listdir(s,false);
   {$ENDIF}
   addtohistory;
   end;
end;

procedure list_un7z_expandall;
var
   i:integer;
   s:utf8string;
begin
Form_peach.EditUn7zaFilter.Text:='*';
Form_peach.EditUn7zaFilterExclude.Text:='';
i:=list_un7z('flat');
status0:=txt_list_flat;
browsingmode7z:=1;
archive_content:=status1;
if archive_content=txt_list_nomatch+txt_2_7_list_tryflatorpw then archive_content:=txt_list_nostats;
teststatus(i,s);
Form_peach.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename)+'; '+archive_content+'; '+s;
addtohistory;
end;

procedure list_un7z_expandall_silent; //don't add to history
var
   i:integer;
   s:utf8string;
begin
Form_peach.EditUn7zaFilter.Text:='*';
Form_peach.EditUn7zaFilterExclude.Text:='';
i:=list_un7z('silent');
archive_content:=status1;
if archive_content=txt_list_nomatch+txt_2_7_list_tryflatorpw then archive_content:=txt_list_nostats;
teststatus(i,s);
Form_peach.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename)+'; '+archive_content+'; '+s;
end;

procedure list_un7z_search;
begin
Form_peach.EditUn7zaFilterExclude.Text:='';
list_un7z('browse');
status0:=txt_list_found;
if Form7.CheckBoxAdvFilters.State=cbUnChecked then addtohistory;
end;

procedure list_un7z_do;
begin
list_un7z('browse');
status0:=txt_list_browsing;
browsingmode7z:=0;
addtohistory;
end;

procedure ratio_info_reduced(i:integer; size,csize:qword);
var cratio:qword;
begin
status0:=txt_list_flat;
try
   status1:=inttostr(i)+' '+txt_displayed_obj+'; '+inttostr(size)+' B > '+inttostr(csize)+' B';
   try cratio:=((csize div 1024) *100) div (size div 1024); except end;
   if (cratio<101) and (cratio>1) then Form6.ShapeIO7.Width:=cratio * 200 div 100
   else Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:=inttostr(cratio)+'%';
except
   status1:=txt_list_na;
   Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:='??%';
end;
if (cratio>1) and (cratio<200) then status1:=status1+' ('+inttostr(cratio)+'%)';
Form_peach.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename);
end;

procedure browse_single;
var
   s:utf8string;
   f:file of byte;
   b:byte;
   size,csize:qword;
begin
Form_peach.StringGridList.RowCount:=2;
s:=uextractfilename(Form_peach.OpenDialog3.Filename);
if (fun<>'UNCUSTOM') and (fun<>'UNPEA') then cutextension(s);
Form_peach.StringGridList.Cells[1,1]:=s;
Form_peach.StringGridList.Cells[2,1]:=uextractfileext(Form_peach.StringGridList.Cells[1,1]);
size:=-1;
csize:=-1;
Form_peach.StringGridList.Cells[3,1]:='';
Form_peach.StringGridList.Cells[4,1]:='';
try
uassignfile(f,Form_peach.EditOpenIn.Text);
filemode:=0;
reset(f);
if fun='UNQUAD' then
   begin
   size:=0;
   if uextractfileext(Form_peach.EditOpenIn.Text)='.quad' then
      blockread(f,size,4)
   else
      begin
      blockread(f,b,1);
      blockread(f,size,4);
      end;
   end;
if fun='UNLPAQ' then
   begin
   blockread(f,size,4);//discarded
   size:=0;
   blockread(f,b,1);
   size:=size xor (b shl 24);
   blockread(f,b,1);
   size:=size xor (b shl 16);
   blockread(f,b,1);
   size:=size xor (b shl 8);
   blockread(f,b,1);
   size:=size xor b;
   end;
srcfilesize(Form_peach.EditOpenIn.Text,csize);
closefile(f);
except
end;
if size<>-1 then Form_peach.StringGridList.Cells[3,1]:=inttostr(size); //expressed in B
if csize<>-1 then Form_peach.StringGridList.Cells[4,1]:=inttostr(csize); //expressed in B
Form_peach.StringGridList.Cells[5,1]:='';
Form_peach.StringGridList.Cells[6,1]:='';
Form_peach.StringGridList.Cells[8,1]:=s;
Form_peach.StringGridList.Cells[9,1]:='';
Form_peach.StringGridList.Cells[10,1]:='';
Form_peach.StringGridList.Cells[11,1]:='';
Form_peach.StringGridList.Cells[12,1]:='1';
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
listsizecol('flat');
if Form_peach.StringGridList.RowCount>1 then Form_peach.StringGridList.Cells[12,1]:='1';
Form_peach.StringGridList.Row:=1;
ratio_info_reduced(1,size,csize);
if fun='UNPEA' then
   begin
   status0:=txt_list_browsing;
   status1:=txt_list_na;
   end;
end;

procedure open_prepare_unpea;
begin
Form_peach.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename);
browse_single;
addtohistory;
end;

procedure browse_paq;
var
   f:file of byte;
   b:byte;
   s:utf8string;
   i:integer;
   size,csize:qword;
   end_browse:boolean;
begin
try
uassignfile(f,Form_peach.OpenDialog3.Filename);
filemode:=0;
reset(f);
srcfilesize(Form_peach.EditOpenIn.Text,csize);
repeat //discard PAQ file header (generic procedure to be quite adaptable to possible different models)
   blockread(f,b,1);
until b=$0a;
except
   Form_peach.StringGridList.RowCount:=1;
   MessageDlg(txt_list_error, mtWarning, [mbOK], 0);
   exit;
end;
Form_peach.StringGridList.RowCount:=1;
i:=0;
size:=0;
end_browse:=false;
repeat
   s:='';
   repeat
      blockread(f,b,1);
      if b=$1a then
         begin
         end_browse:=true;
         break;
         end;
      if (b<>$0d) then s:=s+char(b);
   until b=$0d;
   if end_browse=true then break;
   i:=i+1;
   Form_peach.StringGridList.RowCount:=Form_peach.StringGridList.RowCount+1;
   Form_peach.StringGridList.Cells[3,i]:=copy(s,1,pos(char($09),s)-1);
   Form_peach.StringGridList.Cells[10,i]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,i]))))+inttostr(length(Form_peach.StringGridList.Cells[3,i]))+Form_peach.StringGridList.Cells[3,i];
   size:=size+strtoqword(Form_peach.StringGridList.Cells[3,i]);
   Form_peach.StringGridList.Cells[8,i]:=ansitoutf8(copy(s,pos(char($09),s)+1,length(s)));
   Form_peach.StringGridList.Cells[1,i]:=uextractfilename(Form_peach.StringGridList.Cells[8,i]);
   Form_peach.StringGridList.Cells[2,i]:=uextractfileext(Form_peach.StringGridList.Cells[1,i]);
   Form_peach.StringGridList.Cells[12,i]:='0';
   blockread(f,b,1);
until end_browse=true;
closefile(f);
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
listsizecol('flat');
if Form_peach.StringGridList.RowCount>1 then
   begin
   Form_peach.StringGridList.Cells[12,1]:='1';
   Form_peach.StringGridList.Row:=1;
   end;
ratio_info_reduced(i,size,csize);
end;

procedure browse_zpaq;
var
   s,s1,s2,s3,cl,jobcode,outname:utf8string;
   size,csize:qword;
   i,k,ks,ks1:integer;
begin
//compose cl to list zpaq
compose_unzpaq_cl(cl,jobcode,outname,false,'list','all');
//execute cl
execute_cl(cl,'list');
//list content in stringgrid
size:=0;
csize:=0;
k:=0;
Form_peach.StringGridList.RowCount:=Form_peach.MemoList.Lines.Count;
for i:=1 to Form_peach.MemoList.Lines.Count-1 do
   begin
   s:=Form_peach.MemoList.Lines[i];
   if s[1]=' ' then
      begin
      k:=k+1;
      s:=copy(Form_peach.MemoList.Lines[i],2,length(Form_peach.MemoList.Lines[i])-1);
      s1:=uextractfileext(s);
      cutextension(s);
      ks:=pos('  ',s1);
      ks1:=pos(' -> ',s1);
      s2:=copy(s1,ks+2,ks1-ks-2);
      s3:=copy(s1,ks1+4,length(s1)-ks1+1);
      s1:=copy(s1,1,ks-1);
      s:=s+s1;
      Form_peach.StringGridList.Cells[8,k]:=s;
      Form_peach.StringGridList.Cells[1,k]:=uextractfilename(s);
      Form_peach.StringGridList.Cells[2,k]:=s1;
      Form_peach.StringGridList.Cells[9,k]:='2'+Form_peach.StringGridList.Cells[1,k];
      Form_peach.StringGridList.Cells[3,k]:=s2;
      Form_peach.StringGridList.Cells[4,k]:=s3;
      try
      size:=size+strtoqword(Form_peach.StringGridList.Cells[3,k]);
      csize:=csize+strtoqword(Form_peach.StringGridList.Cells[4,k]);
      except
      end;
      Form_peach.StringGridList.Cells[5,k]:='';
      Form_peach.StringGridList.Cells[6,k]:='';
      end
   else
      begin
      end;
   end;
Form_peach.StringGridList.RowCount:=k+1;
try
Form_peach.StringGridList.SortColRow(true,listsortcol);
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
listsizecol('flat');
if Form_peach.StringGridList.RowCount>1 then
   begin
   Form_peach.StringGridList.Cells[12,1]:='1';
   Form_peach.StringGridList.Row:=1;
   end;
i:=Form_peach.StringGridList.Rowcount-1;
ratio_info_reduced(i,size,csize);
except
   MessageDlg(txt_list_error, mtWarning, [mbOK], 0);
   ratio_info_reduced(0,0,0);
end;
end;

procedure browse_ace;
var
   lincontrol,s,cl,jobcode,outname:utf8string;
   size,csize:qword;
   i,k,k2,title_lines_ace:integer;
begin
{$IFDEF MSWINDOWS}title_lines_ace:=3;{$ENDIF}
{$IFDEF LINUX}title_lines_ace:=10;{$ENDIF}
//compose cl to list ace
compose_unace_cl(cl,jobcode,outname,false,'list','all');
//execute cl
execute_cl(cl,'list');
//list content in stringgrid
size:=0;
csize:=0;
{$IFDEF LINUX}
lincontrol:=copy(Form_peach.MemoList.Lines[8],1,4);
if lincontrol<>'Date' then title_lines_ace:=8;
{$ENDIF}
Form_peach.StringGridList.RowCount:=Form_peach.MemoList.Lines.Count-title_lines_ace-1;
for i:=title_lines_ace to Form_peach.MemoList.Lines.Count-3 do
   begin
   {$IFDEF MSWINDOWS}
   s:=copy(Form_peach.MemoList.Lines[i],9,length(Form_peach.MemoList.Lines[i])-8);
   k:=pos('  (',s);//theorically it's not an univocous marker!
   s:=copy(s,1,k-1);
   Form_peach.StringGridList.Cells[8,i-title_lines_ace+1]:=s;
   Form_peach.StringGridList.Cells[1,i-title_lines_ace+1]:=uextractfilename(s);
   Form_peach.StringGridList.Cells[2,i-title_lines_ace+1]:=uextractfileext(Form_peach.StringGridList.Cells[1,i-title_lines_ace+1]);
   Form_peach.StringGridList.Cells[9,i-title_lines_ace+1]:='2'+Form_peach.StringGridList.Cells[1,i-title_lines_ace+1];
   s:=copy(Form_peach.MemoList.Lines[i],9,length(Form_peach.MemoList.Lines[i])-8);
   k:=pos('  (',s);
   k2:=pos(' byte uncompressed, ',s);
   s:=copy(s,k+3,k2-k-3);
   Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]:=s;
   try
    size:=size+strtoqword(Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]);
   except
   end;
   Form_peach.StringGridList.Cells[10,i-title_lines_ace+1]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]))))+inttostr(length(Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]))+Form_peach.StringGridList.Cells[3,i-title_lines_ace+1];
   s:=copy(Form_peach.MemoList.Lines[i],9,length(Form_peach.MemoList.Lines[i])-8);
   k:=pos(' byte uncompressed, ',s);
   k2:=pos(' byte compressed)',s);
   s:=copy(s,k+20,k2-k-20);
   Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]:=s;
   try
    csize:=csize+strtoqword(Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]);
   except
   end;
   Form_peach.StringGridList.Cells[11,i-title_lines_ace+1]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]))))+inttostr(length(Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]))+Form_peach.StringGridList.Cells[4,i-title_lines_ace+1];
   //try to guess if it's a folder (there is no univocous marker): no extension and size 0 (may be an empty file without extension!)
   if (Form_peach.StringGridList.Cells[2,i-title_lines_ace+1]='') and
      (Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]='0') then
      begin
      Form_peach.StringGridList.Cells[2,i-title_lines_ace+1]:=txt_list_isfolder;
      Form_peach.StringGridList.Cells[9,i-title_lines_ace+1]:='1'+Form_peach.StringGridList.Cells[1,i-title_lines_ace+1];
      end;
   Form_peach.StringGridList.Cells[5,i-title_lines_ace+1]:='';//date/time (not featured by the RadioGroupQuad)
   Form_peach.StringGridList.Cells[6,i-title_lines_ace+1]:='';//attributes (not featured by the RadioGroupQuad)
   {$ENDIF}
   {$IFDEF LINUX}
   s:=copy(Form_peach.MemoList.Lines[i],45,length(Form_peach.MemoList.Lines[i])-44);
   Form_peach.StringGridList.Cells[8,i-title_lines_ace+1]:=s;
   Form_peach.StringGridList.Cells[1,i-title_lines_ace+1]:=uextractfilename(s);
   Form_peach.StringGridList.Cells[2,i-title_lines_ace+1]:=uextractfileext(Form_peach.StringGridList.Cells[1,i-title_lines_ace+1]);
   Form_peach.StringGridList.Cells[9,i-title_lines_ace+1]:='2'+Form_peach.StringGridList.Cells[1,i-title_lines_ace+1];
   //date/time
   Form_peach.StringGridList.Cells[5,i-title_lines_ace+1]:=copy(Form_peach.MemoList.Lines[i],1,8)+' '+copy(Form_peach.MemoList.Lines[i],10,5);
   //size
   Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]:=copy(Form_peach.MemoList.Lines[i],28,9);
   try
    size:=size+strtoqword(Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]);
   except
   end;
   Form_peach.StringGridList.Cells[10,i-title_lines_ace+1]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]))))+inttostr(length(Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]))+Form_peach.StringGridList.Cells[3,i-title_lines_ace+1];
   //packed
   Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]:=copy(Form_peach.MemoList.Lines[i],16,11);
   try
    csize:=csize+strtoqword(Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]);
   except
   end;
   Form_peach.StringGridList.Cells[11,i-title_lines_ace+1]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]))))+inttostr(length(Form_peach.StringGridList.Cells[4,i-title_lines_ace+1]))+Form_peach.StringGridList.Cells[4,i-title_lines_ace+1];
   //try to guess if it's a folder (there is no univocous marker): no extension and size 0 (may be an empty file without extension!)
   if (Form_peach.StringGridList.Cells[2,i-title_lines_ace+1]='') and
      (Form_peach.StringGridList.Cells[3,i-title_lines_ace+1]='0') then
      begin
      Form_peach.StringGridList.Cells[2,i-title_lines_ace+1]:=txt_list_isfolder;
      Form_peach.StringGridList.Cells[9,i-title_lines_ace+1]:='1'+Form_peach.StringGridList.Cells[1,i-title_lines_ace+1];
      end;
   Form_peach.StringGridList.Cells[6,i-title_lines_ace+1]:='';//attributes (not featured by the RadioGroupQuad)
   {$ENDIF}
   Form_peach.StringGridList.Cells[12,i-title_lines_ace+1]:='0';
   end;
try
Form_peach.StringGridList.SortColRow(true,listsortcol);
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
listsizecol('flat');
if Form_peach.StringGridList.RowCount>1 then
   begin
   Form_peach.StringGridList.Cells[12,1]:='1';
   Form_peach.StringGridList.Row:=1;
   end;
i:=Form_peach.StringGridList.Rowcount-1;
ratio_info_reduced(i,size,csize);
except
   MessageDlg(txt_list_error, mtWarning, [mbOK], 0);
   ratio_info_reduced(0,0,0);
end;
end;

procedure browse_arc(mode:utf8string); //note3: future development, try to identify arc with encrypted content
var
   cl,jobcode,outname,dummy,s:utf8string;
   size,csize:qword;
   i,j,h,title_lines_arc,bottom_lines_arc,rr_lines_arc:integer;
   f:file of byte;
begin
compose_unarc_cl(cl,jobcode,outname,false,'info','all');
execute_cl(cl,'list');
title_lines_arc:=3;
{$IFDEF MSWINDOWS}bottom_lines_arc:=4;{$ENDIF}
{$IFDEF LINUX}bottom_lines_arc:=5;{$ENDIF}
rr_lines_arc:=3;
size:=0;
csize:=0;
if Form_peach.MemoList.Lines.Count=0 then
   begin
   seemencrypted:=true;
   setfocus_pw;
   end
else
   begin
   //add rr_lines_arc to title lines if the archive contains recovery records
   dummy:=Form_peach.MemoList.Lines.Strings[1];
   if dummy[1]<>'D' then title_lines_arc:=title_lines_arc+rr_lines_arc;
   if (mode='browse') or (mode='flat') then
      begin
      Form_peach.StringGridList.RowCount:=Form_peach.MemoList.Lines.Count-title_lines_arc-bottom_lines_arc+2;
      for i:=title_lines_arc to Form_peach.MemoList.Lines.Count-bottom_lines_arc do
         begin
         h:=i-title_lines_arc+1;
         s:=ansitoutf8(copy(Form_peach.MemoList.Lines[i],70,length(Form_peach.MemoList.Lines[i])-69));
         tempcharcodefix(s);
         Form_peach.StringGridList.Cells[1,h]:=uextractfilename(s);
         if seemencrypted=true then Form_peach.StringGridList.Cells[1,h]:=Form_peach.StringGridList.Cells[1,h]+' *';
         if copy(Form_peach.MemoList.Lines[i],22,1)='D' then
            begin
            Form_peach.StringGridList.Cells[8,h]:=s;
            Form_peach.StringGridList.Cells[2,h]:=txt_list_isfolder;
            Form_peach.StringGridList.Cells[9,h]:='1'+Form_peach.StringGridList.Cells[1,h];
            end
         else
            begin
            Form_peach.StringGridList.Cells[8,h]:=s;
            Form_peach.StringGridList.Cells[2,h]:=uextractfileext(Form_peach.StringGridList.Cells[8,h]);
            Form_peach.StringGridList.Cells[9,h]:='2'+Form_peach.StringGridList.Cells[1,h];
            end;
         Form_peach.StringGridList.Cells[3,h]:=trimleft(copy(Form_peach.MemoList.Lines[i],29,15));
         Form_peach.StringGridList.Cells[10,h]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,h]))))+inttostr(length(Form_peach.StringGridList.Cells[3,h]))+Form_peach.StringGridList.Cells[3,h];
         Form_peach.StringGridList.Cells[4,h]:=trimleft(copy(Form_peach.MemoList.Lines[i],45,15));
         Form_peach.StringGridList.Cells[11,h]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[4,h]))))+inttostr(length(Form_peach.StringGridList.Cells[4,h]))+Form_peach.StringGridList.Cells[4,h];
         try
            size:=size+strtoqword(Form_peach.StringGridList.Cells[3,h]);
            csize:=csize+strtoqword(Form_peach.StringGridList.Cells[4,h]);
         except
         end;
         Form_peach.StringGridList.Cells[5,h]:=copy(Form_peach.MemoList.Lines[i],1,19);
         Form_peach.StringGridList.Cells[6,h]:=copy(Form_peach.MemoList.Lines[i],21,7);
         Form_peach.StringGridList.Cells[7,h]:=copy(Form_peach.MemoList.Lines[i],61,8);
         Form_peach.StringGridList.Cells[12,h]:='0';
         end;
      end;
   end;
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
listsizecol('flat');
if Form_peach.StringGridList.RowCount>1 then
   begin
   Form_peach.StringGridList.SortColRow(true,listsortcol);
   Form_peach.StringGridList.Cells[12,1]:='1';
   Form_peach.StringGridList.Row:=1;
   end;
if csize=0 then
   begin
   try
      srcfilesize(Form_peach.OpenDialog3.Filename,csize); //expressed in B
   except
   end;
   end;//if csize is not specified, file size is used
status0:=txt_list_flat;
status1:=inttostr(Form_peach.StringGridList.RowCount-1)+' '+txt_displayed_obj+'; '+inttostr(size)+' B > '+inttostr(csize)+' B';
try
   j:=((csize div 1024) *100) div (size div 1024);
   if (j<101) and (j>1) then Form6.ShapeIO7.Width:=200 * j div 100
   else Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:=inttostr(j)+'%';
except
   Form6.ShapeIO7.Width:=200;
   Form6.Labelpercent1.Caption:='??%';
end;
if (j>1) and (j<200) then status1:=status1+' ('+inttostr(j)+'%)';
if Form_peach.StringGridList.RowCount>1 then
   begin
   Form_peach.StringGridList.SortColRow(true,listsortcol);
   Form_peach.StringGridList.Cells[12,1]:='1';
   Form_peach.StringGridList.Row:=1;
   end
else
   begin
   status1:=txt_list_nomatch+txt_2_7_list_tryflatorpw;
   if mode='silent' then seemencrypted:=true;
   end;
if (mode='browse') or (mode='flat') then Form_peach.StringGridList.Visible:=true
else Form_peach.StringGridList.Visible:=false;
end;

function cleardir(var s:utf8string; cleardirname:boolean):integer;
var
  P: TProcess;
  cl:utf8string;
  pcount:integer;
begin
cleardir:=-1;
if s='' then
   begin
   cleardir:=0;
   exit;
   end;
if not(udirectoryexists((s))) then
   begin
   cleardir:=0;
   exit;
   end;
if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); cleardir:=0; exit; end; //should not happen, because string must be valid for the filesystem
repeat
Form_peach.StringGridList.Cursor:=crHourGlass;
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.Visible=true then Application.ProcessMessages;
if s<>'' then
   begin
   {$IFDEF MSWINDOWS}
   P:=TProcess.Create(nil);
   P.Options := [poNoConsole];
   if upcase(wincomspec)='COMMAND.COM' then cl:=wincomspec+' /c deltree /y "'+s+'"'
   else cl:=wincomspec+' /c rmdir "'+s+'" /s /q';
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   P.Execute;
   pcount:=1;
   while P.Running do
      begin
      animation_busy(pcount);
      if Form_peach.Visible=true then Application.ProcessMessages;
      sleep(100);
      end;
   cleardir:=P.ExitStatus;
   P.Free;
   {$ELSE} //system needs to support rm command
   P:=TProcess.Create(nil);
   P.Options := [poNoConsole, poWaitOnExit];
   cl:='rm -r '+''''+s+'''';
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   P.Execute;
   cleardir:=P.ExitStatus;
   P.Free;
   {$ENDIF}
   end;
if udirectoryexists((s)) then cleardir:=-1
else cleardir:=0;
if s=ptmpdir then
   if cleardir<>0 then
         if MessageDlg(txt_not_removable+char($0D)+char($0A)+s+char($0D)+char($0A)+txt_tempdir, mtWarning, [mbOK,mbAbort], 0)=3 then break
         else
   else
else
   if cleardir<>0 then
         if MessageDlg(txt_not_removable+char($0D)+char($0A)+s, mtWarning, [mbOK,mbAbort], 0)=3 then break
         else
   else;
until cleardir=0;
if cleardirname=true then s:='';
ptmpcode:='';
Form_peach.StringGridList.Cursor:=crDefault;
Form_peach.ctrlrefresh.Glyph:=Brefresh;
end;

function clearfile(s:utf8string):integer;
begin
clearfile:=-1;
if s='' then
   begin
   clearfile:=0;
   exit;
   end;
if not(ufileexists((s))) then
   begin
   clearfile:=0;
   exit;
   end;
Form_peach.StringGridList.Cursor:=crHourGlass;
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.Visible=true then Application.ProcessMessages;
repeat
   if udeletefile((s))=false then
         if MessageDlg(txt_not_removable_file+char($0D)+char($0A)+s, mtWarning, [mbOK,mbAbort], 0)=3 then break
         else
   else clearfile:=0;
until clearfile=0;
Form_peach.StringGridList.Cursor:=crDefault;
Form_peach.ctrlrefresh.Glyph:=Brefresh;
end;

function setpcmenuicon(i:integer):TBitmap;
begin
case i of
   1: setpcmenuicon:=Bfd_supported;
   2: setpcmenuicon:=Bremovable_supported;
   3: setpcmenuicon:=Bhd_supported;
   4: setpcmenuicon:=Bremote_supported;
   5: setpcmenuicon:=Bdvd_supported;
   6: setpcmenuicon:=Bram_supported;
   else setpcmenuicon:=nil;
   end;
end;

procedure countunits(var nunits:qword);
var
   s:utf8string;
   i:qword;
   drivestr:array[1..255] of char;
begin
{$IFDEF MSWINDOWS}
GetLogicalDriveStrings(255,@drivestr);
i:=1;
nunits:=0;
repeat
   s:='';
   nunits:=nunits+1;
   while (i<=255) and (drivestr[i]<>#00) do
      begin
      s:=s+char(drivestr[i]);
      inc(i);
      end;
   inc(i);
until length(s)=0;
nunits:=nunits-1;
{$ENDIF}
end;

procedure populatepcmenu;
var
   s,s1,dtype,dname,uncname:utf8string;
   devname:array [1..26] of utf8string;
   devtype:array [1..26] of integer;
   c:char;
   i,j,k,t:qword;
   sn,mc,flags:dword;
   bufVolumeName, bufFSName: array[0..255] of Char;
   drivestr:array[1..255] of char;
begin
{$IFDEF MSWINDOWS}
if (winver<>'nt6+') and (winver<>'nt5') then
   form_peach.jdspacer.Visible:=false
else
begin
GetLogicalDriveStrings(255,@drivestr);
i:=1;
j:=1;
k:=0;
repeat
   s:='';
   dtype:='';
   dname:='';
   uncname:='';
   bufVolumeName:='';
   k:=k+1;
   while (i<=255) and (drivestr[i]<>#00) do
      begin
      s:=s+char(drivestr[i]);
      inc(i);
      end;
   inc(i);
   if length(s)>0 then
      begin
      c:=upcase(s[1]);
      j:=ord(c)-64;
      if j>2 then
         begin
         //drive type
         t:=getdrivetype(Pchar(s));
         case t of
            2: dtype:=txt_unit_removable;
            3: dtype:=txt_unit_hd;
            4: dtype:=txt_unit_remote;
            5: dtype:=txt_unit_cd;
            6: dtype:=txt_unit_ram;
            else dtype:=txt_unit_unknown;
            end;
         devtype[k]:=t;
         devicon[j]:=setpcmenuicon(devtype[k]);
         //drive name
         GetVolumeInformation(Pchar(s),
         @bufVolumeName, sizeof(bufVolumeName),
         @sn, mc, flags,
         @bufFSName, sizeof(bufFSName));
         if bufVolumeName<>'' then dname:=bufVolumeName
         else dname:=dtype;
         //unc name
         uncname:=getUNCname(Pchar(s));
         if uncname<>'' then
            if uncname[length(uncname)]=directoryseparator then setlength(uncname,length(uncname)-1);
         s1:=s;
         setlength(s1,length(s1)-1);
         if uncname='' then devname[k]:=dname+' ('+s1+')'
         else devname[k]:=uextractfilename(uncname)+' '+txt_on+' '+uextractfilepath(uncname)+' '+dname+' ('+s1+')';
         end
      else
         begin
         s1:=s;
         setlength(s1,length(s1)-1);
         devname[k]:=txt_unit_floppy+' ('+s1+')';
         devtype[k]:=1;
         devicon[j]:=setpcmenuicon(devtype[k]);
         end;
      end;
until length(s)=0;
if devname[1]<>'' then
   begin
   Form_peach.jd1.Visible:=true;
   Form_peach.jd1.Caption:=devname[1];
   Form_peach.jd1.bitmap:=setpcmenuicon(devtype[1]);
   Form_peach.pojd1.Visible:=true;
   Form_peach.pojd1.Caption:=devname[1];
   Form_peach.pojd1.bitmap:=setpcmenuicon(devtype[1]);
   Form_peach.pmjd1.Visible:=true;
   Form_peach.pmjd1.Caption:=devname[1];
   Form_peach.pmjd1.bitmap:=setpcmenuicon(devtype[1]);
   Form_peach.pojdn1.Visible:=true;
   Form_peach.pojdn1.Caption:=devname[1];
   Form_peach.pojdn1.bitmap:=setpcmenuicon(devtype[1]);
   Form_peach.pajdn1.Visible:=true;
   Form_peach.pajdn1.Caption:=devname[1];
   Form_peach.pajdn1.bitmap:=setpcmenuicon(devtype[1]);
   end
else
   begin
   Form_peach.jd1.Visible:=false;
   Form_peach.pojd1.Visible:=false;
   Form_peach.pmjd1.Visible:=false;
   Form_peach.pojdn1.Visible:=false;
   Form_peach.pajdn1.Visible:=false;
   end;
if devname[2]<>'' then
   begin
   Form_peach.jd2.Visible:=true;
   Form_peach.jd2.Caption:=devname[2];
   Form_peach.jd2.bitmap:=setpcmenuicon(devtype[2]);
   Form_peach.pojd2.Visible:=true;
   Form_peach.pojd2.Caption:=devname[2];
   Form_peach.pojd2.bitmap:=setpcmenuicon(devtype[2]);
   Form_peach.pmjd2.Visible:=true;
   Form_peach.pmjd2.Caption:=devname[2];
   Form_peach.pmjd2.bitmap:=setpcmenuicon(devtype[2]);
   Form_peach.pojdn2.Visible:=true;
   Form_peach.pojdn2.Caption:=devname[2];
   Form_peach.pojdn2.bitmap:=setpcmenuicon(devtype[2]);
   Form_peach.pajdn2.Visible:=true;
   Form_peach.pajdn2.Caption:=devname[2];
   Form_peach.pajdn2.bitmap:=setpcmenuicon(devtype[2]);
   end
else
   begin
   Form_peach.jd2.Visible:=false;
   Form_peach.pojd2.Visible:=false;
   Form_peach.pmjd2.Visible:=false;
   Form_peach.pojdn2.Visible:=false;
   Form_peach.pajdn2.Visible:=false;
   end;
if devname[3]<>'' then
   begin
   Form_peach.jd3.Visible:=true;
   Form_peach.jd3.Caption:=devname[3];
   Form_peach.jd3.bitmap:=setpcmenuicon(devtype[3]);
   Form_peach.pojd3.Visible:=true;
   Form_peach.pojd3.Caption:=devname[3];
   Form_peach.pojd3.bitmap:=setpcmenuicon(devtype[3]);
   Form_peach.pmjd3.Visible:=true;
   Form_peach.pmjd3.Caption:=devname[3];
   Form_peach.pmjd3.bitmap:=setpcmenuicon(devtype[3]);
   Form_peach.pojdn3.Visible:=true;
   Form_peach.pojdn3.Caption:=devname[3];
   Form_peach.pojdn3.bitmap:=setpcmenuicon(devtype[3]);
   Form_peach.pajdn3.Visible:=true;
   Form_peach.pajdn3.Caption:=devname[3];
   Form_peach.pajdn3.bitmap:=setpcmenuicon(devtype[3]);
   end
else
   begin
   Form_peach.jd3.Visible:=false;
   Form_peach.pojd3.Visible:=false;
   Form_peach.pmjd3.Visible:=false;
   Form_peach.pojdn3.Visible:=false;
   Form_peach.pajdn3.Visible:=false;
   end;
if devname[4]<>'' then
   begin
   Form_peach.jd4.Visible:=true;
   Form_peach.jd4.Caption:=devname[4];
   Form_peach.jd4.bitmap:=setpcmenuicon(devtype[4]);
   Form_peach.pojd4.Visible:=true;
   Form_peach.pojd4.Caption:=devname[4];
   Form_peach.pojd4.bitmap:=setpcmenuicon(devtype[4]);
   Form_peach.pmjd4.Visible:=true;
   Form_peach.pmjd4.Caption:=devname[4];
   Form_peach.pmjd4.bitmap:=setpcmenuicon(devtype[4]);
   Form_peach.pojdn4.Visible:=true;
   Form_peach.pojdn4.Caption:=devname[4];
   Form_peach.pojdn4.bitmap:=setpcmenuicon(devtype[4]);
   Form_peach.pajdn4.Visible:=true;
   Form_peach.pajdn4.Caption:=devname[4];
   Form_peach.pajdn4.bitmap:=setpcmenuicon(devtype[4]);
   end
else
   begin
   Form_peach.jd4.Visible:=false;
   Form_peach.pojd4.Visible:=false;
   Form_peach.pmjd4.Visible:=false;
   Form_peach.pojdn4.Visible:=false;
   Form_peach.pajdn4.Visible:=false;
   end;
if devname[5]<>'' then
   begin
   Form_peach.jd5.Visible:=true;
   Form_peach.jd5.Caption:=devname[5];
   Form_peach.jd5.bitmap:=setpcmenuicon(devtype[5]);
   Form_peach.pojd5.Visible:=true;
   Form_peach.pojd5.Caption:=devname[5];
   Form_peach.pojd5.bitmap:=setpcmenuicon(devtype[5]);
   Form_peach.pmjd5.Visible:=true;
   Form_peach.pmjd5.Caption:=devname[5];
   Form_peach.pmjd5.bitmap:=setpcmenuicon(devtype[5]);
   Form_peach.pojdn5.Visible:=true;
   Form_peach.pojdn5.Caption:=devname[5];
   Form_peach.pojdn5.bitmap:=setpcmenuicon(devtype[5]);
   Form_peach.pajdn5.Visible:=true;
   Form_peach.pajdn5.Caption:=devname[5];
   Form_peach.pajdn5.bitmap:=setpcmenuicon(devtype[5]);
   end
else
   begin
   Form_peach.jd5.Visible:=false;
   Form_peach.pojd5.Visible:=false;
   Form_peach.pmjd5.Visible:=false;
   Form_peach.pojdn5.Visible:=false;
   Form_peach.pajdn5.Visible:=false;
   end;
if devname[6]<>'' then
   begin
   Form_peach.jd6.Visible:=true;
   Form_peach.jd6.Caption:=devname[6];
   Form_peach.jd6.bitmap:=setpcmenuicon(devtype[6]);
   Form_peach.pojd6.Visible:=true;
   Form_peach.pojd6.Caption:=devname[6];
   Form_peach.pojd6.bitmap:=setpcmenuicon(devtype[6]);
   Form_peach.pmjd6.Visible:=true;
   Form_peach.pmjd6.Caption:=devname[6];
   Form_peach.pmjd6.bitmap:=setpcmenuicon(devtype[6]);
   Form_peach.pojdn6.Visible:=true;
   Form_peach.pojdn6.Caption:=devname[6];
   Form_peach.pojdn6.bitmap:=setpcmenuicon(devtype[6]);
   Form_peach.pajdn6.Visible:=true;
   Form_peach.pajdn6.Caption:=devname[6];
   Form_peach.pajdn6.bitmap:=setpcmenuicon(devtype[6]);
   end
else
   begin
   Form_peach.jd6.Visible:=false;
   Form_peach.pojd6.Visible:=false;
   Form_peach.pmjd6.Visible:=false;
   Form_peach.pojdn6.Visible:=false;
   Form_peach.pajdn6.Visible:=false;
   end;
if devname[7]<>'' then
   begin
   Form_peach.jd7.Visible:=true;
   Form_peach.jd7.Caption:=devname[7];
   Form_peach.jd7.bitmap:=setpcmenuicon(devtype[7]);
   Form_peach.pojd7.Visible:=true;
   Form_peach.pojd7.Caption:=devname[7];
   Form_peach.pojd7.bitmap:=setpcmenuicon(devtype[7]);
   Form_peach.pmjd7.Visible:=true;
   Form_peach.pmjd7.Caption:=devname[7];
   Form_peach.pmjd7.bitmap:=setpcmenuicon(devtype[7]);
   Form_peach.pojdn7.Visible:=true;
   Form_peach.pojdn7.Caption:=devname[7];
   Form_peach.pojdn7.bitmap:=setpcmenuicon(devtype[7]);
   Form_peach.pajdn7.Visible:=true;
   Form_peach.pajdn7.Caption:=devname[7];
   Form_peach.pajdn7.bitmap:=setpcmenuicon(devtype[7]);
   end
else
   begin
   Form_peach.jd7.Visible:=false;
   Form_peach.pojd7.Visible:=false;
   Form_peach.pmjd7.Visible:=false;
   Form_peach.pojdn7.Visible:=false;
   Form_peach.pajdn7.Visible:=false;
   end;
if devname[8]<>'' then
   begin
   Form_peach.jd8.Visible:=true;
   Form_peach.jd8.Caption:=devname[8];
   Form_peach.jd8.bitmap:=setpcmenuicon(devtype[8]);
   Form_peach.pojd8.Visible:=true;
   Form_peach.pojd8.Caption:=devname[8];
   Form_peach.pojd8.bitmap:=setpcmenuicon(devtype[8]);
   Form_peach.pmjd8.Visible:=true;
   Form_peach.pmjd8.Caption:=devname[8];
   Form_peach.pmjd8.bitmap:=setpcmenuicon(devtype[8]);
   Form_peach.pojdn8.Visible:=true;
   Form_peach.pojdn8.Caption:=devname[8];
   Form_peach.pojdn8.bitmap:=setpcmenuicon(devtype[8]);
   Form_peach.pajdn8.Visible:=true;
   Form_peach.pajdn8.Caption:=devname[8];
   Form_peach.pajdn8.bitmap:=setpcmenuicon(devtype[8]);
   end
else
   begin
   Form_peach.jd8.Visible:=false;
   Form_peach.pojd8.Visible:=false;
   Form_peach.pmjd8.Visible:=false;
   Form_peach.pojdn8.Visible:=false;
   Form_peach.pajdn8.Visible:=false;
   end;
if devname[9]<>'' then
   begin
   Form_peach.jd9.Visible:=true;
   Form_peach.jd9.Caption:=devname[9];
   Form_peach.jd9.bitmap:=setpcmenuicon(devtype[9]);
   Form_peach.pojd9.Visible:=true;
   Form_peach.pojd9.Caption:=devname[9];
   Form_peach.pojd9.bitmap:=setpcmenuicon(devtype[9]);
   Form_peach.pmjd9.Visible:=true;
   Form_peach.pmjd9.Caption:=devname[9];
   Form_peach.pmjd9.bitmap:=setpcmenuicon(devtype[9]);
   Form_peach.pojdn9.Visible:=true;
   Form_peach.pojdn9.Caption:=devname[9];
   Form_peach.pojdn9.bitmap:=setpcmenuicon(devtype[9]);
   Form_peach.pajdn9.Visible:=true;
   Form_peach.pajdn9.Caption:=devname[9];
   Form_peach.pajdn9.bitmap:=setpcmenuicon(devtype[9]);
   end
else
   begin
   Form_peach.jd9.Visible:=false;
   Form_peach.pojd9.Visible:=false;
   Form_peach.pmjd9.Visible:=false;
   Form_peach.pojdn9.Visible:=false;
   Form_peach.pajdn9.Visible:=false;
   end;
if devname[10]<>'' then
   begin
   Form_peach.jd10.Visible:=true;
   Form_peach.jd10.Caption:=devname[10];
   Form_peach.jd10.bitmap:=setpcmenuicon(devtype[10]);
   Form_peach.pojd10.Visible:=true;
   Form_peach.pojd10.Caption:=devname[10];
   Form_peach.pojd10.bitmap:=setpcmenuicon(devtype[10]);
   Form_peach.pmjd10.Visible:=true;
   Form_peach.pmjd10.Caption:=devname[10];
   Form_peach.pmjd10.bitmap:=setpcmenuicon(devtype[10]);
   Form_peach.pojdn10.Visible:=true;
   Form_peach.pojdn10.Caption:=devname[10];
   Form_peach.pojdn10.bitmap:=setpcmenuicon(devtype[10]);
   Form_peach.pajdn10.Visible:=true;
   Form_peach.pajdn10.Caption:=devname[10];
   Form_peach.pajdn10.bitmap:=setpcmenuicon(devtype[10]);
   end
else
   begin
   Form_peach.jd10.Visible:=false;
   Form_peach.pojd10.Visible:=false;
   Form_peach.pmjd10.Visible:=false;
   Form_peach.pojdn10.Visible:=false;
   Form_peach.pajdn10.Visible:=false;
   end;
if devname[11]<>'' then
   begin
   Form_peach.jd11.Visible:=true;
   Form_peach.jd11.Caption:=devname[11];
   Form_peach.jd11.bitmap:=setpcmenuicon(devtype[11]);
   Form_peach.pojd11.Visible:=true;
   Form_peach.pojd11.Caption:=devname[11];
   Form_peach.pojd11.bitmap:=setpcmenuicon(devtype[11]);
   Form_peach.pmjd11.Visible:=true;
   Form_peach.pmjd11.Caption:=devname[11];
   Form_peach.pmjd11.bitmap:=setpcmenuicon(devtype[11]);
   Form_peach.pojdn11.Visible:=true;
   Form_peach.pojdn11.Caption:=devname[11];
   Form_peach.pojdn11.bitmap:=setpcmenuicon(devtype[11]);
   Form_peach.pajdn11.Visible:=true;
   Form_peach.pajdn11.Caption:=devname[11];
   Form_peach.pajdn11.bitmap:=setpcmenuicon(devtype[11]);
   end
else
   begin
   Form_peach.jd11.Visible:=false;
   Form_peach.pojd11.Visible:=false;
   Form_peach.pmjd11.Visible:=false;
   Form_peach.pojdn11.Visible:=false;
   Form_peach.pajdn11.Visible:=false;
   end;
if devname[12]<>'' then
   begin
   Form_peach.jd12.Visible:=true;
   Form_peach.jd12.Caption:=devname[12];
   Form_peach.jd12.bitmap:=setpcmenuicon(devtype[12]);
   Form_peach.pojd12.Visible:=true;
   Form_peach.pojd12.Caption:=devname[12];
   Form_peach.pojd12.bitmap:=setpcmenuicon(devtype[12]);
   Form_peach.pmjd12.Visible:=true;
   Form_peach.pmjd12.Caption:=devname[12];
   Form_peach.pmjd12.bitmap:=setpcmenuicon(devtype[12]);
   Form_peach.pojdn12.Visible:=true;
   Form_peach.pojdn12.Caption:=devname[12];
   Form_peach.pojdn12.bitmap:=setpcmenuicon(devtype[12]);
   Form_peach.pajdn12.Visible:=true;
   Form_peach.pajdn12.Caption:=devname[12];
   Form_peach.pajdn12.bitmap:=setpcmenuicon(devtype[12]);
   end
else
   begin
   Form_peach.jd12.Visible:=false;
   Form_peach.pojd12.Visible:=false;
   Form_peach.pmjd12.Visible:=false;
   Form_peach.pojdn12.Visible:=false;
   Form_peach.pajdn12.Visible:=false;
   end;
if devname[13]<>'' then
   begin
   Form_peach.jd13.Visible:=true;
   Form_peach.jd13.Caption:=devname[13];
   Form_peach.jd13.bitmap:=setpcmenuicon(devtype[13]);
   Form_peach.pojd13.Visible:=true;
   Form_peach.pojd13.Caption:=devname[13];
   Form_peach.pojd13.bitmap:=setpcmenuicon(devtype[13]);
   Form_peach.pmjd13.Visible:=true;
   Form_peach.pmjd13.Caption:=devname[13];
   Form_peach.pmjd13.bitmap:=setpcmenuicon(devtype[13]);
   Form_peach.pojdn13.Visible:=true;
   Form_peach.pojdn13.Caption:=devname[13];
   Form_peach.pojdn13.bitmap:=setpcmenuicon(devtype[13]);
   Form_peach.pajdn13.Visible:=true;
   Form_peach.pajdn13.Caption:=devname[13];
   Form_peach.pajdn13.bitmap:=setpcmenuicon(devtype[13]);
   end
else
   begin
   Form_peach.jd13.Visible:=false;
   Form_peach.pojd13.Visible:=false;
   Form_peach.pmjd13.Visible:=false;
   Form_peach.pojdn13.Visible:=false;
   Form_peach.pajdn13.Visible:=false;
   end;
if devname[14]<>'' then
   begin
   Form_peach.jd14.Visible:=true;
   Form_peach.jd14.Caption:=devname[14];
   Form_peach.jd14.bitmap:=setpcmenuicon(devtype[14]);
   Form_peach.pojd14.Visible:=true;
   Form_peach.pojd14.Caption:=devname[14];
   Form_peach.pojd14.bitmap:=setpcmenuicon(devtype[14]);
   Form_peach.pmjd14.Visible:=true;
   Form_peach.pmjd14.Caption:=devname[14];
   Form_peach.pmjd14.bitmap:=setpcmenuicon(devtype[14]);
   Form_peach.pojdn14.Visible:=true;
   Form_peach.pojdn14.Caption:=devname[14];
   Form_peach.pojdn14.bitmap:=setpcmenuicon(devtype[14]);
   Form_peach.pajdn14.Visible:=true;
   Form_peach.pajdn14.Caption:=devname[14];
   Form_peach.pajdn14.bitmap:=setpcmenuicon(devtype[14]);
   end
else
   begin
   Form_peach.jd14.Visible:=false;
   Form_peach.pojd14.Visible:=false;
   Form_peach.pmjd14.Visible:=false;
   Form_peach.pojdn14.Visible:=false;
   Form_peach.pajdn14.Visible:=false;
   end;
if devname[15]<>'' then
   begin
   Form_peach.jd15.Visible:=true;
   Form_peach.jd15.Caption:=devname[15];
   Form_peach.jd15.bitmap:=setpcmenuicon(devtype[15]);
   Form_peach.pojd15.Visible:=true;
   Form_peach.pojd15.Caption:=devname[15];
   Form_peach.pojd15.bitmap:=setpcmenuicon(devtype[15]);
   Form_peach.pmjd15.Visible:=true;
   Form_peach.pmjd15.Caption:=devname[15];
   Form_peach.pmjd15.bitmap:=setpcmenuicon(devtype[15]);
   Form_peach.pojdn15.Visible:=true;
   Form_peach.pojdn15.Caption:=devname[15];
   Form_peach.pojdn15.bitmap:=setpcmenuicon(devtype[15]);
   Form_peach.pajdn15.Visible:=true;
   Form_peach.pajdn15.Caption:=devname[15];
   Form_peach.pajdn15.bitmap:=setpcmenuicon(devtype[15]);
   end
else
   begin
   Form_peach.jd15.Visible:=false;
   Form_peach.pojd15.Visible:=false;
   Form_peach.pmjd15.Visible:=false;
   Form_peach.pojdn15.Visible:=false;
   Form_peach.pajdn15.Visible:=false;
   end;
if devname[16]<>'' then
   begin
   Form_peach.jd16.Visible:=true;
   Form_peach.jd16.Caption:=devname[16];
   Form_peach.jd16.bitmap:=setpcmenuicon(devtype[16]);
   Form_peach.pojd16.Visible:=true;
   Form_peach.pojd16.Caption:=devname[16];
   Form_peach.pojd16.bitmap:=setpcmenuicon(devtype[16]);
   Form_peach.pmjd16.Visible:=true;
   Form_peach.pmjd16.Caption:=devname[16];
   Form_peach.pmjd16.bitmap:=setpcmenuicon(devtype[16]);
   Form_peach.pojdn16.Visible:=true;
   Form_peach.pojdn16.Caption:=devname[16];
   Form_peach.pojdn16.bitmap:=setpcmenuicon(devtype[16]);
   Form_peach.pajdn16.Visible:=true;
   Form_peach.pajdn16.Caption:=devname[16];
   Form_peach.pajdn16.bitmap:=setpcmenuicon(devtype[16]);
   end
else
   begin
   Form_peach.jd16.Visible:=false;
   Form_peach.pojd16.Visible:=false;
   Form_peach.pmjd16.Visible:=false;
   Form_peach.pojdn16.Visible:=false;
   Form_peach.pajdn16.Visible:=false;
   end;
if devname[17]<>'' then
   begin
   Form_peach.jd17.Visible:=true;
   Form_peach.jd17.Caption:=devname[17];
   Form_peach.jd17.bitmap:=setpcmenuicon(devtype[17]);
   Form_peach.pojd17.Visible:=true;
   Form_peach.pojd17.Caption:=devname[17];
   Form_peach.pojd17.bitmap:=setpcmenuicon(devtype[17]);
   Form_peach.pmjd17.Visible:=true;
   Form_peach.pmjd17.Caption:=devname[17];
   Form_peach.pmjd17.bitmap:=setpcmenuicon(devtype[17]);
   Form_peach.pojdn17.Visible:=true;
   Form_peach.pojdn17.Caption:=devname[17];
   Form_peach.pojdn17.bitmap:=setpcmenuicon(devtype[17]);
   Form_peach.pajdn17.Visible:=true;
   Form_peach.pajdn17.Caption:=devname[17];
   Form_peach.pajdn17.bitmap:=setpcmenuicon(devtype[17]);
   end
else
   begin
   Form_peach.jd17.Visible:=false;
   Form_peach.pojd17.Visible:=false;
   Form_peach.pmjd17.Visible:=false;
   Form_peach.pojdn17.Visible:=false;
   Form_peach.pajdn17.Visible:=false;
   end;
if devname[18]<>'' then
   begin
   Form_peach.jd18.Visible:=true;
   Form_peach.jd18.Caption:=devname[18];
   Form_peach.jd18.bitmap:=setpcmenuicon(devtype[18]);
   Form_peach.pojd18.Visible:=true;
   Form_peach.pojd18.Caption:=devname[18];
   Form_peach.pojd18.bitmap:=setpcmenuicon(devtype[18]);
   Form_peach.pmjd18.Visible:=true;
   Form_peach.pmjd18.Caption:=devname[18];
   Form_peach.pmjd18.bitmap:=setpcmenuicon(devtype[18]);
   Form_peach.pojdn18.Visible:=true;
   Form_peach.pojdn18.Caption:=devname[18];
   Form_peach.pojdn18.bitmap:=setpcmenuicon(devtype[18]);
   Form_peach.pajdn18.Visible:=true;
   Form_peach.pajdn18.Caption:=devname[18];
   Form_peach.pajdn18.bitmap:=setpcmenuicon(devtype[18]);
   end
else
   begin
   Form_peach.jd18.Visible:=false;
   Form_peach.pojd18.Visible:=false;
   Form_peach.pmjd18.Visible:=false;
   Form_peach.pojdn18.Visible:=false;
   Form_peach.pajdn18.Visible:=false;
   end;
if devname[19]<>'' then
   begin
   Form_peach.jd19.Visible:=true;
   Form_peach.jd19.Caption:=devname[19];
   Form_peach.jd19.bitmap:=setpcmenuicon(devtype[19]);
   Form_peach.pojd19.Visible:=true;
   Form_peach.pojd19.Caption:=devname[19];
   Form_peach.pojd19.bitmap:=setpcmenuicon(devtype[19]);
   Form_peach.pmjd19.Visible:=true;
   Form_peach.pmjd19.Caption:=devname[19];
   Form_peach.pmjd19.bitmap:=setpcmenuicon(devtype[19]);
   Form_peach.pojdn19.Visible:=true;
   Form_peach.pojdn19.Caption:=devname[19];
   Form_peach.pojdn19.bitmap:=setpcmenuicon(devtype[19]);
   Form_peach.pajdn19.Visible:=true;
   Form_peach.pajdn19.Caption:=devname[19];
   Form_peach.pajdn19.bitmap:=setpcmenuicon(devtype[19]);
   end
else
   begin
   Form_peach.jd19.Visible:=false;
   Form_peach.pojd19.Visible:=false;
   Form_peach.pmjd19.Visible:=false;
   Form_peach.pojdn19.Visible:=false;
   Form_peach.pajdn19.Visible:=false;
   end;
if devname[20]<>'' then
   begin
   Form_peach.jd20.Visible:=true;
   Form_peach.jd20.Caption:=devname[20];
   Form_peach.jd20.bitmap:=setpcmenuicon(devtype[20]);
   Form_peach.pojd20.Visible:=true;
   Form_peach.pojd20.Caption:=devname[20];
   Form_peach.pojd20.bitmap:=setpcmenuicon(devtype[20]);
   Form_peach.pmjd20.Visible:=true;
   Form_peach.pmjd20.Caption:=devname[20];
   Form_peach.pmjd20.bitmap:=setpcmenuicon(devtype[20]);
   Form_peach.pojdn20.Visible:=true;
   Form_peach.pojdn20.Caption:=devname[20];
   Form_peach.pojdn20.bitmap:=setpcmenuicon(devtype[20]);
   Form_peach.pajdn20.Visible:=true;
   Form_peach.pajdn20.Caption:=devname[20];
   Form_peach.pajdn20.bitmap:=setpcmenuicon(devtype[20]);
   end
else
   begin
   Form_peach.jd20.Visible:=false;
   Form_peach.pojd20.Visible:=false;
   Form_peach.pmjd20.Visible:=false;
   Form_peach.pojdn20.Visible:=false;
   Form_peach.pajdn20.Visible:=false;
   end;
if devname[21]<>'' then
   begin
   Form_peach.jd21.Visible:=true;
   Form_peach.jd21.Caption:=devname[21];
   Form_peach.jd21.bitmap:=setpcmenuicon(devtype[21]);
   Form_peach.pojd21.Visible:=true;
   Form_peach.pojd21.Caption:=devname[21];
   Form_peach.pojd21.bitmap:=setpcmenuicon(devtype[21]);
   Form_peach.pmjd21.Visible:=true;
   Form_peach.pmjd21.Caption:=devname[21];
   Form_peach.pmjd21.bitmap:=setpcmenuicon(devtype[21]);
   Form_peach.pojdn21.Visible:=true;
   Form_peach.pojdn21.Caption:=devname[21];
   Form_peach.pojdn21.bitmap:=setpcmenuicon(devtype[21]);
   Form_peach.pajdn21.Visible:=true;
   Form_peach.pajdn21.Caption:=devname[21];
   Form_peach.pajdn21.bitmap:=setpcmenuicon(devtype[21]);
   end
else
   begin
   Form_peach.jd21.Visible:=false;
   Form_peach.pojd21.Visible:=false;
   Form_peach.pmjd21.Visible:=false;
   Form_peach.pojdn21.Visible:=false;
   Form_peach.pajdn21.Visible:=false;
   end;
if devname[22]<>'' then
   begin
   Form_peach.jd22.Visible:=true;
   Form_peach.jd22.Caption:=devname[22];
   Form_peach.jd22.bitmap:=setpcmenuicon(devtype[22]);
   Form_peach.pojd22.Visible:=true;
   Form_peach.pojd22.Caption:=devname[22];
   Form_peach.pojd22.bitmap:=setpcmenuicon(devtype[22]);
   Form_peach.pmjd22.Visible:=true;
   Form_peach.pmjd22.Caption:=devname[22];
   Form_peach.pmjd22.bitmap:=setpcmenuicon(devtype[22]);
   Form_peach.pojdn22.Visible:=true;
   Form_peach.pojdn22.Caption:=devname[22];
   Form_peach.pojdn22.bitmap:=setpcmenuicon(devtype[22]);
   Form_peach.pajdn22.Visible:=true;
   Form_peach.pajdn22.Caption:=devname[22];
   Form_peach.pajdn22.bitmap:=setpcmenuicon(devtype[22]);
   end
else
   begin
   Form_peach.jd22.Visible:=false;
   Form_peach.pojd22.Visible:=false;
   Form_peach.pmjd22.Visible:=false;
   Form_peach.pojdn22.Visible:=false;
   Form_peach.pajdn22.Visible:=false;
   end;
if devname[23]<>'' then
   begin
   Form_peach.jd23.Visible:=true;
   Form_peach.jd23.Caption:=devname[23];
   Form_peach.jd23.bitmap:=setpcmenuicon(devtype[23]);
   Form_peach.pojd23.Visible:=true;
   Form_peach.pojd23.Caption:=devname[23];
   Form_peach.pojd23.bitmap:=setpcmenuicon(devtype[23]);
   Form_peach.pmjd23.Visible:=true;
   Form_peach.pmjd23.Caption:=devname[23];
   Form_peach.pmjd23.bitmap:=setpcmenuicon(devtype[23]);
   Form_peach.pojdn23.Visible:=true;
   Form_peach.pojdn23.Caption:=devname[23];
   Form_peach.pojdn23.bitmap:=setpcmenuicon(devtype[23]);
   Form_peach.pajdn23.Visible:=true;
   Form_peach.pajdn23.Caption:=devname[23];
   Form_peach.pajdn23.bitmap:=setpcmenuicon(devtype[23]);
   end
else
   begin
   Form_peach.jd23.Visible:=false;
   Form_peach.pojd23.Visible:=false;
   Form_peach.pmjd23.Visible:=false;
   Form_peach.pojdn23.Visible:=false;
   Form_peach.pajdn23.Visible:=false;
   end;
if devname[24]<>'' then
   begin
   Form_peach.jd24.Visible:=true;
   Form_peach.jd24.Caption:=devname[24];
   Form_peach.jd24.bitmap:=setpcmenuicon(devtype[24]);
   Form_peach.pojd24.Visible:=true;
   Form_peach.pojd24.Caption:=devname[24];
   Form_peach.pojd24.bitmap:=setpcmenuicon(devtype[24]);
   Form_peach.pmjd24.Visible:=true;
   Form_peach.pmjd24.Caption:=devname[24];
   Form_peach.pmjd24.bitmap:=setpcmenuicon(devtype[24]);
   Form_peach.pojdn24.Visible:=true;
   Form_peach.pojdn24.Caption:=devname[24];
   Form_peach.pojdn24.bitmap:=setpcmenuicon(devtype[24]);
   Form_peach.pajdn24.Visible:=true;
   Form_peach.pajdn24.Caption:=devname[24];
   Form_peach.pajdn24.bitmap:=setpcmenuicon(devtype[24]);
   end
else
   begin
   Form_peach.jd24.Visible:=false;
   Form_peach.pojd24.Visible:=false;
   Form_peach.pmjd24.Visible:=false;
   Form_peach.pojdn24.Visible:=false;
   Form_peach.pajdn24.Visible:=false;
   end;
if devname[25]<>'' then
   begin
   Form_peach.jd25.Visible:=true;
   Form_peach.jd25.Caption:=devname[25];
   Form_peach.jd25.bitmap:=setpcmenuicon(devtype[25]);
   Form_peach.pojd25.Visible:=true;
   Form_peach.pojd25.Caption:=devname[25];
   Form_peach.pojd25.bitmap:=setpcmenuicon(devtype[25]);
   Form_peach.pmjd25.Visible:=true;
   Form_peach.pmjd25.Caption:=devname[25];
   Form_peach.pmjd25.bitmap:=setpcmenuicon(devtype[25]);
   Form_peach.pojdn25.Visible:=true;
   Form_peach.pojdn25.Caption:=devname[25];
   Form_peach.pojdn25.bitmap:=setpcmenuicon(devtype[25]);
   Form_peach.pajdn25.Visible:=true;
   Form_peach.pajdn25.Caption:=devname[25];
   Form_peach.pajdn25.bitmap:=setpcmenuicon(devtype[25]);
   end
else
   begin
   Form_peach.jd25.Visible:=false;
   Form_peach.pojd25.Visible:=false;
   Form_peach.pmjd25.Visible:=false;
   Form_peach.pojdn25.Visible:=false;
   Form_peach.pajdn25.Visible:=false;
   end;
if devname[26]<>'' then
   begin
   Form_peach.jd26.Visible:=true;
   Form_peach.jd26.Caption:=devname[26];
   Form_peach.jd26.bitmap:=setpcmenuicon(devtype[26]);
   Form_peach.pojd26.Visible:=true;
   Form_peach.pojd26.Caption:=devname[26];
   Form_peach.pojd26.bitmap:=setpcmenuicon(devtype[26]);
   Form_peach.pmjd26.Visible:=true;
   Form_peach.pmjd26.Caption:=devname[26];
   Form_peach.pmjd26.bitmap:=setpcmenuicon(devtype[26]);
   Form_peach.pojdn26.Visible:=true;
   Form_peach.pojdn26.Caption:=devname[26];
   Form_peach.pojdn26.bitmap:=setpcmenuicon(devtype[26]);
   Form_peach.pajdn26.Visible:=true;
   Form_peach.pajdn26.Caption:=devname[26];
   Form_peach.pajdn26.bitmap:=setpcmenuicon(devtype[26]);
   end
else
   begin
   Form_peach.jd26.Visible:=false;
   Form_peach.pojd26.Visible:=false;
   Form_peach.pmjd26.Visible:=false;
   Form_peach.pojdn26.Visible:=false;
   Form_peach.pajdn26.Visible:=false;
   end;
end;
{$ELSE}
{$ENDIF}
end;

procedure set_lastobj;
var
   i:integer;
   lastobjfound:boolean;
begin
for i:=1 to Form_peach.StringGridList.Rowcount-1 do
   if Form_peach.StringGridList.Cells[12,i]='0' then
   else Form_peach.StringGridList.Cells[12,i]:='0';
lastobjfound:=false;
if length(lastobj)>1 then
   if lastobj[length(lastobj)]=directoryseparator then setlength(lastobj,length(lastobj)-1);
if lastobj<>'' then
   for i:=1 to Form_peach.StringGridList.RowCount-1 do
      if Form_Peach.StringGridList.Cells[8,i]=lastobj then
         begin
         lastobjfound:=true;
         break;
         end;
if lastobjfound=false then i:=1;
Form_peach.StringGridList.Cells[12,i]:='1';
Form_peach.StringGridList.Row:=i;
end;

procedure sort_az_stringgridlist(c:integer);
begin
Form_peach.StringGridList.SortColRow(true,c);
az:=true;
end;

procedure sort_za_stringgridlist;
var
   i,rc:integer;
begin
Form_peach.StringGridTmp.ColCount:=Form_peach.StringGridList.ColCount;
Form_peach.StringGridTmp.RowCount:=Form_peach.StringGridList.RowCount;
rc:=Form_peach.StringGridList.Rowcount;
for i:=1 to Form_peach.StringGridList.Rowcount-1 do Form_peach.StringGridTmp.Rows[i]:=Form_peach.StringGridList.Rows[rc-i];
for i:=1 to Form_peach.StringGridList.Rowcount-1 do Form_peach.StringGridList.Rows[i]:=Form_peach.StringGridTmp.Rows[i];
az:=false;
Form_peach.StringGridTmp.Clear;
end;

procedure listdir(s:utf8string; recmode:boolean);
var
   nfiles,ndirs,size,sizefree,sizetotal,perfree: qword;
   flist: TFoundList;
   fsizes: TFoundListSizes;
   ftimes: TFoundListAges;
   fattr: TFoundListAttrib;
   fattr_dec: TFoundList;
   fname:utf8string;
   st,s1:utf8string;
   d:char;
   i,j:integer;
begin
Form_peach.Timer2.Interval:=500;
{$IFDEF MSWINDOWS}
if s=txt_mypc then
   begin
   listpc;
   exit;
   end;
{$ENDIF}
if listingdir=true then exit;
listingdir:=true;
prepare_filebrowser;
if not(udirectoryexists((s))) then
   begin
   s1:=s;
   setlength(s1,length(s1)-1);
   s1:=uextractfilepath(s1);
   if s1=directoryseparator+directoryseparator then listpc
   else
      MessageDlg(s+' '+txt_not_accessible, mtWarning, [mbOK], 0);
      listingdir:=false;
      exit;
   end;
getcurrentfs(s,currentfs);
currentcomp:='';
status2:=currentfs;
if s<>'' then if s[length(s)]<>directoryseparator then Form_peach.EditOpenIn.Text:=s+directoryseparator
else Form_peach.EditOpenIn.Text:=s;
setbuttonmenus;
Form_peach.EditOpenOut.Text:=Form_peach.EditOpenIn.Text;
Form_peach.StringGridList.Cells[4,0]:=txt_compressed_size;
Form_peach.StringGridList.Cells[5,0]:=txt_datetime;
if recmode=true then Form_peach.StringGridList.Cursor:=crHourGlass;
setsearch_enabled;
if ptmpcode<>'' then cleardir(ptmpdir,true);
Form_peach.StringGridList.RowCount:=1;
status0:=txt_list_searching;
status1:=txt_please_wait;
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.visible=true then Application.Processmessages;
if Form_peach.EditUn7zaFilter.Text='' then Form_peach.EditUn7zaFilter.Text:='*';
listdetails(s,Form_peach.EditUn7zaFilter.Text,faAnyFile,recmode,false,nfiles,ndirs,flist,fsizes,ftimes,fattr,fattr_dec);
Form_peach.StringGridList.RowCount:=nfiles+1;
size:=0;
for i:=1 to (nfiles) do
   begin
   fname:=flist[i-1];
   if fname<>'' then
   if fname[length(fname)]=directoryseparator then
      begin
      Form_Peach.StringGridList.Cells[2,i]:=txt_list_isfolder;
      setlength(fname,length(fname)-1);
      Form_Peach.StringGridList.Cells[9,i]:='1'+uextractfilename(fname);
      end
   else
      begin
      Form_Peach.StringGridList.Cells[2,i]:=uextractfileext(fname);
      Form_Peach.StringGridList.Cells[9,i]:='2'+uextractfilename(fname);
      end;
   Form_Peach.StringGridList.Cells[1,i]:=uextractfilename(fname);
   Form_Peach.StringGridList.Cells[8,i]:=(fname);
   Form_Peach.StringGridList.Cells[3,i]:=inttostr(fsizes[i-1]);
   size:=size+fsizes[i-1];
   Form_Peach.StringGridList.Cells[10,i]:=inttostr(length(inttostr(length(Form_peach.StringGridList.Cells[3,i]))))+inttostr(length(Form_peach.StringGridList.Cells[3,i]))+Form_Peach.StringGridList.Cells[3,i];
   Form_Peach.StringGridList.Cells[5,i]:=datetimetostr(filedatetodatetime(ftimes[i-1]));
   dword2decodedFileAttributes(fattr[i-1],fname);
   Form_Peach.StringGridList.Cells[6,i]:=fname;
   Form_peach.StringGridList.Cells[12,i]:='0';
   end;
if Form_peach.StringGridList.Rowcount<2 then Form_peach.StringGridList.Rowcount:=2;
if recmode=true then listsizecol('flat') else listsizecol('browse');
status0:=txt_list_sorting;
status1:=inttostr(nfiles)+' '+txt_displayed_obj+'; '+inttostr(size)+' B';
if recmode=true then Form_peach.labelstatus.Caption:=status0+' '+status1+' ['+status2+']';
Form_peach.ctrlrefresh.Glyph:=Bp0;
if Form_peach.visible=true then Application.Processmessages;
Form_peach.StringGridList.SortColRow(true,listsortcol);
set_lastobj;
lastobj:=Form_peach.EditOpenIn.Text;
if recmode=false then status0:=txt_list_browsing
else status0:=txt_list_flat;
if recmode=true then Form_peach.labelstatus.Caption:=status0+' '+status1+' ['+status2+']';
Form_peach.StringGridList.Cursor:=crDefault;
Form_peach.ctrlrefresh.Glyph:=Brefresh;
st:=Form_peach.EditOpenIn.Text;
setlength(st,length(st)-1);
if uextractfilename(st)<>'' then st:=uextractfilename(st);
{$IFDEF MSWINDOWS}
if st<>'' then if st[length(st)]=':' then st:=st+directoryseparator;
{$ENDIF}//disk names
Form_peach.Caption:=st+'; '+status1;
Form6.LabelUn7zInputStatus2.Caption:=status0+' '+status1;
Form6.LabelUn7zInputStatus3.Caption:=status2;
{$IFDEF MSWINDOWS}
d:=Form_peach.EditOpenIn.Text[1];
if d<>directoryseparator then
   begin
   j:=ord(upcase(d))-64;
   if j>2 then
      begin
      sizefree:=diskfree(j);
      sizetotal:=disksize(j);
      perfree:=(sizefree*100) div sizetotal;
      end;
   Form6.LabelUn7zInputStatus1.Caption:=copy(Form_peach.EditOpenIn.Caption,1,3)+' '+inttostr(sizetotal)+' B, '+inttostr(sizefree)+' B '+txt_free2+' ('+inttostr(perfree)+'%)';
   Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus1.Caption;
   Form6.ImageInfoOpen5.Visible:=true;
   Form6.ImageInfoOpen6.Visible:=false;
   Form6.ShapeIO6.Visible:=true;
   Form6.ShapeIO7.Visible:=false;
   Form6.ShapeIO8.Visible:=true;
   Form6.ShapeIO9.Visible:=false;
   Form6.LabelPercent.Visible:=true;
   Form6.LabelPercent1.Visible:=false;
   Form6.ShapeIO6.Width:=200 * perfree div 100;
   Form6.ShapeIO6.Hint:='HD';
   Form6.LabelPercent.Caption:=inttostr(perfree)+'%';
   Form6.ImageInfoOpen5.Picture.Bitmap:=Bhd_supported;
   Form6.ImageInfoOpen5.Transparent:=true;
   Form_peach.ImageIO.Glyph:=Form6.ImageInfoOpen5.Picture.Bitmap;
   end
else //unmapped network unit
   begin
   Form6.LabelUn7zInputStatus3.Caption:=Form_peach.EditOpenIn.Caption;
   Form6.LabelUn7zInputStatus1.Caption:=' ';
   Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus3.Caption;
   Form6.LabelPercent.Caption:='na%'; //both must be valorized because of a control on pagechange (calling refresh if page2 is not correctly valorized, i.e. because it was not available during form initialization)
   Form6.ImageInfoOpen5.Visible:=false;
   Form6.ImageInfoOpen6.Visible:=false;
   Form6.ShapeIO6.Visible:=false;
   Form6.ShapeIO7.Visible:=false;
   Form6.ShapeIO8.Visible:=false;
   Form6.ShapeIO9.Visible:=false;
   Form6.LabelPercent.Visible:=false;
   Form6.LabelPercent1.Visible:=false;
   end;
{$ELSE}
Form6.LabelUn7zInputStatus1.Caption:=' ';
Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus2.Caption;
Form6.LabelPercent.Caption:='na%';
Form6.ImageInfoOpen5.Visible:=false;
Form6.ImageInfoOpen6.Visible:=false;
Form6.ShapeIO6.Visible:=false;
Form6.ShapeIO7.Visible:=false;
Form6.ShapeIO8.Visible:=false;
Form6.ShapeIO9.Visible:=false;
Form6.LabelPercent.Visible:=false;
Form6.LabelPercent1.Visible:=false;
{$ENDIF}
indir:=s;
//menuitemopen_ext_caption;
listingdir:=false;
end;

procedure open_prepare_reduced;
begin
Form_peach.StringGridList.Visible:=true;
Form_peach.EditUn7zaFilter.Text:='*';
if fun='UNCUSTOM' then
   begin
   Form_peach.EditNameCustom1.Caption:=Form_peach.pcustom1.caption;
   end;
Form_peach.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename);
status0:=txt_list_browsing;
status1:=txt_list_na;
if fun='FILEBROWSER' then listdir(Form_peach.OpenDialog3.Filename, false)
else
   begin
   indir:=uextractfilepath(Form_peach.OpenDialog3.Filename);
   end;
if fun='UNCUSTOM' then browse_single;
if fun='UNLPAQ' then browse_single;
if fun='UNPAQ' then browse_paq;
if fun='UNZPAQ' then browse_zpaq;
if fun='UNQUAD' then browse_single;
if fun='UNACE' then browse_ace;
if fun='UNARC' then
   begin
   browse_arc('browse');//presently supported only flat browsing
   testencrypted_open;
   end;
Form6.ShapeIO6.Width:=Form6.ShapeIO7.Width;
Form6.Labelpercent.Caption:=Form6.Labelpercent1.Caption;
setfocus_stringgridlist;
addtohistory;
end;

procedure open_prepare_un7z;
begin
Form_peach.sm_delete.Enabled:=true;
Form_peach.po_securedelete.Visible:=false;
Form_peach.po_delete.Caption:=txt_2_5_delete_fromarchive;
Form_peach.pmsecuredelete.Visible:=false;
Form_peach.pmcpyto.Visible:=false;
Form_peach.pmmvto.Visible:=false;
Form_peach.pmquickdelete.Caption:=txt_2_5_delete_fromarchive;
end;

procedure open_prepare_unarc;
begin
Form_peach.sm_delete.Enabled:=true;
Form_peach.po_securedelete.Visible:=false;
Form_peach.po_delete.Caption:=txt_2_5_delete_fromarchive;
Form_peach.pmsecuredelete.Visible:=false;
Form_peach.pmcpyto.Visible:=false;
Form_peach.pmmvto.Visible:=false;
Form_peach.pmquickdelete.Caption:=txt_2_5_delete_fromarchive;
end;

procedure open_7z_prepare;
begin
case browserstyle of
   0: begin
      status0:=txt_list_browsing;
      browsingmode7z:=0;
      end;
   1: begin
      status0:=txt_list_flat;
      browsingmode7z:=1;
      end;
   else //last used
      if browsingmode7z=0 then status0:=txt_list_browsing
      else status0:=txt_list_flat;
   end;
if form7.CheckBoxAdvFilters.State=cbUnchecked then setsearch_enabled;
end;

procedure do_prebrowse;
begin
list_un7z_expandall_silent;
Form6.ShapeIO6.Width:=Form6.ShapeIO7.Width;
Form6.Labelpercent.Caption:=Form6.Labelpercent1.Caption;
end;

procedure browse_special_cases; //set browse_option evaluating for special cases of formats needing non -slt listing
var
   s:utf8string;
begin
browse_option:='slt'; //default
s:=lowercase(Form_peach.OpenDialog3.Filename);
if (uextractfileext(s)='.xz') or
   (uextractfileext(s)='.lzma') or
   (uextractfileext(s)='.rpm') or
   (uextractfileext(s)='.slp') or
   (uextractfileext(s)='.bz2') or
   (uextractfileext(s)='.bzip2') or
   (uextractfileext(s)='.tbz2') or
   (uextractfileext(s)='.tbz') then browse_option:='l';
cutextension(s);
if uextractfileext(s)='.cpio' then browse_option:='l'; //cpio.gz in rpm archives
end;

procedure open_7z;
begin
Form_peach.StringGridList.Cursor:=crHourGlass;
browse_special_cases;
do_prebrowse;
open_7z_prepare;
if browsingmode7z=0 then list_root
else list_un7z_expandall;
if Form_peach.StringGridList.Cells[1,1]='' then
   if archive_content<>txt_list_nostats then list_un7z_expandall;
testencrypted_open;
Form_peach.StringGridList.PopupMenu:=Form_peach.PopupOpen;
Form_peach.StringGridList.Cursor:=crDefault;
end;

procedure open_supported;
begin
Form_Peach.gbConsoleCreate1.Enabled:=true;
Form_peach.iConsoleCreateImport1.Enabled:=true;
Form_peach.sm_delete.Enabled:=false;
Form_peach.ButtonUn7zaTo.PopupMenu:=nil;
Form_peach.ButtonUn7za1.PopupMenu:=nil;
setsearch_disabled;
currentcomp:=upcase(uextractfileext(Form_peach.OpenDialog3.Filename));
currentcomp:=copy(currentcomp,2,length(currentcomp)-1);
currentfs:='';
status2:=currentcomp;
if (fun='UN7Z') or (fun='FILEBROWSER') then
   begin
   Form_peach.mFlat.Visible:=true;
   end
else
   begin
   Form_peach.mFlat.Visible:=false;
   end;
Form6.ImageInfoOpen5.Visible:=true;
Form6.ImageInfoOpen6.Visible:=true;
Form6.ShapeIO6.Visible:=true;
Form6.ShapeIO7.Visible:=true;
Form6.ShapeIO8.Visible:=true;
Form6.ShapeIO9.Visible:=true;
Form6.LabelPercent.Visible:=true;
Form6.LabelPercent1.Visible:=true;
Form6.ImageInfoOpen5.Picture.Bitmap:=BArchivesupported;
Form6.ImageInfoOpen5.Transparent:=true;
Form_peach.ImageIO.Glyph:=Form6.ImageInfoOpen5.Picture.Bitmap;
Form6.ImageInfoOpen6.Picture.Bitmap:=Blisted;
Form6.ImageInfoOpen6.Transparent:=true;
Form6.ShapeIO7.Width:=200;
Form6.Labelpercent1.Caption:='??%';
Form6.ShapeIO6.Width:=Form6.ShapeIO7.Width;
Form6.Labelpercent.Caption:=Form6.Labelpercent1.Caption;
browse_option:='';
Form_peach.EditOpenIn.Text:=Form_peach.OpenDialog3.Filename;
end;

procedure create_ptmpcode(var s:utf8string);
begin
s:=STR_TMP+inttostr(random(100000));
ptmpdir:='';
end;

function open_inputselect:integer;
var
   f: file of byte;
begin
if listingdir=true then exit;
listingdir:=true;
if Form_peach.OpenDialog3.Filename='' then Form_peach.OpenDialog3.Filename:=Form_peach.EditOpenIn.Text;
Form_peach.EditOpenOut.Text:=uextractfilepath(Form_peach.OpenDialog3.Filename);
if {$IFDEF MSWINDOWS}not(Form_peach.OpenDialog3.Filename=txt_mypc) and{$ENDIF}
   not(ufileexists((Form_peach.OpenDialog3.Filename))) and
   not(udirectoryexists((Form_peach.OpenDialog3.Filename))) then
   begin
   if (Form_peach.OpenDialog3.Filename=txt_none) or (Form_peach.OpenDialog3.Filename='<none>') then
      begin
      if udirectoryexists((uextractfilepath(Form_peach.StringGrid1.Cells[8,Form_peach.StringGrid1.Row]))) then
         Form_peach.OpenDialog3.Filename:=uextractfilepath(Form_peach.StringGrid1.Cells[8,Form_peach.StringGrid1.Row])
      else
         Form_peach.OpenDialog3.Filename:=indir;
      end
   else
      begin
         MessageDlg(Form_peach.OpenDialog3.Filename+' '+txt_not_accessible, mtWarning, [mbOK], 0);
         listingdir:=false;
         exit;
      end;
   end;
if ufileexists((Form_peach.OpenDialog3.Filename)) then addtoarchivehistory(Form_peach.OpenDialog3.Filename); //add only archive files
archiveopened:=true;
lastobj:=Form_peach.OpenDialog3.Filename;
if ptmpcode<>'' then cleardir(ptmpdir,true);
seemencrypted:=false;
seemnamesencrypted:=false;
Form_peach.LabelOpenBack2.Caption:=uextractfilename(Form_peach.OpenDialog3.Filename);
fun:='';
fun_status:='';
subfun:='';
create_ptmpcode(ptmpcode);
Form_peach.NoteBookOpen.Pageindex:=0;
try
   srcfilesize_multipart(Form_peach.OpenDialog3.Filename,tsize); //expressed in B
except
end;
testarctype(Form_peach.OpenDialog3.Filename,open_inputselect,fun);
setbuttonmenus;
Form6.ShapeIO6.Hint:=txt_compressionratio_hint;
Form6.ShapeIO7.Hint:=txt_cr_current;
if fun='UNCUSTOM' then
   begin
   open_supported;
   open_prepare_reduced;
   end;
if fun='UNPEA' then
   begin
   open_supported;
   open_prepare_unpea;
   Form6.Labelpercent1.Caption:='??%';
   end;
if fun='RFJ' then
   begin
   open_supported;
   open_prepare_unpea;
   Form_peach.CheckBoxUnPeaAttributes.Visible:=false;
   Form6.Labelpercent1.Caption:='??%';
   end;
if fun='UN7Z' then
   begin
   open_supported;
   open_prepare_un7z;
   open_7z;
   setfocus_stringgridlist;
   end;
if fun='UNPAQ' then
   begin
   open_supported;
   open_prepare_reduced;
   end;
if fun='UNZPAQ' then
   begin
   open_supported;
   open_prepare_reduced;
   end;
if fun='UNQUAD' then
   begin
   open_supported;
   open_prepare_reduced;
   end;
if fun='UNACE' then
   begin
   if not(ufileexists((executable_path+'res'+directoryseparator+'unace'+directoryseparator+'unace'+EXEEXT))) then
      begin
         MessageDlg(txt_ace_missing, mtWarning, [mbOK], 0);
         Form_peach.Close;//don't skip on form close event
         listingdir:=false;
         exit;
      end;
   open_supported;
   open_prepare_reduced;
   end;
if fun='UNLPAQ' then
   begin
   open_supported;
   open_prepare_reduced;
   end;
if fun='UNARC' then
   begin
   open_supported;
   open_prepare_unarc;
   open_prepare_reduced;
   setfocus_stringgridlist;
   end;
if fun='FILEBROWSER' then
   begin
   listingdir:=false;//allows listing directory
   open_supported;
   open_prepare_reduced;
   end;
if fun<>'UN7Z' then Form6.LabelUn7zInputStatus1.Caption:=status2;
Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus1.Caption;
Form6.LabelUn7zInputStatus2.Caption:=txt_size+': '+inttostr(tsize)+' B; '+archive_content;
Form6.LabelUn7zInputStatus3.Caption:=status0+' '+status1;
if (fun='UN7Z') or (fun='UNARC') then Form_peach.ImageUn7zPj.PopupMenu:=Form_peach.pmOpenPJ else Form_peach.ImageUn7zPj.PopupMenu:=nil;
//menuitemopen_ext_caption;
listingdir:=false;
end;

procedure showbar(s:utf8string);
begin
with form_peach do
begin
if s='none' then
   begin
   showbar('clip');
   showbar('bookmarks');
   splitter1.top:=panelside.top-form_peach.panelfilters.top;
   lastbar:=0;
   stringgridhistory.visible:=false;
   stringgridbookmarks.visible:=false;
   panelfilebrowseroptions.visible:=false;
   end;
if s='bookmarks' then
   if stringgridbookmarks.visible=true then
      begin
      splitter1.top:=panelside.top-form_peach.panelfilters.top;
      lastbar:=0;
      stringgridbookmarks.visible:=false;
      end
   else
      begin
      stringgridhistory.visible:=false;
      stringgridbookmarks.visible:=true;
      panelfilebrowseroptions.visible:=false;
      lastbar:=1;
      splitter1.top:=panelside.top-form_peach.panelfilters.top-splittersize;
      end;
if s='history' then
   if stringgridhistory.visible=true then
      begin
      splitter1.top:=panelside.top-form_peach.panelfilters.top;
      lastbar:=0;
      stringgridhistory.visible:=false;
      end
   else
      begin
      stringgridhistory.visible:=true;
      stringgridbookmarks.visible:=false;
      panelfilebrowseroptions.visible:=false;
      lastbar:=2;
      splitter1.top:=panelside.top-form_peach.panelfilters.top-splittersize;
      end;
if s='clip' then
   if panelfilebrowseroptions.visible=true then
      begin
      splitter1.top:=panelside.top-form_peach.panelfilters.top;
      lastbar:=0;
      panelfilebrowseroptions.visible:=false;
      end
   else
      begin
      stringgridhistory.visible:=false;
      stringgridbookmarks.visible:=false;
      panelfilebrowseroptions.visible:=true;
      Form_peach.StringGridClipboard.Autosizecolumns;
      lastbar:=3;
      splitter1.top:=panelside.top-form_peach.panelfilters.top-splittersize;
      end;
end;
end;

procedure showpanel_prepare; //make invisible all panels
begin
with Form_peach do
   begin
   {$IFDEF MSWINDOWS}
   PanelArchiveMain.Visible:=false;
   PanelOpen.Visible:=false;
   PanelExtract.Visible:=false;
   PanelDefaults.Visible:=false;
   {$ELSE}//sometimes panels remains visible, so hide them otherwise
   PanelArchiveMain.Top:=10000;
   PanelOpen.Top:=10000;
   PanelExtract.Top:=10000;
   PanelDefaults.Top:=10000;
   {$ENDIF}
   mEdit.Visible:=false;
   mPackedSize.Visible:=false;
   mArchive.Visible:=false;
   mLayout.Visible:=false;
   end;
end;

procedure prepare_form2;
begin
Form2.Caption:='PeaZip - '+txt_pw;
Form2.EditUn7zaPW.Height:=editheight;
Form2.EditUn7zaPW1.Height:=editheight;
Form2.EditName3.Height:=editheight;
Form2.ButtonEditName3.Height:=editheight;
Form2.ButtonEditName4.Height:=editheight;
Form2.lablelistPath1.Caption:=txt_pw;
Form2.lablelistPath3.Caption:=txt_pw_confirm;
Form2.lablelistPath2.Caption:=txt_keyfile;
Form2.lablelistPath2.Hint:=txt_2_7_optional;
Form2.OpenDialogKF.Title:=txt_keyfile;
Form2.mpwreset.Caption:=txt_reset;
Form2.mpwexplore.Caption:=txt_explore_path;
Form2.CheckBoxEncFn.Caption:=txt_2_7_encfn;
Form2.CheckBoxShowPWField.Caption:=txt_showpw;
Unit3.validate_txt:=txt_2_7_validatecl;
if showpwfield=1 then
   begin
   Form2.CheckBoxShowPWField.State:=cbChecked;
   Form2.EditUn7zaPW.PasswordChar:=#0;
   Form2.lablelistPath3.Visible:=false;
   Form2.EditUn7zaPW1.visible:=false;
   end
else
   begin
   Form2.CheckBoxShowPWField.State:=cbUnChecked;
   Form2.EditUn7zaPW.PasswordChar:='*';
   Form2.lablelistPath3.Visible:=true;
   Form2.EditUn7zaPW1.visible:=true;
   end;
setpwicons;
end;

procedure prepare_form4;
begin
with form4 do
begin
Form4.Caption:=txt_create_keyfile;
Imageinfoarchive1.Hint:=txt_ent_hint;
GroupBoxKF.Caption:=txt_ent_tools;
EditEnt.Text:=txt_random_keys;
EditEnt.Hint:=txt_random_keys_hint;
ProgressBar1.Hint:=txt_ent;
EditSuggestPW.Hint:=txt_ramdompw_hint;
SpinEditSuggestPW.Hint:=txt_pwlength;
LabelSampleT.Caption:=txt_sample;
ButtonKFLoadFile.Caption:=txt_loadfile;
ButtonKF.Caption:=txt_create_keyfile;
ButtonSuggestPW.Caption:=txt_suggestpw;
ImageInfoArchive1.Picture.Bitmap:=Binfo;
ImageInfoArchive1.Transparent:=true;
ButtonKF.Height:=stdbtnheight;
SpinEditSuggestPW.Height:=editheight;
EditEnt.Height:=editheight;
ButtonKFLoadFile.Height:=editheight;
EditSuggestPW.Height:=editheight;
ButtonSuggestPW.Height:=editheight;
//ButtonPanel1.Closebutton.Height:=stdbtnheight; not translated and not resized; currently the item is only capable to accept changes to caption
//ButtonPanel1.Closebutton.Caption:=txt_done;
SaveDialog1.Title:=txt_saveas;
OpenDialog2.Title:=txt_open_file;
end;
Unit5.persistent_source:=persistent_source;
Unit5.txt_keyfile_notcreated:=txt_keyfile_notcreated;
Unit5.txt_error_openfile:=txt_error_openfile;
end;

function pInputQuery ( const Caption, Prompt : utf8string; var UserValue : utf8string; chkb:boolean ) : Boolean;
begin
pInputQuery:=false;
form5.CheckBox1.Caption:=txt_2_6_rec;
form5.CheckBox1.Visible:=chkb;
form5.caption:=Caption;
form5.labelinputquery.caption:=Prompt;
Form5.EditInputQuery.Text:=UserValue;
Form5.Showmodal;
if form5.ModalResult=1 then
   begin
   UserValue:=Form5.EditInputQuery.Text;
   pInputQuery:=true;
   end
else UserValue:='';
end;

procedure prepare_form5;
begin
form5.EditInputQuery.Height:=editheight;
end;

procedure prepare_form6;
begin
with form6 do
begin
Form6.Caption:=Form_peach.EditOpenIn1.Text;
GroupBoxInputInfo.Caption:=txt_inputinfo;
LabelUn7zInputStatus1.Caption:=txt_un7z_pw_untested;
Form_peach.ImageIO.Hint:=LabelUn7zInputStatus1.Caption;
LabelUn7zInputStatus2.Caption:=txt_size;
LabelUn7zInputStatus3.Caption:=txt_content;
ShapeIO6.Hint:=txt_compressionratio_hint;
ShapeIO7.Hint:=txt_cr_current;
ImageInfoOpen4.Picture.Bitmap:=Binfo;
ImageInfoOpen4.Transparent:=true;
ImageInfoOpen5.Picture.Bitmap:=BArchivesupported;
ImageInfoOpen5.Transparent:=true;
ImageInfoOpen6.Picture.Bitmap:=Bfolder;
ImageInfoOpen6.Transparent:=true;
end;
end;

procedure prepare_form7;
begin
Form7.ButtonClearFilters.Height:=stdbtnheight;
Form7.Caption:=txt_adv_filters;
Form7.Label1.Caption:=txt_2_7_advdesc;
Form7.CheckboxAdvFilters.Caption:=txt_advfilters;
Form7.CheckBoxAdvRecurse.Caption:=txt_inclusion_recourse;
Form7.CheckBoxAdvRecurse1.Caption:=txt_exclusion_recourse;
Form7.LabelAdvInclude.Caption:=txt_inclusion;
Form7.LabelAdvExclude.Caption:=txt_exclusion;
Form7.ButtonClearFilters.Caption:=txt_clear;
Form7.ImageInfoArchive4.Picture.Bitmap:=Binfo;
Form7.ImageInfoArchive4.Transparent:=true;
end;

procedure showpanel(panelname:utf8string); //make all panels invisible (showpanel_prepare) then make visible the called panel
var
   s:utf8string;
begin
{$IFDEF MSWINDOWS}if Form_peach.paneldefaults.visible=true then{$ELSE}if Form_peach.paneldefaults.top=0 then{$ENDIF}
   begin
   save_conf;
   prepare_form2;
   prepare_form4;
   prepare_form5;
   prepare_form7;
   end;//save configuration exiting from paneldefaults
showpanel_prepare;
with Form_peach do
begin
if panelname='archive' then
   begin
   prevpanel:='archive';
   PanelArchiveMain.Top:=0;
   PanelArchiveMain.Visible:=true;
   pmaddfolder.enabled:=true;
   mEdit.Visible:=true;
   mLayout.Visible:=true;
   mtogglenew.visible:=false;
   mgotype.visible:=true;
   mextractarchive.Caption:=txt_archive;
   if archive_type_selected=-1 then
      if df<14 then archive_type_select(Form_peach.cbType.Items[df])
      else archive_type_select(Form_peach.cbType.Items[pf])
   else archive_type_select(Form_peach.cbType.Items[archive_type_selected]);
   status1:=inttostr(tdirs)+' '+txt_dirs+' '+inttostr(tfiles)+' '+txt_files+' '+inttostr(tsize)+' B';
   labelstatus1.Caption:=status1;
   Form_peach.labelstatus4.Caption:=txt_archiving;
   Notebook2.PageIndex:=0;
   ButtonEditname12.Enabled:=true;
   Edit5.enabled:=true;
   ButtonEditname3.Enabled:=true;
   GroupBoxCreateOut.Enabled:=true;
   ButtonEditname13.Enabled:=true;
   pmbrowser.Enabled:=true;
   mgobrowser.Enabled:=true;
   updatingarchive:=false;
   ares:=-1;
   try
   except
   Form_peach.cbType.Setfocus;
   end;
   end;
if panelname='open' then
   begin
   prevpanel:='open';
   Caption:='PeaZip';
   status0:=txt_list_browsing;
   if EditOpenIn1.Caption<>txt_mypc then
      if currentfs='' then
         if currentcomp='' then status2:=''
         else
            if seemencrypted=true then status2:=currentcomp+', '+txt_encrypted
            else status2:=currentcomp
      else status2:=currentfs;
   PanelOpen.Top:=0;
   PanelOpen.Visible:=true;
   mEdit.Visible:=true;
   mPackedSize.Visible:=true;
   mArchive.Visible:=true;
   if setlastbar=true then
      begin
      case lastbar of
         0: showbar('none');
         1: showbar('bookmarks');
         2: showbar('history');
         3: showbar('clip');
         end;
      setlastbar:=false;
      end;
   end;
if panelname='extract' then
   begin
   prevpanel:='extract';
   pmaddfolder.enabled:=false;
   Caption:=txt_caption_extract;
   PanelExtract.Top:=0;
   NotebookOpen.PageIndex:=0;
   PanelExtract.Visible:=true;
   mEdit.Visible:=true;
   mLayout.Visible:=true;
   mtogglenew.visible:=true;
   mgotype.visible:=false;
   mextractarchive.Caption:=txt_caption_extract;
   s:=EditOpenOut.Text;
   control_outpath(s);
   EditOpenOut.Text:=s;
   end;
if panelname='defaults' then
   begin
   //prevpanel left unchanged, to be able to return to previous panel when editing is done
   if Caption<>'PeaZip' then prevcaption:=Caption;
   Caption:='PeaZip';
   PanelDefaults.Top:=0;
   PanelDefaults.Visible:=true;
   end;
end;
do_resize_trick;
end;

procedure showpanel_trick;
//Workaround: on some Linux systems the GTK1 version of PeaZip may sometimes not
//show correctly 'archive' panel (shows 'open' toolbar instead); this code fixes
//the issue without harming if it's not the case
begin
{$IFDEF MSWINDOWS}
{$ELSE}
Form_peach.Repaint;
showpanel('archive');
{$ENDIF}
end;

procedure open_archive_do;
begin
   if Form_peach.OpenDialog3.Filename<>'' then
      begin
      archive_content:='';
      if testinput(Form_peach.OpenDialog3.Filename,true)=0 then
            if MessageDlg(Form_peach.OpenDialog3.Filename+' '+txt_type_custom_selected, mtConfirmation, [mbYes,mbNo], 0)=7 then exit;
      showpanel('open');
      Form6.LabelUn7zInputStatus1.Caption:=status2+'; '+txt_status+': '+txt_un7z_pw_untested;
      Form_peach.ImageIO.Hint:=Form6.LabelUn7zInputStatus1.Caption;
      open_inputselect;
      end;
end;

procedure open_archive;
begin
Form_peach.OpenDialog3.Filename:=''; //clear filename
if Form_peach.OpenDialog3.Execute then open_archive_do;
end;

procedure open_archive_fromname(s:utf8string);
begin
if not(ufileexists((s))) then
   begin
      MessageDlg(s+' '+txt_not_accessible, mtWarning, [mbOK], 0);
      exit;
   end;
Form_peach.OpenDialog3.Filename:=s;
open_archive_do;
end;

function check_input:integer;
begin
check_input:=1;
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
      MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
      exit;
   end;
check_input:=0;
end;

{///////////////////////////////////////////////////////////////////////////////

compose command line

///////////////////////////////////////////////////////////////////////////////}

procedure get_new_archive_name(var out_param:utf8string);
var
   i:integer;
   s:utf8string;
   name_ok:boolean;
begin
if ufileexists((out_param)) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
   end;
end;

procedure apply_timestamptoname(var s:utf8string; checktar:integer);
var
   dt_stamp,s0,s1,s2:utf8string;
begin
if checktar=1 then
   if Form_peach.CheckBoxTarBefore.State=cbChecked then exit;
dt_stamp:=formatdatetime('.yyyymmdd-hhnnss',now);
s0:=uextractfilepath(s);
s1:=uextractfilename(s);
cutextension(s1);
s2:=uextractfileext(s);
if Form_peach.CheckBoxDate.State=cbChecked then s1:=s1+dt_stamp;
s:=s0+s1+s2;
end;

function getoutname(var s,archive_type:utf8string; i:integer):integer;
begin
getoutname:=-1;
suggest_outname(s,Form_peach.cbType.Text,i);
   s:=uextractfilename(s); //keep only filename, set path to last accepted by SaveDialog2 control
   Form_peach.SaveDialog2.FileName:=addext(s);
   if lastoutpath='' then lastoutpath:=local_desktop; //if no path was previously used, or after configuration reset, show desktop as default
   Form_peach.SaveDialog2.InitialDir:=uextractfilepath(Form_peach.Edit5.Caption);//Initialdir:=lastoutpath;
   if Form_peach.SaveDialog2.Execute then
      if Form_peach.SaveDialog2.FileName<>'' then
         begin
         s:=Form_peach.SaveDialog2.FileName;
         cutextension(s);
         end
      else exit
   else exit;
   lastoutpath:=uextractfilepath(Form_peach.SaveDialog2.FileName);
getoutname:=0;
end;

function setoutname(var s,archive_type:utf8string; i:integer):integer;
begin
setoutname:=-1;
suggest_outname(s,Form_peach.cbType.Text,i);
s:=uextractfilename(s); //keep only filename, set path to last used
if Form_peach.StringGrid1.RowCount>1 then s:=uextractfilepath(Form_peach.StringGrid1.Cells[8,Form_peach.StringGrid1.Row])+s
else s:=local_desktop+s;
Form_peach.EditName3.Text:=uextractfilepath(s);
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
setoutname:=0;
end;

procedure get_in_param_arc(var in_param,sel:utf8string);
var
   dstring:utf8string;
   i:integer;
begin
if sel='all' then
   for i:=1 to Form_peach.StringGrid1.RowCount-1 do
      begin
      if Form_peach.StringGrid1.Cells[2,i]=txt_list_isfolder then
         begin
         dstring:=Form_peach.StringGrid1.Cells[8,i];
         if dstring<>'' then if dstring[length(dstring)]<>'/'then dstring:=dstring+'/' //directoryseparator sometimes don't work as expected here
         end
      else dstring:=Form_peach.StringGrid1.Cells[8,i];
      in_param:=in_param+delimiter+escapefilename(dstring,desk_env)+delimiter+' ';
      end
else
   begin
   i:=strtoint(sel);
   if Form_peach.StringGrid1.Cells[2,i]=txt_list_isfolder then
      begin
      dstring:=Form_peach.StringGrid1.Cells[8,i];
      if dstring<>'' then if dstring[length(dstring)]<>'/'then dstring:=dstring+'/' //directoryseparator sometimes don't work as expected here
      end
   else dstring:=Form_peach.StringGrid1.Cells[8,i];
   in_param:=delimiter+escapefilename(dstring,desk_env)+delimiter+' ';
   end;
end;

procedure get_in_param(var in_param,sel:utf8string);
var
   i:integer;
begin
if sel='all' then
   for i:=1 to Form_peach.StringGrid1.RowCount-1 do in_param:=in_param+delimiter+escapefilename(Form_peach.StringGrid1.Cells[8,i],desk_env)+delimiter+' '
else
   begin
   i:=strtoint(sel);
   in_param:=delimiter+escapefilename(Form_peach.StringGrid1.Cells[8,i],desk_env)+delimiter+' ';
   end;
end;

function compose_arc_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   i:integer;
   s,out_param,in_param,exclude_param,include_param,archive_function,solid_option,sfx_option,pw_option,pw,
   bin_name,compression_level,encalgo,dstring,recovery_option:utf8string;
begin
compose_arc_cl:=-1;
subfun:='archive';
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
//output name
if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
out_param:=s+'.'+archive_type;
apply_timestamptoname(out_param,1);
//archive function
case Form_peach.ComboBoxARC2.ItemIndex of
   0: begin
      archive_function:='a';
      get_new_archive_name(out_param);
      end;
   1: archive_function:='a';
   2: archive_function:='u';
   end;
//solid
case Form_peach.ComboBoxARCsolid.ItemIndex of
   0: solid_option:='';
   1: solid_option:='-s';
   2: solid_option:='-se';
   end;
//recovery records
if Form_peach.CheckBoxARC1.state=cbChecked then recovery_option:='-rr';
//sfx
if Form_peach.CheckBoxARC2.state=cbChecked then sfx_option:='-sfx'+Form_peach.Comboboxarcsfx.text;
//Compression level
compression_level:='-m'+inttostr(Form_peach.ComboBoxARC.ItemIndex);
//encryption
if Form2.EditUn7zaPW.Text<>'' then
   if Form2.CheckBoxEncFn.State= cbChecked then pw_option:='-hp'
   else pw_option:='-p'
else pw_option:='';
if zaout>0 then
   begin
   if pw_option<>'' then
      if Form2.EditUn7zapw.Text<>'' then
         begin
         if Form2.CheckBoxShowPWField.State=cbUnchecked then
            if Form2.EditUn7zapw.Text<>Form2.EditUn7zapw1.Text then
               begin
               MessageDlg(txt_error_passwordnotmatch, mtWarning, [mbOK], 0);
               exit;
               end;
         pw:=Form2.EditUn7zapw.Text;
         if Form2.EditName3.Text<>'' then
            if prepend_keyfile(pw,Form2.EditName3.Text)<>0 then
               begin
               MessageDlg(Form2.EditName3.Text+' '+txt_keyfile_not_found, mtError,[mbOk],0);
               exit;
               end;
         if pw4cl(pw_option,pw)<>0 then
            begin
            msg_pw_errorchar;
            exit;
            end;
         end
      else
         begin
         MessageDlg(txt_pw_empty, mtWarning, [mbOK], 0);
         exit;
         end;
   end
else
   if pw_option<>'' then pw:=pw_option;
//encryption algorithm
if Form_peach.ComboBoxARCalgo.Visible=true then
   case Form_peach.ComboBoxARCalgo.Itemindex of
      0: encalgo:='-ae=aes';
      1: encalgo:='-ae=serpent';
      2: encalgo:='-ae=twofish';
      3: encalgo:='-ae=blowfish';
      end;
//arc uses -- as tag to distinguish a filename starting with - from a switch, however since PeaZip uses absolute filenames it should never occour and filenames are not checked for that condition
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
get_in_param(in_param,sel);
if Form7.CheckBoxAdvFilters.State=cbChecked then
begin
   for i:=0 to Form7.MemoAdvExclude.Lines.Count-1 do
      begin
      exclude_param:=exclude_param+' '+delimiter+'-x'+Form7.MemoAdvExclude.Lines[i]+delimiter+' ';
      end;
   for i:=0 to Form7.MemoAdvInclude.Lines.Count-1 do
      begin
      include_param:=include_param+' '+delimiter+'-n'+Form7.MemoAdvInclude.Lines[i]+delimiter+' ';
      end;
end;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'arc'+DirectorySeparator+'arc'+EXEEXT+delimiter;
cl:=bin_name+' '+archive_function+' '+compression_level;
if solid_option<>'' then cl:=cl+' '+solid_option;
if recovery_option<>'' then cl:=cl+' '+recovery_option;
if sfx_option<>'' then cl:=cl+' '+sfx_option;
if pw<>'' then cl:=cl+' '+pw;
if encalgo<>'' then cl:=cl+' '+encalgo;
if Form_peach.EditOParc.Text<>'' then cl:=cl+' '+Form_peach.EditOParc.Text;
cl:=cl+' '+out_param+' '+in_param+' '+exclude_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_arc_cl:=0;
end;

procedure set_outpath(var out_param:utf8string);
var
   ptestf:file of byte;
   ptest:utf8string;
begin
ptest:=out_param;
if ptest<>'' then if ptest[length(ptest)]<>directoryseparator then ptest:=ptest+directoryseparator;
ptest:=ptest+'.peazip_testout';
try
   uassignfile(ptestf,ptest);
   rewrite(ptestf);
   closefile(ptestf);
   udeletefile(ptest);
except
   {$IFDEF MSWINDOWS}
   out_param:=local_desktop;
   {$ELSE}
   get_usrtmp_path(out_param);
   {$ENDIF}
   try
      closefile(ptestf);
   except
   end;
   try
      udeletefile(ptest);
   except
   end;
end;
end;

function test_outpath(out_param:utf8string):integer;
var
   ptestf:file of byte;
   ptest:utf8string;
begin
test_outpath:=-1;
ptest:=out_param;
if ptest<>'' then if ptest[length(ptest)]<>directoryseparator then ptest:=ptest+directoryseparator;
ptest:=ptest+'.peazip_testout'+inttostr(random(100000));
try
   uassignfile(ptestf,ptest);
   rewrite(ptestf);
   closefile(ptestf);
   udeletefile(ptest);
   test_outpath:=0;
except
   try
      closefile(ptestf);
   except
   end;
   try
      udeletefile(ptest);
   except
   end;
end;
end;

function select_outpath:integer;
begin
select_outpath:=-1;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      begin
      Form_peach.EditOpenOut.Text:=Form_peach.SelectDirectoryDialog1.FileName;
      if Form_peach.EditOpenOut.Text<>'' then
      if Form_peach.EditOpenOut.Text[length(Form_peach.EditOpenOut.Text)]<>directoryseparator then
         Form_peach.EditOpenOut.Text:=Form_peach.EditOpenOut.Text+directoryseparator;
      end
   else exit
else exit;
if ptmpcode<>'' then cleardir(ptmpdir,true);
create_ptmpcode(ptmpcode);
select_outpath:=0;
end;

function control_outpath(var out_param:utf8string):integer;
begin
   control_outpath:=-1;
   if pos(STR_TMP,out_param)<>0 then out_param:=copy(out_param,0,pos(STR_TMP,out_param)-1);
   if test_outpath(out_param)<>0 then
      begin
      if MessageDlg(out_param+' '+txt_2_4_path, mtWarning,[mbYes, mbNo],0)=6 then
         begin
         if select_outpath=0 then out_param:=Form_peach.EditOpenOut.Text
         else exit;
         end
      else exit;
      end;
   control_outpath:=0;
end;

procedure set_output_option(var out_param:utf8string);
begin
out_param:=Form_peach.EditOpenOut.Text;
if out_param<>'' then if out_param[length(out_param)] <> directoryseparator then out_param:=out_param+directoryseparator;
end;

procedure set_output_folder(var out_param:utf8string; in_param:utf8string; real_extract:boolean; action:integer);
var
   s,s1:utf8string;
   i:integer;
   out_created:boolean;
begin
out_created:=false;
if Form_peach.CheckBoxFolder.State=cbChecked then
      begin
      if action<2 then
         begin
         try umkdir(out_param); except end;
         s1:=uextractfilename(in_param);
         cutextension(s1);
         s:=out_param+s1;
         i:=0;
         repeat
            try
               umkdir(s);
               out_created:=true;
            except
               i:=i+1;
               s:=out_param+s1+'-'+inttostr(i);
               if i=1000 then //to break recursivity if filename is not valid (ie unsupported character encoding)
                  begin
                  s:=out_param+'unsupported_name';
                  out_created:=true;
                  end;
            end;
         until out_created=true;
         if copy(s,length(s)-1,1)<>directoryseparator then s:=s+directoryseparator;
         out_param:=s;
         if real_extract=false then
            try
               urmdir(s); //simulated operation mode for composing command line
            except
            end;
         end;
      end;
end;

function compose_unarc_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
//mode: neutral (read in options), ext, extandrun, list, info, test, (add is in compose_archive_cl only), delete
//selection: all, displayed (apply filters), selected (evaluate selected), single (single selected object)
var
   filter1,out_param,in_param,bin_name,archive_function,overwrite_policy,pw:utf8string;
   i:integer;
   contains_folder:boolean;
begin
compose_unarc_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if mode='preview' then
   begin
   set_outpath(out_param); //on preview switch to user temp transparently if output path is read only
   out_param:=out_param+ptmpcode;//directoryseparator is added in next step
   ptmpdir:=out_param;
   end
else //check path to avoid extraction in peazip's temporary path, swithc out of first temporary level (by default archive's path)
   begin
   if (mode='ext') or (mode='extandrun') or ((mode='neutral') and (Form_peach.RadioGroupActionARC.ItemIndex<2)) then
      if control_outpath(out_param)<>0 then exit;
   end;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,Form_peach.RadioGroupActionARC.ItemIndex);
if out_param<>'' then if out_param[length(out_param)]=directoryseparator then setlength(out_param,length(out_param)-1); //arc doesn't want directoryseparator as last character
//overwrite policy
case Form_peach.ComboBoxOverwriteARC.ItemIndex of
   0: overwrite_policy:='-op';//ask (use in Console mode)
   1: overwrite_policy:='-o+';//overwrite
   2: overwrite_policy:='-o-';//skip
   end;
outname:=out_param;
if zaout>0 then
   begin
   pw:=Form2.EditUn7zaPW.Text;
   if Form2.EditName3.Text<>'' then
      if prepend_keyfile(pw,Form2.EditName3.Text)<>0 then
         begin
         MessageDlg(Form2.EditName3.Text+' '+txt_keyfile_not_found, mtError,[mbOk],0);
         exit;
         end;
   end
else pw:='';
if pw<>'' then
   begin
   if pw4cl('-p',pw)<>0 then
      begin
      msg_pw_errorchar;
      exit;
      end;
   end
else if zaout>0 then pw:='-pdefault'; //workaround: pw is never left blank in pipe mode; if archive is not encrypted the pw will not be checked by 7za, otherwise the archive will not open but will not hang the application in pipe mode that is waiting for input
//archive function
if mode='neutral' then
   case Form_peach.RadioGroupActionARC.ItemIndex of
      0: archive_function:='x';
      1: archive_function:='e';
      2: begin
         archive_function:='l';
         subfun:='list';
         end;
      3: begin
         archive_function:='v';
         subfun:='list';
         end;
      4: begin
         archive_function:='t';
         subfun:='test';
         end;
      5: begin
         archive_function:='r';
         subfun:='test';//can be treated like test mode
         end;
      end;
if mode='delete' then //delete special feature, replaces archive function and overwrite policy
   begin
   archive_function:='d';
   overwrite_policy:='';
   end;
if mode='list' then
   begin
   archive_function:='l';
   subfun:='list';
   end;
if mode='info' then
   begin
   archive_function:='v';
   subfun:='list';
   end;
if mode='test' then
   begin
   archive_function:='t';
   subfun:='test';
   end;
if mode='ext' then archive_function:='x';
if archive_function='x' then //set optional ignore paths for displayed or selected objects, if a directory is in the group the rule is overrieddn and x function is used to preserve paths structure
   begin
   if (selection='displayed') and (ignorepathdisp=1) then
      begin
      contains_folder:=false;
      i:=1;
      while (i<Form_peach.StringGridList.Rowcount) and (contains_folder=false) do
         begin
         if Form_peach.StringGridList.Cells[2,i]=txt_list_isfolder then contains_folder:=true;
         i:=i+1;
         end;
      if contains_folder=false then archive_function:='e';
      end;
   if (selection='selected') and (ignorepathsel=1) then
      begin
      contains_folder:=false;
      i:=1;
      while (i<Form_peach.StringGridList.Rowcount) and (contains_folder=false) do
         begin
         if Form_peach.StringGridList.Cells[12,i]='1' then
            if Form_peach.StringGridList.Cells[2,i]=txt_list_isfolder then contains_folder:=true;
         i:=i+1;
         end;
      if contains_folder=false then archive_function:='e';
      end;
   end;
if mode='extandrun' then //extandrun special feature, replaces archive function and overwrite policy: always extract the object, to be opened immediately after; if the object exist, ask about overwriting or skipping it
   begin
   if archive_function='e' then //extract without paths
      begin
      tempstring:=uextractfilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]);
      end
   else
      begin
      archive_function:='x';
      tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
      end;
   if ignorepathextand=1 then //option ignore archived paths for Extract and... override standard path policy; checking it it's possible ignoring paths only for Extract and... functions
      begin
      archive_function:='e';
      tempstring:=uextractfilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]);
      end;
   if Form_peach.StringGridList.Cells[2,Form_peach.StringGridList.Row]=txt_list_isfolder then //if the object is a directory, paths are always preserved (this rule override all other rules)
      begin
      archive_function:='x';
      tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
      end;
   if ufileexists((out_param+tempstring)) then
         if MessageDlg(out_param+tempstring+' '+txt_confirm_overwrite,mtWarning,[mbYes, mbNo],0)<>6 then exit;
   overwrite_policy:='-o+';
   end;
if mode='preview' then //preview special feature, replaces archive function and overwrite policy as 'extract and...' but never ignores paths
   begin
   archive_function:='x';
   tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
   overwrite_policy:='-o+';
   end;
if (archive_function<>'x') and (archive_function<>'e') then //clear overwrite policies for non extraction modes
   begin
   overwrite_policy:='';
   out_param:='';
   end;
//end archive function attribution
if out_param<>'' then out_param:=delimiter+'-dp'+escapefilename(out_param,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'arc'+DirectorySeparator+'arc'+EXEEXT+delimiter;
cl:=bin_name+' '+archive_function;
if overwrite_policy<>'' then cl:=cl+' '+overwrite_policy;
cl:=cl+' '+in_param;
if pw<>'' then cl:=cl+' '+pw;
if out_param<>'' then cl:=cl+' '+out_param;
//filter(s)
if selection='all' then //apply only advanced filter(s), if used
   if Form7.CheckBoxAdvFilters.State=cbChecked then adv_filters(cl);
if selection='displayed' then //apply advanced filters, if used, or basic filters (but without excludes, in order to extract also all nested objects)
   if Form7.CheckBoxAdvFilters.State=cbChecked then adv_filters(cl)
   else
      if Form_peach.EditUn7zaFilter.Text<>'' then
         begin
         if Form_peach.cbUn7zaRecursive.State=cbChecked then cl:=cl+' -r';
         filter1:=Form_peach.EditUn7zaFilter.Text;
         if copy(filter1,1,1)='.' then filter1:='*'+copy(filter1,2,length(filter1)-1);
         cl:=cl+' -- '+delimiter+escapefilename(filter1,desk_env)+delimiter;
         end;
if selection='selected' then //extract object in the range of selection
   begin
   cl:=cl+' --';
   for i:=1 to Form_peach.StringGridList.RowCount-1 do
      begin
      if Form_peach.StringGridList.Cells[12,i]='1' then
         begin
         if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
         filter1:=Form_peach.StringGridList.Cells[8,i];
         if copy(filter1,1,1)='.' then filter1:='*'+copy(filter1,2,length(filter1)-1);
         cl:=cl+' '+delimiter+escapefilename(filter1,desk_env)+delimiter;
         end;
      end;
   end;
if ((mode='extandrun') or (mode='preview')) and (selection='single') then //extract a single object
   begin
   if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
   filter1:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
   if copy(filter1,1,1)='.' then filter1:='*'+copy(filter1,2,length(filter1)-1);
   cl:=cl+' -- '+delimiter+escapefilename(filter1,desk_env)+delimiter;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_unarc_cl:=0;
end;

function compose_custom_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   i:integer;
   s,out_param,in_param,param_param,bin_name,ext_param:utf8string;
begin
compose_custom_cl:=-1;
subfun:='archive';
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
//check for custom RadioGroupQuad's name
if (Form_peach.EditnameCustom.Text='') or (Form_peach.EditnameCustom.Text=' ') then
   begin
   MessageDlg(txt_custom_executable_missing, mtWarning, [mbOK], 0);
   exit;
   end;
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
ext_param:=Form_peach.EditExtCustom.Text;
//output name
s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
out_param:=s+'.'+ext_param;
apply_timestamptoname(out_param,1);
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
get_in_param(in_param,sel);
bin_name:=delimiter+escapefilename(Form_peach.EditnameCustom.Text,desk_env)+delimiter;
if Form_peach.EditOPCustom.Text<>'' then param_param:=Form_peach.EditOPCustom.Text else param_param:='';
case Form_peach.ComboBoxArchiveCustom.ItemIndex of
   0: cl:=bin_name+' '+param_param+' '+in_param+' '+out_param;
   1: cl:=bin_name+' '+param_param+' '+out_param+' '+in_param;
   2: cl:=bin_name+' '+in_param+' '+param_param+' '+out_param;
   3: cl:=bin_name+' '+in_param+' '+out_param+' '+param_param;
   4: cl:=bin_name+' '+out_param+' '+param_param+' '+in_param;
   5: cl:=bin_name+' '+out_param+' '+in_param+' '+param_param;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_custom_cl:=0;
end;

function compose_uncustom_cl(var cl,jobcode,outname:utf8string; real_extract:boolean):integer;
var
   out_param,in_param,param_param,bin_name:utf8string;
begin
compose_uncustom_cl:=-1;
//check for custom RadioGroupQuad's name
if (Form_peach.EditnameCustom1.Text='') or (Form_peach.EditnameCustom1.Text=' ') then
   begin
   MessageDlg(txt_custom_executable_missing, mtWarning, [mbOK], 0);
   exit;
   end;
subfun:='extract';
set_output_option(out_param);
if control_outpath(out_param)<>0 then exit;
if out_param<>'' then if out_param[length(out_param)] <> directoryseparator then out_param:=out_param+directoryseparator;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,0);
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
outname:=out_param;
if ufileexists((out_param)) then
      if MessageDlg(out_param+' '+txt_confirm_overwrite,mtWarning,[mbYes, mbNo],0)<>6 then exit
      else
         if real_extract=true then clearfile(out_param); //avoid overwriting existing files since some programs will and return error exit code
if out_param<>'' then out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(Form_peach.EditnameCustom1.Text,desk_env)+delimiter;
if Form_peach.EditOPCustom1.Text<>'' then param_param:=Form_peach.EditOPCustom1.Text else param_param:='';
case Form_peach.ComboBoxArchiveCustom1.ItemIndex of
   0: cl:=bin_name+' '+param_param+' '+in_param+' '+out_param;
   1: cl:=bin_name+' '+param_param+' '+out_param+' '+in_param;
   2: cl:=bin_name+' '+in_param+' '+param_param+' '+out_param;
   3: cl:=bin_name+' '+in_param+' '+out_param+' '+param_param;
   4: cl:=bin_name+' '+out_param+' '+param_param+' '+in_param;
   5: cl:=bin_name+' '+out_param+' '+in_param+' '+param_param;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_uncustom_cl:=0;
end;

function compose_archive_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   i:integer;
   s,out_param,in_param,exclude_param,include_param,archive_function,solid_option,pw_option,pw,pw2_option,mail_option,
   sfx_option,bin_name,type_option,compressor_option,thread_option,dictionary_option,
   word_option,passes_option,compression_level,encalgo,openfiles_option:utf8string;
   vol_size:qword;
   name_ok:boolean;
begin
compose_archive_cl:=-1;
subfun:='archive';
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) then begin end //single input file
else //folder(s) and/or multiple files
   begin
   if (archive_type='bz2') or (archive_type='gz') then
      begin
      MessageDlg(archive_type+' '+txt_error_input_multi, mtWarning, [mbOK], 0);
      exit;
      end;
   end;
//output name
if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
if (Form_peach.CheckBoxArchive6.Visible=true) and (Form_peach.CheckBoxArchive6.State=cbChecked) then out_param:=s+'.exe'
else out_param:=s+'.'+archive_type;
apply_timestamptoname(out_param,1);
//archive function
case Form_peach.ComboBoxArchive7.ItemIndex of
   0: begin
      archive_function:='a';
      if Form_peach.ComboBoxArchive1.ItemIndex=0 then get_new_archive_name(out_param)//single archive
      else //multi volume archive
         begin
         if (ufileexists((out_param+'.001'))) or (ufileexists((out_param))) then
            begin
            i:=1;
            name_ok:=false;
            s:=out_param;
            cutextension(s);
            repeat
               if (ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param)+'.001'))) or (ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param)))) then inc(i,1)
               else name_ok:=true;
            until name_ok = true;
            out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
            end;
         end;
      end;
   1: archive_function:='a';
   2: archive_function:='u';
   end;
//solid archive
if (archive_type='7z') then
   case Form_peach.ComboBoxArchiveSolid.ItemIndex of
      0: solid_option:='-ms=off';
      1: solid_option:='-ms=1m';
      2: solid_option:='-ms=2m';
      3: solid_option:='-ms=4m';
      4: solid_option:='-ms=8m';
      5: solid_option:='-ms=16m';
      6: solid_option:='-ms=32m';
      7: solid_option:='-ms=64m';
      8: solid_option:='-ms=128m';
      9: solid_option:='-ms=256m';
      10: solid_option:='-ms=512m';
      11: solid_option:='-ms=1g';
      12: solid_option:='-ms=2g';
      13: solid_option:='-ms=4g';
      14: solid_option:='-ms=8g';
      15: solid_option:='-ms=16g';
      16: solid_option:='-ms=32g';
      17: solid_option:='-ms=64g';
      18: solid_option:='-ms=on';
      19: solid_option:='-ms=e';
      end;
//solid archive
if Form_peach.CheckBoxArchiveOF.State=cbChecked then openfiles_option:='-ssw' else openfiles_option:='';
//encryption
if Form2.EditUn7zaPW.Text<>'' then
   if Form2.CheckBoxEncFn.State= cbChecked then
      begin
      pw_option:='-p';
      pw2_option:='-mhe=on';
      end
   else
      begin
      pw_option:='-p';
      pw2_option:='';
      end
else
   begin
   pw_option:='';
   pw2_option:='';
   end;
if zaout>0 then
   begin
   if pw_option<>'' then
      if Form2.EditUn7zapw.Text<>'' then
         begin
         if Form2.CheckBoxShowPWField.State=cbUnchecked then
            if Form2.EditUn7zapw.Text<>Form2.EditUn7zapw1.Text then
               begin
               MessageDlg(txt_error_passwordnotmatch, mtWarning, [mbOK], 0);
               exit;
               end;
         pw:=Form2.EditUn7zaPW.Text;
         if Form2.EditName3.Text<>'' then
            if prepend_keyfile(pw,Form2.EditName3.Text)<>0 then
               begin
               MessageDlg(Form2.EditName3.Text+' '+txt_keyfile_not_found, mtError,[mbOk],0);
               exit;
               end;
         if pw4cl('-p',pw)<>0 then
            begin
            msg_pw_errorchar;
            exit;
            end;
         end
      else
         begin
         MessageDlg(txt_pw_empty, mtWarning, [mbOK], 0);
         exit;
         end;
   end
else
   if pw_option<>'' then pw:='-p';
//encryption algorithm
if Form_peach.ComboBox7zalgo.Visible=true then
   if archive_type='zip' then
      case Form_peach.ComboBox7zalgo.Itemindex of
         0: encalgo:='-mem=AES256';
         1: encalgo:=''; //if em is omitted, ZipCrypto is used; omitting the parameter rather than explicitely declare it allows to use transparently also older 7z implementation not supporting that switch, providing the user unckeck the use AES option (or the binary to be recompiled hiding the AES option checkbox)
      end;
//send by mail
if (Form_peach.CheckBoxArchive4.Visible=true) and (Form_peach.CheckBoxArchive4.State=cbChecked) then mail_option:='-seml' else mail_option:='';
//multi volumes
case Form_peach.ComboBoxArchive1.ItemIndex of
   0: vol_size:=0; //will trigger to not use -v(size) switch
   1: begin
      try
         vol_size:=Form_peach.SpinEditArchive1.Value;
         case Form_peach.ComboBoxArchive2.ItemIndex of
            1: vol_size:=vol_size*1024;
            2: vol_size:=vol_size*1024*1024;
            3: vol_size:=vol_size*1024*1024*1024;
            end;
      except
         MessageDlg(txt_chunk_size, mtWarning, [mbOK], 0);
         exit;
      end;
      end;
   2: vol_size:=1457664;//FD
   3: vol_size:=5*1024*1024;//limit for attachment size of some mail services
   4: vol_size:=10*1024*1024;//limit for attachment size of some mail services
   5: vol_size:=650*1024*1024;//CD 650 MB
   6: vol_size:=700*1024*1024;//CD 700 MB
   7: vol_size:=4*1024*1024*1024;//max file size for FAT32 filesystem
   8: vol_size:=4700372992;//size DVD+R (slightly smaller than DVD-R), source Wikipedia
   9: vol_size:=8543666176;//size for DVD-R DL (slightly smaller than DVD+R DL), source Wikipedia
   end;
//sfx (generate self extracting Windows RadioGroupQuad)
if (Form_peach.CheckBoxArchive6.Visible=true) and (Form_peach.CheckBoxArchive6.State=cbChecked) then
   begin
   sfx_option:='-sfx';
   if Form_peach.RadioGroupArchive.ItemIndex=1 then sfx_option:=sfx_option+'7z.sfx';
   end
else sfx_option:='';
if sfx_option<>'' then vol_size:=0; //sfx cannot be spanned on multiple archives
//archive type
if archive_type='7z' then type_option:='-t7z';
if archive_type='bz2' then type_option:='-tbzip2';
if archive_type='gz' then type_option:='-tgzip';
if archive_type='tar' then type_option:='-ttar';
if archive_type='zip' then type_option:='-tzip';
//Compression level
if Form_peach.ComboBoxArchive4.Text=txt_level_store then compression_level:='-mx0';
if Form_peach.ComboBoxArchive4.Text=txt_level_fastest then compression_level:='-mx1';
if Form_peach.ComboBoxArchive4.Text=txt_level_fast then compression_level:='-mx3';
if Form_peach.ComboBoxArchive4.Text=txt_level_normal then compression_level:='-mx5';
if Form_peach.ComboBoxArchive4.Text=txt_level_maximum then compression_level:='-mx7';
if Form_peach.ComboBoxArchive4.Text=txt_level_ultra then compression_level:='-mx9';
//compressor type
if compression_level<>'-mx0' then
   begin
   if type_option='-t7z' then compressor_option:='-m0='+compression_method;
   if type_option='-tzip' then compressor_option:='-mm='+compression_method;
   //gzip and bzip2 doesn't need this param since use a single compressor
   end
else
   begin
   if type_option='-t7z' then compressor_option:='-m0=Copy';
   if type_option='-tzip' then compressor_option:='-mm=Copy';
   end;
// multithreading
thread_option:='';
try
s:=Form_peach.ComboBoxArchive3.Text;
except
s:='';
end;
if (s='LZMA') or (s='LZMA2') or (s='BZip2') then
   case Form_peach.ComboBoxArchive8.ItemIndex of
      0: thread_option:='';
      1: thread_option:='-mmt=on';
      2: thread_option:='-mmt=2';
      3: thread_option:='-mmt=4';
      4: thread_option:='-mmt=8';
      5: thread_option:='-mmt=16'; //maybe excessive
      6: thread_option:='-mmt=32'; //maybe excessive
   end;
//passes, only for Deflate, Deflate64 and BZip2
if (s='Deflate') or (s='Deflate64') or (s='BZip2') then passes_option:='-mpass='+Form_peach.ComboBoxArchivePasses.Text;
//word size
word_option:='';
if type_option<>'-tbzip2' then //bzip2 doesn't need word size option
   if (compressor_option<>'-m0=BZip2') and (compressor_option<>'-mm=BZip2') and (compressor_option<>'-m0=PPMd') then
      if compression_level<>'-mx0' then word_option:='-mfb='+Form_peach.ComboBoxArchive6.Text
      else word_option:='';
if compressor_option='-m0=PPMd' then word_option:='-mo='+Form_peach.ComboBoxArchive6.Text; //if PPMd Compression is used, use model order instead of fast bytes
//dictionary size
if compression_level<>'-mx0' then
   begin
   s:=Form_peach.ComboBoxArchive5.Items[Form_peach.ComboBoxArchive5.ItemIndex];//.Text may be not correctly valorized if both conditions are met: combobox has a single item and it is not in active tab
   setlength(s,length(s)-3);//remove ' *B' suffix
   dictionary_option:='-md='+s+'k';//treat the value as KB by default (modified if following conditions are encountered
   if compressor_option='-m0=PPMd' then dictionary_option:='-mmem='+s+'m'; //only in 7z format, all PPMd dictionary sizes are expressed in MB; PPMd uses mem param instead of dictionary param
   if ((compressor_option='-m0=LZMA') or (compressor_option='-m0=LZMA2')) and (Form_peach.ComboBoxArchive5.ItemIndex>0) then dictionary_option:='-md='+s+'m'; //only in 7z format, all LZMA dictionary sizes but the first are expressed in MB
   if type_option='-tgzip' then dictionary_option:=''; //gzip doesn't need dictionary size param
   end
else dictionary_option:='';
//7za uses -- as tag to distinguish a filename starting with - from a switch, however since PeaZip uses absolute filenames it should never occour and filenames are not checked for that condition
outname:=out_param;
if vol_size>0 then outname:=outname+'.001'; //give correct output name if archive is split
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
get_in_param(in_param,sel);
exclude_param:='';
include_param:='';
if Form7.CheckBoxAdvFilters.State=cbChecked then
begin
   for i:=0 to Form7.MemoAdvExclude.Lines.Count-1 do
      begin
      if Form7.CheckBoxAdvRecurse1.State=cbChecked then
         exclude_param:=exclude_param+' '+delimiter+'-xr0!'+Form7.MemoAdvExclude.Lines[i]+delimiter+' '
      else
         exclude_param:=exclude_param+' '+delimiter+'-x!'+Form7.MemoAdvExclude.Lines[i]+delimiter+' ';
      end;
   for i:=0 to Form7.MemoAdvInclude.Lines.Count-1 do
      begin
      if Form7.CheckBoxAdvRecurse.State=cbChecked then
         include_param:=include_param+' '+delimiter+'-ir0!'+Form7.MemoAdvInclude.Lines[i]+delimiter+' '
      else
         include_param:=include_param+' '+delimiter+'-i!'+Form7.MemoAdvInclude.Lines[i]+delimiter+' ';
      end;
end;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
cl:=bin_name+' '+archive_function+' '+type_option;
if compressor_option<>'' then cl:=cl+' '+compressor_option;
if thread_option<>'' then cl:=cl+' '+thread_option;
if compression_level<>'' then cl:=cl+' '+compression_level;
if dictionary_option<>'' then cl:=cl+' '+dictionary_option;
if word_option<>'' then cl:=cl+' '+word_option;
if passes_option<>'' then cl:=cl+' '+passes_option;
if solid_option<>'' then cl:=cl+' '+solid_option;
if openfiles_option<>'' then cl:=cl+' '+openfiles_option;
if pw<>'' then cl:=cl+' '+pw;
if pw2_option<>'' then cl:=cl+' '+pw2_option;
if encalgo<>'' then cl:=cl+' '+encalgo;
if mail_option<>'' then cl:=cl+' '+mail_option;
if sfx_option<>'' then cl:=cl+' '+sfx_option;
if vol_size<>0 then cl:=cl+' -v'+inttostr(vol_size);
if Form_peach.EditOP.Text<>'' then cl:=cl+' '+Form_peach.EditOP.Text;
cl:=cl+' '+out_param+' '+in_param;
if exclude_param<>'' then cl:=cl+' '+exclude_param;
if include_param<>'' then cl:=cl+' '+include_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_archive_cl:=0;
end;

function compose_un7z_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
//mode: neutral (read in options), ext, extandrun, list, info, test, (add is in compose_archive_cl only), delete
//selection: all, displayed (apply filters), selected (evaluate selected), single (single selected object)
var
   filter1,out_param,in_param,bin_name,archive_function,overwrite_policy,pw,details:utf8string;
   i:integer;
   contains_folder:boolean;
begin
compose_un7z_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if mode='preview' then
   begin
   set_outpath(out_param); //on preview switch to user temp transparently if output path is read only
   out_param:=out_param+ptmpcode;//directoryseparator is added in next step
   ptmpdir:=out_param;
   end
else //check path to avoid extraction in peazip's temporary path, swithc out of first temporary level (by default archive's path)
   begin
   if (mode='ext') or (mode='extandrun') or ((mode='neutral') and (Form_peach.RadioGroupAction.ItemIndex<2)) then
      if control_outpath(out_param)<>0 then exit;
   end;
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,Form_peach.RadioGroupAction.ItemIndex);
//overwrite policy
case Form_peach.ComboBoxOverwrite.ItemIndex of
   0: overwrite_policy:='-aou';//auto rename extracting files
   1: overwrite_policy:='-aot';//auto rename existing files
   2: overwrite_policy:='-aoa';//overwrite all existing files
   3: overwrite_policy:='-aos';//skip existing files
   end;
if Form_peach.RadioGroupAction.ItemIndex=3 then details:='-slt'
else details:='';
outname:=out_param;
if zaout>0 then
   begin
   pw:=Form2.EditUn7zaPW.Text;
   if Form2.EditName3.Text<>'' then
      if prepend_keyfile(pw,Form2.EditName3.Text)<>0 then
         begin
         MessageDlg(Form2.EditName3.Text+' '+txt_keyfile_not_found, mtError,[mbOk],0);
         exit;
         end;
   end
else pw:='';
if pw<>'' then
   begin
   if pw4cl('-p',pw)<>0 then
      begin
      msg_pw_errorchar;
      exit;
      end;
   end
else if zaout>0 then pw:='-pdefault'; //workaround: pw is never left blank in pipe mode; if archive is not encrypted the pw will not be checked by 7za, otherwise the archive will not open but will not hang the application in pipe mode that is waiting for input
//archive function
if mode='neutral' then
   case Form_peach.RadioGroupAction.ItemIndex of
      0: archive_function:='x';
      1: archive_function:='e';
      2: begin
         archive_function:='l';
         subfun:='list';
         end;
      3: begin
         archive_function:='l';
         subfun:='list';
         end;
      4: begin
         archive_function:='t';
         subfun:='test';
         end;
      end;
if mode='delete' then //delete special feature, replaces archive function and overwrite policy
   begin
   archive_function:='d';
   overwrite_policy:='';
   end;
if mode='list' then
   begin
   archive_function:='l';
   details:='';
   subfun:='list';
   end;
if mode='info' then
   begin
   archive_function:='l';
   details:='-slt';
   subfun:='list';
   end;
if mode='test' then
   begin
   archive_function:='t';
   subfun:='test';
   end;
if mode='ext' then archive_function:='x';
if archive_function='x' then //set optional ignore paths for displayed or selected objects, if a directory is in the group the rule is overrieddn and x function is used to preserve paths structure
   begin
   if (selection='displayed') and (ignorepathdisp=1) then
      begin
      contains_folder:=false;
      i:=1;
      while (i<Form_peach.StringGridList.Rowcount) and (contains_folder=false) do
         begin
         if Form_peach.StringGridList.Cells[2,i]=txt_list_isfolder then contains_folder:=true;
         i:=i+1;
         end;
      if contains_folder=false then archive_function:='e';
      end;
   if (selection='selected') and (ignorepathsel=1) then
      begin
      contains_folder:=false;
      i:=1;
      while (i<Form_peach.StringGridList.Rowcount) and (contains_folder=false) do
         begin
         if Form_peach.StringGridList.Cells[12,i]='1' then
            if Form_peach.StringGridList.Cells[2,i]=txt_list_isfolder then contains_folder:=true;
         i:=i+1;
         end;
      if contains_folder=false then archive_function:='e';
      end;
   end;
if mode='extandrun' then //extandrun special feature, replaces archive function and overwrite policy: always extract the object, to be opened immediately after; if the object exist, ask about overwriting or skipping it
   begin
   if archive_function='e' then //extract without paths
      begin
      tempstring:=uextractfilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]);
      end
   else
      begin
      archive_function:='x';
      tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
      end;
   if ignorepathextand=1 then //option ignore archived paths for Extract and... override standard path policy; checking it it's possible ignoring paths only for Extract and... functions
      begin
      archive_function:='e';
      tempstring:=uextractfilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]);
      end;
   if Form_peach.StringGridList.Cells[2,Form_peach.StringGridList.Row]=txt_list_isfolder then //if the object is a directory, paths are always preserved (this rule override all other rules)
      begin
      archive_function:='x';
      tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
      end;
   if ufileexists((out_param+tempstring)) then
         if MessageDlg(out_param+tempstring+' '+txt_confirm_overwrite,mtWarning,[mbYes, mbNo],0)<>6 then exit;
   overwrite_policy:='-aoa';
   end;
if mode='preview' then //preview special feature, replaces archive function and overwrite policy as 'extract and...' but never ignores paths
   begin
   archive_function:='x';
   tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
   overwrite_policy:='-aoa';
   end;
if (archive_function<>'x') and (archive_function<>'e') then //clear overwrite policies for non extraction modes
   begin
   overwrite_policy:='';
   out_param:='';
   end;
//end archive function attribution
if out_param<>'' then out_param:=delimiter+'-o'+escapefilename(out_param,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
cl:=bin_name+' '+archive_function;
if overwrite_policy<>'' then cl:=cl+' '+overwrite_policy;
if out_param<>'' then cl:=cl+' '+out_param;
if pw<>'' then cl:=cl+' '+pw;
if (details<>'') and (archive_function='l') then cl:=cl+' '+details;
cl:=cl+' '+in_param;
//filter(s)
if selection='all' then //apply only advanced filter(s), if used
   if Form7.CheckBoxAdvFilters.State=cbChecked then adv_filters(cl);
if selection='displayed' then //apply advanced filters, if used, or basic filters (but without excludes, in order to extract also all nested objects)
   if Form7.CheckBoxAdvFilters.State=cbChecked then adv_filters(cl)
   else
      if Form_peach.EditUn7zaFilter.Text<>'' then
         begin
         if Form_peach.cbUn7zaRecursive.State=cbChecked then cl:=cl+' -r';
         filter1:=Form_peach.EditUn7zaFilter.Text;
         if copy(filter1,1,1)='.' then filter1:='*'+copy(filter1,2,length(filter1)-1);
         cl:=cl+' -- '+delimiter+escapefilename(filter1,desk_env)+delimiter;
         end;
if selection='selected' then //extract object in the range of selection
   begin
   cl:=cl+' --';
   for i:=1 to Form_peach.StringGridList.RowCount-1 do
      begin
      if Form_peach.StringGridList.Cells[12,i]='1' then
         begin
         if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
         filter1:=Form_peach.StringGridList.Cells[8,i];
         if copy(filter1,1,1)='.' then filter1:='*'+copy(filter1,2,length(filter1)-1);
         cl:=cl+' '+delimiter+escapefilename(filter1,desk_env)+delimiter;
         end;
      end;
   end;
if ((mode='extandrun') or (mode='preview')) and (selection='single') then //extract a single object
   begin
   if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
   filter1:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
   if copy(filter1,1,1)='.' then filter1:='*'+copy(filter1,2,length(filter1)-1);
   cl:=cl+' -- '+delimiter+escapefilename(filter1,desk_env)+delimiter;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_un7z_cl:=0;
end;

function compose_pea_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   i:integer;
   out_param,s,in_param,compr,vol_algo,obj_algo,strm_algo,bin_name,p_ext,rfs_ext:utf8string;
   vol_size:qword;
   name_ok:boolean;
begin
compose_pea_cl:=-1;
subfun:='archive';
if fun='PEA' then
   begin
   if Form_peach.StringGrid1.RowCount=1 then
      begin
      MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
      exit;
      end;
   case Form_peach.ComboBoxArchive1.ItemIndex of
      0: vol_size:=0; //trigger PEA to not split archive in volumes
      1: begin
         try
            vol_size:=Form_peach.SpinEditArchive1.Value;
            case Form_peach.ComboBoxArchive2.ItemIndex of
               1: vol_size:=vol_size*1024;
               2: vol_size:=vol_size*1024*1024;
               3: vol_size:=vol_size*1024*1024*1024;
               end;
         except
            MessageDlg(txt_chunk_size, mtWarning, [mbOK], 0);
            exit;
         end;
         end;
      2: vol_size:=1457664;//FD
      3: vol_size:=5*1024*1024;//limit for attachment size of some mail services
      4: vol_size:=10*1024*1024;//limit for attachment size of some mail services
      5: vol_size:=650*1024*1024;//CD 650 MB
      6: vol_size:=700*1024*1024;//CD 700 MB
      7: vol_size:=4*1024*1024*1024;//max file size for FAT32 filesystem
      8: vol_size:=4700372992;//size DVD+R (slightly smaller than DVD-R), source Wikipedia
      9: vol_size:=8543666176;//size for DVD-R DL (slightly smaller than DVD+R DL), source Wikipedia
      end;
   if vol_size=0 then p_ext:='.pea' else p_ext:='.000001.pea';
   //out_param
   if Form_peach.CheckBoxTarBefore.State=cbChecked then out_param:=Form_peach.StringGrid1.Cells[8,1]
   else
   out_param:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
   apply_timestamptoname(out_param,0);
   if ufileexists((out_param+p_ext)) then
      begin
      i:=1;
      name_ok:=false;
      s:=out_param;
      cutextension(s);
      repeat
         if ufileexists((s+'-'+inttostr(i)+p_ext)) then inc(i,1)
         else name_ok:=true;
      until name_ok = true;
      out_param:=s+'-'+inttostr(i);
      end;
   outname:=out_param+p_ext;
   case Form_peach.ComboBoxPEACompression1.ItemIndex of
      0: compr:='PCOMPRESS3';
      1: compr:='PCOMPRESS2';
      2: compr:='PCOMPRESS1';
      3: compr:='PCOMPRESS0';
      end;
   case Form_peach.ComboBoxPEA2.ItemIndex of //volume checks
      0: strm_algo:='EAX256 INTERACTIVE';
      1: strm_algo:='EAX INTERACTIVE';
      2: strm_algo:='HMAC INTERACTIVE';
      3: strm_algo:='WHIRLPOOL BATCH';
      4: strm_algo:='SHA512 BATCH';
      5: strm_algo:='SHA256 BATCH';
      6: strm_algo:='RIPEMD160 BATCH';
      7: strm_algo:='SHA1 BATCH';
      8: strm_algo:='MD5 BATCH';
      9: strm_algo:='CRC64 BATCH';
      10: strm_algo:='CRC32 BATCH';
      11: strm_algo:='ADLER32 BATCH';
      12: strm_algo:='NOALGO BATCH';
      end;
   case Form_peach.ComboBoxPEA3.ItemIndex of //object checks
      0: obj_algo:='WHIRLPOOL';
      1: obj_algo:='SHA512';
      2: obj_algo:='SHA256';
      3: obj_algo:='RIPEMD160';
      4: obj_algo:='SHA1';
      5: obj_algo:='MD5';
      6: obj_algo:='CRC64';
      7: obj_algo:='CRC32';
      8: obj_algo:='ADLER32';
      9: obj_algo:='NOALGO';
      end;
   case Form_peach.ComboBoxPEA4.ItemIndex of //stream checks
      0: vol_algo:='WHIRLPOOL';
      1: vol_algo:='SHA512';
      2: vol_algo:='SHA256';
      3: vol_algo:='RIPEMD160';
      4: vol_algo:='SHA1';
      5: vol_algo:='MD5';
      6: vol_algo:='CRC64';
      7: vol_algo:='CRC32';
      8: vol_algo:='ADLER32';
      9: vol_algo:='NOALGO';
      end;
   out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
   get_in_param(in_param,sel);
   in_param:='FROMCL '+in_param;
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
   cl:=bin_name+' '+fun+' '+out_param+' '+inttostr(vol_size)+' '+compr+' '+vol_algo+' '+obj_algo+' '+strm_algo+' '+in_param;
   end;
if fun='RFS' then
   begin
   if Form_peach.StringGrid1.RowCount=1 then
      begin
      MessageDlg(txt_split_noinput, mtWarning, [mbOK], 0);
      exit;
      end
   else
      if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) then
      else
         begin
         MessageDlg(txt_split+' '+txt_error_input_multi, mtWarning, [mbOK], 0);
         exit;
         end;
   //out_param
   out_param:=Form_peach.StringGrid1.Cells[8,1];
   rfs_ext:='001';
   out_param:=Form_peach.Editname3.Text;
   if ufileexists((out_param+'.001')) then
      begin
      i:=1;
      name_ok:=false;
      s:=out_param;
      cutextension(s);
      repeat
         if ufileexists((s+'-'+inttostr(i)+'.001')) then inc(i,1)
         else name_ok:=true;
      until name_ok = true;
      out_param:=s+'-'+inttostr(i);
      end;
   outname:=out_param+'.001';
   case Form_peach.ComboBoxArchive1.ItemIndex of
      0: vol_size:=0; //trigger PEA to not split archive in volumes
      1: begin
         try
            vol_size:=Form_peach.SpinEditArchive1.Value;
            case Form_peach.ComboBoxArchive2.ItemIndex of
               1: vol_size:=vol_size*1024;
               2: vol_size:=vol_size*1024*1024;
               3: vol_size:=vol_size*1024*1024*1024;
               end;
         except
            MessageDlg(txt_chunk_size, mtWarning, [mbOK], 0);
            exit;
         end;
         end;
      2: vol_size:=1457664;//FD
      3: vol_size:=5*1024*1024;//limit for attachment size of some mail services
      4: vol_size:=10*1024*1024;//limit for attachment size of some mail services
      5: vol_size:=650*1024*1024;//CD 650 MB
      6: vol_size:=700*1024*1024;//CD 700 MB
      7: vol_size:=4*1024*1024*1024;//max file size for FAT32 filesystem
      8: vol_size:=4700372992;//size DVD+R (slightly smaller than DVD-R), source Wikipedia
      9: vol_size:=8543666176;//size for DVD-R DL (slightly smaller than DVD+R DL), source Wikipedia
      end;
   case Form_peach.ComboBoxSplit3.ItemIndex of //stream checks
      0: vol_algo:='WHIRLPOOL';
      1: vol_algo:='SHA512';
      2: vol_algo:='SHA256';
      3: vol_algo:='RIPEMD160';
      4: vol_algo:='SHA1';
      5: vol_algo:='MD5';
      6: vol_algo:='CRC64';
      7: vol_algo:='CRC32';
      8: vol_algo:='ADLER32';
      9: vol_algo:='NOALGO';
      end;
   out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
   in_param:=delimiter+escapefilename(Form_peach.StringGrid1.Cells[8,1],desk_env)+delimiter;
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
   cl:=bin_name+' '+fun+' '+out_param+' '+inttostr(vol_size)+' '+vol_algo+' BATCH '+in_param;
end;
if (fun<>'PEA') and (fun<>'RFS') then
   begin
   MessageDlg(txt_error_function, mtError, [mbOK], 0);
   exit;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
compose_pea_cl:=0;
end;

function compose_unpea_cl(var cl,jobcode,outname:utf8string):integer;
var
   attr_param,out_param,in_param,bin_name,s:utf8string;
begin
compose_unpea_cl:=-1;
subfun:='extract';
outname:='';//unused
if Form_peach.EditOpenIn.Text=txt_none then
   begin
   MessageDlg(txt_open_noinput, mtWarning, [mbOK], 0);
   exit;
   end;
set_output_option(out_param);
if control_outpath(out_param)<>0 then exit;
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
s:=uextractfilename(Form_peach.EditOpenIn.Text);
if upcase(copy(s,length(s)-10,11))='.000001.PEA' then setlength(s,length(s)-11)
else setlength(s,length(s)-4);
out_param:=out_param+s;
if Form_peach.CheckBoxUnPeaAttributes.State=cbChecked then attr_param:='SETATTR'
else attr_param:='RESETATTR';
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
if fun='UNPEA' then cl:=bin_name+' '+fun+' '+in_param+' '+out_param+' RESETDATE '+attr_param+' EXTRACT2DIR INTERACTIVE';
if fun='RFJ' then cl:=bin_name+' '+fun+' '+in_param+' BATCH '+out_param;
if (fun<>'UNPEA') and (fun<>'RFJ') then
   begin
   MessageDlg(txt_error_function, mtError, [mbOK], 0);
   exit;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
compose_unpea_cl:=0;
end;

{//read only support for LPAQ from 2.7, after introduction of zpaq
function compose_lpaq_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   i:integer;
   s,out_param,in_param,bin_name,compression_level,lpaq_ver:utf8string;
begin
compose_lpaq_cl:=-1;
subfun:='archive';
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) then begin end //single input file
else //folder(s) and/or multiple files
   begin
   MessageDlg(archive_type+' '+txt_error_input_multi, mtWarning, [mbOK], 0);
   exit;
   end;
//LPAQ version
case Form_peach.RadioGroupLpaq.ItemIndex of
   0: lpaq_ver:='lpaq1';
   1: lpaq_ver:='lpaq5';
   2: lpaq_ver:='lpaq8';
   end;
//output name
if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
s:=s+'.'+lpaq_ver;
apply_timestamptoname(s,1);
get_new_archive_name(s);//check if name, with extension, exists
out_param:=s;
outname:=out_param;
//LPAQ Compression level 0..9
compression_level:=inttostr(Form_peach.ComboBoxLpaqComp.ItemIndex);
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
get_in_param(in_param,sel);
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'lpaq'+DirectorySeparator+lpaq_ver+EXEEXT+delimiter;
cl:=bin_name+' '+compression_level+' '+in_param+' '+out_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_lpaq_cl:=0;
end;}

function compose_unlpaq_cl(var cl,jobcode,outname:utf8string; real_extract:boolean):integer;
var
   s1,out_param,in_param,bin_name,lpaq_ver:utf8string;
begin
compose_unlpaq_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if control_outpath(out_param)<>0 then exit;
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,0);
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   s1:=uextractfilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row])
else
   s1:=uextractfilename(Form_peach.EditOpenIn.Text);
cutextension(s1);
out_param:=out_param+s1;
if out_param<>'' then if out_param[length(out_param)] = directoryseparator then setlength(out_param,length(out_param)-1);
outname:=out_param;
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   lpaq_ver:=copy(uextractfileext(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row]),2,length(uextractfileext(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row]))-1)
else
   lpaq_ver:=copy(uextractfileext(Form_peach.EditOpenIn.Text),2,length(uextractfileext(Form_peach.EditOpenIn.Text))-1);
if ufileexists((out_param)) then
      if MessageDlg(out_param+' '+txt_confirm_overwrite,mtWarning,[mbYes, mbNo],0)<>6 then exit
      else
         if real_extract=true then clearfile(out_param);
if out_param<>'' then out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'lpaq'+DirectorySeparator+lpaq_ver+EXEEXT+delimiter;
cl:=bin_name+' d '+in_param+' '+out_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_unlpaq_cl:=0;
end;

function compose_unpaq_cl(var cl,jobcode,outname:utf8string; real_extract:boolean):integer;
var
   out_param,in_param,bin_name,paq_ver:utf8string;
begin
compose_unpaq_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if control_outpath(out_param)<>0 then exit;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,0);
if out_param<>'' then if out_param[length(out_param)]=directoryseparator then out_param:=copy(out_param,1,length(out_param)-1);
outname:=out_param; //PAQ will skip extraction over existing object
//PAQ version, by extension; supported versions, as PAQ8F, must use -d switch for extraction and must follow extension convention with extension=name of the RadioGroupQuad: .PAQ?* where ? is release level and * is the variant
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   paq_ver:=copy(uextractfileext(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row]),2,length(uextractfileext(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row]))-1)
else
   paq_ver:=copy(uextractfileext(Form_peach.EditOpenIn.Text),2,length(uextractfileext(Form_peach.EditOpenIn.Text))-1);
if out_param<>'' then out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'paq'+DirectorySeparator+paq_ver+EXEEXT+delimiter;
cl:=bin_name+' -d '+in_param+' '+out_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_unpaq_cl:=0;
end;

function compose_zpaq_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   i:integer;
   s,out_param,in_param,bin_name,compression_level,paq_ver:utf8string;
begin
compose_zpaq_cl:=-1;
case Form_peach.RadioGroupPaq.ItemIndex of
   0: begin
      subfun:='archive';
      updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
      //check if input is suitable for chosen archive_type
      if Form_peach.StringGrid1.RowCount=1 then
         begin
         MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
         exit;
         end;
      if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) then begin end //single input file
      else //folder(s) and/or multiple files
         begin
         MessageDlg(archive_type+' '+txt_error_input_multi, mtWarning, [mbOK], 0);
         exit;
         end;
      paq_ver:='lpaq8';
      //output name
      if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
      else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
      s:=s+'.'+paq_ver;
      apply_timestamptoname(s,1);
      get_new_archive_name(s);//check if name, with extension, exists
      out_param:=s;
      outname:=out_param;
      case Form_peach.ComboBoxPAQ1.ItemIndex of
         0: compression_level:='3';
         1: compression_level:='6';
         2: compression_level:='9';
         end;
      out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
      get_in_param(in_param,sel);
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'lpaq'+DirectorySeparator+paq_ver+EXEEXT+delimiter;
      cl:=bin_name+' '+compression_level+' '+in_param+' '+out_param;
      end;
   1: begin
      paq_ver:='paq8o';
      subfun:='archive';
      updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
      //check if input is suitable for chosen archive_type
      if Form_peach.StringGrid1.RowCount=1 then
         begin
         MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
         exit;
         end;
      //output name
      if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
      else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
      s:=s+'.'+paq_ver;
      apply_timestamptoname(s,1);
      get_new_archive_name(s);//check if name, with extension, exists
      cutextension(s); //cut extension since paq RadioGroupQuad doesn't want extension
      out_param:=s;
      //Compression level; PAQ versions supported uses -0..8 as PAQ8F
      compression_level:='-'+inttostr(Form_peach.ComboBoxPAQ1.ItemIndex);
      case Form_peach.ComboBoxPAQ1.ItemIndex of
         0: compression_level:='-3';
         1: compression_level:='-5';
         2: compression_level:='-7';
         end;
      outname:=out_param+'.'+paq_ver;
      out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
      get_in_param(in_param,sel);
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'paq'+DirectorySeparator+paq_ver+EXEEXT+delimiter;
      cl:=bin_name+' '+compression_level+' '+out_param;
      if out_param<>trimright(in_param) then cl:=cl+' '+in_param;
      end;
   2: begin
      paq_ver:='zpaq';
      subfun:='archive';
      updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
      //check if input is suitable for chosen archive_type
      if Form_peach.StringGrid1.RowCount=1 then
         begin
         MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
         exit;
         end;
      //output name
      if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
      else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
      s:=s+'.zpaq';
      apply_timestamptoname(s,1);
      get_new_archive_name(s);//check if name, with extension, exists
      cutextension(s); //cut extension since paq RadioGroupQuad doesn't want extension
      out_param:=s;
      //Compression level
      case Form_peach.ComboBoxPAQ1.ItemIndex of
         0: compression_level:='a'+delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'zpaq'+DirectorySeparator+'min.cfg'+delimiter;
         1: compression_level:='a'+delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'zpaq'+DirectorySeparator+'mid.cfg'+delimiter;
         2: compression_level:='a'+delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'zpaq'+DirectorySeparator+'max.cfg'+delimiter;
         end;
      outname:=out_param+'.zpaq';
      out_param:=delimiter+escapefilename(out_param,desk_env)+'.zpaq'+delimiter;
      get_in_param(in_param,sel);
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'zpaq'+DirectorySeparator+'zpaq'+EXEEXT+delimiter;
      cl:=bin_name+' '+compression_level+' '+out_param;
      cl:=cl+' '+in_param;
      end;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_zpaq_cl:=0;
end;

function compose_unzpaq_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
var
   out_param,in_param,bin_name,modefun:utf8string;
begin
compose_unzpaq_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if control_outpath(out_param)<>0 then exit;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,0);
if out_param<>'' then if out_param[length(out_param)]=directoryseparator then out_param:=copy(out_param,1,length(out_param)-1);
outname:=out_param;
if out_param<>'' then out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'zpaq'+DirectorySeparator+'zpaq'+EXEEXT+delimiter;
if mode='list' then modefun:='l'
else modefun:='x';
//selection is currently unused
cl:=bin_name+' '+modefun+' '+in_param;//+' '+out_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_unzpaq_cl:=0;
end;

function compose_upx_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   P:TProcess;
   numread,numwritten:integer;
   fin,fout:file of byte;
   read_data:array[0..65535]of byte;
   in_param,bin_name,compression_level:utf8string;
begin
compose_upx_cl:=-1;
subfun:='archive';
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) and
   ((upcase(Form_peach.StringGrid1.Cells[2,1])='.EXE') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.COM') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.DLL') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.OCX') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.SCR') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.SFX') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.SYS') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.ELF') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.O') or
   (upcase(Form_peach.StringGrid1.Cells[2,1])='.SO') or
   ((Form_peach.StringGrid1.Cells[2,1]=''))) then //single input executable file
else //folder(s) and/or multiple files
   begin
   MessageDlg(archive_type+' '+txt_error_input_upx, mtWarning, [mbOK], 0);
   exit;
   end;
if (Form_peach.ComboBoxUPX.ItemIndex=0) and (Form_peach.CheckBoxUPX1.State=cbUnchecked) then
   begin
   MessageDlg(txt_upxorstrip, mtWarning, [mbOK], 0);
   exit;
   end;
if Form_peach.ComboBoxUPX.ItemIndex=10 then compression_level:='--best'
else compression_level:='-'+inttostr((Form_peach.ComboBoxUPX.ItemIndex));
get_in_param(in_param,sel);
outname:=escapefilename(Form_peach.StringGrid1.Cells[8,1],desk_env);
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'upx'+DirectorySeparator+'upx'+EXEEXT+delimiter;
cl:=bin_name+' '+compression_level;
cl:=cl+' '+in_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
//do backup (silent) before strip/upx
if Form_peach.CheckBoxUPX2.State=cbChecked then
   begin
   uassignfile(fin,Form_peach.StringGrid1.Cells[8,1]);
   filemode:=0;
   reset(fin);
   uassignfile(fout,Form_peach.StringGrid1.Cells[8,1]+'.backup');
   rewrite(fout);
   repeat
      blockread (fin,read_data,65536,numread);
      blockwrite (fout,read_data,numread,numwritten);
   until ((numread=0) or (numread<>numwritten));
   closefile(fin);
   closefile(fout);
   end;
//strip (silent) before upx
if Form_peach.CheckBoxUPX1.State=cbChecked then
   begin
   P:=TProcess.Create(nil);
   {$IFDEF MSWINDOWS}
   P.CommandLine:=utf8toenv('"'+executable_path+'res'+DirectorySeparator+'upx'+DirectorySeparator+'strip.exe" '+in_param);
   {$ENDIF}
   {$IFDEF UNIX}//assuming generic unix coming qith strip command
   P.CommandLine:=utf8toenv('strip '+in_param);
   {$ENDIF}
   if Form_peach.Visible=true then Application.ProcessMessages;
   P.Options := [poNoConsole, poWaitOnExit];
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
   end;
if Form_peach.ComboBoxUPX.ItemIndex=0 then //exit with -1 exitcode, not triggering launch_cl
   begin
   cl:='';
   exit;
   end;
compose_upx_cl:=0;
end;

function compose_quad_cl(var cl,jobcode,outname,sel:utf8string):integer;
var
   s,out_param,in_param,bin_name,compression_level,exetype:utf8string;
begin
compose_quad_cl:=-1;
subfun:='archive';
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,false);
//check if input is suitable for chosen archive_type
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
if (tdirs=0) and (Form_peach.StringGrid1.RowCount<3) then begin end //single input file
else //folder(s) and/or multiple files
   begin
   MessageDlg(archive_type+' '+txt_error_input_multi, mtWarning, [mbOK], 0);
   exit;
   end;
//output name
if Form_peach.CheckBoxTarBefore.State=cbChecked then s:=Form_peach.StringGrid1.Cells[8,1]
else s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
apply_timestamptoname(s,1);
//max Compression mode on/off
exetype:=Form_peach.RadioGroupQuad.Items[Form_peach.RadioGroupQuad.ItemIndex];
out_param:=s+'.'+exetype;
get_new_archive_name(out_param);//issue: it adds progressive number after extension (it's however quite intuitive to remove)
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
get_in_param(in_param,sel);
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'quad'+DirectorySeparator+exetype+EXEEXT+delimiter;
cl:=bin_name;
case Form_peach.RadioGroupQuad.ItemIndex of
   0: begin
      if Form_peach.CheckBoxQuadMax.State=cbChecked then compression_level:='-x'
      else compression_level:='';
      if compression_level<>'' then cl:=cl+' '+compression_level;
      end;
   1: begin
      cl:=cl+' e';
      if Form_peach.CheckBoxQuadMax.State=cbChecked then compression_level:='x'
      else compression_level:='';
      cl:=cl+compression_level;
      end;
   end;
cl:=cl+' '+in_param+' '+out_param;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_quad_cl:=0;
end;

function compose_unquad_cl(var cl,jobcode,outname:utf8string; real_extract:boolean):integer;
var
   s1,out_param,in_param,bin_name:utf8string;
begin
compose_unquad_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if control_outpath(out_param)<>0 then exit;
if out_param<>'' then if out_param[length(out_param)] <> directoryseparator then out_param:=out_param+directoryseparator;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,0);
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   s1:=uextractfilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row])
else
   s1:=uextractfilename(Form_peach.EditOpenIn.Text);
cutextension(s1);
out_param:=out_param+s1;
if out_param<>'' then if out_param[length(out_param)] = directoryseparator then setlength(out_param,length(out_param)-1);
outname:=out_param;
if ufileexists((out_param)) then
      if MessageDlg(out_param+' '+txt_confirm_overwrite,mtWarning,[mbYes, mbNo],0)<>6 then exit
      else
         if real_extract=true then clearfile(out_param); //otherwise QUAD will not overwrite existing files and returns error exit code
if out_param<>'' then out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
if uextractfileext(Form_peach.EditOpenIn.Text)='.quad' then
   begin
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'quad'+DirectorySeparator+'quad'+EXEEXT+delimiter;
   cl:=bin_name+' -d '+in_param+' '+out_param;
   end
else
   begin
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'quad'+DirectorySeparator+'balz'+EXEEXT+delimiter;
   cl:=bin_name+' d '+in_param+' '+out_param;
   end;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_unquad_cl:=0;
end;

function compose_unace_cl(var cl,jobcode,outname:utf8string; real_extract:boolean; mode,selection:utf8string):integer;
var
   out_param,in_param,bin_name,archive_function:utf8string;
begin
compose_unace_cl:=-1;
subfun:='extract';
set_output_option(out_param);
if (mode='ext') or (mode='extandrun') or ((mode='neutral') and (Form_peach.RadioGroupAction1.ItemIndex<2)) then
   if control_outpath(out_param)<>0 then exit;
if out_param<>'' then if out_param[length(out_param)] <> directoryseparator then out_param:=out_param+directoryseparator;
//in_param
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   in_param:=delimiter+escapefilename(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],desk_env)+delimiter
else
   in_param:=delimiter+escapefilename(Form_peach.EditOpenIn.Text,desk_env)+delimiter;
//folder policy
set_output_folder(out_param,in_param,real_extract,Form_peach.RadioGroupAction1.ItemIndex);
outname:=out_param;
//archive function
if mode='neutral' then
   case Form_peach.RadioGroupAction1.ItemIndex of
      0: archive_function:='x';
      1: archive_function:='e';
      2: begin
         archive_function:='l';
         subfun:='list';
         end;
      3: begin
         archive_function:='t';
         subfun:='test';
         end;
      end;
if mode='list' then //override function, if using list mode
   begin
   archive_function:='l';
   subfun:='list';
   end;
if mode='ext' then archive_function:='x';
if mode='extandrun' then //extandrun special feature, replaces archive function and overwrite policy: always extract the object, to be opened immediately after; if the object exist, ask about overwriting or skipping it
   begin
   if archive_function='e' then //extract without paths
      begin
      tempstring:=uextractfilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]);
      end
   else
      begin
      archive_function:='x';
      tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
      end;
   if Form_peach.CheckBox4.State=cbChecked then //option ignore archived paths for Extract and... override standard path policy; checking it it's possible ignoring paths only for Extract and... functions
      begin
      archive_function:='e';
      tempstring:=uextractfilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]);
      end;
   if Form_peach.StringGridList.Cells[2,Form_peach.StringGridList.Row]=txt_list_isfolder then //if the object is a directory, paths are always preserved (this rule override all other rules)
      begin
      archive_function:='x';
      tempstring:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
      end;
   if ufileexists((out_param+tempstring)) then
         if MessageDlg(out_param+tempstring+' '+txt_confirm_overwrite,mtWarning,[mbYes, mbNo],0)<>6 then exit;
   end;
if (archive_function<>'x') and (archive_function<>'e') then
   begin
   out_param:='';
   end;
//if out_param<>'' then out_param:=delimiter+escapefilename(out_param)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'unace'+DirectorySeparator+'unace'+EXEEXT+delimiter;
cl:=bin_name+' '+archive_function;
cl:=cl+' '+in_param;
//if out_param<>'' then cl:=cl+' '+out_param; changed the way to communicate out param; it's implicitely communicated setting working path since Win demo executable fails extracting to paths containing space
if out_param<>'' then usetcurrentdir((out_param));
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
compose_unace_cl:=0;
end;

procedure check_files(var in_param,cl:utf8string);
var bin_name,disp_type,algo_list:utf8string;
begin
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
case Form_peach.radiogroup1.ItemIndex of
   0: disp_type:='HEX';
   1: disp_type:='BASE64';
   2: disp_type:='LSBHEX';
   end;
if Form_peach.cbcheckall.state=cbchecked then algo_list:='ALL'
else
   begin
   algo_list:='';
   if Form_peach.cbadler.state=cbchecked then algo_list:=algo_list+'ADLER32 ';
   if Form_peach.cbcrc16.state=cbchecked then algo_list:=algo_list+'CRC16 ';
   if Form_peach.cbcrc24.state=cbchecked then algo_list:=algo_list+'CRC24 ';
   if Form_peach.cbcrc32.state=cbchecked then algo_list:=algo_list+'CRC32 ';
   if Form_peach.cbcrc64.state=cbchecked then algo_list:=algo_list+'CRC64 ';
   if Form_peach.cbed2k.state=cbchecked then algo_list:=algo_list+'ED2K ';
   if Form_peach.cbmd4.state=cbchecked then algo_list:=algo_list+'MD4 ';
   if Form_peach.cbmd5.state=cbchecked then algo_list:=algo_list+'MD5 ';
   if Form_peach.cbrmd.state=cbchecked then algo_list:=algo_list+'RIPEMD160 ';
   if Form_peach.cbsha1.state=cbchecked then algo_list:=algo_list+'SHA1 ';
   if Form_peach.cbsha224.state=cbchecked then algo_list:=algo_list+'SHA224 ';
   if Form_peach.cbsha256.state=cbchecked then algo_list:=algo_list+'SHA256 ';
   if Form_peach.cbsha384.state=cbchecked then algo_list:=algo_list+'SHA384 ';
   if Form_peach.cbsha512.state=cbchecked then algo_list:=algo_list+'SHA512 ';
   if Form_peach.cbwhirlpool.state=cbchecked then algo_list:=algo_list+'WHIRLPOOL ';
   end;
cl:=bin_name+' CHECK '+disp_type+' '+algo_list+' ON '+in_param;
end;

procedure decode_7za_exitcode(i,insize:integer; cl,s,outname:utf8string);
var
   outsize: qword;
   comp_ratio: integer;
   f: file of byte;
begin
case i of
   0: exit; //the report is shown only if an error condition's exit code is encountered
   1: begin
      s:=txt_7z_exitcode1+s;
      MessageDlg(s,mtWarning, [mbOK], 0);
      end;
   2: begin
      s:=txt_7z_exitcode2+s+char($0d)+char($0a)+' - '+txt_cl+' '+cl;
      MessageDlg(s,mtError, [mbOK], 0);
      end;
   7: begin
      s:=txt_7z_exitcode7+s+char($0d)+char($0a)+' - '+txt_cl+' '+cl;
      MessageDlg(s,mtError, [mbOK], 0);
      end;
   8: begin
      s:=txt_7z_exitcode8+s+char($0d)+char($0a)+' - '+txt_cl+' '+cl;
      MessageDlg(s,mtError, [mbOK], 0);
      end;
   255:
      begin
      s:=txt_7z_exitcode255+s;
      MessageDlg(s,mtWarning, [mbOK], 0);
      end;
   else
      begin
      s:=inttostr(i)+txt_7z_exitcodeunknown+' '+s+char($0d)+char($0a)+' - '+txt_cl+' '+cl;
      MessageDlg(s,mtError, [mbOK], 0);
      end;
   end;
end;

{procedure process_monitor_and_free(P:tprocess; s:utf8string);
begin
while P.Running do
   begin
   if Form_peach.Visible=true then application.processmessages;
   sleep(100);
   end;
P.Free;
end;}

procedure launch_cl(cl,jobcode,outname:utf8string); //control command line size then launch cl
var
   P: TProcess;
   tsin,tsout:TTimeStamp;
   tdiff,speed,insize:qword;
   q:qword;
   s,s1,jobtype:utf8string;
   i:integer;

   procedure withconsole;
   var
      i:integer;
   begin
   s:=char($0d)+char($0a)+' - '+txt_job_code+' '+jobcode+' '+char($0d)+char($0a);
   if (fun='7Z')  or (fun='PAQ') or (fun='UPX') then s:=s+' - '+txt_input+' '+inttostr(tfiles)+' '+txt_files+' '+inttostr(tdirs)+' '+txt_dirs+' '+inttostr(tsize)+' B'
   else
      begin
      s:=s+' - '+txt_input+' '+Form_peach.EditOpenIn.Text;
      s:=s+', '+inttostr(tsize)+' B';
      end;
   s1:=char($0d)+char($0a)+txt_input_list+char($0d)+char($0a);
   if (fun='7Z') or (fun='PAQ') then for i:=1 to Form_peach.StringGrid1.RowCount-1 do s1:=s1+Form_peach.StringGrid1.Cells[8,i]+char($0d)+char($0a)
   else s1:=s1+Form_peach.EditOpenIn.Text;
   tsin:=datetimetotimestamp(now);
   P.Options:=[poNewConsole];
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   while P.Running do
      begin
      if Form_peach.Visible=true then Application.ProcessMessages;
      sleep(100);
      end;
   tsout:=datetimetotimestamp(now);
   tdiff:=tsout.time-tsin.time;
   if tdiff=0 then tdiff:=1;
   q:=tsize * 1000; //speed is in B/s
   speed:=q div tdiff;
   s:=s+char($0d)+char($0a)+' - '+txt_elapsed+' '+inttostr(tdiff)+' ms';
   if speed>0 then s:=s+', '+txt_speed+' '+inttostr(speed)+' B/s';
   decode_7za_exitcode(P.ExitStatus,insize,cl,s,outname);
   end;

begin
//CharToOem(@cl,@cl2) and similar commands seem crashing (0.9.26)
if length(cl)>MAX_CL then
   begin
   MessageDlg(txt_cl_long, mtError, [mbOK], 0);
   exit;
   end;
//conf is no longer checked for being modified in current session since it is saved autonomously each time application exit from paneldefault
P:=TProcess.Create(nil);
if Form_peach.CheckBoxTarBefore.State=cbChecked then P.Options := [poWaitOnExit];
if updatingarchive=true then P.Options := [poWaitOnExit];
cl:=utf8toenv(cl); //passed to command line as ansi if needed
P.CommandLine:=cl;
i:=validatecl(cl);
if i<>0 then begin MessageDlg(txt_2_7_validatecl+' '+cl, mtWarning, [mbOK], 0); exit; end;
s:=Form_peach.EditOpenIn.Text;
if (fun='PEA') or (fun='UNPEA') or (fun='RFJ') or (fun='RFS') then P.Execute //launch Pea
else //launch either or pealauncher or ConsoleCreate application, depending on rules
   begin
   insize:=tsize;
   if (fun='UPX') or ((fun='UNACE') and (subfun<>'list')) then //UPX and UNACE(extract and test) should run without pealauncher (the latter to allowing input in console), ACE listing is possible in pealauncher because doesn't require feedback even if archive is encrypted
      begin
      withconsole;
      exit;
      end;
   if (zaout>0) or (subfun='list') or (subfun='test') or (subfun='bench') then
      begin
      if (subfun='list') or (subfun='test') then jobtype:='1'
      else
         if subfun='bench' then jobtype:='2'
         else
            {if (zaout=2) and (fun<>'UNARC') and (fun<>'ARC') then jobtype:='10' //arc needs either console or pipes
            else}
               if (zaout=1) then jobtype:='20'
               else jobtype:='0';
      if jobtype='2' then outname:='na';//empty parameter strings would be cut when passed as command line on *x-like systems
      P.CommandLine:=utf8toenv(delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pealauncher'+EXEEXT+delimiter+' '+jobtype+' '+inttostr(tsize)+' '+delimiter+escapefilename(outname,desk_env)+delimiter+' ')+cl;
      P.Execute;
      end
   else
      begin
      withconsole;
      end;
   end;
P.Free;
{if (subfun<>'list') and (subfun<>'info') and (subfun<>'test') then
   begin
   if fun_status='FILEBROWSER' then
      begin
      if (fun<>'RFJ') and (fun<>'RFS') then
         begin
         s:=extractfilepath(s);
         Form_peach.EditOpenIn.Text:=s;
         end;
      fun:='FILEBROWSER';
      end;
   process_monitor_and_free(P,s);
   end
else
   P.Free; }//possible implementation: wait each process to exit before launching the following ones
end;

function archive_mainsequence(var sel:utf8string):integer;
var
   cl,jobcode,outname:utf8string;
begin
archive_mainsequence:=-1;
if Form_peach.cbType.Text=STR_ZPAQ then
   begin
   if compose_zpaq_cl(cl,jobcode,outname,sel)=0 then
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end;
   exit;
   end;
if Form_peach.cbType.Text=STR_QUAD then
   begin
   if compose_quad_cl(cl,jobcode,outname,sel)=0 then
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end;
   exit;
   end;
if Form_peach.cbType.Text=STR_UPX then
   begin
   if compose_upx_cl(cl,jobcode,outname,sel)=0 then
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end
   else archive_mainsequence:=0;//exception for executable's compression: if only strip is used (silently) then UPX's cl is not composed and job is not launched, but operation is completed anyway
   exit;
   end;
if (Form_peach.cbType.Text=STR_ARC) or (Form_peach.cbType.Text=txt_sfx+' '+STR_ARC) then
   begin
   if compose_arc_cl(cl,jobcode,outname,sel)=0 then
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end;
   exit;
   end;
if Form_peach.cbType.Text=txt_custom then
   begin
   if compose_custom_cl(cl,jobcode,outname,sel)=0 then
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end;
   exit;
   end;
if (Form_peach.cbType.Text=STR_PEA) or (Form_peach.cbType.Text=txt_split) then
   begin
   if compose_pea_cl(cl,jobcode,outname,sel)=0 then
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end;
   end
else
   begin
   if compose_archive_cl(cl,jobcode,outname,sel)=0 then //7z types (including sfx 7z menu entry)
      begin
      launch_cl(cl,jobcode,outname);
      addtoarchivehistory(outname);
      archive_mainsequence:=0;
      end;
   end;
end;

procedure tarbefore(var sel:utf8string);
var
   P:TProcess;
   i:integer;
   cl,s,outname,out_param,in_param,archive_function,bin_name,type_option:utf8string;
   name_ok:boolean;
begin
using_tarbefore:=true;
//check input
if Form_peach.StringGrid1.RowCount=1 then
   begin
   MessageDlg(txt_archive_noinput_tolist, mtWarning, [mbOK], 0);
   exit;
   end;
//output name
s:=Form_peach.Editname3.Text+Form_peach.Edit4.Text;
out_param:=s+'.tar';
apply_timestamptoname(out_param,0);
//archive function
archive_function:='a';
if Form_peach.ComboBoxArchive1.ItemIndex=0 then //single archive
   begin
   if ufileexists((out_param)) then
      begin
      i:=1;
      name_ok:=false;
      s:=out_param;
      cutextension(s);
      repeat
         if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(i,1)
         else name_ok:=true;
      until name_ok = true;
      out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
      end;
   end;
//archive type
type_option:='-ttar';
// 7za uses -- as tag to distinguish a filename starting with - from a switch, however since Peach uses absolute filenames it should never occour and filenames are not checked for that condition
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
get_in_param(in_param,sel);
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
cl:=bin_name+' '+archive_function+' '+type_option;
cl:=cl+' '+out_param+' '+in_param;
Form_peach.StringGrid1.Cursor:=crHourGlass;
P:=TProcess.Create(nil);
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
P.Options := [poNoConsole, poWaitOnExit]; //wait for tar to be finished before starting compression
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
Form_peach.StringGrid1.Cursor:=crDefault;
Form_peach.StringGrid1.RowCount:=1;
addfilestr(Form_peach.StringGrid1,outname);
Form_peach.StringGrid1.AutoSizeColumns;
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
if Form_peach.Visible=true then Application.processmessages;
ares:=archive_mainsequence(sel);
using_tarbefore:=false;
clearfile(outname);
end;

{///////////////////////////////////////////////////////////////////////////////

quick PeaZip functions

///////////////////////////////////////////////////////////////////////////////}

function test_in_params(var j:integer):integer;
var i:integer;
begin
test_in_params:=-1;
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
j:=0;
for i:=2 to paramcount do
   if ufilegetattr(uexpandfilename(envtoutf8(paramstr(i)))) > 0 then j:=j+1
   else msg_not_accessible(i);
if j=0 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
test_in_params:=0;
end;

function test_in_params_7z(var j:integer):integer;
var i:integer;
begin
test_in_params_7z:=-1;
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
j:=0;
for i:=2 to paramcount do
   if testname(uexpandfilename(envtoutf8(paramstr(i))), Form_peach.StringGrid1)=0 then
      if ufilegetattr(uexpandfilename(envtoutf8(paramstr(i)))) > 0 then
         begin
         j:=j+1;
         if ufilegetattr(uexpandfilename(envtoutf8(paramstr(i)))) and faDirectory =0 then addfilestr(Form_peach.StringGrid1,uexpandfilename(envtoutf8(paramstr(i))))
         else addfolderstr(Form_peach.StringGrid1,uexpandfilename(envtoutf8(paramstr(i))));
         end
      else msg_not_accessible(i);
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
if j=0 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
test_in_params_7z:=0;
end;

function test_in_singleparam:integer;
begin
test_in_singleparam:=-1;
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
if paramcount>3 then
   begin
   MessageDlg(txt_split+' '+txt_error_input_multi, mtWarning, [mbOK], 0);
   Application.Terminate;
   end;
if ufilegetattr(uexpandfilename(envtoutf8(paramstr(2)))) <> 0 then
else
   begin
   msg_no_input;
   Application.Terminate;
   end;
test_in_singleparam:=0;
end;

function control_outparam(var out_param:utf8string):integer; //as control_outpath, but takes a filename instead of dirname as input
var
   out_path,out_name:utf8string;
begin
control_outparam:=-1;
out_path:=uextractfilepath(out_param);
out_name:=uextractfilename(out_param);
if control_outpath(out_path)<>0 then exit;
if out_param<>'' then if out_path[length(out_path)]<>directoryseparator then out_path:=out_path+directoryseparator;
out_param:=out_path+out_name;
control_outparam:=0;
end;

procedure add2pea; //directly create .pea archive with default parameters
var
   i,j:integer;
   cl,s,jobcode,out_param,in_param,bin_name:utf8string;
   name_ok:boolean;
begin
if test_in_params(j)<>0 then Application.Terminate;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'PEA';
fun:='PEA';
fun_status:=fun;
subfun:='archive';
//out_param
out_param:=uexpandfilename(envtoutf8(paramstr(2)));
if control_outparam(out_param)<>0 then exit;
if (paramcount=2) and (upcase(copy(out_param,length(out_param)-3,4))='.TAR') then //don't strip .tar extension
else
   if udirectoryexists((out_param)) then
   else cutextension(out_param);
if ufileexists((out_param+'.pea')) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+'.pea')) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i);
   end;
in_param:='';
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
for i:=2 to j+1 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
cl:=bin_name+' '+fun+' '+out_param+' 0 PCOMPRESS2 SHA256 CRC64 NOALGO BATCH FROMCL '+in_param;
launch_cl(cl,jobcode,out_param);
end;

procedure add2wipe;
var
   i,j:integer;
   P:TProcess;
   cl,in_param,eraselevel,bin_name:utf8string;
begin
if test_in_params(j)<>0 then Application.Terminate;
   if MessageDlg(txt_wipe,mtWarning,[mbYes,mbNo], 0)=6 then
      begin
      for i:=2 to j+1 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter;
      case erasepasses of
         0: eraselevel:='FAST';
         1: eraselevel:='MEDIUM';
         2: eraselevel:='SLOW';
         3: eraselevel:='VERY_SLOW';
      end;
      P:=TProcess.Create(nil);
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      cl:=bin_name+' WIPE '+eraselevel+' '+in_param;
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
end;

procedure add2compare;
var
   i,j:integer;
   P:TProcess;
   cl,in_param,bin_name:utf8string;
begin
if test_in_params(j)<>0 then Application.Terminate;
for i:=2 to 3 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter;
P:=TProcess.Create(nil);
{$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
cl:=bin_name+' COMPARE '+in_param;
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
end;

procedure direct_split(var fname: UTF8string);
var
   i:integer;
   cl,s,s1,jobcode,out_param,in_param,bin_name:utf8string;
   name_ok:boolean;
begin
s1:=fun;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'RFS';
fun:='RFS';
fun_status:='FILEBROWSER';
subfun:='archive';
//out_param
out_param:=uexpandfilename(fname);
if control_outparam(out_param)<>0 then exit;
if ufileexists((out_param+'.001')) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+'.001')) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i);
   end;
in_param:='';
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
in_param:=delimiter+escapefilename(uexpandfilename(fname),desk_env)+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
cl:=bin_name+' '+fun+' '+out_param+' ASK ASK BATCH '+in_param;
launch_cl(cl,jobcode,out_param);
fun:=s1;
fun_status:=fun;
end;

procedure direct_join(var fname: UTF8string);
var
   cl,s,s1,jobcode,in_param,out_param,bin_name:utf8string;
begin
s1:=fun;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'RFS';
fun:='RFJ';
fun_status:='FILEBROWSER';
subfun:='extract';
s:=escapefilename(uexpandfilename(fname),desk_env);
in_param:=delimiter+s+delimiter;
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
out_param:=uextractfilepath(s);
if control_outpath(out_param)<>0 then exit;
if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
s:=uextractfilename(s);
cutextension(s);
out_param:=out_param+s;
cl:=bin_name+' '+fun+' '+in_param+' BATCH '+delimiter+out_param+delimiter;
launch_cl(cl,jobcode,in_param);
fun:=s1;
fun_status:=fun;
end;

procedure add2split; //directly split file asking for MB size
var
   fname:UTF8string;
begin
if test_in_singleparam<>0 then Application.Terminate;
fname:=envtoutf8(paramstr(2));
direct_split(fname);
end;

procedure add2crypt; //directly create encrypted .pea archive with default parameters
var
   i,j:integer;
   cl,s,jobcode,out_param,in_param,bin_name:utf8string;
   name_ok:boolean;
begin
if test_in_params(j)<>0 then Application.Terminate;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'PEA';
fun:='PEA';
fun_status:=fun;
subfun:='archive';
//out_param
out_param:=uexpandfilename(envtoutf8(paramstr(2)));
if control_outparam(out_param)<>0 then exit;
if (paramcount=2) and (upcase(copy(out_param,length(out_param)-3,4))='.TAR') then //don't strip .tar extension
else
   if udirectoryexists((out_param)) then
   else cutextension(out_param);
if ufileexists((out_param+'.pea')) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+'.pea')) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i);
   end;
in_param:='';
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
for i:=2 to j+1 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
cl:=bin_name+' '+fun+' '+out_param+' 0 PCOMPRESS2 SHA256 CRC64 EAX256 INTERACTIVE FROMCL '+in_param;
launch_cl(cl,jobcode,out_param);
end;

procedure add27z(mailfun:utf8string); //directly create .7z archive with default parameters
var
   i,j:integer;
   s,cl,jobcode,out_param,in_param,bin_name,outname:utf8string;
   name_ok:boolean;
begin
if test_in_params_7z(j)<>0 then Application.Terminate;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'7z_archive';
fun:='7Z';
fun_status:=fun;
subfun:='archive';
//output name
s:=uexpandfilename(envtoutf8(paramstr(2)));
if (paramcount=2) and (upcase(copy(s,length(s)-3,4))='.TAR') then //don't strip .tar extension
else
   if udirectoryexists((s)) then
   else cutextension(s);
out_param:=s+'.7z';
if control_outparam(out_param)<>0 then exit;
if ufileexists((out_param)) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
   end;
in_param:='';
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
for i:=2 to j+1 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
case level_7z of
   0: cl:=bin_name+' a -t7z -m0=Copy -mmt=on -mx0 ';
   1: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx1 ';
   2: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx3 ';
   3: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx5 ';
   4: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx7 ';
   5: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx9 ';
   end;
if mailfun='mail' then cl:=cl+'-seml ';
cl:=cl+out_param+' '+in_param;
launch_cl(cl,jobcode,outname);
end;

procedure add2separate7z; //directly create separate .7z archives, one for input object, with default parameters
var
   i,j:integer;
   s,cl,jobcode,out_param,in_param,bin_name,outname:utf8string;
   name_ok:boolean;
begin
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
for i:=2 to paramcount do
   begin
   if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
      begin
      in_param:=uexpandfilename(envtoutf8(paramstr(i)));
      jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'7z_archive';
      fun:='7Z';
      fun_status:=fun;
      subfun:='archive';
      //output name
      s:=in_param;
      if (upcase(copy(s,length(s)-3,4))='.TAR') then //never strip .tar extension (since is archiving single objects)
      else
         if udirectoryexists((s)) then
         else cutextension(s);
      out_param:=s+'.7z';
      if control_outparam(out_param)<>0 then exit;
      if ufileexists((out_param)) then
         begin
         j:=1;
         name_ok:=false;
         s:=out_param;
         cutextension(s);
         repeat
            if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(j,1)
            else name_ok:=true;
         until name_ok = true;
         out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
         end;
      outname:=out_param;
      out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
      in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
      case level_7z of
         0: cl:=bin_name+' a -t7z -m0=Copy -mmt=on -mx0 ';
         1: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx1 ';
         2: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx3 ';
         3: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx5 ';
         4: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx7 ';
         5: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx9 ';
         end;
      cl:=cl+out_param+' '+in_param;
      launch_cl(cl,jobcode,outname);
      sleep(random(500)); //if multiple archives are submitted sequentially, let some times between accesses to configuration / backend programs etc
      end
   else msg_not_accessible(i);
   end;
end;

procedure add2sfx7z(mailfun:utf8string); //directly create sfx self extracting 7z archive with default parameters
var
   i,j:integer;
   s,cl,jobcode,out_param,in_param,bin_name,outname:utf8string;
   name_ok:boolean;
begin
if test_in_params_7z(j)<>0 then Application.Terminate;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'7z_archive';
fun:='7Z';
fun_status:=fun;
subfun:='archive';
//output name
//if (paramcount=2) and (upcase(copy(s,length(s)-3,4))='.TAR') then //don't strip .tar extension
//else... mechanism not used for sfx
s:=uexpandfilename(envtoutf8(paramstr(2)));
if udirectoryexists((s)) then
else cutextension(s);
out_param:=s+'.exe';
if control_outparam(out_param)<>0 then exit;
if ufileexists((out_param)) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
   end;
in_param:='';
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
for i:=2 to j+1 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
case level_7z of
   0: cl:=bin_name+' a -t7z -m0=Copy -mmt=on -mx0 ';
   1: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx1 ';
   2: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx3 ';
   3: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx5 ';
   4: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx7 ';
   5: cl:=bin_name+' a -t7z -m0='+method_7z+' -mmt=on -mx9 ';
   end;
cl:=cl+' -sfx7z.sfx ';
if mailfun='mail' then cl:=cl+'-seml ';
cl:=cl+out_param+' '+in_param;
launch_cl(cl,jobcode,outname);
end;

procedure add2zip(mailfun:utf8string); //directly create .zip archive with default parameters, optionally send by mail (Outlook/OE)
var
   i,j:integer;
   s,cl,jobcode,out_param,in_param,bin_name,outname:utf8string;
   name_ok:boolean;
begin
if test_in_params_7z(j)<>0 then Application.Terminate;
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'7z_archive';
fun:='7Z';
fun_status:=fun;
subfun:='archive';
//output name
s:=uexpandfilename(envtoutf8(paramstr(2)));
if (paramcount=2) and (upcase(copy(s,length(s)-3,4))='.TAR') then //don't strip .tar extension
else
   if udirectoryexists((s)) then
   else cutextension(s);
out_param:=s+'.zip';
if control_outparam(out_param)<>0 then exit;
if ufileexists((out_param)) then
   begin
   i:=1;
   name_ok:=false;
   s:=out_param;
   cutextension(s);
   repeat
      if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(i,1)
      else name_ok:=true;
   until name_ok = true;
   out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
   end;
in_param:='';
outname:=out_param;
out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
for i:=2 to j+1 do in_param:=in_param+delimiter+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)+delimiter+' ';
bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
case level_zip of
   0: cl:=bin_name+' a -tzip -mm=Copy -mmt=on -mx0 ';
   1: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx1 ';
   2: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx3 ';
   3: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx5 ';
   4: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx7 ';
   5: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx9 ';
   end;
if mailfun='mail' then cl:=cl+'-seml ';
cl:=cl+out_param+' '+in_param;
launch_cl(cl,jobcode,outname);
end;

procedure add2separatezip; //directly create separate .zip archives, one for input object, with default parameters
var
   i,j:integer;
   s,cl,jobcode,out_param,in_param,bin_name,outname:utf8string;
   name_ok:boolean;
begin
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
for i:=2 to paramcount do
   begin
   if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
      begin
      in_param:=uexpandfilename(envtoutf8(paramstr(i)));
      jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+'7z_archive';
      fun:='7Z';
      fun_status:=fun;
      subfun:='archive';
      //output name
      s:=in_param;
      if (upcase(copy(s,length(s)-3,4))='.TAR') then //never strip .tar extension (since is archiving single objects)
      else
         if udirectoryexists((s)) then
         else cutextension(s);
      out_param:=s+'.zip';
      if control_outparam(out_param)<>0 then exit;
      if ufileexists((out_param)) then
         begin
         j:=1;
         name_ok:=false;
         s:=out_param;
         cutextension(s);
         repeat
            if ufileexists((s+'-'+inttostr(i)+uextractfileext(out_param))) then inc(j,1)
            else name_ok:=true;
         until name_ok = true;
         out_param:=s+'-'+inttostr(i)+uextractfileext(out_param);
         end;
      outname:=out_param;
      out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
      in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
      case level_zip of
         0: cl:=bin_name+' a -tzip -mm=Copy -mmt=on -mx0 ';
         1: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx1 ';
         2: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx3 ';
         3: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx5 ';
         4: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx7 ';
         5: cl:=bin_name+' a -tzip -mm='+method_zip+' -mmt=on -mx9 ';
         end;
      cl:=cl+out_param+' '+in_param;
      launch_cl(cl,jobcode,outname);
      sleep(random(500));
      end
   else msg_not_accessible(i);
   end;
end;

function pw_pre_test(in_param,exename:utf8string):integer;
var
   P:TProcess;
   cl:utf8string;
begin
P:=TProcess.Create(nil);
if exename='arc' then
   begin
   P.Options := [poNoconsole, poWaitOnExit, poUsePipes];
   cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'arc'+DirectorySeparator+'arc'+EXEEXT+delimiter+' t '+delimiter+escapefilename(in_param,desk_env)+delimiter+' -pdefault';
   end
else
   begin
   P.Options := [poNoConsole, poWaitOnExit];
   cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter+' t -pdefault '+delimiter+escapefilename(in_param,desk_env)+delimiter;
   end;
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
pw_pre_test:=P.exitstatus;
P.Free;
end;

procedure reopen_7z;
var
   s1,s2:utf8string;
begin
Form_peach.StringGridList.Cursor:=crHourGlass;
s1:=Form_peach.EditUn7zaFilter.Text;
s2:=Form_peach.EditUn7zaFilterExclude.Text;
browse_special_cases;
do_prebrowse;
Form_peach.EditUn7zaFilter.Text:=s1;
Form_peach.EditUn7zaFilterExclude.Text:=s2;
if browsingmode7z=0 then
   begin
   list_un7z('browse');
   status0:=txt_list_browsing;
   end
else list_un7z_expandall;
Form_peach.StringGridList.PopupMenu:=Form_peach.PopupOpen;
Form_peach.StringGridList.Cursor:=crDefault;
end;

function ask_pwkeyfile_core:integer;
var
   s1,s1b,s2:utf8string;
   encfn,shp:boolean;
begin
ask_pwkeyfile_core:=-1;
//read previous values
s1:=Form2.EditUn7zaPW.Text;
s1b:=Form2.EditUn7zaPW1.Text;
s2:=Form2.EditName3.Text;
if Form2.CheckBoxEncFn.State=cbChecked then encfn:=true else encfn:=false;
if Form2.CheckBoxShowPWField.State=cbChecked then shp:=true else shp:=false;
//show form
Form2.Showmodal;
{1 cancel/close, revert to previous values
 0 ok, keep new values
-1 error in validation of values (either new or old)}
if form2.ModalResult=1 then ask_pwkeyfile_core:=0
else
   begin
   ask_pwkeyfile_core:=1;
   Form2.EditUn7zaPW.Text:=s1;
   Form2.EditUn7zaPW1.Text:=s1b;
   Form2.EditName3.Text:=s2;
   if encfn=true then Form2.CheckBoxEncFn.State:=cbChecked else Form2.CheckBoxEncFn.State:=cbUnChecked;
   if shp=true then Form2.CheckBoxShowPWField.State:=cbChecked else Form2.CheckBoxShowPWField.State:=cbUnChecked;
   if Form2.CheckBoxShowPWField.State=cbChecked then
      begin
      Form2.EditUn7zaPW.PasswordChar:=#0;
      Form2.lablelistPath3.Visible:=false;
      Form2.EditUn7zaPW1.visible:=false;
      Form2.EditUn7zaPW1.Text:='';
      end
   else
      begin
      Form2.EditUn7zaPW.PasswordChar:='*';
      Form2.lablelistPath3.Visible:=true;
      Form2.EditUn7zaPW1.visible:=true;
      end;
   end;
if Form2.CheckBoxShowPWField.State=cbChecked then showpwfield:=1 else showpwfield:=0;
if showpwfield=0 then
   if Form2.EditUn7zaPW.Text<>Form2.EditUn7zaPW1.Text then
      begin
      MessageDlg(txt_error_passwordnotmatch, mtWarning, [mbOK], 0);
      ask_pwkeyfile_core:=-1;
      exit;
      end;
s1:=Form2.EditUn7zaPW.Text;
s2:=Form2.EditName3.Text;
if (s1='') and (s2<>'') then
   begin
   MessageDlg(txt_pw_empty, mtWarning, [mbOK], 0);
   ask_pwkeyfile_core:=-1;
   exit;
   end;
if s2<>'' then
   if prepend_keyfile(s1,s2)<>0 then
      begin
      MessageDlg(s2+' '+txt_keyfile_not_found, mtError,[mbOk],0);
      ask_pwkeyfile_core:=-1;
      exit;
      end;
if pw4cl('-p',s1)<>0 then
   begin
   msg_pw_errorchar;
   ask_pwkeyfile_core:=-1;
   exit;
   end;
setpwicons;
end;

function ask_pwkeyfile:integer;
begin
ask_pwkeyfile:=-1;
repeat
ask_pwkeyfile:=ask_pwkeyfile_core;
until ask_pwkeyfile>=0;
if ask_pwkeyfile=0 then
   if form_peach.visible=true then
      if form_peach.stringgridlist.Cells[1,1]='' then
         begin
         if fun='UN7Z' then reopen_7z;
         if fun='UNARC' then browse_arc('flat');
         end;
end;

function ask_pwkeyfile_context(var pw:utf8string):integer;
var
   s1,s2:utf8string;
begin
ask_pwkeyfile_context:=-1;
if ask_pwkeyfile<>0 then exit;
if zaout>0 then
   begin
   pw:=Form2.EditUn7zaPW.Text;
   if Form2.EditName3.Text<>'' then
      if prepend_keyfile(pw,Form2.EditName3.Text)<>0 then
         begin
         MessageDlg(Form2.EditName3.Text+' '+txt_keyfile_not_found, mtError,[mbOk],0);
         exit;
         end;
   end
else pw:='';
if pw<>'' then
   begin
   if pw4cl('-p',pw)<>0 then
      begin
      msg_pw_errorchar;
      exit;
      end;
   end
else if zaout>0 then pw:='-pdefault';
ask_pwkeyfile_context:=0;
end;

function testencrypted:integer;
var
   s1,s2:utf8string;
begin
testencrypted:=-1;
if (fun<>'UNARC') and
   (fun<>'UN7Z') then exit;
if seemencrypted=true then
   if Form2.Editun7zaPW.Text='' then
      begin
      if ask_pwkeyfile<>0 then exit;
      end;
testencrypted:=0;
end;

function testencrypted_open:integer;
var
   s1,s2:utf8string;
begin
testencrypted_open:=-1;
if (fun<>'UNARC') and
   (fun<>'UN7Z') then exit;
if seemencrypted=true then
   if Form2.Editun7zaPW.Text='' then
      begin
      if Form_peach.StringGridList.Rowcount>1 then
         if Form_peach.StringGridList.Cells[1,1]='' then
            if ask_pwkeyfile<>0 then exit;
      end;
testencrypted_open:=0;
end;

procedure extractfromname(var in_param,out_param:utf8string);
var
   j:integer;
   P:TProcess;
   s,cl,jobcode,bin_name,overwrite_policy,archive_function,paq_ver,lpaq_ver,outname,pw,ext:utf8string;
   f: file of byte;
   fsize:qword;
begin
   j:=testinput(in_param,true);
   case j of
      0: begin
         //if MessageDlg(envtoutf8(in_param)+' '+txt_type_custom_selected, mtConfirmation, [mbYes,mbNo], 0)=7 then
         //else
         //   begin
            P:=TProcess.Create(nil);
            in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
            bin_name:=delimiter+escapefilename(executable_path,desk_env)+'peazip'+EXEEXT+delimiter;
            {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
            cl:=bin_name+' -ext2open '+in_param; //ext2open handles a single input in open interface
            cl:=utf8toenv(cl);
            P.CommandLine:=cl;
            if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
            P.Execute;
            P.Free;
            exit;
         //   end;
         end;
      1: begin
         //out_param:='AUTONAME';
         out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
         cl:=bin_name+' UNPEA '+in_param+' '+out_param+' RESETDATE SETATTR EXTRACT2DIR INTERACTIVE';
         fun:='UNPEA';
         fun_status:=fun;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         launch_cl(cl,jobcode,out_param);
         end;
      2: begin
         out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
         cl:=bin_name+' RFJ '+in_param+' BATCH AUTONAME';
         fun:='RFJ';
         fun_status:=fun;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         launch_cl(cl,jobcode,out_param);
         end;
      5: begin
         fun:='UNPAQ';
         fun_status:=fun;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         paq_ver:=copy(uextractfileext(in_param),2,length(uextractfileext(in_param))-1);
         outname:=out_param;
         if out_param<>'' then if out_param[length(out_param)]=directoryseparator then setlength(out_param,length(out_param)-1);
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'paq'+DirectorySeparator+paq_ver+EXEEXT+delimiter;
         cl:=bin_name+' -d '+in_param+' '+out_param;
         launch_cl(cl,jobcode,outname);
         end;
      6: begin
         fun:='UNQUAD';
         fun_status:=fun;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         ext:=copy(uextractfileext(cl),2,length(uextractfileext(cl))-1);
         s:=uextractfilename(in_param);
         cutextension(s);
         out_param:=out_param+s;
         outname:=out_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'quad'+DirectorySeparator+'quad'+EXEEXT+delimiter;
         if ext='quad' then cl:=bin_name+' -d '+in_param+' '+out_param
         else cl:=bin_name+' d '+in_param+' '+out_param;
         launch_cl(cl,jobcode,outname);
         end;
      7: begin
         if not(ufileexists((executable_path+'res'+directoryseparator+'unace'+directoryseparator+'unace'+EXEEXT))) then
            begin
            MessageDlg(txt_ace_missing, mtWarning, [mbOK], 0);
            Form_peach.Close;//don't skip on form close event
            exit;
            end;
         fun:='UNACE';
         fun_status:=fun;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         outname:=out_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'unace'+DirectorySeparator+'unace'+EXEEXT+delimiter;
         cl:=bin_name+' x '+in_param;
         if out_param<>'' then usetcurrentdir((out_param));
         launch_cl(cl,jobcode,outname);
         end;
      8: begin
         fun:='UNLPAQ';
         fun_status:=fun;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         lpaq_ver:=copy(uextractfileext(in_param),2,length(uextractfileext(in_param))-1);
         s:=uextractfilename(in_param);
         cutextension(s);
         out_param:=out_param+s;
         outname:=out_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'lpaq'+DirectorySeparator+lpaq_ver+EXEEXT+delimiter;
         cl:=bin_name+' d '+in_param+' '+out_param;
         launch_cl(cl,jobcode,outname);
         end;
      9: begin
         fun:='UNARC';
         fun_status:=fun;
         pw:='-pdefault';
         if zaout>0 then
            begin
            case extpw of
               1: begin
                  try
                  srcfilesize_multipart(in_param,fsize);
                  if fsize < 100*1024*1024 then
                     if pw_pre_test(in_param,'arc')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;
                     end;
                  except
                  end;
                  end;
               2: begin
                  if pw_pre_test(in_param,'arc')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;
                     end;
                  end;
               3: begin
                  if ask_pwkeyfile_context(pw)<>0 then exit;
                  end;
               end;
            end;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         //folder policy
         set_output_folder(out_param,in_param,true,0);
         if out_param<>'' then if out_param[length(out_param)]=directoryseparator then setlength(out_param,length(out_param)-1); //arc doesn't want directoryseparator as last character
         //overwrite policy
         case extoptarc of
            //0: overwrite_policy:='-op';//ask (use in Console mode)
            1: overwrite_policy:='-o+';//overwrite
            2: overwrite_policy:='-o-';//skip
            else overwrite_policy:='-o-';//for avoid blocking out of console mode, skip existing files replaces ask option
            end;
         outname:=out_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         out_param:=delimiter+'-dp'+escapefilename(out_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'arc'+DirectorySeparator+'arc'+EXEEXT+delimiter;
         cl:=bin_name+' x '+overwrite_policy+' '+in_param;
         if pw<>'' then cl:=cl+' '+pw;
         cl:=cl+' '+out_param;
         launch_cl(cl,jobcode,outname);
         end;
      else
         begin
         fun:='UN7Z';
         fun_status:=fun;
         pw:='-pdefault';
         if zaout>0 then
            begin
            case extpw of
               1: begin
                  try
                  srcfilesize_multipart(in_param,fsize);
                  if fsize < 100*1024*1024 then
                     if pw_pre_test(in_param,'7z')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;
                     end;
                  except
                  end;
                  end;
               2: begin
                  if pw_pre_test(in_param,'7z')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;
                     end;
                  end;
               3: begin
                  if ask_pwkeyfile_context(pw)<>0 then exit;
                  end;
               end;
            end;
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
            try
               srcfilesize_multipart(in_param,tsize); //expressed in B
            except
            end;
         //folder policy
         set_output_folder(out_param,in_param,true,0);
         //overwrite policy
         case extopt7z of
            0: overwrite_policy:='-aou';//auto rename extracting files
            1: overwrite_policy:='-aot';//auto rename existing files
            2: overwrite_policy:='-aoa';//overwrite all existing files
            3: overwrite_policy:='-aos';//skip existing files
            end;
         outname:=out_param;
         out_param:=delimiter+'-o'+escapefilename(out_param,desk_env)+delimiter;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
         archive_function:='x';
         cl:=bin_name+' '+archive_function;
         cl:=cl+' '+overwrite_policy;
         cl:=cl+' '+out_param;
         cl:=cl+' '+pw;
         cl:=cl+' '+in_param;
         launch_cl(cl,jobcode,outname);
         end;
      end;
end;

procedure ext2here(tooption,folderoption:utf8string); //directly extract archive's content in archive's folder; for the moment direct extract/list/test functions doesn't ask for keyfile, but keyfile's hash can be prepended to the password
var
   i,k:integer;
   s,s1,out_param,in_param:utf8string;
   out_created:boolean;
   label 1;
begin
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
for i:=2 to paramcount do
   begin
   if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
   else
      begin
      msg_not_accessible(i);
      goto 1;
      end;
   subfun:='extract';
   in_param:=uexpandfilename(envtoutf8(paramstr(i)));
   //determination of out_param
   if tooption='askto' then
      begin
      if testinput(in_param,true)<>0 then
         begin
         Form_peach.SelectDirectoryDialog1.InitialDir:=local_desktop;
         if Form_peach.SelectDirectoryDialog1.Execute then
            if Form_peach.SelectDirectoryDialog1.FileName<>'' then
               begin
               out_param:=Form_peach.SelectDirectoryDialog1.FileName;
               end
            else exit
         else exit;
         end
      else //jump to handling custom file types
         begin
         extractfromname(in_param,out_param);
         exit;
         end;
      end
   else
      begin
      out_param:=uextractfilepath(in_param);
      if control_outpath(out_param)<>0 then exit;
      end;
   if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
   out_created:=true;
   if folderoption='askfolder' then //unused mode, for now
      begin
         if MessageDlg(txt_autofolder, mtConfirmation,[mbYes, mbNo],0)=6 then
         begin
         s1:=uextractfilename(in_param);
         cutextension(s1);
         s:=out_param+s1;
         k:=1;
         repeat
            try
            umkdir(s);
            out_created:=true;
            except
            out_created:=false;
            s:=out_param+s1+'-'+inttostr(k);
            k:=k+1;
            end;
         until out_created=true;
         out_param:=s;
         end;
      end;
   if folderoption='newfolder' then
      begin
      s1:=uextractfilename(in_param);
      cutextension(s1);
      s:=out_param+s1;
      k:=1;
      repeat
         try
         umkdir(s);
         out_created:=true;
         except
         out_created:=false;
         s:=out_param+s1+'-'+inttostr(k);
         k:=k+1;
         end;
      until out_created=true;
      out_param:=s;
      end;
   if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
   //end determination of out_param
   extractfromname(in_param,out_param);
   1:
   end;
end;

procedure ext2full;
var
   P:TProcess;
   cl,bin_name,in_param,out_param,s:utf8string;
   f:file of byte;
   i:integer;
   label 1;
begin
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
for i:=2 to paramcount do
   begin
   if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
   else
      begin
      msg_not_accessible(i);
      goto 1;
      end;
   subfun:='extract';
   in_param:=uexpandfilename(envtoutf8(paramstr(i)));
   try
      srcfilesize_multipart(in_param,tsize); //expressed in B
   except
   end;
   out_param:=uextractfilepath(in_param);
   if control_outpath(out_param)<>0 then exit;
   if out_param<>'' then if out_param[length(out_param)]<>directoryseparator then out_param:=out_param+directoryseparator;
   case testinput(in_param,true) of //custom handled by -ext2open, pea and rfj handled by pea
      0: begin
         extractfromname(in_param,out_param);
         exit;
         end;
      1: begin
         P:=TProcess.Create(nil);
         {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
         out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
         cl:=bin_name+' UNPEA '+in_param+' '+out_param+' RESETDATE SETATTR EXTRACT2DIR INTERACTIVE';
         cl:=utf8toenv(cl);
         P.CommandLine:=cl;
         if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
         P.Execute;
         P.Free;
         exit;
         end;
      2: begin
         P:=TProcess.Create(nil);
         {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
         //out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter;
         cl:=bin_name+' RFJ '+in_param+' BATCH AUTONAME';
         cl:=utf8toenv(cl);
         P.CommandLine:=cl;
         if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
         P.Execute;
         P.Free;
         exit;
         end;
      7: begin
         if not(ufileexists((executable_path+'res'+directoryseparator+'unace'+directoryseparator+'unace'+EXEEXT))) then
            begin
            MessageDlg(txt_ace_missing, mtWarning, [mbOK], 0);
            exit;
            end;
         end;
      end;
   P:=TProcess.Create(nil);
   in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
   out_param:=delimiter+escapefilename(out_param,desk_env)+delimiter;
   {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
   if (zaout=1) then s:='120' else s:='100'; //jobtype/modeofuse for extraction only, in PeaLauncher's fullmode
   cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pealauncher'+EXEEXT+delimiter+' '+s+' '+inttostr(tsize);
   cl:=cl+' '+out_param+' '+in_param;
   P.CommandLine:=utf8toenv(cl);
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
   1:
   end;
end;

procedure ext2lt(ltfun:utf8string); //directly list or MenuItemOpen_test archive(s)
var
   i,j:integer;
   cl,jobcode,out_param,in_param,bin_name,archive_function:utf8string;
   f: file of byte;
   fsize:qword;
   label 1;
begin
if paramcount<2 then
   begin
   msg_no_input;
   Application.Terminate;
   end;
for i:=2 to paramcount do
   begin
   if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
   else
      begin
      msg_not_accessible(i);
      goto 1;
      end;
   in_param:=uexpandfilename(envtoutf8(paramstr(i)));
   j:=testinput(in_param,true);
   case j of
      0: begin
         MessageDlg(envtoutf8(paramstr(i))+' '+txt_type_unsupported+'. '+txt_type_unsupported_select+READE_LIST, mtWarning, [mbOK], 0);
         goto 1;
         end;
      1: begin
         MessageDlg(STR_PEA+': '+txt_listtest, mtWarning, [mbOK], 0);
         goto 1;
         end;
      2: begin
         MessageDlg(txt_split+': '+txt_listtest, mtWarning, [mbOK], 0);
         goto 1;
         end;
      5: begin
         MessageDlg(STR_ZPAQ+': '+txt_listtest, mtWarning, [mbOK], 0);
         goto 1;
         end;
      6: begin
         MessageDlg(STR_QUAD+': '+txt_listtest, mtWarning, [mbOK], 0);
         goto 1;
         end;
      7: begin
         if not(ufileexists((executable_path+'res'+directoryseparator+'unace'+directoryseparator+'unace'+EXEEXT))) then
            begin
            MessageDlg(txt_ace_missing, mtWarning, [mbOK], 0);
            Form_peach.Close;//don't skip on form close event
            exit;
            end;
         fun:='UNACE';
         fun_status:=fun;
         if ltfun='l' then subfun:='list'
         else subfun:='test';
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         out_param:=in_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'unace'+DirectorySeparator+'unace'+EXEEXT+delimiter;
         //archive function
         archive_function:=ltfun;
         cl:=bin_name+' '+archive_function;
         cl:=cl+' '+in_param;
         launch_cl(cl,jobcode,out_param);
         end;
      8: begin
         MessageDlg('LPAQ: '+txt_listtest, mtWarning, [mbOK], 0);
         goto 1;
         end;
      9: begin
         fun:='UNARC';
         fun_status:=fun;
         pw:='-pdefault';
         if zaout>0 then
            begin
            case extpw of
               1: begin
                  try
                  srcfilesize_multipart(in_param,fsize);
                  if fsize < 100*1024*1024 then
                     if pw_pre_test(in_param,'arc')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;
                     end;
                  except
                  end;
                  end;
               2: begin
                  if pw_pre_test(in_param,'arc')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;;
                     end;
                  end;
               3: begin
                  if ask_pwkeyfile_context(pw)<>0 then exit;
                  end;
               end;
            end;
         if ltfun='l' then subfun:='list'
         else subfun:='test';
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
         out_param:=in_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'arc'+DirectorySeparator+'arc'+EXEEXT+delimiter;
         //archive function
         archive_function:=ltfun;
         cl:=bin_name+' '+archive_function;
         cl:=cl+' '+in_param;
         cl:=cl+' '+pw;
         launch_cl(cl,jobcode,out_param);
         end;
      else
         begin
         fun:='UN7Z';
         fun_status:=fun;
         pw:='-pdefault';
         if zaout>0 then
            begin
            case extpw of
               1: begin
                  try
                  srcfilesize_multipart(in_param,fsize);
                  if fsize < 100*1024*1024 then
                     if pw_pre_test(in_param,'7z')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;;
                     end;
                  except
                  end;
                  end;
               2: begin
                  if pw_pre_test(in_param,'7z')<>0 then
                     begin
                     if ask_pwkeyfile_context(pw)<>0 then exit;
                     end;
                  end;
               3: begin
                  if ask_pwkeyfile_context(pw)<>0 then exit;
                  end;
               end;
            end;
         if ltfun='l' then subfun:='list'
         else subfun:='test';
         jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms',now)+fun;
            try
               srcfilesize_multipart(in_param,tsize); //expressed in B
            except
            end;
         out_param:=in_param;
         in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
         bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
         //archive function
         archive_function:=ltfun;
         cl:=bin_name+' '+archive_function;
         cl:=cl+' '+pw;
         cl:=cl+' '+in_param;
         launch_cl(cl,jobcode,out_param);
         end;
      end;
   1:
   end;
end;

procedure ext2browse; //take multiple objects as input; if they are supported archive types each one is opened in a new PeaZip instance
var
   i,j:integer;
   P: TProcess;
   cl,bin_name,in_param:utf8string;
label 1;
begin
if paramcount<2 then
   begin
   msg_no_input;
   goto 1;
   end;
for i:=2 to paramcount do
   begin
   if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
   else
      begin
      msg_not_accessible(i);
      break;
      end;
   in_param:=uexpandfilename(envtoutf8(paramstr(i)));
   j:=testinput(in_param,true);
   case j of
      0: if MessageDlg(envtoutf8(paramstr(i))+' '+txt_type_custom_selected, mtConfirmation, [mbYes,mbNo], 0)=7 then goto 1;
      end;
   P:=TProcess.Create(nil);
   in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'peazip'+EXEEXT+delimiter;
   {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
   cl:=bin_name+' -ext2open '+in_param; //ext2open handles a single input in open interface
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
   1:
   end;
end;

procedure ext2browsepath; //take multiple objects as input, each one is opened in a new PeaZip instance
var
   i:integer;
   P: TProcess;
   cl,bin_name,in_param:utf8string;
begin
if paramcount<2 then
   begin
   msg_no_input;
   end
else
   for i:=2 to paramcount do
      begin
      if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
         else
         begin
         msg_not_accessible(i);
         break;
         end;
      in_param:=uexpandfilename(envtoutf8(paramstr(i)));
      if udirectoryexists(in_param) then
      else in_param:=uextractfilepath(in_param);
      P:=TProcess.Create(nil);
      in_param:=delimiter+escapefilename(in_param,desk_env)+delimiter;
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'peazip'+EXEEXT+delimiter;
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      cl:=bin_name+' -ext2open '+in_param; //ext2open handles a single input in open interface
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
end;

procedure open_cp(s:utf8string);
var
   P:TProcess;
   cl:utf8string;
begin
P:=TProcess.Create(nil);
{$IFDEF MSWINDOWS}
if winver='9x' then cl:=wincomspec+' /k cd "'+s+'"'
else cl:=wincomspec+' /k cd /d "'+s+'"';
{$ELSE}//system needs to support konsole, gnome-tem or xterm
case desk_env of
   0: cl:='xterm';//unknown, not win or osx
   1: cl:='gnome-terminal --working-directory='''+s+'''';//Gnome
   2: cl:='konsole --workdir '''+s+'''';//KDE
   end;
{$ENDIF}
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
end;

procedure ext2commandprompt; //take multiple objects as input, for each one a command prompt is opened in its path
var
   i:integer;
   in_param:utf8string;
begin
if paramcount<2 then
   begin
   msg_no_input;
   end
else
   for i:=2 to paramcount do
      begin
      if ufilegetattr(envtoutf8(paramstr(i))) > 0 then
         else
         begin
         msg_not_accessible(i);
         break;
         end;
      in_param:=uexpandfilename(envtoutf8(paramstr(i)));
      if udirectoryexists(in_param) then
      else in_param:=uextractfilepath(in_param);
      if in_param<>'' then open_cp(in_param);
      end;
end;

procedure on_exit;
begin
if ptmpcode<>'' then cleardir(ptmpdir,true);
if (browserstyle=20) or (browserstyle=21) then //if browser style to be saved is last used, always save defaults;
   begin
   if browsingmode7z=0 then browserstyle:=20 else browserstyle:=21;
   end;
if Form_peach.Visible=true then
   if Form_peach.WindowState=wsNormal then
      begin
      ws_left:=Form_peach.Left;
      ws_top:=Form_peach.Top;
      ws_width:=Form_peach.Width;
      ws_height:=Form_peach.Height;
      end;
save_conf;
save_bookmarks;
save_custedit;
end;

procedure changelanguage;
var
   P:TProcess;
   cl:utf8string;
begin
Form_peach.OpenDialog1.InitialDir:=executable_path+'res'+directoryseparator+'lang'+directoryseparator;
if Form_peach.OpenDialog1.execute then
   if Form_peach.OpenDialog1.FileName<>'' then
      begin
      lang_file:=uextractfilename(Form_peach.OpenDialog1.FileName);
      //texts(lang_file);
      on_exit;
      P:=TProcess.Create(nil);
      cl:=delimiter+executable_path+'peazip'+EXEEXT+delimiter;
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      Form_peach.Close;
      end
   else exit
else exit;
end;

procedure TForm_peach.ButtonLangClick(Sender: TObject);
begin
changelanguage;
end;

procedure open_custedit(i:integer; s:utf8string);
var
   {$IFDEF MSWINDOWS}
   w:widestring;
   {$ELSE}
   P:TProcess;
   cl:utf8string;
   {$ENDIF}
begin
if checkfiledirname(s)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+s, mtWarning, [mbOK], 0); exit; end;
{$IFDEF MSWINDOWS}
w:='"'+utf8decode(s)+'"';
case i of
   1: if validatecl(custedit1)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit1, mtWarning, [mbOK], 0); exit; end;
   2: if validatecl(custedit2)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit2, mtWarning, [mbOK], 0); exit; end;
   3: if validatecl(custedit3)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit3, mtWarning, [mbOK], 0); exit; end;
   4: if validatecl(custedit4)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit4, mtWarning, [mbOK], 0); exit; end;
   5: if validatecl(custedit5)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit5, mtWarning, [mbOK], 0); exit; end;
   6: if validatecl(custedit6)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit6, mtWarning, [mbOK], 0); exit; end;
   7: if validatecl(custedit7)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit7, mtWarning, [mbOK], 0); exit; end;
   8: if validatecl(custedit8)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+custedit8, mtWarning, [mbOK], 0); exit; end;
   end;
case i of
   1: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit1)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   2: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit2)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   3: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit3)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   4: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit4)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   5: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit5)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   6: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit6)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   7: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit7)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   8: ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar('"'+utf8decode(custedit8)+'"'), PWideChar (w), PWideChar (''), SW_SHOWNORMAL);
   end;
{$ELSE}
try
P:=TProcess.Create(nil);
case i of
   1: cl:=delimiter+custedit1+delimiter+' '+delimiter+s+delimiter;
   2: cl:=delimiter+custedit2+delimiter+' '+delimiter+s+delimiter;
   3: cl:=delimiter+custedit3+delimiter+' '+delimiter+s+delimiter;
   4: cl:=delimiter+custedit4+delimiter+' '+delimiter+s+delimiter;
   5: cl:=delimiter+custedit5+delimiter+' '+delimiter+s+delimiter;
   6: cl:=delimiter+custedit6+delimiter+' '+delimiter+s+delimiter;
   7: cl:=delimiter+custedit7+delimiter+' '+delimiter+s+delimiter;
   8: cl:=delimiter+custedit8+delimiter+' '+delimiter+s+delimiter;
   end;
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
except
end;
{$ENDIF}
end;

procedure open_advcustedit(i:integer; s:utf8string);
var
   P:TProcess;
   cl:utf8string;
begin
if checkfiledirname(s)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+s, mtWarning, [mbOK], 0); exit; end;
try
P:=TProcess.Create(nil);
case i of
   1: cl:=advedit1before+delimiter+s+delimiter+advedit1after;
   2: cl:=advedit2before+delimiter+s+delimiter+advedit2after;
   3: cl:=advedit3before+delimiter+s+delimiter+advedit3after;
   4: cl:=advedit4before+delimiter+s+delimiter+advedit4after;
   5: cl:=advedit5before+delimiter+s+delimiter+advedit5after;
   6: cl:=advedit6before+delimiter+s+delimiter+advedit6after;
   7: cl:=advedit7before+delimiter+s+delimiter+advedit7after;
   8: cl:=advedit8before+delimiter+s+delimiter+advedit8after;
   end;
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
except
   MessageDlg(txt_2_5_cannotrun+' '+envtoutf8(cl), mtWarning, [mbOK], 0);
end;
end;

procedure TForm_peach.aowcustom1Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(1, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(1, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom10Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(2, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(2, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom11Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(3, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(3, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom12Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(4, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(4, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom13Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(5, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(5, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom14Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(6, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(6, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom15Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(7, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(7, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom16Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(8, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(8, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom2Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(2, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(2, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom3Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(3, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(3, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom4Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(4, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(4, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom5Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(5, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(5, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom6Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(6, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(6, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom7Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(7, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(7, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom8Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_custedit(8, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_custedit(8, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure TForm_peach.aowcustom9Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount>1 then open_advcustedit(1, StringGrid1.Cells[8,StringGrid1.Row])
   else
else
   if StringGrid2.Rowcount>1 then open_advcustedit(1, StringGrid2.Cells[8,StringGrid2.Row]);
end;

procedure tobrowser_fromarchiver;
begin
clearlist;
Form_peach.EditName3.Text:='';
Form_peach.Edit4.Text:='';
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
Form_peach.OpenDialog3.Filename:=Form_peach.EditOpenIn.Text;
showpanel('open');
open_inputselect;
end;

procedure tobrowser_fromextractor;
begin
clearlist_ext;
Form_peach.OpenDialog3.Filename:=Form_peach.EditOpenIn.Text;
showpanel('open');
open_inputselect;
end;

procedure gocancelarchive;
begin
if stayopen=false then Form_peach.Close
else tobrowser_fromarchiver;
end;

procedure TForm_peach.ButtonArchive1Click(Sender: TObject);
begin
gocancelarchive;
end;

procedure TForm_peach.ButtonDeskClick(Sender: TObject);
begin
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then local_desktop:=Form_peach.SelectDirectoryDialog1.FileName;
if local_desktop<>'' then if local_desktop[length(local_desktop)]<>directoryseparator then local_desktop:=local_desktop+directoryseparator;
EditDesk.Text:=local_desktop;
end;

procedure TForm_peach.ButtonEditName12Click(Sender: TObject);
begin
popupmenupath1.popup();
end;

procedure TForm_peach.ButtonEditName13Click(Sender: TObject);
begin
popupmenutype.popup();
end;

procedure TForm_peach.CheckBoxARC2Click(Sender: TObject);
begin
if CheckBoxArc2.state=cbChecked then ComboboxArcSfx.Visible:=true
else ComboboxArcSfx.Visible:=false;
end;

procedure TForm_peach.CheckBoxAutoOpenClick(Sender: TObject);
begin
if Form_peach.CheckBoxAutoOpen.State=cbChecked then autoopen:=1 else autoopen:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxEncodingClick(Sender: TObject);
begin
if Form_peach.CheckBoxEncoding.State=cbChecked then archivenameenc:=1 else archivenameenc:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxIHClick(Sender: TObject);
begin
Panel3.Visible:=not(Panel3.Visible);
if CheckBoxIH.State=cbChecked then autosizeitemheight:=1 else autosizeitemheight:=0;
apply_theme;
end;

procedure TForm_peach.CheckBoxJobEncodingClick(Sender: TObject);
begin
if Form_peach.CheckBoxJobEncoding.State=cbChecked then jobdefenc:=1 else jobdefenc:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxSeparateClick(Sender: TObject);
var
   s:utf8string;
begin
if CheckBoxSeparate.State=cbChecked then
   begin
   Edit4.Enabled:=false;
   end
else
   begin
   Edit4.Enabled:=true;
   suggest_outname(s,Form_peach.cbType.Text,Form_peach.StringGrid1.Row);
   Edit4.Text:=uextractfilename(s);
   Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
   end;
end;

procedure TForm_peach.CheckBoxTBWClick(Sender: TObject);
var
   P:TProcess;
   cl:utf8string;
begin //oddly buttons are not correctly rendered after autosize is set to false (height is not autosized, while width is) until application restarts
if CheckBoxTBW.State=cbChecked then autosizetoolbarbutton:=1 else autosizetoolbarbutton:=0;
P:=TProcess.Create(nil);
cl:=delimiter+executable_path+'peazip'+EXEEXT+delimiter;
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
Close;
end;

procedure TForm_peach.ComboBoxARC2Change(Sender: TObject);
begin
if (Form_peach.CheckBoxArc2.State=cbChecked) then
   Form_peach.Caption:=txt_create+' '+txt_sfx+' ('+STR_ARC+'), '+Form_peach.ComboboxARC2.Text
else
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxARC2.Text;
end;

procedure TForm_peach.ComboBoxArchive7Change(Sender: TObject);
begin
if ((Form_peach.CheckBoxArchive6.Visible=true) and (Form_peach.CheckBoxArchive6.State=cbChecked)) then
   Form_peach.Caption:=txt_create+' '+txt_sfx+' ('+STR_7Z+'), '+Form_peach.ComboboxArchive7.Text
else
   Form_peach.Caption:=txt_create+' .'+archive_type+', '+Form_peach.ComboboxArchive7.Text;
end;

procedure TForm_peach.ctrlrefreshClick(Sender: TObject);
begin
navrefresh;
setfocus_stringgridlist;
end;

procedure TForm_peach.EditOpenIn1KeyPress(Sender: TObject; var Key: char);
begin
if Key=char(13) then
   begin
   EditOpenIn.Text:=EditOpenIn1.Text;
   {$IFDEF MSWINDOWS}
   if EditOpenIn.Text=txt_mypc then
      begin
      fun:='FILEBROWSER';
      fun_status:=fun;
      listpc;
      addtohistory;
      exit;
      end;
   {$ENDIF}
   if udirectoryexists((EditOpenIn.Text)) then
      begin
      fun:='FILEBROWSER';
      fun_status:=fun;
      listdir(EditOpenIn.Text,false);
      addtohistory;
      end
   else open_archive_fromname(EditOpenIn.Text);
   end;
end;

procedure TForm_peach.EditOpenInChange(Sender: TObject);
begin
EditOpenIn1.Text:=EditOpenIn.Text;
end;

procedure nrsearch;
begin
if fun='UN7Z' then
   begin
   Form_peach.EditUn7zaFilterExclude.Text:=uextractfilepath(Form_peach.EditUn7zaFilter.Text)+'*'+directoryseparator+'*';
   list_un7z('browse');
   status0:=txt_list_browsing;
   browsingmode7z:=0;
   addtohistory;
   end;
if fun='FILEBROWSER' then
   begin
   listdir(Form_peach.EditOpenIn.Text,false);
   addtohistory;
   end;
end;

procedure TForm_peach.EditUn7zaFilter1KeyPress(Sender: TObject; var Key: char);
begin
if Key=char(13) then
   begin
   EditUn7zaFilter.Text:=EditUn7zaFilter1.Text;
   nrsearch;
   end;
end;

procedure TForm_peach.EditUn7zaFilterChange(Sender: TObject);
begin
EditUn7zaFilter1.Text:=EditUn7zaFilter.Text;
end;

procedure eowcustom(j:integer; custmode:utf8string);
var
   i:integer;
   cl,jobcode,outname,s:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=Form_peach.StringGridList.Row;
if Form_peach.StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'extandrun')=0 then
            begin
            s:=outname;
            if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
            s:=s+Form_peach.StringGridList.Cells[1,i];
            if custmode='cust' then open_custedit(j, s)
            else open_advcustedit(j, s);
            end
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'preview')=0 then
            begin
            s:=outname;
            if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
            s:=s+Form_peach.StringGridList.Cells[1,i];
            if custmode='cust' then open_custedit(j, s)
            else open_advcustedit(j, s);
            end
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.eowcustom10Click(Sender: TObject);
begin
eowcustom(2,'adv');
end;

procedure TForm_peach.eowcustom11Click(Sender: TObject);
begin
eowcustom(3,'adv');
end;

procedure TForm_peach.eowcustom12Click(Sender: TObject);
begin
eowcustom(4,'adv');
end;

procedure TForm_peach.eowcustom13Click(Sender: TObject);
begin
eowcustom(5,'adv');
end;

procedure TForm_peach.eowcustom14Click(Sender: TObject);
begin
eowcustom(6,'adv');
end;

procedure TForm_peach.eowcustom15Click(Sender: TObject);
begin
eowcustom(7,'adv');
end;

procedure TForm_peach.eowcustom16Click(Sender: TObject);
begin
eowcustom(8,'adv');
end;

procedure TForm_peach.eowcustom1Click(Sender: TObject);
begin
eowcustom(1,'cust');
end;

procedure TForm_peach.eowcustom2Click(Sender: TObject);
begin
eowcustom(2,'cust');
end;

procedure TForm_peach.eowcustom3Click(Sender: TObject);
begin
eowcustom(3,'cust');
end;

procedure TForm_peach.eowcustom4Click(Sender: TObject);
begin
eowcustom(4,'cust');
end;

procedure TForm_peach.eowcustom5Click(Sender: TObject);
begin
eowcustom(5,'cust');
end;

procedure TForm_peach.eowcustom6Click(Sender: TObject);
begin
eowcustom(6,'cust');
end;

procedure TForm_peach.eowcustom7Click(Sender: TObject);
begin
eowcustom(7,'cust');
end;

procedure TForm_peach.eowcustom8Click(Sender: TObject);
begin
eowcustom(8,'cust');
end;

procedure TForm_peach.eowcustom9Click(Sender: TObject);
begin
eowcustom(1,'adv');
end;

procedure TForm_peach.ImageBookmarksClick(Sender: TObject);
begin
showbar('bookmarks');
end;

procedure TForm_peach.ImageClipClick(Sender: TObject);
begin
showbar('clip');
end;

procedure TForm_peach.ImageInfoArchive2Click(Sender: TObject);
begin
MessageDlg(ImageInfoArchive2.Hint,mtInformation,[mbOk], 0);
end;

procedure TForm_peach.ImageInfoDefault1Click(Sender: TObject);
begin
MessageDlg(txt_cl_hint, mtInformation, [mbOK], 0);
end;

procedure TForm_peach.ImageInfoDefault2Click(Sender: TObject);
begin
MessageDlg(txt_2_5_langhint, mtInformation, [mbOK], 0);
end;

procedure TForm_peach.ImageInfoDefault3Click(Sender: TObject);
begin
MessageDlg(txt_quickbrowse_hint, mtInformation, [mbOK], 0);
end;

procedure TForm_peach.ImageIOClick(Sender: TObject);
begin
Form6.Caption:=Form_peach.EditOpenIn1.Text;
Form6.Showmodal;
end;

procedure TForm_peach.ImageIOMouseEnter(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
if (fun='FILEBROWSER') and ((Form6.LabelPercent.Caption='??%') or (Form6.LabelUn7zInputStatus1.Caption='') or (Form6.LabelUn7zInputStatus1.Caption=usr_name)) then navrefresh; //workaround since shapes and labels are not set correctly at startup
{$ENDIF}
end;

procedure TForm_peach.ImageListHistoryClick(Sender: TObject);
begin
showbar('history');
end;

procedure TForm_peach.ImagePassword1Click(Sender: TObject);
var
   i:integer;
begin
repeat
i:=ask_pwkeyfile_core;
until i>=0;
end;

procedure TForm_peach.Button1Click(Sender: TObject);
begin
showpanel(prevpanel);
Caption:=prevcaption;
end;

procedure goarchive;
var
   sel,s:utf8string;
   i:integer;
begin

if Form_peach.StringGrid1.RowCount>1 then
   if (Form_peach.EditName3.Text='') or (Form_peach.Edit4.Text='') then setoutname(s,archive_type,0);

if Form_peach.CheckBoxSeparate.State=cbChecked then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      begin
      sel:=inttostr(i);
      suggest_outname(s,Form_peach.cbType.Text,i);
      s:=uextractfilename(s); //keep only filename, set path to last used
      Form_peach.Edit4.Text:=s;
      Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
      ares:=archive_mainsequence(sel);
      end;
   pf:=Form_peach.cbType.ItemIndex;
   lastoutpath:=uextractfilepath(Form_peach.Editname3.Text);
   sleep(random(500)); //let some time between jobs if launched sequentially
   end
else
   begin
   sel:='all';
   if Form_peach.CheckBoxTarBefore.State=cbChecked then
      begin
      tarbefore(sel);
      pf:=Form_peach.cbType.ItemIndex;
      lastoutpath:=uextractfilepath(Form_peach.Editname3.Text);
      end
   else
      begin
      ares:=archive_mainsequence(sel);
      pf:=Form_peach.cbType.ItemIndex;
      lastoutpath:=uextractfilepath(Form_peach.Editname3.Text);
      end;
   end;

if ares=0 then
   if stayopen=false then Form_peach.Close
   else tobrowser_fromarchiver;
end;

procedure TForm_peach.ButtonArchiveClick(Sender: TObject);
begin
goarchive;
end;

procedure TForm_peach.ButtonEditName3Click(Sender: TObject);
var
   s:utf8string;
begin
if getoutname(s,archive_type,0)=0 then
   begin
   EditName3.Text:=uextractfilepath(s);
   if checkboxseparate.state=cbUnchecked then Edit4.Text:=uextractfilename(s);
   Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
   end;
end;

procedure TForm_peach.ButtonEditName4Click(Sender: TObject);
begin
patheditor:=EditNameCustom1;
pcustom.popup();
end;

procedure TForm_peach.ButtonEditName5Click(Sender: TObject);
begin
if EditOpenOut.Text='' then Form_peach.SelectDirectoryDialog1.InitialDir:=local_desktop
else Form_peach.SelectDirectoryDialog1.InitialDir:=EditOpenOut.Text;
if SelectDirectoryDialog1.Execute then
   if SelectDirectoryDialog1.FileName<>'' then
      begin
      EditOpenOut.Text:=SelectDirectoryDialog1.FileName;
      if EditOpenOut.Text<>'' then
         if EditOpenOut.Text[length(EditOpenOut.Text)]<>directoryseparator then EditOpenOut.Text:=EditOpenOut.Text+directoryseparator;
      end;
end;

procedure TForm_peach.ButtonEditName8Click(Sender: TObject);
begin
patheditor:=EditNameCustom1;
pcustom.popup();
end;

procedure TForm_peach.ButtonEditNameCustom1Click(Sender: TObject);
var
   s:utf8string;
begin
if Form_peach.SaveDialog2.Execute then
   if Form_peach.SaveDialog2.FileName<>'' then
      begin
      s:=Form_peach.SaveDialog2.FileName;
      addtocustomhistory(s);
      Form_peach.EditNameCustom1.Text:=s;
      end
   else exit
else exit;
end;

procedure gocancelextract;
begin
if stayopen=false then Form_peach.Close
else tobrowser_fromextractor;
end;

procedure TForm_peach.ButtonExtCancelClick(Sender: TObject);
begin
gocancelextract;
end;

procedure TForm_peach.ButtonUn7za10Click(Sender: TObject);
begin
popupopen1.popup();
end;

procedure TForm_peach.ButtonUn7za6Click(Sender: TObject);
var
   p:tpoint;
begin
p.x:=buttonun7za6.left;
p.y:=buttonun7za6.top+buttonun7za6.height;
p:=clienttoscreen(p);
pmmorefun.popup(p.x,p.y);
end;

procedure TForm_peach.cbfSfx1Click(Sender: TObject);
begin
if Form_peach.cbfsfx1.State=cbChecked then favsfxarc:=1 else favsfxarc:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBox1Click(Sender: TObject);
begin
if CheckBox1.state=cbChecked then esna:=1
else esna:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBox2Click(Sender: TObject);
begin
if CheckBox2.state=cbChecked then euns:=1
else euns:=0;
updateconf_modified;
end;

procedure TForm_peach.ButtonEditNameCustomClick(Sender: TObject);
var
   s:utf8string;
begin
if Form_peach.SaveDialog2.Execute then
   if Form_peach.SaveDialog2.FileName<>'' then
      begin
      s:=Form_peach.SaveDialog2.FileName;
      addtocustomhistory(s);
      Form_peach.EditNameCustom.Text:=s;
      s:=uextractfilename(s);
      cutextension(s);//default: extension = name of the RadioGroupQuad
      Form_peach.EditExtCustom.Text:=s;
      end
   else exit
else exit;
end;

procedure TForm_peach.ButtonInputArchiveClick(Sender: TObject);
begin
open_archive;
end;

procedure TForm_peach.ButtonPWOpenClick(Sender: TObject);
begin
if fun='UN7Z' then reopen_7z;
if fun='UNARC' then browse_arc('flat');
end;

procedure archive_testall;
var
   cl,jobcode,outname:utf8string;
begin
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'test','all')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'test','all')=0 then launch_cl(cl,jobcode,outname);
end;

procedure archive_listall;
var
   cl,jobcode,outname:utf8string;
begin
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'list','all')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'list','all')=0 then launch_cl(cl,jobcode,outname);
end;

procedure archive_infoall;
var
   cl,jobcode,outname:utf8string;
begin
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'info','all')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'list','all')=0 then launch_cl(cl,jobcode,outname);//detailed list not featured in ARC
end;

procedure filebrowser_funall(funct:utf8string);//test, list, info
var
   P:TProcess;
   s,cl,jobcode,outname:utf8string;
   i,j,k:integer;
   scheduleclip:array of array [0..13] of utf8string;
begin
s:=Form_peach.EditOpenIn.Text;
k:=0;
setlength(scheduleclip,1);
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   if Form_peach.StringGridList.Cells[12,i]='1' then
      begin
      if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
      for j:=0 to Form_peach.StringGridList.ColCount-1 do
         scheduleclip[k,j]:=Form_peach.StringGridList.Cells[j,i];
      setlength(scheduleclip,length(scheduleclip)+1);
      k:=k+1;
      end;
for i:=0 to k-1 do
   begin
   Form_peach.EditOpenIn.Text:=scheduleclip[i,8];
   testarctype(Form_peach.EditOpenIn.Text,j,fun);
   fun_status:='FILEBROWSER';
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,false,funct,'all')=0 then launch_cl(cl,jobcode,outname)
         else
      else
         if fun='UNARC' then
            if compose_unarc_cl(cl,jobcode,outname,false,funct,'all')=0 then launch_cl(cl,jobcode,outname)
            else
         else
            begin
            if funct='test' then
            else
               begin
               P:=TProcess.Create(nil);
               cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter+' list '+funct+' '+delimiter+Form_peach.EditOpenIn.Text+delimiter;
               cl:=utf8toenv(cl);
               P.CommandLine:=cl;
               if Form_peach.Visible=true then Application.ProcessMessages;
               if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
               P.Execute;
               P.Free;
               end;
            end;
   end;
setlength(scheduleclip,0);
Form_peach.EditOpenIn.Text:=s;
fun:='FILEBROWSER';
fun_status:=fun;
end;

procedure TForm_peach.ButtonUn7za1Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if fun='FILEBROWSER' then filebrowser_funall('test')
else archive_testall;
setfocus_stringgridlist;
end;

function test_shouldnotbechanged:integer; //test if it's a known modified format similar to read/write supported ones; it can be read but write operation should be disabled
begin
test_shouldnotbechanged:=0;
case testext(Form_peach.EditOpenIn.Text) of
   503: test_shouldnotbechanged:=1;
   502: test_shouldnotbechanged:=1;
   501: test_shouldnotbechanged:=1; //non-archives
   //500: result:=BExeSupported; //Windows executables, installers
   22: test_shouldnotbechanged:=1;//Puppy Linux packages
   23: test_shouldnotbechanged:=1;//PK* packages
   25: test_shouldnotbechanged:=1;//content of RPM and SLP packages
   28: test_shouldnotbechanged:=1;//U3P app package
   10001: test_shouldnotbechanged:=1;//PEA archive, don't support update
   end;
end;

procedure addtoarchive_selected;
var
   i:integer;
begin
Form_peach.StringGridList.Cursor:=crHourglass;
Application.ProcessMessages;
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   if Form_peach.StringGridList.Cells[12,i]='1' then
      if testname(Form_peach.StringGridList.Cells[8,i], Form_peach.StringGrid1)=0 then
         begin
         Form_peach.Caption:=txt_2_4_adding+' '+Form_peach.StringGridList.Cells[8,i];
         Application.ProcessMessages;
         if ufilegetattr(Form_peach.StringGridList.Cells[8,i]) > 0 then
            if ufilegetattr(Form_peach.StringGridList.Cells[8,i]) and faDirectory =0 then begin addfilestr(Form_peach.StringGrid1,Form_peach.StringGridList.Cells[8,i]); end
            else addfolderstr(Form_peach.StringGrid1,Form_peach.StringGridList.Cells[8,i])
         else msg_not_accessible_list(Form_peach.StringGridList.Cells[8,i]);
         end;
Form_peach.StringGrid1.AutoSizeColumns;
showpanel('archive');
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
Form_peach.StringGridList.Cursor:=crDefault;
Form_peach.ComboBoxArchive7.ItemIndex:=0;
Form_peach.comboboxarc2.ItemIndex:=0;
end;

procedure addtoarchive_inarchive;
var
   i:integer;
   s,fext:utf8string;
begin
Form_peach.CheckBoxSeparate.State:=cbUnchecked;
Form_peach.StringGrid1.AutoSizeColumns;
s:=Form_peach.EditOpenIn.Text;
Form_peach.EditName3.Text:=uextractfilepath(s);
fext:=upcase(uextractfileext(s));
//set type, set action to update
if (upcase(fext)='.7Z') or (upcase(fext)='.TAR') or (upcase(fext)='.ZIP') then
   begin
   if upcase(fext)='.7Z' then archive_type_select(STR_7Z);
   if upcase(fext)='.TAR' then archive_type_select(STR_TAR);
   if upcase(fext)='.ZIP' then archive_type_select(STR_ZIP);
   Form_peach.ComboBoxArchive7.ItemIndex:=2;
   end
else
   if upcase(fext)='.ARC' then
      begin
      archive_type_select(STR_ARC);
      Form_peach.comboboxarc2.ItemIndex:=2;
      end
   else
      begin
      MessageDlg(txt_add_error, mtWarning, [mbOK], 0);
      tobrowser_fromarchiver;
      exit;
      end;
cutextension(s);
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
showpanel('archive');
Form_peach.ButtonEditname12.Enabled:=false;
Form_peach.Edit5.enabled:=false;
Form_peach.ButtonEditname3.Enabled:=false;
Form_peach.GroupBoxCreateOut.Enabled:=false;
Form_peach.ButtonEditname13.Enabled:=false;
Form_peach.pmbrowser.Enabled:=false;
Form_peach.mgobrowser.Enabled:=false;
Form_peach.labelstatus1.Caption:=status1;
Form_peach.labelstatus4.Caption:=txt_2_7_updating;
updatingarchive:=true;
ares:=-1;
end;

function checkfdwin:integer;//under Windows systems prevents adding a floppy volume to the archive (open and add files instead), because if the floppy is not found there are rised exceptions difficult and slow to handle
begin
checkfdwin:=-1;
{$IFDEF MSWINDOWS}
if (Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]='A:\') or (Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]='B:\') then exit;
{$ENDIF}
checkfdwin:=0;
end;

procedure TForm_peach.ButtonUn7za4Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
if fun='FILEBROWSER' then
   if checkfdwin=0 then addtoarchive_selected;
if fun='UN7Z' then addtoarchive_inarchive;
if fun='UNARC' then addtoarchive_inarchive;
end;

function extract_main_bytype(s:utf8string; var cl,jobcode,outname:utf8string; realext:boolean): integer;
begin
if fun='FILEBROWSER' then testarctype(Form_peach.EditOpenIn.Text,extract_main_bytype,fun);
extract_main_bytype:=-1;
if (fun='UNCUSTOM') then
   if compose_uncustom_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNPAQ') then
   if compose_unpaq_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNZPAQ') then
   if compose_unzpaq_cl(cl,jobcode,outname,realext,s,'all')<>0 then exit;
if (fun='UNLPAQ') then
   if compose_unlpaq_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNQUAD') then
   if compose_unquad_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNACE') then
   if compose_unace_cl(cl,jobcode,outname,realext,s,'all')<>0 then exit;
if (fun='UNARC') then
   if compose_unarc_cl(cl,jobcode,outname,realext,s,'all')<>0 then exit;
if (fun='UNPEA') or (fun='RFJ') then
   if compose_unpea_cl(cl,jobcode,outname)<>0 then exit;
if (fun='UN7Z') then
   if compose_un7z_cl(cl,jobcode,outname,realext,s,'all')<>0 then exit;
extract_main_bytype:=0;
end;

function extract_finalize_bytype(fn,s:utf8string; var cl,jobcode,outname,mode:utf8string; realext:boolean): integer;
begin
if fun='FILEBROWSER' then testarctype(fn,extract_finalize_bytype,fun);
extract_finalize_bytype:=-1;
if (fun='UNCUSTOM') then
   if compose_uncustom_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNPAQ') then
   if compose_unpaq_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNZPAQ') then
   if compose_unzpaq_cl(cl,jobcode,outname,realext,s,mode)<>0 then exit;
if (fun='UNLPAQ') then
   if compose_unlpaq_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNQUAD') then
   if compose_unquad_cl(cl,jobcode,outname,realext)<>0 then exit;
if (fun='UNACE') then
   if compose_unace_cl(cl,jobcode,outname,realext,s,mode)<>0 then exit;
if (fun='UNARC') then
   if compose_unarc_cl(cl,jobcode,outname,realext,s,mode)<>0 then exit;
if (fun='UNPEA') or (fun='RFJ') then
   if compose_unpea_cl(cl,jobcode,outname)<>0 then exit;
if (fun='UN7Z') then
   if compose_un7z_cl(cl,jobcode,outname,realext,s,mode)<>0 then exit;
extract_finalize_bytype:=0;
end;

function build_path(s:utf8string):integer; //try to build the output path if it does not exists
begin
build_path:=-1;
try
uForceDirectories(s);
except
if control_outpath(s)<>0 then exit;
end;
build_path:=0;
end;

procedure goextract;
var
   i,k:integer;
   cl,jobcode,outname:utf8string;
begin
if fun='' then fun:='FILEBROWSER';
if Form_peach.EditOpenOut.Text='' then
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then
      Form_peach.EditOpenOut.Text:=uextractfilepath(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row])
   else
      Form_peach.EditOpenOut.Text:=uextractfilepath(Form_peach.EditOpenIn.Text);
if not(udirectoryexists(Form_peach.EditOpenOut.Text)) then
   if build_path(Form_peach.EditOpenOut.Text)<>0 then exit;
if fun='FILEBROWSER' then
   begin
   k:=Form_peach.StringGrid2.RowCount;
   Form_peach.StringGrid2.Row:=1;
   Form_peach.StringGrid2.Cells[11,Form_peach.StringGrid2.Row]:='1';
   for i:=1 to k-1 do
      begin
      Form_peach.StringGrid2.Cells[11,Form_peach.StringGrid2.Row]:='0';
      end;
   Form_peach.StringGrid2.Repaint;
   for i:=1 to k-1 do
      begin
      try
      srcfilesize_multipart(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],tsize); //expressed in B
      except
      end;
      if extract_finalize_bytype(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row],'ext',cl,jobcode,outname,extsel,true)=0 then launch_cl(cl,jobcode,outname);
      fun:='FILEBROWSER';
      Form_peach.StringGrid2.Cells[11,Form_peach.StringGrid2.Row]:='0';
      if Form_peach.StringGrid2.Row<k then Form_peach.StringGrid2.Row:=Form_peach.StringGrid2.Row+1;
      Form_peach.StringGrid2.Cells[11,Form_peach.StringGrid2.Row]:='1';
      Form_peach.StringGrid2.Repaint;
      sleep(random(500)); //let some time between operations
      end;
   end
else
   begin
   //test if the file seem encrypted
   if (fun='UNARC') or (fun='UN7Z') then
      if testencrypted=0 then
         begin
         if extract_finalize_bytype(Form_peach.EditOpenIn.Text,'ext',cl,jobcode,outname,extsel,true)=0 then launch_cl(cl,jobcode,outname)
         end
      else
         begin
         if stayopen=false then Form_peach.Close
         else tobrowser_fromextractor;
         end
   else
      if extract_finalize_bytype(Form_peach.EditOpenIn.Text,'ext',cl,jobcode,outname,extsel,true)=0 then launch_cl(cl,jobcode,outname);
   end;
lastoutpath:=Form_peach.EditOpenOut.Text;
if stayopen=false then Form_peach.Close
else tobrowser_fromextractor;
end;

procedure TForm_peach.ButtonExtOkClick(Sender: TObject);
begin
goextract;
end;

procedure extractall;
var
   cl,jobcode,outname:utf8string;
begin
if (fun='UN7Z') or
   (fun='UNARC') then
      if testencrypted<0 then exit;
if extract_main_bytype('neutral',cl,jobcode,outname,true)=0 then launch_cl(cl,jobcode,outname);
end;

procedure browsehistory;
var
   i:integer;
   s1,s2:utf8string;
begin
browsinghistory:=true;
with Form_peach do
   begin
   {$IFDEF MSWINDOWS}
   if StringGridHistory.Cells[2,StringGridHistory.Row]=txt_mypc then
      begin
      listpc;
      //PanelHistory.Visible:=false;
      browsinghistory:=false;
      exit;
      end;
   {$ENDIF}
   if not(ufileexists((StringGridHistory.Cells[2,StringGridHistory.Row]))) and
      not(udirectoryexists((StringGridHistory.Cells[2,StringGridHistory.Row]))) then
         begin
         MessageDlg(StringGridHistory.Cells[2,StringGridHistory.Row]+' '+txt_not_accessible, mtWarning, [mbOK], 0);
         browsinghistory:=false;
         exit;
         end;
   testarctype(StringGridHistory.Cells[2,StringGridHistory.Row],i,fun);
   setbuttonmenus;
   if fun='UN7Z' then
      begin
      s1:=StringGridHistory.Cells[3,StringGridHistory.Row];
      s2:=StringGridHistory.Cells[4,StringGridHistory.Row];
      if StringGridHistory.Cells[2,StringGridHistory.Row]<>EditOpenIn.Text then open_archive_fromname(StringGridHistory.Cells[2,StringGridHistory.Row]);
      EditUn7zaFilter.Text:=s1;
      EditUn7zaFilterExclude.Text:=s2;
      list_un7z('browse');
      status0:=StringGridHistory.Cells[1,StringGridHistory.Row];
      //don't add to history
      end;
   if fun='FILEBROWSER' then
      begin
      EditUn7zaFilter.Text:=StringGridHistory.Cells[3,StringGridHistory.Row];
      if StringGridHistory.Cells[1,StringGridHistory.Row]=txt_list_browsing then listdir(StringGridHistory.Cells[2,StringGridHistory.Row],false)
      else listdir(StringGridHistory.Cells[2,StringGridHistory.Row],true);
      status0:=StringGridHistory.Cells[1,StringGridHistory.Row];
      //don't add to history
      end;
   if (fun<>'UN7Z') and (fun<>'FILEBROWSER') then
      begin
      if StringGridHistory.Cells[2,StringGridHistory.Row]<>EditOpenIn.Text then open_archive_fromname(StringGridHistory.Cells[2,StringGridHistory.Row]);
      end;
   browsinghistory:=false;
   end;
end;

procedure navrefresh;
begin
{$IFDEF MSWINDOWS}if form_peach.PanelOpen.Visible=true then{$ELSE}if form_peach.PanelOpen.Top=0 then{$ENDIF}
begin
if fun='FILEBROWSER' then
   if Form_peach.StringGridHistory.RowCount>1 then browsehistory
   else
else
   if (fun='UN7Z') then
      begin
      list_un7z('browse');
      status0:=txt_list_browsing;
      browsingmode7z:=0;
      end
   else
      begin
      open_archive_fromname(Form_peach.EditOpenIn.Text);
      end;
end;
end;

function set_extractto_path:integer;
begin
set_extractto_path:=-1;
Form_peach.SelectDirectoryDialog1.InitialDir:=local_desktop;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      begin
      Form_peach.EditOpenOut.Text:=Form_peach.SelectDirectoryDialog1.FileName;
      if Form_peach.EditOpenOut.Text<>'' then
         if Form_peach.EditOpenOut.Text[length(Form_peach.EditOpenOut.Text)] <> directoryseparator then Form_peach.EditOpenOut.Text:=Form_peach.EditOpenOut.Text+directoryseparator;
      end
   else exit
else exit;
set_extractto_path:=0;
end;

function set_extractto_pathto(s:utf8string):integer;
begin
set_extractto_pathto:=-1;
Form_peach.EditOpenOut.Text:=s;
if Form_peach.EditOpenOut.Text<>'' then
   if Form_peach.EditOpenOut.Text[length(Form_peach.EditOpenOut.Text)] <> directoryseparator then Form_peach.EditOpenOut.Text:=Form_peach.EditOpenOut.Text+directoryseparator;
//menuitemopen_ext_caption; //will change session output path to user-selected path (all functions will point to the new path: extract, ext&run, preview)
set_extractto_pathto:=0;
end;

procedure list_toextractor(act:utf8string);
var
   i,rc,c,k:integer;
   okfile:boolean;
begin
if act='multi' then
   begin
   extsel:='all';
   Form_peach.StringGrid2.popupmenu:=Form_peach.popupmenu1;
   Form_peach.StringGrid2.Hint:=txt_2_7_exthint;
   Form_peach.labelstatus2.Caption:=txt_2_7_ext;
   Form_peach.LabelStatus3.Hint:='';
   rc:=Form_peach.StringGridList.RowCount;
   k:=Form_peach.StringGrid2.RowCount;
   for i:=1 to rc-1 do
      if Form_peach.StringGridList.Cells[12,i]='1' then
         if Form_peach.StringGridList.Cells[2,i]<>txt_list_isfolder then  //no folders
            begin
            test_extfile(Form_peach.StringGridList.Cells[8,i],okfile);
            if okfile=true then
               begin
               if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
               Form_peach.StringGrid2.RowCount:=k+1;
               for c:=1 to 13 do Form_peach.StringGrid2.Cells[c,k]:=Form_peach.StringGridList.Cells[c,i];
               k:=k+1;
               end;
            end;
   Form_peach.StringGrid2.RowCount:=k;
   updatecontent_ext;
   if k=1 then exit;
   end;
if act='none' then
   begin
   extsel:='all';
   Form_peach.StringGrid2.popupmenu:=Form_peach.popupmenu1;
   Form_peach.StringGrid2.Hint:=txt_2_7_exthint;
   Form_peach.labelstatus2.Caption:=txt_2_7_ext;
   Form_peach.LabelStatus3.Hint:='';
   updatecontent_ext;
   end;
if act='all' then
   begin
   extsel:=act;
   Form_peach.StringGrid2.popupmenu:=nil;
   Form_peach.StringGrid2.Hint:='';
   Form_peach.labelstatus2.Caption:=txt_2_7_extfrom+' '+Form_peach.EditOpenIn.Text;
   Form_peach.LabelStatus3.Hint:='';
   Form_peach.labelstatus3.Caption:='('+txt_all+')';
   Form_peach.StringGrid2.RowCount:=2;
   Form_peach.StringGrid2.Cells[1,1]:='('+txt_all+')';
   autosize_stringgrid2;
   end;
if act='displayed' then
   begin
   extsel:=act;
   Form_peach.StringGrid2.popupmenu:=nil;
   Form_peach.StringGrid2.Hint:='';
   Form_peach.labelstatus2.Caption:=txt_2_7_extfrom+' '+Form_peach.EditOpenIn.Text;
   Form_peach.LabelStatus3.Hint:=txt_2_7_dirsize;
   rc:=Form_peach.StringGridList.RowCount;
   Form_peach.StringGrid2.RowCount:=rc;
   k:=1;
   for i:=1 to rc-1 do
      begin
      if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
      Form_peach.StringGrid2.RowCount:=k+1;
      for c:=1 to 13 do Form_peach.StringGrid2.Cells[c,k]:=Form_peach.StringGridList.Cells[c,i];
      k:=k+1;
      end;
   Form_peach.StringGrid2.RowCount:=k;
   updatecontent_ext;
   end;
if act='selected' then
   begin
   extsel:=act;
   Form_peach.StringGrid2.popupmenu:=nil;
   Form_peach.StringGrid2.Hint:='';
   Form_peach.labelstatus2.Caption:=txt_2_7_extfrom+' '+Form_peach.EditOpenIn.Text;
   Form_peach.LabelStatus3.Hint:=txt_2_7_dirsize;
   rc:=Form_peach.StringGridList.RowCount;
   Form_peach.StringGrid2.RowCount:=rc;
   k:=1;
   for i:=1 to rc-1 do
      if Form_peach.StringGridList.Cells[12,i]='1' then
         begin
         if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
         Form_peach.StringGrid2.RowCount:=k+1;
         for c:=1 to 13 do Form_peach.StringGrid2.Cells[c,k]:=Form_peach.StringGridList.Cells[c,i];
         k:=k+1;
         end;
   Form_peach.StringGrid2.RowCount:=k;
   updatecontent_ext;
   end;
if Form_peach.StringGrid2.RowCount>1 then
   begin
   Form_peach.StringGrid2.Cells[11,1]:='1';
   Form_peach.StringGrid2.Row:=1;
   end;
showpanel('extract');
end;

procedure TForm_peach.ButtonUn7zaToClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbUnChecked;
if fun='FILEBROWSER' then list_toextractor('multi')
else list_toextractor('all');
end;

procedure TForm_peach.cbadlerClick(Sender: TObject);
begin
if Form_peach.cbadler.State=cbChecked then algoadler:=1 else algoadler:=0;
updateconf_modified;
end;

procedure TForm_peach.cbcheckallClick(Sender: TObject);
begin
groupboxcheck.visible:=not(groupboxcheck.visible);
if Form_peach.cbcheckall.State=cbChecked then algoall:=1 else algoall:=0;
updateconf_modified;
end;

procedure TForm_peach.cbcrc16Click(Sender: TObject);
begin
if Form_peach.cbcrc16.State=cbChecked then algocrc16:=1 else algocrc16:=0;
updateconf_modified;
end;

procedure TForm_peach.cbcrc24Click(Sender: TObject);
begin
if Form_peach.cbcrc24.State=cbChecked then algocrc24:=1 else algocrc24:=0;
updateconf_modified;
end;

procedure TForm_peach.cbcrc32Click(Sender: TObject);
begin
if Form_peach.cbcrc32.State=cbChecked then algocrc32:=1 else algocrc32:=0;
updateconf_modified;
end;

procedure TForm_peach.cbcrc64Click(Sender: TObject);
begin
if Form_peach.cbcrc64.State=cbChecked then algocrc64:=1 else algocrc64:=0;
updateconf_modified;
end;

procedure TForm_peach.cbed2kClick(Sender: TObject);
begin
if Form_peach.cbed2k.State=cbChecked then algoed2k:=1 else algoed2k:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf11Click(Sender: TObject);
begin
if cbf11.State=cbChecked then favzip:=1 else favzip:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf1Click(Sender: TObject);
begin
if cbf1.State=cbChecked then fav7z:=1 else fav7z:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf2Click(Sender: TObject);
begin
if cbf2.State=cbChecked then favbz:=1 else favbz:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf3Click(Sender: TObject);
begin
if cbf3.State=cbChecked then favgz:=1 else favgz:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf5Click(Sender: TObject);
begin
if cbf5.State=cbChecked then favzpaq:=1 else favzpaq:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf6Click(Sender: TObject);
begin
if cbf6.State=cbChecked then favpea:=1 else favpea:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf7Click(Sender: TObject);
begin
if cbf7.State=cbChecked then favquad:=1 else favquad:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf8Click(Sender: TObject);
begin
if cbf8.State=cbChecked then favsplit:=1 else favsplit:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf9Click(Sender: TObject);
begin
if cbf9.State=cbChecked then favtar:=1 else favtar:=0;
updateconf_modified;
end;

procedure TForm_peach.cbf10Click(Sender: TObject);
begin
if cbf10.State=cbChecked then favupx:=1 else favupx:=0;
updateconf_modified;
end;

procedure TForm_peach.cbfARCClick(Sender: TObject);
begin
if cbfARC.State=cbChecked then favARC:=1 else favARC:=0;
updateconf_modified;
end;

procedure TForm_peach.cbfCustomClick(Sender: TObject);
begin
if cbfCustom.State=cbChecked then favCustom:=1 else favCustom:=0;
updateconf_modified;
end;

procedure TForm_peach.cbfSfxClick(Sender: TObject);
begin
if Form_peach.cbfsfx.State=cbChecked then favsfx:=1 else favsfx:=0;
updateconf_modified;
end;

procedure TForm_peach.cbmd4Click(Sender: TObject);
begin
if Form_peach.cbmd4.State=cbChecked then algomd4:=1 else algomd4:=0;
updateconf_modified;
end;

procedure TForm_peach.cbmd5Click(Sender: TObject);
begin
if Form_peach.cbmd5.State=cbChecked then algomd5:=1 else algomd5:=0;
updateconf_modified;
end;

procedure TForm_peach.cbrmdClick(Sender: TObject);
begin
if Form_peach.cbrmd.State=cbChecked then algoripemd160:=1 else algoripemd160:=0;
updateconf_modified;
end;

procedure TForm_peach.cbsha1Click(Sender: TObject);
begin
if Form_peach.cbsha1.State=cbChecked then algosha1:=1 else algosha1:=0;
updateconf_modified;
end;

procedure TForm_peach.cbsha224Click(Sender: TObject);
begin
if Form_peach.cbsha224.State=cbChecked then algosha224:=1 else algosha224:=0;
updateconf_modified;
end;

procedure TForm_peach.cbsha256Click(Sender: TObject);
begin
if Form_peach.cbsha256.State=cbChecked then algosha256:=1 else algosha256:=0;
updateconf_modified;
end;

procedure TForm_peach.cbsha384Click(Sender: TObject);
begin
if Form_peach.cbsha384.State=cbChecked then algosha384:=1 else algosha384:=0;
updateconf_modified;
end;

procedure TForm_peach.cbsha512Click(Sender: TObject);
begin
if Form_peach.cbsha512.State=cbChecked then algosha512:=1 else algosha512:=0;
updateconf_modified;
end;

procedure TForm_peach.cbTypeChange(Sender: TObject);
begin
archive_type_select(cbType.Text);
end;

procedure TForm_peach.ComboBox7zalgoChange(Sender: TObject);
begin
//no action taken; AES256 is provided as standard algorithm from the frontend and ZypCrypto is provided only as legacy
end;

procedure TForm_peach.ComboBoxArchiveCustom1Change(Sender: TObject);
begin
customsyntax1:=ComboBoxArchiveCustom1.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.ComboBoxArchiveCustomChange(Sender: TObject);
begin
customsyntax:=ComboBoxArchiveCustom.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.ComboBoxOverwriteARCChange(Sender: TObject);
begin
extoptarc:=ComboBoxOverwriteARC.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.ComboBoxOverwriteChange(Sender: TObject);
begin
extopt7z:=ComboBoxOverwrite.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.ComboBoxPassesChange(Sender: TObject);
begin
erasepasses:=ComboBoxPasses.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.ImageDefaults3Click(Sender: TObject);
var
   P:TProcess;
   cl:utf8string;
begin
if MessageDlg(txt_2_4_deletebookmarks, mtInformation,[mbYes, mbNo],0)=6 then
   begin
   udeletefile(confpath+'bookmarks.txt');
   StringGridBookmarks.RowCount:=1;
   P:=TProcess.Create(nil);
   cl:=delimiter+executable_path+'peazip'+EXEEXT+delimiter;
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
   Close;
   end;
end;

procedure TForm_peach.ImagePassword2Click(Sender: TObject);
var
   i:integer;
begin
repeat
i:=ask_pwkeyfile_core;
until i>=0;
end;

procedure toggle_view;
begin
if fun='UN7Z' then
   if status0=txt_list_browsing then list_un7z_expandall
   else list_root;
if fun='FILEBROWSER' then
   begin
   if Form_peach.EditOpenIn.Text=txt_mypc then exit;
   if status0=txt_list_browsing then
         if MessageDlg(txt_toggle_warning, mtWarning,[mbYes, mbNo],0)=6 then
            begin
            Form_peach.EditUn7zaFilter.Text:='*';
            listdir(Form_peach.EditOpenIn.Text,true);
            end
         else
   else
      begin
      Form_peach.EditUn7zaFilter.Text:='*';
      listdir(Form_peach.EditOpenIn.Text,false);
      end;
   addtohistory;
   end;
end;

procedure openpath;
var
   s:utf8string;
begin
fun:='FILEBROWSER';
fun_status:=fun;
if ufileexists((Form_peach.EditOpenIn.Text)) then Form_peach.SelectDirectoryDialog1.InitialDir:=uextractfilepath(Form_peach.EditOpenIn.Text)
else Form_peach.SelectDirectoryDialog1.InitialDir:=Form_peach.EditOpenIn.Text;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      Form_peach.EditOpenIn.Text:=Form_peach.SelectDirectoryDialog1.FileName
   else exit
else exit;
s:=Form_peach.EditOpenIn.Text;
if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
//Form_peach.EditOpenIn.Text:=s;
Form_peach.EditUn7zaFilter.Text:='*';
listdir(s,false);
addtohistory;
end;

procedure opendevice(s:utf8string);
var
   s1:utf8string;
begin
fun:='FILEBROWSER';
fun_status:=fun;
Form_peach.EditUn7zaFilter.Text:='*';
s1:=copy(s,length(s)-2,2)+directoryseparator;
showpanel('open');
listdir(s1,false);
addtohistory;
end;

procedure TForm_peach.jd10Click(Sender: TObject);
begin
opendevice(jd10.caption);
end;

procedure TForm_peach.jd11Click(Sender: TObject);
begin
opendevice(jd11.caption);
end;

procedure TForm_peach.jd12Click(Sender: TObject);
begin
opendevice(jd12.caption);
end;

procedure TForm_peach.jd13Click(Sender: TObject);
begin
opendevice(jd13.caption);
end;

procedure TForm_peach.jd14Click(Sender: TObject);
begin
opendevice(jd14.caption);
end;

procedure TForm_peach.jd15Click(Sender: TObject);
begin
opendevice(jd15.caption);
end;

procedure TForm_peach.jd16Click(Sender: TObject);
begin
opendevice(jd16.caption);
end;

procedure TForm_peach.jd17Click(Sender: TObject);
begin
opendevice(jd17.caption);
end;

procedure TForm_peach.jd18Click(Sender: TObject);
begin
opendevice(jd18.caption);
end;

procedure TForm_peach.jd19Click(Sender: TObject);
begin
opendevice(jd19.caption);
end;

procedure TForm_peach.jd1Click(Sender: TObject);
begin
opendevice(jd1.caption);
end;

procedure TForm_peach.jd20Click(Sender: TObject);
begin
opendevice(jd20.caption);
end;

procedure TForm_peach.jd21Click(Sender: TObject);
begin
opendevice(jd21.caption);
end;

procedure TForm_peach.jd22Click(Sender: TObject);
begin
opendevice(jd22.caption);
end;

procedure TForm_peach.jd23Click(Sender: TObject);
begin
opendevice(jd23.caption);
end;

procedure TForm_peach.jd24Click(Sender: TObject);
begin
opendevice(jd24.caption);
end;

procedure TForm_peach.jd25Click(Sender: TObject);
begin
opendevice(jd25.caption);
end;

procedure TForm_peach.jd26Click(Sender: TObject);
begin
opendevice(jd26.caption);
end;

procedure TForm_peach.jd2Click(Sender: TObject);
begin
opendevice(jd2.caption);
end;

procedure TForm_peach.jd3Click(Sender: TObject);
begin
opendevice(jd3.caption);
end;

procedure TForm_peach.jd4Click(Sender: TObject);
begin
opendevice(jd4.caption);
end;

procedure TForm_peach.jd5Click(Sender: TObject);
begin
opendevice(jd5.caption);
end;

procedure TForm_peach.jd6Click(Sender: TObject);
begin
opendevice(jd6.caption);
end;

procedure TForm_peach.jd7Click(Sender: TObject);
begin
opendevice(jd7.caption);
end;

procedure TForm_peach.jd8Click(Sender: TObject);
begin
opendevice(jd8.caption);
end;

procedure TForm_peach.jd9Click(Sender: TObject);
begin
opendevice(jd9.caption);
end;

procedure browsebookmark(i:integer);
var
   s:utf8string;
begin
s:=Form_peach.StringGridBookmarks.Cells[2,i];
if s='' then exit;
fun:='FILEBROWSER';
fun_status:=fun;
Form_peach.EditUn7zaFilter.Text:=Form_peach.StringGridBookmarks.Cells[3,i];
showpanel('open');
{$IFDEF MSWINDOWS}
if s=txt_mypc then listpc
else {$ENDIF}
   if udirectoryexists(s) then
      begin
      if Form_peach.StringGridBookmarks.Cells[1,i]=txt_list_browsing then listdir(s,false)
      else listdir(s,true);
      addtohistory;
      status0:=Form_peach.StringGridBookmarks.Cells[1,Form_peach.StringGridBookmarks.Row];
      end
   else
      begin
      open_archive_fromname(s);
      if fun='UN7Z' then
         begin
         Form_peach.EditUn7zaFilter.Text:=Form_peach.StringGridBookmarks.Cells[3,i];
         if Form_peach.StringGridBookmarks.Cells[1,i]=txt_list_browsing then
            begin
            Form_peach.EditUn7zaFilterExclude.Text:=uextractfilepath(Form_peach.EditUn7zaFilter.Text)+'*'+directoryseparator+'*';
            list_un7z('browse');
            status0:=txt_list_browsing;
            browsingmode7z:=0;
            addtohistory;
            end
         else
            begin
            list_un7z_search;
            end;
         end;
      end;
end;

procedure jumpto(jumpdest:utf8string);
var
   s:utf8string;
begin
//preliminar tests
if jumpdest='bookmark1' then
   if Form_peach.StringGridBookmarks.Rowcount<2 then exit;
if jumpdest='bookmark2' then
   if Form_peach.StringGridBookmarks.Rowcount<3 then exit;
if jumpdest='bookmark3' then
   if Form_peach.StringGridBookmarks.Rowcount<4 then exit;
if jumpdest='mostrecent' then
   if Form_peach.mRecent1.Caption='' then exit;
if jumpdest='mostrecent2' then
   if Form_peach.mRecent2.Caption='' then exit;
if jumpdest='mostrecent3' then
   if Form_peach.mRecent3.Caption='' then exit;
fun:='FILEBROWSER';
fun_status:=fun;
if jumpdest='root' then
   begin
   {$IFDEF MSWINDOWS}
   listpc;
   {$ELSE}
   Form_peach.EditUn7zaFilter.Text:='*';
   s:='/';
   Form_peach.EditOpenIn.Text:=s;
   listdir(s,false);
   {$ENDIF}
   end;
if jumpdest='home' then
   begin
   Form_peach.EditUn7zaFilter.Text:='*';
   s:=uextractfilepath(home_path);
   Form_peach.EditOpenIn.Text:=s;
   listdir(s,false);
   end;
if jumpdest='desktop' then
   begin
   fun:='FILEBROWSER';
   fun_status:=fun;
   Form_peach.EditUn7zaFilter.Text:='*';
   s:=local_desktop;
   Form_peach.EditOpenIn.Text:=s;
   listdir(s,false);
   end;
if jumpdest='usrdocuments' then
   begin
   fun:='FILEBROWSER';
   fun_status:=fun;
   Form_peach.EditUn7zaFilter.Text:='*';
   s:=usr_documents;
   Form_peach.EditOpenIn.Text:=s;
   listdir(s,false);
   end;
if jumpdest='bookmark1' then
   browsebookmark(1);
if jumpdest='bookmark2' then
   browsebookmark(2);
if jumpdest='bookmark3' then
   browsebookmark(3);
if jumpdest='mostrecent' then
   open_archive_fromname(Form_peach.mRecent1.Caption);
if jumpdest='mostrecent2' then
   open_archive_fromname(Form_peach.mRecent2.Caption);
if jumpdest='mostrecent3' then
   open_archive_fromname(Form_peach.mRecent3.Caption);
addtohistory;
{$IFDEF MSWINDOWS}if Form_peach.PanelOpen.Visible=true then{$ELSE}if Form_peach.PanelOpen.Top=0 then{$ENDIF}
   if Form_peach.StringGridList.Visible=true then setfocus_stringgridlist;
end;

procedure TForm_peach.jddesktopClick(Sender: TObject);
begin
showpanel('open');
jumpto('desktop');
end;

procedure TForm_peach.jddocumentsClick(Sender: TObject);
begin
showpanel('open');
jumpto('usrdocuments');
end;

procedure TForm_peach.jdhomeClick(Sender: TObject);
begin
showpanel('open');
jumpto('home');
end;

procedure TForm_peach.jdrootClick(Sender: TObject);
begin
showpanel('open');
jumpto('root');
end;

procedure TForm_peach.Label4Click(Sender: TObject);
begin
NotebookOpen.PageIndex:=1;
end;

procedure TForm_peach.Labelcb17Click(Sender: TObject);
begin
cp_open(EditDesk.Text,desk_env);
end;

procedure set_adveditcl(i:integer);
begin
case i of
   1: Form_peach.Labelcb15.caption:=advedit1before+delimiter+'%f'+delimiter+advedit1after;
   2: Form_peach.Labelcb15.caption:=advedit2before+delimiter+'%f'+delimiter+advedit2after;
   3: Form_peach.Labelcb15.caption:=advedit3before+delimiter+'%f'+delimiter+advedit3after;
   4: Form_peach.Labelcb15.caption:=advedit4before+delimiter+'%f'+delimiter+advedit4after;
   5: Form_peach.Labelcb15.caption:=advedit5before+delimiter+'%f'+delimiter+advedit5after;
   6: Form_peach.Labelcb15.caption:=advedit6before+delimiter+'%f'+delimiter+advedit6after;
   7: Form_peach.Labelcb15.caption:=advedit7before+delimiter+'%f'+delimiter+advedit7after;
   8: Form_peach.Labelcb15.caption:=advedit8before+delimiter+'%f'+delimiter+advedit8after;
   end;
end;

procedure TForm_peach.Labelcb7Click(Sender: TObject);
begin
reset_custedit;
set_adveditcl(StringGridCustEdit1.Row);
end;

procedure TForm_peach.LabelConfClick(Sender: TObject);
begin
cp_open(LabelConf.Caption,desk_env);
end;

procedure TForm_peach.LabelDefaultColor7Click(Sender: TObject);
begin
itemheight:=itemheightd;
apply_theme;
end;

procedure TForm_peach.LabelLang2Click(Sender: TObject);
begin
cp_open(executable_path+'res'+directoryseparator+'lang'+directoryseparator,desk_env);
end;

procedure TForm_peach.LabelStatusClick(Sender: TObject);
begin
showbar('none');
end;

procedure TForm_peach.mAboutClick(Sender: TObject);
begin
MessageDlg ('PeaZip '+txt_release+' '+PEAZIPVERSION+PEAZIPREVISION+' - '+caption_build+char($0d)+char($0a)
           +txt_license+' LGPLv3, '+txt_projectadmin+' Giorgio Tani'+char($0d)+char($0a)
           +char($0d)+char($0a)
           +txt_using+char($0d)+char($0a)
           +' - '+APPLICATION1+char($0d)+char($0a)
           +' - '+APPLICATION2+char($0d)+char($0a)
           +' - '+APPLICATION3+char($0d)+char($0a)
           +' - '+APPLICATION4+char($0d)+char($0a)
           +' - '+APPLICATION5+char($0d)+char($0a)
           +' - '+APPLICATION6+char($0d)+char($0a)
           +' - '+APPLICATION7, mtInformation, [mbOk], 0);
end;

procedure TForm_peach.mAddClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
if checkfdwin=0 then addtoarchive_selected;
end;

procedure set_advfilters;
var
   a1,a2:array of utf8string;
   recinc,recexc,useadv:boolean;
   i:integer;
begin
if Form7.CheckBoxAdvFilters.State=cbChecked then useadv:=true else useadv:=false;
if Form7.CheckBoxAdvRecurse.State=cbChecked then recinc:=true else recinc:=false;
if Form7.CheckBoxAdvRecurse1.State=cbChecked then recexc:=true else recexc:=false;
setlength(a1,Form7.MemoAdvInclude.Lines.Count);
setlength(a2,Form7.MemoAdvExclude.Lines.Count);
for i:=0 to Form7.MemoAdvInclude.Lines.Count-1 do a1[i]:=Form7.MemoAdvInclude.Lines[i];
for i:=0 to Form7.MemoAdvExclude.Lines.Count-1 do a2[i]:=Form7.MemoAdvExclude.Lines[i];
Form7.Showmodal;
if form7.ModalResult=1 then
   begin
   advfiltersupdate;
   {$IFDEF MSWINDOWS}if Form_peach.PanelOpen.Visible=true{$ELSE}if Form_peach.PanelOpen.Top=0{$ENDIF} then navrefresh;
   end
else
   begin
   if useadv=true then
      begin
      Form7.CheckBoxAdvFilters.State:=cbChecked;
      Form7.GroupBoxOpenFilters.Enabled:=true;
      end
   else
      begin
      Form7.CheckBoxAdvFilters.State:=cbUnChecked;
      Form7.GroupBoxOpenFilters.Enabled:=false;
      end;
   if recinc=true then Form7.CheckBoxAdvRecurse.State:=cbChecked else Form7.CheckBoxAdvRecurse.State:=cbUnChecked;
   if recexc=true then Form7.CheckBoxAdvRecurse1.State:=cbChecked else Form7.CheckBoxAdvRecurse1.State:=cbUnChecked;
   Form7.MemoAdvInclude.Clear;
   Form7.MemoAdvExclude.Clear;
   for i:=0 to length(a1)-1 do Form7.MemoAdvInclude.Append(a1[i]);
   for i:=0 to length(a2)-1 do Form7.MemoAdvExclude.Append(a2[i]);
   end;
end;

procedure TForm_peach.madvfiltersClick(Sender: TObject);
begin
set_advfilters;
end;

procedure TForm_peach.mb10Click(Sender: TObject);
begin
browsebookmark(10);
end;

procedure TForm_peach.mb11Click(Sender: TObject);
begin
browsebookmark(11);
end;

procedure TForm_peach.mb12Click(Sender: TObject);
begin
browsebookmark(12);
end;

procedure TForm_peach.mb13Click(Sender: TObject);
begin
browsebookmark(13);
end;

procedure TForm_peach.mb14Click(Sender: TObject);
begin
browsebookmark(14);
end;

procedure TForm_peach.mb15Click(Sender: TObject);
begin
browsebookmark(15);
end;

procedure TForm_peach.mb16Click(Sender: TObject);
begin
browsebookmark(16);
end;

procedure TForm_peach.mb17Click(Sender: TObject);
begin
browsebookmark(17);
end;

procedure TForm_peach.mb18Click(Sender: TObject);
begin
browsebookmark(18);
end;

procedure TForm_peach.mb19Click(Sender: TObject);
begin
browsebookmark(19);
end;

procedure TForm_peach.mb1Click(Sender: TObject);
begin
browsebookmark(1);
end;

procedure TForm_peach.mb20Click(Sender: TObject);
begin
browsebookmark(20);
end;

procedure TForm_peach.mb21Click(Sender: TObject);
begin
browsebookmark(21);
end;

procedure TForm_peach.mb22Click(Sender: TObject);
begin
browsebookmark(22);
end;

procedure TForm_peach.mb23Click(Sender: TObject);
begin
browsebookmark(23);
end;

procedure TForm_peach.mb24Click(Sender: TObject);
begin
showpanel('open');
if stringgridbookmarks.visible=false then showbar('bookmarks');
navrefresh;
end;

procedure TForm_peach.mb2Click(Sender: TObject);
begin
browsebookmark(2);
end;

procedure TForm_peach.mb3Click(Sender: TObject);
begin
browsebookmark(3);
end;

procedure TForm_peach.mb4Click(Sender: TObject);
begin
browsebookmark(4);
end;

procedure TForm_peach.mb5Click(Sender: TObject);
begin
browsebookmark(5);
end;

procedure TForm_peach.mb6Click(Sender: TObject);
begin
browsebookmark(6);
end;

procedure TForm_peach.mb7Click(Sender: TObject);
begin
browsebookmark(7);
end;

procedure TForm_peach.mb8Click(Sender: TObject);
begin
browsebookmark(8);
end;

procedure TForm_peach.mb9Click(Sender: TObject);
begin
browsebookmark(9);
end;

procedure addarchive_tobookmarks;
var
   s:utf8string;
begin
if ufileexists((Form_peach.EditOpenIn.Text)) then Form_peach.OpenDialog3.InitialDir:=uextractfilepath(Form_peach.EditOpenIn.Text)
else Form_peach.OpenDialog3.InitialDir:=Form_peach.EditOpenIn.Text;
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.Filename<>'' then
      begin
      s:=Form_peach.OpenDialog3.Filename;
      addtobookmarks(s);
      end
   else exit
else exit;
end;

procedure TForm_peach.mbAddArchiveClick(Sender: TObject);
begin
addarchive_tobookmarks;
end;

procedure addpath_tobookmarks;
var
   s:utf8string;
begin
if ufileexists((Form_peach.EditOpenIn.Text)) then Form_peach.SelectDirectoryDialog1.InitialDir:=uextractfilepath(Form_peach.EditOpenIn.Text)
else Form_peach.SelectDirectoryDialog1.InitialDir:=Form_peach.EditOpenIn.Text;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      begin
      s:=Form_peach.SelectDirectoryDialog1.FileName;
      addtobookmarks(s);
      end
   else exit
else exit;
end;

procedure TForm_peach.mbAddPathClick(Sender: TObject);
begin
addpath_tobookmarks;
end;

procedure TForm_peach.mbDeleteClick(Sender: TObject);
begin
if StringGridBookmarks.Rowcount<2 then exit;
StringGridBookmarks.DeleteColRow(false,StringGridBookmarks.Row);
populatebookmarks;
end;

procedure addnote_tobookmarks;
var
   s:utf8string;
begin
with Form_peach do
begin
s:=StringGridBookmarks.Cells[5,StringGridBookmarks.Row];
if pInputQuery(txt_note, StringGridBookmarks.Cells[2,StringGridBookmarks.Row], s, false) then
   begin
   StringGridBookmarks.Cells[5,StringGridBookmarks.Row]:=s;
   StringGridBookmarks.AutoSizeColumns;
   StringGridBookmarks.ColWidths[6]:=0;
   StringGridBookmarks.ColWidths[0]:=ROWSIZE;
   end;
end;
end;

procedure TForm_peach.mbNotesClick(Sender: TObject);
begin
if StringGridBookmarks.Rowcount<2 then exit;
addnote_tobookmarks;
end;

procedure TForm_peach.mbOpenClick(Sender: TObject);
begin
if StringGridBookmarks.Rowcount<2 then exit;
browsebookmark(StringGridBookmarks.Row);
end;

procedure TForm_peach.mcancelClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then gocancelarchive
else
   {$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.Top=0{$ENDIF} then gocancelextract;
end;

procedure open_compmanagement;
begin
{$IFDEF MSWINDOWS}
ShellExecute(Form_peach.Handle, PChar ('open'), PChar('compmgmt.msc'), PChar (''), PChar (''), SW_SHOWNORMAL);
{$ENDIF}
end;

procedure TForm_peach.mCompClick(Sender: TObject);
begin
open_compmanagement;
end;

procedure TForm_peach.mcpClick(Sender: TObject);
var
   s:utf8string;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then sg:=StringGrid1
else sg:=StringGrid2;
if sg.Rowcount=1 then exit;
if sg.Cells[2,sg.Row]=txt_list_isfolder then s:=sg.Cells[8,sg.Row]
else s:=uextractfilepath(sg.Cells[8,sg.Row]);
if s<>'' then open_cp(s);
end;

procedure TForm_peach.mctrf2Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('home');
end;

procedure TForm_peach.mctrlshiftf2Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then list_root;
end;

procedure TForm_peach.mgobrowserClick(Sender: TObject);
begin
stayopen:=true;
Form_peach.OpenDialog3.Filename:=EditOpenIn.Text;
showpanel('open');
open_inputselect;
end;

procedure TForm_peach.mextractarchiveClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then goarchive
else
   {$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.Top=0{$ENDIF} then goextract;
end;

procedure TForm_peach.MenuItemArchive_documents1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=usr_documents;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.MenuItemArchive_home1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=uextractfilepath(home_path);
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.MenuItemextexploreClick(Sender: TObject);
var
   s:utf8string;
begin
s:=EditOpenOut.Text;
if s='' then exit;
if s[1]='<' then exit;
{$IFDEF MSWINDOWS}
winexplorepath(s);
{$ELSE}
cp_open(uextractfilepath(s),desk_env);
{$ENDIF}
end;

procedure TForm_peach.MenuItemOpen_LastClick(Sender: TObject);
begin
if lastoutpath='' then lastoutpath:=local_desktop;
Form_peach.EditOpenOut.Text:=uextractfilepath(lastoutpath);
end;

procedure TForm_peach.mf8Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('bookmark1');
end;

procedure TForm_peach.mgotoextlayoutClick(Sender: TObject);
begin
list_toextractor('none');
end;

procedure TForm_peach.mgotype7zClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   cbType.ItemIndex:=0;
   archive_type_select('7Z');
   end;
end;

procedure TForm_peach.mgotype7zsfxClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   cbType.ItemIndex:=7;
   archive_type_select(txt_sfx+' '+STR_7Z);
   end;
end;

procedure TForm_peach.mgotypebz2Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   cbType.ItemIndex:=2;
   archive_type_select('BZip2');
   end;
end;

procedure TForm_peach.mgotypegzClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   cbType.ItemIndex:=4;
   archive_type_select(STR_GZIP);
   end;
end;

procedure TForm_peach.mgotypetarClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   cbType.ItemIndex:=10;
   archive_type_select(STR_TAR);
   end;
end;

procedure TForm_peach.mgotypezipClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   cbType.ItemIndex:=12;
   archive_type_select(STR_ZIP);
   end;
end;

procedure TForm_peach.mLangClick(Sender: TObject);
begin
changelanguage;
end;

procedure TForm_peach.mpathdesk1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=local_desktop;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.mpathexplore1Click(Sender: TObject);
var
   s:utf8string;
begin
s:=EditName3.Text+Edit4.Text;
if s='' then exit;
if s=txt_mypc then exit;
if s[1]='<' then exit;
{$IFDEF MSWINDOWS}
if ((archive_type='7z') and (Form_peach.CheckBoxArchive6.Visible=true) and (Form_peach.CheckBoxArchive6.State=cbChecked)) or
   ((archive_type='arc') and (Form_peach.CheckBoxArc2.State=cbChecked))
then s:=s+'.exe'
else s:=s+'.'+archive_type;
winexplorepath(s);
{$ELSE}
cp_open(uextractfilepath(s),desk_env);
{$ENDIF}
end;

procedure TForm_peach.mpathreset1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=lastoutpath;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.mpathself1Click(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.Rowcount<2 then exit;
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=uextractfilepath(s);
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.msetpwClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelOpen.visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then ask_pwkeyfile
else
   repeat
   i:=ask_pwkeyfile_core;
   until i>=0;
end;

procedure TForm_peach.mshiftf7Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top=0{$ENDIF} then jumpto('mostrecent3');
end;

procedure open_diskclean(s:utf8string);
var
   P:TProcess;
   cl:utf8string;
begin
{$IFDEF MSWINDOWS}
if s<>'' then if checkfiledirname(s)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+s, mtWarning, [mbOK], 0); exit; end;
P:=TProcess.Create(nil);
if s<>'' then cl:='cleanmgr /d '+s
else cl:='cleanmgr';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
{$ENDIF}
end;

procedure disk_convert(s:utf8string);
var
   P:TProcess;
   cl:utf8string;
begin
{$IFDEF MSWINDOWS}
P:=TProcess.Create(nil);
cl:='convert '+s+' /FS:NTFS';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
{$ENDIF}
end;

procedure TForm_peach.mDiskCleanClick(Sender: TObject);
begin
open_diskclean('');
end;

procedure open_cpl;
var
   P:TProcess;
   cl:utf8string;
begin
{$IFDEF MSWINDOWS}
P:=TProcess.Create(nil);
cl:='control';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
{$ENDIF}
end;

procedure open_diskmanagement;
begin
{$IFDEF MSWINDOWS}
ShellExecute(Form_peach.Handle, PChar ('open'), PChar('diskmgmt.msc'), PChar (''), PChar (''), SW_SHOWNORMAL);
{$ENDIF}
end;

procedure TForm_peach.mCPLClick(Sender: TObject);
begin
open_cpl;
end;

procedure open_defrag(s:utf8string);
begin
{$IFDEF MSWINDOWS}
if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); exit; end;
if winver='nt5' then ShellExecute(Form_peach.Handle, PChar ('open'), PChar('dfrg.msc'), PChar (s), PChar (''), SW_SHOWNORMAL)
else ShellExecute(Form_peach.Handle, PChar ('open'), PChar('dfrgui.exe'), PChar (''), PChar (''), SW_SHOWNORMAL);
{$ENDIF}
end;

procedure TForm_peach.mDefragClick(Sender: TObject);
var
   s:utf8string;
begin
{$IFDEF MSWINDOWS}
if winver='nt4' then exit;
if fun='FILEBROWSER' then
   if Form_peach.EditOpenIn.Text=txt_mypc then s:=copy(Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row], 1, 2)
   else s:=copy(Form_peach.EditOpenIn.Text, 1, 2)
else s:='C:';
open_defrag(s);
{$ENDIF}
end;

procedure TForm_peach.mDiskmanagementClick(Sender: TObject);
begin
open_diskmanagement;
end;

procedure TForm_peach.mDiskConvertClick(Sender: TObject);
var
   s:utf8string;
begin
{$IFDEF MSWINDOWS}
if winver='9x' then exit;
if fun='FILEBROWSER' then
   if Form_peach.EditOpenIn.Text=txt_mypc then s:=copy(Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row], 1, 2)
   else s:=copy(Form_peach.EditOpenIn.Text, 1, 2)
else s:='C:';
disk_convert(s);
{$ENDIF}
end;

function convert2webname(var s:utf8string):integer;
var
   i:integer;
   r:boolean;
begin
convert2webname:=-1;
r:=pInputQuery(txt_websearch, txt_searchfor, s, false);
if r then
   begin
   if s='' then exit;
   i:=1;
   repeat
      i:= pos(' ',s);
      if i>0 then s[pos(' ',s)]:='+';
   until i=0;
   convert2webname:=0;
   end;
end;

procedure TForm_peach.MenuItemBooksClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://en.wikibooks.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.MenuItemCommonsClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://commons.wikimedia.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.MenuItemDictionaryClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://en.wiktionary.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.MenuItemHistoryBookClick(Sender: TObject);
begin
addtobookmarks('*history');
end;

procedure TForm_peach.MenuItemNavigateClick(Sender: TObject);
var
   s:utf8string;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then sg:=StringGrid1
else sg:=StringGrid2;
if sg.Rowcount=1 then exit;
if sg.Cells[2,sg.Row]=txt_list_isfolder then
   s:=sg.Cells[8,sg.Row]
else
   s:=uextractfilepath(sg.Cells[8,sg.Row]);
fun:='FILEBROWSER';
fun_status:=fun;
if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
Form_peach.EditUn7zaFilter.Text:='*';
showpanel('open');
listdir(s,false);
addtohistory;
end;

procedure TForm_peach.MenuItemOpen_desktop1Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(local_desktop);
end;

procedure TForm_peach.MenuItemOpen_documents1Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(usr_documents);
end;

procedure TForm_peach.MenuItemOpen_hereClick(Sender: TObject);
begin
if StringGrid2.Rowcount<2 then exit;
if Form_peach.labelstatus2.Caption= txt_2_7_ext then
   Form_peach.EditOpenOut.Text:=uextractfilepath(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row])
else
   Form_peach.EditOpenOut.Text:=uextractfilepath(Form_peach.EditOpenIn.Text);
end;

procedure TForm_peach.MenuItemOpen_home1Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(home_path);
end;

procedure TForm_peach.MenuItemPWClick(Sender: TObject);
var
   i:integer;
begin
repeat
i:=ask_pwkeyfile_core;
until i>=0;
end;

procedure TForm_peach.MenuItemSrcClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://en.wikisource.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.MenuItemVideoClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://video.google.com/videosearch?q='+s,desk_env);
end;

procedure TForm_peach.MenuItemOpen_documentsClick(Sender: TObject);
begin
jumpto('usrdocuments');
end;

procedure TForm_peach.MenuItemOpen_homeClick(Sender: TObject);
begin
jumpto('home');
end;

procedure hexpselected;
var
   cl,in_param:utf8string;
   P:TProcess;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
if sg.Row=0 then exit;
in_param:=delimiter+escapefilename(sg.Cells[8,sg.Row],desk_env)+delimiter;
cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter+' HEXPREVIEW '+in_param;
P:=TProcess.Create(nil);
{$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
end;

procedure TForm_peach.MenuItemToolhexpClick(Sender: TObject);
begin
hexpselected;
end;

procedure TForm_peach.MenuItemwnewsClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://en.wikinews.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.MenuItemyanswersClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://answers.yahoo.com/search/search_result;_ylv=3?p='+s,desk_env);
end;

procedure TForm_peach.MenuItemywebClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://us.search.yahoo.com/search?p='+s,desk_env);
end;

procedure TForm_peach.mEnvstrClick(Sender: TObject);
var
   P:TProcess;
   cl:utf8string;
begin
P:=TProcess.Create(nil);
cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter+' envstr';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
end;

procedure TForm_peach.MenuItemBookmarksClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.Top=0{$ENDIF} then
   if StringGrid1.Rowcount=1 then exit
   else
else
   if StringGrid2.Rowcount=1 then exit;
addtobookmarks('*layout');
end;

procedure TForm_peach.MenuItemOpen_addtobookmarksClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
addtobookmarks('*browser');
end;

procedure TForm_peach.MenuItemOpen_commandpromptClick(Sender: TObject);
var
   s:utf8string;
begin
if checkfdwin<>0 then exit;
if fun='FILEBROWSER' then
   if Form_peach.EditOpenIn.Text=txt_mypc then s:=Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row]
   else
      if Form_peach.StringGridList.Cells[2, Form_peach.StringGridList.Row]=txt_list_isfolder then s:=Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row]
      else
         begin
         s:=uextractfilepath(StringGridList.Cells[8,StringGridList.Row]);
         if s='' then s:=Form_peach.EditOpenIn.Text;
         end
else s:=uextractfilepath(Form_peach.EditOpenIn.Text);
if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
if s<>'' then open_cp(s);
end;

procedure removeunits;
var
   P:TProcess;
   cl:utf8string;
begin
{$IFDEF MSWINDOWS}
if winver='9x' then exit;
P:=TProcess.Create(nil);
cl:='RUNDLL32 SHELL32.DLL,Control_RunDLL hotplug.dll';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
{$ENDIF}
end;

procedure TForm_peach.mRemoveUnitsClick(Sender: TObject);
begin
removeunits;
end;

procedure TForm_peach.mshiftf2Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('root');
end;

procedure TForm_peach.mshiftf8Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('bookmark3');
end;

procedure TForm_peach.mtogglenewClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.Top=0{$ENDIF} then
   if Form_peach.checkboxfolder.State=cbChecked then Form_peach.checkboxfolder.State:=cbUnchecked
   else Form_peach.checkboxfolder.State:=cbChecked;
end;

procedure TForm_peach.mTrayClick(Sender: TObject);
begin
TrayIcon1.visible:=true;
Form_peach.visible:=false;
end;

procedure TForm_peach.mTutorialClick(Sender: TObject);
begin
Form3.Visible:=true;
end;

procedure TForm_peach.mTypeSfxArcClick(Sender: TObject);
begin
cbType.ItemIndex:=8;
archive_type_select(txt_sfx+' '+STR_ARC);
end;

procedure TForm_peach.mUserClick(Sender: TObject);
var
   P:TProcess;
   cl,u:utf8string;
   r:boolean;
begin
P:=TProcess.Create(nil);
{$IFDEF MSWINDOWS}
u:='Administrator';
r:=pInputQuery(txt_run_as, txt_user_name, u, false);
if not(r) then exit;
cl:='runas /env /user:'+u+' "'+executable_path+'peazip.exe"';
{$ELSE}//system needs to support sudo
u:='root';
r:=pInputQuery(txt_run_as, txt_user_name, u, false);
if not(r) then exit;
cl:='sudo -u '+u+' '''+executable_path+'peazip''';
{$ENDIF}
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
Close;
end;

procedure TForm_peach.owcustom10Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(2, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom11Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(3, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom12Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(4, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom13Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(5, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom14Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(6, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom15Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(7, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom16Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(8, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom1Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(1, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom2Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(2, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom3Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(3, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom4Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(4, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom5Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(5, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom6Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(6, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom7Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(7, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom8Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_custedit(8, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.owcustom9Click(Sender: TObject);
begin
if StringGridList.Cells[1,1]<>'' then open_advcustedit(1, StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.pajdn10Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn10.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn11Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn11.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn12Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn12.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn13Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn13.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn14Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn14.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn15Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn15.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn16Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn16.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn17Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn17.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn18Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn18.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn19Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn19.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn1.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn20Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn20.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn21Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn21.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn22Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn22.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn23Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn23.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn24Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn24.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn25Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn25.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn26Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn26.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn2Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn2.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn3Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn3.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn4Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn4.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn5Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn5.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn6Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn6.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn7Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn7.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn8Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn8.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pajdn9Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.Edit4.Text:=uextractfilename(s);
s:=pajdn9.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditName3.Text:=s;
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.PanelSideClick(Sender: TObject);
begin
showbar('none');
end;

procedure clear_clipboard;
var
   i:integer;
begin
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   begin
   Form_peach.StringGridList.Cells[12,i]:='0';
   Form_peach.StringGridList.Cells[0,i]:='';
   end;
Form_peach.StringGridList.Cells[12,1]:='1';
Form_peach.StringGridList.Row:=1;
Form_peach.Refresh;
setlength(clipcontent,0);
Form_peach.StringGridClipboard.Rowcount:=1;
Form_peach.StringGridClipboard.Autosizecolumns;
end;

procedure TForm_peach.pclip_clearclipClick(Sender: TObject);
begin
clear_clipboard;
end;

procedure TForm_peach.pclip_removeitemClick(Sender: TObject);
var
   i:integer;
begin
if Form_peach.StringGridClipboard.Rowcount<2 then exit;
if Form_peach.StringGridClipboard.Row=0 then exit;
//clear visible selection
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   begin
   Form_peach.StringGridList.Cells[12,i]:='0';
   Form_peach.StringGridList.Cells[0,i]:='';
   end;
Form_peach.StringGridList.Cells[12,1]:='1';
Form_peach.StringGridList.Row:=1;
Form_peach.Refresh;
//remove clipboard item
for i:=Form_peach.StringGridClipboard.Row-1 to length(clipcontent)-2 do
   begin
   clipcontent[i,0]:=clipcontent[i+1,0];
   clipcontent[i,1]:=clipcontent[i+1,1];
   clipcontent[i,2]:=clipcontent[i+1,2];
   clipcontent[i,3]:=clipcontent[i+1,3];
   end;
setlength(clipcontent,length(clipcontent)-1);
//update UI clipboard
Form_peach.StringGridClipboard.Rowcount:=Form_peach.StringGridClipboard.Rowcount-1;
if Form_peach.StringGridClipboard.Rowcount>1 then
   for i:=0 to length(clipcontent)-1 do
      begin
      Form_peach.StringGridClipboard.Cells[0,i+1]:=clipcontent[i,0];
      Form_peach.StringGridClipboard.Cells[1,i+1]:=clipcontent[i,1];
      Form_peach.StringGridClipboard.Cells[2,i+1]:=clipcontent[i,2];
      Form_peach.StringGridClipboard.Cells[3,i+1]:=clipcontent[i,3];
      end;
Form_peach.StringGridClipboard.Autosizecolumns;
end;

procedure TForm_peach.pcustompathexploreClick(Sender: TObject);
var
   s:utf8string;
begin
s:=uextractfilepath(patheditor.Text);
cp_open(s,desk_env);
end;

procedure TForm_peach.pcustompathresetClick(Sender: TObject);
begin
patheditor.Text:='';
if patheditor=EditNameCustom then EditExtCustom.Text:='';
end;

procedure TForm_peach.pmaddresstobookmarksClick(Sender: TObject);
begin
addtobookmarks(EditOpenIn.Text);
end;

procedure TForm_peach.po_cplClick(Sender: TObject);
begin
open_cpl;
end;

procedure TForm_peach.po_envstrClick(Sender: TObject);
var
   P:TProcess;
   cl:utf8string;
begin
P:=TProcess.Create(nil);
cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'pea'+EXEEXT+delimiter+' envstr';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
end;

procedure hexpselected_list;
var
   cl,in_param:utf8string;
   P:TProcess;
begin
   if Form_peach.StringGridList.Row=0 then exit;
   if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
   in_param:=delimiter+escapefilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row],desk_env)+delimiter;
   cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter+' HEXPREVIEW '+in_param;
   P:=TProcess.Create(nil);
   {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
end;

procedure TForm_peach.po_hexpClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
hexpselected_list;
end;

procedure TForm_peach.po_layoutClick(Sender: TObject);
begin
showpanel('archive');
showpanel_trick;
end;

procedure TForm_peach.po_manageClick(Sender: TObject);
begin
open_diskmanagement;
end;

procedure TForm_peach.po_removeunitsClick(Sender: TObject);
begin
removeunits;
end;

procedure TForm_peach.po_searchhereClick(Sender: TObject);
var
   s:utf8string;
begin
if (EditUn7zaFilter.Text='*') or (EditUn7zaFilter.Text='') or (EditUn7zaFilter.Text=' ') then
   begin
   s:=EditUn7zaFilter.Text;
   if not pInputQuery(txt_nrsearch, txt_search_refine, s, false) then exit;
   EditUn7zaFilter.Text:=s;
   end;
nrsearch;
end;

procedure TForm_peach.po_searchrecClick(Sender: TObject);
var
   s:utf8string;
begin
if (EditUn7zaFilter.Text='*') or (EditUn7zaFilter.Text='') or (EditUn7zaFilter.Text=' ') then
   begin
   s:=EditUn7zaFilter.Text;
   if not pInputQuery(txt_search, txt_search_refine, s, true) then exit;
   EditUn7zaFilter.Text:=s;
   end;
rsearch;
end;

procedure TForm_peach.po_setpwClick(Sender: TObject);
begin
ask_pwkeyfile;
end;

procedure open_taskman;
var
   P:TProcess;
   cl:utf8string;
begin
{$IFDEF MSWINDOWS}
if winver='9x' then exit;
P:=TProcess.Create(nil);
cl:='taskmgr';
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if Form_peach.Visible=true then Application.ProcessMessages;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
{$ENDIF}
end;

procedure TForm_peach.po_taskmanClick(Sender: TObject);
begin
open_taskman;
end;

procedure TForm_peach.pmArchiveNewsClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.RowCount=1 then exit;
s:=Form_peach.StringGrid1.Cells[1,Form_peach.StringGrid1.Row];
if convert2webname(s)=0 then cp_open('http://news.google.com/news?q='+s,desk_env);
end;

procedure TForm_peach.pmBrowserClick(Sender: TObject);
begin
stayopen:=true;
Form_peach.OpenDialog3.Filename:=EditOpenIn.Text;
showpanel('open');
open_inputselect;
end;

procedure TForm_peach.pmcommandClick(Sender: TObject);
var
   s:utf8string;
begin
if checkfdwin<>0 then exit;
if fun='FILEBROWSER' then
   if Form_peach.EditOpenIn.Text=txt_mypc then s:=Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row]
   else s:=uextractfilepath(Form_peach.EditOpenIn.Text)
else s:=uextractfilepath(Form_peach.EditOpenIn.Text);
if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
if s<>'' then open_cp(s);
end;

procedure TForm_peach.pmCustEditAdd1Click(Sender: TObject);
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.Filename<>'' then
      begin
      StringGridCustEdit1.Cells[2,StringGridCustEdit1.Row]:=Form_peach.OpenDialog3.Filename;
      StringGridCustEdit1.AutoSizeColumns;
      end
   else exit
else exit;
set_custedit_fromtable;
set_custedit_menus;
set_adveditcl(Form_peach.StringGridCustEdit1.Row);
end;

procedure TForm_peach.pmCustEditAddClick(Sender: TObject);
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.Filename<>'' then
      begin
      StringGridCustEdit.Cells[2,StringGridCustEdit.Row]:=Form_peach.OpenDialog3.Filename;
      StringGridCustEdit.AutoSizeColumns;
      end
   else exit
else exit;
set_custedit_fromtable;
set_custedit_menus;
end;

procedure setdescription_advcustedit;
var
   s:utf8string;
begin
with Form_peach do
begin
s:=StringGridCustEdit1.Cells[1,StringGridCustEdit1.Row];
if pInputQuery(txt_description, '', s, false) then
   begin
   StringGridCustEdit1.Cells[1,StringGridCustEdit1.Row]:=s;
   StringGridCustEdit1.AutoSizeColumns;
   end;
end;
set_custedit_fromtable;
set_custedit_menus;
set_adveditcl(Form_peach.StringGridCustEdit1.Row);
end;

procedure TForm_peach.pmCustEditDesc1Click(Sender: TObject);
begin
setdescription_advcustedit;
end;

procedure setdescription_custedit;
var
   s:utf8string;
begin
with Form_peach do
begin
s:=StringGridCustEdit.Cells[1,StringGridCustEdit.Row];
if pInputQuery(txt_description, '', s, false) then
   begin
   StringGridCustEdit.Cells[1,StringGridCustEdit.Row]:=s;
   StringGridCustEdit.AutoSizeColumns;
   end;
end;
set_custedit_fromtable;
set_custedit_menus;
end;

procedure TForm_peach.pmCustEditDescClick(Sender: TObject);
begin
setdescription_custedit;
end;

procedure editstring2_advcustedit;
var
   s:utf8string;
begin
with Form_peach do
begin
s:=StringGridCustEdit1.Cells[3,StringGridCustEdit1.Row];
if pInputQuery(txt_2_5_strafter, '', s, false) then
   begin
   StringGridCustEdit1.Cells[3,StringGridCustEdit1.Row]:=s;
   StringGridCustEdit1.AutoSizeColumns;
   end;
end;
set_custedit_fromtable;
set_custedit_menus;
set_adveditcl(Form_peach.StringGridCustEdit1.Row);
end;

procedure TForm_peach.pmCustEditEdit1afterClick(Sender: TObject);
begin
editstring2_advcustedit;
end;

procedure editstring1_advcustedit;
var
   s:utf8string;
begin
with Form_peach do
begin
s:=StringGridCustEdit1.Cells[2,StringGridCustEdit1.Row];
if pInputQuery(txt_2_5_strbefore, '', s, false) then
   begin
   StringGridCustEdit1.Cells[2,StringGridCustEdit1.Row]:=s;
   StringGridCustEdit1.AutoSizeColumns;
   end;
end;
set_custedit_fromtable;
set_custedit_menus;
set_adveditcl(Form_peach.StringGridCustEdit1.Row);
end;

procedure TForm_peach.pmCustEditEdit1Click(Sender: TObject);
begin
editstring1_advcustedit;
end;

procedure editapp_custedit;
var
   s:utf8string;
begin
with Form_peach do
begin
s:=StringGridCustEdit.Cells[2,StringGridCustEdit.Row];
if pInputQuery(txt_edit, '', s, false) then
   begin
   StringGridCustEdit.Cells[2,StringGridCustEdit.Row]:=s;
   StringGridCustEdit.AutoSizeColumns;
   end;
end;
set_custedit_fromtable;
set_custedit_menus;
end;

procedure TForm_peach.pmCustEditEditClick(Sender: TObject);
begin
editapp_custedit;
end;

procedure TForm_peach.pmCustEditRemove1Click(Sender: TObject);
begin
StringGridCustEdit1.Cells[1,StringGridCustEdit1.Row]:='';
StringGridCustEdit1.Cells[2,StringGridCustEdit1.Row]:='';
StringGridCustEdit1.Cells[3,StringGridCustEdit1.Row]:='';
StringGridCustEdit1.AutoSizeColumns;
set_custedit_fromtable;
set_custedit_menus;
set_adveditcl(StringGridCustEdit1.Row);
end;

procedure TForm_peach.pmCustEditRemoveClick(Sender: TObject);
begin
StringGridCustEdit.Cells[1,StringGridCustEdit.Row]:='';
StringGridCustEdit.Cells[2,StringGridCustEdit.Row]:='';
StringGridCustEdit.AutoSizeColumns;
set_custedit_fromtable;
set_custedit_menus;
end;

procedure TForm_peach.pmexploreClick(Sender: TObject);
var
   s:utf8string;
begin
if checkfdwin<>0 then exit;
if fun='FILEBROWSER' then
   begin
   {$IFDEF MSWINDOWS}
   if StringGridHistory.Cells[2,StringGridHistory.Row]=txt_mypc then
      begin
      cp_open(StringGridList.Cells[8,StringGridList.Row],desk_env);
      exit;
      end;
   {$ENDIF}
   {$IFDEF MSWINDOWS}
   if checkfiledirname(EditOpenIn.Text)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+EditOpenIn.Text, mtWarning, [mbOK], 0); exit; end;
   winexplorepath(EditOpenIn.Text);
   {$ELSE}
   cp_open(uextractfilepath(EditOpenIn.Text),desk_env);
   {$ENDIF}
   end
else
   {$IFDEF MSWINDOWS}
   begin
   if checkfiledirname(EditOpenIn.Text)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+EditOpenIn.Text, mtWarning, [mbOK], 0); exit; end;
   winexplorepath(EditOpenIn.Text);
   end;
   {$ELSE}
   cp_open(uextractfilepath(EditOpenIn.Text),desk_env);
   {$ENDIF}
end;

procedure TForm_peach.pmGolayoutClick(Sender: TObject);
begin
showpanel('archive');
showpanel_trick;
end;

procedure TForm_peach.pmh1Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>1 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-1;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh2Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>2 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-2;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh3Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>3 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-3;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh4Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>4 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-4;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh5Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>5 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-5;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh6Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>6 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-6;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh7Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>7 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-7;
   browsehistory;
   end;
end;

procedure TForm_peach.pmh8Click(Sender: TObject);
begin
if Form_peach.StringGridHistory.RowCount>8 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.RowCount-8;
   browsehistory;
   end;
end;

procedure TForm_peach.pmjb10Click(Sender: TObject);
begin
browsebookmark(10);
end;

procedure TForm_peach.pmjb11Click(Sender: TObject);
begin
browsebookmark(11);
end;

procedure TForm_peach.pmjb12Click(Sender: TObject);
begin
browsebookmark(12);
end;

procedure TForm_peach.pmjb13Click(Sender: TObject);
begin
browsebookmark(13);
end;

procedure TForm_peach.pmjb14Click(Sender: TObject);
begin
browsebookmark(14);
end;

procedure TForm_peach.pmjb15Click(Sender: TObject);
begin
browsebookmark(15);
end;

procedure TForm_peach.pmjb16Click(Sender: TObject);
begin
browsebookmark(16);
end;

procedure TForm_peach.pmjb17Click(Sender: TObject);
begin
browsebookmark(17);
end;

procedure TForm_peach.pmjb18Click(Sender: TObject);
begin
browsebookmark(18);
end;

procedure TForm_peach.pmjb19Click(Sender: TObject);
begin
browsebookmark(19);
end;

procedure TForm_peach.pmjb1Click(Sender: TObject);
begin
browsebookmark(1);
end;

procedure TForm_peach.pmjb20Click(Sender: TObject);
begin
browsebookmark(20);
end;

procedure TForm_peach.pmjb21Click(Sender: TObject);
begin
browsebookmark(21);
end;

procedure TForm_peach.pmjb22Click(Sender: TObject);
begin
browsebookmark(22);
end;

procedure TForm_peach.pmjb23Click(Sender: TObject);
begin
browsebookmark(23);
end;

procedure TForm_peach.pmjb24Click(Sender: TObject);
begin
if stringgridbookmarks.visible=false then showbar('bookmarks');
end;

procedure TForm_peach.pmjb2Click(Sender: TObject);
begin
browsebookmark(2);
end;

procedure TForm_peach.pmjb3Click(Sender: TObject);
begin
browsebookmark(3);
end;

procedure TForm_peach.pmjb4Click(Sender: TObject);
begin
browsebookmark(4);
end;

procedure TForm_peach.pmjb5Click(Sender: TObject);
begin
browsebookmark(5);
end;

procedure TForm_peach.pmjb6Click(Sender: TObject);
begin
browsebookmark(6);
end;

procedure TForm_peach.pmjb7Click(Sender: TObject);
begin
browsebookmark(7);
end;

procedure TForm_peach.pmjb8Click(Sender: TObject);
begin
browsebookmark(8);
end;

procedure TForm_peach.pmjb9Click(Sender: TObject);
begin
browsebookmark(9);
end;

procedure TForm_peach.pmjbaddClick(Sender: TObject);
begin
addtobookmarks('*current');
end;

procedure TForm_peach.MenuItemNewsClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://news.google.com/news?q='+s,desk_env);
end;

procedure TForm_peach.MenuItemOpen_archiveClick(Sender: TObject);
begin
open_archive;
end;

procedure TForm_peach.MenuItemOpen_desktopClick(Sender: TObject);
begin
jumpto('desktop');
end;

procedure TForm_peach.MenuItemOpen_exploreClick(Sender: TObject);
var
   s:utf8string;
begin
if checkfdwin<>0 then exit;
if fun='FILEBROWSER' then
   begin
   {$IFDEF MSWINDOWS}
   if StringGridHistory.Cells[2,StringGridHistory.Row]=txt_mypc then
      begin
      cp_open(StringGridList.Cells[8,StringGridList.Row],desk_env);
      exit;
      end;
   {$ENDIF}
   if StringGridList.Cells[2,StringGridList.Row]=txt_list_isfolder then
      cp_open(StringGridList.Cells[8,StringGridList.Row],desk_env) //open folder
   else //go to object's path
      begin
      s:=StringGridList.Cells[8,StringGridList.Row];
      if s='' then s:=Form_peach.EditOpenIn.Text;
      {$IFDEF MSWINDOWS}
      if checkfiledirname(s)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+s, mtWarning, [mbOK], 0); exit; end;
      winexplorepath(s);
      {$ELSE}
      cp_open(uextractfilepath(s),desk_env);
      {$ENDIF}
      end
   end
else
   {$IFDEF MSWINDOWS}
   begin
   if checkfiledirname(EditOpenIn.Text)<>0 then begin MessageDlg(txt_2_7_validatefn+' '+EditOpenIn.Text, mtWarning, [mbOK], 0); exit; end;
   winexplorepath(EditOpenIn.Text);
   end;
   {$ELSE}
   cp_open(uextractfilepath(EditOpenIn.Text),desk_env);
   {$ENDIF}
end;

procedure TForm_peach.MenuItemOpen_ExtractHereClick(Sender: TObject);
begin

end;

procedure TForm_peach.MenuItemOpen_myClick(Sender: TObject);
begin
jumpto('root');
end;

procedure TForm_peach.MenuItemOpen_pathClick(Sender: TObject);
begin
openpath;
end;

procedure TForm_peach.MenuItemOpen_refreshClick(Sender: TObject);
begin
navrefresh;
end;

procedure TForm_peach.mgotolayoutClick(Sender: TObject);
begin
showpanel('archive');
showpanel_trick;
end;

procedure TForm_peach.mPathsClick(Sender: TObject);
begin
//populatepcmenu;
end;

procedure TForm_peach.mRefreshClick(Sender: TObject);
begin
navrefresh;
end;

procedure removeselected;
var
   i,rc:integer;
begin
i:=1;
rc:=Form_peach.StringGrid1.RowCount;
while i<rc do
   begin
   if Form_peach.StringGrid1.Cells[11,i]='1' then
      begin
      Form_peach.StringGrid1.DeleteColRow(false,i);
      i:=i-1;
      rc:=Form_peach.StringGrid1.RowCount;
      end;
   i:=i+1;
   end;
updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
end;

procedure removeselected_ext;
var
   i,rc:integer;
begin
i:=1;
rc:=Form_peach.StringGrid2.RowCount;
while i<rc do
   begin
   if Form_peach.StringGrid2.Cells[11,i]='1' then
      begin
      Form_peach.StringGrid2.DeleteColRow(false,i);
      i:=i-1;
      rc:=Form_peach.StringGrid2.RowCount;
      end;
   i:=i+1;
   end;
updatecontent_ext;
end;

procedure TForm_peach.pmr1Click(Sender: TObject);
begin
open_archive_fromname(pmr1.Caption);
end;

procedure TForm_peach.pmr2Click(Sender: TObject);
begin
open_archive_fromname(pmr2.Caption);
end;

procedure TForm_peach.pmr3Click(Sender: TObject);
begin
open_archive_fromname(pmr3.Caption);
end;

procedure TForm_peach.pmr4Click(Sender: TObject);
begin
open_archive_fromname(pmr4.Caption);
end;

procedure TForm_peach.pmr5Click(Sender: TObject);
begin
open_archive_fromname(pmr5.Caption);
end;

procedure TForm_peach.pmr6Click(Sender: TObject);
begin
open_archive_fromname(pmr6.Caption);
end;

procedure TForm_peach.pmr7Click(Sender: TObject);
begin
open_archive_fromname(pmr7.Caption);
end;

procedure TForm_peach.pmr8Click(Sender: TObject);
begin
open_archive_fromname(pmr8.Caption);
end;

procedure TForm_peach.pmr9Click(Sender: TObject);
begin
if stringgridhistory.visible=false then showbar('history');
end;

procedure TForm_peach.pnb10Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb10.Caption);
end;

procedure TForm_peach.pnb11Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb11.Caption);
end;

procedure TForm_peach.pnb12Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb12.Caption);
end;

procedure TForm_peach.pnb13Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb13.Caption);
end;

procedure TForm_peach.pnb14Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb14.Caption);
end;

procedure TForm_peach.pnb15Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb15.Caption);
end;

procedure TForm_peach.pnb16Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb16.Caption);
end;

procedure TForm_peach.pnb17Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb17.Caption);
end;

procedure TForm_peach.pnb18Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb18.Caption);
end;

procedure TForm_peach.pnb19Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb19.Caption);
end;

procedure TForm_peach.pnb1Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb1.Caption);
end;

procedure TForm_peach.pnb20Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb20.Caption);
end;

procedure TForm_peach.pnb21Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb21.Caption);
end;

procedure TForm_peach.pnb22Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb22.Caption);
end;

procedure TForm_peach.pnb23Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb23.Caption);
end;

procedure TForm_peach.pnb2Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb2.Caption);
end;

procedure TForm_peach.pnb3Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb3.Caption);
end;

procedure TForm_peach.pnb4Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb4.Caption);
end;

procedure TForm_peach.pnb5Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb5.Caption);
end;

procedure TForm_peach.pnb6Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb6.Caption);
end;

procedure TForm_peach.pnb7Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb7.Caption);
end;

procedure TForm_peach.pnb8Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb8.Caption);
end;

procedure TForm_peach.pnb9Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnb9.Caption);
end;

procedure TForm_peach.pnba10Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba10.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba11Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba11.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba12Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba12.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba13Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba13.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba14Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba14.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba15Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba15.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba16Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba16.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba17Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba17.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba18Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba18.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba19Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba19.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba1.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba20Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba20.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba21Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba21.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba22Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba22.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba23Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba23.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba2Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba2.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba3Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba3.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba4Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba4.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba5Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba5.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba6Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba6.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba7Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba7.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba8Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba8.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnba9Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnba9.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnr1Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr1.Caption);
end;

procedure TForm_peach.pnr2Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr2.Caption);
end;

procedure TForm_peach.pnr3Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr3.Caption);
end;

procedure TForm_peach.pnr4Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr4.Caption);
end;

procedure TForm_peach.pnr5Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr5.Caption);
end;

procedure TForm_peach.pnr6Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr6.Caption);
end;

procedure TForm_peach.pnr7Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr7.Caption);
end;

procedure TForm_peach.pnr8Click(Sender: TObject);
begin
Form_peach.EditOpenOut.Text:=uextractfilepath(pnr8.Caption);
end;

procedure TForm_peach.pnra1Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra1.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra2Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra2.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra3Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra3.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra4Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra4.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra5Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra5.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra6Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra6.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra7Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra7.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.pnra8Click(Sender: TObject);
var
   s:utf8string;
begin
suggest_outname(s,Form_peach.cbType.Text,0);
Form_peach.EditName3.Text:=pnra8.Caption;
Form_peach.Edit4.Text:=uextractfilename(s);
Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
end;

procedure TForm_peach.podevicesClick(Sender: TObject);
begin
//populatepcmenu;
end;

procedure TForm_peach.pojd10Click(Sender: TObject);
begin
opendevice(pojd10.caption);
end;

procedure TForm_peach.pojd11Click(Sender: TObject);
begin
opendevice(pojd11.caption);
end;

procedure TForm_peach.pojd12Click(Sender: TObject);
begin
opendevice(pojd12.caption);
end;

procedure TForm_peach.pojd13Click(Sender: TObject);
begin
opendevice(pojd13.caption);
end;

procedure TForm_peach.pojd14Click(Sender: TObject);
begin
opendevice(pojd14.caption);
end;

procedure TForm_peach.pojd15Click(Sender: TObject);
begin
opendevice(pojd15.caption);
end;

procedure TForm_peach.pojd16Click(Sender: TObject);
begin
opendevice(pojd16.caption);
end;

procedure TForm_peach.pojd17Click(Sender: TObject);
begin
opendevice(pojd17.caption);
end;

procedure TForm_peach.pojd18Click(Sender: TObject);
begin
opendevice(pojd18.caption);
end;

procedure TForm_peach.pojd19Click(Sender: TObject);
begin
opendevice(pojd19.caption);
end;

procedure TForm_peach.pojd1Click(Sender: TObject);
begin
opendevice(pojd1.caption);
end;

procedure TForm_peach.pojd20Click(Sender: TObject);
begin
opendevice(pojd20.caption);
end;

procedure TForm_peach.pojd21Click(Sender: TObject);
begin
opendevice(pojd21.caption);
end;

procedure TForm_peach.pojd22Click(Sender: TObject);
begin
opendevice(pojd22.caption);
end;

procedure TForm_peach.pojd23Click(Sender: TObject);
begin
opendevice(pojd23.caption);
end;

procedure TForm_peach.pojd24Click(Sender: TObject);
begin
opendevice(pojd24.caption);
end;

procedure TForm_peach.pojd25Click(Sender: TObject);
begin
opendevice(pojd25.caption);
end;

procedure TForm_peach.pojd26Click(Sender: TObject);
begin
opendevice(pojd26.caption);
end;

procedure TForm_peach.pojd2Click(Sender: TObject);
begin
opendevice(pojd2.caption);
end;

procedure TForm_peach.pojd3Click(Sender: TObject);
begin
opendevice(pojd3.caption);
end;

procedure TForm_peach.pojd4Click(Sender: TObject);
begin
opendevice(pojd4.caption);
end;

procedure TForm_peach.pojd5Click(Sender: TObject);
begin
opendevice(pojd5.caption);
end;

procedure TForm_peach.pojd6Click(Sender: TObject);
begin
opendevice(pojd6.caption);
end;

procedure TForm_peach.pojd7Click(Sender: TObject);
begin
opendevice(pojd7.caption);
end;

procedure TForm_peach.pojd8Click(Sender: TObject);
begin
opendevice(pojd8.caption);
end;

procedure TForm_peach.pojd9Click(Sender: TObject);
begin
opendevice(pojd9.caption);
end;

procedure TForm_peach.pojdn10Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn10.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn11Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn11.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn12Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn12.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn13Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn13.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn14Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn14.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn15Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn15.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn16Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn16.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn17Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn17.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn18Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn18.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn19Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn19.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn1Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn1.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn20Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn20.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn21Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn21.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn22Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn22.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn23Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn23.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn24Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn24.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn25Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn25.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn26Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn26.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn2Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn2.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn3Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn3.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn4Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn4.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn5Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn5.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn6Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn6.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn7Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn7.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn8Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn8.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pojdn9Click(Sender: TObject);
var
   s:utf8string;
begin
s:=pojdn9.Caption;
s:=copy(s,length(s)-2,2)+directoryseparator;
Form_peach.EditOpenOut.Text:=uextractfilepath(s);
end;

procedure TForm_peach.pomb10Click(Sender: TObject);
begin
browsebookmark(10);
end;

procedure TForm_peach.pomb11Click(Sender: TObject);
begin
browsebookmark(11);
end;

procedure TForm_peach.pomb12Click(Sender: TObject);
begin
browsebookmark(12);
end;

procedure TForm_peach.pomb13Click(Sender: TObject);
begin
browsebookmark(13);
end;

procedure TForm_peach.pomb14Click(Sender: TObject);
begin
browsebookmark(14);
end;

procedure TForm_peach.pomb15Click(Sender: TObject);
begin
browsebookmark(15);
end;

procedure TForm_peach.pomb16Click(Sender: TObject);
begin
browsebookmark(16);
end;

procedure TForm_peach.pomb17Click(Sender: TObject);
begin
browsebookmark(17);
end;

procedure TForm_peach.pomb18Click(Sender: TObject);
begin
browsebookmark(18);
end;

procedure TForm_peach.pomb19Click(Sender: TObject);
begin
browsebookmark(19);
end;

procedure TForm_peach.pomb1Click(Sender: TObject);
begin
browsebookmark(1);
end;

procedure TForm_peach.pomb20Click(Sender: TObject);
begin
browsebookmark(20);
end;

procedure TForm_peach.pomb21Click(Sender: TObject);
begin
browsebookmark(21);
end;

procedure TForm_peach.pomb22Click(Sender: TObject);
begin
browsebookmark(22);
end;

procedure TForm_peach.pomb23Click(Sender: TObject);
begin
browsebookmark(23);
end;

procedure TForm_peach.pomb24Click(Sender: TObject);
begin
if stringgridbookmarks.visible=false then showbar('bookmarks');
end;

procedure TForm_peach.pomb2Click(Sender: TObject);
begin
browsebookmark(2);
end;

procedure TForm_peach.pomb3Click(Sender: TObject);
begin
browsebookmark(3);
end;

procedure TForm_peach.pomb4Click(Sender: TObject);
begin
browsebookmark(4);
end;

procedure TForm_peach.pomb5Click(Sender: TObject);
begin
browsebookmark(5);
end;

procedure TForm_peach.pomb6Click(Sender: TObject);
begin
browsebookmark(6);
end;

procedure TForm_peach.pomb7Click(Sender: TObject);
begin
browsebookmark(7);
end;

procedure TForm_peach.pomb8Click(Sender: TObject);
begin
browsebookmark(8);
end;

procedure TForm_peach.pomb9Click(Sender: TObject);
begin
browsebookmark(9);
end;

procedure TForm_peach.porecent1Click(Sender: TObject);
begin
open_archive_fromname(poRecent1.Caption);
end;

procedure TForm_peach.porecent2Click(Sender: TObject);
begin
open_archive_fromname(poRecent2.Caption);
end;

procedure TForm_peach.porecent3Click(Sender: TObject);
begin
open_archive_fromname(poRecent3.Caption);
end;

procedure TForm_peach.porecent4Click(Sender: TObject);
begin
open_archive_fromname(poRecent4.Caption);
end;

procedure TForm_peach.porecent5Click(Sender: TObject);
begin
open_archive_fromname(poRecent5.Caption);
end;

procedure TForm_peach.porecent6Click(Sender: TObject);
begin
open_archive_fromname(poRecent6.Caption);
end;

procedure TForm_peach.porecent7Click(Sender: TObject);
begin
open_archive_fromname(poRecent7.Caption);
end;

procedure TForm_peach.porecent8Click(Sender: TObject);
begin
open_archive_fromname(poRecent8.Caption);
end;

procedure TForm_peach.porecent9Click(Sender: TObject);
begin
if stringgridhistory.visible=false then showbar('history');
end;

procedure powcustom(j:integer; custmode:utf8string);
var
   i:integer;
   cl,jobcode,outname,s:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=Form_peach.StringGridList.Row;
if Form_peach.StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then
            begin
            s:=outname;
            if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
            s:=s+Form_peach.StringGridList.Cells[8,i];
            if custmode='cust' then open_custedit(j, s)
            else open_advcustedit(j, s);
            end
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then
            begin
            s:=outname;
            if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
            s:=s+Form_peach.StringGridList.Cells[8,i];
            if custmode='cust' then open_custedit(j, s)
            else open_advcustedit(j, s);
            end
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.powcustom10Click(Sender: TObject);
begin
powcustom(2,'adv');
end;

procedure TForm_peach.powcustom11Click(Sender: TObject);
begin
powcustom(3,'adv');
end;

procedure TForm_peach.powcustom12Click(Sender: TObject);
begin
powcustom(4,'adv');
end;

procedure TForm_peach.powcustom13Click(Sender: TObject);
begin
powcustom(5,'adv');
end;

procedure TForm_peach.powcustom14Click(Sender: TObject);
begin
powcustom(6,'adv');
end;

procedure TForm_peach.powcustom15Click(Sender: TObject);
begin
powcustom(7,'adv');
end;

procedure TForm_peach.powcustom16Click(Sender: TObject);
begin
powcustom(8,'adv');
end;

procedure TForm_peach.powcustom1Click(Sender: TObject);
begin
powcustom(1,'cust');
end;

procedure TForm_peach.powcustom2Click(Sender: TObject);
begin
powcustom(2,'cust');
end;

procedure TForm_peach.powcustom3Click(Sender: TObject);
begin
powcustom(3,'cust');
end;

procedure TForm_peach.powcustom4Click(Sender: TObject);
begin
powcustom(4,'cust');
end;

procedure TForm_peach.powcustom5Click(Sender: TObject);
begin
powcustom(5,'cust');
end;

procedure TForm_peach.powcustom6Click(Sender: TObject);
begin
powcustom(6,'cust');
end;

procedure TForm_peach.powcustom7Click(Sender: TObject);
begin
powcustom(7,'cust');
end;

procedure TForm_peach.powcustom8Click(Sender: TObject);
begin
powcustom(8,'cust');
end;

procedure TForm_peach.powcustom9Click(Sender: TObject);
begin
powcustom(1,'adv');
end;

procedure TForm_peach.po_computerClick(Sender: TObject);
begin
open_compmanagement;
end;

procedure TForm_peach.po_convertClick(Sender: TObject);
begin
disk_convert(copy(Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row], 1, 2));;
end;

procedure TForm_peach.mTaskmanagerClick(Sender: TObject);
begin
open_taskman;
end;

procedure TForm_peach.mTypeSfxClick(Sender: TObject);
begin
cbType.ItemIndex:=7;
archive_type_select(txt_sfx+' '+STR_7Z);
end;

procedure TForm_peach.cbwhirlpoolClick(Sender: TObject);
begin
if Form_peach.cbwhirlpool.State=cbChecked then algowhirl:=1 else algowhirl:=0;
updateconf_modified;
end;

procedure on_CheckBox4_click;
begin
if Form_peach.CheckBox4.State=cbChecked then ignorepathextand:=1 else ignorepathextand:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBox4Click(Sender: TObject);
begin
on_CheckBox4_click;
end;

procedure TForm_peach.CheckBoxARC1Click(Sender: TObject);
begin
if CheckBoxARC1.state=cbChecked then rr_arc:=1
else rr_arc:=0;
end;

procedure TForm_peach.CheckBoxArchive6Click(Sender: TObject);
begin
on_CheckBoxArchive6_click;
end;

procedure on_CheckBoxArchiveHistory_click;
begin
if Form_peach.CheckBoxArchiveHistory.State=cbChecked then savehistory:=1 else savehistory:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxArchiveHistoryClick(Sender: TObject);
begin
on_CheckBoxArchiveHistory_click;
end;

procedure CheckBoxArchiveOF_onclick;
begin
with Form_peach do
begin
if CheckBoxArchiveOF.State=cbChecked then openw_all7z:=1 else openw_all7z:=0;
end;
end;

procedure TForm_peach.CheckBoxArchiveOFClick(Sender: TObject);
begin
CheckBoxArchiveOF_onclick;
end;

procedure on_CheckBoxIgnoreDisp_click;
begin
if Form_peach.CheckBoxIgnoreDisp.State=cbChecked then ignorepathdisp:=1 else ignorepathdisp:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxIgnoreDispClick(Sender: TObject);
begin
on_CheckBoxIgnoreDisp_click;
end;

procedure on_CheckBoxIgnoreSel_click;
begin
if Form_peach.CheckBoxIgnoreSel.State=cbChecked then ignorepathsel:=1 else ignorepathsel:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxIgnoreSelClick(Sender: TObject);
begin
on_CheckBoxIgnoreSel_click;
end;

procedure on_CheckBoxPrebrowse_click;
begin
if Form_peach.CheckBoxPrebrowse.State=cbChecked then prebrowse:=1 else prebrowse:=0;
prebrowsesize:=Form_peach.SpinEditPrebrowse.Value;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxPrebrowseClick(Sender: TObject);
begin
on_CheckBoxPrebrowse_click;
end;

procedure on_CheckBoxQuadMax_click;
begin
if Form_peach.CheckBoxQuadMax.State=cbChecked then level_quad:=1 else level_quad:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxQuadMaxClick(Sender: TObject);
begin
on_CheckBoxQuadMax_click;
end;

procedure TForm_peach.CheckBoxShowHintsClick(Sender: TObject);
begin
if Form_peach.CheckBoxShowHints.State=cbChecked then showguihints:=1 else showguihints:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxTarBeforeClick(Sender: TObject);
var
   s:utf8string;
begin
if CheckBoxTarBefore.State=cbChecked then
   begin
   if CheckBoxSeparate.State=cbChecked then
      begin
      Edit4.Enabled:=true;
      suggest_outname(s,Form_peach.cbType.Text,Form_peach.StringGrid1.Row);
      Edit4.Text:=uextractfilename(s);
      Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
      end;
   CheckBoxSeparate.Enabled:=false;
   CheckBoxSeparate.State:=cbUnChecked;
   end
else
   CheckBoxSeparate.Enabled:=true;
end;

procedure on_CheckBoxWindowStatus_click;
begin
if Form_peach.CheckBoxWindowStatus.State=cbChecked then ws:=1 else ws:=0;
updateconf_modified;
end;

procedure TForm_peach.CheckBoxWindowStatusClick(Sender: TObject);
begin
on_CheckBoxWindowStatus_click;
end;

procedure TForm_peach.ColorButton1ColorChanged(Sender: TObject);
begin
color1:=ColorToString(ColorButton1.ButtonColor);
apply_theme;
end;

procedure TForm_peach.ColorButton2ColorChanged(Sender: TObject);
begin
color2:=ColorToString(ColorButton2.ButtonColor);
apply_theme;
end;

procedure TForm_peach.ColorButton3ColorChanged(Sender: TObject);
begin
color3:=ColorToString(ColorButton3.ButtonColor);
apply_theme;
end;

procedure TForm_peach.ColorButton4ColorChanged(Sender: TObject);
begin
color4:=ColorToString(ColorButton4.ButtonColor);
apply_theme;
end;

procedure TForm_peach.ColorButton5ColorChanged(Sender: TObject);
begin
color5:=ColorToString(ColorButton5.ButtonColor);
apply_theme;
end;

procedure TForm_peach.ComboBoxARCalgoChange(Sender: TObject);
begin
algo_arc:=Form_peach.ComboBoxARCalgo.ItemIndex;
end;

procedure TForm_peach.ComboBoxARCChange(Sender: TObject);
begin
level_arc:=Form_peach.ComboBoxARC.ItemIndex;
end;

procedure ComboBoxArchive1_onchange;
begin
with Form_peach do
begin
if ComboBoxArchive1.ItemIndex > 0 then CheckBoxArchive4.Visible:=false
else CheckBoxArchive4.Visible:=true;
change_imagesplit;
if ComboBoxArchive1.ItemIndex = 1 then
   begin
   SpinEditArchive1.Visible:=true;
   ComboBoxArchive2.Visible:=true;
   end
else
   begin
   SpinEditArchive1.Visible:=false;
   ComboBoxArchive2.Visible:=false;
   end;
end;
end;

procedure TForm_peach.ComboBoxArchive1Change(Sender: TObject);
begin
ComboBoxArchive1_onchange;
end;

procedure ComboBoxArchive3_onchange;
begin
//if prev_type='' then exit; //may be needed in rare cases: at first run, on some platform/compilers combination, avoid the compression method to be altered from combobok onchange function possibly invoked at startup
if settingvalues=false then
   with Form_peach do
      begin
      compression_method:=ComboBoxArchive3.Text;
      if cbType.Text='7Z' then method_7z:=compression_method;
      if cbType.Text='ZIP' then method_zip:=compression_method;
      set_values(archive_type,compression_method,compression_level);
      end;
end;

procedure TForm_peach.ComboBoxArchive3Change(Sender: TObject);
begin
ComboBoxArchive3_onchange;
end;

procedure ComboBoxArchive4_onchange;
begin
if settingvalues=false then
   with Form_peach do
      begin
      compression_level:=ComboBoxArchive4.Text;
      if cbType.Text='7Z' then level_7z:=ComboBoxArchive4.ItemIndex;
      if cbType.Text='BZip2' then level_bzip2:=ComboBoxArchive4.ItemIndex;
      if cbType.Text='GZip' then level_gz:=ComboBoxArchive4.ItemIndex;
      if cbType.Text='ZIP' then level_zip:=ComboBoxArchive4.ItemIndex;
      set_values(archive_type,compression_method,compression_level);
      end;
end;

procedure TForm_peach.ComboBoxArchive4Change(Sender: TObject);
begin
ComboBoxArchive4_onchange;
end;

procedure TForm_peach.ComboBoxArcSolidChange(Sender: TObject);
begin
solid_arc:=Form_peach.ComboBoxARCsolid.ItemIndex;
end;

procedure TForm_peach.ComboBoxPAQ1Change(Sender: TObject);
begin
level_paq:=Form_peach.ComboBoxPAQ1.ItemIndex;
end;

procedure ComboBoxPEA2_onchange;
begin
pstream:=Form_peach.ComboBoxPEA2.ItemIndex;
if Form_peach.ComboBoxPEA2.ItemIndex<3 then Form_peach.CheckBoxPEAEncryption.State:=cbChecked
else Form_peach.CheckBoxPEAEncryption.State:=cbUnchecked;
end;

procedure TForm_peach.ComboBoxPEA2Change(Sender: TObject);
begin
ComboBoxPEA2_onchange;
end;

procedure ComboBoxPEA3_onchange;
begin
pobj:=Form_peach.ComboBoxPEA3.ItemIndex;
end;

procedure TForm_peach.ComboBoxPEA3Change(Sender: TObject);
begin
ComboBoxPEA3_onchange;
end;

procedure ComboBoxPEA4_onchange;
begin
pvol:=Form_peach.ComboBoxPEA4.ItemIndex;
end;

procedure TForm_peach.ComboBoxPEA4Change(Sender: TObject);
begin
ComboBoxPEA4_onchange;
end;

procedure ComboBoxPEACompression1_onchange;
begin
pcompr:=Form_peach.ComboBoxPEACompression1.ItemIndex;
end;

procedure TForm_peach.ComboBoxPEACompression1Change(Sender: TObject);
begin
ComboBoxPEACompression1_onchange;
end;

procedure ComboBoxSplit3_onchange;
begin
salgo:=Form_peach.ComboBoxSplit3.ItemIndex;
end;

procedure TForm_peach.ComboBoxSplit3Change(Sender: TObject);
begin
ComboBoxSplit3_onchange;
end;

procedure extracttheme(var s:utf8string);
var
   cl,in_param,out_param,bin_name:utf8string;
   P:TProcess;
begin
if s<>'' then
   begin
   P:=TProcess.Create(nil);
   P.Options := [poNoConsole,poWaitOnExit];
   in_param:=delimiter+escapefilename(s,desk_env)+delimiter;
   out_param:=delimiter+'-o'+escapefilename(executable_path+'res'+directoryseparator+'themes'+directoryseparator,desk_env)+delimiter;
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter;
   cl:=bin_name+' x -aoa '+out_param+' '+in_param;
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   if Form_peach.Visible=true then Application.ProcessMessages;
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
   cutextension(s);
   s:=s+directoryseparator+'theme.txt';
   end;
end;

procedure ComboBoxTheme_onchange;
var
   s:utf8string;
begin
with Form_peach do
begin
case ComboBoxTheme.ItemIndex of
   0: begin
      OpenDialog4.initialdir:=executable_path+'res'+directoryseparator+'themes'+directoryseparator;
      OpenDialog4.execute;
      if OpenDialog4.FileName<>'' then
         try
            if pos(executable_path,OpenDialog4.FileName)=0 then
               begin
               MessageDlg(txt_theme_not_loadable,mtWarning,[mbOK],0);
               exit;
               end;
            s:=OpenDialog4.FileName;
            if (upcase(uextractfileext(s))='.ZIP') or (upcase(uextractfileext(s))='.7Z') or (upcase(uextractfileext(s))='.RAR') then
               extracttheme(s);
            theme_path:=extractrelativepath(executable_path,s);
            load_theme;
            check_theme_failure;
         except
            theme_failure;
            exit;
         end
      else exit;
      end;
   1: begin
      try
         s:=executable_path+'res'+directoryseparator+'themes'+directoryseparator+'firecrystal'+directoryseparator+'theme.txt';
         theme_path:=extractrelativepath(executable_path,s);
         load_theme;
         check_theme_failure;
      except
         theme_failure;
         exit;
      end;
      end;
   2: begin
      try
         s:=executable_path+'res'+directoryseparator+'themes'+directoryseparator+'nuovext'+directoryseparator+'theme.txt';
         if not(fileexists(s)) then
            begin
            s:=executable_path+'res'+directoryseparator+'themes'+directoryseparator+'nuovext.7z';
            extracttheme(s);
            end;
         theme_path:=extractrelativepath(executable_path,s);
         load_theme;
         check_theme_failure;
      except
         theme_failure;
         exit;
      end;
      end;
   3: begin
      try
         s:=executable_path+'res'+directoryseparator+'themes'+directoryseparator+'tango'+directoryseparator+'theme.txt';
         if not(fileexists(s)) then
            begin
            s:=executable_path+'res'+directoryseparator+'themes'+directoryseparator+'tango.7z';
            extracttheme(s);
            end;
         theme_path:=extractrelativepath(executable_path,s);
         load_theme;
         check_theme_failure;
      except
         theme_failure;
         exit;
      end;
      end;
   end;
if ((s<>'') and (theme_path<>'')) then
   try
      uassigntext(conf,confpath+'conf.txt');
      rewrite(conf);
      write_header(conf);
      writeln(conf,s);
      writeln(conf,graphicsfolderd);
      writeln(conf,opacityd);
      writeln(conf,color1d);
      writeln(conf,color2d);
      writeln(conf,color3d);
      writeln(conf,color4d);
      writeln(conf,color5d);
      writeln(conf,rowheightd);
      writeln(conf,itemheightd);
      writeln(conf,autosizeitemheightd);
      writeln(conf,autosizetoolbarbuttond);
      CloseFile(conf);
      theme_label.caption:='['+theme_name+']';
   except
      MessageDlg(txt_conf_cannotsave, mtWarning, [mbOK], 0);
      try
         CloseFile(conf);
      except
      end;
   end;
graphicsfolder:=graphicsfolderd;
DirectoryEditthemegraphics.Text:=executable_path+graphicsfolderd;
opacity:=opacityd;
color1:=color1d;
color2:=color2d;
color3:=color3d;
color4:=color4d;
color5:=color5d;
rowheight:=rowheightd;
itemheight:=itemheightd;
autosizeitemheight:=autosizeitemheightd;
autosizetoolbarbutton:=autosizetoolbarbuttond;
imgloaded:=false;
apply_theme;
end;
end;

procedure setmswindowsopacity;
begin
{$IFDEF MSWINDOWS}
osVerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
if GetVersionEx(osVerInfo) then
   begin
   if osVerInfo.dwMajorVersion>4 then //the system is NTx and most pmRecent than NT4
      begin
      {following code make problems on Win9x: while it correctly avoid importing
      user32.dll if SetLayeredWindowAttributes is not supported, it raises randomic
      errors on 9x systems, so the code is made unreachable for those systems and
      for NT4 that will not support that function anyway}
      Form_peach.Panel1.Visible:=true;
      if opacity<100 then
         begin
         try
            hUser32 := GetModuleHandle(PChar('USER32.DLL'));
            if hUser32 <> 0 then
               begin
               pointer(SetLayeredWindowAttributes) := GetProcAddress(hUser32, 'SetLayeredWindowAttributes');
               if @SetLayeredWindowAttributes <> nil then
                  begin
                  SetWindowLongPtr(Form_peach.Handle, GWL_EXSTYLE, GetWindowLong(Form_peach.Handle, GWL_EXSTYLE) OR WS_EX_LAYERED);
                  SetLayeredWindowAttributes(Form_peach.Handle, 0, 255+opacity-100, LWA_ALPHA);//not supported on 98 and NT4, called dynamically since has no meaning checking it at compile time
                  end;
               end;
         except
            Form_peach.Panel1.Visible:=false;
         end;
         end;
      end;
   end;
Form_peach.SpinEdit1.Value:=opacity;
{$ENDIF}
end;

procedure TForm_peach.ComboBoxThemeChange(Sender: TObject);
begin
ComboBoxTheme_onchange;
setmswindowsopacity;
end;

procedure TForm_peach.ComboBoxUPXChange(Sender: TObject);
begin
level_upx:=ComboBoxUPX.ItemIndex;
end;

procedure navgoback;
begin
if Form_peach.StringGridHistory.RowCount>2 then
   begin
   Form_peach.StringGridHistory.Row:=Form_peach.StringGridHistory.Row-1;
   browsehistory;
   end;
end;

procedure TForm_peach.ctrlbackClick(Sender: TObject);
begin
navgoback;
setfocus_stringgridlist;
end;

procedure TForm_peach.ctrlfwdClick(Sender: TObject);
begin
if StringGridHistory.Row<StringGridHistory.RowCount-1 then
   begin
   StringGridHistory.Row:=StringGridHistory.Row+1;
   browsehistory;
   setfocus_stringgridlist;
   end;
end;

procedure navgoup;
var
   s:utf8string;
begin
if (Form7.CheckBoxAdvFilters.State=cbUnChecked) and (fun='UN7Z') and (Form_peach.StringGridHistory.Cells[3,Form_peach.StringGridHistory.Row]<>'*') then
   begin
   if (Form_peach.EditUn7zaFilter.Text='*') and (Form_peach.EditUn7zaFilterExclude.Text='*'+directoryseparator+'*') then exit;
   s:=uextractfilepath(copy(uextractfilepath(Form_peach.EditUn7zaFilter.Text),1,length(uextractfilepath(Form_peach.EditUn7zaFilter.Text))-1));
   Form_peach.EditUn7zaFilter.Text:=s+'*';
   Form_peach.EditUn7zaFilterExclude.Text:=s+'*'+directoryseparator+'*';
   list_un7z('browse');
   status0:=txt_list_browsing;
   browsingmode7z:=0;
   addtohistory;
   end
else
   begin
   fun:='FILEBROWSER';
   fun_status:=fun;
   s:=Form_peach.EditOpenIn.Text;
   if s<>'' then if s[length(s)]=directoryseparator then setlength(s,length(s)-1);
   s:=uextractfilepath(s);
   {$IFDEF MSWINDOWS}
   if Form_peach.EditOpenIn.Text=txt_mypc then
      begin
      openpath;
      exit;
      end;
   if s<>'' then if s[length(s)]=':' then
      begin
      s:=s+directoryseparator;
      listpc;
      addtohistory;
      exit;
      end;
   {$ENDIF}
   Form_peach.EditUn7zaFilter.Text:='*';
   listdir(s,false);
   addtohistory;
   end;
end;

procedure TForm_peach.ctrlupClick(Sender: TObject);
begin
navgoup;
setfocus_stringgridlist;
end;

procedure TForm_peach.DirectoryEditthemegraphicsChange(Sender: TObject);
begin
if DirectoryEditthemegraphics.Text<>'' then
   if DirectoryEditthemegraphics.Text<>graphicsfolder then
      begin
      if pos(executable_path,DirectoryEditthemegraphics.Text)=0 then
         begin
         MessageDlg(txt_theme_not_loadable,mtWarning,[mbOK],0);
         exit;
         end;
      if DirectoryEditthemegraphics.Text<>'' then
         if DirectoryEditthemegraphics.Text[length(DirectoryEditthemegraphics.Text)]<>directoryseparator then DirectoryEditthemegraphics.Text:=DirectoryEditthemegraphics.Text+directoryseparator;
      graphicsfolder:=extractrelativepath(executable_path,DirectoryEditthemegraphics.Text);
      imgloaded:=false;
      apply_theme;
      end;
end;

procedure TForm_peach.EditUn7zaFilterKeyPress(Sender: TObject; var Key: char);
begin
//if Key=char(13) then nrsearch;
end;

procedure prepare_GUI;
var
   i:integer;
begin
if showguihints=1 then Form_peach.ShowHint:=true else Form_peach.ShowHint:=false;
with Form_peach.PanelArchiveMain do begin Top:=0; Left:=0; Width:=Form_peach.Width; end;
with Form_peach.PanelOpen do begin Top:=0; Left:=0; Width:=Form_peach.Width; end;
with Form_peach.PanelDefaults do begin Top:=0; Left:=0; Width:=Form_peach.Width; end;
with Form_peach do
begin
for i:=1 to 8 do StringGridCustEdit.Cells[0,i]:=inttostr(i);
StringGridCustEdit.Cells[1,0]:=txt_description;
StringGridCustEdit.Cells[2,0]:=txt_2_5_execommand;
StringGridCustEdit.AutoSizeColumns;
for i:=1 to 8 do StringGridCustEdit1.Cells[0,i]:=inttostr(i);
StringGridCustEdit1.Cells[1,0]:=txt_description;
StringGridCustEdit1.Cells[2,0]:=txt_2_5_strbefore;
StringGridCustEdit1.Cells[3,0]:=txt_2_5_strafter;
StringGridCustEdit1.AutoSizeColumns;
StringGrid1.Cells[1,0]:=txt_name;
StringGrid1.Cells[2,0]:=txt_type;
StringGrid1.Cells[3,0]:=txt_sizeb;
StringGrid1.Cells[4,0]:=txt_nfolders;
StringGrid1.Cells[5,0]:=txt_nfiles;
StringGrid1.Cells[6,0]:=txt_datetime;
StringGrid1.Cells[7,0]:=txt_attributes;
StringGrid1.Cells[8,0]:=txt_name_full;
StringGrid1.AutoSizeColumns;
StringGrid1.ColWidths[0]:=ROWSIZE;
StringGrid1.ColWidths[1]:=StringGrid1.ColWidths[1]+48;
StringGrid1.ColWidths[9]:=0;
StringGrid1.ColWidths[10]:=0;
StringGrid1.ColWidths[11]:=0;
StringGrid1.ColWidths[12]:=0;
StringGrid1.colwidths[13]:=8;
StringGrid2.Cells[1,0]:=txt_name;
StringGrid2.Cells[2,0]:=txt_type;
StringGrid2.Cells[3,0]:=txt_sizeb;
StringGrid2.Cells[4,0]:=txt_compressed_size;
StringGrid2.Cells[5,0]:=txt_datetime;
StringGrid2.Cells[6,0]:=txt_attributes;
StringGrid2.Cells[7,0]:=STR_CRC;
StringGrid2.Cells[8,0]:=txt_name_full;
autosize_stringgrid2;
StringGridList.Cells[1,0]:=txt_name;
StringGridList.Cells[2,0]:=txt_type;
StringGridList.Cells[3,0]:=txt_sizeb;
StringGridList.Cells[4,0]:=txt_compressed_size;
StringGridList.Cells[5,0]:=txt_datetime;
StringGridList.Cells[6,0]:=txt_attributes;
StringGridList.Cells[7,0]:=STR_CRC;
StringGridList.Cells[8,0]:=txt_name_full;
listsizecol('browse');
StringGridBookmarks.Cells[1,0]:=txt_bookmarks;
StringGridBookmarks.Cells[3,0]:=txt_inclusion;
StringGridBookmarks.Cells[4,0]:=txt_exclusion;
StringGridBookmarks.Cells[5,0]:=txt_description;
StringGridBookmarks.AutoSizeColumns;
StringGridBookmarks.ColWidths[6]:=0;
StringGridBookmarks.ColWidths[0]:=ROWSIZE;
StringGridClipboard.Cells[0,0]:=txt_2_4_operation;
StringGridClipboard.Cells[1,0]:=txt_name;
StringGridClipboard.Cells[2,0]:=txt_type;
StringGridClipboard.Cells[3,0]:=txt_name_full;
StringGridClipboard.AutoSizeColumns;
StringGridClipboard.ColWidths[0]:=ROWSIZE;
Notebook1.Pageindex:=0;
Notebook2.Pageindex:=0;
NotebookOpen.Pageindex:=0;
StringGridHistory.ColWidths[0]:=ROWSIZE;
StringGridHistory.Cells[1,0]:=txt_list_history;
StringGridHistory.Cells[3,0]:=txt_inclusion;
StringGridHistory.Cells[4,0]:=txt_exclusion;
with Form_peach.Panel7za do begin Top:=0; Left:=0; Width:=580 end;
with Form_peach.PanelPAQ do begin Top:=0; Left:=0; Width:=580 end;
with Form_peach.PanelQUAD do begin Top:=0; Left:=0; Width:=580 end;
with Form_peach.PanelUPX do begin Top:=0; Left:=0; Width:=580 end;
with Form_peach.PanelPEA do begin Top:=0; Left:=0; Width:=580 end;
with Form_peach.PanelArc do begin Top:=0; Left:=0; Width:=580 end;
with Form_peach.PanelCustom do begin Top:=0; Left:=0; Width:=580 end;
openwithcustom.Visible:=false;
MenuItemPreCustom.Visible:=false;
po_open_custom.Visible:=false;
MenuItemERCustom.Visible:=false; //uses Windows specific open with method
Panel1.Visible:=false; //it gives access to transparence control, obtained using Windows-specific method
EditUn7zaFilterExclude.Enabled:=false;
//fix position of some elements that tend to change without an apparent reason in the IDE
DirectoryEditThemeGraphics.Text:=executable_path+graphicsfolder;
if ws=1 then //load last window status
   begin
   if ws_status=0 then Form_peach.WindowState:=wsNormal else Form_peach.WindowState:=wsMaximized;
   Form_peach.Left:=ws_left;
   Form_peach.Top:=ws_top;
   Form_peach.Width:=ws_width;
   Form_peach.Height:=ws_height;
   end;
status0:=caption_release;
jd1.Visible:=false;
jd2.Visible:=false;
jd3.Visible:=false;
jd4.Visible:=false;
jd5.Visible:=false;
jd6.Visible:=false;
jd7.Visible:=false;
jd8.Visible:=false;
jd9.Visible:=false;
jd10.Visible:=false;
jd11.Visible:=false;
jd12.Visible:=false;
jd13.Visible:=false;
jd14.Visible:=false;
jd15.Visible:=false;
jd16.Visible:=false;
jd17.Visible:=false;
jd18.Visible:=false;
jd19.Visible:=false;
jd20.Visible:=false;
jd21.Visible:=false;
jd22.Visible:=false;
jd23.Visible:=false;
jd24.Visible:=false;
jd25.Visible:=false;
jd26.Visible:=false;
pojd1.Visible:=false;
pojd2.Visible:=false;
pojd3.Visible:=false;
pojd4.Visible:=false;
pojd5.Visible:=false;
pojd6.Visible:=false;
pojd7.Visible:=false;
pojd8.Visible:=false;
pojd9.Visible:=false;
pojd10.Visible:=false;
pojd11.Visible:=false;
pojd12.Visible:=false;
pojd13.Visible:=false;
pojd14.Visible:=false;
pojd15.Visible:=false;
pojd16.Visible:=false;
pojd17.Visible:=false;
pojd18.Visible:=false;
pojd19.Visible:=false;
pojd20.Visible:=false;
pojd21.Visible:=false;
pojd22.Visible:=false;
pojd23.Visible:=false;
pojd24.Visible:=false;
pojd25.Visible:=false;
pojd26.Visible:=false;
pojdn1.Visible:=false;
pojdn2.Visible:=false;
pojdn3.Visible:=false;
pojdn4.Visible:=false;
pojdn5.Visible:=false;
pojdn6.Visible:=false;
pojdn7.Visible:=false;
pojdn8.Visible:=false;
pojdn9.Visible:=false;
pojdn10.Visible:=false;
pojdn11.Visible:=false;
pojdn12.Visible:=false;
pojdn13.Visible:=false;
pojdn14.Visible:=false;
pojdn15.Visible:=false;
pojdn16.Visible:=false;
pojdn17.Visible:=false;
pojdn18.Visible:=false;
pojdn19.Visible:=false;
pojdn20.Visible:=false;
pojdn21.Visible:=false;
pojdn22.Visible:=false;
pojdn23.Visible:=false;
pojdn24.Visible:=false;
pojdn25.Visible:=false;
pojdn26.Visible:=false;
pajdn1.Visible:=false;
pajdn2.Visible:=false;
pajdn3.Visible:=false;
pajdn4.Visible:=false;
pajdn5.Visible:=false;
pajdn6.Visible:=false;
pajdn7.Visible:=false;
pajdn8.Visible:=false;
pajdn9.Visible:=false;
pajdn10.Visible:=false;
pajdn11.Visible:=false;
pajdn12.Visible:=false;
pajdn13.Visible:=false;
pajdn14.Visible:=false;
pajdn15.Visible:=false;
pajdn16.Visible:=false;
pajdn17.Visible:=false;
pajdn18.Visible:=false;
pajdn19.Visible:=false;
pajdn20.Visible:=false;
pajdn21.Visible:=false;
pajdn22.Visible:=false;
pajdn23.Visible:=false;
pajdn24.Visible:=false;
pajdn25.Visible:=false;
pajdn26.Visible:=false;
pmjd1.Visible:=false;
pmjd2.Visible:=false;
pmjd3.Visible:=false;
pmjd4.Visible:=false;
pmjd5.Visible:=false;
pmjd6.Visible:=false;
pmjd7.Visible:=false;
pmjd8.Visible:=false;
pmjd9.Visible:=false;
pmjd10.Visible:=false;
pmjd11.Visible:=false;
pmjd12.Visible:=false;
pmjd13.Visible:=false;
pmjd14.Visible:=false;
pmjd15.Visible:=false;
pmjd16.Visible:=false;
pmjd17.Visible:=false;
pmjd18.Visible:=false;
pmjd19.Visible:=false;
pmjd20.Visible:=false;
pmjd21.Visible:=false;
pmjd22.Visible:=false;
pmjd23.Visible:=false;
pmjd24.Visible:=false;
pmjd25.Visible:=false;
pmjd26.Visible:=false;
Labelcb13.Visible:=false;
RadioGrouppwrequest.Visible:=false;
CheckBoxIgnoreDisp.Visible:=false;
CheckBoxIgnoreSel.Visible:=false;
CheckBox4.Visible:=false;
pea.Visible:=false;
end;
end;

procedure setbasevars;
begin
executable_path:=uextractfilepath(envtoutf8(paramstr(0))); //valorize application's paths
if executable_path<>'' then if executable_path[length(executable_path)]<>directoryseparator then executable_path:=executable_path+directoryseparator;
usetcurrentdir(executable_path);
archiveopened:=false;
browsinghistory:=false;
imgloaded:=false;
openstarted:=false;
dragcancelled:=false;
listingdir:=false;
control_listingdir:=false;
waitdrawok:=false;
listcol:=1;
prev_type:='';
prev_method:='';
prev_level:='';
archive_type_selected:=-1;
using_tarbefore:=false;
seemencrypted:=false;
seemnamesencrypted:=false;
settingvalues:=false;
subfun:='';
listsortcol1:=9;
listsortcol2:=9;
listsortcol:=9;
listsortbook:=0;
az:=true;
az1:=true;
az2:=true;
azbook:=true;
setlength(clipcontent,0);
setlength(clipcontent1,0);
filecopying:=false;
destdir:='';
prev_destdir:='';
getdesk_env(desk_env,caption_build,delimiter); //desktop environment: 0 unknown; 1 Gnome; 2 KDE; 10 Windows; 20; OSX
desktop_path:='';
{$IFDEF MSWINDOWS}
wingetdesk;
wingetdocuments;
wingetuserprofile;
{$ELSE}
get_desktop_path(desktop_path);
get_home_path(home_path);
{$ENDIF}
if udirectoryexists(home_path) then
else home_path:=executable_path;
if udirectoryexists(desktop_path) then
else desktop_path:=home_path;
caption_release:='PeaZip '+PEAZIPVERSION+PEAZIPREVISION;
setlastbar:=true;
updatingarchive:=false;
pcmenupopulated:=false;
ares:=-1;
end;

procedure open_singleinput(s:utf8string);
var
   i:integer;
begin
case testinput(s,true) of
   0: begin
      showpanel('archive');
      showpanel_trick;
      if ufilegetattr(escapefilename(uexpandfilename(s),desk_env)) > 0 then
         if ufilegetattr(escapefilename(uexpandfilename(s),desk_env)) and faDirectory =0 then addfilestr(Form_peach.StringGrid1,escapefilename(s,desk_env))
         else addfolderstr(Form_peach.StringGrid1,escapefilename(uexpandfilename(s),desk_env))
      else msg_not_accessible_s(s);
      Form_peach.StringGrid1.AutoSizeColumns;
      updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
      end;
   else
      begin
      i:=testext(s);
      if (i<0) //not a PeaZip supported filetype
         or (i=503)
         or (i=502)
         or (i=501) //file types usually not handled as archives
         or (i=500) //exe or msi Windows' executable file
      then
         begin
         showpanel('archive');
         showpanel_trick;
         if ufilegetattr(escapefilename(uexpandfilename(s),desk_env)) > 0 then
            if ufilegetattr(escapefilename(uexpandfilename(s),desk_env)) and faDirectory =0 then addfilestr(Form_peach.StringGrid1,escapefilename(s,desk_env))
            else addfolderstr(Form_peach.StringGrid1,escapefilename(uexpandfilename(s),desk_env))
         else msg_not_accessible_s(s);
         Form_peach.StringGrid1.AutoSizeColumns;
         updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
         end
      else
         begin
         showpanel('open');
         Form_peach.EditOpenIn.Text:=escapefilename(uexpandfilename(s),desk_env);
         open_inputselect;
         end;
      end;
   end;
end;

procedure openstart;
//on Windows it is executed during formcreate, on other platform it is delayed
//being launched by a timer to allow form to be completely draw before those
//operations, which otherwise will result in different errors depending on the
//platform and the compiler revision
var
   i:integer;
   s:utf8string;
   okfile:boolean;
begin
openstarted:=true;
stayopen:=true;
with form_peach do
begin
//shell integration: if program is called with one of the following parameter as paramstr(1) use quick procedures without prompting PeaZip interface
if paramstr(1)='-add2pea' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2pea;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2crypt' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2crypt;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2wipe' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2wipe;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2compare' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2compare;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2split' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2split;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add27z' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add27z('archive');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add27zmail' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add27z('mail');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2separate7z' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2separate7z;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2sfx7z' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2sfx7z('archive');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2sfx7zmail' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2sfx7z('mail');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2zip' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2zip('archive');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2zipmail' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2zip('mail');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-add2separatezip' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   add2separatezip;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2full' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   Application.ProcessMessages;
   ext2full;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2here' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2here('','');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2folder' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2here('','newfolder');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2to' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2here('askto','');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2tofolder' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2here('askto','newfolder');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2list' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2lt('l');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2test' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2lt('t');
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2browse' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2browse;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2browsepath' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2browsepath;
   Form_peach.Close;
   exit;
   end;
if paramstr(1)='-ext2commandprompt' then
   begin
   Form_peach.Windowstate:=wsMinimized;
   Form_peach.Visible:=false;
   ext2commandprompt;
   Form_peach.Close;
   exit;
   end;
//shell integration: if program is called with filename(s) as param(s) try to understand what functionality is likely to be requested otherwise start the requested user interface
if paramcount=0 then
   begin
   showpanel('open');
   Form_peach.EditOpenIn.Text:=escapefilename(indir,desk_env); //indir not expanded with uexpandfilename
   fun:='FILEBROWSER';
   fun_status:=fun;
   open_inputselect;
   end
else
   if (paramcount=1) then //single input
      begin
      if paramstr(1)='-add2archive' then //directly calling -add2archive shortcut open archive panel
         begin
         showpanel('archive');
         showpanel_trick;
         StringGrid1.AutoSizeColumns;
         status1:='0 '+txt_dirs+' 0 '+txt_files+' 0 B';
         do_resize_trick;
         stayopen:=false;
         exit;
         end
      else
         if paramstr(1)='-ext2main' then
            begin
            Form_peach.EditOpenOut.Text:=local_desktop;
            list_toextractor('none');
            stayopen:=false;
            end
         else
            open_singleinput(envtoutf8(paramstr(1)));
      end
   else //multiple input
      begin
      if paramstr(1)='-ext2open' then //open a single file as archive, if type is supported; invoked by -ext2browse which handles multiple input
         begin
         if envtoutf8(paramstr(2))<>'' then
            if udirectoryexists(envtoutf8(paramstr(2))) then
               begin
               fun:='FILEBROWSER';
               fun_status:=fun;
               s:=envtoutf8(paramstr(2));
               if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
               Form_peach.EditUn7zaFilter.Text:='*';
               showpanel('open');
               listdir(s,false);
               addtohistory;
               end
            else
               open_archive_fromname(envtoutf8(paramstr(2)));
         exit;
         end;
      if paramstr(1)='-ext2main' then
         begin
         Form_peach.EditOpenOut.Text:=local_desktop;
         list_toextractor('none');
         for i:=2 to paramcount do
            begin
            test_extfile(escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env),okfile);
            if okfile=true then
               begin
               if checkfiledirname(escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env))<>0 then begin MessageDlg(txt_2_7_validatefn+' '+escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env), mtWarning, [mbOK], 0); exit; end;
               deselectingrid(Form_peach.StringGrid2);
               addfilestr(Form_peach.StringGrid2,escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env));
               end;
            end;
         if Form_peach.StringGrid2.Rowcount>1 then
            Form_peach.EditOpenOut.Text:=uextractfilepath(Form_peach.StringGrid2.Cells[8,Form_peach.StringGrid2.Row])
         else
            Form_peach.EditOpenOut.Text:=local_desktop;
         updatecontent_ext;
         stayopen:=false;
         exit;
         end;
      showpanel('archive'); //otherwise, add to archive
      showpanel_trick;
      for i:=1 to paramcount do
         begin
         if paramstr(i)='-add2archive' then
         else
            if testname(escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env), StringGrid1)=0 then
               if ufilegetattr(escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)) > 0 then
                  if ufilegetattr(escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env)) and faDirectory =0 then addfilestr(Form_peach.StringGrid1,escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env))
                  else addfolderstr(Form_peach.StringGrid1,escapefilename(uexpandfilename(envtoutf8(paramstr(i))),desk_env))
               else msg_not_accessible(i);
         end;
      StringGrid1.AutoSizeColumns;
      updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
      stayopen:=false;
      end;
end;
end;

procedure TForm_peach.FormCreate(Sender: TObject);
begin
{
entropy is collected by the system state at application startup (fingerprint);
mouse, keyboard and file contexts are initialized at app startup and never
finalized: at each key file generation they are copied and the copies are
finalized, while the contexts remain opened for sampling utter entropy;
fingerprint is regenerated each time a key is created (however, it will introduce
less entropy than the first time since less system states will be changed)
}
randomize;
setbasevars;
{$IFDEF MSWINDOWS}getwinenv(wincomspec,winver);{$ENDIF}
load_conf; //parse conf.txt in res folder for GUI theming and default operation parameters
load_bookmarks;
load_custedit;
showpanel_prepare;
prepare_GUI; //give starting positions to panels (that in the RAD may be misplaced)
Application.CreateForm(TForm2, Form2);
prepare_form2;
Application.CreateForm(TForm4, Form4);
prepare_form4;
Application.CreateForm(TForm5, Form5);
prepare_form5;
Application.CreateForm(TForm6, Form6);
prepare_form6;
Application.CreateForm(TForm7, Form7);
prepare_form7;
po_systemtools.Visible:=false;
//mSystem
 mDiskConvert.Visible:=false;
 mDiskClean.Visible:=false;
 mDefrag.Visible:=false;
 mDiskmanagement.Visible:=false;
 mRemoveUnits.Visible:=false;
 MenuItem4.Visible:=false;
 mCPL.Visible:=false;
 mComp.Visible:=false;
 mTaskmanager.Visible:=false;
 MenuItem20.Visible:=false;
mUser.Visible:=false;
mDate.Visible:=false;
MenuItem33.Visible:=false;
pmJumpDocuments.Visible:=false;
jddocuments.Visible:=false;
MenuItemOpen_documents.Visible:=false;
MenuItem11.Visible:=false;
{$IFDEF MSWINDOWS}
po_systemtools.Visible:=true;
//mSystem
 mDiskConvert.Visible:=true;
 mDiskClean.Visible:=true;
 mDefrag.Visible:=true;
 mDiskmanagement.Visible:=true;
 mRemoveUnits.Visible:=true;
 MenuItem4.Visible:=true;
 mCPL.Visible:=true;
 mComp.Visible:=true;
 mTaskmanager.Visible:=true;
 MenuItem20.Visible:=true;
mUser.Visible:=true;
mDate.Visible:=true;
MenuItem33.Visible:=true;
pmJumpDocuments.Visible:=true;
jddocuments.Visible:=true;
MenuItemOpen_documents.Visible:=true;
MenuItem11.Visible:=true;
openwithcustom.Visible:=true;
po_open_custom.Visible:=true;
MenuItemPreCustom.Visible:=true;
MenuItemERCustom.Visible:=true;
setmswindowsopacity;
if winver='9x' then
   begin
   local_desktop:=GetEnvironmentVariable('WINBOOTDIR')+'\Desktop\'; //fixes desktop path for 9x
   MenuItem11.Visible:=false;
   mUser.Visible:=false;
   MenuItem33.Visible:=false;
   mDiskManagement.Visible:=false;
   mRemoveUnits.Visible:=false;
   mComp.Visible:=false;
   mTaskmanager.Visible:=false;
   end;
if winver='nt4' then
   begin
   MenuItem11.Visible:=false;
   mUser.Visible:=false;
   MenuItem33.Visible:=false;
   mDefrag.Visible:=false;
   end;
{$ENDIF}
status1:=caption_build;
populatebookmarks;
populaterecenticons;
{$IFDEF MSWINDOWS}
openstart;
//populatepcmenu;
{$ENDIF}
do_resize_trick;
end;

procedure TForm_peach.FormDropFiles(Sender: TObject;
  const FileNames: array of String);

procedure drag_addtolayout;
var
   i: integer;
   s: utf8string;
begin
s:=Form_peach.Caption;
for i := 0 to High(FileNames) do
   if testname(escapefilename(FileNames[i],desk_env), Form_peach.StringGrid1)=0 then
      begin
      Form_peach.Caption:=txt_2_4_adding+' '+FileNames[i];
      Application.ProcessMessages;
      deselectingrid(Form_peach.StringGrid1);
      if (ufilegetattr(escapefilename(FileNames[i],desk_env)) and faDirectory) <> 0 then addfolderstr(Form_peach.StringGrid1,escapefilename(FileNames[i],desk_env))
      else addfilestr(Form_peach.StringGrid1,escapefilename(FileNames[i],desk_env));
      Form_peach.StringGrid1.AutoSizeColumns;
      updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
      end;
Form_peach.Caption:=s;
end;

procedure drag_addtoextlayout;
var
   i: integer;
   s: utf8string;
   okfile:boolean;
begin
if Form_peach.labelstatus2.Caption<>txt_2_7_ext then exit;
s:=Form_peach.Caption;
for i := 0 to High(FileNames) do
      begin
      test_extfile(FileNames[i],okfile);
      if okfile=true then
         begin
         if checkfiledirname(FileNames[i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+FileNames[i], mtWarning, [mbOK], 0); exit; end;
         Form_peach.Caption:=txt_2_4_adding+' '+FileNames[i];
         Application.ProcessMessages;
         deselectingrid(Form_peach.StringGrid2);
         addfilestr(Form_peach.StringGrid2,escapefilename(FileNames[i],desk_env));
         end;
      end;
updatecontent_ext;
Form_peach.Caption:=s;
end;

procedure drag_addtobookmarks;
var
   i: integer;
begin
for i := 0 to High(FileNames) do
   //if testname(escapefilename(FileNames[i],desk_env), Form_peach.StringGrid1)=0 then
      //begin
      addtobookmarks(escapefilename(FileNames[i],desk_env));
      //end;
end;

procedure drag_addtoarchive;
var
   cl,dfn,s:utf8string;
   i:integer;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
addtoarchive_inarchive;
for i := 0 to High(FileNames) do
   begin
   dfn:=escapefilename(FileNames[i],desk_env);
   if (ufilegetattr(dfn) and faDirectory) <> 0 then
      begin
      s:=Form_peach.Caption;
      if dfn<>'' then
         if testname(dfn, Form_peach.StringGrid1)=0 then
            begin
            Form_peach.StringGrid1.Cursor:=crHourGlass;
            Form_peach.Caption:=txt_2_4_adding+' '+dfn;
            Application.ProcessMessages;
            deselectingrid(Form_peach.StringGrid1);
            addfolderstr(Form_peach.StringGrid1,dfn);
            Form_peach.StringGrid1.AutoSizeColumns;
            updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
            Form_peach.StringGrid1.Cursor:=crDefault;
         end;
      Form_peach.Caption:=s;
      end
   else
      begin
      if dfn<>'' then
         begin
         Form_peach.StringGrid1.Cursor:=crHourGlass;
         deselectingrid(Form_peach.StringGrid1);
         if testname(dfn, Form_peach.StringGrid1)=0 then
            begin
            //show names of items being added is not implemented here since adding files is very fast
            addfilestr(Form_peach.StringGrid1,dfn);
            Form_peach.StringGrid1.AutoSizeColumns;
            updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
            end;
         Form_peach.StringGrid1.Cursor:=crDefault;
         end;
      end;
   end;
end;

var
   i:integer; x,y,row,column:integer;lpPoint: TPoint;
begin
if Form_peach.visible=false then exit;
{if (PanelDefaults.Visible=true) and (PanelDefaults.Top<10000) then
   begin
   if High(FileNames)=0 then
      begin
      i:=testext(escapefilename(FileNames[0],desk_env));
      if (i<0) //not a PeaZip supported filetype
         or (i=503)
         or (i=502)
         or (i=501) //file types usually not handled as archives
         or (i=500) //exe or msi Windows executable file
      then
         begin
         showpanel('archive');
         showpanel_trick;
         drag_addtolayout;
         end
      else
         if ((testinput(escapefilename(FileNames[0],desk_env),true)>0) and (testinput(escapefilename(FileNames[0],desk_env),true)<>1000)) then
            case MessageDlg(txt_dis+' '+txt_2_7_drag_archive, mtWarning,[mbYes, mbNo],0) of //executed only when the form is visible
               6: open_archive_fromname(escapefilename(FileNames[0],desk_env));
               7: begin
                  showpanel('archive');
                  showpanel_trick;
                  drag_addtolayout;
                  end
               end
         else
            begin
            showpanel('archive');
            showpanel_trick;
            drag_addtolayout;
            end
      end
   else
      begin
      showpanel('archive');
      showpanel_trick;
      drag_addtolayout;
      end;
   exit;
   end;}
if (PanelArchiveMain.Visible=true)  and (PanelArchiveMain.Top<10000) then drag_addtolayout;
if (PanelExtract.Visible=true)  and (PanelExtract.Top<10000) then drag_addtoextlayout;
if (PanelOpen.Visible=true)  and (PanelOpen.Top<10000) then
   begin
   {$IFDEF MSWINDOWS}
   lpPoint := StringGridBookmarks.ScreenToClient(Mouse.CursorPos) ;
   if lpPoint.y>0 then
      begin
      if stringgridbookmarks.visible=false then showbar('bookmarks');
      drag_addtobookmarks;
      exit;
      end;
   {$ENDIF}
   if High(FileNames)=0 then
      begin
      i:=testext(escapefilename(FileNames[0],desk_env));
      if (i<0) //not a PeaZip supported filetype
         or (i=503)
         or (i=502)
         or (i=501) //file types usually not handled as archives
         or (i=500) //exe or msi Windows executable file
      then
         if fun='FILEBROWSER' then
            begin
            showpanel('archive');
            drag_addtolayout;
            end
         else
            drag_addtoarchive
      else
         if (testinput(escapefilename(FileNames[0],desk_env),true)>0) and (testinput(escapefilename(FileNames[0],desk_env),true)<>1000) then
            case MessageDlg(txt_dis+' '+txt_2_7_drag_archive, mtWarning,[mbYes, mbNo],0) of //executed only when the form is visible
               6: open_archive_fromname(escapefilename(FileNames[0],desk_env));
               7: if fun='FILEBROWSER' then
                     begin
                     showpanel('archive');
                     drag_addtolayout;
                     end
                  else
                     drag_addtoarchive;
               end
         else
            if fun='FILEBROWSER' then
               begin
               showpanel('archive');
               drag_addtolayout;
               end
            else
               drag_addtoarchive
      end
   else
      if fun='FILEBROWSER' then
         begin
         showpanel('archive');
         drag_addtolayout;
         end
      else
         drag_addtoarchive;
   end;
end;

procedure TForm_peach.FormResize(Sender: TObject);
begin
ws_width:=Form_peach.Width;
ws_height:=Form_peach.Height;
end;

procedure TForm_peach.FormWindowStateChange(Sender: TObject);
begin
if Form_peach.WindowState=wsNormal then
   begin
   Form_peach.Left:=ws_left;
   Form_peach.Top:=ws_top;
   Form_peach.Width:=ws_width;
   Form_peach.Height:=ws_height;
   do_resize_trick;
   end;
end;

procedure TForm_peach.HowToClick(Sender: TObject);
begin
showpanel('howto');
end;

procedure TForm_peach.iConsoleCreateImport1Click(Sender: TObject);
begin
mConsoleCreate1.Clear;
lConsoleCreate_jobcode1.Caption:='';
lConsoleCreate_outname1.Caption:='';
end;

procedure TForm_peach.iConsoleCreateImportClick(Sender: TObject);
begin
mConsoleCreate.Clear;
lConsoleCreate_jobcode.Caption:='';
lConsoleCreate_outname.Caption:='';
end;

function importjobcl(var cl,jobcode,outname:utf8string):integer;
var
   sel:utf8string;
begin
importjobcl:=0;
with Form_peach do
begin
if CheckBoxTarBefore.State=cbChecked then
   begin
   MessageDlg(txt_tarbeforepj, mtWarning, [mbOK], 0);
   exit;
   end;
if CheckBoxSeparate.State=cbChecked then
   begin
   MessageDlg(txt_2_7_separateerror, mtWarning, [mbOK], 0);
   exit;
   end;
sel:='all';
if (cbType.Text=txt_custom) then
   if compose_custom_cl(cl,jobcode,outname,sel)<>0 then exit;
if (cbType.Text=STR_ZPAQ) then
   if compose_zpaq_cl(cl,jobcode,outname,sel)<>0 then exit;
if (cbType.Text=STR_QUAD) then
   if compose_quad_cl(cl,jobcode,outname,sel)<>0 then exit;
if (cbType.Text=STR_UPX) then
   if compose_upx_cl(cl,jobcode,outname,sel)<>0 then exit;
if (cbType.Text=STR_ARC) or (cbType.Text=txt_sfx+' '+STR_ARC) then
   if compose_arc_cl(cl,jobcode,outname,sel)<>0 then exit;
if (cbType.Text=STR_PEA) or (cbType.Text=txt_split) then
   if compose_pea_cl(cl,jobcode,outname,sel)<>0 then exit;
if (cbType.Text=STR_7Z) or (cbType.Text=STR_BZIP2) or (cbType.Text=STR_GZIP) or (cbType.Text=STR_TAR) or (cbType.Text=STR_ZIP) or (cbType.Text=txt_sfx+' '+STR_7Z) then
   if compose_archive_cl(cl,jobcode,outname,sel)<>0 then exit;
mConsoleCreate.Clear;
mConsoleCreate.Append(cl);
lConsoleCreate_jobcode.Caption:=jobcode;
lConsoleCreate_outname.Caption:=outname;
end;
importjobcl:=1;
end;

function importjobcl_extract(var cl,jobcode,outname:utf8string):integer;
begin
importjobcl_extract:=0;
with Form_peach do
begin
if extract_main_bytype('neutral',cl,jobcode,outname,false)<>0 then exit;
mConsoleCreate1.Clear;
mConsoleCreate1.Append(cl);
lConsoleCreate_jobcode1.Caption:=jobcode;
lConsoleCreate_outname1.Caption:=outname;
end;
importjobcl_extract:=1;
end;

procedure save_pj(cl:utf8string);
begin
if Form_peach.SaveDialogPJ.Execute then
   if Form_peach.SaveDialogPJ.FileName<>'' then
      begin
      uassigntext(t,Form_peach.SaveDialogPJ.FileName);
      rewrite(t);
      if jobdefenc=1 then write_header(t);
      write(t,cl);
      closefile(t);
      end;
end;

procedure TForm_peach.Image7zPj1Click(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
cl:=mConsoleCreate.Text;
setlength(cl,length(cl)-2); //remove text feeds, which are illegal characters for the command line
jobcode:=lConsoleCreate_jobcode.Caption;
outname:=lConsoleCreate_outname.Caption;
{real output name could have been modified by user in the memo (not checked), in
this case it will not be possible to open output path and do compression statistics
from pealauncher}
if (cl<>'') and (length(cl)>3) then launch_cl(cl,jobcode,outname)
else
   if importjobcl(cl,jobcode,outname)=1 then launch_cl(cl,jobcode,outname);
pf:=Form_peach.cbType.ItemIndex;
end;

procedure TForm_peach.Image7zPj3Click(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
cl:=mConsoleCreate1.Text;
setlength(cl,length(cl)-2); //remove text feeds, which are illegal characters for the command line
jobcode:=lConsoleCreate_jobcode1.Caption;
outname:=lConsoleCreate_outname1.Caption;
{real output name could have been modified by user in the memo (not checked), in
this case it will not be possible to open output path and do compression statistics
from pealauncher}
if (cl<>'') and (length(cl)>3) then launch_cl(cl,jobcode,outname)
else
   if importjobcl_extract(cl,jobcode,outname)=1 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.Image7zPjClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
cl:=mConsoleCreate.Text;
if (cl<>'') and (length(cl)>3) then save_pj(cl)
else
   if importjobcl(cl,jobcode,outname)=1 then save_pj(cl);
end;

procedure TForm_peach.ImageDefaults2Click(Sender: TObject);
var
   P:TProcess;
   cl:utf8string;
begin
udeletefile(confpath+'conf.txt');
default_defaults;
cl:=delimiter+executable_path+'peazip'+EXEEXT+delimiter;
P:=TProcess.Create(nil);
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
Application.Terminate; //don't use Close to skip on form close event
end;

procedure TForm_peach.ImageExploreOutClick(Sender: TObject);
var
   s:utf8string;
begin
if EditOpenOut.Text='' then
   begin
   s:=uextractfilepath(EditOpenIn.Text);
   if s<>'' then EditOpenOut.Text:=s;
   end;
s:=EditOpenOut.Text;
cp_open(s,desk_env);
end;

procedure TForm_peach.ImageListRootClick(Sender: TObject);
var p:tpoint;
begin
p.x:=EditopenIn.left;
p.y:=EditopenIn.top+EditopenIn.height+PanelBarOpen.height;
p:=clienttoscreen(p);
pmjump.PopUp(p.x,p.y);
end;

procedure rsearch;
begin
if fun='UN7Z' then list_un7z_search;
if fun='FILEBROWSER' then
   begin
   listdir(Form_peach.EditOpenIn.Text,true);
   status0:=txt_list_found;
   addtohistory;
   end;
end;

procedure launch_rsearch;
var
   s:utf8string;
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
s:=Form_peach.EditUn7zaFilter1.Text;
if not pInputQuery(txt_search, txt_search_refine, s, true) then exit;
Form_peach.EditUn7zaFilter.Text:=s;
Form_peach.EditUn7zaFilter1.Text:=s;
if Form5.CheckBox1.State=cbChecked then rsearch
else nrsearch;
setfocus_stringgridlist;
end;

procedure launch_nrsearch;
var
   s:utf8string;
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
s:=Form_peach.EditUn7zaFilter1.Text;
if not pInputQuery(txt_nrsearch, txt_search_refine, s, false) then exit;
Form_peach.EditUn7zaFilter.Text:=s;
Form_peach.EditUn7zaFilter1.Text:=s;
nrsearch;
setfocus_stringgridlist;
end;

procedure TForm_peach.mctrlf3Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then launch_nrsearch;
end;

procedure TForm_peach.mctrlf7Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('mostrecent2');
end;

procedure TForm_peach.mctrlf8Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('bookmark2');
end;

procedure TForm_peach.ImageListSearchClick(Sender: TObject);
begin
launch_rsearch;
end;

procedure TForm_peach.ImageNav1Click(Sender: TObject);
begin
showpanel('main');
end;

procedure TForm_peach.ImagePasswordClick(Sender: TObject);
begin
ask_pwkeyfile;
end;

procedure TForm_peach.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
on_exit;
end;

procedure TForm_peach.ImageUn7zPjClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
cl:=mConsoleCreate1.Text;
if (cl<>'') and (length(cl)>3) then save_pj(cl)
else
   if importjobcl_extract(cl,jobcode,outname)=1 then save_pj(cl);
end;

procedure TForm_peach.LabelDefaultColor1Click(Sender: TObject);
begin
color1:=color1d;
apply_theme;
end;

procedure TForm_peach.LabelDefaultColor2Click(Sender: TObject);
begin
color2:=color2d;
apply_theme;
end;

procedure TForm_peach.LabelDefaultColor3Click(Sender: TObject);
begin
color3:=color3d;
apply_theme;
end;

procedure TForm_peach.LabelDefaultColor4Click(Sender: TObject);
begin
color4:=color4d;
apply_theme;
end;

procedure TForm_peach.LabelDefaultColor5Click(Sender: TObject);
begin
color5:=color5d;
apply_theme;
end;

procedure TForm_peach.LabelDefaultColor6Click(Sender: TObject);
begin
rowheight:=rowheightd;
apply_theme;
end;

procedure TForm_peach.LabelDefaultGraphicsFolderClick(Sender: TObject);
begin
graphicsfolder:=graphicsfolderd;
Form_peach.DirectoryEditThemeGraphics.Text:=executable_path+graphicsfolder;
imgloaded:=false;
apply_theme;
end;

procedure TForm_peach.LabelDefaultOpacityClick(Sender: TObject);
begin
SpinEdit1.Value:=opacityd;
SpinEdit1Change(sender);
end;

procedure TForm_peach.LabelUn7zaFilterClick(Sender: TObject);
begin
EditUn7zaFilter.Visible:=not(EditUn7zaFilter.Visible);
cbUn7zaRecursive.Visible:=not(cbUn7zaRecursive.Visible);
end;

procedure TForm_peach.LableListBackClick(Sender: TObject);
begin
if StringGridHistory.RowCount>2 then
   begin
   StringGridHistory.Row:=StringGridHistory.Row-1;
   browsehistory;
   end;
end;

procedure TForm_peach.lConsoleCreate2Click(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
importjobcl(cl,jobcode,outname);
end;

procedure TForm_peach.lConsoleCreate4Click(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
importjobcl_extract(cl,jobcode,outname);
end;

procedure TForm_peach.mAddDirClick(Sender: TObject);
begin
addfolder;
end;

procedure TForm_peach.mAddFileClick(Sender: TObject);
begin
addfile;
end;

procedure TForm_peach.mAddfilesClick(Sender: TObject);
begin
if fun='UN7Z' then addtoarchive_inarchive;
if fun='UNARC' then addtoarchive_inarchive;
end;

procedure TForm_peach.mArchiveSearchAddClick(Sender: TObject);
begin
cp_search(desk_env);
end;

procedure TForm_peach.mAttributesClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.Top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[7,i]=Form_peach.StringGrid1.Cells[7,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[7,i]=Form_peach.StringGrid2.Cells[7,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[6,i]=Form_peach.StringGridList.Cells[6,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mBenchClick(Sender: TObject);
var
   s,cl,jobcode:utf8string;
begin
if MessageDlg(txt_benchmark, mtConfirmation, [mbYes,mbNo], 0)=7 then exit;
s:=fun;
cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+DirectorySeparator+'7z'+DirectorySeparator+'7z'+EXEEXT+delimiter+' b';
fun:='7Z';
fun_status:=fun;
subfun:='bench';
jobcode:=formatdatetime('yyyymmdd_hh.nn.ss.ms_',now)+fun;
launch_cl(cl,jobcode,'');
fun:=s;
fun_status:=fun;
end;

procedure TForm_peach.mCheckClick(Sender: TObject);
var
   P:TProcess;
   cl,in_param:utf8string;
   i:integer;
begin
if Form_peach.OpenDialog2.Execute then
   if Form_peach.OpenDialog2.FileName<>'' then
      begin
      for i:=0 to Form_peach.OpenDialog2.Files.Count-1 do
         in_param:=in_param+delimiter+escapefilename(Form_peach.OpenDialog2.Files[i],desk_env)+delimiter+' ';
      check_files(in_param,cl);
      P:=TProcess.Create(nil);
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
end;

procedure TForm_peach.mClearListClick(Sender: TObject);
begin
clearlist;
end;

procedure TForm_peach.mCompareClick(Sender: TObject);
var
   cl,bin_name,filea,fileb:utf8string;
   P:TProcess;
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.FileName<>'' then
      begin
      filea:=delimiter+escapefilename(Form_peach.OpenDialog3.Filename,desk_env)+delimiter;
      MessageDlg(txt_compare_second+' '+filea, mtInformation, [mbOK], 0);
      if Form_peach.OpenDialog3.Execute then
         if Form_peach.OpenDialog3.FileName<>'' then
            begin
            fileb:=delimiter+escapefilename(Form_peach.OpenDialog3.Filename,desk_env)+delimiter;
            P:=TProcess.Create(nil);
            bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter;
            {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
            cl:=bin_name+' COMPARE '+filea+' '+fileb;
            cl:=utf8toenv(cl);
            P.CommandLine:=cl;
            if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
            P.Execute;
            P.Free;
            end;
      end;
end;

procedure TForm_peach.mDateEOlderClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid1.Cells[6,i])<=strtodatetime(Form_peach.StringGrid1.Cells[6,StringGrid1.Row]) then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid2.Cells[6,i])<=strtodatetime(Form_peach.StringGrid2.Cells[6,StringGrid2.Row]) then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[5,i]<=Form_peach.StringGridList.Cells[5,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mDateEqualClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[6,i]=Form_peach.StringGrid1.Cells[6,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid2.Cells[6,i])=strtodatetime(Form_peach.StringGrid2.Cells[6,StringGrid2.Row]) then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[5,i]=Form_peach.StringGridList.Cells[5,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mDateERecentClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid1.Cells[6,i])>=strtodatetime(Form_peach.StringGrid1.Cells[6,StringGrid1.Row]) then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid2.Cells[6,i])>=strtodatetime(Form_peach.StringGrid2.Cells[6,StringGrid2.Row]) then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[5,i]>=Form_peach.StringGridList.Cells[5,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mDateOlderClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid1.Cells[6,i])<strtodatetime(Form_peach.StringGrid1.Cells[6,StringGrid1.Row]) then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid2.Cells[6,i])<strtodatetime(Form_peach.StringGrid2.Cells[6,StringGrid2.Row]) then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[5,i]<Form_peach.StringGridList.Cells[5,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mDateRecentClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid1.Cells[6,i])>strtodatetime(Form_peach.StringGrid1.Cells[6,StringGrid1.Row]) then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if strtodatetime(Form_peach.StringGrid2.Cells[6,i])>strtodatetime(Form_peach.StringGrid2.Cells[6,StringGrid2.Row]) then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[5,i]>Form_peach.StringGridList.Cells[5,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mDefaultsClick(Sender: TObject);
begin
showpanel('defaults');
end;

procedure deleteselected_frombrowser;
var
   i,j,k:integer;
   s,s1,cl,jobcode,outname:utf8string;
   scheduleclip:array of array [0..13] of utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if fun='FILEBROWSER' then
   begin
   if MessageDlg(txt_delete,mtWarning,[mbYes,mbNo], 0)=6 then
      begin
      s1:=Form_peach.EditOpenIn.Text;
      k:=0;
      setlength(scheduleclip,1);
      for i:=1 to Form_peach.StringGridList.RowCount-1 do
         begin
         if Form_peach.EditOpenIn.Text<>s1 then break;
         if Form_peach.StringGridList.Cells[12,i]='1' then
            begin
            if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
            for j:=0 to Form_peach.StringGridList.ColCount-1 do
               scheduleclip[k,j]:=Form_peach.StringGridList.Cells[j,i];
            setlength(scheduleclip,length(scheduleclip)+1);
            k:=k+1;
            end;
         end;
      for i:=0 to k-1 do
         begin
         s:=scheduleclip[i,8];
         if scheduleclip[i,2]=txt_list_isfolder then cleardir(s,false)
         else clearfile(s);
         end;
      setlength(scheduleclip,0);
      //if Form_peach.EditOpenIn.Text=s1 then navrefresh; //deactivated, refresh is now automatic //MenuItemRefresh only if browser is still in the same dir
      end;
   exit;
   end;
if test_shouldnotbechanged=1 then
   begin
   MessageDlg(txt_add_error, mtWarning, [mbOK], 0);
   exit;
   end;
if fun='UN7Z' then
   begin
   if testencrypted<0 then exit;
      if MessageDlg(txt_delete,mtWarning,[mbYes,mbNo], 0)=6 then
         if compose_un7z_cl(cl,jobcode,outname,false,'delete','selected')=0 then
            if execute_cl(cl,'delete')=0 then list_un7z('browse')
            else MessageDlg(txt_add_error, mtWarning, [mbOK], 0);
   end;
if fun='UNARC' then
   begin
   //if StringGridList.Rowcount=2 then exit; //prevent to delete last object into archive (empty archive gets deleted by freearc)
   //if testencrypted<0 then exit;
      if MessageDlg(txt_delete,mtWarning,[mbYes,mbNo], 0)=6 then
         if compose_unarc_cl(cl,jobcode,outname,false,'delete','selected')=0 then
            if execute_cl(cl,'delete')=0 then browse_arc('flat')
            else MessageDlg(txt_add_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.MenuItem1Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then removeselected
else removeselected_ext;
end;

procedure TForm_peach.MenuItem3Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then clearlist
else clearlist_ext;
end;

procedure TForm_peach.MenuItem6Click(Sender: TObject);
var
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then sg:=StringGrid1
else sg:=StringGrid2;
if sg.Rowcount=1 then exit;
if sg.Cells[2,sg.Row]=txt_list_isfolder then
   cp_open(sg.Cells[8,sg.Row],desk_env) //open folder
else //go to object's path
   {$IFDEF MSWINDOWS}
   if checkfiledirname(sg.Cells[8,sg.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+sg.Cells[8,sg.Row], mtWarning, [mbOK], 0); exit; end;
   winexplorepath(sg.Cells[8,sg.Row]);
   {$ELSE}
   cp_open(uextractfilepath(sg.Cells[8,sg.Row]),desk_env);
   {$ENDIF}
end;

procedure TForm_peach.MenuItemBlogClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://www.google.com/blogsearch?q='+s,desk_env);
end;

procedure TForm_peach.MenuItemERAssociatedClick(Sender: TObject);
var
   i:integer;
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=StringGridList.Row;
if StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'extandrun')=0 then execute_obj(3,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(3,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.MenuItemERCustomClick(Sender: TObject);
var
   i:integer;
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=StringGridList.Row;
if StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'extandrun')=0 then openw_obj(outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(3,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.MenuItemERPeaZipClick(Sender: TObject);
var
   i:integer;
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=StringGridList.Row;
if StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'extandrun')=0 then execute_obj(2,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(2,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.MenuItemImgClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://images.google.com/images?q='+s,desk_env);
end;

procedure TForm_peach.MenuItemOpen_backClick(Sender: TObject);
begin
if StringGridHistory.RowCount>2 then
   begin
   StringGridHistory.Row:=StringGridHistory.Row-1;
   browsehistory;
   end;
end;

procedure TForm_peach.MenuItemOpen_detete_selectedClick(Sender: TObject);
begin
deleteselected_frombrowser;
end;

procedure TForm_peach.MenuItemOpen_ext_allClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbUnChecked;
list_toextractor('all');
end;

procedure extractdisplayed;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if (fun='UNARC') then
   if compose_unarc_cl(cl,jobcode,outname,true,'neutral','displayed')=0 then launch_cl(cl,jobcode,outname);
if (fun='UN7Z') then
   if compose_un7z_cl(cl,jobcode,outname,true,'neutral','displayed')=0 then launch_cl(cl,jobcode,outname);
end;

procedure extractselected;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if (fun='UNARC') then
   if compose_unarc_cl(cl,jobcode,outname,true,'neutral','selected')=0 then launch_cl(cl,jobcode,outname);
if (fun='UN7Z') then
   if compose_un7z_cl(cl,jobcode,outname,true,'neutral','selected')=0 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.MenuItemOpen_ext_displayedClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbUnChecked;
list_toextractor('displayed');
end;

procedure TForm_peach.MenuItemOpen_ext_selectedClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbUnChecked;
list_toextractor('selected');
end;

procedure TForm_peach.MenuItemOpen_ExtractAllToClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbChecked;
list_toextractor('all');
//extractallto('to','');
end;

procedure extractdisplayedto;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if set_extractto_path<>0 then exit;
if (fun='UNARC') then
   if compose_unarc_cl(cl,jobcode,outname,true,'ext','displayed')=0 then launch_cl(cl,jobcode,outname);
if (fun='UN7Z') then
   if compose_un7z_cl(cl,jobcode,outname,true,'ext','displayed')=0 then launch_cl(cl,jobcode,outname);
end;

procedure extractselectedto;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if set_extractto_path<>0 then exit;
if (fun='UNARC') then
   if compose_unarc_cl(cl,jobcode,outname,true,'ext','selected')=0 then launch_cl(cl,jobcode,outname);
if (fun='UN7Z') then
   if compose_un7z_cl(cl,jobcode,outname,true,'ext','selected')=0 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.MenuItemOpen_ExtractDisplayedToClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbChecked;
list_toextractor('displayed');
end;

procedure TForm_peach.MenuItemOpen_ExtractSelectedToClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbChecked;
list_toextractor('selected');
end;

procedure TForm_peach.MenuItemOpen_FlatClick(Sender: TObject);
begin
toggle_view;
end;

procedure TForm_peach.MenuItemOpen_fwdClick(Sender: TObject);
begin
if StringGridHistory.Row<StringGridHistory.RowCount-1 then
   begin
   StringGridHistory.Row:=StringGridHistory.Row+1;
   browsehistory;
   end;
end;

procedure TForm_peach.MenuItemOpen_info_allClick(Sender: TObject);
begin
//if StringGridList.Cells[1,1]='' then exit;
archive_infoall;
end;

procedure TForm_peach.MenuItemOpen_info_displayedClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'info','displayed')=0 then launch_cl(cl,jobcode,outname);
//not featured in ARC
end;

procedure TForm_peach.MenuItemOpen_info_selectedClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'info','selected')=0 then launch_cl(cl,jobcode,outname);
//not featured in ARC
end;

procedure TForm_peach.MenuItemOpen_jumpClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if ListRow=0 then exit;
if fun='FILEBROWSER' then
   begin
   if StringGridList.Cells[2,StringGridList.Row]=txt_list_isfolder then //open folder
      s:=StringGridList.Cells[8,StringGridList.Row]
   else //go to object's path
      s:=uextractfilepath(StringGridList.Cells[8,StringGridList.Row]);
   listdir(s,false);
   addtohistory;
   end;
if fun='UN7Z' then
   begin
   if StringGridList.Cells[2,StringGridList.Row]=txt_list_isfolder then //open folder
      begin
      EditUn7zaFilter.Text:=StringGridList.Cells[8,StringGridList.Row]+directoryseparator+'*';
      EditUn7zaFilterExclude.Text:=StringGridList.Cells[8,StringGridList.Row]+directoryseparator+'*'+directoryseparator+'*';
      list_un7z_do;
      end
   else //go to object's path
      begin
      EditUn7zaFilter.Text:=uextractfilepath(StringGridList.Cells[8,StringGridList.Row])+'*';
      EditUn7zaFilterExclude.Text:=uextractfilepath(StringGridList.Cells[8,StringGridList.Row])+'*'+directoryseparator+'*';
      list_un7z_do;
      end;
   end;
end;

procedure TForm_peach.MenuItemOpen_list_allClick(Sender: TObject);
begin
archive_listall;
end;

procedure listdisplayed;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'list','displayed')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'list','displayed')=0 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.MenuItemOpen_list_displayedClick(Sender: TObject);
begin
listdisplayed;
end;

procedure listselected;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'list','selected')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'list','selected')=0 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.MenuItemOpen_list_selectedClick(Sender: TObject);
begin
listselected;
end;

procedure TForm_peach.MenuItemOpen_rootClick(Sender: TObject);
begin
list_root;
end;

procedure TForm_peach.MenuItemOpen_SearchAddClick(Sender: TObject);
begin
cp_search(desk_env);
end;

procedure TForm_peach.MenuItemOpen_test_allClick(Sender: TObject);
begin
archive_testall;
end;

procedure testdisplayed;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'test','displayed')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'test','displayed')=0 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.MenuItemOpen_test_displayedClick(Sender: TObject);
begin
testdisplayed;
end;

procedure testselected;
var
   cl,jobcode,outname:utf8string;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'test','selected')=0 then launch_cl(cl,jobcode,outname);
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'test','selected')=0 then launch_cl(cl,jobcode,outname);
end;

procedure TForm_peach.MenuItemOpen_test_selectedClick(Sender: TObject);
begin
testselected;
end;

procedure TForm_peach.MenuItemOpen_upClick(Sender: TObject);
begin
navgoup;
end;

procedure TForm_peach.MenuItemPreAssociatedClick(Sender: TObject);
var
   i:integer;
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=StringGridList.Row;
if StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(3,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(3,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.MenuItemPreCustomClick(Sender: TObject);
var
   i:integer;
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=StringGridList.Row;
if StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then openw_obj(outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then openw_obj(outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure TForm_peach.MenuItemPrePeaZipClick(Sender: TObject);
var
   i:integer;
   cl,jobcode,outname:utf8string;
begin
if StringGridList.Cells[1,1]='' then exit;
if testencrypted<0 then exit;
i:=StringGridList.Row;
if StringGridList.Cells[12,i]<>'1' then exit;
if i>0 then
   begin
   if fun='UN7Z' then
      if compose_un7z_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(2,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   if fun='UNARC' then
      if compose_unarc_cl(cl,jobcode,outname,true,'preview','single')=0 then
         if execute_cl(cl,'preview')=0 then execute_obj(2,outname)
         else
            MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
   end;
end;

procedure checkselected;
var
   cl,in_param:utf8string;
   P:TProcess;
   i:integer;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
if sg.Row=0 then exit;
for i:=1 to sg.Rowcount-1 do
   if sg.Cells[11,i]='1' then in_param:=in_param+delimiter+escapefilename(sg.Cells[8,i],desk_env)+delimiter+' ';
check_files(in_param,cl);
P:=TProcess.Create(nil);
{$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
end;

procedure checkselected_list;
var
   cl,in_param:utf8string;
   P:TProcess;
   i:integer;
begin
   if Form_peach.StringGridList.Row=0 then exit;
   for i:=1 to Form_peach.StringGridList.Rowcount-1 do
      if Form_peach.StringGridList.Cells[12,i]='1' then
         begin
         if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
         in_param:=in_param+delimiter+escapefilename(Form_peach.StringGridList.Cells[8,i],desk_env)+delimiter+' ';
         end;
   check_files(in_param,cl);
   P:=TProcess.Create(nil);
   {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
end;

procedure TForm_peach.MenuItemToolCheckClick(Sender: TObject);
begin
checkselected;
end;

procedure compareselected;
var
   cl,bin_name,filea,fileb:utf8string;
   P:TProcess;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
if sg.Row=0 then exit;
if sg.Cells[8,sg.Row]='' then exit;
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.FileName<>'' then
      begin
      filea:=delimiter+escapefilename(sg.Cells[8,sg.Row],desk_env)+delimiter;
      fileb:=delimiter+escapefilename(Form_peach.OpenDialog3.Filename,desk_env)+delimiter;
      P:=TProcess.Create(nil);
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter;
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      cl:=bin_name+' COMPARE '+filea+' '+fileb;
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
end;

procedure compareselected_list;
var
   cl,bin_name,filea,fileb:utf8string;
   P:TProcess;
begin
if Form_peach.StringGridList.Row=0 then exit;
if Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]='' then exit;
if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.FileName<>'' then
      begin
      filea:=delimiter+escapefilename(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row],desk_env)+delimiter;
      fileb:=delimiter+escapefilename(Form_peach.OpenDialog3.Filename,desk_env)+delimiter;
      P:=TProcess.Create(nil);
      bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter;
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      cl:=bin_name+' COMPARE '+filea+' '+fileb;
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
end;

procedure TForm_peach.MenuItemToolCompareClick(Sender: TObject);
begin
compareselected;
end;

procedure TForm_peach.MenuItemToolJoinClick(Sender: TObject);
var
   fname:UTF8string;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
if sg.Row=0 then exit;
fname:=sg.Cells[8,sg.Row];
direct_join(fname);
end;

procedure splitselected;
var
   fname:UTF8string;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
if sg.Row=0 then exit;
fname:=sg.Cells[8,sg.Row];
direct_split(fname);
end;

procedure TForm_peach.MenuItemToolSplitClick(Sender: TObject);
begin
splitselected;
end;

procedure eraseselected_list;
var
   P:TProcess;
   cl,bin_name,in_param,eraselevel,s:utf8string;
   i:integer;
begin
if Form_peach.StringGridList.Row=0 then exit;
if Form_peach.StringGridList.Cells[1,1]='' then exit;
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if Form_peach.visible=true then
   if MessageDlg(txt_wipe,mtWarning,[mbYes,mbNo], 0)=6 then
   begin
   P:=TProcess.Create(nil);
   bin_name:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter;
   for i:=1 to Form_peach.StringGridList.Rowcount-1 do
      if Form_peach.StringGridList.Cells[12,i]='1' then
         begin
         if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
         in_param:=in_param+delimiter+escapefilename(Form_peach.StringGridList.Cells[8,i],desk_env)+delimiter+' ';
         end;
   case erasepasses of
      0: eraselevel:='FAST';
      1: eraselevel:='MEDIUM';
      2: eraselevel:='SLOW';
      3: eraselevel:='VERY_SLOW';
      end;
   {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
   cl:=bin_name+' WIPE '+eraselevel+' '+in_param;
   cl:=utf8toenv(cl);
   P.CommandLine:=cl;
   //s:=Form_peach.EditOpenIn.Text;
   if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
   P.Execute;
   P.Free;
   //process_monitor_and_free(P,s);
   end;
end;

procedure TForm_peach.pmsecuredeleteClick(Sender: TObject);
begin
eraseselected_list;
end;

procedure TForm_peach.MenuItemUsenetClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://groups.google.com/groups?q='+s,desk_env);
end;

procedure TForm_peach.MenuItemWebClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://www.google.com/search?q='+s,desk_env);
end;

procedure TForm_peach.MenuItemWikipediaClick(Sender: TObject);
var
   s:utf8string;
begin
//if StringGridList.Cells[1,1]='' then exit;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if convert2webname(s)=0 then cp_open('http://en.wikipedia.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.mFlatClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if (fun='UN7Z') or (fun='FILEBROWSER') then toggle_view;
end;

procedure TForm_peach.mInvertSelectionClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[11,i]='0' then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[11,i]='0' then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[12,i]='0' then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mJoinFilesClick(Sender: TObject);
var
   fname:UTF8string;
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.Filename<>'' then
      begin
      fname:=Form_peach.OpenDialog3.Filename;
      direct_join(fname);
      end;
end;

procedure TForm_peach.mKeyFileClick(Sender: TObject);
begin
get_fingerprint (fingerprint,false);
Whirl_Init(ment);
Whirl_Init(kent);
Whirl_Init(fent);
Form4.Showmodal;
end;

procedure TForm_peach.mOptionsClick(Sender: TObject);
begin
showpanel('options');
end;

procedure TForm_peach.mPEAClick(Sender: TObject);
begin
showpanel('archive');
showpanel_trick;
end;

procedure TForm_peach.mPEAhelpClick(Sender: TObject);
begin
cp_open('http://peazip.sourceforge.net/peazip-help.html',desk_env);
end;

procedure TForm_peach.mPeaSiteClick(Sender: TObject);
begin
cp_open('http://peazip.sourceforge.net/index.html',desk_env);
end;

procedure TForm_peach.mPSELargerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then exit;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[11,i]>=Form_peach.StringGridList.Cells[11,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mPSEqualClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then exit;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[11,i]=Form_peach.StringGridList.Cells[11,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mPSESmallerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then exit;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[11,i]<=Form_peach.StringGridList.Cells[11,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mPSLargerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then exit;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[11,i]>Form_peach.StringGridList.Cells[11,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mPSSmallerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then exit;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[11,i]<Form_peach.StringGridList.Cells[11,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mQuitClick(Sender: TObject);
begin
on_exit;
Close;
end;

procedure TForm_peach.mRecent1Click(Sender: TObject);
begin
open_archive_fromname(mRecent1.Caption);
end;

procedure TForm_peach.mRecent2Click(Sender: TObject);
begin
open_archive_fromname(mRecent2.Caption);
end;

procedure TForm_peach.mRecent3Click(Sender: TObject);
begin
open_archive_fromname(mRecent3.Caption);
end;

procedure TForm_peach.mRecent4Click(Sender: TObject);
begin
open_archive_fromname(mRecent4.Caption);
end;

procedure TForm_peach.mRecent5Click(Sender: TObject);
begin
open_archive_fromname(mRecent5.Caption);
end;

procedure TForm_peach.mRecent6Click(Sender: TObject);
begin
open_archive_fromname(mRecent6.Caption);
end;

procedure TForm_peach.mRecent7Click(Sender: TObject);
begin
open_archive_fromname(mRecent7.Caption);
end;

procedure TForm_peach.mRecent8Click(Sender: TObject);
begin
open_archive_fromname(mRecent8.Caption);
end;

procedure TForm_peach.mRFJClick(Sender: TObject);
begin
showpanel('join');
end;

procedure TForm_peach.mRFSClick(Sender: TObject);
begin
showpanel('split');
end;

procedure TForm_peach.mSelectAllClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      Form_peach.StringGrid1.Cells[11,i]:='1';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   if StringGrid2.Visible=true then
      begin
      for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
         Form_peach.StringGrid2.Cells[11,i]:='1';
      Form_peach.StringGrid2.Repaint;
      end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         Form_peach.StringGridList.Cells[12,i]:='1';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mSizeELargerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[10,i]>=Form_peach.StringGrid1.Cells[10,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[10,i]>=Form_peach.StringGrid2.Cells[10,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[10,i]>=Form_peach.StringGridList.Cells[10,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mSizeEqualClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[11,i]=Form_peach.StringGrid1.Cells[11,StringGrid1.Row] then Form_peach.StringGrid1.Cells[12,i]:='1'
      else Form_peach.StringGrid1.Cells[12,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[10,i]=Form_peach.StringGrid2.Cells[10,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[10,i]=Form_peach.StringGridList.Cells[10,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mSizeESmallerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[10,i]<=Form_peach.StringGrid1.Cells[10,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[10,i]<=Form_peach.StringGrid2.Cells[10,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[10,i]<=Form_peach.StringGridList.Cells[10,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mSizeLargerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[10,i]>Form_peach.StringGrid1.Cells[10,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[10,i]>Form_peach.StringGrid2.Cells[10,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[10,i]>Form_peach.StringGridList.Cells[10,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mSizeSmallerClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[10,i]<Form_peach.StringGrid1.Cells[10,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[10,i]<Form_peach.StringGrid2.Cells[10,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[10,i]<Form_peach.StringGridList.Cells[10,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure sort_az_stringgrid1(c:integer);
begin
Form_peach.StringGrid1.SortColRow(true,c);
az1:=true;
Form_peach.StringGrid1.Repaint;
end;

procedure sort_za_stringgrid1;
var
   i,rc:integer;
begin
Form_peach.StringGridTmp.ColCount:=Form_peach.StringGrid1.ColCount;
Form_peach.StringGridTmp.RowCount:=Form_peach.StringGrid1.RowCount;
rc:=Form_peach.StringGrid1.Rowcount;
for i:=1 to Form_peach.StringGrid1.Rowcount-1 do Form_peach.StringGridTmp.Rows[i]:=Form_peach.StringGrid1.Rows[rc-i];
for i:=1 to Form_peach.StringGrid1.Rowcount-1 do Form_peach.StringGrid1.Rows[i]:=Form_peach.StringGridTmp.Rows[i];
az1:=false;
Form_peach.StringGridTmp.Clear;
Form_peach.StringGrid1.Repaint;
end;

procedure sort_az_stringgrid2(c:integer);
begin
Form_peach.StringGrid2.SortColRow(true,c);
az2:=true;
Form_peach.StringGrid2.Repaint;
end;

procedure sort_za_stringgrid2;
var
   i,rc:integer;
begin
Form_peach.StringGridTmp.ColCount:=Form_peach.StringGrid2.ColCount;
Form_peach.StringGridTmp.RowCount:=Form_peach.StringGrid2.RowCount;
rc:=Form_peach.StringGrid2.Rowcount;
for i:=1 to Form_peach.StringGrid2.Rowcount-1 do Form_peach.StringGridTmp.Rows[i]:=Form_peach.StringGrid2.Rows[rc-i];
for i:=1 to Form_peach.StringGrid2.Rowcount-1 do Form_peach.StringGrid2.Rows[i]:=Form_peach.StringGridTmp.Rows[i];
az2:=false;
Form_peach.StringGridTmp.Clear;
Form_peach.StringGrid2.Repaint;
end;

procedure sort_az_stringgridbookmarks(c:integer);
begin
Form_peach.StringGridBookmarks.SortColRow(true,c);
azbook:=true;
Form_peach.StringGridBookmarks.Repaint;
end;

procedure sort_za_stringgridbookmarks;
var
   i,rc:integer;
begin
Form_peach.StringGridTmp.ColCount:=Form_peach.StringGridBookmarks.ColCount;
Form_peach.StringGridTmp.RowCount:=Form_peach.StringGridBookmarks.RowCount;
rc:=Form_peach.StringGridBookmarks.Rowcount;
for i:=1 to Form_peach.StringGridBookmarks.Rowcount-1 do Form_peach.StringGridTmp.Rows[i]:=Form_peach.StringGridBookmarks.Rows[rc-i];
for i:=1 to Form_peach.StringGridBookmarks.Rowcount-1 do Form_peach.StringGridBookmarks.Rows[i]:=Form_peach.StringGridTmp.Rows[i];
azbook:=false;
Form_peach.StringGridTmp.Clear;
Form_peach.StringGridBookmarks.Repaint;
end;

procedure updatelayout;
var
   i,Column:integer;
   objectname:utf8string;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
if sg.RowCount=1 then exit;
sg.Cursor:=crHourGlass;
{if Form_peach.PanelArchiveMain.Visible=true then
   begin
   Column:=9;
   if Column=listsortcol1 then
      if az1=true then sort_za_stringgrid1
      else sort_az_stringgrid1(Column)
   else sort_az_stringgrid1(Column);
   listsortcol1:=Column;
   end
else
   begin
   Column:=9;
   if Column=listsortcol2 then
      if az2=true then sort_za_stringgrid2
      else sort_az_stringgrid2(Column)
   else sort_az_stringgrid2(Column);
   listsortcol2:=Column;
   end;}
for i:=1 to sg.rowcount-1 do
   begin
   objectname:=sg.Cells[8,1];
   sg.DeleteColRow(false,1);
   if ufilegetattr(objectname) > 0 then
      if ufilegetattr(objectname) and faDirectory =0 then addfilestr(sg,objectname)
      else addfolderstr(sg,objectname)
   else msg_not_accessible_list(objectname);
   end;
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true)
else updatecontent_ext;
sg.Cursor:=crDefault;
end;

procedure TForm_peach.MenuItemRefreshClick(Sender: TObject);
begin
updatelayout;
end;

procedure TForm_peach.mf2Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('desktop');
end;

procedure TForm_peach.mf3Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then launch_rsearch;
end;

procedure TForm_peach.mf4Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then
   if (fun='UN7Z') or (fun='FILEBROWSER') then navgoup;
end;

procedure TForm_peach.mf7Click(Sender: TObject);
begin
if {$IFDEF MSWINDOWS}PanelOpen.Visible=true{$ELSE}PanelOpen.Top<>10000{$ENDIF} then jumpto('mostrecent');
end;

procedure TForm_peach.mhexpClick(Sender: TObject);
var
   cl,in_param:utf8string;
   P:TProcess;
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.FileName<>'' then
      begin
      in_param:=delimiter+escapefilename(Form_peach.OpenDialog3.Filename,desk_env)+delimiter;
      cl:=delimiter+escapefilename(executable_path,desk_env)+'res'+directoryseparator+'pea'+EXEEXT+delimiter+' HEXPREVIEW '+in_param;
      P:=TProcess.Create(nil);
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
end;

procedure TForm_peach.mpathexploreClick(Sender: TObject);
var
   s:utf8string;
begin
s:=patheditor.Text;
if s='' then exit;
if s=txt_mypc then exit;
if s[1]='<' then exit;
s:=uextractfilepath(s);
cp_open(s,desk_env);
end;

procedure TForm_peach.mpeahelpofflineClick(Sender: TObject);
begin
cp_open(executable_path+'peazip_help.pdf',desk_env);
end;

procedure TForm_peach.mRecent9Click(Sender: TObject);
begin
showpanel('open');
if stringgridhistory.visible=false then showbar('history');
navrefresh;
end;

procedure TForm_peach.mSortBySelectionClick(Sender: TObject);
var Column:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   Column:=11;
   if Column=listsortcol1 then
      if az1=true then sort_za_stringgrid1
      else sort_az_stringgrid1(Column)
   else sort_az_stringgrid1(Column);
   listsortcol1:=Column;
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   Column:=11;
   if Column=listsortcol2 then
      if az2=true then sort_za_stringgrid2
      else sort_az_stringgrid2(Column)
   else sort_az_stringgrid2(Column);
   listsortcol2:=Column;
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      Column:=12;
      if Column=listsortcol then
         if az=true then sort_za_stringgridlist
         else sort_az_stringgridlist(Column)
      else sort_az_stringgridlist(Column);
      listsortcol:=12;
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mSplitFileClick(Sender: TObject);
var
   fname:UTF8string;
begin
if Form_peach.OpenDialog3.Execute then
   if Form_peach.OpenDialog3.Filename<>'' then
      begin
      fname:=Form_peach.OpenDialog3.Filename;
      direct_split(fname);
      end;
end;

procedure TForm_peach.mType7zClick(Sender: TObject);
begin
cbType.ItemIndex:=0;
archive_type_select('7Z');
end;

procedure TForm_peach.mTypeARCClick(Sender: TObject);
begin
cbType.ItemIndex:=1;
archive_type_select('ARC');
end;

procedure TForm_peach.mTypebzip2Click(Sender: TObject);
begin
cbType.ItemIndex:=2;
archive_type_select('BZip2');
end;

procedure TForm_peach.mTypeClick(Sender: TObject);
var
   i:integer;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
      if Form_peach.StringGrid1.Cells[2,i]=Form_peach.StringGrid1.Cells[2,StringGrid1.Row] then Form_peach.StringGrid1.Cells[11,i]:='1'
      else Form_peach.StringGrid1.Cells[11,i]:='0';
   Form_peach.StringGrid1.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelExtract.Visible=true{$ELSE}if PanelExtract.top=0{$ENDIF} then
   begin
   for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
      if Form_peach.StringGrid2.Cells[2,i]=Form_peach.StringGrid2.Cells[2,StringGrid2.Row] then Form_peach.StringGrid2.Cells[11,i]:='1'
      else Form_peach.StringGrid2.Cells[11,i]:='0';
   Form_peach.StringGrid2.Repaint;
   end;
{$IFDEF MSWINDOWS}if PanelOpen.Visible=true{$ELSE}if PanelOpen.Top=0{$ENDIF} then
   if StringGridList.Visible=true then
      begin
      for i:=1 to Form_peach.StringGridList.Rowcount-1 do
         if Form_peach.StringGridList.Cells[2,i]=Form_peach.StringGridList.Cells[2,StringGridList.Row] then Form_peach.StringGridList.Cells[12,i]:='1'
         else Form_peach.StringGridList.Cells[12,i]:='0';
      Form_peach.StringGridList.Repaint;
      end;
end;

procedure TForm_peach.mTypeCustomClick(Sender: TObject);
begin
cbType.ItemIndex:=3;
archive_type_select(txt_custom);
end;

procedure TForm_peach.mTypegzClick(Sender: TObject);
begin
cbType.ItemIndex:=4;
archive_type_select(STR_GZIP);
end;

procedure TForm_peach.mTypePAQClick(Sender: TObject);
begin
cbType.ItemIndex:=13;
archive_type_select(STR_ZPAQ);
end;

procedure TForm_peach.mTypePEAClick(Sender: TObject);
begin
cbType.ItemIndex:=5;
archive_type_select(STR_PEA);
end;

procedure TForm_peach.mTypeQUADClick(Sender: TObject);
begin
cbType.ItemIndex:=6;
archive_type_select(STR_QUAD);
end;

procedure TForm_peach.mTypeSplitClick(Sender: TObject);
begin
cbType.ItemIndex:=9;
archive_type_select(txt_split);
end;

procedure TForm_peach.mTypeTARClick(Sender: TObject);
begin
cbType.ItemIndex:=10;
archive_type_select(STR_TAR);
end;

procedure TForm_peach.mTypeUPXClick(Sender: TObject);
begin
cbType.ItemIndex:=11;
archive_type_select(STR_UPX);
end;

procedure TForm_peach.mTypeZIPClick(Sender: TObject);
begin
cbType.ItemIndex:=12;
archive_type_select(STR_ZIP);
end;

procedure peazip_browse;
var
   s:utf8string;
begin
fun:='FILEBROWSER';
fun_status:=fun;
if ufileexists((Form_peach.EditOpenIn.Text)) then Form_peach.SelectDirectoryDialog1.InitialDir:=uextractfilepath(Form_peach.EditOpenIn.Text)
else Form_peach.SelectDirectoryDialog1.InitialDir:=Form_peach.EditOpenIn.Text;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      Form_peach.EditOpenIn.Text:=Form_peach.SelectDirectoryDialog1.FileName
   else exit
else exit;
s:=Form_peach.EditOpenIn.Text;
if s<>'' then if s[length(s)]<>directoryseparator then s:=s+directoryseparator;
Form_peach.EditUn7zaFilter.Text:='*';
showpanel('open');
listdir(s,false);
addtohistory;
end;

procedure TForm_peach.mUnPEAbrowseClick(Sender: TObject); //custom version of openpath procedure, adding showpanel('open') in proper place
begin
peazip_browse;
end;

procedure TForm_peach.mUnPEAClick(Sender: TObject);
begin
open_archive;
end;

procedure open_associated;
var
   i:integer;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if Form_peach.PanelArchiveMain.Visible=true{$ELSE}if Form_peach.PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
i:=sg.Row;
if i>0 then cp_open(sg.Cells[8,i],desk_env);
end;

procedure TForm_peach.openwithassociatedClick(Sender: TObject);
begin
open_associated;
end;

procedure TForm_peach.openwithcustomClick(Sender: TObject);
var
   i:integer;
   wname:widestring;
   s:utf8string;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then sg:=Form_peach.StringGrid1
else sg:=Form_peach.StringGrid2;
i:=sg.Row;
if i>0 then
{$IFDEF MSWINDOWS}
wname:=utf8decode(sg.Cells[8,i]);
s:=sg.Cells[8,i];
if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); exit; end;
shellexecutew(Form_peach.handle,PWideChar('open'),PWideChar('RUNDLL32.EXE'),PWideChar('shell32.dll,OpenAs_RunDLL '+wname),PWideChar (''), SW_SHOWNORMAL);
{$ENDIF}
end;

procedure TForm_peach.openwithpeazipClick(Sender: TObject);
var
   i,j:integer;
   P:TProcess;
   sg:TStringGrid;
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then sg:=StringGrid1
else sg:=StringGrid2;
i:=sg.Row;
if i>0 then
   begin
   j:=testinput(sg.Cells[8,i],true);
   if (j>=0) then //and (j<>1000) then
      begin
      P:=TProcess.Create(nil);
      {$IFDEF MSWINDOWS}P.Options := [poNoConsole];{$ELSE}P.Options := [poNoConsole, poWaitOnExit];{$ENDIF}
      P.CommandLine:=utf8toenv(delimiter+escapefilename(executable_path,desk_env)+'peazip'+EXEEXT+delimiter+' -ext2open '+delimiter+escapefilename(sg.Cells[8,i],desk_env)+delimiter);
      if Form_peach.Visible=true then Application.ProcessMessages;
      if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
      P.Execute;
      P.Free;
      end;
   end;
end;

procedure clickcustom(s:utf8string);
var
   s1:utf8string;
begin
s1:=s;
{$IFDEF MSWINDOWS}if Form_peach.panelarchivemain.visible=true{$ELSE}if Form_peach.panelarchivemain.top=0{$ENDIF} then
   begin
   Form_peach.EditNameCustom.Text:=s1;
   s1:=uextractfilename(s1);
   cutextension(s1);//default: extension = name of the RadioGroupQuad
   Form_peach.EditExtCustom.Text:=s1;
   end;
{$IFDEF MSWINDOWS}if Form_peach.panelextract.visible=true{$ELSE}if Form_peach.panelextract.top=0{$ENDIF} then
   begin
   Form_peach.EditNameCustom1.Text:=s1;
   end;
addtocustomhistory(s);
end;

procedure TForm_peach.pcustom1Click(Sender: TObject);
begin
clickcustom(pcustom1.caption);
end;

procedure TForm_peach.pcustom2Click(Sender: TObject);
begin
clickcustom(pcustom2.caption);
end;

procedure TForm_peach.pcustom3Click(Sender: TObject);
begin
clickcustom(pcustom3.caption);
end;

procedure TForm_peach.pcustom4Click(Sender: TObject);
begin
clickcustom(pcustom4.caption);
end;

procedure TForm_peach.pcustom5Click(Sender: TObject);
begin
clickcustom(pcustom5.caption);
end;

procedure TForm_peach.pcustom6Click(Sender: TObject);
begin
clickcustom(pcustom6.caption);
end;

procedure TForm_peach.pcustom7Click(Sender: TObject);
begin
clickcustom(pcustom7.caption);
end;

procedure TForm_peach.pcustom8Click(Sender: TObject);
begin
clickcustom(pcustom8.caption);
end;

procedure TForm_peach.pmAddFileClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then addfile
else addfile_ext;
end;

procedure TForm_peach.pmAddFolderClick(Sender: TObject);
begin
addfolder;
end;

procedure TForm_peach.pmArchiveBlogClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.RowCount=1 then exit;
s:=Form_peach.StringGrid1.Cells[1,Form_peach.StringGrid1.Row];
if convert2webname(s)=0 then cp_open('http://www.google.com/blogsearch?q='+s,desk_env);
end;

procedure TForm_peach.pmArchiveImageClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.RowCount=1 then exit;
s:=Form_peach.StringGrid1.Cells[1,Form_peach.StringGrid1.Row];
if convert2webname(s)=0 then cp_open('http://images.google.com/images?q='+s,desk_env);
end;

procedure TForm_peach.pmArchiveUsenetClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.RowCount=1 then exit;
s:=Form_peach.StringGrid1.Cells[1,Form_peach.StringGrid1.Row];
if convert2webname(s)=0 then cp_open('http://groups.google.com/groups?q='+s,desk_env);
end;

procedure TForm_peach.pmArchiveWikipediaClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.RowCount=1 then exit;
s:=Form_peach.StringGrid1.Cells[1,Form_peach.StringGrid1.Row];
if convert2webname(s)=0 then cp_open('http://en.wikipedia.org/wiki/Special:Search?search='+s+'&fulltext=Search',desk_env);
end;

procedure TForm_peach.pmJump1Click(Sender: TObject);
begin
jumpto('root');
end;

procedure TForm_peach.pmJump3Click(Sender: TObject);
begin
jumpto('home');
end;

procedure TForm_peach.pmJumpDocumentsClick(Sender: TObject);
begin
jumpto('usrdocuments');
end;

procedure TForm_peach.pmmorefun_infoClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if fun='FILEBROWSER' then filebrowser_funall('info')
else archive_infoall;
setfocus_stringgridlist;
end;

procedure TForm_peach.pmmorefun_listClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if fun='FILEBROWSER' then filebrowser_funall('list')
else archive_listall;
setfocus_stringgridlist;
end;

procedure TForm_peach.pmmorefun_testClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if fun='FILEBROWSER' then filebrowser_funall('test')
else archive_testall;
setfocus_stringgridlist;
end;

procedure TForm_peach.pmJump2Click(Sender: TObject);
begin
jumpto('desktop');
end;

procedure TForm_peach.pmJumpArchiveClick(Sender: TObject);
begin
open_archive;
end;

procedure TForm_peach.pmJumpPathClick(Sender: TObject);
begin
openpath;
end;

procedure TForm_peach.pmLoadListClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then loadlist(StringGrid1)
else loadlist(StringGrid2);
end;

procedure display_cl(var cl,jobcode,outname:utf8string);
begin
with form_peach do
   begin
   mConsoleCreate1.Clear;
   mConsoleCreate1.Append(cl);
   lConsoleCreate_jobcode1.Caption:=jobcode;
   lConsoleCreate_outname1.Caption:=outname;
   end;
end;

procedure TForm_peach.pmOpenPJallClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'neutral','all')<>0 then exit;
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'neutral','all')<>0 then exit;
display_cl(cl,jobcode,outname);
end;

procedure TForm_peach.pmOpenPJdisplayedClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'neutral','displayed')<>0 then exit;
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'neutral','displayed')<>0 then exit;
display_cl(cl,jobcode,outname);
end;

procedure TForm_peach.pmOpenPJselectedClick(Sender: TObject);
var
   cl,jobcode,outname:utf8string;
begin
if fun='UN7Z' then
   if compose_un7z_cl(cl,jobcode,outname,false,'neutral','selected')<>0 then exit;
if fun='UNARC' then
   if compose_unarc_cl(cl,jobcode,outname,false,'neutral','selected')<>0 then exit;
display_cl(cl,jobcode,outname);
end;

procedure TForm_peach.pmSaveListClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if PanelArchiveMain.Visible=true{$ELSE}if PanelArchiveMain.top=0{$ENDIF} then savelist(StringGrid1)
else savelist(StringGrid2);
end;

procedure TForm_peach.pmSearchAddClick(Sender: TObject);
begin
cp_search(desk_env);
end;

procedure TForm_peach.po_addtoarchiveClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
if fun='FILEBROWSER' then
   if checkfdwin=0 then addtoarchive_selected;
if fun='UN7Z' then addtoarchive_inarchive;
if fun='UNARC' then addtoarchive_inarchive;
end;

procedure TForm_peach.po_checkClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
checkselected_list;
end;

procedure TForm_peach.po_compareClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
compareselected_list;
end;

procedure update_clipboard;
var
   i:integer;
begin
Form_peach.StringGridClipboard.Rowcount:=length(clipcontent)+1;
for i:=0 to length(clipcontent)-1 do
   begin
   Form_peach.StringGridClipboard.Cells[0,i+1]:=clipcontent[i,3];
   Form_peach.StringGridClipboard.Cells[1,i+1]:=clipcontent[i,1];
   Form_peach.StringGridClipboard.Cells[2,i+1]:=clipcontent[i,2];
   Form_peach.StringGridClipboard.Cells[3,i+1]:=clipcontent[i,0];
   end;
Form_peach.StringGridClipboard.AutosizeColumns;
end;

function test_clip(s:utf8string):integer;
var
   i:integer;
begin
test_clip:=0;
for i:=0 to length(clipcontent)-1 do
   begin
   if clipcontent[i,0]=s then
      begin
      test_clip:=1;
      exit;
      end;
   end;
end;

procedure list_copy;
var
   i,k:integer;
begin
   if fun='FILEBROWSER' then
      begin
      {$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
      if filecopying=true then
         begin
         MessageDlg(txt_copy_wait, mtInformation, [mbOK], 0);
         exit;
         end;
      prev_destdir:='';
      if clipmode=0 then setlength(clipcontent,0);
      k:=length(clipcontent);
      for i:=1 to Form_peach.StringGridList.RowCount-1 do
         if Form_peach.StringGridList.Cells[12,i]='1' then
            begin
            if clipmode=1 then
               if test_clip(Form_peach.StringGridList.Cells[8,i])=1 then
               else
                  begin
                  Form_peach.StringGridList.Cells[0,i]:='';
                  setlength(clipcontent,length(clipcontent)+1);
                  clipcontent[k,0]:=Form_peach.StringGridList.Cells[8,i];
                  clipcontent[k,1]:=Form_peach.StringGridList.Cells[1,i];
                  clipcontent[k,2]:=Form_peach.StringGridList.Cells[2,i];
                  clipcontent[k,3]:='copy';
                  k:=k+1;
                  end
            else
               begin
               Form_peach.StringGridList.Cells[0,i]:='';
               setlength(clipcontent,length(clipcontent)+1);
               clipcontent[k,0]:=Form_peach.StringGridList.Cells[8,i];
               clipcontent[k,1]:=Form_peach.StringGridList.Cells[1,i];
               clipcontent[k,2]:=Form_peach.StringGridList.Cells[2,i];
               clipcontent[k,3]:='copy';
               k:=k+1;
               end;
            end
         else Form_peach.StringGridList.Cells[0,i]:='';
      update_clipboard;
      end;
end;

procedure TForm_peach.po_copyClick(Sender: TObject);
begin
list_copy;
end;

procedure list_cut;
var
   i,k:integer;
begin
   if fun='FILEBROWSER' then
      begin
      {$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
      if filecopying=true then
         begin
         MessageDlg(txt_copy_wait, mtInformation, [mbOK], 0);
         exit;
         end;
      prev_destdir:='';
      if clipmode=0 then setlength(clipcontent,0);
      k:=length(clipcontent);
      for i:=1 to Form_peach.StringGridList.RowCount-1 do
         if Form_peach.StringGridList.Cells[12,i]='1' then
            begin
            if clipmode=1 then
               if test_clip(Form_peach.StringGridList.Cells[8,i])=1 then
               else
                  begin
                  Form_peach.StringGridList.Cells[0,i]:=' ';
                  setlength(clipcontent,length(clipcontent)+1);
                  clipcontent[k,0]:=Form_peach.StringGridList.Cells[8,i];
                  clipcontent[k,1]:=Form_peach.StringGridList.Cells[1,i];
                  clipcontent[k,2]:=Form_peach.StringGridList.Cells[2,i];
                  clipcontent[k,3]:='move';
                  k:=k+1;
                  end
            else
               begin
               Form_peach.StringGridList.Cells[0,i]:=' ';
               setlength(clipcontent,length(clipcontent)+1);
               clipcontent[k,0]:=Form_peach.StringGridList.Cells[8,i];
               clipcontent[k,1]:=Form_peach.StringGridList.Cells[1,i];
               clipcontent[k,2]:=Form_peach.StringGridList.Cells[2,i];
               clipcontent[k,3]:='move';
               k:=k+1;
               end;
            end
         else Form_peach.StringGridList.Cells[0,i]:='';
      update_clipboard;
      end;
end;

procedure TForm_peach.po_cutClick(Sender: TObject);
begin
list_cut;
end;

procedure TForm_peach.po_defragClick(Sender: TObject);
begin
open_defrag(copy(Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row], 1, 2));
end;

procedure TForm_peach.po_deleteClick(Sender: TObject);
begin
deleteselected_frombrowser;
end;

procedure TForm_peach.po_diskcleanClick(Sender: TObject);
begin
open_diskclean(Form_peach.StringGridList.Cells[8, Form_peach.StringGridList.Row]);
end;

procedure TForm_peach.po_exthereClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbUnChecked;
if fun='FILEBROWSER' then list_toextractor('multi')
else list_toextractor('all'); //should not apply since the entry is shown in filebrowser only
end;

procedure TForm_peach.po_exttoClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
CheckBoxFolder.State:=cbChecked;
if fun='FILEBROWSER' then list_toextractor('multi')
else list_toextractor('all'); //should not apply since the entry is shown in filebrowser only
end;

procedure TForm_peach.po_infoClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
if checkfdwin<>0 then exit;
filebrowser_funall('info');
end;

procedure TForm_peach.po_joinClick(Sender: TObject);
var
   fname:utf8string;
begin
if Form_peach.StringGridList.Row=0 then exit;
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
fname:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
direct_join(fname);
end;

procedure TForm_peach.po_listClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
if checkfdwin<>0 then exit;
filebrowser_funall('list');
end;

procedure TForm_peach.po_newfolderClick(Sender: TObject);
var
   P:TProcess;
   cl,s:utf8string;
   i:integer;
begin
{$IFDEF MSWINDOWS}
if EditOpenIn.Text=txt_mypc then exit;
s:=txt_cnewfolder;
{$ELSE}
s:='Newfolder';
{$ENDIF}
if not pInputQuery(txt_foldername, txt_name_provide, s, false) then exit;
if checkfilename(s)<>0 then
   begin
   MessageDlg('"'+s+'" '+txt_checkname_failed+char($0D)+char($0A)+txt_name_provide, mtWarning, [mbOK], 0);
   exit;
   end;
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   if s=Form_peach.StringGridList.Cells[1,i] then exit;
P:=TProcess.Create(nil);
P.Options := [poNoConsole, poWaitOnExit];
{$IFDEF MSWINDOWS}
cl:=wincomspec+' /c mkdir "'+Form_peach.EditOpenIn.Text+s+'"';
{$ELSE}//system needs to support mkdir command
cl:='mkdir '''+Form_peach.EditOpenIn.Text+s+'''';
{$ENDIF}
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
//navrefresh; //deactivated, refresh is now automatic
end;

procedure TForm_peach.po_open_associatedClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
execute_obj(3,StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.po_open_customClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
openw_obj(StringGridList.Cells[8,StringGridList.Row]);
end;

procedure TForm_peach.po_open_peazipClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
execute_obj(2,StringGridList.Cells[8,StringGridList.Row]);
end;

function getnewdestname(var s:utf8string):integer;
var
   s1,s2:utf8string;
begin
getnewdestname:=-1;
s1:=uextractfilename(s);
s2:=s1;
if not pInputQuery(txt_rename, txt_name_provide, s1, false) then exit;
if s1=s2 then exit;
if checkfilename(s1)<>0 then
   begin
   MessageDlg('"'+s1+'" '+txt_checkname_failed+char($0D)+char($0A)+txt_name_provide, mtWarning, [mbOK], 0);
   exit;
   end;
s2:=uextractfilepath(s);
if s2<>'' then if s2[length(s2)]<>directoryseparator then s2:=s2+directoryseparator;
s:=s2+s1;
getnewdestname:=0;
end;

function cp_copy_filedir(clipcontent:TClipcontent; var destdir:utf8string):integer;
var
   pcount,ech,ecode,i:integer;
   overall:boolean;
   cl,src,dest,destf,opcommand:utf8string;
   P:TProcess;
   label 1;
begin
if filecopying=true then //cuncurrent operation
   begin
   if clipcontent[0,3]='move' then exit; //if the src is being moved, avoid other paste operations
   end;
//if destdir=prev_destdir then samedir:=true else samedir:=false; //ask new name if pasting src and destination are the same dir
filecopying:=true;
cp_copy_filedir:=-1;
Form_peach.StringGridList.Cursor:=crHourGlass;
Form_peach.ctrlrefresh.Glyph:=Bp0;
ech:=0;//standard error check, changed conditionally if needed;
overall:=false; //overwrite all, default false;
if Form_peach.Visible=true then Application.ProcessMessages;
for i:=0 to length(clipcontent)-1 do
   begin //it could perform mixed move/copy operation on a list of files
      prev_destdir:=destdir;
      pcount:=1;
      cl:='';
      src:='';
      dest:='';
      destf:='';
      P:=TProcess.Create(nil);
      src:=clipcontent[i,0];
      if checkfiledirname(src)<>0 then
         begin
         MessageDlg(txt_2_7_validatefn+' '+src, mtWarning, [mbOK], 0);
         Form_peach.StringGridList.Cursor:=crDefault;
         Form_peach.ctrlrefresh.Glyph:=Brefresh;
         filecopying:=false;
         exit;
         end;
      {$IFDEF MSWINDOWS}
      if winver='9x' then
         if clipcontent[i,2]=txt_list_isfolder then
            begin
            dest:=destdir+clipcontent[i,1];
            if src=dest then
               if getnewdestname(dest)<>0 then
               begin
               filecopying:=false;
               Form_peach.StringGridList.Cursor:=crDefault;
               Form_peach.ctrlrefresh.Glyph:=Brefresh;
               exit;
               end;
            if udirectoryexists((dest)) then
               if overall=true then
               else
                     case MessageDlg('"'+uextractfilename(dest)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: cl:=wincomspec+' /c xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /Y';
                     7: cl:=wincomspec+' /c xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /-Y'; //there is not an explicit options to skip existing files, will ask confirmation in console
                     10: begin overall:=true; cl:=wincomspec+' /c xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /Y'; end;
                     end;
            if cl='' then cl:=wincomspec+' /c xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /Y';
            end
         else
            begin
            dest:=destdir;
            if dest<>'' then if dest[length(dest)]<>directoryseparator then dest:=dest+directoryseparator;
            destf:=dest+clipcontent[i,1];
            if src=destf then
               if getnewdestname(destf)<>0 then
                  begin
                  filecopying:=false;
                  Form_peach.StringGridList.Cursor:=crDefault;
                  Form_peach.ctrlrefresh.Glyph:=Brefresh;
                  exit;
                  end
               else
                  begin
                  cl:=wincomspec+' /c xcopy '+delimiter+src+delimiter+' '+delimiter+destf+delimiter+' /C /H /K /R /Y';
                  end
            else
               begin
               if ufileexists((destf)) then
                  if overall=true then clearfile(destf)
                  else
                     case MessageDlg('"'+uextractfilename(destf)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: clearfile(destf);
                     7: goto 1;
                     10: begin clearfile(destf); overall:=true; end;
                     end;
               cl:=wincomspec+' /c xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /H /K /R /Y';
               end;
            end;
      if winver='nt5' then
         if clipcontent[i,2]=txt_list_isfolder then
            begin
            dest:=destdir+clipcontent[i,1];
            if src=dest then
               if getnewdestname(dest)<>0 then
               begin
               filecopying:=false;
               Form_peach.StringGridList.Cursor:=crDefault;
               Form_peach.ctrlrefresh.Glyph:=Brefresh;
               exit;
               end;
            if udirectoryexists((dest)) then
               if overall=true then
               else
                     case MessageDlg('"'+uextractfilename(dest)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /Y /Z';
                     7: cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /-Y /Z'; //there is not an explicit options to skip existing files, will ask confirmation in console
                     10: begin overall:=true; cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /Y /Z'; end;
                     end;
            if cl='' then cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /E /H /K /I /R /Y /Z';
            end
         else
            begin
            dest:=destdir;
            if dest<>'' then if dest[length(dest)]<>directoryseparator then dest:=dest+directoryseparator;
            destf:=dest+clipcontent[i,1];
            if src=destf then
               if getnewdestname(destf)<>0 then
                  begin
                  filecopying:=false;
                  Form_peach.StringGridList.Cursor:=crDefault;
                  Form_peach.ctrlrefresh.Glyph:=Brefresh;
                  exit;
                  end
               else
                  begin
                  cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+destf+delimiter+' /C /H /K /R /Y /Z';
                  end
            else
               begin
               if ufileexists((destf)) then
                  if overall=true then clearfile(destf)
                  else
                     case MessageDlg('"'+uextractfilename(destf)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: clearfile(destf);
                     7: goto 1;
                     10: begin clearfile(destf); overall:=true; end;
                     end;
               cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /C /H /K /R /Y /Z';
               end;
            end;
      if winver='nt6+' then
         begin
         ech:=1;
         if clipcontent[i,2]=txt_list_isfolder then
            begin
            dest:=destdir+clipcontent[i,1];
            if src=dest then
               if getnewdestname(dest)<>0 then
                  begin
                  filecopying:=false;
                  Form_peach.StringGridList.Cursor:=crDefault;
                  Form_peach.ctrlrefresh.Glyph:=Brefresh;
                  exit;
                  end;
            if udirectoryexists((dest)) then
               if overall=true then
               else
                     case MessageDlg('"'+uextractfilename(dest)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: cl:='robocopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /E /Z';
                     7: cl:='robocopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /E /XO /XN /Z'; //exclude existing files (either older and newer) to skip overwriting
                     10: begin overall:=true; cl:='robocopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /E /Z'; end;
                     end;
            if cl='' then cl:='robocopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' /E /Z';
            if clipcontent[i,3]='move' then cl:=cl+' /MOV';
            end
         else
            begin
            //if src=dest then
            if uextractfilepath(src)=destdir then
               begin
               ech:=0;
               dest:=destdir;
               if dest<>'' then if dest[length(dest)]=directoryseparator then setlength(dest,length(dest)-1);
               destf:=dest+directoryseparator+clipcontent[i,1];
               if getnewdestname(destf)<>0 then
                  begin
                  filecopying:=false;
                  Form_peach.StringGridList.Cursor:=crDefault;
                  Form_peach.ctrlrefresh.Glyph:=Brefresh;
                  exit;
                  end
               else
                  begin
                  cl:='xcopy '+delimiter+src+delimiter+' '+delimiter+destf+delimiter+' /C /H /K /R /Y /Z'; //robocopy doesn't like to copy to a different filename
                  end;
               end
            else
               begin
               src:=uextractfilepath(src);
               if src<>'' then if src[length(src)]=directoryseparator then setlength(src,length(src)-1);
               dest:=destdir;
               if dest<>'' then if dest[length(dest)]=directoryseparator then setlength(dest,length(dest)-1);
               destf:=dest+directoryseparator+clipcontent[i,1];
               if ufileexists((destf)) then
                  if overall=true then clearfile(destf)
                  else
                        case MessageDlg('"'+uextractfilename(destf)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                        2: goto 1;
                        6: clearfile(destf);
                        7: goto 1;
                        10: begin clearfile(destf); overall:=true; end;
                        end;
               cl:='robocopy '+delimiter+src+delimiter+' '+delimiter+dest+delimiter+' '+delimiter+clipcontent[i,1]+delimiter+' /Z /COPY:DT';
               if clipcontent[i,3]='move' then cl:=cl+' /MOVE';
               end;
            usetcurrentdir((src)); //robocopy will fail otherwise in case of src or dest <3 characters
            end;
         end;
      {$ELSE}//generic Unix-like cp command
      P.Options := [poWaitOnExit];
      if clipcontent[i,3]='move' then opcommand:='mv '
      else opcommand:='cp -p -r ';
      if clipcontent[i,2]=txt_list_isfolder then
         begin
         dest:=destdir+clipcontent[i,1];
         if src=dest then
            if getnewdestname(dest)<>0 then
               begin
               filecopying:=false;
               Form_peach.StringGridList.Cursor:=crDefault;
               Form_peach.ctrlrefresh.Glyph:=Brefresh;
               exit;
               end;
         if udirectoryexists((dest)) then
            if overall=true then
            else
                  case MessageDlg('"'+uextractfilename(dest)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: cl:=opcommand+'-f '+delimiter+src+delimiter+' '+delimiter+dest+delimiter;
                     7: cl:=opcommand+delimiter+src+delimiter+' '+delimiter+dest+delimiter;
                     10: begin overall:=true; cl:=opcommand+'-f '+delimiter+src+delimiter+' '+delimiter+dest+delimiter; end;
                     end;
         if cl='' then cl:=opcommand+'-f '+delimiter+src+delimiter+' '+delimiter+dest+delimiter;
         end
      else
         begin
         dest:=destdir;
         if dest<>'' then if dest[length(dest)]<>directoryseparator then dest:=dest+directoryseparator;
         destf:=dest+clipcontent[i,1];
         if src=destf then
            if getnewdestname(destf)<>0 then
               begin
               filecopying:=false;
               Form_peach.StringGridList.Cursor:=crDefault;
               Form_peach.ctrlrefresh.Glyph:=Brefresh;
               exit;
               end;
         if ufileexists((destf)) then
            if overall=true then clearfile(destf)
            else
                  case MessageDlg('"'+uextractfilename(destf)+'" '+txt_overwrite, mtInformation, [mbYesToAll, mbYes, mbNo, mbCancel],0) of
                     2: goto 1;
                     6: clearfile(destf);
                     7: goto 1;
                     10: begin clearfile(destf); overall:=true; end;
                     end;
         cl:=opcommand+'-f '+delimiter+src+delimiter+' '+delimiter+destf+delimiter;
         end;
      {$ENDIF}
      if cl='' then goto 1;
      cl:=utf8toenv(cl);
      P.CommandLine:=cl;
      if validatecl(P.Commandline)<>0 then
         begin
         MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0);
         Form_peach.StringGridList.Cursor:=crDefault;
         Form_peach.ctrlrefresh.Glyph:=Brefresh;
         filecopying:=false;
         exit;
         end;
      P.Execute;
      while P.Running do
         begin
         animation_busy(pcount);
         if Form_peach.Visible=true then Application.ProcessMessages;
         Sleep(100);
         end;
      usetcurrentdir((executable_path));
      ecode:=P.ExitStatus;
      if ech=0 then
         if ecode<>0 then
               if MessageDlg(cl+char($0D)+char($0A)+txt_copy_error+' '+inttostr(ecode)+'. '+txt_abort, mtError, [mbYes, mbNo],0)=6 then
                  begin
                  break;
                  end
               else goto 1
         else cp_copy_filedir:=0;
      if ech=1 then
         if ecode>3 then
               if MessageDlg(cl+char($0D)+char($0A)+txt_copy_error+' '+inttostr(ecode)+'. '+txt_abort, mtError, [mbYes, mbNo],0)=6 then
                  begin
                  break;
                  end
               else goto 1
         else cp_copy_filedir:=0;
      {$IFDEF MSWINDOWS}
      //remove src on move operations: only executed if no errors are encountered
      if winver='nt5' then
         if clipcontent[i,3]='move' then
            if clipcontent[i,2]=txt_list_isfolder then cp_copy_filedir:=cleardir(src,false)
            else clearfile(src);
      if winver='nt6+' then
         if clipcontent[i,3]='move' then
            if clipcontent[i,2]=txt_list_isfolder then
               if udirectoryexists((src)) then cp_copy_filedir:=cleardir(src,false)
               else
            else
               if ufileexists(src) then clearfile(src);
      {$ENDIF}
      1:
      P.Free;
   end;
Form_peach.StringGridList.Cursor:=crDefault;
Form_peach.ctrlrefresh.Glyph:=Brefresh;
filecopying:=false;
end;

procedure list_paste;
var
   i:integer;
begin
   if fun='FILEBROWSER' then
      begin
      if length(clipcontent)>0 then
         if clipcontent[0,0]<>'' then
            begin
            destdir:=Form_peach.EditOpenIn.Text;//always refer to currently browsed path
            i:=cp_copy_filedir(clipcontent,destdir);
            end;
      if filecopying=false then
         if length(clipcontent)>0 then
            if (clipcontent[0,3]='move') or (clipmode=1) then
               if i=0 then
                  begin
                  prev_destdir:='';
                  setlength(clipcontent,0);
                  Form_peach.StringGridClipboard.Rowcount:=1;
                  Form_peach.StringGridClipboard.Autosizecolumns;
                  end;
      end;
end;

procedure TForm_peach.po_pasteClick(Sender: TObject);
begin
   if Form_peach.StringGridList.RowCount-1<length(clipcontent) then
      begin
      Form_peach.StringGridList.RowCount:=length(clipcontent)+1;
      if Form_peach.StringGridList.ColWidths[8]=0 then listsizecol('flat') else
         if Form_peach.Edit1.Text=txt_mypc then listsizecol('pc')
         else listsizecol('browse');
      end;
list_paste;
end;

procedure clickcopyto;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
list_copy;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      begin
      listdir(Form_peach.SelectDirectoryDialog1.FileName,false);
      addtohistory;
      list_paste;
      end;
end;

procedure clickmoveto;
begin
if Form_peach.StringGridList.Cells[1,1]='' then exit;
list_cut;
if Form_peach.SelectDirectoryDialog1.Execute then
   if Form_peach.SelectDirectoryDialog1.FileName<>'' then
      begin
      listdir(Form_peach.SelectDirectoryDialog1.FileName,false);
      addtohistory;
      list_paste;
      end;
end;

procedure TForm_peach.pmcpytoClick(Sender: TObject);
begin
clickcopyto;
end;

procedure TForm_peach.pmmvtoClick(Sender: TObject);
begin
clickmoveto;
end;

procedure TForm_peach.pmquickdeleteClick(Sender: TObject);
begin
deleteselected_frombrowser;
end;

procedure TForm_peach.po_copytoClick(Sender: TObject);
begin
clickcopyto;
end;

procedure TForm_peach.po_movetoClick(Sender: TObject);
begin
clickmoveto;
end;

procedure TForm_peach.po_renameClick(Sender: TObject);
var
   P:TProcess;
   cl,s:utf8string;
begin
{$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
if StringGridList.Cells[1,1]='' then exit;
if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
s:=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row];
if not pInputQuery(txt_rename, txt_name_provide, s, false) then exit;
if s=Form_peach.StringGridList.Cells[1,Form_peach.StringGridList.Row] then exit;
if checkfilename(s)<>0 then
   begin
   MessageDlg('"'+s+'" '+txt_checkname_failed+char($0D)+char($0A)+txt_name_provide, mtWarning, [mbOK], 0);
   exit;
   end;
P:=TProcess.Create(nil);
P.Options := [poNoConsole, poWaitOnExit];
{$IFDEF MSWINDOWS}
cl:=wincomspec+' /c ren "'+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]+'" "'+s+'"';
{$ELSE}//system needs to support mv command
s:=uextractfilepath(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])+s;
cl:='mv '''+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row]+''' '''+s+'''';
{$ENDIF}
cl:=utf8toenv(cl);
P.CommandLine:=cl;
if validatecl(P.Commandline)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+P.Commandline, mtWarning, [mbOK], 0); exit; end;
P.Execute;
P.Free;
navrefresh;
end;

procedure TForm_peach.po_securedeleteClick(Sender: TObject);
begin
eraseselected_list;
end;

procedure TForm_peach.po_splitClick(Sender: TObject);
var
   fname:utf8string;
begin
if Form_peach.StringGridList.Row=0 then exit;
if Form_peach.StringGridList.Cells[1,1]='' then exit;
if checkfiledirname(Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row], mtWarning, [mbOK], 0); exit; end;
fname:=Form_peach.StringGridList.Cells[8,Form_peach.StringGridList.Row];
direct_split(fname);
end;

procedure TForm_peach.po_testClick(Sender: TObject);
begin
if StringGridList.Cells[1,1]='' then exit;
filebrowser_funall('test');
end;

procedure TForm_peach.pwArchiveWebsitesClick(Sender: TObject);
var
   s:utf8string;
begin
if StringGrid1.RowCount=1 then exit;
s:=Form_peach.StringGrid1.Cells[1,Form_peach.StringGrid1.Row];
if convert2webname(s)=0 then cp_open('http://www.google.com/search?q='+s,desk_env);
end;

procedure TForm_peach.RadioGroup0Change(Sender: TObject);
begin
df:=RadioGroup0.ItemIndex;
archive_type_selected:=-1;
updateconf_modified;
end;

procedure TForm_peach.RadioGroup1Click(Sender: TObject);
begin
algodisp:=RadioGroup1.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.RadioGroup4Change(Sender: TObject);
begin
zaout:=RadioGroup4.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.RadioGroup6Change(Sender: TObject);
begin
autoclosegwrap:=RadioGroup6.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.RadioGroupBrowserChange(Sender: TObject);
begin
case RadioGroupBrowser.ItemIndex of
   0: browserstyle:=0;
   1: browserstyle:=1;
   2: if browsingmode7z=0 then browserstyle:=20 else browserstyle:=21;
   end;
updateconf_modified;
end;

procedure TForm_peach.RadioGroupClipClick(Sender: TObject);
begin
clipmode:=RadioGroupClip.ItemIndex;
if clipmode=0 then clear_clipboard;
updateconf_modified;
end;

procedure TForm_peach.RadioGroupLPAQClick(Sender: TObject);
begin
{case RadioGroupLpaq.ItemIndex of
   0: archive_type:='lpaq1';
   1: archive_type:='lpaq5';
   2: archive_type:='lpaq8';
   end;
Form_peach.Caption:=txt_create+' .'+archive_type;
display_lpaq_memory;}
end;

procedure TForm_peach.RadioGroupPaqClick(Sender: TObject);
var
   s:utf8string;
begin
case RadioGroupPaq.ItemIndex of
   0: begin
      archive_type:='lpaq8';
      check_tarbefore(tdirs);
      if Form_peach.StringGrid1.Rowcount>1 then
         begin
         suggest_outname(s,Form_peach.cbType.Text,0);
         Form_peach.EditName3.Text:=uextractfilepath(s);
         Form_peach.Edit4.Text:=uextractfilename(s);
         Form_peach.Edit5.Text:=Form_peach.EditName3.Text+addext(Form_peach.Edit4.Text);
         end;
      end;
   1: archive_type:='paq8o';
   2: archive_type:='zpaq';
   end;
Form_peach.Caption:=txt_create+' .'+archive_type;
end;

procedure TForm_peach.RadioGrouppwrequestChange(Sender: TObject);
begin
extpw:=RadioGrouppwrequest.ItemIndex;
updateconf_modified;
end;

procedure TForm_peach.RadioGroupQuadClick(Sender: TObject);
begin
case RadioGroupQuad.ItemIndex of
   0: archive_type:='quad';
   1: archive_type:='balz';
   end;
Form_peach.Caption:=txt_create+' .'+archive_type;
end;

procedure TForm_peach.SpeedButton4Click(Sender: TObject);
begin
if SpeedButton4.Caption='+' then
   begin
   GroupBox13.Width:=340;
   GroupBox13.Height:=192;
   SpeedButton4.Caption:='-';
   Labelcb13.Visible:=true;
   RadioGrouppwrequest.Visible:=true;
   CheckBoxIgnoreDisp.Visible:=true;
   CheckBoxIgnoreSel.Visible:=true;
   CheckBox4.Visible:=true;
   end
else
   begin
   GroupBox13.Width:=292;
   GroupBox13.Height:=20;
   SpeedButton4.Caption:='+';
   Labelcb13.Visible:=false;
   RadioGrouppwrequest.Visible:=false;
   CheckBoxIgnoreDisp.Visible:=false;
   CheckBoxIgnoreSel.Visible:=false;
   CheckBox4.Visible:=false;
   end

end;

procedure TForm_peach.SpinEdit1Change(Sender: TObject);
begin
opacity:=SpinEdit1.Value;
setmswindowsopacity;
end;

procedure TForm_peach.SpinEdit2Change(Sender: TObject);
begin
rowheight:=SpinEdit2.Value;
apply_theme;
end;

procedure TForm_peach.SpinEdit3Change(Sender: TObject);
begin
itemheight:=SpinEdit3.Value;
apply_theme;
end;

procedure TForm_peach.SpinEditPrebrowseChange(Sender: TObject);
begin
prebrowsesize:=Form_peach.SpinEditPrebrowse.Value;
prebrowse:=1;
updateconf_modified;
end;

procedure TForm_peach.Splitter1Moved(Sender: TObject);
begin
splittersize:=panelside.top-panelfilters.top-splitter1.top;
end;

procedure TForm_peach.StringGrid1DblClick(Sender: TObject);
begin
if currow1=0 then
else open_associated;
end;

procedure drawcellicon(sg:tstringgrid; arow:integer);
var
   s:utf8string;
   l,t:integer;
begin
l:=sg.CellRect(0,arow).Left+2;
t:=sg.CellRect(0,arow).Top+1;
if (sg=Form_peach.StringGridList) and (Form_peach.EditOpenIn.Text=txt_mypc) then
   begin
   if sg.Cells[2,arow] = txt_unit_removable then sg.Canvas.Draw(l,t,Bremovable_supported);
   if sg.Cells[2,arow] = txt_unit_hd then sg.Canvas.Draw(l,t,Bhd_supported);
   if sg.Cells[2,arow] = txt_unit_remote then sg.Canvas.Draw(l,t,Bremote_supported);
   if sg.Cells[2,arow] = txt_unit_cd then sg.Canvas.Draw(l,t,Bdvd_supported);
   if sg.Cells[2,arow] = txt_unit_ram then sg.Canvas.Draw(l,t,Bram_supported);
   if sg.Cells[2,arow] = txt_unit_floppy then sg.Canvas.Draw(l,t,Bfd_supported);
   exit;
   end;
if sg.Cells[2,arow] = txt_list_isfolder then
   begin
   if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BFolder)
   else sg.Canvas.Draw(l,t,BCutfolder);
   exit;
   end;
if sg.Cells[2,arow] = '.lnk' then
   begin
   if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BLink)
   else sg.Canvas.Draw(l,t,BCutlink);
   exit;
   end;
s:=sg.Cells[8,arow];
if sg.Cells[1,arow]<>'' then
   if (testinput(s,false)=0) then
      if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BUnsupported) //unsupported file type
      else sg.Canvas.Draw(l,t,BCutunsupported)
   else
   begin
      case testext(s) of
         503: if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BFileSupported)
            else sg.Canvas.Draw(l,t,BCutfile);
         502: if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BFileSupported)
            else sg.Canvas.Draw(l,t,BCutfile);
         501: if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BFileSupported) //non-archives
             else sg.Canvas.Draw(l,t,BCutfile);
         500: if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BExeSupported) //Windows executables, installers
             else sg.Canvas.Draw(l,t,BCutexe);
         else
            if sg.Cells[0,arow]='' then sg.Canvas.Draw(l,t,BArchiveSupported)
            else sg.Canvas.Draw(l,t,BCutarchive);
         end;
      end;
end;

procedure TForm_peach.StringGrid1DrawCell(Sender: TObject; aCol, aRow: Integer;
  aRect: TRect; aState: TGridDrawState);
var
   s:utf8string;
   rowcol:TColor;
begin
if (arow and 1)=0 then rowcol:=StringGrid1.Color
else rowcol:=StringGrid1.AlternateColor;//col2
if (gdFixed in aState) then
   begin
   if aCol=0 then StringGrid1.Canvas.Draw(StringGrid1.CellRect(0,arow).Left+2,StringGrid1.CellRect(0,arow).Top+1,Brefresh_up);
   exit;
   end;

   if StringGrid1.Cells[11,arow] = '1' then
      begin
      StringGrid1.Canvas.Brush.Color := clHighlight;
      StringGrid1.Canvas.FillRect(aRect);
      StringGrid1.Canvas.Font.Color := clHighlightText;
      if aCol=0 then drawcellicon(StringGrid1,arow);
      end
   else
      begin
      StringGrid1.Canvas.Brush.Color := rowcol;
      StringGrid1.Canvas.FillRect(aRect);
      StringGrid1.Canvas.Font.Color := clWindowText;
      if aCol=0 then drawcellicon(StringGrid1,arow);
      if aCol=1 then
         begin
         if StringGrid1.Cells[2,arow] = txt_list_isfolder then
            begin
            StringGrid1.Canvas.Brush.Color := stringtocolor(COLFOLDERROW);//(COLGROUP1);
            StringGrid1.Canvas.Pen.Color := rowcol;
            StringGrid1.Canvas.Pen.Width:=1;
            StringGrid1.Canvas.FillRect(aRect);
            StringGrid1.Canvas.RoundRect(arect,1,1);
            end;
         s:=StringGrid1.Cells[8,arow];
         if (testinput(s,false)>0) then
            begin
            StringGrid1.Canvas.Brush.Color := stringtocolor(COLARCHIVEROW);//(COLGROUP2);
            StringGrid1.Canvas.Pen.Color := rowcol;
            StringGrid1.Canvas.Pen.Width:=1;
            StringGrid1.Canvas.FillRect(aRect);
            StringGrid1.Canvas.RoundRect(arect,1,1);
            end;
         end;
      end;

{$IFDEF MSWINDOWS}
//{$ELSE}//patch the GTK/GTK2 issue of text remaining visible when column width is set to 0
if aCol=9 then StringGrid1.Canvas.Font.Color := rowcol;
if aCol=10 then StringGrid1.Canvas.Font.Color := rowcol;
if aCol=11 then StringGrid1.Canvas.Font.Color := rowcol;
{$ENDIF}
end;

procedure TForm_peach.StringGrid1HeaderClick(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
var
   Column:integer;
begin
if StringGrid1.Rowcount<2 then exit;
Column:=Index;
   if (Column=0) then //force MenuItemRefresh of layout content (update number of files, folders and total size)
      begin
      updatelayout;
      exit;
      end;
   if Column=1 then Column:=9;
   if Column=3 then Column:=10;
   if Column=13 then Column:=8;
   if Column=listsortcol1 then
      if az1=true then sort_za_stringgrid1
      else sort_az_stringgrid1(Column)
   else sort_az_stringgrid1(Column);
   listsortcol1:=Column;
   Form_peach.StringGrid1.Repaint;
end;

procedure TForm_peach.StringGrid1HeaderSized(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
begin
if index=0 then StringGrid1.ColWidths[0]:=ROWSIZE;
end;

procedure TForm_peach.StringGrid1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
   i,j:integer;
begin
//patch for Ctrl+X behaviour
if (Key=ord(char(88))) and (shift=[ssCtrl]) then
   begin
   setlength(patchclip,Form_peach.StringGrid1.RowCount-1);
   for i:=1 to Form_peach.StringGrid1.RowCount-1 do
      for j:=0 to StringGrid1.ColCount-1 do patchclip[i-1,j]:=StringGrid1.Cells[j,i];
   end;

//ensure Ctrl+V has enough rows to paste the content, since content could be pasted from clipboard it consider at excess 2048 rows
if (Key=ord(char(86))) and (shift=[ssCtrl]) then
   begin
   setlength(patchclip,Form_peach.StringGrid1.RowCount-1);
   for i:=1 to Form_peach.StringGrid1.RowCount-1 do
      for j:=0 to StringGrid1.ColCount-1 do patchclip[i-1,j]:=StringGrid1.Cells[j,i];
   Form_peach.StringGrid1.RowCount:=2048;
   {if Form_peach.StringGrid1.RowCount-1<length(clipcontent) then
      begin
      Form_peach.StringGrid1.RowCount:=1024;//length(clipcontent)+1;}
   Form_peach.StringGrid1.AutoSizeColumns;
   Form_peach.StringGrid1.ColWidths[0]:=ROWSIZE;
   Form_peach.StringGrid1.ColWidths[1]:=Form_peach.StringGrid1.ColWidths[1]+48;
   Form_peach.StringGrid1.ColWidths[9]:=0;
   Form_peach.StringGrid1.ColWidths[10]:=0;
   Form_peach.StringGrid1.ColWidths[11]:=0;
   Form_peach.StringGrid1.ColWidths[12]:=0;
   Form_peach.StringGrid1.colwidths[13]:=8;
   end;
end;

procedure patch_stringgrid(sg:TStringGrid);
var
   i,j:integer;
begin
for i:=1 to sg.RowCount-1 do
   for j:=0 to sg.ColCount-1 do sg.Cells[j,i]:=patchclip[i-1,j];
end;

procedure TForm_peach.StringGrid1KeyPress(Sender: TObject; var Key: char);
var
   i:integer;
begin
//patch for Ctrl+X behaviour
if Key=char(24) then
   patch_stringgrid(StringGrid1);

//patch Ctrl+V
if Key=char(22) then
   begin
   StringGrid1.RowCount:=length(patchclip)+1;
   patch_stringgrid(StringGrid1);
   end;

if Key=char(127) then removeselected;//ctrl+backspace

if Key=char(13) then open_associated;//enter

if key='?' then checkselected;//checksum/hash selected
if key='=' then compareselected;//compare selected

for i:=StringGrid1.Row+1 to StringGrid1.Rowcount-1 do
   if upcase(copy(StringGrid1.Cells[1,i],1,1))=upcase(Key) then
      begin
      StringGrid1.Row:=i;
      exit;
      end;
for i:=1 to StringGrid1.Row do
   if upcase(copy(StringGrid1.Cells[1,i],1,1))=upcase(Key) then
      begin
      StringGrid1.Row:=i;
      exit;
      end;
end;

procedure TForm_peach.StringGrid1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if Key=46 then //Cancel
   if shift=[] then removeselected;
end;

procedure TForm_peach.StringGrid1MouseDown(Sender: TOBject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Column, Row,i: Integer;
begin
if StringGrid1.RowCount=1 then exit;
StringGrid1.MouseToCell(X, Y, Column, Row);
currow1:=Row;
case Button of
   mbRight:
      begin
      Form_peach.StringGrid1.Row:=Row;
      if Form_peach.StringGrid1.Cells[11,Row]='0' then
         begin
         for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
            Form_peach.StringGrid1.Cells[11,i]:='0';
         Form_peach.StringGrid1.Cells[11,Row]:='1';
         Form_peach.StringGrid1.Repaint;
         end;
      exit;
      end;
   end;
end;

procedure TForm_peach.StringGrid1MouseUp(Sender: TOBject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
   i,Column,Row:integer;
begin
if StringGrid1.Rowcount<2 then exit;
StringGrid1.MouseToCell(X, Y, Column, Row);
if Row=0 then exit;
case Button of
   mbRight: exit; //moved to mousedown
   mbMiddle: exit;
   mbLeft:
      begin
         if Shift=[ssCtrl] then
            for i:=Form_peach.StringGrid1.Selection.Top to Form_peach.StringGrid1.Selection.Bottom do
               if Form_peach.StringGrid1.Cells[11,i]='1' then Form_peach.StringGrid1.Cells[11,i]:='0'
               else Form_peach.StringGrid1.Cells[11,i]:='1'
         else
            begin
            for i:=1 to Form_peach.StringGrid1.Rowcount-1 do
               Form_peach.StringGrid1.Cells[11,i]:='0';
            for i:=Form_peach.StringGrid1.Selection.Top to Form_peach.StringGrid1.Selection.Bottom do
               Form_peach.StringGrid1.Cells[11,i]:='1';
            end;
         Form_peach.StringGrid1.Repaint;
      end;
   end;
end;

procedure TForm_peach.StringGrid2DblClick(Sender: TObject);
begin
if currow1=0 then
else
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then open_associated;
end;

procedure TForm_peach.StringGrid2DrawCell(Sender: TObject; aCol, aRow: Integer;
  aRect: TRect; aState: TGridDrawState);
var
   s:utf8string;
   rowcol:TColor;
begin
if (arow and 1)=0 then rowcol:=StringGrid2.Color
else rowcol:=StringGrid2.AlternateColor;//col2
if (gdFixed in aState) then
   begin
   if aCol=0 then StringGrid2.Canvas.Draw(StringGrid2.CellRect(0,arow).Left+2,StringGrid2.CellRect(0,arow).Top+1,Brefresh_up);
   exit;
   end;

   if StringGrid2.Cells[11,arow] = '1' then
      begin
      StringGrid2.Canvas.Brush.Color := clHighlight;
      StringGrid2.Canvas.FillRect(aRect);
      StringGrid2.Canvas.Font.Color := clHighlightText;
      if aCol=0 then drawcellicon(StringGrid2,arow);
      end
   else
      begin
      StringGrid2.Canvas.Brush.Color := rowcol;
      StringGrid2.Canvas.FillRect(aRect);
      StringGrid2.Canvas.Font.Color := clWindowText;
      if aCol=0 then drawcellicon(StringGrid2,arow);
      if aCol=1 then
         begin
         if StringGrid2.Cells[2,arow] = txt_list_isfolder then
            begin
            StringGrid2.Canvas.Brush.Color := stringtocolor(COLFOLDERROW);//(COLGROUP1);
            StringGrid2.Canvas.Pen.Color := rowcol;
            StringGrid2.Canvas.Pen.Width:=1;
            StringGrid2.Canvas.FillRect(aRect);
            StringGrid2.Canvas.RoundRect(arect,1,1);
            end;
         s:=StringGrid2.Cells[8,arow];
         if (testinput(s,false)>0) then
            begin
            StringGrid2.Canvas.Brush.Color := stringtocolor(COLARCHIVEROW);//(COLGROUP2);
            StringGrid2.Canvas.Pen.Color := rowcol;
            StringGrid2.Canvas.Pen.Width:=1;
            StringGrid2.Canvas.FillRect(aRect);
            StringGrid2.Canvas.RoundRect(arect,1,1);
            end;
         end;
      end;

{$IFDEF MSWINDOWS}
//{$ELSE}//patch the GTK/GTK2 issue of text remaining visible when column width is set to 0
if aCol=9 then StringGrid2.Canvas.Font.Color := rowcol;
if aCol=10 then StringGrid2.Canvas.Font.Color := rowcol;
if aCol=11 then StringGrid2.Canvas.Font.Color := rowcol;
{$ENDIF}
end;

procedure TForm_peach.StringGrid2HeaderClick(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
var
   Column:integer;
begin
if StringGrid2.Rowcount<2 then exit;
Column:=Index;
if (Column=0) then //force MenuItemRefresh of layout content (update number of files, folders and total size)
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then
      begin
      updatelayout;
      exit;
      end
   else exit;
if Column=1 then Column:=9;
if Column=3 then Column:=10;
if Column=13 then Column:=8;
if Column=listsortcol2 then
   if az2=true then sort_za_stringgrid2
   else sort_az_stringgrid2(Column)
else sort_az_stringgrid2(Column);
listsortcol2:=Column;
Form_peach.StringGrid2.Repaint;
end;

procedure TForm_peach.StringGrid2HeaderSized(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
begin
if index=0 then StringGrid2.ColWidths[0]:=ROWSIZE;
end;

procedure TForm_peach.StringGrid2KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
   i,j:integer;
begin
//patch for Ctrl+X behaviour
if (Key=ord(char(88))) and (shift=[ssCtrl]) then
   begin
   setlength(patchclip,Form_peach.StringGrid2.RowCount-1);
   for i:=1 to Form_peach.StringGrid2.RowCount-1 do
      for j:=0 to StringGrid2.ColCount-1 do patchclip[i-1,j]:=StringGrid2.Cells[j,i];
   end;

//ensure Ctrl+V has enough rows to paste the content, since content could be pasted from clipboard it consider at excess 2048 rows
if (Key=ord(char(86))) and (shift=[ssCtrl]) then
   begin
   setlength(patchclip,Form_peach.StringGrid2.RowCount-1);
   for i:=1 to Form_peach.StringGrid2.RowCount-1 do
      for j:=0 to StringGrid2.ColCount-1 do patchclip[i-1,j]:=StringGrid2.Cells[j,i];
   Form_peach.StringGrid2.RowCount:=2048;
   autosize_stringgrid2;
   end;

end;

procedure TForm_peach.StringGrid2KeyPress(Sender: TObject; var Key: char);
var
   i:integer;
begin
//patch for Ctrl+X behaviour
if Key=char(24) then
   patch_stringgrid(StringGrid2);

//patch Ctrl+V
if Key=char(22) then
   begin
   StringGrid2.RowCount:=length(patchclip)+1;
   patch_stringgrid(StringGrid2);
   end;

if Key=char(127) then
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then removeselected_ext;//ctrl+backspace

if Key=char(13) then
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then open_associated;//enter

if key='?' then
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then checkselected;//checksum/hash selected
if key='=' then
   if Form_peach.labelstatus2.Caption= txt_2_7_ext then compareselected;//compare selected

for i:=StringGrid2.Row+1 to StringGrid2.Rowcount-1 do
   if upcase(copy(StringGrid2.Cells[1,i],1,1))=upcase(Key) then
      begin
      StringGrid2.Row:=i;
      exit;
      end;
for i:=1 to StringGrid2.Row do
   if upcase(copy(StringGrid2.Cells[1,i],1,1))=upcase(Key) then
      begin
      StringGrid2.Row:=i;
      exit;
      end;
end;

procedure TForm_peach.StringGrid2KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if Key=46 then //Cancel
   if shift=[] then
      if Form_peach.labelstatus2.Caption= txt_2_7_ext then removeselected_ext;
end;

procedure TForm_peach.StringGrid2MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Column, Row,i: Integer;
begin
if StringGrid2.RowCount=1 then exit;
StringGrid2.MouseToCell(X, Y, Column, Row);
currow1:=Row;
case Button of
   mbRight:
      begin
      Form_peach.StringGrid2.Row:=Row;
      if Form_peach.StringGrid2.Cells[11,Row]='0' then
         begin
         for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
            Form_peach.StringGrid2.Cells[11,i]:='0';
         Form_peach.StringGrid2.Cells[11,Row]:='1';
         Form_peach.StringGrid2.Repaint;
         end;
      exit;
      end;
   end;
end;

procedure TForm_peach.StringGrid2MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
   i,Column,Row:integer;
begin
if StringGrid2.Rowcount<2 then exit;
StringGrid2.MouseToCell(X, Y, Column, Row);
if Row=0 then exit;
case Button of
   mbRight: exit; //moved to mousedown
   mbMiddle: exit;
   mbLeft:
      begin

         if Shift=[ssCtrl] then
            for i:=Form_peach.StringGrid2.Selection.Top to Form_peach.StringGrid2.Selection.Bottom do
               if Form_peach.StringGrid2.Cells[11,i]='1' then Form_peach.StringGrid2.Cells[11,i]:='0'
               else Form_peach.StringGrid2.Cells[11,i]:='1'
         else
            begin
            for i:=1 to Form_peach.StringGrid2.Rowcount-1 do
               Form_peach.StringGrid2.Cells[11,i]:='0';
            for i:=Form_peach.StringGrid2.Selection.Top to Form_peach.StringGrid2.Selection.Bottom do
               Form_peach.StringGrid2.Cells[11,i]:='1';
            end;
         Form_peach.StringGrid2.Repaint;

      end;
   end;
end;

procedure TForm_peach.StringGridBookmarksColRowMoved(Sender: TObject;
  IsColumn: Boolean; sIndex, tIndex: Integer);
var
   i:integer;
begin
populatebookmarks;
end;

procedure TForm_peach.StringGridBookmarksDblClick(Sender: TObject);
begin
if StringGridBookmarks.Rowcount<2 then exit;
browsebookmark(StringGridBookmarks.Row);
end;

procedure drawbarcellicon(sg:tstringgrid; arow:integer);
var
   s,s1:utf8string;
   l,t,i:integer;
begin
l:=sg.CellRect(0,arow).Left+2;
t:=sg.CellRect(0,arow).Top+1;
if sg.Cells[1,arow] = txt_list_found then
   begin
   sg.Canvas.Draw(l,t,Bsearch);
   exit;
   end;
if sg.Cells[1,arow] = txt_list_flat then
   begin
   sg.Canvas.Draw(l,t,Bexpand);
   exit;
   end;
s:=sg.Cells[2,arow];
s1:=sg.Cells[3,arow];
if s1<>'' then s1:=s1[length(s1)];
{$IFDEF MSWINDOWS}
if s = txt_mypc then
   begin
   sg.Canvas.Draw(l,t,Bsystemtools);
   exit;
   end;
if length(s) = 3 then
   begin
   i:=ord(s[1])-64;
   if s1='*' then sg.Canvas.Draw(l,t,devicon[i])
   else sg.Canvas.Draw(l,t,Bgopath);
   exit;
   end;
{$ENDIF}
if s<>'' then if s[length(s)] = directoryseparator then
   begin
   if s1='*' then sg.Canvas.Draw(l,t,BFolder)
   else sg.Canvas.Draw(l,t,Bgopath);
   exit;
   end;
if uextractfileext(s) = '.lnk' then
   begin
   sg.Canvas.Draw(l,t,BLink);
   exit;
   end;
if s1='*' then
   if (testinput(s,false)=0) then sg.Canvas.Draw(l,t,BUnsupported) //unsupported file type
   else
      begin
      case testext(s) of
         503: sg.Canvas.Draw(l,t,BFileSupported);
         502: sg.Canvas.Draw(l,t,BFileSupported);
         501: sg.Canvas.Draw(l,t,BFileSupported); //non-archives
         500: sg.Canvas.Draw(l,t,BExeSupported); //Windows executables, installers
         else
            sg.Canvas.Draw(l,t,BArchiveSupported);
         end;
      end
else sg.Canvas.Draw(l,t,Bgopath);
end;

procedure TForm_peach.StringGridBookmarksDrawCell(Sender: TObject; aCol,
  aRow: Integer; aRect: TRect; aState: TGridDrawState);
var
   rowcol:TColor;
begin
if arow=0 then exit;
if (arow and 1)=0 then rowcol:=StringGridBookmarks.Color
else rowcol:=StringGridBookmarks.AlternateColor;//col2
if StringGridBookmarks.Cells[6,arow] = '1' then
   begin
   StringGridBookmarks.Canvas.Brush.Color := clHighlight;
   StringGridBookmarks.Canvas.FillRect(aRect);
   StringGridBookmarks.Canvas.Font.Color := clHighlightText;
   end
else
   begin
   StringGridBookmarks.Canvas.Brush.Color := rowcol;
   StringGridBookmarks.Canvas.FillRect(aRect);
   StringGridBookmarks.Canvas.Font.Color := clWindowText;
   end;
if acol=0 then drawbarcellicon(StringGridBookmarks,aRow);
end;

procedure TForm_peach.StringGridBookmarksHeaderClick(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
var
   i,Column:integer;
begin
Column:=Index;
if Column=0 then exit;
if iscolumn=false then exit;
if Column=listsortbook then
   if azbook=true then sort_za_stringgridbookmarks
   else sort_az_stringgridbookmarks(Column)
else sort_az_stringgridbookmarks(Column);
for i:=1 to StringGridBookmarks.RowCount-1 do
   begin
   //StringGridBookmarks.Cells[0,i]:=inttostr(i);
   StringGridBookmarks.Cells[6,i]:='0';
   end;
StringGridBookmarks.Cells[6,1]:='1';
StringGridBookmarks.Row:=1;
StringGridBookmarks.Repaint;
listsortbook:=Column;
populatebookmarks;
end;

procedure TForm_peach.StringGridBookmarksHeaderSized(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
begin
if index=0 then StringGridBookmarks.ColWidths[0]:=ROWSIZE;
end;

procedure TForm_peach.StringGridBookmarksKeyPress(Sender: TObject; var Key: char
  );
var
   i:integer;
begin
if StringGridBookmarks.Rowcount<2 then exit;
//search char
for i:=StringGridBookmarks.Row+1 to StringGridBookmarks.Rowcount-1 do
   if upcase(copy(StringGridBookmarks.Cells[2,i],1,1))=upcase(Key) then
      begin
      StringGridBookmarks.Row:=i;
      exit;
      end;
for i:=1 to StringGridBookmarks.Row do
   if upcase(copy(StringGridBookmarks.Cells[2,i],1,1))=upcase(Key) then
      begin
      StringGridBookmarks.Row:=i;
      exit;
      end;
end;

procedure TForm_peach.StringGridBookmarksMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   Column, Row,i: Integer;
begin
if StringGridBookmarks.Rowcount<2 then exit;
StringGridBookmarks.MouseToCell(X, Y, Column, Row);
StringGridBookmarks.Row:=Row;
if StringGridBookmarks.Cells[6,Row]='0' then
   begin
   for i:=1 to StringGridBookmarks.Rowcount-1 do
      StringGridBookmarks.Cells[6,i]:='0';
   StringGridBookmarks.Cells[6,Row]:='1';
   StringGridBookmarks.Repaint;
   end;
end;

procedure TForm_peach.StringGridClipboardDblClick(Sender: TObject);
begin
if StringGridClipboard.Row>0 then cp_open(StringGridClipboard.Cells[3,StringGridClipboard.Row],desk_env);
end;

procedure TForm_peach.StringGridClipboardKeyPress(Sender: TObject; var Key: char
  );
begin
   if Key=char(27) then //Esc to cancel copy/move operations clearing selection and cupy/cut clipboard; cancel also drag operations performed on the same keyboard event, if panelopen is visible, by Timer1
      begin
      clear_clipboard;
      end;
end;

procedure TForm_peach.StringGridCustEdit1ColRowMoved(Sender: TObject;
  IsColumn: Boolean; sIndex, tIndex: Integer);
var
   i:integer;
begin
for i:=1 to 8 do StringGridCustEdit1.Cells[0,i]:=inttostr(i);
set_custedit_fromtable;
set_custedit_menus;
end;

procedure TForm_peach.StringGridCustEdit1DblClick(Sender: TObject);
begin
case StringGridCustEdit1.Col of
   1: setdescription_advcustedit;
   2: editstring1_advcustedit;
   3: editstring2_advcustedit;
   end;
end;

procedure TForm_peach.StringGridCustEdit1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   Column,Row:integer;
begin
StringGridCustEdit1.MouseToCell(X, Y, Column, Row);
StringGridCustEdit1.Row:=Row;
set_adveditcl(Row);
end;

procedure TForm_peach.StringGridCustEdit1SelectCell(Sender: TObject; aCol,
  aRow: Integer; var CanSelect: Boolean);
begin
set_adveditcl(aRow);
end;

procedure TForm_peach.StringGridCustEditColRowMoved(Sender: TObject;
  IsColumn: Boolean; sIndex, tIndex: Integer);
var
   i:integer;
begin
for i:=1 to 8 do StringGridCustEdit.Cells[0,i]:=inttostr(i);
set_custedit_fromtable;
set_custedit_menus;
end;

procedure TForm_peach.StringGridCustEditDblClick(Sender: TObject);
begin
case StringGridCustEdit.Col of
   1: setdescription_custedit;
   2: editapp_custedit;
   end;
end;

procedure TForm_peach.StringGridCustEditMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   Column,Row:integer;
begin
StringGridCustEdit.MouseToCell(X, Y, Column, Row);
StringGridCustEdit.Row:=Row;
end;

procedure TForm_peach.StringGridHistoryDblClick(Sender: TObject);
begin
   if browsinghistory=true then exit;
   browsehistory;
   setfocus_stringgridlist;
end;

procedure TForm_peach.StringGridHistoryDrawCell(Sender: TObject; aCol,
  aRow: Integer; aRect: TRect; aState: TGridDrawState);
begin
if arow=0 then exit;
if acol=0 then drawbarcellicon(StringGridHistory,aRow);
end;

procedure TForm_peach.StringGridHistoryHeaderSized(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
begin
if index=0 then StringGridHistory.ColWidths[0]:=ROWSIZE;
end;

procedure grid_dir_open;
begin
with Form_peach do
begin
if ListRow=0 then exit;
{$IFDEF MSWINDOWS}
if Form_peach.EditOpenIn.Text=txt_mypc then
   begin
   listdir(StringGridList.Cells[8,StringGridList.Row],false);
   addtohistory;
   exit;
   end;
{$ENDIF}
if StringGridList.Cells[2,StringGridList.Row]=txt_list_isfolder then
   begin
   if fun='UN7Z' then
      begin
      if Form7.CheckBoxAdvFilters.State=cbChecked then exit;
      EditUn7zaFilter.Text:=StringGridList.Cells[8,StringGridList.Row]+directoryseparator+'*';
      EditUn7zaFilterExclude.Text:=StringGridList.Cells[8,StringGridList.Row]+directoryseparator+'*'+directoryseparator+'*';
      list_un7z_do;
      end;
   if fun='FILEBROWSER' then
      begin
      listdir(StringGridList.Cells[8,StringGridList.Row],false);
      addtohistory;
      end;
   end;
end;
end;

procedure grid_obj_open;
var
   i,j:integer;
   cl,jobcode,outname,s:utf8string;
   w:widestring;
begin
with Form_peach do
begin
if ListRow=0 then exit;
if StringGridList.Cells[1,StringGridList.Row]='' then exit;
{$IFDEF MSWINDOWS}
if Form_peach.EditOpenIn.Text=txt_mypc then
   begin
   listdir(StringGridList.Cells[8,StringGridList.Row],false);
   addtohistory;
   exit;
   end;
{$ENDIF}
if StringGridList.Cells[2,StringGridList.Row]=txt_list_isfolder then grid_dir_open
else
   begin
   i:=StringGridList.Row;
   if i>0 then
      begin
      if checkfiledirname(Form_peach.StringGridList.Cells[8,i])<>0 then begin MessageDlg(txt_2_7_validatefn+' '+Form_peach.StringGridList.Cells[8,i], mtWarning, [mbOK], 0); exit; end;
      if fun='UNLPAQ' then
         begin
         if StringGridList.Rowcount<3 then
            if compose_unlpaq_cl(cl,jobcode,outname,true)=0 then
               if execute_cl(cl,'extandrun')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0)
            else
         else
            MessageDlg(txt_error_partial, mtWarning, [mbOK], 0); //never happens since LPAQ supports single file Compression
         exit;
         end;
      if fun='UNPAQ' then
         begin
         if StringGridList.Rowcount<3 then //partial extraction not implemented, so this method is allowed only if a single object is contained (useful for .tar.*)
            if compose_unpaq_cl(cl,jobcode,outname,true)=0 then
               begin
               if outname<>'' then if outname[length(outname)]<>directoryseparator then outname:=outname+directoryseparator;
               outname:=outname+StringGridList.Cells[1,1];
               if execute_cl(cl,'extandrun')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
               end
            else
         else
            MessageDlg(txt_error_partial, mtWarning, [mbOK], 0);
         exit;
         end;
      if fun='UNZPAQ' then
         begin
         if StringGridList.Rowcount<3 then //partial extraction not implemented, so this method is allowed only if a single object is contained (useful for .tar.*)
            if compose_unzpaq_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
               begin
               if outname<>'' then if outname[length(outname)]<>directoryseparator then outname:=outname+directoryseparator;
               outname:=outname+StringGridList.Cells[1,1];
               if execute_cl(cl,'extandrun')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
               end
            else
         else
            MessageDlg(txt_error_partial, mtWarning, [mbOK], 0);
         exit;
         end;
      if fun='UNPEA' then
         begin
         if extract_main_bytype('neutral',cl,jobcode,outname,true)=0 then launch_cl(cl,jobcode,outname);
         exit;
         end;
      if fun='UNQUAD' then
         begin
         if StringGridList.Rowcount<3 then
            begin
            if compose_unquad_cl(cl,jobcode,outname,true)=0 then
               if execute_cl(cl,'extandrun')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
            end
         else
            MessageDlg(txt_error_partial, mtWarning, [mbOK], 0); //never happens since QUAD/BALZ supports single file Compression
         exit;
         end;
      if fun='UNACE' then
         begin
         if StringGridList.Rowcount<3 then //partial extraction not implemented, so this method is allowed only if a single object is contained (useful for .tar.*)
            begin
            if compose_unace_cl(cl,jobcode,outname,true,'extandrun','single')=0 then
               begin
               if execute_cl(cl,'extandrun')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
               end;
            end
         else
            MessageDlg(txt_error_partial, mtWarning, [mbOK], 0);
         exit;
         end;
      if fun='FILEBROWSER' then
         begin
         j:=testext(StringGridList.Cells[8,StringGridList.Row]);
         if (j<0) //not a PeaZip supported filetype
            or (j=503)
            or (j=502)
            or (j=501) //file types usually not handled as archives
         then cp_open(StringGridList.Cells[8,StringGridList.Row],desk_env)
         else
            begin
            {$IFDEF MSWINDOWS}//if it's a Windows executable, offer the possibility to run it instead of open it in PeaZip (Windows only)
            if j=500 then
            begin
            s:=StringGridList.Cells[8,StringGridList.Row];
            udodirseparators(s);
            if MessageDlg(txt_dis+' '+s+' '+txt_type_exe, mtWarning,[mbYes, mbNo],0)=6 then
               begin
               w:=utf8decode(s);
               if validatecl(s)<>0 then begin MessageDlg(txt_2_7_validatecl+' '+s, mtWarning, [mbOK], 0); exit; end;
               ShellExecuteW(Form_peach.Handle, PWideChar ('open'), PWideChar(w), PWideChar (''), PWideChar (''), SW_SHOWNORMAL);
               exit;
               end;
            end;
            {$ENDIF}
            open_archive_fromname(StringGridList.Cells[8,StringGridList.Row]);
            end;
         exit;
         end;
      if testencrypted<0 then exit; //handles programmed extraction through 7z or arc (both supports partial extraction) if previous exit conditions are not met
      if fun='UN7Z' then
         begin
            if compose_un7z_cl(cl,jobcode,outname,true,'preview','single')=0 then
               if execute_cl(cl,'preview')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
         exit;
         end;
      if fun='UNARC' then
         begin
            if compose_unarc_cl(cl,jobcode,outname,true,'preview','single')=0 then
               if execute_cl(cl,'preview')=0 then execute_obj(3,outname)
               else
                  MessageDlg(txt_extraction_error, mtWarning, [mbOK], 0);
         exit;
         end;
      end;
   end;
end;
end;

procedure TForm_peach.StringGridListDblClick(Sender: TObject);
begin
grid_obj_open;
end;

function testspecialname(var s:utf8string):integer;
begin
testspecialname:=-1;
//licensing
if pos('COPYING',upcase(s))>0 then testspecialname:=1;
if pos('EULA',upcase(s))>0 then testspecialname:=1;
if pos('LICENSE',upcase(s))>0 then testspecialname:=1;
//support
if pos('HELP',upcase(s))>0 then testspecialname:=2;
if pos('README',upcase(s))>0 then testspecialname:=2;
//setup
if pos('SETUP',upcase(s))>0 then testspecialname:=3;
if pos('INSTALL',upcase(s))>0 then testspecialname:=3;
if pos('.MSI',upcase(s))>0 then testspecialname:=3;
//uninstall
if pos('UNINS',upcase(s))>0 then testspecialname:=4;
if pos('UNWISE',upcase(s))>0 then testspecialname:=4;
end;

procedure TForm_peach.StringGridListDrawCell(Sender: TObject; aCol,
  aRow: Integer; aRect: TRect; aState: TGridDrawState);
var
   s:utf8string;
   rowcol:TColor;
begin
if (arow and 1)=0 then rowcol:=StringGridList.Color
else rowcol:=StringGridList.AlternateColor;//col2
if (gdFixed in aState) then
   begin
   if aCol=0 then StringGridList.Canvas.Draw(StringGridList.CellRect(0,arow).Left+2,StringGridList.CellRect(0,arow).Top+1,Brefresh_up);
   exit;
   end;

   if StringGridList.Cells[12,arow] = '1' then
      begin
      StringGridList.Canvas.Brush.Color := clHighlight;
      StringGridList.Canvas.FillRect(aRect);
      StringGridList.Canvas.Font.Color := clHighlightText;
      if aCol=0 then drawcellicon(StringGridList,arow);
      end
   else
      begin
      StringGridList.Canvas.Brush.Color := rowcol;
      StringGridList.Canvas.FillRect(aRect);
      StringGridList.Canvas.Font.Color := clWindowText;
      if aCol=0 then drawcellicon(StringGridList,arow);
      if aCol=listcol then
         begin
         if StringGridList.Cells[2,arow] = txt_list_isfolder then
            begin
            StringGridList.Canvas.Brush.Color := stringtocolor(COLFOLDERROW);//(COLGROUP1);
            StringGridList.Canvas.Pen.Color := rowcol;
            StringGridList.Canvas.Pen.Width:=1;
            StringGridList.Canvas.FillRect(aRect);
            StringGridList.Canvas.RoundRect(arect,1,1);
            end;
         s:=StringGridList.Cells[8,arow];
         if (testinput(s,false)>0) then
            begin
            StringGridList.Canvas.Brush.Color := stringtocolor(COLARCHIVEROW);//(COLGROUP2);
            StringGridList.Canvas.Pen.Color := rowcol;
            StringGridList.Canvas.Pen.Width:=1;
            StringGridList.Canvas.FillRect(aRect);
            StringGridList.Canvas.RoundRect(arect,1,1);
            end;
         end;
      end;

if aCol=0 then StringGridList.Canvas.Font.Color := rowcol;
{$IFDEF MSWINDOWS}
{$ELSE}//patch the GTK/GTK2 issue of text remaining visible when column width is set to 0
if aCol=9 then StringGridList.Canvas.Font.Color := rowcol;
if aCol=10 then StringGridList.Canvas.Font.Color := rowcol;
if aCol=11 then StringGridList.Canvas.Font.Color := rowcol;
if aCol=12 then StringGridList.Canvas.Font.Color := rowcol;
{$ENDIF}
{//higlighting disabled by default for now
i:=testspecialname(s);
case i of
   0: c:=$00ffffff;
   1: c:=$00ccddff;
   2: c:=$00eeccff;
   3: c:=$00cceeff;
   4: c:=$00ccffee;
   end;
if i>0 then
   begin
   if (gdFixed in aState) or (gdSelected in aState) then exit;
   if (aCol=1) or (aCol=2) then
      begin
      StringGridList.Canvas.Brush.Color := c;
      StringGridList.Canvas.Pen.Color := $00FFFFFF;
      StringGridList.Canvas.Pen.Width:=1;
      StringGridList.Canvas.FillRect(aRect);
      StringGridList.Canvas.RoundRect(arect,1,1);
      end;
   end;}
end;

procedure TForm_peach.StringGridListHeaderClick(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
var
   Column:integer;
begin
Column:=Index;
   if (Column=0) then //refresh
      begin
      navrefresh;
      exit;
      end;
   listcol:=Column;
   if Column=1 then Column:=9;
   if Column=3 then Column:=10;
   if Column=4 then Column:=11;
   if Column=13 then Column:=8;
   if Column=listsortcol then
      if az=true then sort_za_stringgridlist
      else sort_az_stringgridlist(Column)
   else sort_az_stringgridlist(Column);
   listsortcol:=Column;
   Form_peach.StringGridList.Repaint;
end;

procedure TForm_peach.StringGridListHeaderSized(Sender: TObject;
  IsColumn: Boolean; Index: Integer);
begin
if index=0 then StringGridList.ColWidths[0]:=ROWSIZE;
end;

procedure mass_cp_copy_filedir;
var
   i,j:integer;
begin
if filecopying=true then
   begin
   MessageDlg(txt_copy_wait, mtInformation, [mbOK], 0);
   exit;
   end;
prev_destdir:='';
setlength(clipcontent1,0);
j:=0;
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   if Form_peach.StringGridList.Cells[12,i]='1' then
      begin
      setlength(clipcontent1,length(clipcontent1)+1);
      clipcontent1[j,0]:=Form_peach.StringGridList.Cells[8,i];
      clipcontent1[j,1]:=Form_peach.StringGridList.Cells[1,i];
      clipcontent1[j,2]:=Form_peach.StringGridList.Cells[2,i];
      clipcontent1[j,3]:='copy';
      j:=j+1;
      end;
destdir:=Form_peach.EditOpenOut.Text;
if j>0 then
   if clipcontent1[0,0]<>'' then cp_copy_filedir(clipcontent1,destdir);
end;

procedure TForm_peach.StringGridListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
   i,j:integer;
begin
//patch for Ctrl+X behaviour
if (Key=ord(char(88))) and (shift=[ssCtrl]) then
   begin
   setlength(patchclip,Form_peach.StringGridList.RowCount-1);
   for i:=1 to Form_peach.StringGridList.RowCount-1 do
      for j:=0 to StringGridList.ColCount-1 do patchclip[i-1,j]:=StringGridList.Cells[j,i];
   end;

//ensure Ctrl+V has enough rows to paste the content, since content could be pasted from clipboard it consider at excess 2048 rows
if (Key=ord(char(86))) and (shift=[ssCtrl]) then
   begin
   Form_peach.StringGridList.RowCount:=2048;
   if Form_peach.StringGridList.ColWidths[8]=0 then listsizecol('flat') else
      if Form_peach.Edit1.Text=txt_mypc then listsizecol('pc')
      else listsizecol('browse');
   end;
//patch for Ctrl+V behaviour in "My computer" level (Windows only)
{$IFDEF MSWINDOWS}
if Form_peach.EditOpenIn.Text=txt_mypc then
   if (Key=ord(char(86))) and (shift=[ssCtrl]) then
      begin
      setlength(patchclip,Form_peach.StringGridList.RowCount-1);
      for i:=1 to Form_peach.StringGridList.RowCount-1 do
         for j:=0 to StringGridList.ColCount-1 do patchclip[i-1,j]:=StringGridList.Cells[j,i];
      end;
{$ENDIF}

if Key=13 then
   if shift=[ssShift] then //shift+enter extract selected to new folder
      begin
      {$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
      if StringGridList.Cells[1,1]='' then exit;
      CheckBoxFolder.State:=cbChecked;
      if fun='FILEBROWSER' then list_toextractor('multi')
      else list_toextractor('selected');
      end
   else
      if shift=[ssCtrl] then //ctrl+enter extract selected
         begin
         {$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
         if StringGridList.Cells[1,1]='' then exit;
         CheckBoxFolder.State:=cbUnChecked;
         if fun='FILEBROWSER' then list_toextractor('multi')
         else list_toextractor('selected');
         end
      else grid_obj_open;//enter: open if dir, use default doubleclick action if file (default is preview)

//for unknown reason F2 is not correctly handled if a stringgrid is focused
if Key=113 then //F2 go to desktop, Shift+F2 computer's root, Ctr+F2 user's home
   if shift=[ssShift] then jumpto('root')
   else
      if shift=[ssCtrl] then jumpto('home')
      else jumpto('desktop');

//functional keys are handled by hidden entries in browser submenu of main menu
end;

procedure TForm_peach.StringGridListKeyPress(Sender: TObject; var Key: char);
var
   i:integer;
begin

//patch for Ctrl+X behaviour
if Key=char(24) then
   begin
   patch_stringgrid(StringGridList);
   list_cut;
   end;

//Ctrl+C
if Key=char(3) then list_copy;

//Ctrl+V
if Key=char(22) then
   begin
   if fun='FILEBROWSER' then
      begin
      {$IFDEF MSWINDOWS}
      if Form_peach.EditOpenIn.Text=txt_mypc then
         begin
         patch_stringgrid(StringGridList);
         //navrefresh; //deactivated, refresh is now automatic
         exit;
         end;
      {$ENDIF}
      list_paste;
      end
   else //paste in archive
      if (fun='UNARC') or (fun='UN7Z') then
         if Form_peach.StringGridList.Cells[1,1]<>'' then
            if length(clipcontent)>0 then
               begin
               addtoarchive_inarchive;
               Form_peach.StringGrid1.Cursor:=crHourGlass;
               deselectingrid(Form_peach.StringGrid1);
               for i:=0 to length(clipcontent)-1 do
                  begin
                  if clipcontent[i,2]=txt_list_isfolder then addfolderstr(Form_peach.StringGrid1,clipcontent[i,0])
                  else addfilestr(Form_peach.StringGrid1,clipcontent[i,0]);
                  end;
               Form_peach.StringGrid1.AutoSizeColumns;
               updatecontent(Form_peach.StringGrid1,tvolumes,tdirs,tfiles,tsize,true);
               Form_peach.StringGrid1.Cursor:=crDefault;
               end
            else open_archive_fromname(EditOpenIn.Text)
         else open_archive_fromname(EditOpenIn.Text)
      else open_archive_fromname(EditOpenIn.Text);
   end;

//filter functional standard keys
//removed: if Key=char(30) then deleteselected_frombrowser;  //cancel
//removed: if Key=char(127) then deleteselected_frombrowser; //ctrl+backspace

   if Key=char(27) then //Esc to cancel copy/move operations clearing selection and cupy/cut clipboard; cancel also drag operations performed on the same keyboard event, if panelopen is visible, by Timer1
      begin
      clear_clipboard;
      end;

if (fun='UN7Z') or (fun='FILEBROWSER') then
   begin
   if Key=char('<') then grid_dir_open;
   if Key=char('*') then toggle_view;
   end;

   if Key=char('>') then navgoup;
   if Key=char(8) then navgoback;  //backspace

{if key='?' then //test all (all selected archives in file browser)
   if fun='FILEBROWSER' then
      begin
      if StringGridList.Cells[1,1]='' then exit;
      filebrowser_funall('test');
      end
   else
       if (fun='UN7Z') or (fun='UNARC') then archive_testall;}

if fun='FILEBROWSER' then
   begin
   if key='?' then //checksum/hash
      begin
      if StringGridList.Cells[1,1]='' then exit;
      checkselected_list;
      end;
   if key='=' then //compare
      begin
      if StringGridList.Cells[1,1]='' then exit;
      compareselected_list;
      end;
   end;

//search char
for i:=StringGridList.Row+1 to StringGridList.Rowcount-1 do
   if upcase(copy(StringGridList.Cells[1,i],1,1))=upcase(Key) then
      begin
      StringGridList.Row:=i;
      exit;
      end;
for i:=1 to StringGridList.Row do
   if upcase(copy(StringGridList.Cells[1,i],1,1))=upcase(Key) then
      begin
      StringGridList.Row:=i;
      exit;
      end;
end;

procedure TForm_peach.StringGridListKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if Key=46 then //Cancel
   if shift=[ssShift] then
      if fun='FILEBROWSER' then eraseselected_list
      else
   else
      if shift=[] then deleteselected_frombrowser;
end;

procedure TForm_peach.StringGridListMouseDown(Sender: TOBject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   Column, Row,i: Integer;
begin
//on some rare case (temporary system issues?) on WinXP the Bottom selection remains
//valorized with values higher than Rowcount, which can cause error on mousedown
//(oddly it is not catched if using Try Except on the event)
if Form_peach.StringGridList.Rowcount<=Form_peach.StringGridList.Selection.Bottom then
 begin
 sort_za_stringgridlist;
 sort_az_stringgridlist(listsortcol);
 end;
{$IFDEF MSWINDOWS}dragcancelled:=false;{$ENDIF}
StringGridList.MouseToCell(X, Y, Column, Row);
listrow:=Row;
case Button of
   mbRight:
      begin
      Form_peach.StringGridList.Row:=Row;
      if (Form_peach.StringGridList.Cells[12,Row]='0') then
         begin
         for i:=1 to Form_peach.StringGridList.Rowcount-1 do
            Form_peach.StringGridList.Cells[12,i]:='0';
         Form_peach.StringGridList.Cells[12,Row]:='1';
         Form_peach.StringGridList.Repaint;
         end;
      exit;
      end;
   mbMiddle: //middle button extract selected
      begin
      {if shift=[ssShift] then //shift+middle button extract selected to new folder
         begin
         {$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
         if StringGridList.Cells[1,1]='' then exit;
         CheckBoxFolder.State:=cbChecked;
         if fun='FILEBROWSER' then list_toextractor('multi')
         else list_toextractor('selected');
         end} //shift status seem not working for middle button
         {$IFDEF MSWINDOWS}if Form_peach.EditOpenIn.Text=txt_mypc then exit;{$ENDIF}
         if StringGridList.Cells[1,1]='' then exit;
         CheckBoxFolder.State:=cbUnChecked;
         if fun='FILEBROWSER' then list_toextractor('multi')
         else list_toextractor('selected');
      exit;
      end;
   end;
end;

function dragtowin(var dragdest:utf8string):integer;
{$IFDEF MSWINDOWS}
var
   j:integer;
   lpPoint: TPoint;
   s,t:array[0..256] of Char;
   st:utf8string;
   haddress,htest:HWND;
   {$ENDIF}
begin
   {$IFDEF MSWINDOWS}
   dragtowin:=-1;
   dragdest:='<unsupported>';
   GetCursorPos(lpPoint);
   //get main window handle from cursor point
   haddress:=WindowFromPoint(lpPoint);
   GetClassName(haddress,s,sizeof(s));
    //workaround for identifying desktop
    j := GetWindowTextLength(haddress)+1;
    SetLength(st, j-1);
    GetWindowText(haddress,@st[1],j);
    st:=s+st;
   if (s<>'CabinetWClass') and (s<>'ExploreWClass') then
      repeat
      haddress := getparent(haddress);
      GetClassName(haddress,s,sizeof(s));
      until (s='CabinetWClass') or (s='ExploreWClass') or (haddress=0);
    st:=s+st;
   if haddress<>0 then //get address from an explorer's window control (system specific)
      begin
      if winver='9x' then //works with both browse and explore window type
         begin
         haddress := FindWindowEx(haddress, 0, 'Worker', nil);
         haddress := FindWindowEx(haddress, 0, 'ReBarWindow32', nil);
         haddress := FindWindowEx(haddress, 0, 'ComboBoxEx32', nil);
         haddress := FindWindowEx(haddress, 0, 'ComboBox', nil);
         haddress := FindWindowEx(haddress, 0, 'Edit', nil);
         end;
      if (winver='nt4') or (winver='nt5') then //(not sure it is ok for NT4)
         begin
         htest:= FindWindowEx(haddress, 0, 'ExploreWClass', nil);//test if using "Explore" window type
         if htest<>0 then haddress:=htest;
         haddress := FindWindowEx(haddress, 0, 'WorkerW', nil);
         haddress := FindWindowEx(haddress, 0, 'ReBarWindow32', nil);
         haddress := FindWindowEx(haddress, 0, 'ComboBoxEx32', nil);
         haddress := FindWindowEx(haddress, 0, 'ComboBox', nil);
         haddress := FindWindowEx(haddress, 0, 'Edit', nil);
         end;
      if winver='nt6+' then
         begin
         haddress := FindWindowEx(haddress, 0, 'WorkerW', nil);
         haddress := FindWindowEx(haddress, 0, 'ReBarWindow32', nil);
         haddress := FindWindowEx(haddress, 0, 'Address Band Root', nil);
         haddress := FindWindowEx(haddress, 0, 'msctls_progress32', nil);
         haddress := FindWindowEx(haddress, 0, 'ComboBoxEx32', nil);
         haddress := FindWindowEx(haddress, 0, 'ComboBox', nil);
         haddress := FindWindowEx(haddress, 0, 'Edit', nil);
         end;
      if haddress=0 then t:='<unsupported>' //address bar is hidden, exporer's window cannot be queried for output path
      else
         begin
         SendMessage(haddress, WM_GETTEXT, SizeOf(t), integer(@t));
         if not(udirectoryexists(envtoutf8(t))) then //special folders with conventional name instead of path (documets etc)
            begin
            if winver='9x' then t:=uextractfilepath(GetEnvironmentVariable('WINDIR'))+t+'\'
            else t:=home_path+'\'+t+'\';
            t:=utf8toenv(t);
            if not(udirectoryexists(envtoutf8(t))) then t:='<unsupported>'; //string is not a path, or the path don't exists (i.e. Control panel etc)
            end;
         end;
      end
   else //recognize if it is desktop window, or unsupported application's window
      begin
      t:='<unsupported>'; //application cannot be queried for output path
      if winver='9x' then
         if st='SysListView32' then //activedesktop not supported
            t:=GetEnvironmentVariable('WINDIR')+'\Desktop\';
      if (winver='nt4') or (winver='nt5') then
         if (st='ProgmanSysListView32FolderView') or //XP
            (st='ProgmanSysListView32') or //Win2k
            (st='ProgmanInternet Explorer_Server') then //activedesktop
            t:=local_desktop;
      if winver='nt6+' then
         if st='ProgmanSysListView32FolderView' then
            t:=local_desktop;
      t:=utf8toenv(t);
      end;
   dragdest:=envtoutf8(t);
   dragtowin:=0;
   {$ENDIF}
end;

procedure get_drag_information(var t_files,t_folders:integer; var name_file,name_folder:utf8string);
var
   i:integer;
begin
t_files:=0;
t_folders:=0;
name_file:='';
name_folder:='';
for i:=1 to Form_peach.StringGridList.RowCount-1 do
   if Form_peach.StringGridList.Cells[12,i]='1' then
      if Form_peach.StringGridList.Cells[2,i]=txt_list_isfolder then
         begin
         t_folders:=t_folders+1;
         if t_folders<4 then
            name_folder:=name_folder+Form_peach.StringGridList.Cells[1,i]+', ';
         end
      else
         begin
         t_files:=t_files+1;
         if t_files<4 then
            name_file:=name_file+Form_peach.StringGridList.Cells[1,i]+', ';
         end;
if t_folders>3 then name_folder:=name_folder+'...'
else setlength(name_folder,length(name_folder)-2);
if t_files>3 then name_file:=name_file+'...'
else setlength(name_file,length(name_file)-2);
end;

procedure TForm_peach.StringGridListMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
{$IFDEF MSWINDOWS}
var
   nfiles,nfolders:integer;
   name_file,name_folder,dragsrc,dragdest:utf8string;
   lpPoint: TPoint;
   {$ENDIF}
begin
{$IFDEF MSWINDOWS}
if dragcancelled=false then
if EditOpenIn.Caption=txt_mypc then
else
if (x<-8) or (y<-16) or (x>stringgridlist.width+8) or (y>stringgridlist.height+8) then
   begin
   GetCursorPos(lpPoint);
   get_drag_information(nfiles,nfolders,name_file,name_folder);
   if (name_file='') and (name_folder='') then exit;
   form1.visible:=true;
   form1.top:=lpPoint.y+24;
   form1.left:=lpPoint.x+24;
   form1.Label2.caption:=txt_nfolders+' ('+inttostr(nfolders)+') '+name_folder;
   form1.Label3.caption:=txt_nfiles+' ('+inttostr(nfiles)+') '+name_file;
   form1.Label2.Font.Color:=stringtocolor(color4);
   form1.Label3.Font.Color:=stringtocolor(color4);
   form1.image1.picture.bitmap:=Bft;
   form1.image2.picture.bitmap:=Bfolder;
   form1.image3.picture.bitmap:=Bunsupported;
   form1.image2.transparent:=true;
   form1.image3.transparent:=true;
   dragsrc:=EditOpenIn.Text;
   if dragsrc<>'' then if dragsrc[length(dragsrc)]=directoryseparator then setlength(dragsrc,length(dragsrc)-1);
   if fun='FILEBROWSER' then
      begin
      if uextractfilename(dragsrc)<>'' then form1.label5.Caption:=txt_2_4_copyfrom+' '+uextractfilename(dragsrc)
      else form1.label5.Caption:=txt_2_4_copyfrom+' '+copy(EditOpenIn.Caption,1,3);
      form1.image4.picture.bitmap:=Bhd_supported;
      form1.image4.transparent:=true;
      end
   else
      begin
      form1.label5.Caption:=txt_2_4_extractfrom+' '+uextractfilename(dragsrc);
      form1.image4.picture.bitmap:=Bextractall;
      form1.image4.transparent:=true;
      end;
   dragtowin(dragdest);
   if dragdest='<unsupported>' then
      begin
      Form1.color:=clInfoBk;
      form1.label5.Caption:=form1.label5.Caption+' '+txt_to+' ... '+txt_2_4_draghint;
      end
   else
      begin
      Form1.color:=clWindow;
      if dragdest<>'' then if dragdest[length(dragdest)]=directoryseparator then setlength(dragdest,length(dragdest)-1);
      if uextractfilename(dragdest)<>'' then form1.label5.Caption:=form1.label5.Caption+' '+txt_to+' '+uextractfilename(dragdest)
      else form1.label5.Caption:=form1.label5.Caption+' '+txt_to+' '+copy(dragdest,1,2)+directoryseparator;
      end;
   end
else form1.visible:=false;
{$ENDIF}
end;

procedure TForm_peach.StringGridListMouseUp(Sender: TOBject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   i,Column,Row,totrow:integer;
   {$IFDEF MSWINDOWS}
   cl,jobcode,outname,dragdest,olddest:utf8string;
   {$ENDIF}
begin
totrow:=StringGridList.Rowcount;
if totrow<2 then exit;
{$IFDEF MSWINDOWS}
{drag from application to system, Windows: get path of drop destination and use
as output path.
Does not rely on the undocumented CWM_GETPATH function since being unoffcial it
can be not supported on some MS systems.
Pros:
- don't need to use system's temp folder as by design does drag and drop, this means:
-- data doesn't need to be copied twice;
-- data goes directly to desired output location which has desired security policies
Cons:
- requires Address bar to be visible, as default (not an issue on Vista);
- does not show drag and drop cursor icon (but shows custom form with detailed drag and drop information);
- it is not able to open items on taskbar, to get path from folders on desktop or
  treewiews, or to get paths from open/save dialog windows.
}
form1.visible:=false;
if dragcancelled=false then
begin
if EditOpenIn.Caption=txt_mypc then
else
if (x<-8) or (y<-16) or (x>stringgridlist.width+8) or (y>stringgridlist.height+8) then
   begin
   if dragtowin(dragdest)<>0 then exit;
   if dragdest='<unsupported>' then
      begin
      if set_extractto_path<>0 then exit;
      dragdest:=Form_peach.EditOpenOut.Text;
      end;
   olddest:=Form_peach.EditOpenOut.Text;
   Form_peach.EditOpenOut.Text:=dragdest;
   if Form_peach.EditOpenOut.Text<>'' then
      if Form_peach.EditOpenOut.Text[length(Form_peach.EditOpenOut.Text)] <> directoryseparator then Form_peach.EditOpenOut.Text:=Form_peach.EditOpenOut.Text+directoryseparator;
   //menuitemopen_ext_caption;
   if fun='UNPEA' then
      if totrow<3 then
         if compose_unpea_cl(cl,jobcode,outname)=0 then launch_cl(cl,jobcode,outname)
         else
      else
         MessageDlg(txt_error_partial, mtWarning, [mbOK], 0); //never happens since PEA visualize content as a single object
   if fun='UNLPAQ' then
      if totrow<3 then
         if compose_unlpaq_cl(cl,jobcode,outname,true)=0 then launch_cl(cl,jobcode,outname)
         else
      else
         MessageDlg(txt_error_partial, mtWarning, [mbOK], 0); //never happens since LPAQ supports single file Compression
   if fun='UNPAQ' then
      if totrow<3 then //partial extraction not implemented, so this method is allowed only if a single object is contained (useful for .tar.*)
         if compose_unpaq_cl(cl,jobcode,outname,true)=0 then launch_cl(cl,jobcode,outname)
         else
      else
         MessageDlg(txt_error_partial, mtWarning, [mbOK], 0);
   if fun='UNZPAQ' then
      if totrow<3 then //partial extraction not implemented, so this method is allowed only if a single object is contained (useful for .tar.*)
         if compose_unzpaq_cl(cl,jobcode,outname,true,'ext','single')=0 then launch_cl(cl,jobcode,outname)
         else
      else
         MessageDlg(txt_error_partial, mtWarning, [mbOK], 0);
   if fun='UNQUAD' then
      if totrow<3 then
         if compose_unquad_cl(cl,jobcode,outname,true)=0 then launch_cl(cl,jobcode,outname)
         else
      else
         MessageDlg(txt_error_partial, mtWarning, [mbOK], 0); //never happens since QUAD/BALZ supports single file Compression
   if fun='UNACE' then
      if totrow<3 then //partial extraction not implemented, so this method is allowed only if a single object is contained (useful for .tar.*)
         if compose_unace_cl(cl,jobcode,outname,true,'ext','single')=0 then launch_cl(cl,jobcode,outname)
         else
      else
         MessageDlg(txt_error_partial, mtWarning, [mbOK], 0);
   if (fun='FILEBROWSER') then mass_cp_copy_filedir;//drag from filebrowser: copy
   if testencrypted=0 then
      begin
      if (fun='UNARC') then
         if compose_unarc_cl(cl,jobcode,outname,true,'ext','selected')=0 then launch_cl(cl,jobcode,outname);
      if (fun='UN7Z') then
         if compose_un7z_cl(cl,jobcode,outname,true,'ext','selected')=0 then launch_cl(cl,jobcode,outname);
      end;
   setfocus_stringgridlist;//note: needs to set focus to form
   lastoutpath:=Form_peach.EditOpenOut.Text;
   Form_peach.EditOpenOut.Text:=olddest;
   exit;
   end;
end;
{$ENDIF}
StringGridList.MouseToCell(X, Y, Column, Row);
if Row=0 then exit;
case Button of
   mbRight: exit; //moved to mousedown
   mbMiddle: exit;
   mbLeft:
      begin
         if Shift=[ssCtrl] then
            for i:=Form_peach.StringGridList.Selection.Top to Form_peach.StringGridList.Selection.Bottom do
               if Form_peach.StringGridList.Cells[12,i]='1' then Form_peach.StringGridList.Cells[12,i]:='0'
               else Form_peach.StringGridList.Cells[12,i]:='1'
         else
            begin
            for i:=1 to totrow-1 do
               Form_peach.StringGridList.Cells[12,i]:='0';
            for i:=Form_peach.StringGridList.Selection.Top to Form_peach.StringGridList.Selection.Bottom do
               Form_peach.StringGridList.Cells[12,i]:='1';
            end;
         Form_peach.StringGridList.Repaint;
      end;
   end;
end;

procedure TForm_peach.theme_label1Click(Sender: TObject);
begin
load_default_theme;
imgloaded:=false;
check_theme_failure;
apply_theme;
graphicsfolder:=graphicsfolderd;
opacity:=opacityd;
color1:=color1d;
color2:=color2d;
color3:=color3d;
color4:=color4d;
color5:=color5d;
rowheight:=rowheightd;
itemheight:=itemheightd;
autosizeitemheight:=autosizeitemheightd;
autosizetoolbarbutton:=autosizetoolbarbuttond;
load_icons;
DirectoryEditthemegraphics.Text:=executable_path+graphicsfolderd;
setmswindowsopacity;
end;

procedure TForm_peach.themesave_label_infoClick(Sender: TObject);
begin
Panel2.Visible:=save_theme_clicked;
save_theme_clicked:=not(save_theme_clicked);
end;

procedure TForm_peach.themesave_labelClick(Sender: TObject);
var
   i:integer;
begin
if edit1.text='' then
   begin
   MessageDlg(txt_no_theme_name, mtWarning, [mbOK], 0);
   exit;
   end;
if DirectoryEdit2.text='' then DirectoryEdit2.text:=executable_path+'res'+directoryseparator+'themes'+directoryseparator;
i:=length(Form_peach.DirectoryEdit2.text);
if not (Form_peach.DirectoryEdit2.text[i]=DirectorySeparator) then Form_peach.DirectoryEdit2.text:=Form_peach.DirectoryEdit2.text+DirectorySeparator;
themedir:=extractrelativepath(executable_path,DirectoryEdit2.text+edit1.text+DirectorySeparator);
save_new_theme;
theme_path:=themedir+'theme.txt';
end;

procedure TForm_peach.theme_labelClick(Sender: TObject);
begin
cp_open(executable_path+'res'+directoryseparator+'themes'+directoryseparator,desk_env);
end;

procedure TForm_peach.Timer1Timer(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
if PanelOpen.Visible=true then
   if (getkeystate(27)<>1) and (getkeystate(27)<>0) then
      begin form1.visible:=false; dragcancelled:=true; end;
if pcmenupopulated=true then
   exit
else
   begin
   pcmenupopulated:=true;
   populatepcmenu;
   end;
{$ELSE}
if openstarted=true then exit else openstart;
{$ENDIF}
if waitdrawok=false then waitdrawok:=true;
end;

procedure TForm_peach.Timer2Timer(Sender: TObject);//optimized counting, skip if counting is in progress, flexible timing from 0.5 to 3 sec (increases for flat mode and for many items)
var
   recmode:boolean;
   nfiles,ndirs,nunits,rc,sc,tsize: qword;
   flist: TFoundList;
   fsizes: TFoundListSizes;
   ftimes: TFoundListAges;
   fattr: TFoundListAttrib;
   fattr_dec: TFoundList;
   i,torefresh:integer;
   navrefreshed:boolean;
begin
{$IFDEF MSWINDOWS}if panelopen.visible=true then{$ELSE}if panelopen.top=0 then{$ENDIF}
   if fun='FILEBROWSER' then
      begin
      if listingdir=true then exit; //skip if a directory is being listed
      if control_listingdir=true then begin timer2.enabled:=false; exit; end;
      control_listingdir:=true;
      labelstatus.Caption:=status0+' '+status1+' ['+status2+']';
      if StringGridHistory.Cells[1,StringGridHistory.Row]=txt_list_browsing then recmode:=false
      else recmode:=true;
      if recmode=true then Timer2.Interval:=1000
      else Timer2.Interval:=500;
      rc:=StringGridList.rowcount;
      sc:=0;
      for i:=1 to rc-1 do try sc:=sc+strtoqword(StringGridList.Cells[3,i]); except end;
      if EditOpenIn.Text<>txt_mypc then
         begin
         if udirectoryexists(EditOpenIn.Text) then
            begin
            nfiles:=0;
            ndirs:=0;
            tsize:=0;
            rcountsize(EditOpenIn.Text,EditUn7zaFilter.Text,faAnyFile,recmode,nfiles,ndirs,tsize);
            if nfiles>10000 then
               if nfiles>50000 then Timer2.Interval:=Timer2.Interval*3
               else Timer2.Interval:=Timer2.Interval*2;
            navrefreshed:=false;
            case nfiles of
               0: if StringGridList.Cells[1,1]<>'' then begin navrefresh; navrefreshed:=true; end;
               1: begin
                  if nfiles+1<>rc then begin navrefresh; navrefreshed:=true; end
                  else
                     if StringGridList.Cells[1,1]='' then begin navrefresh; navrefreshed:=true; end;
                  end;
               else if nfiles+1<>rc then begin navrefresh; navrefreshed:=true; end;
               end;
            if navrefreshed=false then
               if tsize<>sc then //slower refresh rate if only size is changing
                  begin
                  torefresh:=random(4);
                  if torefresh=0 then navrefresh;
                  end;
            end;
         end
      else
         begin
         {$IFDEF MSWINDOWS}
         Timer2.Interval:=1000;
         countunits(nunits);
         if nunits+1<>rc then
            begin
            navrefresh;
            populatepcmenu;
            end;
         {$ENDIF}
         end;
      control_listingdir:=false;
      end
   else labelstatus.Caption:=status0+' '+status1+' ['+status2+']';
end;

procedure TForm_peach.tmcreateClick(Sender: TObject);
begin
TrayIcon1.visible:=false;
Form_peach.visible:=true;
showpanel('archive');
end;

procedure TForm_peach.tmopenarchiveClick(Sender: TObject);
begin
TrayIcon1.visible:=false;
Form_peach.visible:=true;
open_archive;
end;

procedure TForm_peach.tmopenpathClick(Sender: TObject);
begin
TrayIcon1.visible:=false;
Form_peach.visible:=true;
peazip_browse;
end;

procedure TForm_peach.tmpeazipClick(Sender: TObject);
begin
TrayIcon1.visible:=false;
Form_peach.visible:=true;
end;

procedure TForm_peach.tmquitClick(Sender: TObject);
begin
Form_peach.visible:=true;
Form_peach.Close;
end;

procedure TForm_peach.TrayIcon1DblClick(Sender: TObject);
begin
TrayIcon1.visible:=false;
Form_peach.visible:=true;
end;

initialization
  {$I peach.lrs}

  {$IFDEF MSWINDOWS}
  OleInitialize(nil);
  {$ENDIF}

finalization
  {$IFDEF MSWINDOWS}
  OleUninitialize
  {$ENDIF}

end.
